﻿#pragma TextEncoding = "UTF-8"		// For details execute DisplayHelpTopic "The TextEncoding Pragma"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.



Menu "Utilities"
	"-"
		"Save Graph as Copy", /Q, CopyGraphData("", "", "")
	//SubMenu "Axes"
		"-"
		"Paste Bottom axis range", /Q, CopyPaste_Bottom_AxisRange("", 1)
		"Copy Bottom axis range", /Q, CopyPaste_Bottom_AxisRange("", 0)
		"Auto Bottom axis range", /Q, SetAxis/A bottom
		"-"
		"Paste Left axis range", /Q, CopyPaste_Left_AxisRange("", 1)
		"Copy Left axis range", /Q, CopyPaste_Left_AxisRange("", 0)
		"Auto Left axis range", /Q, SetAxis/A left
		"-"
		"Play Displayed Sound", /Q, GraphMarquee_tryToPlaySound()
	//End
End

// =================================================================================================
// GraphMarquee Menu for obtaining time measurements from "bottom" axis
// =================================================================================================
Menu "GraphMarquee"
	"Play sound", /Q, GraphMarquee_tryToPlaySound()
	"dY (left axis)", /Q, GraphMarquee_dY_left()
	"dX (bottom axis)", /Q, GraphMarquee_dX_bottom()
End

// =================================================================================================
// GraphMarquee_dX_bottom(), GraphMarquee function for obtaining time measurements from "bottom" axis
// =================================================================================================
function GraphMarquee_dX_bottom()
	String str, traceListStr = TraceNameList("",";",1+4)	// only visible normal traces
	variable i,fs, last_fs, printPnts = 1
	for(i=0;i<ItemsInList(traceListStr);++i)
		str = StringFromList(i,traceListStr)
		wave w = TraceNameToWaveRef("", str )
		fs = round(1.0/dimdelta(w,0))
		if(i>0 && fs!=last_fs)
			printPnts = 0
		endif
		last_fs = fs
	endfor
	if(fs <= 1)
		printPnts = 0
	endif
	string AxesListStr = AxisList(""), AxisStr
	for(i=0;i<ItemsInList(AxesListStr);++i)
		AxisStr = StringFromList(i,AxesListStr)
		if(strsearch(AxisStr, "bottom", 0, 2) >=0)
			GetMarquee/Z $AxisStr
			string unitsStr = StringByKey("UNITS", AxisInfo("",AxisStr))
			strswitch (unitsStr)
				case "s":
					Printf "%g s (%g ms)(%g µs) (%g Hz)" V_right-V_left, (V_right-V_left)*1000, (V_right-V_left)*1000*1000, 1/(V_right-V_left)
					break
				case "ms":
					Printf "%g ms (%g s)(%g µs) (%g Hz)" V_right-V_left, (V_right-V_left)/1000, (V_right-V_left)*1000, 1/((V_right-V_left)/1000)
					break
				case "µs":
					Printf "%g µs (%g ms)(%g s) (%g Hz)" V_right-V_left, (V_right-V_left)/1000, (V_right-V_left)/1000000, 1/((V_right-V_left)/1000000)
					break
				default:
					Printf "%g %s (%g 1/%s)" V_right-V_left, unitsStr, 1/(V_right-V_left), unitsStr
					break
			endswitch
			if(printPnts)
				strswitch (unitsStr)
					case "s":
						Printf " (%d points)", round((V_right-V_left)*fs)
						break
					case "ms":
						Printf " (%d points)", round(((V_right-V_left)/1000)*fs)
						break
					case "µs":
						Printf " (%d points)", round(((V_right-V_left)/1000000)*fs)
						break
					default:
						break
				endswitch
			endif
			strswitch (unitsStr)
				case "s":
					Printf " (%.2f m @20°C)(%.2f' @68°F)", (V_right-V_left)*343, (V_right-V_left)*1235
					break
				case "ms":
					Printf " (%.2f m @20°C)(%.2f' @68°F)", ((V_right-V_left)/1000)*343,  ((V_right-V_left)/1000)*1235
					break
				case "µs":
					Printf " (%.2f m @20°C)(%.2f' @68°F)", ((V_right-V_left)/1000000)*343, ((V_right-V_left)/1000000)*1235
					break
				default:
					break
			endswitch
			
			Printf "\r"
		endif
	endfor
end

// =================================================================================================
// GraphMarquee_dY_left(), GraphMarquee function for obtaining measurements from the "left" axis
// =================================================================================================
function GraphMarquee_dY_left()
	String str, traceListStr = TraceNameList("",";",1+4)	// only visible normal traces
	variable i,fs, last_fs, printPnts = 1
	for(i=0;i<ItemsInList(traceListStr);++i)
		str = StringFromList(i,traceListStr)
		wave w = TraceNameToWaveRef("", str )
		fs = round(1.0/dimdelta(w,0))
		if(i>0 && fs!=last_fs)
			printPnts = 0
		endif
		last_fs = fs
	endfor
	string AxesListStr = AxisList(""), AxisStr
	for(i=0;i<ItemsInList(AxesListStr);++i)
		AxisStr = StringFromList(i,AxesListStr)
		if(strsearch(AxisStr, "left", 0, 2) >=0)
			GetMarquee/Z $AxisStr
			string unitsStr = StringByKey("UNITS", AxisInfo("",AxisStr))
			Printf "Top=%g %s;  Bottom=%g %s;  Top-Bottom= %g %s\r", V_top, unitsStr, V_bottom, unitsStr, V_top-V_bottom, unitsStr
		endif
	endfor
end


// =================================================================================================
// GraphMarquee_tryToPlaySound(), GraphMarquee try to play first playable trace 
// - must have appropriate sampling rate and be displayed using "bottom" axis
// =================================================================================================
function GraphMarquee_tryToPlaySound()
	String traceListStr = TraceNameList("",";",1+4)	// only visible normal traces
	QuerySoundManager("SoundTemp")
	SVAR rateList = $":SoundTemp:rateList"
	String rateListStr = rateList
	killdatafolder $"SoundTemp"
	String str, str2
	variable fs
	int64 i,j
	for(i=0;i<ItemsInList(traceListStr);++i)
		str = StringFromList(i,traceListStr)
		wave w = TraceNameToWaveRef("", str )
		fs = round(1.0/dimdelta(w,0))
		for(i=0;i<ItemsInList(rateListStr);++i)
			str2 = StringFromList(i,rateListStr)
			if(fs == str2num(str2))
				// Play
				GetMarquee/Z bottom
				if(V_flag)
					Duplicate/O/R=(V_left, V_right)/FREE w, temp
				else
					Duplicate/O/FREE w, temp
				endif
				wavestats/Q/M=1 temp
				temp /= v_max
				SpHr_RampOnOff 20, 20, temp
				insertPoints 0, 2000, temp
				redimension/N=(dimsize(temp,0)+2000, -1) temp
				Variable keys = GetKeyState(0)
				if(keys == 2 || keys == 10)
					Duplicate/O/FREE temp, temp2
					temp[][0] = temp2[p][1]
					temp[][1] = temp2[p][0]
				endif
				PlaySound/A temp
				return 0
			endif
		endfor
	endfor
end

// =================================================================================================
// ExternalControlPanel(...), creates an external panel around host window
//	HostWinStr, Name of host window
//	NameStr, Name of window
//	TitleStr, Title of window
//	side == 0, right side
//	side == 1, left side
//	side == 2, bottom side
//	side == 3, top side
// size, width of height
// panelType, 0: regular window, 1: floating, 2: floating with no close box
// Pleft,Ptop,Pright,Pbottom, places window at a specific location
//	ExternalControlPanel("HostWindName", "TestExt", "TestExt", 3, 30, -1,-1,-1,-1)
// =================================================================================================
function/S ExternalControlPanel(HostWinStr, NameStr, TitleStr, side, size, panelType, Pleft,Ptop,Pright,Pbottom)
	String HostWinStr, NameStr, TitleStr
	variable side, size, panelType, Pleft,Ptop,Pright,Pbottom
	if(Pleft<0 || Ptop<0 || Pright<0 || Pbottom<0) // defauls
		switch(side)
			case 0: 	// Right
				string screenInfo = StringByKey("SCREEN1", IgorInfo(0))	// Number of active displays
				variable screenHoriz = str2num(ParseFilePath(0, screenInfo, ",", 1, 1))
				GetWindow/Z $HostWinStr wsize // wsizeDC
				if(V_right + size > screenHoriz) // shift widow to the left if panel is off screen to the right
					MoveWindow/W=$HostWinStr V_left-((V_right+size)-screenHoriz)-3, V_top, V_right-((V_right+size)-screenHoriz)-3, V_bottom
				endif
				Pright= size
				Ptop= 0
				Pleft= 0
				Pbottom= 0
				break
			case 1:	// Left
				GetWindow/Z $HostWinStr wsize // wsizeDC
				if(V_left < size) // shift widow to the right if panel is off screen to the left
					MoveWindow/W=$HostWinStr V_left+(size-V_left)+3, V_top, V_right+(size-V_left)+3, V_bottom
				endif
				Pright= 0
				Ptop= 0
				Pleft= size
				Pbottom= 0
				//return ""
				break
			case 2:	// Bottom (not worrying about this location)
				GetWindow/Z $HostWinStr wsizeDC
				Pright= V_right
				Ptop= 0
				Pleft= 0
				Pbottom= size
				break
			case 3:	// Top
				GetWindow/Z $HostWinStr wsizeDC
				Pright= V_right
				Ptop= size
				Pleft= 0
				Pbottom= 0
				break
		endswitch
	endif
	NameStr = CleanUpName(NameStr,0,50)
	killwindow/Z $HostWinStr+"#"+NameStr // kill previous window instance
	NewPanel/HOST=$HostWinStr/N=$NameStr/EXT=(side)/W=(Pleft,Ptop,Pright,Pbottom)/FLT=2 as TitleStr
	DoWindow/F $HostWinStr
	SetWindow $HostWinStr+"#"+NameStr, hook($NameStr)=ExternalControlPanelHook
	return NameStr
end
// =================================================================================================
// ExternalControlPanelHook(s), hook function for ExternalControlPanel(...)
// =================================================================================================
Function ExternalControlPanelHook(s)
	STRUCT WMWinHookStruct &s
	switch(s.eventCode)
		case 2:		// kill
			getwindow $s.winName wsizeOuter // external window
			variable Ext_left = V_left, Ext_right = V_right
			variable Ext_top = V_top, Ext_bottom = V_bottom
			variable Ext_height = Ext_bottom-Ext_top
			string windowNameStr = ParseFilePath(0, s.winName, "#", 0, 0)
			getwindow $windowNameStr wsize	// host window
			variable Host_left = V_left, Host_right = V_right
			variable Host_top = V_top, Host_bottom = V_bottom
			variable Host_height = Host_bottom-Host_top
			// move Host window up when top external window is close to the top (above and left/right adjusted)
			if(Host_top > Ext_top && ApproxEqual(Ext_left, Ext_left, 2) && ApproxEqual(Ext_right, Host_right, 2))
				if(Ext_top<80)
					MoveWindow/W=$windowNameStr Host_left, Host_top-Ext_height, Host_right, Host_bottom-Ext_height
				endif
			endif
			killwindow/Z $s.winName // external window only
			break
		default:
			break
	endswitch
End

// =================================================================================================
// CopyGraphData(...), rename graph, move waves to new datafolder (in root), duplicate waves
// back to original datafolder
// =================================================================================================
Function CopyGraphData(GraphNameStr, NewGraphNameStr, NewDFStr)
	String GraphNameStr, NewGraphNameStr, NewDFStr
	if(strlen(GraphNameStr)<1)
	 	GraphNameStr = WinName(0,1)
	endif
	// check for windows that you may NOT want to copy
	if(stringmatch(GraphNameStr, "SpHrViewSettings"))
		GetWindow $GraphNameStr, wtitle		
		DoAlert/T="Copy Graph Data" 1, "Copy "+S_value+"  (SpHrViewSettings)?"
		if(v_flag!=1) // Yes
			return 0
		endif
	endif
	// prompt for new graph and datafolder names if already used or not passed
	if(CheckName(NewGraphNameStr, 6) || CheckName(NewDFStr, 11) || strlen(NewGraphNameStr)<1 || strlen(NewDFStr)<1 )
			if(strlen(NewGraphNameStr)<1)
				NewGraphNameStr = UniqueName("Graph", 6, 0) // "Graph"
			endif
			if(strlen(NewDFStr)<1)
				NewDFStr =  ReplaceString("Graph", NewGraphNameStr, "GraphData") // UniqueName("GraphData", 11, 1) // "GraphData"
			endif
		Prompt NewGraphNameStr, "New Graph name:"
		Prompt NewDfStr, "New Datafolder name (root:)"
		DoPrompt "Copy graph & data", NewGraphNameStr, NewDfStr
		if (V_Flag)
			return -1	// User canceled
		endif
	endif
	
	String NewGraphTitleStr = NewGraphNameStr // uncleaned title
	NewGraphNameStr = CleanupName(NewGraphNameStr, 0, 100)
	if(CheckName(NewGraphNameStr, 6))
		Print "Invalid Graph name"
		return 0
	endif
	
	NewDfStr = CleanupName(NewDFStr, 0, 100)
	if(CheckName(NewDFStr, 11))
		Print "Invalid Datafolder name"
		return 0
	endif
	
	NewDfStr = "root:"+NewDFStr
	NewDataFolder/O $NewDfStr
	// can't re-create window before moving waves...
	//Execute WinRecreation(GraphNameStr, 0) //DoWindow/C $NewGraphNameStr //DoWindow/F $NewGraphNameStr //DoWindow/T $NewGraphNameStr, NewGraphNameStr //GetWindow $NewGraphNameStr, wsize //MoveWindow/W=$NewGraphNameStr V_left+18, V_top+22, -1, -1
	// rename window
	DoWindow/W=$GraphNameStr/C $NewGraphNameStr
	DoWindow/T $NewGraphNameStr, NewGraphTitleStr
	// remove controls
	String ChildWinList = ChildWindowList(NewGraphNameStr)
	String winStr, ctrlListStr, ctrlStr
	variable i, j
	// kill controls in window
	ctrlListStr = ControlNameList(NewGraphNameStr)
	for(j=0;j<ItemsInList(ctrlListStr);j++)
		ctrlStr = StringFromList(j,ctrlListStr)
		KillControl/W=$NewGraphNameStr $ctrlStr
	endfor
	// kill controls in child windows
	for(i=0;i<ItemsInList(ChildWinList);i++)
		winStr = StringFromList(i,ChildWinList)
//		ctrlListStr = ControlNameList("#"+winStr)
//		for(j=0;j<ItemsInList(ctrlListStr);j++)
//			ctrlStr = StringFromList(j,ctrlListStr)
//			KillControl/W=$NewGraphNameStr+"#"+winStr $ctrlStr
//		endfor
		KillWindow/Z $NewGraphNameStr+"#"+winStr // killing subwindow kills controls
	endfor
	// original graph size
	GetWindow $NewGraphNameStr gsize
	variable origGraphWidth = V_right-V_left
	variable origGraphHeight = V_bottom-V_top
	ControlBar/T/W=$NewGraphNameStr 0
	ControlBar/B/W=$NewGraphNameStr 0
	ControlBar/L/W=$NewGraphNameStr 0
	ControlBar/R/W=$NewGraphNameStr 0
	SetWindow $NewGraphNameStr note=""
	doupdate	// apparently necessary to resize window 
	// new graph size
	GetWindow $NewGraphNameStr gsize
	variable newGraphWidth = V_right-V_left
	variable newGraphHeight = V_bottom-V_top
	GetWindow $NewGraphNameStr wsize
	// adjust window size to account for loss of controlbars
	GetWindow $NewGraphNameStr psize
	ModifyGraph width=(V_right-V_left)-(newGraphWidth-origGraphWidth), height=(V_bottom-V_top)-(newGraphHeight-origGraphHeight)
	// move widow over a bit
	GetWindow $NewGraphNameStr, wsize
	MoveWindow/W=$NewGraphNameStr V_left+0, V_top, -1, -1
	// set window hook
	SetWindow $NewGraphNameStr, hook(CopyGraph)=$""
	SetWindow $NewGraphNameStr, hook(CopyGraph)=CopyGraphWindowHook
	// move waves to new datafolder and then duplicate them back to their original locations
	GetWindow $NewGraphNameStr, wavelist
	wave/T wlist=W_WaveList
	String origDfWaveStr, origWaveStr, newStr
	for(i=0;i<dimsize(wlist,0);i++)
		origDfWaveStr = wlist[i][1]
		origWaveStr = wlist[i][0]
		wave w = $origDfWaveStr// w = $"root"+origDfWaveStr
		MoveWave w, $NewDfStr+":"+origWaveStr								// move to new df
		Duplicate/O $NewDfStr+":"+origWaveStr, $origDfWaveStr//$"root"+origDfWaveStr	// duplicate back to original df
	endfor
	KillWaves/z wlist
	KillStrings/Z S_value
	Killvariables/Z V_value,V_top,V_right,V_Red,V_minWidth,V_minHeight,V_maxWidth,V_maxHeight,V_left,V_Green,V_Flag,V_bottom,V_Blue,V_Alpha
end

// =================================================================================================
// CopyGraphWindowHook(...), used by CopyGraphData(...) 
// =================================================================================================
Function CopyGraphWindowHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0	
	switch(s.eventCode)
		case 2:		// kill
			CopyGraphKillDatafolders(s.winName)
			hookResult = 0
			break	
		default:
			break	
	endswitch
	return hookResult		// 0 if nothing done, else 1
end

// =================================================================================================
// CopyGraphKillDatafolders(...), used by CopyGraphWindowHook(...) and CopyGraphData(...) 
//	kills datafolder holding CopyGraph wave copies
// =================================================================================================
Function CopyGraphKillDatafolders(GraphNameStr)
	String GraphNameStr
	if(strlen(GraphNameStr)<1)
	 	GraphNameStr = WinName(0,1)
	endif
	setdatafolder root:
	GetWindow $GraphNameStr, wavelist
	killwindow/Z $GraphNameStr
	String DfWaveStr, DfStr
	wave/T W_WaveList=W_WaveList
	if(dimsize(W_WaveList,0)>0)
		DfWaveStr = W_WaveList[0][1]
		if(strlen(DfWaveStr)>0)
			wave w1 = $DfWaveStr
			DfStr = GetWavesDataFolder(w1, 1)
			killdatafolder/Z $DfStr
		endif
	endif
	killwaves/Z W_WaveList
end



// =================================================================================================
// QuerySoundManager(sndDF), loads information about computer's sound manager
// =================================================================================================
Function QuerySoundManager(sndDF)
	String sndDF				// full path to the data folder containing vars
	String dfSav= GetDataFolder(1)
	NewDataFolder/O/S $sndDF
	NewDataFolder/O/S qsmTmp
	SoundInStatus
	if( V_SoundInAGC == 1 )
		SoundInSet agc= 0 // turn agc off if it is on
	endif
	SetDataFolder ::
	String/G InputDeviceName = S_SoundInName
	//Variable/G stereoOK= V_SoundInChansAv > 1
	Variable/G SoundInChannels= V_SoundInChansAv
	//Variable/G isStereo
	//Variable/G sixteenOK=( V_SoundInSampSize&2) != 0
	//Variable/G floatOK=( V_SoundInSampSize&8) != 0
	//Variable/G isSixteen
	Variable/G GainUsed= V_SoundInGain
	Variable/G gainOK= GainUsed >= 0
	if(strsearch(InputDeviceName, "Built", 0) < 0)
		gainOK = 0
		//GainUsed = NaN
	endif
	variable nval = NumVarOrDefault(sndDF+":Snd_fs", 44100)
	Variable/G Snd_fs = nval
	Wave w = :qsmTmp:W_SoundInRates
	String/G rateList= ""
	variable i=1,nrates=numpnts(w),curRateOK= 0
	do
		if( Snd_fs==w[i] )
			curRateOK= 1
		endif
		rateList += num2istr(w[i])+";" //num2Str(w[i])+";"
		i+=1
	while(i<nrates)
	if( !curRateOK )
		Snd_fs= w[1]
	endif
	
	//Print V_SoundInSampSize
	//Wave w = :qsmTmp:V_SoundInSampSize // why ????
	String/G bitList= ""
	if(V_SoundInSampSize&2^0)
		bitList += "8-bit integer;"
	endif
	if(V_SoundInSampSize&2^1)
		bitList += "16-bit integer;"
	endif
	if(V_SoundInSampSize&2^2)
		bitList += "32-bit integer;"
	endif
	if(V_SoundInSampSize&2^3)
		bitList += "32-bit floating point;"
	endif
	if(V_SoundInSampSize&2^4)
		bitList += "64-bit floating point;"
	endif

	KillDataFolder qsmTmp
	SetDataFolder dfSav
	return 0
end


// =================================================================================================
// StopPlaySound(), plays short sound to stop playback
// =================================================================================================
Function StopPlaySound()
	Make/O/N=1024 temp = 0
	Setscale/P x 0, 1/44100, "s", temp
	PlaySound/A=2 temp
	killwaves temp
end

// =================================================================================================
// SoundLoadWave_Error()
//	modest sound loading error report
// =================================================================================================
function SoundLoadWave_Error(Error)
	variable Error
	if(Error)
		if(Error==1)
			DoAlert/T="Sound load error" 0, "Not enough memory to load sound file"
		else
			DoAlert/T="Sound load error" 0, "Unknown error"
		endif
	endif
end

// =================================================================================================
// SetSoundGain(value), attempts to set gain to value and then returns actual gain
// =================================================================================================
Function SetSoundGain(value)
	variable value	// value should range from 0 to 1
	NewDataFolder/S ssgTmp
	SoundInSet gain = value
	SoundInStatus
	value= V_SoundInGain
	KillDataFolder :
	return value
end




function CopyPaste_Bottom_AxisRange(GraphNameStr, doPaste)
	string GraphNameStr
	variable doPaste
	String AxesListStr, AxisStr
	variable i
	AxesListStr = AxisList(GraphNameStr)
	for(i=0;i<ItemsInList(AxesListStr);++i)
		AxisStr = StringFromList(i,AxesListStr)
		if(strsearch(AxisStr, "bottom", 0, 2) >=0) // any axis with "bottom" in name
			if(doPaste)
				if(exists("root:bottom_min_max") == 2)
					NVAR/C bottom_min_max=root:bottom_min_max
					SetAxis/W=$GraphNameStr $AxisStr real(bottom_min_max), imag(bottom_min_max)
				else
					return 0
				endif
			else
				GetAxis/W=$GraphNameStr/Q $AxisStr
				Variable/G/C root:bottom_min_max
				NVAR/C bottom_min_max=root:bottom_min_max
				bottom_min_max = cmplx(v_min, v_max )
			endif
		endif
	endfor
end
function CopyPaste_Left_AxisRange(GraphNameStr, doPaste)
	string GraphNameStr
	variable doPaste
	String AxesListStr, AxisStr
	variable i
	AxesListStr = AxisList(GraphNameStr)
	for(i=0;i<ItemsInList(AxesListStr);++i)
		AxisStr = StringFromList(i,AxesListStr)
		if(strsearch(AxisStr, "left", 0, 2) >=0) // any axis with "left" in name
			if(doPaste)
				if(exists("root:left_min_max") == 2)
					NVAR/C left_min_max=root:left_min_max
					SetAxis/W=$GraphNameStr $AxisStr real(left_min_max), imag(left_min_max)
				else
					return 0
				endif
			else
				GetAxis/W=$GraphNameStr/Q $AxisStr
				Variable/G/C root:left_min_max
				NVAR/C left_min_max=root:left_min_max
				left_min_max = cmplx(v_min, v_max )
			endif
		endif
	endfor
end


Function AxisSliderProc(sa) : SliderControl
	STRUCT WMSliderAction &sa
	Variable curval = sa.curval
	variable i, vmin, vmax, vrange
	String AxesListStr, AxisStr
	strswitch( sa.ctrlName )
		case "sl_leftAxisRange": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							//Print AxesListStr
							//Print strsearch(AxisStr, "left", 0, 2)
							if(strsearch(AxisStr, "left", 0, 2) >=0) // any axis with "left" in name
								GetAxis/Q $AxisStr
								if(!V_flag)
									if(abs(V_min)==abs(V_max)) 	// if symetric about zero
										SetAxis/Z $AxisStr -curval,curval
									elseif(V_min == 0)	// if scaled from zero
										SetAxis/Z $AxisStr 0,curval
									endif
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		case "sl_leftAxisRangeLog": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							//Print AxesListStr
							//Print strsearch(AxisStr, "left", 0, 2)
							if(strsearch(AxisStr, "left", 0, 2) >=0) // any axis with "left" in name
								GetAxis/Q $AxisStr
								if(!V_flag)
									if(abs(V_min)==abs(V_max)) 	// if symetric about zero
										SetAxis/Z $AxisStr -10^curval, 10^curval
									elseif(V_min == 0)	// if scaled from zero
										SetAxis/Z $AxisStr 0, 10^curval // ??
									endif
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		case "sl_leftAxisMax": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "left", 0, 2) >=0)	// any axis with "left" in name
								GetAxis/Q $AxisStr
								//Printf "%g, %g, %g\r", V_min, V_max, curval
								if(!V_flag && curval > V_min)
									SetAxis/Z $AxisStr V_min,curval
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		case "sl_rightAxisMax": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "right", 0, 2) >=0)	// any axis with "right" in name
								GetAxis/Q $AxisStr
								//Printf "%g, %g, %g\r", V_min, V_max, curval
								if(!V_flag && curval > V_min)
									SetAxis/Z $AxisStr V_min,curval
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		case "sl_leftAxisMin": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "left", 0, 2) >=0)	// any axis with "left" in name
								GetAxis/Q $AxisStr
								if(!V_flag && curval < V_max)
									SetAxis/Z $AxisStr curval, V_max
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		
		case "sl_bottomAxisRange": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "bottom", 0, 2) >=0) // any axis with "bottom" in name
								GetAxis/Q $AxisStr
								if(!V_flag)
									if(abs(V_min)==abs(V_max)) 	// if symetric about zero
										SetAxis/Z $AxisStr -curval,curval
									elseif(V_min == 0)	// if scaled from zero
										SetAxis/Z $AxisStr 0,curval
									endif
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
//		case "sl_bottomAxisRangeLog": 
//			switch( sa.eventCode )
//				case -1: // control being killed
//					break
//				default:
//					if( sa.eventCode & 3 ) // & 1?
//						AxesListStr = AxisList("")
//						for(i=0;i<ItemsInList(AxesListStr);++i)
//							AxisStr = StringFromList(i,AxesListStr)
//							if(strsearch(AxisStr, "bottom", 0, 2) >=0) // any axis with "bottom" in name
//								GetAxis/Q $AxisStr
//								if(!V_flag)
//									if(abs(V_min)==abs(V_max)) 	// if symetric about zero
//										SetAxis/Z $AxisStr -10^curval,10^curval
//									elseif(V_min == 0)	// if scaled from zero
//										SetAxis/Z $AxisStr 0,10^curval
//									endif
//								endif
//							endif
//						endfor
//					endif // sa.eventCode & 3
//					break
//			endswitch // sa.eventCode
//			break
		case "sl_bottomAxisMax": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
								GetAxis/Q $AxisStr
								if(!V_flag && curval > V_min)
									SetAxis/Z $AxisStr V_min,curval
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		case "sl_bottomAxisMaxLog": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
								GetAxis/Q $AxisStr
								if(!V_flag && 10^curval > V_min)
									SetAxis/Z $AxisStr V_min, 10^curval
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
			
		case "sl_bottomAxisMin": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
								GetAxis/Q $AxisStr
								if(!V_flag && curval < V_max)
									SetAxis/Z $AxisStr curval, V_max
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
		case "sl_bottomAxisMinLog": 
			switch( sa.eventCode )
				case -1: // control being killed
					break
				default:
					if( sa.eventCode & 3 ) // & 1?
						AxesListStr = AxisList("")
						for(i=0;i<ItemsInList(AxesListStr);++i)
							AxisStr = StringFromList(i,AxesListStr)
							if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
								GetAxis/Q $AxisStr
								if(!V_flag && 10^curval < V_max)
									SetAxis/Z $AxisStr 10^curval, V_max
								endif
							endif
						endfor
					endif // sa.eventCode & 3
					break
			endswitch // sa.eventCode
			break
			
		case "sl_bottomAxisShift_reZero": 
			switch( sa.eventCode )
				case 2:
				case 3:
				case 4:
				case 9:
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
							GetAxis/Q $AxisStr
							if(!V_flag)
								vmin= V_min; vmax= V_max
								vmin = max(0,vmin)
								vrange = vmax-vmin
								SetAxis/Z $AxisStr curval, curval+vrange
							endif
						endif
					endfor
					break
				case -1: // control being killed
					break
				default:
					break
			endswitch // sa.eventCode
			break
	endswitch // sa.ctrlName
	return 0
End


// SetVarProcIndex
// gets previous or next value in wave
// valWave - wave of values to step through with SetVarProc
// setVal - stepped value (limits={x,x,1})
// getClosestIndex -  getClosestIndex=0 if mouse up (or down) event, getClosestIndex=1 if Enter key, Live update
function SetVarProcIndex(valWave, setVal, getClosestIndex)
	wave valWave
	variable setVal, getClosestIndex
	variable Index //, val
	if(getClosestIndex)
		duplicate/O/FREE valWave, tempvalWave
		tempvalWave -= setVal
		tempvalWave = (tempvalWave < 0) ? -tempvalWave : tempvalWave
		wavestats/Q/M=1 tempvalWave
		Index = V_minloc
		//val = round(valWave[Index])
		return Index
	endif
	variable InterpVal = BinarySearchInterp(valWave, setVal )
	if(numtype(InterpVal)==0)
		Index = round(InterpVal)
		variable direction = InterpVal - Index
		if(direction < 0)
			Index -= 1
		elseif(direction > 0)
			Index += 1
		endif
		//val= round(valWave[Index])
	else
		//Printf "%g, %g, %g, %g, %g\r", setVal, InterpVal, val, valWave[0], valWave[numpnts(valWave)-1]
		if(setVal < valWave[0])
			//val = round(valWave[0])
			Index= 0
		elseif(setVal > valWave[numpnts(valWave)-1])
			//val = round(valWave[numpnts(valWave)-1])
			Index= numpnts(valWave)-1
		endif
	endif
	return Index
end 

Function AxisButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	variable i
	String AxesListStr, AxisStr
	variable minX, maxX, rangeX, keyPressed
	strswitch( ba.ctrlName )
		case "b_bottomAxisAuto": 
			switch( ba.eventCode )
				case 2: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
							GetAxis/Q $AxisStr
							if(!V_flag)
								SetAxis/A/E=0 $AxisStr
							endif
							// disable slider(s)
							ControlInfo sl_bottomAxisShift_reZero
							if(V_flag==7) 
								Slider sl_bottomAxisShift_reZero, disable=2
							endif
						endif
					endfor
					break
				case -1: // control being killed
					break
			endswitch // ba.eventCode
			break
		case "b_bottomAxisAuto_SymZero": 
			switch( ba.eventCode )
				case 2: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
							GetAxis/Q $AxisStr
							if(!V_flag)
								SetAxis/A/E=2 $AxisStr
							endif
						endif
					endfor
					break
				case -1: // control being killed
					break
			endswitch // ba.eventCode
			break
		case "b_bottomAxisAuto_reZero": 
			switch( ba.eventCode )
				case 2: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "bottom" in name
							GetAxis/Q $AxisStr
							if(!V_flag)
								SetAxis/A/E=1 bottom
							endif
						endif
					endfor
					break
				case -1: // control being killed
					break
			endswitch // ba.eventCode
			break
		case "b_bottomAxisExpendContract": 
			switch( ba.eventCode )
				case 1: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "bottom", 0, 2) >=0)	// any axis with "left" in name
							GetMarquee/Z $AxisStr
							if(V_flag)
								SetAxis $AxisStr v_left, v_right
								GetMarquee/K
							else
								GetAxis/Q $AxisStr
								minX= v_min; maxX= v_max; rangeX= v_max-v_min
								keyPressed = GetKeyState(0)
								// contract if command, option, or control pressed
								if(keyPressed == 1 || keyPressed == 2 || keyPressed == 16) // command, option, or control
									minX -= rangeX/2//4
									maxX += rangeX/2//4
								else // zoom if keys not pressed
									minX += rangeX/4
									maxX -= rangeX/4
								endif
								SetAxis $AxisStr minX, maxX
							endif
						endif
					endfor
					break
				case -1: // control being killed
				break
			endswitch // ba.eventCode
			break
			
		case "b_leftAxisAuto": 
			switch( ba.eventCode )
				case 2: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "left", 0, 2) >=0)	// any axis with "left" in name
							GetAxis/Q $AxisStr
							if(!V_flag)
								SetAxis/A=2/E=0 $AxisStr
							endif
						endif
					endfor
					break
				case -1: // control being killed
					break
			endswitch // ba.eventCode
			break
			
		case "b_leftAxisAuto_reZero": 
			switch( ba.eventCode )
				case 2: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "left", 0, 2) >=0)	// any axis with "left" in name
							GetAxis/Q $AxisStr
							if(!V_flag)
								SetAxis/A=2/E=1 $AxisStr
							endif
						endif
					endfor
					break
				case -1: // control being killed
					break
			endswitch // ba.eventCode
			break
			
		case "b_leftAxisExpendContract": 
			switch( ba.eventCode )
				case 1: // mouse up
					AxesListStr = AxisList("")
					for(i=0;i<ItemsInList(AxesListStr);++i)
						AxisStr = StringFromList(i,AxesListStr)
						if(strsearch(AxisStr, "left", 0, 2) >=0)	// any axis with "left" in name
							GetMarquee/Z $AxisStr
							if(V_flag)
								SetAxis $AxisStr v_left, v_right
								GetMarquee/K
							else
								GetAxis/Q $AxisStr
								minX= v_min; maxX= v_max; rangeX= v_max-v_min
								keyPressed = GetKeyState(0)
								// contract if command, option, or control pressed
								if(keyPressed == 1 || keyPressed == 2 || keyPressed == 16) // command, option, or control
									minX -= rangeX/2//4
									maxX += rangeX/2//4
								else // zoom if keys not pressed
									minX += rangeX/4
									maxX -= rangeX/4
								endif
								SetAxis $AxisStr minX, maxX
							endif
						endif
					endfor
					break
				case -1: // control being killed
				break
			endswitch // ba.eventCode
			break
	endswitch // ba.ctrlName
	return 0
End

// == general function to determine is two values (a & b) are roughly equal
// a, b: input variables
// slop: ± range "slop" for determining if the a & b are roughout equal
function ApproxEqual(a, b, slop)
	variable a, b, slop
	if(a <= b+slop && a >= b-slop)
		return 1
	else
		return 0
	endif
end



Function MakeImageEdgesWave(centers, edgesWave)
	Wave centers	// Input
	Wave edgesWave	// Receives output
	Variable N=numpnts(centers)
	Redimension/N=(N+1) edgesWave
	edgesWave[0]=centers[0]-0.5*(centers[1]-centers[0])
	edgesWave[N]=centers[N-1]+0.5*(centers[N-1]-centers[N-2])
	edgesWave[1,N-1]=centers[p]-0.5*(centers[p]-centers[p-1])
End

function ColorScaleLookUp(dfStr, waveNameStr, minHz, maxHz, CFs)
	string dfStr, waveNameStr
	variable minHz, maxHz
	wave CFs
	if(minHz<0 || maxHz<0)
		Duplicate/O CFs, $dfStr+waveNameStr//"CFs_ColorScaleLookUp"
		wave CFs_ColorScaleLookUp = $dfStr+waveNameStr//"CFs_ColorScaleLookUp"
		wavestats/Q/M=1 CFs
		CFs_ColorScaleLookUp -= v_min
		CFs_ColorScaleLookUp /= v_max-v_min
	else
		variable minHzIndex = round(BinarySearchInterp(CFs, minHz))
		if(!numtype(minHzIndex)==0)
			if(minHz < CFs[0])
				minHzIndex= 0
			elseif(minHz > CFs[numpnts(CFs)-1])
				minHzIndex= numpnts(CFs)-1
			endif
		endif
		variable maxHzIndex = round(BinarySearchInterp(CFs, maxHz))
		if(!numtype(maxHzIndex)==0)
			if(maxHz < CFs[0])
				maxHzIndex= 0
			elseif(maxHz > CFs[numpnts(CFs)-1])
				maxHzIndex= numpnts(CFs)-1
			endif
		endif
		Duplicate/O/R=[minHzIndex,maxHzIndex]  CFs, $dfStr+waveNameStr//"CFs_ColorScaleLookUp"
		wave CFs_ColorScaleLookUp = $dfStr+waveNameStr//"CFs_ColorScaleLookUp"
		wavestats/Q/M=1/R=[minHzIndex,maxHzIndex] CFs
		CFs_ColorScaleLookUp -= v_min
		CFs_ColorScaleLookUp /= v_max-v_min
	endif
end

// Generate frequency tick marks corresponding to center frequency point numbers
// passing a negative desiredTicksStr value generates a tick without a label
// getFrequencyTicks(dfStr, "-50;100;-150;200;-250;300;400;500;600;-700;800;-900;1000;-1250;1500;-1750;2000;-3000;4000;-6000;8000", CFs, 1)
function getFrequencyTicks(dfStr, desiredTicksStr, CFs, kHzTicks)
	string dfStr, desiredTicksStr
	wave CFs
	variable kHzTicks // convert Hz to kHz
	variable numticks = itemsinlist(desiredTicksStr)
	variable numCFs = numpnts(CFs)
	variable CFmax = wavemax(CFs)
	variable CFmin = wavemin(CFs)
	variable CFdelta = CFmax - CFmin
	make/O/N=(numticks) $dfStr+"CFs_TickVals" = NAN
	wave CFs_TickVals = $dfStr+"CFs_TickVals"
	make/O/N=(numticks)/T $dfStr+"CFs_TickTxt" = ""
	wave/T CFs_TickTxt = $dfStr+"CFs_TickTxt"
	variable i, val, cnt=0
	string str
	for(i=0;i<numticks;++i)
		str = stringfromlist(i,desiredTicksStr)
		val = str2num(str)
		if(val<0) // tick without label if < 0
			FindLevel/Q/P CFs, -val
			if(!v_flag)
				CFs_TickVals[cnt] = V_LevelX
				cnt+=1
			endif
		else
			FindLevel/Q/P CFs, val
			if(!v_flag)
				CFs_TickVals[cnt] = V_LevelX
				if(kHzTicks)
					sprintf str, "%.2g", val/1000 // kHz
					if(strsearch(str,"0.",0)>=0)
						str = str[1,strlen(str)]
					endif
					CFs_TickTxt[cnt] = str
				else
					CFs_TickTxt[cnt] = str
				endif
				cnt+=1
			endif
		endif
	endfor
	Redimension/N=(cnt) CFs_TickVals, CFs_TickTxt
end

//function golay(n)
//	variable  n
//	// create a pair of golay codes of order n. Use assorted randomizations to preserve complementary series properties
//	// n = order of golay sequences, a = golay sequence 1, b = golay sequence 2
//	// note : xcorr(a) + xcorr(b) = delta_fn. Derived from: Anthony Leonardo, Caltech, 3/01, MatLab code
//	variable a1 = sign(1)
//	variable a2 = sign(1)
//	Make/O/N=(2) a , b
//	a[0] = a1
//	a[1] = a2
//	b[0] = a1
//	b[1] = -a2
//	variable i
//	for(i=1;i<n;i+=1)
//		duplicate/O a,  pa
//		duplicate/O b, nb, pb
//		nb = -nb
//		b *= sign(enoise(1))
//		Concatenate/NP/O  {pa, nb}, b
//		a *= sign(enoise(1))
//		Concatenate/NP/O  {pa, pb}, a
//		if(sign(enoise(1)) > 0)
//			Reverse a
//		endif
//		if(sign(enoise(1)) > 0)
//			Reverse b
//		endif
//	endfor
//	killwaves pa, pb, nb
//End


// Per column StatsQuartiles
// 0=minValue
// 1=maxValue
// 2=Median
// 3=Q25
// 4=Q75
// 5=IQR,
// 6=triMean
// 7=Mode
// 8=MedianAbsoluteDeviation
function StatsQuantilesPCST(dfStr, wStr)
	string dfStr, wStr
	wave w = $dfStr+wStr
	if(dimsize(w,1) <= 0)
		return 0
	endif
	variable i
	for(i=0;i<dimsize(w,1);++i)
		MatrixOp/O/FREE temp=col(w,i)
		wavestats/Q temp // check number of NaNs ?
		if(V_npnts > 3)
			StatsQuantiles/Q/iNaN/TM temp
			wave W_StatsQuantiles=W_StatsQuantiles
			if(i==0)
				Make/O/N=(dimsize(W_StatsQuantiles,0), dimsize(w,1)) $dfStr+"W_StatsQuantilesPCST"
			endif
			wave W_StatsQuantilesPCST = $dfStr+"W_StatsQuantilesPCST"
			W_StatsQuantilesPCST[][i] = W_StatsQuantiles[p]
		else
			if(i==0)
				Make/O/N=(dimsize(W_StatsQuantiles,0), dimsize(w,1)) $dfStr+"W_StatsQuantilesPCST"
			endif
			W_StatsQuantilesPCST[][i] = NaN
		endif
	endfor
	//killwaves/Z W_StatsQuantiles
	SetScale d 0,0,WaveUnits(w, -1), W_StatsQuantilesPCST
end

function lastNANrow(wave w)
	FindValue/FNAN/Z w
	variable lastNaNPnt = V_value
	if(lastNaNPnt < 0)
		return dimsize(w,0)
	else
		return lastNaNPnt
	endif
end

function [ Variable maxVal, Variable maxValLoc ] CorrelationPeak(wave w1, wave w2, variable Normalize, string CorrelationWaveStr)
	if(strlen(CorrelationWaveStr)>1)
		Duplicate/O w1, $CorrelationWaveStr
		wave Correlation = $CorrelationWaveStr
	else
		Duplicate/FREE/O w1, Correlation
	endif
	Correlate/NODC w2, Correlation
	if(Normalize)
		variable normVal = norm(w1)*norm(w2)
		Correlation /= normVal
	endif
	wavestats/Q/M=1 Correlation
	maxVal = V_max
	maxValLoc = V_maxLoc
End

Function RemoveTraces(WindowStr)
	String WindowStr
	//DoWindow/F $WindowStr
	String str, traceListStr= TraceNameList(WindowStr, ";", 1)
	variable i
	for(i=ItemsInList(traceListStr)-1;i>=0;--i) // backwards
		str = StringFromList(i,traceListStr)
		RemoveFromGraph/W=$WindowStr $str
	endfor
End
Function RemoveImages(WindowStr)
	String WindowStr
	//DoWindow/F $WindowStr
	String str, imageListStr= ImageNameList(WindowStr, ";")
	variable i
	for(i=ItemsInList(imageListStr)-1;i>=0;--i) // backwards
		str = StringFromList(i,imageListStr)
		RemoveImage/W=$WindowStr $str
	endfor
End


Function [ Variable wLeft, Variable wRight, Variable wTop, Variable wBottom ] getWindowSize(String WinNameStr)
	DoWindow $WinNameStr
	wLeft = 0; wRight = 0; wTop = 0; wBottom = 0 // 500  500
	if(v_flag)
		GetWindow/Z $WinNameStr wsize // wsizeRM // wsizeOuter
		wLeft = V_left
		wRight = V_right
		wTop = V_top
		wBottom = V_bottom
	endif
	return [ wLeft, wRight, wTop, wBottom]
End

Function [ Variable mLeft, Variable mRight, Variable mTop, Variable mBottom ] getWindowMargins(String WinNameStr)
	DoWindow $WinNameStr
	variable gLeft, gRight, gTop, gBottom
	mLeft = 0; mRight = 0; mTop = 0; mBottom = 0
	if(v_flag)
		GetWindow/Z $WinNameStr gsize
		gLeft = V_left
		gRight = V_right
		gTop = V_top
		gBottom = V_bottom
		GetWindow/Z $WinNameStr psize
		mLeft = round(V_left-gLeft)
		mRight = round(gRight-V_right)
		mTop = round(V_top-gTop)
		mBottom = round(gBottom-V_bottom)
	endif
	return [ mLeft, mRight, mTop, mBottom]
End


// note that getColorTableRGB calls ColorTab2Wave each time...
// [r,g,b] = getColorTableRGB(ColorTable, ReverseTable, i, minIndex, maxIndex) // 0,0,0 if minIndex == maxIndex   //  minIndex >= 1 ?
Function [Variable r, Variable g, Variable b, Variable a, Variable err] getColorTableRGB(String ColorTableStr, Variable ReverseTable, Variable Index, Variable minIndex, Variable maxIndex)
	Print "Use SpHr_ColorTable instead of getColorTableRGB()"
	r=0;g=0;b=0
	err=0
	if(WhichListItem(ColorTableStr, CTabList())>=0)
		ColorTab2Wave $ColorTableStr
		wave ColorTable=M_colors
	else
		wave ColorTable=$ColorTableStr // parseFilePath(0, ColorTableStr, ":", 1, 0)
		if(!waveexists(ColorTable))
			r=0;g=0;b=0
			DoAlert/T="Warning" 0, "ColorTable wave not built-in or found as wave in current datafolder: "+ColorTableStr
			err=1
			return [r,g,b,a,err]
		endif
	endif
	// reverse
	if(SpHr_BitTest(ReverseTable,0))  // bit 0
		Reverse/P/DIM=0 ColorTable
	endif														// -1 ?
	Variable ind = ((Index-minIndex)/(maxIndex-minIndex-0)) * (DimSize(ColorTable,0)-1)
	ind = min(ind, dimsize(ColorTable,0)-1)
	ind = max(ind, 0)
	if(numtype(ind)!=0)
		killwaves/Z M_colors
		return [0,0,0,65535,err]
	endif
	r= ColorTable[ind][0]
	g= ColorTable[ind][1]
	b= ColorTable[ind][2]
	if(dimsize(ColorTable,1)>=4)
		a= ColorTable[ind][3]
	endif
	killwaves/Z M_colors
	return [r,g,b,a,err]
End


Function/wave GetColorTable(String ColorTableStr, Variable ReverseInvertTable, Variable Alfa, String WaveNameStr)
	Print "Use SpHr_ColorTable instead of GetColorTable()"
	if(WhichListItem(ColorTableStr, CTabList())>=0)
		ColorTab2Wave $ColorTableStr
		wave M_colors=M_colors
		Duplicate/O M_colors, $WaveNameStr
		KillWaves/Z M_colors
		wave tableWave = $WaveNameStr
	else
		Variable N = 201
		strswitch(ColorTableStr)
			case "RedGreenBlue":
				N = 201
				Make/O/W/U/N=(N,4) $WaveNameStr
				wave tableWave = $WaveNameStr
				tableWave = 0
				tableWave[][0] =  x*(65535/(N-1))
				
				tableWave[0,(N/2)-1][1] = x*(65535/(N-1))
				tableWave[(N/2),N-1][1] = 65535 - x*(65535/(N-1))
				
				tableWave[][1] = min(tableWave[p][1],32767)
				tableWave[][1] = ((tableWave[p][1])*2) // boost gray to green
				
				tableWave[][2] = 65535 - x*(65535/(N-1))
				tableWave[][3] = Alfa*65535
				break
			case "RedGreenBlueDark":
				N = 201
				Make/O/W/U/N=(N,4) $WaveNameStr = 0
				wave tableWave = $WaveNameStr
				tableWave[0, (N*0.75)-1][0] = x*(65535/(((N-1)*0.75)))
				tableWave[N*0.75, N-1][0] = (65535*2.5) - x*(65535/(((N-1)*0.5)))
				
				tableWave[0,N/2-1][1] = x*(65535/(N-1))
				tableWave[N/2,N-1][1] = 65535 - x*(65535/(N-1))
				tableWave[][1] *= 2 // boost gray to green
				
				tableWave[(N*0.25)-1, N-1][2] = (65535/0.75) - x*(65535/(((N-1)*0.75)))
				tableWave[0, (N*0.25)-0][2] = (65535*0.5) + x*(65535/((N-1)*0.5))
				
				tableWave[][3] = Alfa*65535
				break
				
			case "RedGrayBlue":
			case "RedGreyBlue":
				N = 201
				Make/O/W/U/N=(N,4) $WaveNameStr
				wave tableWave = $WaveNameStr
				if(0)
					variable i
					for(i=0;i<N;++i)
               		tableWave[i][0] = i*(65535/(N-1));
//               		if(i>195)
//               			Printf "%g, %g, %g\r", i, (65535/(N-1)), i*(65535/(N-1))
//               		endif
               	endfor
				else
					tableWave[][0] = x*(65535/(N-1))
					
					tableWave[0,N/2-1][1] = x*(65535/(N-1))
					tableWave[N/2,N-1][1] = 65535 - x*(65535/(N-1))
					
					tableWave[][2] = 65535 - x*(65535/(N-1))
					tableWave[][3] = Alfa*65535
				endif
				break

			case "RedGrayBlueDark":
			//case "RedGrayBlueDark":
				N = 201
				Make/O/W/U/N=(N,4) $WaveNameStr = 0
				wave tableWave = $WaveNameStr
				tableWave[0, (N*0.75)-1][0] = x*(65535/(((N-1)*0.75)))
				tableWave[N*0.75, N-1][0] = (65535*2.5) - x*(65535/(((N-1)*0.5)))
				
				tableWave[0,N/2-1][1] = x*(65535/(N-1))
				tableWave[N/2,N-1][1] = 65535 - x*(65535/(N-1))
				tableWave[][1] *= 1.333 // boost gray to green
				
				tableWave[(N*0.25)-1, N-1][2] = (65535/0.75) - x*(65535/(((N-1)*0.75)))
				tableWave[0, (N*0.25)-0][2] = (65535*0.5) + x*(65535/((N-1)*0.5))
				
				tableWave[][3] = Alfa*65535
				break
				
				//case "RedGrayBlueLight":
				//case "RedGrayBlueLight":
				//	N = 201
				//	Make/O/N=(N,4) $WaveNameStr = 0
				//	wave tableWave = $WaveNameStr
				//	tableWave[(N*0.25)-1, N-1][0] = x*(65535/(((N-1)*0.75))) - (65535*0.3333)
				//	tableWave[0, (N*0.25)][0] = (65535) - x*(65535/(((N-1)*0.25)))
				//	tableWave[0, (N*0.25)][1] = (65535) - x*(65535/(((N-1)*0.25)))
				//	tableWave[(N*0.25)-1, N-1][1] = x*(65535/(((N-1)*0.25))) - (65535*3)
				//	//tableWave[(N*0.25)-1, (N*0.75)-1][1] = 0//x*(65535/(((N-1)*0.25))) - (65535*3)
				//	tableWave[(N*0.25)-1, N-1][2] = x*(65535/(((N-1)*0.25))) - (65535*3)
				//	tableWave[0, (N*0.75)-1][2] = (65535*1) - x*(65535/((N-1)*0.75))
				//	tableWave[][3] = Alfa*65535
				//	break

			default:
				DoAlert/T="Warning" 0, "ColorTable not built-in: "+ColorTableStr
				return $""
		endswitch
	endif

	// reverse
	if(SpHr_BitTest(ReverseInvertTable,0))  // bit 0
		Reverse/P/DIM=0 tableWave
	endif
	if(SpHr_BitTest(ReverseInvertTable,1))  // bit 1,  invert red
		tableWave[][0] = 65535 - tableWave
	endif
	if(SpHr_BitTest(ReverseInvertTable,2))  // bit 2,  invert green
		tableWave[][1] = 65535 - tableWave
	endif
	if(SpHr_BitTest(ReverseInvertTable,3))  // bit 3,  invert blue
		tableWave[][2] = 65535 - tableWave
	endif

	tableWave = max(0,tableWave)
	tableWave = min(65535,tableWave)
	
	return tableWave
End


// Execute/Z/Q/P "KillAllWavesInDatafolder(\""+DFtoKillStr+"\")"
// Execute/Z/Q/P "KillAllWavesInDatafolder(\""+DFtoKillStr[0,strlen(DFtoKillStr)-2]+"\")" // remove trailing ":"
Function KillAllWavesInDatafolder(DFtoKillStr)
	string DFtoKillStr
	string dfSavStr = GetDataFolder(1)
	SetDataFolder $DFtoKillStr
	KillWaves/Z/A
	SetDataFolder $dfSavStr
End
// Execute/Z/Q/P "KillAllVariablesInDatafolder(\""+DFtoKillStr+"\")"
// Execute/Z/Q/P "KillAllVariablesInDatafolder(\""+DFtoKillStr[0,strlen(DFtoKillStr)-2]+"\")" // remove trailing ":"
Function KillAllVariablesInDatafolder(DFtoKillStr)
	string DFtoKillStr
	string dfSavStr = GetDataFolder(1)
	SetDataFolder $DFtoKillStr
	KillVariables/Z/A
	SetDataFolder $dfSavStr
End

// WaveListString= TraceNameList("Waveforms_Window", ";", 1)
// WaveListString= ImageNameList("Waveforms_Window", ";")
// with empty SuffixStr:
// Execute/Z/Q/P "KillWavesInListString(\""+WaveListString+"\",\""+datafolderString+"\",\"\")"
// with SuffixStr:
// Execute/Z/Q/P "KillWavesInListString(\""+WaveListString+"\",\""+datafolderString+"\",\"SuffixStr\")"	
Function KillWavesInListString(WaveListString, datafolderString, SuffixStr)
	string WaveListString, datafolderString, SuffixStr
	string wStr
	int64 i
	for(i=0;i<ItemsInList(WaveListString);++i)
		wStr = StringFromList(i,WaveListString)
		wStr += SuffixStr // optional suffix string
		wave/Z w= $datafolderString+wStr
		if(waveexists(w))
			killwaves/Z w
		endif
	endfor
End


// Kill all Graphs with NoteStrKey in note field
// Execute/Z/Q/P "KillGraphsWithNoteField(\" NoteStrKey \")"	
Function KillGraphsWithNoteField(String NoteStrKey)
	String str, GraphNameList = WinList("*", ";","WIN:1")
	int64 i
	for(i=0;i<itemsinlist(GraphNameList);++i)
		str =stringfromlist(i, GraphNameList)
		GetWindow/Z $str note 
		if(strsearch(S_Value, NoteStrKey, 0)>=0)
			KillWindow/Z $str
		endif
	endfor
	KillStrings/Z S_Value
End