﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

/// Version 1.00
/// © 2015 Brian Nelson, all rights reserved.

/// General Procedures
#include "IgorGeneralProcedures"
/// SpHrView Procedures
#include "SpHrView_Window"
#include "SpHrView_Filter"
#include "SpHrView_Spectra"
#include "SpHrView_Waves"
#include "SpHrView_Record"
#include "SpHrView_BinauralSignal"
#include "SpHrView_Distribution"
#include "SpHrView_Cues"
#include "SpHrView_IPDNeurons"
#include "SpHrView_ILDNeurons"
#include "SpHrView_ActivityAtTime"
#include "SpHrView_ActivityOverTime"
#include "SpHrView_IR"
#include "SpHrView_Correlation"
#include "SpHrView_Transfer"
#include "SpHrView_Realtime"

/// Main SpHrView menu
Menu "SpatialHearing"
	"Open Sound File...", /Q, SpHrView_OpenFile(0)
	"-"
	"Recent Sound Files...", /Q, SpHrView_OpenFile(1)
	"Select Wave", /Q, SpHrView_WaveListWindow(SpHrView_df_waves, 1)
	"Save Sound...", /Q, SpHrView_SaveWave()
	"-"
	submenu "View (v)"
		"Carrier Waveforms (w)", /Q, SpHrView_DisplayCarrierWaves(0,0)
		"Carrier Envelopes (e)", /Q, SpHrView_DisplayCarrierWaves(1,0)
		"-"
		"Filter Bank Waveforms", /Q, SpHrView_DisplayFilteredWaves(0)
		"Filter Bank Envelopes", /Q, SpHrView_DisplayFilteredWaves(1)
		"-"
		"Amplitude Spectra", /Q, SpHrView_DisplaySpectra()
		"Amplitude Spectra Over Time", /Q, SpHrView_DisplaySTFT()
		"-"
		"Correlation", /Q, SpHr_Correlation()
		"-"
		"Cue Distributions", /Q, SpHrView_CueHistogramOptionsWindow(-1,-1)
		"Cues Over Time", /Q, SpHrView_CuesOptionsWindow(-1,-1)
		"-"
		"Representations At Time" , /Q, SpHrView_AtTime_OptionsWindow(-1,-1)
		"Representations Over Time" , /Q, SpHrView_RepOverTime_OptionsWindow(-1,-1)
		"-"
		"Waveform Statistics", /Q, SpHrView_WaveformStatistics(1)
	end
	"-"
	"Record...", /Q, SpHrView_Rec_Window()
	"Realtime...", /Q, SpHrView_Realtime_Window() 
	"Generate Binaural Signals...", /Q, SpHrView_BinauralSignalWindow(-1)
	"Generate BRIRs...", /Q, SpHrView_TransferFuncLogSweepWindow()
	"-"
	submenu "Settings"
		"View", /Q, SpHrView_Settings(0,0)
	end
End

///// general color table popup
//Menu "SpHrView_ColorTablesPopup", contextualmenu
//	SpHr_CTabList()+"-"+CTabList()
//	//"*COLORTABLEPOP*" // using plain lists to add SpHr_CTabList()
//End


Function/S ColorTablePopup(String CurrentSelection)
	SpHr_ColorTable/List ""
	String ListStr = S_ColorTableList+"-"+CTabList()
	Variable SelectedNum = WhichListItem(CurrentSelection, listStr)
	listStr = RemoveListItem(SelectedNum, ListStr)
	listStr = AddListItem(CurrentSelection+"!", listStr, ";", SelectedNum)
	PopupContextualMenu listStr
	if(V_flag<=0)
		Return ""
	endif
	Return ReplaceString("!", S_selection, "")
End


static Function BeforeFileOpenHook(refNum,fileName,path,type,creator,kind)
	Variable refNum,kind
	String fileName,path,type,creator
	Variable handledOpen= 0
	// Load sounds and movies using SpHrView
	PathInfo $path
	String filePathStr = S_path
	string infostr = FunctionInfo("SpHrView_LoadPeaksFromFile") 
	if (strlen(infostr))
		NewPath/C/O/Q SoundsPath, filePathStr
		string suffixStr = (ParseFilePath(4, fileName, ":", 0, 0))
		if(stringmatch(suffixStr,"wav") || stringmatch(suffixStr,"aif") || stringmatch(suffixStr,"aiff") || stringmatch(suffixStr,"mov"))
			String filePathAndName = filePathStr+fileName
			KillWindow/Z SpHrViewPeaksWindow
			Execute/Q "SpHrView_Defaults()" // check defaults
			Execute/Q "SpHrView_LoadPeaksFromFile(\""+filePathAndName+"\", 0, 0, 0, 0, 0, 0)"
			handledOpen= 1
		endif
	endif
	return handledOpen	 // 1 tells Igor not to open the file
End

static Function IgorBeforeQuitHook(unsavedExp, unsavedNotebooks, unsavedProcedures )
	variable unsavedExp, unsavedNotebooks, unsavedProcedures
	DFREF saveDFR = GetDataFolderDFR()
	if(DataFolderExists(SpHrView_df_waves))
		SetDataFolder $SpHrView_df_waves
		string waveListStr = WaveList("*",";","")
		variable sizeSum = 0
		Int64 i
		for(i=0;i<ItemsInList(waveListStr);++i)
			wave w = $StringFromList(i, waveListStr) 
			sizeSum += NumberByKey("SIZEINBYTES", WaveInfo(w,0))/1000000
		endfor
		if(sizeSum > 200) // MB
			string AlertStr
			SPrintf AlertStr, "Waves in datafolder:  %s are using %g MB.\rDelete these waves before quitting Igor Pro?", SpHrView_df_waves, sizeSum
			DoAlert/T="Warning" 1, AlertStr
			if(v_flag==1)
				for(i=0;i<ItemsInList(waveListStr);++i)
					wave w = $StringFromList(i, waveListStr) 
					killwaves/Z w
				endfor
			endif
		endif
	endif
	SetDataFolder saveDFR
	return 0
End


//Function IgorStartOrNewHook(igorApplicationNameStr)
//	String igorApplicationNameStr
//	PathInfo Igor						// puts path value into (local) S_path
//	printf "\"%s\" (re)starting\r", S_path + igorApplicationNameStr
//End