﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// SpHrView_Correlation

strconstant SpHrView_df_correlation = "root:SpHrView:Correlation:"

// =================================================================================================
// SpHrView_Cues_Defaults(), Binaural cues defaults
// =================================================================================================
Function SpHrView_Correlation_Defaults()
	DFREF saveDFR = GetDataFolderDFR()
	String FolderStr
	// Make "SpHrView" folder, SpHrView_df
	if(!DataFolderExists(SpHrView_df))
		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Correlation" folder, SpHrView_df_correlation
	if(!DataFolderExists(SpHrView_df_correlation))
		FolderStr= SpHrView_df_correlation[0,strlen(SpHrView_df_correlation)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_correlation
	variable nval
	nval= NumVarOrDefault(SpHrView_df_correlation+"NoDC", 1)
	variable/G NoDC = nval
	nval= NumVarOrDefault(SpHrView_df_correlation+"Normalize", 1)
	variable/G Normalize = nval
	String sval
	sval= StrVarOrDefault(SpHrView_df_correlation+"ColorTableStr", "RedWhiteBlue")	
	string/G ColorTableStr = sval
	nval= NumVarOrDefault(SpHrView_df_correlation+"ReverseColorTable", 1)
	variable/G ReverseColorTable = nval
	nval= NumVarOrDefault(SpHrView_df_correlation+"displayAverage", 1)
	variable/G displayAverage = nval
	nval= NumVarOrDefault(SpHrView_df_correlation+"displayMaximumLags", 1)
	variable/G displayMaximumLags = nval

	SetDataFolder saveDFR
End

Function SpHr_Correlation()
	DFREF saveDFR = GetDataFolderDFR()
	SpHrView_Correlation_Defaults()
	SetDataFolder $SpHrView_df_correlation
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, usingAxis
	[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
	wave waveforms=$SpHrView_df_files+"waveforms"
	variable numCols = dimsize(waveforms,1)
	if(numCols>2)
		DoAlert 0, "SpHrView_Cues: waveform has > 2 channels/columns"
		return 0
	endif
	if(usingAxis)
		SpHrView_DrawSelection(2, startSec, endSec)
	else
		SpHrView_DrawSelection(1, startSec, endSec)
	endif
	variable fs = round(1/dimdelta(waveforms,0))
	// get filter banks
	SpHrView_CenterFrequencies()
	wave CFs = $SpHrView_df_filter+"CFs"
	SpHr_FilterBankBiquadCoefs fs, CFs, coefBank
	SpHr_FilterBankBiquad /RMS/Dest=$SpHrView_df_filter+"FilterBanks" CFs, coefBank, waveforms
	wave FilterBanks = $SpHrView_df_filter+"FilterBanks"
	killwaves/Z coefBank
	
	NVAR NoDC = $SpHrView_df_correlation+"NoDC"
	NVAR Normalize = $SpHrView_df_correlation+"Normalize"
	SpHr_Correlate/NODC=(NoDC)/NORM=(Normalize)/Dest=Correlation/E={CorrelationPeaks}/A=CorrelationAvg FilterBanks

	SpHrView_CorrelationWindow()
	
	SetDataFolder saveDFR
End

// =================================================================================================
// SpHrView_CorrelationWindow()
// =================================================================================================
Function SpHrView_CorrelationWindow()
	DFREF saveDFR = GetDataFolderDFR()
	
	wave/Z Correlation = $SpHrView_df_correlation+"Correlation"
	if(!waveexists(Correlation))
		return 0
	endif
	variable bottomAxisRangeMin = DimOffset(Correlation, 0)
	String WinTitleStr = "Correlation", WinNameStr = "Correlation_Window"
	DoWindow $WinNameStr
	variable WindowExists = V_flag
	String traceListStr
	if(WindowExists) // remove traces and images
		//GetAxis /W=$WinNameStr/Q bottom // this screws up when updating
		//bottomAxisRangeMin = max(abs(V_min),V_max)
		RemoveTraces(WinNameStr)
		RemoveImages(WinNameStr)
	else
		NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
		NVAR screenVert= $SpHrView_df_globals+"screenVert"
		NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
		NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
		variable wdth=screenHoriz*ViewWidthProportion/2
		variable hgt=screenVert*ViewHeightProportion*2
		Variable  winleft=(screenHoriz/6), winRight, winTop=400, winBottom=270
		DoWindow SpHrViewPeaksWindow
		if(V_flag)
			[winleft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
			winleft = winleft
			winTop = winBottom+30
			Variable marginleft, marginRight, marginTop, marginBottom
			[marginleft, marginRight, marginTop, marginBottom] = getWindowMargins("SpHrViewPeaksWindow")	
		endif		
		Display /K=1/W=(winleft, winTop, winleft+wdth, winTop+hgt) as WinTitleStr
		DoWindow/C $WinNameStr
		ModifyGraph/W=$WinNameStr margin(left)=marginleft, margin(top)=25, margin(right)=marginRight, margin(bottom)=35
		ControlBar/W=$WinNameStr /T 37
		NewPanel/HOST=$WinNameStr/FG=(FL,FT,GR,GT)/HOST=# 
		RenameWindow #,PTop
		
		// controls
		Variable topPos = 11
		Variable leftPos = 25
		GroupBox group_ColorTable win=$WinNameStr#PTop, pos={leftPos-8,6.5},size={175,23}
		SetVariable sv_ColorTable, win=$WinNameStr#PTop, pos={leftPos,topPos},size={107,14.00},bodyWidth=75,title="Colors"
		SetVariable sv_ColorTable, win=$WinNameStr#PTop, fSize=9,focusRing=0
		SetVariable sv_ColorTable, win=$WinNameStr#PTop, value= $SpHrView_df_correlation+"ColorTableStr"
		leftPos += 108
		Button b_ColorTable, win=$WinNameStr#PTop, pos={leftPos,topPos-1.5},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,16.00},title=""
		Button b_ColorTable, win=$WinNameStr#PTop, proc=SpHrView_Correlation_ButtonProc,focusRing=0,help={"Choose a new color table"}
		leftPos += 20
		CheckBox ck_ReverseColorTable win=$WinNameStr#PTop, title="⤵︎",pos={leftPos,topPos-1.5},side=1, fSize=12
		CheckBox ck_ReverseColorTable win=$WinNameStr#PTop, variable= $SpHrView_df_correlation+"ReverseColorTable"
		CheckBox ck_ReverseColorTable win=$WinNameStr#PTop, proc=SpHrView_Correlation_CheckProc,focusRing=0,help={"Reverse color table"}
		leftPos += 53
		GroupBox group_correlationOptions win=$WinNameStr#PTop, pos={leftPos-8,6.5},size={160,23}
		CheckBox ck_NODC win=$WinNameStr#PTop, title="Covariance",pos={leftPos,topPos-1.5},side=0, fSize=9, focusRing=0
		CheckBox ck_NODC win=$WinNameStr#PTop, variable= $SpHrView_df_correlation+"NoDC"
		CheckBox ck_NODC win=$WinNameStr#PTop, proc=SpHrView_Correlation_CheckProc,focusRing=0,help={"Remove DC of all signals before correlation"}
		leftPos += 80
		CheckBox ck_NORM win=$WinNameStr#PTop, title="Normalize",pos={leftPos,topPos-1.5},side=0, fSize=9, focusRing=0
		CheckBox ck_NORM win=$WinNameStr#PTop, variable= $SpHrView_df_correlation+"Normalize"
		CheckBox ck_NORM win=$WinNameStr#PTop, proc=SpHrView_Correlation_CheckProc,focusRing=0,help={"Normalize ot maximum of 1"}
		leftPos += 85
		GroupBox group_displayOptions win=$WinNameStr#PTop, pos={leftPos-8,6.5},size={145,23}
		CheckBox ck_displayAverage win=$WinNameStr#PTop, title="Average",pos={leftPos,topPos-1.5},side=0, fSize=9, focusRing=0
		CheckBox ck_displayAverage win=$WinNameStr#PTop, variable= $SpHrView_df_correlation+"displayAverage"
		CheckBox ck_displayAverage win=$WinNameStr#PTop, proc=SpHrView_Correlation_CheckProc,focusRing=0,help={"Display across frequency average"}
		leftPos += 65
		CheckBox ck_displayMaximumLags win=$WinNameStr#PTop, title="Lag times",pos={leftPos,topPos-1.5},side=0, fSize=9, focusRing=0
		CheckBox ck_displayMaximumLags win=$WinNameStr#PTop, variable= $SpHrView_df_correlation+"displayMaximumLags"
		CheckBox ck_displayMaximumLags win=$WinNameStr#PTop, proc=SpHrView_Correlation_CheckProc,focusRing=0,help={"Display lags a maxima"}
		
		leftPos += 80
		Button b_UpdateCorrelation, win=$WinNameStr#PTop, pos={leftPos,topPos-4}, size={50.00,21.00},fSize=10,title="Update"
		Button b_UpdateCorrelation, win=$WinNameStr#PTop, proc=SpHrView_Correlation_ButtonProc,focusRing=0

		leftPos += 65
		Slider sl_bottomAxisRange win=$WinNameStr#PTop, vert=0, pos={leftPos,topPos+1},size={120,20},side=0,proc=SpHrView_Correlation_SliderProc
		Slider sl_bottomAxisRange win=$WinNameStr#PTop, limits={bottomAxisRangeMin/500,bottomAxisRangeMin,0}, value= bottomAxisRangeMin/2
		Slider sl_bottomAxisRange win=$WinNameStr#PTop, help={"Adjust bottom time axis range"}
		leftPos += 130
		TitleBox t_adjustTimeAxis pos={leftPos,topPos},title="time axis range",frame=0
		
		SetActiveSubwindow $WinNameStr //##
		ModifyGraph/W=$WinNameStr cbRGB=(65534,65534,65534)
	endif
	
	AppendImage/W=$WinNameStr Correlation
	SVAR ColorTableStr =  $SpHrView_df_correlation+"ColorTableStr"
	NVAR ReverseColorTable = $SpHrView_df_correlation+"ReverseColorTable"
	NVAR Normalize = $SpHrView_df_correlation+"Normalize"
	SpHr_ColorTable/R=(ReverseColorTable)/Dest=ColorTable ColorTableStr
	if(Normalize)
		ModifyImage/W=$WinNameStr Correlation ctab= {-1,1,ColorTable,0}
	else
		ModifyImage/W=$WinNameStr Correlation ctab= {*,*,ColorTable,0}
	endif
	ColorScale/W=$WinNameStr/C/N=text0/F=0/B=1/M/H={15,0,5}/A=MT/X=0.00/Y=0.00/E=2 vert=0,width=130,height=5,frame=0.00,image=Correlation,font="Arial",fsize=10,tickLen=2.00,tickThick=0.50,ZisZ=1
	
	Label/W=$WinNameStr left "frequency (kHz) \\u#2"
	Label/W=$WinNameStr bottom "time (\\U)"
	SetAxis/W=$WinNameStr bottom bottomAxisRangeMin/2, -bottomAxisRangeMin/2
	ModifyGraph/W=$WinNameStr lblMargin=5
	
	NVAR displayAverage = $SpHrView_df_correlation+"displayAverage"
	if(displayAverage)
		wave CorrelationAvg = $SpHrView_df_correlation+"CorrelationAvg"
		AppendToGraph/W=$WinNameStr/R CorrelationAvg
		ModifyGraph/W=$WinNameStr lsize=0.5,rgb=(0,0,0)
		SetAxis/W=$WinNameStr/A=2/E=2 right
	endif
	
	NVAR displayMaximumLags = $SpHrView_df_correlation+"displayMaximumLags"
	if(displayMaximumLags)
		wave CorrelationPeaks = $SpHrView_df_correlation+"CorrelationPeaks"
		AppendToGraph/W=$WinNameStr/Vert CorrelationPeaks[][0] // maximum lag times [0]
		ModifyGraph/W=$WinNameStr rgb=(0,0,0),mode(CorrelationPeaks)=3,marker(CorrelationPeaks)=8,msize(CorrelationPeaks)=2
	endif
	
	ModifyGraph/W=$WinNameStr userticks(left)={::Filter:CFs_TickVals,::Filter:CFs_TickTxt}
	ModifyGraph/W=$WinNameStr tick=2,fSize=10,axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,standoff=0,btLen=4,btThick=0.5,stLen=2,stThick=0.5,font="Arial"
	ModifyGraph/W=$WinNameStr nticks(bottom)=10
	ModifyGraph/W=$WinNameStr zero(bottom)=1,zeroThick(bottom)=0.5
	
	Slider sl_bottomAxisRange win=$WinNameStr#PTop, limits={bottomAxisRangeMin/500,bottomAxisRangeMin,0}, value= bottomAxisRangeMin/2
	
	SetDataFolder saveDFR
End


Function SpHrView_Correlation_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 1: // Mouse down
			strswitch(ba.ctrlName)
				case "b_UpdateCorrelation":
					SpHr_Correlation()
					break
				case "b_ColorTable":
					SVAR ColorTableStr = $SpHrView_df_correlation+"ColorTableStr"
					String tempStr = ColorTablePopup(ColorTableStr)
					if(strlen(tempStr) > 1)
						ColorTableStr = tempStr
						SpHr_Correlation()
					endif
//					PopupContextualMenu/N "SpHrView_ColorTablesPopup" //"*COLORTABLEPOP*"
//					if(V_flag>0)// && V_kind==13)
//						SVAR ColorTableStr =  $SpHrView_df_correlation+"ColorTableStr"
//						ColorTableStr = S_selection
//						SpHr_Correlation()
//					endif
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch
	return 0
End

Function SpHrView_Correlation_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	switch( cba.eventCode )
		case 2: // mouse up
			Variable checked = cba.checked
			strswitch(cba.ctrlName)
				case "ck_ReverseColorTable":
				case "ck_NODC":
				case "ck_NORM":
				case "ck_displayAverage":
				case "ck_displayMaximumLags":
					SpHr_Correlation()
				break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch
	return 0
End

Function SpHrView_Correlation_SliderProc(sa) : SliderControl
	STRUCT WMSliderAction &sa
	switch( sa.eventCode )
		case -3: // Control received keyboard focus
		case -2: // Control lost keyboard focus
		case -1: // Control being killed
			break
		default:
			if( sa.eventCode & 1 ) // value set
				Variable curval = sa.curval
				strswitch(sa.ctrlName)
					case "sl_bottomAxisRange":
						string winstr = sa.win
						winstr = ReplaceString("#PTop", winstr, "") 
						GetAxis/W=$winstr/Q bottom
						if(V_flag==0)
							SetAxis/W=$winstr bottom -curval,curval
						endif
						break
				endswitch
				
			endif
			break
	endswitch

	return 0
End
