﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// SpHrView_Filter

strconstant SpHrView_df_filter = "root:SpHrView:Filter:"

Menu "SpatialHearing"
	submenu "Settings"
		"Filter Bank", /Q, SpHrView_Settings_FilterBank(0,0)
	end
End

function SpHr_DimensionsScaleCheck(wave/Z w_new, wave/Z w_ref)
	variable Update = 0
	if(!waveexists(w_new))
		Update = 1
	else
		// update if dimensions of w differ re: FilterBanks
		if(dimsize(w_new,0) != dimsize(w_ref,0))
			Update = 1
		else
			// update if w scaling differs > 0.1 ms re: FilterBanks
			if( !SpHr_equal( DimOffset(w_new,0), leftx(w_ref), 0.0001))
				Update = 1
			endif
			if( !SpHr_equal( DimSize(w_new,0)*DimDelta(w_new,0)+DimOffset(w_new,0), DimSize(w_ref,0)*DimDelta(w_ref,0)+DimOffset(w_ref,0), 0.0001))
				Update = 1
			endif
		endif
	endif
	return Update
End


// =================================================================================================
// SpHrView_getCFindex(), Return neartest index and corresponding freqeuncy
//	wave CFs = $SpHrView_df_filter+"CFs"
//	Variable index, Hz
//	[index, Hz] = SpHrView_getCFindex(CFs, dval)
// =================================================================================================
Function [Variable index, Variable Hz] SpHrView_getCFindex(Wave CFs, Variable Freq)
	index = round(BinarySearchInterp(CFs, Freq))
	if(numtype(index)==0)
		Hz= round(CFs[index])
	else
		if(Hz < CFs[0])
			Hz = round(CFs[0])
			index= 0
		elseif(Hz > CFs[numpnts(CFs)-1])
			Hz = round(CFs[numpnts(CFs)-1])
			index= numpnts(CFs)-1
		else
			// skip if zero
		endif
	endif
	return [ index, Hz]
End


// =================================================================================================
// SpHrView_FilterDefaults(), Make SpHrView:Filter datafolder and defaults
// Called from function SpHrView_Defaults() if function SpHrView_FilterDefaults() exits
// =================================================================================================
Function SpHrView_FilterDefaults()
	DFREF DFRsaved= GetDataFolderDFR()
	String sval, FolderStr
	// Make "SpHrView" folder, SpHrView_df
	if(!DataFolderExists(SpHrView_df))
		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Filter" folder, SpHrView_df_filter
	if(!DataFolderExists(SpHrView_df_filter))
		FolderStr= SpHrView_df_filter[0,strlen(SpHrView_df_filter)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_filter // Set to SpHrView_df_filter
	Variable nval
	nval= NumVarOrDefault(SpHrView_df_filter+"filterbankUpdateFlag", 1)			// flag to indicate that filter banks need updating
	variable/G filterbankUpdateFlag = nval															
	nval= NumVarOrDefault(SpHrView_df_filter+"lowFreq", 100)							// filter bank lowest frequency
	variable/G lowFreq = nval															
	nval= NumVarOrDefault(SpHrView_df_filter+"highFreq", 6000)						// filter bank highest frequency
	variable/G highFreq = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"numCFs", 64)								// number of filters in filter bank
	variable/G numCFs = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"RMSscale", 1)							// filters have equal RMS amplitudes or equal areas
	variable/G RMSscale = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"clearDelays", 1)						// clear delay lines when filtering
	variable/G clearDelays = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"analysisFreq", 500)					// current analysis frequency
	variable/G analysisFreq = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"analysisFreq_Envelope", 1500)		// current Envelope analysis frequency
	variable/G analysisFreq_Envelope = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"analysisFreq_Carrier", 500)			// current Carrier analysis frequency
	variable/G analysisFreq_Carrier = nval
	
	sval= StrVarOrDefault(SpHrView_df_filter+"CFsSpacingTypeStr", "Logarithmic")	// Glasberg & Moore, Logarithmic, Linear
	string/G CFsSpacingTypeStr = sval
	
	sval= StrVarOrDefault(SpHrView_df_filter+"FilterBankEnvelopeColorCode", "SeaLandAndFire")	// Envelope image color scheme
	string/G FilterBankEnvelopeColorCode = sval
	sval= StrVarOrDefault(SpHrView_df_filter+"FilterBankWaveformColorCode", "RedWhiteBlue")		// Waveform image color scheme
	string/G FilterBankWaveformColorCode = sval
	nval= NumVarOrDefault(SpHrView_df_filter+"FilterBankEnvelopeColorReverse", 0)					// reverse Envelope color scheme
	variable/G FilterBankEnvelopeColorReverse = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"FilterBankWaveformColorReverse", 0)				// reverse color scheme
	variable/G FilterBankWaveformColorReverse = nval

	nval= NumVarOrDefault(SpHrView_df_filter+"FilterBank_DisplaydB", 1)								// display filter bank as dB
	variable/G FilterBank_DisplaydB = nval
	
	nval= NumVarOrDefault(SpHrView_df_filter+"waveformFilter", 0)				// apply waveform filter
	variable/G waveformFilter = nval
	sval= StrVarOrDefault(SpHrView_df_filter+"waveformFilterTypeStr", "Auditory filter")		// apply waveform filter type
	string/G waveformFilterTypeStr = sval
	nval= NumVarOrDefault(SpHrView_df_filter+"waveformFilterHz1", 500)						// waveform filter frequency #1
	variable/G waveformFilterHz1 = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"waveformFilterHz2", 1000)					// waveform filter frequency #2
	variable/G waveformFilterHz2 = nval
	nval= NumVarOrDefault(SpHrView_df_filter+"waveformFilterVal", 2)						// waveform Value e.g., Q
	variable/G waveformFilterVal = nval
	SetDataFolder DFRsaved
End

// =================================================================================================
// SpHrView_FilterPeaksWindowControls(), Add filter check to SpHrViewPeaksWindow
// Called from function SpHrView_PeaksWindow() if function SpHrView_FilterPeaksWindowControls() exits
// =================================================================================================
Function SpHrView_FilterPeaksWindowControls()
	DoWindow SpHrViewPeaksWindow
	if(v_flag)
		NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
		//NVAR screenVert= $SpHrView_df_globals+"screenVert"
		NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
		//NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
		variable wdth=screenHoriz*ViewWidthProportion
		CheckBox ck_waveformFilter win=SpHrViewPeaksWindow#PTop, pos={wdth-145,10}, side=1, title="filter", variable=$SpHrView_df_filter+"waveformFilter"
		CheckBox ck_waveformFilter win=SpHrViewPeaksWindow#PTop, help={"Apply auditory filter to processed waveforms (models, envelope views, etc.)"}
		CheckBox ck_waveformFilter win=SpHrViewPeaksWindow#PTop, proc=SpHrView_FilterCheckProc
	endif
End

// =================================================================================================
// SpHrView_FilterCheckProc(cba), Checkbox procedure for SpHrView_Filter
// =================================================================================================
Function SpHrView_FilterCheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	Variable checked = cba.checked
	string traceListStr="", wStr=""
	switch( cba.eventCode )
		case 2: // mouse up
			strswitch( cba.ctrlName )
				case "ck_clearDelays":
				case "ck_RMSscale":
					DoWindow FilterBank_Window
					if(v_flag)
						SpHrView_DisplayFilteredWaves(-1) // update
					endif
					break
			
				case "ck_waveformFilter":
					if(checked)
						SpHrView_WaveformFilterWindow()
					else
						killwindow/Z WaveFilterControlWindow
					endif
					break
					
				case "b_FilterBankEnvelopedB":
				case "b_FilterBankEnvelopeColorReverse":
					SpHrView_DisplayFilteredWaves(1)
					break
				case "b_FilterBankWaveformColorReverse":
					SpHrView_DisplayFilteredWaves(0)
					break
					
			endswitch
		case -1: // control being killed
			break
	endswitch
End

// =================================================================================================
// SpHrView_FilterSetVarProc(sva), SetVariable procedure for SpHrView_Filter
// =================================================================================================
Function SpHrView_FilterSetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	Variable dval = sva.dval
	//String sval = sva.sval
	variable analysisIndex
	strswitch( sva.ctrlName )
		case "sv_lowFreq":
		case "sv_highFreq":
		case "sv_numCFs":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
				case 3: // Live update
					SpHrView_CenterFrequencies()
					DoWindow FilterBank_Window
					if(v_flag)
						SpHrView_DisplayFilteredWaves(-1) // update
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
		case "sv_waveformFilterHz1":
		case "sv_waveformFilterHz2":
		case "sv_waveformFilterVal":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
				case 3: // Live update
				case 4: // Mouse scroll wheel up
				case 5: // Mouse scroll wheel downS
				case 8: // End edit 
					DoWindow SpHrViewPeaksWindow 
					if(v_flag)
						DoWindow/F SpHrViewPeaksWindow 
					endif
					DoWindow FilterBank_Window
					if(v_flag)
						SpHrView_DisplayFilteredWaves(-1) // update
					endif
					DoWindow Waveforms_Window
					if(v_flag)
						SpHrView_DisplayCarrierWaves(-1, -1)
					endif
					DoWindow Spectra_Window
					if(v_flag)
						SpHrView_DisplaySpectra()
					endif
					DoWindow STFT_Window
					if(v_flag)
						SpHrView_DisplaySTFT()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
		
	endswitch
	return 0
End


// =================================================================================================
// SpHrView_FilterButtonProc, Button procedure for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_FilterButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	String tempStr
	strswitch( ba.ctrlName )
		case "b_FilterBankOptions":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_Settings_FilterBank(0,0)
					break
			endswitch
		break
		case "b_waveformFilterType":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_WaveformFilterMenu()
					break
			endswitch
			break
		case "b_FilterBankEnvelopeColorCode":
			switch( ba.eventCode )
				case 1: // mouse down
					SVAR FilterBankEnvelopeColorCode = $SpHrView_df_filter+"FilterBankEnvelopeColorCode"
					tempStr = ColorTablePopup(FilterBankEnvelopeColorCode)
					if(strlen(tempStr)>1)
						FilterBankEnvelopeColorCode = tempStr
					endif
					SpHrView_DisplayFilteredWaves(1)
				
//					SVAR FilterBankEnvelopeColorCode = $SpHrView_df_filter+"FilterBankEnvelopeColorCode"
//					PopupContextualMenu/N "SpHrView_ColorTablesPopup" //"*COLORTABLEPOP*"
//					if(V_flag>0)// && V_kind==13)
//						FilterBankEnvelopeColorCode = S_selection
//						DoUpdate
//					endif
//					SpHrView_DisplayFilteredWaves(1)
					break
			endswitch
			break
		case "b_FilterBankWaveformColorCode":
			switch( ba.eventCode )
				case 1: // mouse down
					SVAR FilterBankWaveformColorCode = $SpHrView_df_filter+"FilterBankWaveformColorCode"
					tempStr = ColorTablePopup(FilterBankWaveformColorCode)
					if(strlen(tempStr)>1)
						FilterBankWaveformColorCode = tempStr
					endif
					SpHrView_DisplayFilteredWaves(0)
					
//					SVAR FilterBankWaveformColorCode = $SpHrView_df_filter+"FilterBankWaveformColorCode"
//					PopupContextualMenu/N "SpHrView_ColorTablesPopup" //"*COLORTABLEPOP*"
//					if(V_flag>0)// && V_kind==13)
//						FilterBankWaveformColorCode = S_selection
//						DoUpdate
//					endif
//					SpHrView_DisplayFilteredWaves(0)
					break
			endswitch
			break
			
			
		case "b_CFsSpacingType":
			switch( ba.eventCode )
				case 1: // mouse down
					PopupContextualMenu "Glasberg & Moore;Logarithmic;Linear"
					if(strlen(S_selection)>1)
						SVAR CFsSpacingTypeStr = $SpHrView_df_filter+"CFsSpacingTypeStr"
						CFsSpacingTypeStr = S_selection
						SpHrView_CenterFrequencies()
						DoWindow FilterBank_Window
						if(v_flag)
							SpHrView_DisplayFilteredWaves(-1) // update
						endif
						DoUpdate
					endif
					break
			endswitch
			break
			
		case "b_DisplayWaveforms":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilteredWaves(0)
					break
			endswitch
			break
		case "b_DisplayEnvelopes":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilteredWaves(1)
					break
			endswitch
			break
			
		case "b_viewCFsTable":
			switch( ba.eventCode )
				case 1: // mouse down
					wave CFs = $SpHrView_df_filter+"CFs"
					DoWindow CenterFreqeunciesWindow
					if(v_flag)
						KillWindow/Z CenterFreqeunciesWindow
					else
						GetMouse
						Edit/K=1/W=(V_left,V_top,V_left+100,V_top+400) CFs as "Center frequencies"
						ModifyTable alignment=1
						DoWindow/C CenterFreqeunciesWindow
						// could set filterbankUpdateFlag if wave CFs modified in table 
						//NVAR filterbankUpdateFlag = $SpHrView_df_filter+"filterbankUpdateFlag"
						//filterbankUpdateFlag = 1
					endif
					break
			endswitch
			break
			
		case "b_viewFilterbankImpulses":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilterBankFilters(0,0, 0)
					break
			endswitch
			break
		case "b_viewFilterbankImpulseEnvelopes":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilterBankFilters(0,0, 1)
					break
			endswitch
			break
		case "b_viewFilterbankSpectra":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilterBankFilters(0,0, 2)
					break
			endswitch
			break
		case "b_viewFilterbankEnvWidths":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilterBankFilters(0,0, 3)
					break
			endswitch
			break
		case "b_viewFilterbankRMSscales":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DisplayFilterBankFilters(0,0, 4)
					break
			endswitch
			break

	endswitch
End

// =================================================================================================
// SpHrView_Settings_FilterBank(variable winLeft, variable winTop), auditory filter bank settings
// =================================================================================================
Function SpHrView_Settings_FilterBank(variable winLeft, variable winTop)
	if(winLeft == 0 || winTop == 0)
		GetMouse
		winLeft=V_left; winTop=V_top
	endif
	killwindow/Z FilterBankSettings
	SpHrView_FilterDefaults()
	NewPanel /N=FilterBankSettings/K=1/W=(winLeft, winTop, winLeft+190,winTop+385) as "Filter Bank"
	variable top = 5
	GroupBox group_FilterBands pos={6,top},size={180,115}
	top += 10
	SetVariable sv_numCFs win=FilterBankSettings, pos={10.00,top},size={170,14.00},bodyWidth=60,fSize=10,title="Number of bands"
	SetVariable sv_numCFs win=FilterBankSettings, limits={1,512,4},value= $SpHrView_df_filter+"numCFs", proc=SpHrView_FilterSetVarProc
	top += 24
	SetVariable sv_highFreq win=FilterBankSettings, pos={10.00,top},size={170,14.00},bodyWidth=60,fSize=10,title="High frequency (Hz)"
	SetVariable sv_highFreq win=FilterBankSettings, limits={100,24000,100},value= $SpHrView_df_filter+"highFreq", proc=SpHrView_FilterSetVarProc
	top += 24
	SetVariable sv_lowFreq win=FilterBankSettings, pos={10.00,top},size={170,14.00},bodyWidth=60,fSize=10,title="Low frequency (Hz)"
	SetVariable sv_lowFreq win=FilterBankSettings, limits={10,24000,10},value= $SpHrView_df_filter+"lowFreq", proc=SpHrView_FilterSetVarProc
	top += 27
	SetVariable sv_CFsSpacingTypeStr win=FilterBankSettings, pos={20.00,top},size={143,15.00},bodyWidth=100,fSize=10,title="Spacing",noedit=1
	SetVariable sv_CFsSpacingTypeStr win=FilterBankSettings, value= $SpHrView_df_filter+"CFsSpacingTypeStr", proc=SpHrView_FilterSetVarProc
	Button b_CFsSpacingType win=FilterBankSettings, pos={165.00,top-1},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,16.00},title=""
	Button b_CFsSpacingType win=FilterBankSettings, proc=SpHrView_FilterButtonProc

	top += 35
	GroupBox group_DelaysScale win=FilterBankSettings, pos={6,top},size={180,47}
	top += 5
	CheckBox ck_clearDelays win=FilterBankSettings, title=" Clear delay lines",fSize=10,pos={25,top},side=0,proc=SpHrView_FilterCheckProc
	CheckBox ck_clearDelays win=FilterBankSettings, variable= $SpHrView_df_filter+"clearDelays"
	top += 20
	CheckBox ck_RMSscale win=FilterBankSettings, title=" Scale for equal RMS",fSize=10,pos={25,top},side=0,proc=SpHrView_FilterCheckProc
	CheckBox ck_RMSscale win=FilterBankSettings, variable= $SpHrView_df_filter+"RMSscale"
	
	top += 30
	GroupBox group_Views win=FilterBankSettings, fSize=10,title="View",pos={6,top},size={180,175}
	top += 20
	Button b_viewCFsTable win=FilterBankSettings, pos={22.00,top}, size={150.0,20.0},fSize=10,title="Frequencies Table"
	Button b_viewCFsTable win=FilterBankSettings, proc=SpHrView_FilterButtonProc
	
	top += 25
	Button b_viewFilterbankImpulses win=FilterBankSettings, pos={22.00,top}, size={150.0,20.0},fSize=10,title="Impulses"
	Button b_viewFilterbankImpulses win=FilterBankSettings, proc=SpHrView_FilterButtonProc
	
	top += 25
	Button b_viewFilterbankImpulseEnvelopes win=FilterBankSettings, pos={22.00,top}, size={150.0,20.0},fSize=10,title="Impulse Envelopes"
	Button b_viewFilterbankImpulseEnvelopes win=FilterBankSettings, proc=SpHrView_FilterButtonProc
	
	top += 25
	Button b_viewFilterbankSpectra win=FilterBankSettings, pos={22.00,top}, size={150.0,20.0},fSize=10,title="Spectra"
	Button b_viewFilterbankSpectra win=FilterBankSettings, proc=SpHrView_FilterButtonProc
	
	top += 25
	Button b_viewFilterbankEnvWidths win=FilterBankSettings, pos={22.00,top}, size={150.0,20.0},fSize=10,title="Impulse Envelope Widths"
	Button b_viewFilterbankEnvWidths win=FilterBankSettings, proc=SpHrView_FilterButtonProc
	
	top += 25
	Button b_viewFilterbankRMSscales win=FilterBankSettings, pos={22.00,top}, size={150.0,20.0},fSize=10,title="Equal RMS scales"
	Button b_viewFilterbankRMSscales win=FilterBankSettings, proc=SpHrView_FilterButtonProc
	
	top += 35
	//GroupBox group_keepWindowOpen win=FilterBankSettings, pos={6,top-3},size={180,22}
	CheckBox ck_keepWindowOpen win=FilterBankSettings, title=" 📌 ",fSize=10,pos={25,top},side=0,value=0,help={"Keep window open"}
	
	// window hook
	DoUpdate/W=FilterBankSettings // for Igor 9
	SetWindow FilterBankSettings, hook(FilterBankSettings)=SpHrView_WaveformFilterWindowHook
	
End

// =================================================================================================
// SpHrView_CenterFrequencies(), get new center frequencies for filtering
// =================================================================================================
function SpHrView_CenterFrequencies()
	NVAR filterbankUpdateFlag = $SpHrView_df_filter+"filterbankUpdateFlag"
	filterbankUpdateFlag = 1
	NVAR lowFreq= $SpHrView_df_filter+"lowFreq"
	NVAR highFreq= $SpHrView_df_filter+"highFreq"
	if(lowFreq > highFreq)
		variable temp = lowFreq
		lowFreq = highFreq
		highFreq = temp
	endif
	NVAR numCFs= $SpHrView_df_filter+"numCFs"
	variable SpacingTypeNum = 2;
	SVAR CFsSpacingTypeStr = $SpHrView_df_filter+"CFsSpacingTypeStr"
	strswitch(CFsSpacingTypeStr)
		case "Glasberg & Moore":
			SpacingTypeNum = 1
			break
		case "Logarithmic":
			SpacingTypeNum = 2
			break
		case "Linear":
			SpacingTypeNum = 3
			break
	endswitch
	SpHr_FilterCenterFreqs/R/Type=(SpacingTypeNum) lowFreq, highFreq, numCFs, $SpHrView_df_filter+"CFs" // /E=CFs_imageEdges
	wave CFs = $SpHrView_df_filter+"CFs"
	wave/Z CFs_imageEdges = $SpHrView_df_filter+"CFs_imageEdges"
	if(waveexists(CFs_imageEdges))
		CFs_imageEdges /= 1000
	endif
	
	getFrequencyTicks(SpHrView_df_filter, "-50;100;-150;200;-250;300;400;500;600;-700;800;-900;1000;-1250;1500;-1750;2000;-3000;4000;-6000;8000", CFs, 1)
	// limit analysisFreq
	NVAR analysisFreq= $SpHrView_df_filter+"analysisFreq"
	analysisFreq = min(analysisFreq, highFreq)
	analysisFreq = max(analysisFreq, lowFreq)
	// update color scale loo-up tables
	ColorScaleLookUp(SpHrView_df_filter, "CFs_ColorScaleLookUp", -1, -1, CFs)
//	variable CFmin = wavemin(CFs)
//	NVAR maxCarrierHz = $ksBV_Models_ITD_Globals_dfStr+"maxCarrierHz"
//	ColorScaleLookUp(SpHrView_df_filter, "CFs_CarrierScaleLookUp", CFmin, maxCarrierHz, CFs)
//	variable CFmax = wavemax(CFs)
//	NVAR minEnvelopeHz = $ksBV_Models_ITD_Globals_dfStr+"minEnvelopeHz"
//	ColorScaleLookUp(SpHrView_df_filter, "CFs_EnvelopeScaleLookUp", minEnvelopeHz, CFmax, CFs)
end

// =================================================================================================
//	SpHrView_WaveformFilterWindow(...), settings panel for WaveformFilterApply(...)
// =================================================================================================
function SpHrView_WaveformFilterWindow()
	variable winLeft=250, winTop=50
	SVAR waveformFilterTypeStr= $SpHrView_df_filter+"waveformFilterTypeStr"
	string titleStr = waveformFilterTypeStr// + " filter"
	DoWindow SpHrViewPeaksWindow // for placement & sizing
	if(v_flag)
		GetWindow/Z SpHrViewPeaksWindow wsize
		winLeft = V_right-300
		winTop = V_top-0
	endif
	variable width = 300
	strswitch(waveformFilterTypeStr)
		case "Auditory filter":
			width = 250
			break
		case "Band-pass - Low to High":
			width = 330
			break
		case "Band-pass - Bandwidth":
			width = 330
			break	
		case "Low-pass":
			width = 250
			break
		case "High-pass":
			width = 250
			break
		case "Notch":
			width = 250
			break
		case "No Filter":
			killwindow/Z WaveFilterControlWindow
			return 0
			break
	endswitch
	killwindow/Z WaveFilterControlWindow
	//DoWindow WaveFilterControlWindow
	//if(!v_flag)
		NewPanel/FLT=1/K=1/W=(winLeft, winTop, winLeft+width, winTop+30) as titleStr
		//NewPanel/K=1/W=(winLeft, winTop, winLeft+300, winTop+30) as "Waveform Filter"
		DoWindow/C WaveFilterControlWindow
		TitleBox typeTxt title="Type",pos={7,9},frame=0,fSize=10
		Button b_waveformFilterType win=WaveFilterControlWindow, title="",pos={34,7},size={15,16},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_FilterButtonProc
		Button b_waveformFilterType win=WaveFilterControlWindow, help = {"Filter type"}
		SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={60,8}, title=" ", value=$SpHrView_df_filter+"waveformFilterHz1", proc=SpHrView_FilterSetVarProc
		SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, pos={125,8}, title=" ", value=$SpHrView_df_filter+"waveformFilterHz2", proc=SpHrView_FilterSetVarProc
		SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, pos={185,8}, title=" ", value=$SpHrView_df_filter+"waveformFilterVal", proc=SpHrView_FilterSetVarProc
		SetActiveSubwindow _endfloat_
		DoUpdate/W=WaveFilterControlWindow // for Igor 9
		SetWindow WaveFilterControlWindow,hook(WavefromFilterHook)=SpHrView_WaveformFilterWindowHook
		SetWindow WaveFilterControlWindow note="PACKAGE:SpHrView"
	//endif
	DoWindow/W=WaveFilterControlWindow/T WaveFilterControlWindow, titleStr
	DoUpdate/W=WaveFilterControlWindow
	strswitch(waveformFilterTypeStr)
		case "Auditory filter":
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={70,7}, size={140,13}, fSize=10, title="Frequency (Hz)", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz1"
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, disable=1
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, disable=1
			break
		case "Band-pass - Low to High":
			NVAR waveformFilterHz1= $SpHrView_df_filter+"waveformFilterHz1"
			NVAR waveformFilterHz2= $SpHrView_df_filter+"waveformFilterHz2"
			if(waveformFilterHz1 > waveformFilterHz2) //reverse low and high?
				variable temp = waveformFilterHz1
				waveformFilterHz1 = waveformFilterHz2
				waveformFilterHz2 = temp
			endif
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={60,8}, size={85,13}, fSize=9, title="Low (Hz)", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz1"
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, pos={160,8}, size={90,13}, fSize=9, title="High (Hz)", disable=0, proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz2"
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, pos={260,8}, size={60,13}, disable=0, fSize=9, title="Order", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, limits={1,20,1}, value=$SpHrView_df_filter+"waveformFilterVal"
			break
		case "Band-pass - Bandwidth":
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={55,8}, size={100,13}, fSize=9, title="Center (Hz)", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz1"
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, pos={160,8}, size={95,13}, fSize=9, title="Width (Hz)", disable=0, proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, limits={20,20000,10}, value=$SpHrView_df_filter+"waveformFilterHz2"
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, pos={260,8}, size={60,13}, disable=0, fSize=9, title="Order", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, limits={1,20,1}, value=$SpHrView_df_filter+"waveformFilterVal"
			break
	
		case "Low-pass":
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={65,8}, size={115,13}, fSize=9, title="Frequency (Hz)", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz1"
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, disable=1
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, pos={185,8}, size={60,13}, disable=0, fSize=9, title="Order", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, limits={1,20,1}, value=$SpHrView_df_filter+"waveformFilterVal"
			break
		case "High-pass":
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={65,8}, size={115,13}, fSize=9, title="Frequency (Hz)", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz1"
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, disable=1
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, pos={185,8}, size={60,13}, disable=0, fSize=9, title="Order", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, limits={1,20,1}, value=$SpHrView_df_filter+"waveformFilterVal"
			break
		case "Notch":
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, pos={65,8}, size={117,13},  fSize=9, title="Frequency (Hz)", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterHz1 win=WaveFilterControlWindow, limits={20,20000,50}, value=$SpHrView_df_filter+"waveformFilterHz1"
			SetVariable sv_waveformFilterHz2 win=WaveFilterControlWindow, disable=1
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, pos={190,8}, size={45,13}, disable=0, fSize=9, title="Q", proc=SpHrView_FilterSetVarProc
			SetVariable sv_waveformFilterVal win=WaveFilterControlWindow, limits={1,20,1}, value=$SpHrView_df_filter+"waveformFilterVal"
			break
		case "No Filter":
			killwindow/Z WaveFilterControlWindow
			break
	endswitch
	NVAR waveformFilter = $SpHrView_df_filter+"waveformFilter"
	waveformFilter = 1
	
	// window hook
	DoUpdate/W=WaveFilterControlWindow // for Igor 9
	SetWindow WaveFilterControlWindow, hook(WaveformFilterHook)=SpHrView_WaveformFilterWindowHook
	
	DoWindow SpHrViewPeaksWindow 
	if(v_flag)
		DoWindow/F SpHrViewPeaksWindow 
	endif
	DoWindow FilterBank_Window
	if(v_flag)
		SpHrView_DisplayFilteredWaves(-1) // update
	endif
	DoWindow Waveforms_Window
	if(v_flag)
		SpHrView_DisplayCarrierWaves(-1, -1)
	endif
	DoWindow Spectra_Window
	if(v_flag)
		SpHrView_DisplaySpectra()
	endif
	DoWindow STFT_Window
	if(v_flag)
		SpHrView_DisplaySTFT()
	endif
	return 0
end

// =================================================================================================
//	SpHrView_WaveformFilterWindowHook(...), Hook function for  SpHrView_WaveformFilterWindow
// =================================================================================================
Function SpHrView_WaveformFilterWindowHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0	
	switch(s.eventCode)
		//////////////
		// deactivate
		case 1:
			strswitch(s.winName)
				case "FilterBankSettings":
					ControlInfo/W=FilterBankSettings ck_keepWindowOpen
					if(!V_Value)
						Execute/Z/Q/P "killwindow/Z FilterBankSettings"
						hookResult=1
					endif
					break
			endswitch
			break
			
		// kill event
		case 2:
			strswitch(s.winName)
				case "WaveFilterControlWindow":
					NVAR waveformFilter = $SpHrView_df_filter+"waveformFilter"
					waveformFilter = 0
					Execute/Z/Q/P "killwindow/Z WaveFilterControls"
					DoWindow SpHrViewPeaksWindow 
					if(v_flag)
						DoWindow/F SpHrViewPeaksWindow 
					endif
					hookResult = 1
					
					// ???
					NVAR filterbankUpdateFlag = $SpHrView_df_filter+"filterbankUpdateFlag"
					filterbankUpdateFlag = 1
					
					DoWindow FilterBank_Window
					if(v_flag)
						SpHrView_DisplayFilteredWaves(-1) // update
					endif
					DoWindow Waveforms_Window
					if(v_flag)
						SpHrView_DisplayCarrierWaves(-1, -1)
					endif
					DoWindow Spectra_Window
					if(v_flag)
						SpHrView_DisplaySpectra()
					endif
					DoWindow STFT_Window
					if(v_flag)
						SpHrView_DisplaySTFT()
					endif
					
					break
			endswitch
			break
					
	endswitch
End

// =================================================================================================
//	SpHrView_WaveformFilterMenuSelect(...), used by SpHrView_WaveformFilterWindow(...) and WaveformFilterApply(...)
// =================================================================================================
//PopupContextualMenu/ASYN=SpHrView_WaveformFilterMenuSelect PopUpListStr
function SpHrView_WaveformFilterMenuSelect(String popupStr, String selectedText, Variable menuItemNum)
	SVAR waveformFilterTypeStr= $SpHrView_df_filter+"waveformFilterTypeStr"
	if(strlen(selectedText) > 1)
		waveformFilterTypeStr = selectedText
	endif
	if(stringmatch(waveformFilterTypeStr, "No Filter"))
		waveformFilterTypeStr = "Band-pass - Bandwidth"
		killwindow/Z WaveFilterControlWindow
		return 0
	endif
	SpHrView_WaveformFilterWindow()
	DoWindow SpHrViewPeaksWindow 
	if(v_flag)
		DoWindow/F SpHrViewPeaksWindow 
	endif
	DoWindow FilterBank_Window
	if(v_flag)
		SpHrView_DisplayFilteredWaves(-1) // update
	endif
	DoWindow Waveforms_Window
	if(v_flag)
		SpHrView_DisplayCarrierWaves(-1, -1)
	endif
	DoWindow Spectra_Window
	if(v_flag)
		SpHrView_DisplaySpectra()
	endif
	DoWindow STFT_Window
	if(v_flag)
		SpHrView_DisplaySTFT()
	endif
	return 0
end

// =================================================================================================
//	SpHrView_WaveformFilterMenu(...), used by WaveformFilterMenuSelect(...), SpHrView_WaveformFilterWindow(...), WaveformFilterApply(...)
// =================================================================================================
function SpHrView_WaveformFilterMenu()
	String PopUpListStr = "Auditory filter;\M1-;Low-pass;High-Pass;\M1-;Band-pass - Low to High;Band-pass - Bandwidth;\M1-;Notch;\M1-;No Filter"  //Band-pass - Q;
	PopupContextualMenu/ASYN=SpHrView_WaveformFilterMenuSelect PopUpListStr
	return 0
end

// =================================================================================================
//	SpHrView_WaveformFilterApply(...), function called by getWavesFromFileOrWaveRef() to apply an optional
//	filter prior to analysis
// =================================================================================================
function SpHrView_WaveformFilterApply(loadedWaveStr)
	String loadedWaveStr
	Wave loadedWave = $loadedWaveStr
	SVAR waveformFilterTypeStr= $SpHrView_df_filter+"waveformFilterTypeStr"
	NVAR waveformFilterHz1 = $SpHrView_df_filter+"waveformFilterHz1"
	NVAR waveformFilterHz2 = $SpHrView_df_filter+"waveformFilterHz2"
	NVAR waveformFilterVal = $SpHrView_df_filter+"waveformFilterVal"
	variable cF, Low, High
	Variable dfs = dimdelta(loadedWave,0)
	strswitch(waveformFilterTypeStr)
		case "Auditory filter":
			NVAR waveformFilterHz1 = $SpHrView_df_filter+"waveformFilterHz1"
			SpHr_FilterBiquad/C/RMS /CF=(waveformFilterHz1) loadedWave // using default Q and minBW
			break
		case "Low-pass":
			FilterIIR/CASC/DIM=0/LO=(waveformFilterHz1*dfs)/ORD=(waveformFilterVal) loadedWave
			break
		case "High-pass":
			FilterIIR/CASC/DIM=0/HI=(waveformFilterHz1*dfs)/ORD=(waveformFilterVal) loadedWave
			break

		case "Band-pass - Bandwidth":
			low = waveformFilterHz1 - (waveformFilterHz1/ 2)
			high = waveformFilterHz1 + (waveformFilterHz2/ 2)
   		FilterIIR/CASC/DIM=0 /HI=(low*dfs) /LO=(high*dfs) /ORD=(waveformFilterVal) loadedWave
			break
		case "Band-pass - Low to High":
   		FilterIIR/CASC/DIM=0 /HI=(waveformFilterHz1*dfs) /LO=(waveformFilterHz2*dfs) /ORD=(waveformFilterVal) loadedWave
			break
		case "Notch":
			FilterIIR/CASC/DIM=0/N={waveformFilterHz1*dfs, waveformFilterVal} loadedWave
			break
		case "No Filter":
			killwindow/Z WaveFilterControlWindow
			break
	endswitch
end



// =================================================================================================
// SpHrView_DisplayFilteredWaves(variable getEnvelopes), Filter banks
// =================================================================================================
Function SpHrView_DisplayFilteredWaves(variable getEnvelopes)
	DFREF saveDFR = GetDataFolderDFR()
	// if getEnvelopes < 0 and FilterBank_Window exists, try to determine if displaying Waveforms or Envelopes
	if(getEnvelopes < 0)
		DoWindow FilterBank_Window
		if(v_flag)
			Controlinfo/W=$"FilterBank_Window"+"#PTop" b_DisplayWaveforms
			if(V_disable==0)
				getEnvelopes = 1
			else
				getEnvelopes = 0
			endif
		else
			getEnvelopes = 1
		endif
	endif
	SpHrView_FilterDefaults()
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, usingAxis
	[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
	wave waveforms=$SpHrView_df_files+"waveforms"
	if(!waveexists(waveforms))
		DoAlert 0, "SpHrView_DisplayFilteredWaves: Waveforms are missing"
		return 0
	endif
	variable numCols = dimsize(waveforms,1)
	if(numCols>2)
		DoAlert 0, "SpHrView_DisplayFilteredWaves: waveform has > 2 channels/columns"
		return 0
	endif
	
	NVAR lowFreq = $SpHrView_df_filter+"lowFreq"
	NVAR highFreq = $SpHrView_df_filter+"highFreq"
	NVAR numCFs = $SpHrView_df_filter+"numCFs"
	NVAR RMSscale = $SpHrView_df_filter+"RMSscale"
	NVAR clearDelays = $SpHrView_df_filter+"clearDelays"
	variable fs = round(1/dimdelta(waveforms,0))
	
	//SpHr_FilterCenterFreqs/R lowFreq, highFreq, numCFs, CFs // 32 bands, 20-10,000 Hz
	SpHrView_CenterFrequencies()
	wave CFs = $SpHrView_df_filter+"CFs"
	SpHr_FilterBankBiquadCoefs fs, CFs, coefBank
	//SpHr_FilterBankBiquadCoefs/W=$SpHrView_df_filter+"FilterWidths" fs, CFs, coefBank
	//wave FilterWidths = $SpHrView_df_filter+"FilterWidths"
	SpHr_FilterBankBiquad /RMS=(RMSscale)/C=(clearDelays)/D=$SpHrView_df_filter+"FilterBankDelays"/Dest=$SpHrView_df_filter+"FilterBanks" CFs, coefBank, waveforms
	wave FilterBanks = $SpHrView_df_filter+"FilterBanks"
	
	if(getEnvelopes)
		NVAR FilterBank_DisplaydB = $SpHrView_df_filter+"FilterBank_DisplaydB"
		if(FilterBank_DisplaydB)
			SpHr_HilbertEnvelope/B/O/dB FilterBanks
		else
			SpHr_HilbertEnvelope/B/O/Pa FilterBanks
		endif
	endif
	
	killwaves/Z waveforms, coefBank
	
	// Display FilterBanks
	NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
	NVAR screenVert= $SpHrView_df_globals+"screenVert"
	NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
	NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
	variable wdth=screenHoriz*ViewWidthProportion
	variable hgt=screenVert*ViewHeightProportion*1.75
	Variable  winLeft=(screenHoriz/6), winRight, winTop, winBottom=270
	[winLeft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
	Variable marginLeft, marginRight, marginTop, marginBottom
	[marginLeft, marginRight, marginTop, marginBottom] = getWindowMargins("SpHrViewPeaksWindow")	
	String TitleStr
		if(getEnvelopes)
			TitleStr = "Filter Bank Carrier Envelopes"
		else
			TitleStr = "Filter Bank Carrier Waveforms"
		endif
		
	DoWindow FilterBank_Window
	if(v_flag)
		// Update window
		RemoveTraces("FilterBank_Window")
		RemoveImages("FilterBank_Window")
		DoWindow/T FilterBank_Window, TitleStr
		
	else
		// New window
		Display /K=1/W=(winLeft, winBottom+25, winLeft+wdth, winBottom+25+hgt) as TitleStr
		DoWindow/C FilterBank_Window
		ModifyGraph/W=FilterBank_Window margin(left)=marginLeft,margin(bottom)=30,margin(top)=6,margin(right)=marginRight
		ControlBar/W=FilterBank_Window 35
		NewPanel/HOST=FilterBank_Window/FG=(FL,FT,GR,GT)/HOST=# 
		RenameWindow #,PTop
		
		variable top = 9
		variable left = 40
		
			// Envelopes controls
			SetVariable sv_EnvelopeColorCode, win=$"FilterBank_Window"+"#PTop", pos={left,top+2},size={125.00,14.00},bodyWidth=110,title="Colors"
			SetVariable sv_EnvelopeColorCode, win=$"FilterBank_Window"+"#PTop", fSize=9, proc=SpHrView_FilterSetVarProc
			SetVariable sv_EnvelopeColorCode, win=$"FilterBank_Window"+"#PTop", value= $SpHrView_df_filter+"FilterBankEnvelopeColorCode"
			left += 127
			Button b_FilterBankEnvelopeColorCode, win=$"FilterBank_Window"+"#PTop", pos={left,top+1},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
			Button b_FilterBankEnvelopeColorCode, win=$"FilterBank_Window"+"#PTop", proc=SpHrView_FilterButtonProc
			left += 25
			CheckBox b_FilterBankEnvelopeColorReverse win=$"FilterBank_Window"+"#PTop", title="reverse colors",pos={left,top},side=0,proc=SpHrView_FilterCheckProc
			CheckBox b_FilterBankEnvelopeColorReverse win=$"FilterBank_Window"+"#PTop", variable= $SpHrView_df_filter+"FilterBankEnvelopeColorReverse"
			left += 150
			CheckBox b_FilterBankEnvelopedB win=$"FilterBank_Window"+"#PTop", title="dB scale ",pos={left,top},side=1,proc=SpHrView_FilterCheckProc
			CheckBox b_FilterBankEnvelopedB win=$"FilterBank_Window"+"#PTop", variable= $SpHrView_df_filter+"FilterBank_DisplaydB"
			left += 70
			Button b_DisplayWaveforms, win=$"FilterBank_Window"+"#PTop", pos={left,top-2},title=Char_swapArrow10_9_Str+"View Waveforms",fSize=10,size={120.00,20.00}
			Button b_DisplayWaveforms, win=$"FilterBank_Window"+"#PTop", proc=SpHrView_FilterButtonProc

			// Waveform controls			
			left = 40
			SetVariable sv_WaveformColorCode, win=$"FilterBank_Window"+"#PTop", pos={left,top+2},size={125.00,14.00},bodyWidth=110,title="Colors"
			SetVariable sv_WaveformColorCode, win=$"FilterBank_Window"+"#PTop", fSize=9, proc=SpHrView_FilterSetVarProc
			SetVariable sv_WaveformColorCode, win=$"FilterBank_Window"+"#PTop", value= $SpHrView_df_filter+"FilterBankWaveformColorCode"
			left += 127
			Button b_FilterBankWaveformColorCode, win=$"FilterBank_Window"+"#PTop", pos={left,top+1},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
			Button b_FilterBankWaveformColorCode, win=$"FilterBank_Window"+"#PTop", proc=SpHrView_FilterButtonProc
			left += 25
			CheckBox b_FilterBankWaveformColorReverse win=$"FilterBank_Window"+"#PTop", title="reverse colors",pos={left,top},side=0,proc=SpHrView_FilterCheckProc
			CheckBox b_FilterBankWaveformColorReverse win=$"FilterBank_Window"+"#PTop", variable= $SpHrView_df_filter+"FilterBankWaveformColorReverse"
			left += 150
			left += 70
			Button b_DisplayEnvelopes, win=$"FilterBank_Window"+"#PTop", pos={left,top-2},title=Char_swapArrow10_9_Str+"View Envelopes ",fSize=10,size={120.00,20.00}
			Button b_DisplayEnvelopes, win=$"FilterBank_Window"+"#PTop", proc=SpHrView_FilterButtonProc

			// Common Controls
			left += 200
			Button b_FilterBankOptions, win=$"FilterBank_Window"+"#PTop", pos={left,top-2},title="Filter Bank Settings",fSize=10,size={120.00,20.00}
			Button b_FilterBankOptions, win=$"FilterBank_Window"+"#PTop", proc=SpHrView_FilterButtonProc
		
		
		SetActiveSubwindow FilterBank_Window //##
		ModifyGraph/W=FilterBank_Window cbRGB=(65534,65534,65534)
	endif
	
	if(getEnvelopes)
			SetVariable sv_EnvelopeColorCode, win=$"FilterBank_Window"+"#PTop",disable=0
			Button b_FilterBankEnvelopeColorCode, win=$"FilterBank_Window"+"#PTop",disable=0
			CheckBox b_FilterBankEnvelopeColorReverse win=$"FilterBank_Window"+"#PTop",disable=0
			CheckBox b_FilterBankEnvelopedB win=$"FilterBank_Window"+"#PTop", disable=0
			Button b_DisplayWaveforms, win=$"FilterBank_Window"+"#PTop",disable=0
			
			SetVariable sv_WaveformColorCode, win=$"FilterBank_Window"+"#PTop",disable=1
			Button b_FilterBankWaveformColorCode, win=$"FilterBank_Window"+"#PTop",disable=1
			CheckBox b_FilterBankWaveformColorReverse win=$"FilterBank_Window"+"#PTop",disable=1
			Button b_DisplayEnvelopes, win=$"FilterBank_Window"+"#PTop",disable=1
			
		else

			SetVariable sv_WaveformColorCode, win=$"FilterBank_Window"+"#PTop",disable=0
			Button b_FilterBankWaveformColorCode, win=$"FilterBank_Window"+"#PTop",disable=0
			CheckBox b_FilterBankWaveformColorReverse win=$"FilterBank_Window"+"#PTop",disable=0
			CheckBox b_FilterBankEnvelopedB win=$"FilterBank_Window"+"#PTop", disable=2
			Button b_DisplayEnvelopes, win=$"FilterBank_Window"+"#PTop",disable=0
			
			SetVariable sv_EnvelopeColorCode, win=$"FilterBank_Window"+"#PTop",disable=1
			Button b_FilterBankEnvelopeColorCode, win=$"FilterBank_Window"+"#PTop",disable=1
			CheckBox b_FilterBankEnvelopeColorReverse win=$"FilterBank_Window"+"#PTop",disable=1
			Button b_DisplayWaveforms, win=$"FilterBank_Window"+"#PTop",disable=1
			
		endif
	
	wavestats/Q/M=1 FilterBanks
//	variable minVal = v_min
//	variable maxVal = v_max
	variable ValRange = max(v_max, -v_min)

	// Append images
	String wStr = "FilterBanks"
	AppendImage/W=FilterBank_Window/L=left_L/B=bottom_L FilterBanks
	ModifyGraph/W=FilterBank_Window freePos(left_L)=0
	ModifyGraph/W=FilterBank_Window freePos(bottom_L)={0.53,kwFraction}
	
	
	if(getEnvelopes)
		NVAR ColorCodeReverse = $SpHrView_df_filter+"FilterBankEnvelopeColorReverse"
		SVAR ColorCodeStr = $SpHrView_df_filter+"FilterBankEnvelopeColorCode"
		SpHr_ColorTable/R=(ColorCodeReverse)/Dest=ColorTable ColorCodeStr
		ModifyImage/W=FilterBank_Window $wStr ctab= {0,*,ColorTable,0},ctabAutoscale=1,lookup= $""
	else
		NVAR ColorCodeReverse = $SpHrView_df_filter+"FilterBankWaveformColorReverse"
		SVAR ColorCodeStr = $SpHrView_df_filter+"FilterBankWaveformColorCode"
		SpHr_ColorTable/R=(ColorCodeReverse)/Dest=ColorTable ColorCodeStr
		ModifyImage/W=FilterBank_Window $wStr ctab= {-ValRange,ValRange,ColorTable,0},ctabAutoscale=1,lookup= $""
	endif
	
	// ticks and axes
	Label/W=FilterBank_Window left_L "frequency (\\U), left"
	ModifyGraph/W=FilterBank_Window userticks(left_L)={$SpHrView_df_filter+"CFs_TickVals", $SpHrView_df_filter+"CFs_TickTxt"}
	ModifyGraph/W=FilterBank_Window highTrip(left_L)=10
	SetAxis/W=FilterBank_Window left_L 0,*
	SetAxis/W=FilterBank_Window bottom_L *,*
	if(numCols>1)
		AppendImage/W=FilterBank_Window/L=left_R/B=bottom_R FilterBanks
		ModifyGraph/W=FilterBank_Window axisEnab(left_L)={0.53,1},axisEnab(left_R)={0,0.47}
		ModifyGraph/W=FilterBank_Window freePos(left_R)=0, freePos(bottom_R)=0
		ModifyImage/W=FilterBank_Window $wStr+"#1" plane=1,ctab= {0,*,ColorTable,0},ctabAutoscale=1,lookup= $""
		if(getEnvelopes)
	
		else
			ModifyImage/W=FilterBank_Window $wStr+"#1" ctab= {-ValRange,ValRange,ColorTable,0},ctabAutoscale=1,lookup= $""
		endif
		// ticks and axes
		Label/W=FilterBank_Window left_R "frequency (\\U), right"
		ModifyGraph/W=FilterBank_Window userticks(left_R)={$SpHrView_df_filter+"CFs_TickVals", $SpHrView_df_filter+"CFs_TickTxt"}
		ModifyGraph/W=FilterBank_Window highTrip(left_R)=10
		SetAxis/W=FilterBank_Window left_R 0,*
		SetAxis/W=FilterBank_Window bottom_R *,*
	endif
	
	ModifyGraph/W=FilterBank_Window lblPos(left_L)=30
	ModifyGraph/W=FilterBank_Window lblPos(bottom_L)=20
	ModifyGraph/W=FilterBank_Window nticks(bottom_L)=10
	if(numCols>1)
		Label/W=FilterBank_Window bottom_L "\\u#2"
		Label/W=FilterBank_Window bottom_R "time (\\U)"
		ModifyGraph/W=FilterBank_Window lblPos(bottom_R)=25
		ModifyGraph/W=FilterBank_Window lblPos(left_R)=30
		ModifyGraph/W=FilterBank_Window nticks(bottom_R)=10
	else
		Label/W=FilterBank_Window bottom_L "time (\\U)"
	endif
	ModifyGraph/W=FilterBank_Window tick=2
	ModifyGraph/W=FilterBank_Window font="Arial"
	ModifyGraph/W=FilterBank_Window fSize=10
	ModifyGraph/W=FilterBank_Window standoff=0
	ModifyGraph/W=FilterBank_Window axThick=0.5
	ModifyGraph/W=FilterBank_Window ZisZ=1, zapTZ=1, zapLZ=1
	ModifyGraph/W=FilterBank_Window btLen=4
	ModifyGraph/W=FilterBank_Window btThick=0.5
	ModifyGraph/W=FilterBank_Window stLen=2
	ModifyGraph/W=FilterBank_Window stThick=0.5
	// ColorScale
	ColorScale/W=FilterBank_Window/C/N=WaveformColorScale/F=0/B=1/A=RC/X=0.00/Y=6.50/E=2 vert=1,width=5,height=200,frame=0.00,image=$wStr
	ColorScale/W=FilterBank_Window/C/N=WaveformColorScale font="Arial",fsize=9,tickLen=1.50,tickThick=0.50,ZisZ=1
	ColorScale/W=FilterBank_Window/C/N=WaveformColorScale lblLatPos=-120,lblMargin=20 // lblLatPos=42,lblMargin=11
	ColorScale/W=FilterBank_Window/C/N=WaveformColorScale "\\U"
	
	ControlBar/L 20
	ModifyGraph/W=FilterBank_Window cbRGB=(65534,65534,65534)
	//wave CFs= $ksBV_Filters_dfStr+"CFs"
	variable CF_min, CF_max
	CF_min=0
	CF_max=dimdelta(FilterBanks,1)*dimsize(FilterBanks,1)
	Slider  sl_leftAxisMax win=FilterBank_Window,pos={0,50},size={5,hgt/3.5},side=0,proc=AxisSliderProc	
	Slider  sl_leftAxisMax win=FilterBank_Window,limits={CF_min,CF_max,1}, value=CF_max
	Slider  sl_leftAxisMin win=FilterBank_Window,pos={0,hgt/1.65},size={5,hgt/3.5},side=0,proc=AxisSliderProc	
	Slider  sl_leftAxisMin win=FilterBank_Window,limits={CF_min,CF_max,1}, value=CF_min
	Button  b_leftAxisAuto win=FilterBank_Window,pos={5, hgt-20}, title="",size={18,18},fSize=9,proc=AxisButtonProc,help={"auto scale"} // " 🔍"
	Button  b_leftAxisAuto win=FilterBank_Window,picture=ProcGlobal#SpHrView_pict_GlassZoom

	// Window Hook
	DoUpdate/W=FilterBank_Window // for Igor 9
	SetWindow FilterBank_Window,hook(FilterBankHook)=SpHrView_Spectra_WindowHook
	SetWindow FilterBank_Window note="PACKAGE:SpHrView"

	SetDataFolder saveDFR
End



Function SpHrView_DisplayFilterBankFilters(variable winLeft, variable winTop, variable OutputMode)
	if(winLeft == 0 || winTop == 0)
		GetMouse
		winLeft=V_left-250; winTop=V_top
	endif
	DFREF saveDFR = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_filter
	wave CFs = $SpHrView_df_filter+"CFs"
	string titleStr, nameStr
	variable useRMS = 0, fs = 48000
	NVAR RMSscale = $SpHrView_df_filter+"RMSscale"
	if(RMSscale)
		useRMS = 1000 // reference frequency
	endif
	switch(OutputMode)
		case 0:
			// use /N flag to normalize to max of 1
			SpHr_FilterBankBiquadCoefs/I={Impulses,5}/RMS={tRMSscales, useRMS} fs, CFs //, coefBank (optional)
			wave w = $SpHrView_df_filter+"Impulses"
			killwaves/Z tRMSscales
			titleStr = "Filter bank impulses"
			nameStr = CleanupName(titleStr,0)
			break
		case 1:
			// use /N flag to normalize to max of 1
			SpHr_FilterBankBiquadCoefs/IE={ImpulseEnvelopes,5}/RMS={tRMSscales, useRMS} fs, CFs //, coefBank (optional)
			wave w = $SpHrView_df_filter+"ImpulseEnvelopes"
			killwaves/Z tRMSscales
			titleStr = "Filter bank Impulse Envelopes"
			nameStr = CleanupName(titleStr,0)
			break
		case 2:
			SpHr_FilterBankBiquadCoefs/S={ImpulseSpectra,8192}/RMS={tRMSscales, useRMS} fs, CFs //, coefBank (optional)
			wave w = $SpHrView_df_filter+"ImpulseSpectra"
			killwaves/Z tRMSscales
			titleStr = "Filter bank impulse Spectra"
			nameStr = CleanupName(titleStr,0)
			break
		case 3:
			SpHr_FilterBankBiquadCoefs/W={ImpulseEnvelopeWidths,0.5}/RMS={tRMSscales, useRMS} fs, CFs //, coefBank (optional)
			wave w = $SpHrView_df_filter+"ImpulseEnvelopeWidths"
			killwaves/Z tRMSscales
			// plotting here and then returning...
			titleStr = "Filter bank Impulse Envelope Widths"
			nameStr = CleanupName(titleStr,0)
			DoWindow $nameStr
			if(!v_flag)
				Killwindow/Z $nameStr
				Display/K=1/W=(winLeft,winTop,winLeft+500,winTop+300) as titleStr
				Dowindow/C $nameStr
			else
				Dowindow/F $nameStr
				RemoveTraces(nameStr)
			endif
			AppendToGraph/W=$nameStr w vs CFs
			ModifyGraph/W=$nameStr margin(left)=36,margin(bottom)=34,margin(right)=14,margin(top)=14
			ModifyGraph/W=$nameStr tick=2,fSize=10,axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,standoff=0,btLen=4,btThick=0.5,stLen=2,stThick=0.5,font="Arial"
			ModifyGraph/W=$nameStr axisOnTop(bottom)=1, lblMargin=3
			ModifyGraph/W=$nameStr mode=4,marker=19,msize=2,rgb=(0,0,0),lsize=0.5
			Label/W=$nameStr left "Impulse Envelope Widths (\\U)"
			Label/W=$nameStr bottom "frequency (\\U)"
			ModifyGraph/W=$nameStr log(bottom)=1
			return 0 // return 
			break
		
		case 4:
			// 1000 Hz reference frequency
			SpHr_FilterBankBiquadCoefs/RMS={ImpuluseRMSscales, 1000} 48000, CFs//, coefBank (optional)
			wave w = $SpHrView_df_filter+"ImpuluseRMSscales"
			// plotting here and then returning...
			titleStr = "Filter bank equal RMS Scales"
			nameStr = CleanupName(titleStr,0)
			DoWindow $nameStr
			if(!v_flag)
				Killwindow/Z $nameStr
				Display/K=1/W=(winLeft,winTop,winLeft+500,winTop+300) as titleStr
				Dowindow/C $nameStr
			else
				Dowindow/F $nameStr
				RemoveTraces(nameStr)
			endif
			AppendToGraph/W=$nameStr w vs CFs
			ModifyGraph/W=$nameStr margin(left)=36,margin(bottom)=34,margin(right)=14,margin(top)=14
			ModifyGraph/W=$nameStr tick=2,fSize=10,axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,standoff=0,btLen=4,btThick=0.5,stLen=2,stThick=0.5,font="Arial"
			ModifyGraph/W=$nameStr axisOnTop(bottom)=1, lblMargin=3
			ModifyGraph/W=$nameStr mode=4,marker=19,msize=2,rgb=(0,0,0),lsize=0.5
			Label/W=$nameStr left "Impulse equal RMS Scales"
			Label/W=$nameStr bottom "frequency (\\U)"
			ModifyGraph/W=$nameStr log(bottom)=1
			return 0 // return 
			break
	
		default:
			SetDataFolder saveDFR
			return 0
	endswitch
	DoWindow $nameStr
	if(!v_flag)
		Killwindow/Z $nameStr
		Display/K=1/W=(winLeft,winTop,winLeft+500,winTop+300) as titleStr
		Dowindow/C $nameStr
	else
		Dowindow/F $nameStr
		RemoveTraces(nameStr)
	endif
	variable r,g,b,a,err
	int64 i, numCFs= dimsize(w,1)
	//for(i=0;i<numCFs;++i)
	for(i=numCFs-1;i>=0;--i) // reverse order
		SpHr_ColorTable/R=(0)/RGB={i, 0, numCFs} "Geo"
		AppendToGraph/W=$nameStr w[][i] /TN=$"Hz_"+num2istr(CFs[i])
		ModifyGraph/W=$nameStr rgb($"Hz_"+num2istr(CFs[i]))=(V_R,V_G,V_B,V_A)
	endfor
	ModifyGraph/W=$nameStr margin(left)=36,margin(bottom)=34,margin(right)=14,margin(top)=14
	ModifyGraph/W=$nameStr tick=2,fSize=10,axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,standoff=0,btLen=4,btThick=0.5,stLen=2,stThick=0.5,font="Arial"
	ModifyGraph/W=$nameStr axisOnTop(bottom)=1, lblMargin=3
	Label/W=$nameStr left "amplitude (\\U)"
	switch(OutputMode)
		case 0:
			Label/W=$nameStr bottom "time (\\U)"
			break
		case 1:
			Label/W=$nameStr bottom "time (\\U)"
			break
		case 2:
			Label/W=$nameStr bottom "frequency (\\U)"
			ModifyGraph/W=$nameStr log(bottom)=1
			SetAxis/W=$nameStr left -100,*
			ModifyGraph/W=$nameStr manTick(left)={0,12,0,0},manMinor(left)={1,50}
			break
	endswitch
	
	SetDataFolder saveDFR
End