﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3				// Use modern global access method and strict wave access
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later

strconstant SpHrView_df_ILDNeuron = "root:SpHrView:ILDNeuron:"

// =================================================================================================
// SpHrView_ILDNeuron_Defaults()
// =================================================================================================
Function SpHrView_ILDNeuron_Defaults()
	DFREF saveDFR = GetDataFolderDFR()
	String FolderStr
	// Make "SpHrView" folder, SpHrView_df
	if(!DataFolderExists(SpHrView_df))
		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:ILDNeuron" folder, SpHrView_df_cue
	if(!DataFolderExists(SpHrView_df_ILDNeuron))
		FolderStr= SpHrView_df_ILDNeuron[0,strlen(SpHrView_df_ILDNeuron)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_ILDNeuron
	String sval
	variable nval
	// ILD neuron defaults
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"NumILDneurons", 401)
	variable/G NumILDneurons = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"numILDbins", 401)
	variable/G numILDbins = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"minIpsiHalfmaxILDLabel", -24)
	variable/G minIpsiHalfmaxILDLabel = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"maxContraHalfmaxILDLabel", 24)
	variable/G maxContraHalfmaxILDLabel = nval
//	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"symmetricHalfmaxILDLabels", 0)
//	variable/G symmetricHalfmaxILDLabels = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"ILDrange", 50)
	variable/G ILDrange = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"ILDneuronWidth", 4)
	variable/G ILDneuronWidth = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"ILDSigmoidScaleLimit", 1) // scale activities so that (acts < minIpsiHalfmaxILDLabel) = 0 & (acts > maxContraHalfmaxILDLabel) = 1
	variable/G ILDSigmoidScaleLimit = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"scaleILDvsITD", 1) // Scale ILD so that ILD 'labels' scale with ITD 'labels' as opposed to ILD 'labels'
	variable/G scaleILDvsITD = nval
	nval= NumVarOrDefault(SpHrView_df_ILDNeuron+"ILDLabelShift", 0) // calculated shift in ILD labels (must equal IPDLabelShift)
	variable/G ILDLabelShift = nval
	
	SetDataFolder saveDFR
End

Function SpHrView_ILDNeurons()
	NVAR ILDrange = $SpHrView_df_ILDNeuron+"ILDrange"
	NVAR NumNeurons = $SpHrView_df_ILDNeuron+"NumILDneurons"
	NVAR numBins = $SpHrView_df_ILDNeuron+"numILDbins"
	NVAR ILDneuronWidth = $SpHrView_df_ILDNeuron+"ILDneuronWidth"
	NVAR minIpsi = $SpHrView_df_ILDNeuron+"minIpsiHalfmaxILDLabel"
	NVAR maxContra = $SpHrView_df_ILDNeuron+"maxContraHalfmaxILDLabel"
	NVAR ILDLabelShift = $SpHrView_df_ILDNeuron+"ILDLabelShift"
	NVAR ILDSigmoidScaleLimit = $SpHrView_df_ILDNeuron+"ILDSigmoidScaleLimit"
	DFREF DFR= $SpHrView_df_ILDNeuron
	
	SpHr_NeuronsILD/DF=DFR/R=(ILDrange)/W=(ILDneuronWidth)/S=(ILDSigmoidScaleLimit)/L={minIpsi,maxContra}/N={NumNeurons,numBins}/SL=(ILDLabelShift) ILDLabels, ILDNeurons
	
	// /Max=(1)
End

// =================================================================================================
// SpHrView_ILDNeuron_OptionsWindow(), display options for Cues
// =================================================================================================
Function SpHrView_ILDNeuron_OptionsWindow(variable winLeft, variable winTop)
	if(winLeft == 0 || winTop == 0)
		GetMouse
		winLeft=V_left; winTop=V_top
	endif
	SpHrView_ILDNeuron_Defaults()
	killwindow/Z NeuronsILDWindow
	NewPanel /FLT=1/K=1/W=(winLeft, winTop, winLeft+345, winTop+320) as "ILD neurons"
	DoWindow/C NeuronsILDWindow
	//string globals_dfStr = SpHrView_AtTime_DF_Globals

	variable topPos = 15 // restart topPos
	GroupBox group_Num win=NeuronsILDWindow, pos={6,topPos-6},size={170,100}, fstyle=0,fSize=10, title="Resolution of 'lookup table'"
	GroupBox group_Options win=NeuronsILDWindow, pos={180,topPos-6},size={160,100}, fstyle=0,fSize=10, title="Shape"
	topPos += 15
	SetVariable sv_NumILDneurons win=NeuronsILDWindow, pos={15,topPos},size={145.00,14.00},bodyWidth=40,fSize=10,title="number of neurons"
	SetVariable sv_NumILDneurons win=NeuronsILDWindow, limits={3,10001,0},value= $SpHrView_df_ILDNeuron+"NumILDneurons",focusRing=0
	SetVariable sv_NumILDneurons win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
	
	CheckBox ck_ILDSigmoidScaleLimit win=NeuronsILDWindow, title="'Normalize' at 0 dB", pos={200,topPos},proc=SpHrView_ILDNeuron_CheckProc
	CheckBox ck_ILDSigmoidScaleLimit win=NeuronsILDWindow, variable=$SpHrView_df_ILDNeuron+"ILDSigmoidScaleLimit"
	CheckBox ck_ILDSigmoidScaleLimit win=NeuronsILDWindow, help={"Scale activities so that those at ipsilateral label = 0.0 and those at contralateral label = 1.0"}
	
	topPos += 25
	SetVariable sv_numILDbins win=NeuronsILDWindow, pos={15,topPos},size={145.00,14.00},bodyWidth=40,fSize=10,title="samples per neuron"
	SetVariable sv_numILDbins win=NeuronsILDWindow, limits={3,10001,0},value= $SpHrView_df_ILDNeuron+"numILDbins",focusRing=0
	SetVariable sv_numILDbins win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
	
	SetVariable sv_ILDneuronWidth win=NeuronsILDWindow, pos={167,topPos},size={145.00,14.00},bodyWidth=40,fSize=10,title="Sigmoid width"
	SetVariable sv_ILDneuronWidth win=NeuronsILDWindow, limits={2,10,0},value= $SpHrView_df_ILDNeuron+"ILDneuronWidth",focusRing=0
	SetVariable sv_ILDneuronWidth win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}

	topPos += 25
	SetVariable sv_ILDrange win=NeuronsILDWindow, pos={15,topPos},size={145.00,14.00},bodyWidth=40,fSize=10,title="ILD range (cycles)"
	SetVariable sv_ILDrange win=NeuronsILDWindow, limits={24,96,0},value= $SpHrView_df_ILDNeuron+"ILDrange",focusRing=0
	SetVariable sv_ILDrange win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
	
	
	topPos += 45
	GroupBox group_ILDLabels win=NeuronsILDWindow, pos={6,topPos-6},size={335,145}, fstyle=0,fSize=10, title="ILD 'label' limits (cycles) for left hemisphere (right is mirrored)"

	topPos += 20
	SetVariable sv_minIpsiHalfmaxILDLabel win=NeuronsILDWindow, pos={15,topPos},size={178.00,14.00},bodyWidth=55,fSize=10,title="Half-maximal:  ipsilateral"
	SetVariable sv_minIpsiHalfmaxILDLabel win=NeuronsILDWindow, limits={-100,100,3},value= $SpHrView_df_ILDNeuron+"minIpsiHalfmaxILDLabel",focusRing=0, format="%.3g"
	SetVariable sv_minIpsiHalfmaxILDLabel win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
	
	SetVariable sv_maxContraHalfmaxILDLabel win=NeuronsILDWindow, pos={205,topPos},size={120.00,14.00},bodyWidth=55,fSize=10,title="contralateral"
	SetVariable sv_maxContraHalfmaxILDLabel win=NeuronsILDWindow, limits={-100,100,3},value= $SpHrView_df_ILDNeuron+"maxContraHalfmaxILDLabel",focusRing=0, format="%.3g"
	SetVariable sv_maxContraHalfmaxILDLabel win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
	
	topPos += 27
	SetVariable sv_ILDLabelShift win=NeuronsILDWindow, pos={60,topPos},size={130.00,14.00},bodyWidth=50,title="ipsi/contra 'label' shift",disable=2,fSize=10
	SetVariable sv_ILDLabelShift win=NeuronsILDWindow, limits={-inf,inf,0},noedit=1,value= $SpHrView_df_ILDNeuron+"ILDLabelShift",focusRing=0, format="%.4g"
	SetVariable sv_ILDLabelShift win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={"Label shift (offset). Must be the same as IPD label shift."}
	
	topPos += 25
	NVAR IPDLabelShift = $SpHrView_df_IPDNeuron+"IPDLabelShift"
	//NVAR ILDLabelShift = $SpHrView_df_ILDNeuron+"ILDLabelShift"
	//if(IPDLabelShift != 0)
		NVAR minIpsi = $SpHrView_df_ILDNeuron+"minIpsiHalfmaxILDLabel"
		Variable actualMinIpsi = minIpsi - (minIpsi*IPDLabelShift)
		NVAR maxContra = $SpHrView_df_ILDNeuron+"maxContraHalfmaxILDLabel"
		Variable actualMaxContra = maxContra - (maxContra*IPDLabelShift)

		SetVariable sv_ActualMinIpsiLabel win=NeuronsILDWindow, pos={15,topPos},size={178.00,14.00},bodyWidth=55,disable=2,fSize=10,title="Shifted:  ipsilateral"
		SetVariable sv_ActualMinIpsiLabel win=NeuronsILDWindow, limits={-100,100,3},noedit=1,value= _NUM:actualMinIpsi ,focusRing=0, format="%.3g"
		SetVariable sv_ActualMinIpsiLabel win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
		
		SetVariable sv_ActualMaxContraLabel win=NeuronsILDWindow, pos={205,topPos},size={120.00,14.00},bodyWidth=55,disable=2,fSize=10,title="contralateral"
		SetVariable sv_ActualMaxContraLabel win=NeuronsILDWindow, limits={-100,100,3},noedit=1,value= _NUM:actualMaxContra,focusRing=0, format="%.3g"
		SetVariable sv_ActualMaxContraLabel win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_SetVarProc, help={""}
	//endif
	
	topPos += 35
	CheckBox ck_scaleILDvsITD win=NeuronsILDWindow, title="Scale ILD 'labels' with ITD 'labels' (as opposed to IPD)", pos={30,topPos}
	CheckBox ck_scaleILDvsITD win=NeuronsILDWindow, variable=$SpHrView_df_ILDNeuron+"scaleILDvsITD", proc=SpHrView_ILDNeuron_CheckProc
	CheckBox ck_scaleILDvsITD win=NeuronsILDWindow, help={"Scale modeled ILD activities with ITD, as opposed to IPD"}
	
	// make half-max label limits red if not equal
	NVAR HM_minIpsi = $SpHrView_df_ILDNeuron+"minIpsiHalfmaxILDLabel"
	NVAR HM_maxContra = $SpHrView_df_ILDNeuron+"maxContraHalfmaxILDLabel"
	if(0)
		if(HM_maxContra != -HM_minIpsi)
			SetVariable sv_minIpsiHalfmaxILDLabel win=NeuronsILDWindow, valueColor=(52428,1,1)
			SetVariable sv_maxContraHalfmaxILDLabel win=NeuronsILDWindow, valueColor=(52428,1,1)
		else
			SetVariable sv_minIpsiHalfmaxILDLabel win=NeuronsILDWindow, valueColor=(0,0,0)
			SetVariable sv_maxContraHalfmaxILDLabel win=NeuronsILDWindow, valueColor=(0,0,0)
		endif
	endif
	
	topPos += 50
	TitleBox t_DisplayILDNeurons  win=NeuronsILDWindow, title="Display:",pos={65,topPos+4},frame=0,fSize=10,fstyle=0
	Button b_DisplayILDNeurons win=NeuronsILDWindow, pos={110,topPos}, size={130.00,20.00}, fSize=10,title="ILD neurons"
	Button b_DisplayILDNeurons win=NeuronsILDWindow, proc=SpHrView_ILDNeuron_ButtonProc,focusRing=0
	
	doupdate/W=NeuronsILDWindow
	
	CheckBox ck_keepWindowOpen win=NeuronsILDWindow, title="📌 ",fSize=10,pos={15,topPos},side=0
	CheckBox ck_keepWindowOpen win=NeuronsILDWindow, help={"Keep window open"}
	
	DoUpdate/W=NeuronsILDWindow // for Igor 9
	SetWindow NeuronsILDWindow hook(NeuronsILDWindowHook)=SpHrView_AtTime_ActivityWindowHook  // Why setting this hook and not another?
	//SetWindow NeuronsILDWindow note="PACKAGE:SpHrView;GROUP:ILD"
End


Function SpHrView_ILDNeuron_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			Variable checked = cba.checked
			strswitch(cba.ctrlName)		
				case "ck_ILDSigmoidScaleLimit":
					SpHrView_ILDNeurons()
					break				
				case "ck_scaleILDvsITD":
					if(!checked)
						DoAlert/T="Warning" 2, "ILD activities should scale with ITD. Scale with IPD anyway?"
						if(V_flag == 1)
							
						else
							NVAR scaleILDvsITD = $SpHrView_df_ILDNeuron+"scaleILDvsITD"
							scaleILDvsITD = 1
						endif
					endif
					break	
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

Function SpHrView_ILDNeuron_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			// click code here
			Variable leftCoord, topCoord
			string winStr
			strswitch(ba.ctrlName)
					
				case "b_ILDNeuronParams":
					winStr = ba.win
					if(strsearch(winStr, "#PTop", 0))
						winStr = RemoveEnding(winStr, "#PTop")
					endif
					GetWindow $winStr, wsize// gsize
					leftCoord = V_left + ba.mouseLoc.h
					topCoord = V_Top + ba.mouseLoc.v
					SpHrView_ILDNeuron_OptionsWindow(leftCoord, topCoord)
					break
								
				case "b_DisplayILDNeurons":
					SpHrView_ILDNeuron_Window() //  ILD neurons
					break

						
			endswitch
			break
		case -1: // control being killed
			break
	endswitch
	return 0
End

Function SpHrView_ILDNeuron_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	int ValDepUpdate = 0
	switch( sva.eventCode )
		case 6: // Value changed by dependency update
			ValDepUpdate = 1
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			Variable dval = sva.dval
			String sval = sva.sval
			String winStr
			strswitch(sva.ctrlName)

				case "sv_NumILDneurons":
				case "sv_numILDbins":
				case "sv_ILDrange":
				case "sv_ILDneuronWidth":
					if(!ValDepUpdate)
						SpHrView_ILDNeurons()
						dowindow ILD_selectivity_curves
						if(v_flag)
							SpHrView_ILDNeuron_Window()
						endif
					endif
					break
					
				case "sv_minIpsiHalfmaxILDLabel":
				case "sv_maxContraHalfmaxILDLabel":
					if(!ValDepUpdate)
						SpHrView_ILDNeurons()

					endif
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch
	return 0
End



Function SpHrView_ILDNeuron_Window()
	SpHrView_ILDNeurons()
	wave ILDNeurons = $SpHrView_df_ILDNeuron+"ILDNeurons"
	if(!waveexists(ILDNeurons))
		DoAlert 0, "wave "+SpHrView_df_ILDNeuron+"ILDNeurons is missing"
		return 0
	endif
	wave ILDLabels = $SpHrView_df_ILDNeuron+"ILDLabels"
	if(!waveexists(ILDLabels))
		DoAlert 0, "wave "+SpHrView_df_ILDNeuron+"ILDLabels is missing"
		return 0
	endif
	string titleNameStr = "Carrier ILD selectivity curves"
	string windowNameStr = "ILD_selectivity_curves"
	string traceNameStr = "ILDNeurons"
	
	dowindow $windowNameStr
	if(v_flag)
		dowindow/F $windowNameStr
		DoWindow/T $windowNameStr, titleNameStr
		RemoveTraces(windowNameStr)		
	else
		killwindow/Z NeuronsILDWindow // Display messes up if this window isn't killed
		Display /K=1/W=(300,50,300+600,50+300) as titleNameStr
		DoWindow/C $windowNameStr
		ModifyGraph/W=$windowNameStr margin(left)=36,margin(bottom)=36,margin(right)=36,margin(top)=25
		ControlBar/W=$windowNameStr/T 35
		NewPanel/FG=(FL,FT,GR,GT)/HOST=# 
		RenameWindow #,PTop
		
		Button b_ILDNeuronParams win=$windowNameStr+"#PTop", pos={475,8},title=Char_swapArrow10_9_Str+" Parameters",size={100,18},proc=SpHrView_ILDNeuron_ButtonProc,fSize=10
		SetActiveSubwindow $windowNameStr //##
		ModifyGraph/W=$windowNameStr cbRGB=(65534,65534,65534)
	endif
	int64 traceCnt = 0, i
	int64 traceSkip = round(dimsize(ILDNeurons,1)/20)
	for(i=1;i<dimsize(ILDNeurons,1);i+=traceSkip)
		AppendToGraph/W=$windowNameStr ILDNeurons[*][i]
		traceCnt += 1
	endfor
	ModifyGraph/W=$windowNameStr lsize=0.5,rgb=(52428,52428,52428)
	ModifyGraph/W=$windowNameStr tick=2, nticks(bottom)=10
	ModifyGraph/W=$windowNameStr tick(bottom)=0
	ModifyGraph/W=$windowNameStr font="Arial", fSize=10
	ModifyGraph/W=$windowNameStr lblMargin=5
	ModifyGraph/W=$windowNameStr standoff=0, axThick=0.5
	ModifyGraph/W=$windowNameStr ZisZ=1, zapTZ=1, zapLZ=1
	ModifyGraph/W=$windowNameStr btLen=4, btThick=0.5
	ModifyGraph/W=$windowNameStr stLen=2, stThick=0.5, tlOffset=2
	ModifyGraph/W=$windowNameStr manTick(left)={0,0.5,0,1},manMinor(left)={1,50}

	ModifyGraph/W=$windowNameStr manTick(bottom)={0,12,0,0},manMinor(bottom)={1,50}
	
	Label/W=$windowNameStr left "response probability\\u#2" // override units
	SetAxis/W=$windowNameStr left 0,*
	Label/W=$windowNameStr bottom "ILD (\\U)"
	// append first neuron, last neuron, and neuron where half-maximum activities correspond to 0 cycles (ILD)
	variable lastNeuron = dimsize(ILDNeurons,1)-1
	FindLevel/P/Q ILDLabels, 0
	variable centerNeuron = round(V_LevelX) 
	AppendToGraph/W=$windowNameStr ILDNeurons[*][0], ILDNeurons[*][centerNeuron], ILDNeurons[*][lastNeuron]
	ModifyGraph/W=$windowNameStr mode($traceNameStr+"#"+num2str(traceCnt))=3, mode($traceNameStr+"#"+num2str(traceCnt+1))=3, mode($traceNameStr+"#"+num2str(traceCnt+2))=3
	ModifyGraph/W=$windowNameStr marker($traceNameStr+"#"+num2str(traceCnt))=19, marker($traceNameStr+"#"+num2str(traceCnt+1))=19, marker($traceNameStr+"#"+num2str(traceCnt+2))=19
	ModifyGraph/W=$windowNameStr msize($traceNameStr+"#"+num2str(traceCnt))=1, msize($traceNameStr+"#"+num2str(traceCnt+1))=1, msize($traceNameStr+"#"+num2str(traceCnt+2))=1
	ModifyGraph/W=$windowNameStr rgb($traceNameStr+"#"+num2str(traceCnt))=(0,0,0), rgb($traceNameStr+"#"+num2str(traceCnt+1))=(0,0,0), rgb($traceNameStr+"#"+num2str(traceCnt+2))=(0,0,0)
	NVAR ILDrange = $SpHrView_df_ILDNeuron+"ILDrange" 
	SetAxis/W=$windowNameStr bottom -ILDrange,ILDrange
	SetDrawLayer/W=$windowNameStr/K UserFront
	SetDrawEnv/W=$windowNameStr ycoord= left, linefgc= (65535,16385,16385),dash= 0,linethick= 0.75
	DrawLine/W=$windowNameStr 0,0.5,1,0.5
	SetDrawEnv/W=$windowNameStr xcoord= bottom,linefgc= (65535,16385,16385),dash= 0,linethick= 0.75
	DrawLine/W=$windowNameStr 0,0.0,0,1
	TextBox/W=$windowNameStr/C/N=ILDNeuronLegend/J/M/B=1/F=0/A=LT/E/X=10.00/Y=2.00 "\\JL\\s("+traceNameStr+") \\F'Arial'\\Z10left hemisphere neurons \\s("+traceNameStr+"#"+num2str(traceCnt)+")sample"


End
