﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// SpHrView_Spectra

// =================================================================================================
// SpHrView_Spectra_Defaults(), Spectum defaults
// =================================================================================================
Function SpHrView_Spectra_Defaults()
	DFREF DFRsaved= GetDataFolderDFR()
	String FolderStr
	// Make "SpHrView" folder, SpHrView_df
	if(!DataFolderExists(SpHrView_df))
		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Spectra" folder, SpHrView_df_spectra
	if(!DataFolderExists(SpHrView_df_spectra))
		FolderStr= SpHrView_df_spectra[0,strlen(SpHrView_df_spectra)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_spectra
	variable nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"SpectrumFramePoints", 8192)	// Spectrum DFT points
	variable/G SpectrumFramePoints = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"SpectrumFrameAverage", 1)		// use average of sliding, SpectrumFramePoints, frame
	variable/G SpectrumFrameAverage = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"FrameOverlap", 90)				// sliding frame overlap
	variable/G FrameOverlap = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"SpectrumLogFreqAxis", 1)		// use log10 scale for spectrum frequency axis
	variable/G SpectrumLogFreqAxis = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"SmoothSpectrum", 1)				// Spectrum octave averaging 
	variable/G SmoothSpectrum = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"OctaveDenominator", 48)			// Spectrum octave averaging 
	variable/G OctaveDenominator = nval
	
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_OverlapPercent", 75)		// overlap percent used for STFT_hopSize e.g., 1024*(1-0.75)=256
	variable/G STFT_OverlapPercent = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_hopSize", 256)				// points skipped across succesive FFTS
	variable/G STFT_hopSize = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_SegmentLength", 512)		// sound segment length
	variable/G STFT_SegmentLength = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_PadLength", 1024)			// padded sound segment and FFT length 
	variable/G STFT_PadLength = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_dB", 1)						// scale dB re 20 µPa
	variable/G STFT_dB = nval
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_logHz", 0)					// log frequency
	variable/G STFT_logHz = nval
	
	String sval
	sval= StrVarOrDefault(SpHrView_df_spectra+"STFT_WindowType", "Hanning")	// Window type
	string/G STFT_WindowType = sval
	sval= StrVarOrDefault(SpHrView_df_spectra+"STFT_ColorCode", "SeaLandAndFire")		// color scheme
	string/G STFT_ColorCode = sval
	nval= NumVarOrDefault(SpHrView_df_spectra+"STFT_ColorCodeReverse", 0)		// reverse color scheme
	variable/G STFT_ColorCodeReverse = nval
	
	SetDataFolder DFRsaved
End

// =================================================================================================
// SpHrView_Settings_Spectra(), settings for spectra
// =================================================================================================
//Function SpHrView_Settings_Spectra()
//	SpHrView_Spectra_Defaults()
//	killwindow/Z SpectrumSettings
//	Variable  winLeft=250, winRight=winLeft+170, winTop=50, winBottom=winTop+340
//	[winLeft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
//	NewPanel /K=1/W=(winRight-190, winTop+20, winRight-20,winTop+340) as "Spectra"
//	DoWindow/C SpectrumSettings
//	variable topPos = 10
//	GroupBox group0,pos={8.00,topPos-5},size={157.00, 140}
//	TitleBox title_spectrum title="Spectra", pos={65,topPos},frame=0,fSize=10,fstyle=1
//	SetVariable sv_DFTpoints,pos={44,topPos+20},size={95.00,14.00},bodyWidth=40,title="DFT frame points"
//	SetVariable sv_DFTpoints,fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_DFTpoints,limits={128,65536,0},value= $SpHrView_df_spectra+"SpectrumFramePoints"
//	Button b_DFTsize, pos={140.00,topPos+19},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
//	Button b_DFTsize, proc=SpHrView_Spectra_ButtonProc
//		
//	CheckBox ck_SpectrumFrameAverage title="Sliding frame average",pos={25,topPos+44},side=0, proc=SpHrView_Spectra_CheckProc
//	CheckBox ck_SpectrumFrameAverage variable= $SpHrView_df_spectra+"SpectrumFrameAverage"
//	CheckBox ck_SpectrumFrameAverage help={"Use sliding frame average when DFT-points longer than sound selection (vs. decimation of longer DFT)"}
//	SetVariable sv_SpectrumFrameOverlap,pos={24.00, topPos+62},size={126.00,14.00},bodyWidth=50,title="Frame overlap"
//	SetVariable sv_SpectrumFrameOverlap,fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_SpectrumFrameOverlap,limits={0.5,128,1},value= $SpHrView_df_spectra+"FrameOverlap"
//	
//	CheckBox ck_UseSpectrumOctaveDenominator pos={25,topPos+87},side=0, title="Octave", proc=SpHrView_Spectra_CheckProc
//	CheckBox ck_UseSpectrumOctaveDenominator variable= $SpHrView_df_spectra+"SmoothSpectrum"
//	SetVariable sv_SpectrumOctaveDenominator,pos={24.00, topPos+88},size={126.00,14.00},bodyWidth=50,title="1 / "
//	SetVariable sv_SpectrumOctaveDenominator,fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_SpectrumOctaveDenominator,limits={0.5,128,1},value= $SpHrView_df_spectra+"OctaveDenominator"
//	
//	CheckBox ck_logFreqAxis title="log frequency axis ",pos={25,topPos+110},side=0, proc=SpHrView_Spectra_CheckProc
//	CheckBox ck_logFreqAxis variable= $SpHrView_df_spectra+"SpectrumLogFreqAxis"
//	
//	
//	// Short-term spectra
//	topPos = 160
//	GroupBox group1,pos={8.00,topPos-8},size={157.00,165}
//	TitleBox title_STFT title="Spectra over time", pos={40,topPos},frame=0,fSize=10,fstyle=1
//	SetVariable sv_SegmentPoints,pos={49,topPos+20},size={95.00,14.00},bodyWidth=40,title="Segment points"
//	SetVariable sv_SegmentPoints,fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_SegmentPoints,limits={128,65536,0},value= $SpHrView_df_spectra+"STFT_SegmentLength"
//	Button b_SegmentPoints, pos={145.00,topPos+19.00},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
//	Button b_SegmentPoints, proc=SpHrView_Spectra_ButtonProc
//	SetVariable sv_PadPoints,pos={49,topPos+40.00},size={95.00,14.00},bodyWidth=40,title="FFT points"
//	SetVariable sv_PadPoints,fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_PadPoints,limits={128,65536,0},value= $SpHrView_df_spectra+"STFT_PadLength"
//	Button b_PadPoints, pos={145.00,topPos+39.00},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
//	Button b_PadPoints, proc=SpHrView_Spectra_ButtonProc
//	SetVariable sv_OverlapPercent,pos={68,topPos+60.00},size={85.00,14.00},bodyWidth=40,title="Overlap (%)"
//	SetVariable sv_OverlapPercent,fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_OverlapPercent,limits={0,99.99,5},value= $SpHrView_df_spectra+"STFT_OverlapPercent"
//	SetVariable sv_hopSize,pos={10,topPos+60.00},size={40.00,14.00},bodyWidth=35,title=" "
//	SetVariable sv_hopSize,fSize=9, noedit=1, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_hopSize,limits={0,1000000,0},value= $SpHrView_df_spectra+"STFT_hopSize"
//	SetVariable sv_hopSize valueBackColor=(56797,56797,56797),labelBack=(56797,56797,56797)
//	SetVariable sv_hopSize help={"Hop size in points"}
//	
//	SetVariable sv_STFTWindowType,pos={40,topPos+83},size={105.00,14.00},bodyWidth=90,title="Window"
//	SetVariable sv_STFTWindowType, fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_STFTWindowType, value= $SpHrView_df_spectra+"STFT_WindowType"
//	Button b_STFTWindowType, pos={145.00,topPos+82},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
//	Button b_STFTWindowType, proc=SpHrView_Spectra_ButtonProc
//	
//	SetVariable sv_STFTColorCode,pos={40,topPos+109},size={105.00,14.00},bodyWidth=90,title="Colors"
//	SetVariable sv_STFTColorCode, fSize=9, proc=SpHrView_Spectra_SetVarProc
//	SetVariable sv_STFTColorCode, value= $SpHrView_df_spectra+"STFT_ColorCode"
//	Button b_STFTColorCode, pos={145.00,topPos+108},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
//	Button b_STFTColorCode, proc=SpHrView_Spectra_ButtonProc
//	
//	CheckBox ck_STFTRevColorCode title="reverse colors",pos={60,topPos+127},side=1
//	CheckBox ck_STFTRevColorCode variable= $SpHrView_df_spectra+"STFT_ColorCodeReverse"
//
//End

// =================================================================================================
// SpHrView_Spectra_ButtonProc, Button procedure for Spectra
// =================================================================================================
Function SpHrView_Spectra_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	String PopUpListStr, NameStr="", dfStr
	variable minX, maxX, rangeX, keyPressed, i
	strswitch( ba.ctrlName )
//		case "b_Settings":
//			switch( ba.eventCode )
//				case 1: // mouse down
//					dowindow SpectrumSettings
//					if(v_flag)
//						killwindow/Z SpectrumSettings
//					else
//						SpHrView_Settings_Spectra()
//					endif
//			case -1: // control being killed
//					break
//			endswitch
//			break
			
		case "b_STFTColorCode":
			switch( ba.eventCode )
				case 1: // mouse down
					SVAR STFT_ColorCode= $SpHrView_df_spectra+"STFT_ColorCode"
					String tempStr = ColorTablePopup(STFT_ColorCode)
					if(strlen(tempStr)>1)
						STFT_ColorCode = tempStr
						DoWindow STFT_Window
						if(v_flag)
							SpHrView_DisplaySTFT()
						endif
					endif
//					SVAR STFT_ColorCode= $SpHrView_df_spectra+"STFT_ColorCode"
//					PopupContextualMenu/N "SpHrView_ColorTablesPopup" //"*COLORTABLEPOP*"
//					if(V_flag>0)// && V_kind==13)
//						STFT_ColorCode = S_selection
//						DoWindow STFT_Window
//						if(v_flag)
//							SpHrView_DisplaySTFT()
//						endif
//					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "b_STFTWindowType":
			switch( ba.eventCode )
				case 1: // mouse down
					SVAR STFT_WindowType= $SpHrView_df_spectra+"STFT_WindowType"
					PopupContextualMenu "Bartlett;Blackman367;Blackman361;Blackman492;Blackman474;Cos1;Cos2;Cos3;Cos4;Hamming;Hanning;KaiserBessel20;KaiserBessel25;KaiserBessel30;Parzen;Poisson2;Poisson3;Poisson4;Riemann"
					if(V_flag)
						STFT_WindowType = S_selection
						DoWindow STFT_Window
						if(v_flag)
							SpHrView_DisplaySTFT()
						endif
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
				
		case "b_DFTsize":
			switch( ba.eventCode )
				case 1: // mouse down
				//case 5: // mouse enter
					SpHr_DFTSizeList/S 2^7, 2^15
					PopupContextualMenu S_RateList
					if(V_flag)
						NVAR SpectrumFramePoints= $SpHrView_df_spectra+"SpectrumFramePoints"
						SpectrumFramePoints = str2num(S_selection)
					else
						return 0
					endif
					DoWindow Spectra_Window
					if(v_flag)
						SpHrView_DisplaySpectra()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "b_SegmentPoints":
		case "b_PadPoints":
			switch( ba.eventCode )
				case 1: // mouse down
				//case 5: // mouse enter
					SpHr_DFTSizeList /S 2^7, 2^15
					PopupContextualMenu S_RateList
					if(V_flag)
						strswitch( ba.ctrlName )
							case "b_SegmentPoints":
								NVAR STFT_SegmentLength=$SpHrView_df_spectra+"STFT_SegmentLength"
								NVAR STFT_PadLength=$SpHrView_df_spectra+"STFT_PadLength"
								STFT_SegmentLength = str2num(S_selection)
								STFT_SegmentLength += mod(STFT_SegmentLength,2) // even
								if(STFT_SegmentLength>STFT_PadLength)
									STFT_PadLength = STFT_SegmentLength
								endif
								NVAR STFT_OverlapPercent= $SpHrView_df_spectra+"STFT_OverlapPercent"
								NVAR STFT_hopSize= $SpHrView_df_spectra+"STFT_hopSize"
								STFT_hopSize = round(STFT_PadLength*(1-(STFT_OverlapPercent)/100))
								STFT_hopSize = max(1,STFT_hopSize)
								STFT_hopSize = min(STFT_PadLength,STFT_hopSize)
								break
							case "b_PadPoints":
								NVAR STFT_SegmentLength=$SpHrView_df_spectra+"STFT_SegmentLength"
								NVAR STFT_PadLength=$SpHrView_df_spectra+"STFT_PadLength"
								STFT_PadLength = str2num(S_selection)
								STFT_PadLength += mod(STFT_PadLength,2) // even
								if(STFT_SegmentLength>STFT_PadLength)
									STFT_PadLength = STFT_SegmentLength
								endif
								NVAR STFT_OverlapPercent= $SpHrView_df_spectra+"STFT_OverlapPercent"
								NVAR STFT_hopSize= $SpHrView_df_spectra+"STFT_hopSize"
								STFT_hopSize = round(STFT_PadLength*(1-(STFT_OverlapPercent)/100))
								STFT_hopSize = max(1,STFT_hopSize)
								STFT_hopSize = min(STFT_PadLength,STFT_hopSize)
								break
						endswitch
					else
						return 0
					endif
					DoWindow STFT_Window
					if(v_flag)
						SpHrView_DisplaySTFT()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
						
	endswitch
	return 0
End

// =================================================================================================
// SpHrView_Spectra_SetVarProc(sva), SetVariable procedure for Spectra
// =================================================================================================
Function SpHrView_Spectra_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	Variable dval = sva.dval
	//String sval = sva.sval
	String vName = sva.vName
	variable analysisIndex
	strswitch( sva.ctrlName )
			
		case "sv_DFTpoints":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
					NVAR SpectrumFramePoints= $SpHrView_df_spectra+"SpectrumFramePoints"
					SpectrumFramePoints = SpHr_DFTClosestSize(SpectrumFramePoints, 1)
					DoWindow Spectra_Window
					if(v_flag)
						SpHrView_DisplaySpectra()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
		case "sv_SpectrumOctaveDenominator":
			DoWindow Spectra_Window
			if(v_flag)
				SpHrView_DisplaySpectra()
			endif
			break
			
		case "sv_PadPoints":
		case "sv_SegmentPoints":
		case "sv_OverlapPercent":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
				//case 3: // Live update
					NVAR STFT_OverlapPercent= $SpHrView_df_spectra+"STFT_OverlapPercent"
					NVAR STFT_hopSize= $SpHrView_df_spectra+"STFT_hopSize"
					NVAR STFT_PadLength= $SpHrView_df_spectra+"STFT_PadLength"
					STFT_hopSize = round(STFT_PadLength*(1-(STFT_OverlapPercent)/100))
					STFT_hopSize = max(1,STFT_hopSize)
					STFT_hopSize = min(STFT_PadLength,STFT_hopSize)
					
					DoWindow STFT_Window
					if(v_flag)
						SpHrView_DisplaySTFT()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
						
	endswitch
	return 0
End

// =================================================================================================
// SpHrView_Spectra_CheckProc(cba), Checkbox procedure for Spectra
// =================================================================================================
Function SpHrView_Spectra_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	Variable checked = cba.checked
	string traceListStr="", wStr=""
	strswitch( cba.ctrlName )
		case "ck_SpectrumFrameAverage":
		case "ck_UseSpectrumOctaveDenominator":
		case "ck_logFreqAxis":
			switch( cba.eventCode )
				case 2: // mouse up
					DoWindow Spectra_Window
					if(v_flag)
						SpHrView_DisplaySpectra()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "ck_STFTRevColorCode":
		case "ck_STFT_dB":
		case "ck_STFT_logHz":
			switch( cba.eventCode )
				case 2: // mouse up
					DoWindow STFT_Window
					if(v_flag)
						SpHrView_DisplaySTFT()
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
			
	endswitch
End

// =================================================================================================
// SpHrView_DisplaySpectra(), Display Amplitude spectrum
// =================================================================================================
Function SpHrView_DisplaySpectra()
	SpHrView_Spectra_Defaults()
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, minSec, maxSec
	DoWindow SpHrViewPeaksWindow
	if(!v_flag)
		return 0
	endif
	string traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
	string wStr =  StringFromList(0,traceListStr)
	wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
	string pathAndNameStr= SpHrView_FilePath_Get(w, 0) //note(w)
	NVAR fs= $SpHrView_df_globals+"fs" // assuming this won't change when wavefore segment loaded below
	NVAR SpectrumFramePoints= $SpHrView_df_spectra+"SpectrumFramePoints"
	NVAR SpectrumFrameAverage= $SpHrView_df_spectra+"SpectrumFrameAverage"
	NVAR FrameOverlap= $SpHrView_df_spectra+"FrameOverlap"
	NVAR SmoothSpectrum = $SpHrView_df_spectra+"SmoothSpectrum"
	NVAR OctaveDenominator=$SpHrView_df_spectra+"OctaveDenominator"
	variable DFT_Pnts = SpectrumFramePoints*2
	// get min and max times
	minSec = DimOffset(w, 0)
	maxSec = DimSize(w, 0)*DimDelta(w, 0) + DimOffset(w, 0)
	// get start and stop times
	GetMarquee/W=SpHrViewPeaksWindow/Z bottom
	//GetMarquee/Z bottom // allow marquee from another window...
	if(V_flag)
		endSec = v_right
		variable desiredPnts = (v_right-v_left)*fs
		DFT_Pnts = SpHr_DFTClosestSize(desiredPnts, 1)
		StartSec = v_right-(DFT_Pnts/fs)-(1/fs) // start ~1 point early...
	else
		variable cursor_xval = hcsr(A, "SpHrViewPeaksWindow")
		StartSec = cursor_xval-(DFT_Pnts/fs)-(1/fs) // start ~1 point early...
		EndSec = cursor_xval
	endif
	
	SpHrView_DrawSelection(1, startSec, endSec)
	// load waveforms
	SpHrView_LoadSoundFromFileWave(pathAndNameStr, SpHrView_df_files, "waveforms", startSec, endSec)
	wave waveforms=$SpHrView_df_files+"waveforms"
	Redimension/N=(DFT_Pnts,-1) waveforms // make sure length is correct
	if(!waveexists(waveforms))
		DoAlert 0, "SpHrView_DisplaySpectra: Waveforms are missing"
		return 0
	endif
	variable numCols = dimsize(waveforms,1)
	if(numCols>2)
		DoAlert 0, "SpHrView_DisplaySpectra: waveform has > 2 channels/columns"
		return 0
	endif
	
	// Add frame option
	
	// Spectrum
	if(SpectrumFrameAverage)
		if(SmoothSpectrum)
			SpHr_DFTSpectrum/Oct=(OctaveDenominator)/F={SpectrumFramePoints,FrameOverlap}/dB/Mag=$SpHrView_df_spectra+"Amplitude_Spectra" waveforms
		else
			SpHr_DFTSpectrum/F={SpectrumFramePoints,FrameOverlap}/dB/Mag=$SpHrView_df_spectra+"Amplitude_Spectra" waveforms
		endif
	else
		if(SmoothSpectrum)
			SpHr_DFTSpectrum/Oct=(OctaveDenominator)/N=(SpectrumFramePoints/2)/dB/W/Mag=$SpHrView_df_spectra+"Amplitude_Spectra" waveforms
		else
			SpHr_DFTSpectrum/N=(SpectrumFramePoints/2)/dB/W/Mag=$SpHrView_df_spectra+"Amplitude_Spectra" waveforms
		endif
	endif
	wave Amplitude_Spectra = $SpHrView_df_spectra+"Amplitude_Spectra"
	killwaves/Z waveforms
	numCols = dimsize(Amplitude_Spectra,1)
	variable startHz = 20
	variable startHzPnt = ScaleToIndex(Amplitude_Spectra, startHz, 0 )
	variable endHz =  DimSize(Amplitude_Spectra, 0)*DimDelta(Amplitude_Spectra, 0) + DimOffset(Amplitude_Spectra, 0)
	variable endHzPnt = ScaleToIndex(Amplitude_Spectra, endHz, 0 )
	
	DoWindow Spectra_Window
	if(!v_flag)
		variable spectrum_max, spectrum_min
		NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
		NVAR screenVert= $SpHrView_df_globals+"screenVert"
		NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
		NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
		variable wdth=screenHoriz*ViewWidthProportion*0.5
		variable hgt=screenVert*ViewHeightProportion*1.5
		Variable  winLeft, winRight, winTop, winBottom
		[winLeft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
		Display /K=1/W=(winLeft, winBottom+25, winLeft+wdth, winBottom+25+hgt) as "Spectra"
		DoWindow/C Spectra_Window
		ModifyGraph margin(left)=35,margin(bottom)=35,margin(top)=18,margin(right)=18
		AppendToGraph/L Amplitude_Spectra[][0]
		if(numCols > 1)
			AppendToGraph/L Amplitude_Spectra[][1]
		endif
		ModifyGraph lSize=0.75
		ModifyGraph rgb(Amplitude_Spectra)=(0,0,65535)
		ModifyGraph tick=2
		ModifyGraph font="Arial"
		ModifyGraph fSize=10
		ModifyGraph standoff=0
		ModifyGraph axThick=0.5
		ModifyGraph ZisZ=1, zapTZ=1, zapLZ=1
		ModifyGraph btLen=4
		ModifyGraph btThick=0.5
		ModifyGraph stLen=2
		ModifyGraph stThick=0.5
		ModifyGraph freePos(left)=0
		Label left "amplitude (\\U)"
		Label bottom "frequency (\\U)"
		ModifyGraph lblMargin(bottom)=2
		ModifyGraph lblMargin(left)=3
		ModifyGraph highTrip(bottom)=1000
		//endx = DimSize(Amplitude_Spectra, 0)*DimDelta(Amplitude_Spectra, 0) + DimOffset(Amplitude_Spectra, 0)
		SetAxis bottom startHz, endHz
		ModifyGraph lblPos(left)=35
		// scale
		wavestats/Q/M=1/RMD=[startHzPnt, endHzPnt][0,numCols] Amplitude_Spectra
		spectrum_max = v_max+3
		spectrum_min = v_min
		SetAxis left spectrum_min, spectrum_max
		ModifyGraph cbRGB=(65534,65534,65534)
		// left axis slider
		ControlBar/L 20
		Slider sl_leftAxisMax pos={0,45},size={3,hgt/3.5},side=0,proc=AxisSliderProc	
		Slider sl_leftAxisMax limits={90,0,3}, value=spectrum_max, help={"Maximum y-axis value"}
		Slider sl_leftAxisMin pos={0,hgt/1.65},size={5,hgt/3.5},side=0,proc=AxisSliderProc	
		Slider sl_leftAxisMin limits={80,-90,3}, value=spectrum_min, help={"Minimum y-axis value"}
		
		Button b_leftAxisAuto pos={5, hgt-20}, title="",size={18,18},fSize=9,proc=AxisButtonProc,help={"auto scale"} // " 🔍"
		Button b_leftAxisAuto picture=ProcGlobal#SpHrView_pict_GlassZoom
		

		// Top axis slider
		ControlBar/T 37
		NewPanel/HOST=Spectra_Window/FG=(FL,FT,GR,GT)/HOST=# 
		RenameWindow #,PTop
		
		variable left = 8
		CheckBox ck_holdYaxis win=$"Spectra_Window"+"#PTop", title="hold ↓",pos={left,10}
		left += 55
		TitleBox t_minX win=$"Spectra_Window"+"#PTop", title="min x",pos={left+wdth/20,7},frame=0
		Slider sl_bottomAxisMin win=$"Spectra_Window"+"#PTop", pos={left,15},size={wdth/10, 5},side=0, vert=0, proc=AxisSliderProc	
		Slider sl_bottomAxisMin win=$"Spectra_Window"+"#PTop", limits={1, endHz/20, 0}, value=50, help={"Minimum x-axis value"}
		left += 100
		GroupBox g_DFTpoints win=$"Spectra_Window"+"#PTop", pos={left-7,7},size={125,23}
		SetVariable sv_DFTpoints win=$"Spectra_Window"+"#PTop", pos={left,11.5},size={95.00,14.00},bodyWidth=40,title="DFT points"
		SetVariable sv_DFTpoints win=$"Spectra_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_DFTpoints win=$"Spectra_Window"+"#PTop", limits={128,65536,0},value= $SpHrView_df_spectra+"SpectrumFramePoints"
		left += 95
		Button b_DFTsize win=$"Spectra_Window"+"#PTop", pos={left,10},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
		Button b_DFTsize win=$"Spectra_Window"+"#PTop", proc=SpHrView_Spectra_ButtonProc
		
		left += 40
		GroupBox g_log win=$"Spectra_Window"+"#PTop", pos={left-10,7},size={70,23}
		CheckBox ck_logFreqAxis win=$"Spectra_Window"+"#PTop", title="log Hz",pos={left,10.5},side=0, proc=SpHrView_Spectra_CheckProc
		CheckBox ck_logFreqAxis win=$"Spectra_Window"+"#PTop", variable= $SpHrView_df_spectra+"SpectrumLogFreqAxis"
		
		left += 75
		GroupBox g_OctaveSmooth win=$"Spectra_Window"+"#PTop", pos={left-10,7},size={120,23}
		CheckBox ck_UseSpectrumOctaveDenominator win=$"Spectra_Window"+"#PTop", pos={left,10},side=0, title="Octave:", proc=SpHrView_Spectra_CheckProc
		CheckBox ck_UseSpectrumOctaveDenominator win=$"Spectra_Window"+"#PTop", variable= $SpHrView_df_spectra+"SmoothSpectrum"
		left += 55
		SetVariable sv_SpectrumOctaveDenominator win=$"Spectra_Window"+"#PTop", pos={left,11.5},size={47.00,14.00},bodyWidth=35,title="1/"
		SetVariable sv_SpectrumOctaveDenominator win=$"Spectra_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_SpectrumOctaveDenominator win=$"Spectra_Window"+"#PTop", limits={0.5,128,1},value= $SpHrView_df_spectra+"OctaveDenominator"
		
		left += 70
		GroupBox g_FrameAvg win=$"Spectra_Window"+"#PTop", pos={left-10,7},size={160,23}
		CheckBox ck_SpectrumFrameAverage win=$"Spectra_Window"+"#PTop", title="Average:",pos={left,10},side=0, proc=SpHrView_Spectra_CheckProc
		CheckBox ck_SpectrumFrameAverage win=$"Spectra_Window"+"#PTop", variable= $SpHrView_df_spectra+"SpectrumFrameAverage"
		CheckBox ck_SpectrumFrameAverage win=$"Spectra_Window"+"#PTop", help={"Use sliding frame average when DFT-points longer than sound selection (vs. decimation of longer DFT)"}
		left += 60
		SetVariable sv_SpectrumFrameOverlap win=$"Spectra_Window"+"#PTop", pos={left,11.5},size={77.00,14.00},bodyWidth=40,title="overlap"
		SetVariable sv_SpectrumFrameOverlap win=$"Spectra_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_SpectrumFrameOverlap win=$"Spectra_Window"+"#PTop", limits={0.5,128,1},value= $SpHrView_df_spectra+"FrameOverlap"
		
		TitleBox t_maxX win=$"Spectra_Window"+"#PTop", title="max x",pos={wdth-(wdth/8.5)+wdth/30,7},frame=0
		Slider sl_bottomAxisMax win=$"Spectra_Window"+"#PTop", pos={wdth-(wdth/8.5), 15},size={wdth/10, 5},side=0, vert=0, proc=AxisSliderProc	
		Slider sl_bottomAxisMax win=$"Spectra_Window"+"#PTop", limits={endHz/20, endHz, 0}, value=endHz, help={"Maximum x-axis value"}
		
		
		SetActiveSubwindow Spectra_Window //##
		ModifyGraph/W=Spectra_Window cbRGB=(65534,65534,65534)
		
		// Legend
		if(numCols > 1)
			Legend/C/N=text0/J/E=2/X=42.00/Y=3.00/F=0/B=1/A=LT "\\F'Arial'\\Z09\\s(Amplitude_Spectra) left   \\s(Amplitude_Spectra#1) right"	
		endif
		// Window Hook
		DoUpdate/W=Spectra_Window // for Igor 9
		SetWindow Spectra_Window,hook(SpectrumHook)=SpHrView_Spectra_WindowHook
		SetWindow Spectra_Window note="PACKAGE:SpHrView"
	else
		// scale
		ControlInfo/W=$"Spectra_Window"+"#PTop" ck_holdYaxis
		if(!V_Value)
			GetAxis/W=Spectra_Window/Q bottom
			startHzPnt = ScaleToIndex(Amplitude_Spectra, V_min, 0 )
			endHzPnt = ScaleToIndex(Amplitude_Spectra, V_max, 0 )
			wavestats/Q/M=1/RMD=[startHzPnt, endHzPnt][0,numCols] Amplitude_Spectra
			spectrum_max = v_max+3
			spectrum_min = v_min
			SetAxis/W=Spectra_Window  left spectrum_min, spectrum_max
			Slider sl_leftAxisMin win=Spectra_Window, value=spectrum_min
			Slider sl_leftAxisMax win=Spectra_Window, value=spectrum_max	
		endif
	endif
	NVAR SpectrumLogFreqAxis= $SpHrView_df_spectra+"SpectrumLogFreqAxis"
	if(SpectrumLogFreqAxis)
		ModifyGraph /W=Spectra_Window log(bottom)=1
	else
		ModifyGraph /W=Spectra_Window log(bottom)=0
	endif

End

//	DoWindow STFT_Window
//	if(!v_flag)
//		SpHrView_DisplaySTFT()
//	endif


// =================================================================================================
// SpHrView_DisplaySTFT(), Short-Time Fourier Transformation
// =================================================================================================
Function SpHrView_DisplaySTFT()
	SpHrView_Spectra_Defaults()
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, usingAxis
	[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
	wave waveforms=$SpHrView_df_files+"waveforms"
	if(!waveexists(waveforms))
		DoAlert 0, "SpHrView_DisplaySTFT: Waveforms are missing"
		return 0
	endif
	variable numCols = dimsize(waveforms,1)
	if(numCols>2)
		DoAlert 0, "SpHrView_DisplaySTFT: waveform has > 2 channels/columns"
		return 0
	endif
	// STFT
	DFREF DFRsaved= GetDataFolderDFR()
	SetDataFolder $SpHrView_df_spectra
	Make/O/N=(dimsize(waveforms,0)) temp_L
	temp_L = waveforms[p][0]
	if(numCols>1)
		Make/O/N=(dimsize(waveforms,0)) temp_R
		temp_R = waveforms[p][1]
	endif
	NVAR padPnts = $SpHrView_df_spectra+"STFT_PadLength"
	NVAR segPnts = $SpHrView_df_spectra+"STFT_SegmentLength"
	NVAR hopPnts = $SpHrView_df_spectra+"STFT_hopSize"
	SVAR WindowType = $SpHrView_df_spectra+"STFT_WindowType"
	NVAR STFT_dB = $SpHrView_df_spectra+"STFT_dB"
	NVAR STFT_logHz = $SpHrView_df_spectra+"STFT_logHz"
	padPnts += mod(padPnts,1) 	// make even if odd
	segPnts += mod(segPnts,1)	// make even if odd
	// make padded window to get window scaling factor
	Make/O/N=(segPnts) STFT_FrameWindow = 1
	copyscales/P waveforms, STFT_FrameWindow // for plotting
	WindowFunction $WindowType, STFT_FrameWindow
	//WaveStats/Q STFT_FrameWindow
	//variable winScale = V_rms//*V_rms//V_value//V_rms // window scaling factor
	variable halfDiff = ((padPnts-segPnts)/2)
	insertpoints/M=0 0, halfDiff, STFT_FrameWindow
	insertpoints/M=0 padPnts+halfDiff, halfDiff, STFT_FrameWindow
	//WaveStats/Q STFT_FrameWindow
	//variable winScale = V_rms//*V_rms//V_value//V_rms // window scaling factor
	if(segPnts>padPnts)
		padPnts = segPnts
	endif
	if(segPnts==padPnts)
		STFT/DB=0/HOPS=(hopPnts)/OUT=6/SEGS=(segPnts)/WINF=$WindowType/DEST=STFT_L temp_L
		if(numCols>1)
			STFT/DB=0/HOPS=(hopPnts)/OUT=6/SEGS=(segPnts)/WINF=$WindowType/DEST=STFT_R temp_R
		endif
	else
		STFT/DB=0/HOPS=(hopPnts)/OUT=6/SEGS=(segPnts)/PAD=(padPnts)/WINF=$WindowType/DEST=STFT_L temp_L
		if(numCols>1)
			STFT/DB=0/HOPS=(hopPnts)/OUT=6/SEGS=(segPnts)/PAD=(padPnts)/WINF=$WindowType/DEST=STFT_R temp_R
		endif
	endif
	Make/O/N=(dimsize(STFT_L,0),dimsize(STFT_L,1),2) STFTs
	STFTs [][][0] = STFT_L[p][q]
	if(numCols>1)
		STFTs [][][1] = STFT_R[p][q]
	endif
	STFTs /= (segPnts/2)
	//STFTs /= winScale
	
	if(STFT_dB)
		STFTs = 20*log(STFTs/0.00002) // dB re: 20 µPa
		SetScale d 0,0,"dB", STFTs
	else
		SetScale d 0,0,"Pa", STFTs
	endif
	
	SetScale/P x DimOffset(waveforms, 0)+dimdelta(STFT_L,0)*dimdelta(waveforms,0), dimdelta(STFT_L,0)*dimdelta(waveforms,0),"s", STFTs
	SetScale/P y 0,dimdelta(STFT_L,1)*round((1/dimdelta(waveforms,0))),"Hz", STFTs
	variable waveform_rightAxisVal = IndexToScale(waveforms, dimsize(waveforms,0), 0)
	

	killwaves/Z waveforms
	killwaves/Z temp_L, temp_R, STFT_L, STFT_R
	
	// Display STFTs
	NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
	NVAR screenVert= $SpHrView_df_globals+"screenVert"
	NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
	NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
	variable wdth=screenHoriz*ViewWidthProportion
	variable hgt=screenVert*ViewHeightProportion*1.75
	Variable  winLeft=(screenHoriz/6), winRight, winTop, winBottom=270
	[winLeft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
	Variable marginLeft, marginRight, marginTop, marginBottom
	[marginLeft, marginRight, marginTop, marginBottom] = getWindowMargins("SpHrViewPeaksWindow")	
	DoWindow STFT_Window
	if(v_flag)
		RemoveTraces("STFT_Window")
		RemoveImages("STFT_Window")
	else
		Display /K=1/W=(winLeft, winBottom+25, winLeft+wdth, winBottom+25+hgt) as "Short-Time Fourier Transformation"
		DoWindow/C STFT_Window
		ModifyGraph/W=STFT_Window margin(left)=marginLeft,margin(bottom)=25,margin(top)=6,margin(right)=marginRight
		
		ControlBar/W=STFT_Window/L 20
		ModifyGraph/W=STFT_Window cbRGB=(65534,65534,65534)
		variable Hz_min= dimdelta(STFTs,1)
		variable Hz_max= dimdelta(STFTs,1)*dimsize(STFTs,1)
		Slider sl_leftAxisMax win=STFT_Window, pos={0,45},size={5,hgt/3.5},side=0,proc=AxisSliderProc	
		Slider sl_leftAxisMax win=STFT_Window, limits={Hz_min,Hz_max,1}, value=Hz_max, help={"Maximum y-axis value"}
		Slider sl_leftAxisMin win=STFT_Window, pos={0,hgt/1.65},size={5,hgt/3.5},side=0,proc=AxisSliderProc	
		Slider sl_leftAxisMin win=STFT_Window, limits={Hz_min,Hz_max,1}, value=Hz_min, help={"Minimum y-axis value"}
		Button b_leftAxisAuto win=STFT_Window, pos={5, hgt-20}, title="",size={18,18},fSize=9,proc=AxisButtonProc,help={"auto scale"} // " 🔍"
		Button b_leftAxisAuto win=STFT_Window, picture=ProcGlobal#SpHrView_pict_GlassZoom
		
		// Top axis slider
		ControlBar/T 37
		NewPanel/HOST=STFT_Window/FG=(FL,FT,GR,GT)/HOST=# 
		RenameWindow #,PTop
		
		variable left = 20
		GroupBox g_dB win=$"STFT_Window"+"#PTop", pos={left-7,7},size={50,23}
		CheckBox ck_STFT_dB  win=$"STFT_Window"+"#PTop", title="dB",pos={left,10},side=0, proc=SpHrView_Spectra_CheckProc
		CheckBox ck_STFT_dB  win=$"STFT_Window"+"#PTop", variable= $SpHrView_df_spectra+"STFT_dB"
	
		left += 55
		GroupBox g_logHz win=$"STFT_Window"+"#PTop", pos={left-7,7},size={62,23}
		CheckBox ck_STFT_logHz  win=$"STFT_Window"+"#PTop", title="log Hz",pos={left,10},side=0, proc=SpHrView_Spectra_CheckProc
		CheckBox ck_STFT_logHz  win=$"STFT_Window"+"#PTop", variable= $SpHrView_df_spectra+"STFT_logHz"
		
		left += 70
		GroupBox g_DFTpoints win=$"STFT_Window"+"#PTop", pos={left-7,7},size={145,23}
		SetVariable sv_SegmentPoints win=$"STFT_Window"+"#PTop", pos={left,12.00},size={113,14.00},bodyWidth=40,title="Segment points"
		SetVariable sv_SegmentPoints win=$"STFT_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_SegmentPoints win=$"STFT_Window"+"#PTop", limits={128,65536,0},value= $SpHrView_df_spectra+"STFT_SegmentLength"
		left += 115
		Button b_SegmentPoints win=$"STFT_Window"+"#PTop", pos={left,11},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
		Button b_SegmentPoints win=$"STFT_Window"+"#PTop", proc=SpHrView_Spectra_ButtonProc
		
		left += 35
		GroupBox g_PadPoints win=$"STFT_Window"+"#PTop", pos={left-7,7},size={120,23}
		SetVariable sv_PadPoints win=$"STFT_Window"+"#PTop", pos={left,12.00},size={90,14.00},bodyWidth=40,title="FFT points"
		SetVariable sv_PadPoints win=$"STFT_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_PadPoints win=$"STFT_Window"+"#PTop", limits={128,65536,0},value= $SpHrView_df_spectra+"STFT_PadLength"
		left += 90
		Button b_PadPoints win=$"STFT_Window"+"#PTop", pos={left,11},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
		Button b_PadPoints win=$"STFT_Window"+"#PTop", proc=SpHrView_Spectra_ButtonProc
		
		left += 35
		GroupBox g_Overlap win=$"STFT_Window"+"#PTop", pos={left-7,7},size={190,23}
		SetVariable sv_OverlapPercent win=$"STFT_Window"+"#PTop", pos={left,12.00},size={96,14.00},bodyWidth=40,title="Overlap (%)"
		SetVariable sv_OverlapPercent win=$"STFT_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_OverlapPercent win=$"STFT_Window"+"#PTop", limits={0,99,1},value= $SpHrView_df_spectra+"STFT_OverlapPercent"
		left += 132
		SetVariable sv_hopSize win=$"STFT_Window"+"#PTop", pos={left,12.00},size={40.00,14.00},bodyWidth=35,title="points",fstyle=2
		SetVariable sv_hopSize win=$"STFT_Window"+"#PTop", fSize=9, noedit=1, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_hopSize win=$"STFT_Window"+"#PTop", limits={0,1000000,0},value= $SpHrView_df_spectra+"STFT_hopSize"
		SetVariable sv_hopSize win=$"STFT_Window"+"#PTop", help={"Hop size in points"}
		SetVariable sv_hopSize win=$"STFT_Window"+"#PTop", valueBackColor=(61166,61166,61166)//,labelBack=(61166,61166,61166)

		left += 65
		GroupBox g_Window win=$"STFT_Window"+"#PTop", pos={left-7,7},size={160,23}
		SetVariable sv_STFTWindowType win=$"STFT_Window"+"#PTop", pos={left,12.00},size={129.00,14.00},bodyWidth=90,title="Window"
		SetVariable sv_STFTWindowType win=$"STFT_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_STFTWindowType win=$"STFT_Window"+"#PTop", value= $SpHrView_df_spectra+"STFT_WindowType"
		left += 130
		Button b_STFTWindowType win=$"STFT_Window"+"#PTop", pos={left,11},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
		Button b_STFTWindowType win=$"STFT_Window"+"#PTop", proc=SpHrView_Spectra_ButtonProc

		left += 35
		GroupBox g_STFTColorCode win=$"STFT_Window"+"#PTop", pos={left-7,7},size={220,23}
		SetVariable sv_STFTColorCode win=$"STFT_Window"+"#PTop", pos={left,12.00},size={127.00,14.00},bodyWidth=95,title="Colors"
		SetVariable sv_STFTColorCode win=$"STFT_Window"+"#PTop", fSize=9, proc=SpHrView_Spectra_SetVarProc
		SetVariable sv_STFTColorCode win=$"STFT_Window"+"#PTop", value= $SpHrView_df_spectra+"STFT_ColorCode"
		left += 128
		Button b_STFTColorCode win=$"STFT_Window"+"#PTop", pos={left,11},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,15.00},title=""
		Button b_STFTColorCode win=$"STFT_Window"+"#PTop", proc=SpHrView_Spectra_ButtonProc
		left += 20
		CheckBox ck_STFTRevColorCode  win=$"STFT_Window"+"#PTop", title="reverse",pos={left,10},side=0, proc=SpHrView_Spectra_CheckProc
		CheckBox ck_STFTRevColorCode  win=$"STFT_Window"+"#PTop", variable= $SpHrView_df_spectra+"STFT_ColorCodeReverse"
	
	
		SetActiveSubwindow STFT_Window //##
		ModifyGraph/W=STFT_Window cbRGB=(65534,65534,65534)
		
	endif

	// Append images
	String wStr = "STFTs"
	AppendImage/W=STFT_Window/L=left_L/B=bottom_L STFTs
	ModifyGraph/W=STFT_Window freePos(left_L)=0
	ModifyGraph/W=STFT_Window freePos(bottom_L)={-0.5,left_L}
	SVAR ColorCodeStr = $SpHrView_df_spectra+"STFT_ColorCode"
	NVAR ColorCodeReverse = $SpHrView_df_spectra+"STFT_ColorCodeReverse"
	SpHr_ColorTable/R=(ColorCodeReverse)/Dest=ColorTable ColorCodeStr
	ModifyImage/W=STFT_Window $wStr ctab= {0,*,ColorTable,0},ctabAutoscale=1,lookup= $""
	// ticks and axes
	Label/W=STFT_Window left_L "frequency (\\U)  Left"
	ModifyGraph/W=STFT_Window highTrip(left_L)=10
	SetAxis/W=STFT_Window left_L 0,*
	SetAxis/W=STFT_Window bottom_L *,waveform_rightAxisVal
	if(numCols>1)
		AppendImage/W=STFT_Window/L=left_R/B=bottom_R STFTs
		ModifyGraph/W=STFT_Window axisEnab(left_L)={0.53,1},axisEnab(left_R)={0,0.47}
		ModifyGraph/W=STFT_Window freePos(left_R)=0, freePos(bottom_R)=0
		ModifyImage/W=STFT_Window $wStr+"#1" plane=1,ctab= {0,*,ColorTable,0},ctabAutoscale=1,lookup= $""
		// ticks and axes
		Label/W=STFT_Window left_R "frequency (\\U)  Right"
		ModifyGraph/W=STFT_Window highTrip(left_R)=10
		SetAxis/W=STFT_Window left_R 0,*
		SetAxis/W=STFT_Window bottom_R *,waveform_rightAxisVal
	endif

	
	if(1) // append FFT window on the left
		appendtograph/W=STFT_Window/R=right_L/B=bottom_L STFT_FrameWindow
		ModifyGraph/W=STFT_Window freePos(right_L)=0
		ModifyGraph/W=STFT_Window lsize(STFT_FrameWindow)=0.5,rgb(STFT_FrameWindow)=(0,0,0)
		ModifyGraph/W=STFT_Window tick(right_L)=3,noLabel(right_L)=2,axThick(right_L)=0
		if(numCols>1)
			appendtograph/W=STFT_Window/R=right_R/B=bottom_R STFT_FrameWindow
			ModifyGraph/W=STFT_Window freePos(right_R)=0
			ModifyGraph/W=STFT_Window lsize(STFT_FrameWindow#1)=0.5,rgb(STFT_FrameWindow#1)=(0,0,0)
			ModifyGraph/W=STFT_Window axisEnab(right_L)={0.53,1},axisEnab(right_R)={0,0.47}
			ModifyGraph/W=STFT_Window tick(right_R)=3,noLabel(right_R)=2,axThick(right_R)=0
		endif
		Legend/W=STFT_Window/C/N=windowLegend/B=1/J/F=0/A=LB/E=2/X=3.50/Y=2.00 "\\s(STFT_FrameWindow) window"
	else
		killwaves/Z STFT_FrameWindow
	endif
	
	if(STFT_logHz)
		SetAxis/W=STFT_Window left_L 50,*
		SetAxis/W=STFT_Window left_R 50,*
		ModifyGraph/W=STFT_Window log(left_L)=1, log(left_R)=1
	else
		SetAxis/W=STFT_Window/A left_L
		SetAxis/W=STFT_Window/A left_R
		ModifyGraph/W=STFT_Window log(left_L)=0, log(left_R)=0
	endif
	
	ModifyGraph/W=STFT_Window lblPos(left_L)=30
	ModifyGraph/W=STFT_Window lblPos(bottom_L)=20
	ModifyGraph/W=STFT_Window nticks(bottom_L)=10
	if(numCols>1)
		Label/W=STFT_Window bottom_L "\\u#2"
		Label/W=STFT_Window bottom_R "time (\\U)"
		ModifyGraph/W=STFT_Window lblPos(bottom_R)=25
		ModifyGraph/W=STFT_Window lblPos(left_R)=30,nticks(bottom_R)=10
	else
		Label bottom_L "time (\\U)"
	endif
	ModifyGraph/W=STFT_Window tick=2,font="Arial",fSize=10
	ModifyGraph/W=STFT_Window standoff=0,axThick=0.5
	ModifyGraph/W=STFT_Window ZisZ=1, zapTZ=1, zapLZ=1
	ModifyGraph/W=STFT_Window btLen=4,btThick=0.5,stLen=2,stThick=0.5
	// ColorScale
	ColorScale/W=STFT_Window/C/N=WaveformColorScale/F=0/B=1/A=RC/E=2/X=0.00/Y=6.50 vert=1,width=5
	ColorScale/W=STFT_Window/C/N=WaveformColorScale height=200,frame=0.00,image=$wStr
	ColorScale/W=STFT_Window/C/N=WaveformColorScale font="Arial",fsize=9,tickLen=1.50
	ColorScale/W=STFT_Window/C/N=WaveformColorScale tickThick=0.50,ZisZ=1
	ColorScale/W=STFT_Window/C/N=WaveformColorScale lblLatPos=-120,lblMargin=20 // lblLatPos=42,lblMargin=11
	ColorScale/W=STFT_Window/C/N=WaveformColorScale "\\U"
	
	// Window Hook
	DoUpdate/W=STFT_Window // for Igor 9
	SetWindow STFT_Window,hook(STFTHook)=SpHrView_Spectra_WindowHook
	SetWindow STFT_Window note="PACKAGE:SpHrView"

	SetDataFolder DFRsaved
End



// =================================================================================================
// SpHrView_Spectra_WindowHook(s), Window hook for SpHrView_WaveListWindow
// =================================================================================================
Function SpHrView_Spectra_WindowHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0	
	switch(s.eventCode)
		//////////////
		// kill event
		case 2:
			strswitch(s.winName)	
				case "Spectra_Window":
					Execute/Z/Q/P "killwaves/Z "+SpHrView_df_spectra+"Amplitude_Spectra"
					hookResult = 1
					break
						
				case "STFT_Window":
					//KillWindow/Z STFT_Window
					// Need to test these Execute/Z/Q/P commands
					Execute/Z/Q/P "killwaves/Z "+SpHrView_df_spectra+"STFTs"
					Execute/Z/Q/P "killwaves/Z "+SpHrView_df_spectra+"STFT_FrameWindow"
					hookResult = 1
					break
					
			endswitch
			break
			
		//////////////
		// keyboard event
//		case 11:
//			variable key= getkeystate(0) 
//			switch(s.keycode)
//				case 13:		// return
//					strswitch(s.winName)
//						case "":
//							break
//					endswitch
//					break
//			endswitch
//			break
	endswitch
	return 0
End
