﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// SpHrView_Transfer

strconstant SpHrView_df_transfer = "root:SpHrView:Transfer:"
strconstant SpHrView_df_transfer_impulses = "root:SpHrView:Transfer:ImpulseWaves:"
strconstant SpHrView_df_transfer_globals = "root:SpHrView:Transfer:Globals:"

// Farina, A. (2000): Simultaneous measurement of impulse response and distortion with a swept-sine technique. AES 108th Convention, Paris, Preprint no. 5093
// Weinzierl S., Giese A., Lindau A.. Generalized multiple sweep measurement. AES 126th Convention, Germany.

function SpHrView_TransferFunc_Init()
	variable rt_Mode
	DFREF saveDFR = GetDataFolderDFR()
	Variable nval
	String sval, FolderStr
	if(!DataFolderExists(SpHrView_df))	// Make "SpHrView" folder if it doesn't exist
		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	if(!DataFolderExists(SpHrView_df_transfer))	// Make "SpHrView:Transfer" folder if it doesn't exist
		FolderStr= SpHrView_df_transfer[0,strlen(SpHrView_df_transfer)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_transfer
	if(!DataFolderExists(SpHrView_df_transfer_impulses))	// Make "SpHrView:Transfer:Impulses" folder and global variables
		FolderStr= SpHrView_df_transfer_impulses[0,strlen(SpHrView_df_transfer_impulses)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	if(!DataFolderExists(SpHrView_df_transfer_globals))	// Make "SpHrView:Transfer:Globals" folder and global variables
		FolderStr= SpHrView_df_transfer_globals[0,strlen(SpHrView_df_transfer_globals)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_transfer_globals
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"Snd_fs", 48000)
	Variable/G Snd_fs = nval
	QuerySoundManager(SpHrView_df_transfer+"Globals") // no trailing ":"	
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"FrontBuff_ms", 200)
	Variable/G FrontBuff_ms = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"BackBuff_ms", 200)
	Variable/G BackBuff_ms = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"StartFreq", 20)
	Variable/G StartFreq = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"EndFreq", 20000)
	Variable/G EndFreq = nval
	
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"Duration", 10)
	Variable/G Duration = nval
	
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"CropStart", 1)
	Variable/G CropStart = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"WindowStart", 1)
	Variable/G WindowStart = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"ImpulseStartTime", -10)
	Variable/G ImpulseStartTime = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"ImpulseStartWindow", 5)
	Variable/G ImpulseStartWindow = nval
	
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"CropEnd", 1)
	Variable/G CropEnd = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"WindowEnd", 1)
	Variable/G WindowEnd = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"ImpulseEndTime", 1000)
	Variable/G ImpulseEndTime = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"ImpulseEndWindow", 10)
	Variable/G ImpulseEndWindow = nval
	
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"DisplayRecording", 1)
	Variable/G DisplayRecording = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"DisplaySweep", 0)
	Variable/G DisplaySweep = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"DisplayInverseFilter", 0)
	Variable/G DisplayInverseFilter = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"DisplayImpulses", 1)
	Variable/G DisplayImpulses = nval
	nval = NumVarOrDefault(SpHrView_df_transfer_globals+"DisplaySpectra", 0)
	Variable/G DisplaySpectra = nval
	
	
	SetDataFolder saveDFR
End

Function SpHrView_TransferFuncLogSweepWindow()
	SpHrView_Defaults()
	SpHrView_TransferFunc_Init()
	DFREF saveDFR = GetDataFolderDFR()
	
	DoWindow TransferFunctionLogSweep_window
	if(v_flag)
		killwindow/Z TransferFunctionLogSweep_window
	endif
	Variable winLeft, winRight, winTop, winBottom
	[winLeft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
	winBottom = max(winBottom,30)
	winLeft=15; winTop=winBottom+30
	winRight=winLeft+270; winBottom=winTop+490
	NewPanel /K=1 /W=(winLeft,winTop,winRight,winBottom) as "Transfer function - log sweep"
	DoWindow/C TransferFunctionLogSweep_window
	variable top=7
	Button b_PlaySweep,pos={50.00,top},size={30.00,30.00},title="",picture=ProcGlobal#SpHrView_pict_circlePlay // title="\\K(1,39321,19939)\\f01Play\\f00 \\K(0,0,0)Sweep"
	Button b_PlaySweep,fSize=11,proc=SpHrView_TransferFunc_ButtonProc,help={"Play sweep"}
	Button b_StopSweep,pos={80.00,top},size={30.00,30.00},title="",picture=ProcGlobal#SpHrView_pict_circleStop // title="\\K(1,39321,19939)\\f01Play\\f00 \\K(0,0,0)Sweep"
	Button b_StopSweep,fSize=11,proc=SpHrView_TransferFunc_ButtonProc,help={"Stop playing sweep"}
//	GroupBox g_Play,pos={7.00,top-18},size={130,45.00},bodyWidth=30,title="Play Sweep"
//	GroupBox g_Play,fSize=10,fStyle=1
	
	Button b_PlayRecord,pos={160.00,top},size={70.00,30.00},title="",picture=ProcGlobal#SpHrView_pict_circlePlayRecord //title="\\K(1,39321,19939)\\f01Play\\f00 \\K(0,0,0)and \\K(65535,0,0)\\f01Record\\f00"
	Button b_PlayRecord,fSize=11,proc=SpHrView_TransferFunc_ButtonProc,help={"Play and record sweep to get impulse response(s)"}
//	GroupBox g_PlayRecord,pos={145.00,top-18},size={115,45.00},bodyWidth=30,title="Play & Record"
//	GroupBox g_PlayRecord,fSize=10,fStyle=1
	
	NVAR SoundInChannels = $SpHrView_df_transfer_globals+"SoundInChannels"
	top += 38
	GroupBox g_InputChannels,pos={7.00,top},size={130,42.00},bodyWidth=30,title="Input"
	GroupBox g_InputChannels,fSize=10,fStyle=1
	GroupBox g_samplingrate,pos={145.00,top},size={115,42.00},title="Sample rate"
	GroupBox g_samplingrate,fSize=10,fStyle=1
	top += 20
	SetVariable sv_InputChannels,pos={35.00,top},size={89,16.00},bodyWidth=40,title="Channels"
	SetVariable sv_InputChannels,fSize=10,limits={1,SoundInChannels,1},value=$SpHrView_df_transfer_globals+"SoundInChannels"
	if(SoundInChannels <= 1)
		GroupBox g_InputChannels,disable=2
		SetVariable sv_InputChannels,disable=2
	endif
	SetVariable sv_samplingrate,pos={147.00,top},size={85.00,16.00},bodyWidth=48,title="(Hz)"
	SetVariable sv_samplingrate,fSize=10,limits={22050,192000,0},value=$SpHrView_df_transfer_globals+"Snd_fs"
	Button b_samplingrate,pos={235,top-1},picture=ProcGlobal#SpHrView_pict_PopUpList, size={16.00,17.00},title=""
	Button b_samplingrate,proc=SpHrView_TransferFunc_ButtonProc,focusRing=0,help={"Input Sampling Rate"}
	
	// Duration
	top += 30
	GroupBox g_Duration,pos={7.00,top},size={130,42.00},bodyWidth=30,title="Sweep Duration"
	GroupBox g_Duration,fSize=10,fStyle=1
	top += 20
	SetVariable sv_Duration,pos={15.00,top},size={111.00,16.00},bodyWidth=50,title="Duration (s)"
	SetVariable sv_Duration,fSize=10,limits={0.1,30,0.1},value=$SpHrView_df_transfer_globals+"Duration"
	
	// Frequencies
	top += 26
	GroupBox g_frequencies,pos={7.00,top},size={254.00,42.00},title="Sweep Frequencies"
	GroupBox g_frequencies,fSize=10,fStyle=1
	top += 20
	SetVariable sv_StartFreq,pos={27.00,top},size={97.00,16.00},bodyWidth=55,title="Start (Hz)"
	SetVariable sv_StartFreq,fSize=10,limits={50,48000,10},value=$SpHrView_df_transfer_globals+"StartFreq"
	Button b_swapFreqs,pos={127,top}, size={17.00,16.00},fSize=10,fColor=(61166,61166,61166),title=Char_swapArrowStr // Char_swapArrow10_9_Str
	Button b_swapFreqs,proc=SpHrView_TransferFunc_ButtonProc,focusRing=0,help={"Swap Starting and Ending frequencies"}
	SetVariable sv_EndFreq,pos={143.00,top},size={110.00,16.00},bodyWidth=55,title="End (Hz)"
	SetVariable sv_EndFreq,fSize=10,limits={50,48000,500},value=$SpHrView_df_transfer_globals+"EndFreq"
	
	// time buffers
	top += 26
	GroupBox g_buffers,pos={7.00,top},size={254.00,42.00},title="Sweep Zero Buffers"
	GroupBox g_buffers,fSize=10,fStyle=1
	top += 20
	SetVariable sv_frontBufferSec,pos={17.00,top},size={107.00,16.00},bodyWidth=50,title="Front (ms)"
	SetVariable sv_frontBufferSec,fSize=10,limits={0,10000,0.5},value=$SpHrView_df_transfer_globals+"FrontBuff_ms"
	SetVariable sv_backBufferSec1,pos={148.00,top},size={105.00,16.00},bodyWidth=50,title="Back (ms)"
	SetVariable sv_backBufferSec1,fSize=10,limits={0,10000,0.5},value=$SpHrView_df_transfer_globals+"BackBuff_ms"
	
	// crop and window
	top += 35
	GroupBox g_ImpulseStartTimes,pos={7.00,top},size={254.00,62.00},title="Impulse Front"
	GroupBox g_ImpulseStartTimes,fSize=10,fStyle=1
	top += 20
	CheckBox ck_CropStart,pos={30.00,top},size={66.00,16.00},title="Crop front"
	CheckBox ck_CropStart,fSize=10,variable=$SpHrView_df_transfer_globals+"CropStart"
	CheckBox ck_WindowStart,pos={143.00,top},size={66.00,16.00},title="Window front"
	CheckBox ck_WindowStart,fSize=10,variable=$SpHrView_df_transfer_globals+"WindowStart"
	top += 20
	SetVariable sv_CropBeforeImpulse,pos={17.00,top},size={107.00,16.00},bodyWidth=50,title="Front (-ms)"
	SetVariable sv_CropBeforeImpulse,fSize=10,limits={-5000,-1,1},value=$SpHrView_df_transfer_globals+"ImpulseStartTime"
	SetVariable sv_ImpulseStartWindow,pos={136.00,top},size={117.00,16.00},bodyWidth=50,title="Length (ms)"
	SetVariable sv_ImpulseStartWindow,fSize=10,limits={1,5000,1},value=$SpHrView_df_transfer_globals+"ImpulseStartWindow"
	
	top += 26
	GroupBox g_ImpulseEndTime,pos={7.00,top},size={254.00,62.00},title="Impulse Back"
	GroupBox g_ImpulseEndTime,fSize=10,fStyle=1
	top += 20
	CheckBox ck_CropEnd,pos={30.00,top},size={66.00,16.00},title="Crop back"
	CheckBox ck_CropEnd,fSize=10,variable=$SpHrView_df_transfer_globals+"CropEnd"
	CheckBox ck_WindowEnd,pos={143.00,top},size={66.00,16.00},title="Window back"
	CheckBox ck_WindowEnd,fSize=10,variable=$SpHrView_df_transfer_globals+"WindowEnd"
	top += 20
	SetVariable sv_CropEndImpulse,pos={27.00,top},size={97.00,16.00},bodyWidth=50,title="Back (ms)"
	SetVariable sv_CropEndImpulse,fSize=10,limits={0,20000,100},value=$SpHrView_df_transfer_globals+"ImpulseEndTime"
	SetVariable sv_ImpulseEndWindow,pos={136.00,top},size={117.00,16.00},bodyWidth=50,title="Length (ms)"
	SetVariable sv_ImpulseEndWindow,fSize=10,limits={1,10000,1},value=$SpHrView_df_transfer_globals+"ImpulseEndWindow"
	
	// Display Windows
	top += 32
	GroupBox g_Display,pos={7.00,top},size={254.00,100.00},title="Display"
	GroupBox g_Display,fSize=10,fStyle=1
	top += 28
	CheckBox ck_DisplayRecording,pos={30.00,top},size={66.00,16.00},proc=SpHrView_TransferFunc_CheckProc,title="Recording"
	CheckBox ck_DisplayRecording,fSize=10,variable=$SpHrView_df_transfer_globals+"DisplayRecording"
	CheckBox ck_DisplayImpulses,pos={150.00,top},size={55.00,16.00},proc=SpHrView_TransferFunc_CheckProc,title="Impulse(s)"
	CheckBox ck_DisplayImpulses,fSize=10,variable=$SpHrView_df_transfer_globals+"DisplayImpulses"
	top += 20
	CheckBox ck_DisplaySweep,pos={30.00,top},size={49.00,16.00},proc=SpHrView_TransferFunc_CheckProc,title="Sweep"
	CheckBox ck_DisplaySweep,fSize=10,variable=$SpHrView_df_transfer_globals+"DisplaySweep"
	CheckBox ck_DisplaySpectra,pos={150.00,top},size={64.00,16.00},proc=SpHrView_TransferFunc_CheckProc,title="Spectra"
	CheckBox ck_DisplaySpectra,fSize=10,variable=$SpHrView_df_transfer_globals+"DisplaySpectra"
	top += 20
	CheckBox ck_DisplayInverseFilter,pos={30.00,top},size={80.00,16.00},proc=SpHrView_TransferFunc_CheckProc,title="Inverse Filter"
	CheckBox ck_DisplayInverseFilter,fSize=10,variable=$SpHrView_df_transfer_globals+"DisplayInverseFilter"
		
	DoUpdate/W=TransferFunctionLogSweep_window // for Igor 9
	SetWindow TransferFunctionLogSweep_window, hook(TransferFuncHook)=SpHrView_TransferFunc_WindowHook
	SetWindow TransferFunctionLogSweep_window note="PACKAGE:SpHrView;GROUP:TransferFunc"
	
	SetDataFolder saveDFR
End

// =================================================================================================
// SpHrView_TransferFunc_WindowHook(s), Window hook for TransferFunc windows
// =================================================================================================
Function SpHrView_TransferFunc_WindowHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0	
	switch(s.eventCode)
		//////////////
		// deactivate
		case 1:
			strswitch(s.winName)
				case "TransferFunctionLogSweep_window":
					//killwindow/Z TransferFunctionLogSweep_window // close window when deactivted
					hookResult=1
					break
			endswitch
			break
		//////////////
		// keyboard event
		case 11:
			variable key= getkeystate(0) 
			strswitch(s.winName)
				case "TransferFunctionLogSweep_window":
					switch(s.keycode)
						case 13:		// return
							SpHrView_TransferFuncPlayRec(1)
							hookResult=1
							break
					endswitch
					break
				endswitch
			break
			
		//////////////
		// kill event
		case 2:
			strswitch(s.winName)					
				case "TransferFunctionLogSweep_window":
				case "RecordedSweepWindow":
				case "SweepImpulseSpectraWindow":
				case "SweepImpulsesWindow":
				case "SweepInverseFilterWindow":
				case "SweepWindow":
				
					variable winCnt = SpHrView_CountGroupWindows("TransferFunc")
					if(winCnt<=1) // <= 1 because last window is being killed
						Execute/Z/Q/P "KillAllWavesInDatafolder(\""+SpHrView_df_transfer[0,strlen(SpHrView_df_transfer)-2]+"\")" // remove trailing ":"
					endif
					hookResult = 1
					break
					
			endswitch
			break
			
	endswitch
	return hookResult
End

Function SpHrView_TransferFunc_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			Variable checked = cba.checked
			strswitch(cba.ctrlName)
				case "ck_DisplayRecording":
					wave RecordedSweep = $SpHrView_df_transfer+"RecordedSweep"
					if(checked && waveexists(RecordedSweep))
						SpHrView_RecordedSweepWindow()
					else
						DoWindow RecordedSweepWindow
						if(!checked && v_flag)
							killwindow/Z RecordedSweepWindow
						endif
					endif
					break
				case "ck_DisplaySweep":
					wave BufferedSweep = $SpHrView_df_transfer+"BufferedSweep"
					if(checked && waveexists(BufferedSweep))
						SpHrView_SweepWindow()
					else
						DoWindow SweepWindow
						if(!checked && v_flag)
							killwindow/Z SweepWindow
						endif
					endif
					break
				case "ck_DisplayInverseFilter":
					wave SweepInverseFilter = $SpHrView_df_transfer+"SweepInverseFilter"
					if(checked && waveexists(SweepInverseFilter))
						SpHrView_SweepInverseFilterWindow()
					else
						DoWindow SweepInverseFilterWindow
						if(!checked && v_flag)
							killwindow/Z SweepInverseFilterWindow
						endif
					endif
					break
					
				case "ck_DisplayImpulses":
					wave Impulses = $SpHrView_df_transfer+"Impulses"
					if(checked && waveexists(Impulses))
						SpHrView_SweepImpulsesWindow()
					else
						DoWindow SweepImpulsesWindow
						if(!checked && v_flag)
							killwindow/Z SweepImpulsesWindow
						endif
					endif
					break
				case "ck_DisplaySpectra":
					wave SweepImpulseSpectra = $SpHrView_df_transfer+"SweepImpulseSpectra"
					if(checked && waveexists(SweepImpulseSpectra))
						SpHrView_SweepImpulseSpectraWindow()
					else
						DoWindow SweepImpulseSpectraWindow
						if(!checked && v_flag)
							killwindow/Z SweepImpulseSpectraWindow
						endif
					endif
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

	

Function SpHrView_TransferFunc_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 1: // mouse down
			String WaveNameStr
			strswitch(ba.ctrlName)
				case "b_PlaySweep":
					SpHrView_TransferFuncPlayRec(0)
					break
				case "b_StopSweep":
					StopPlaySound()
					break
				case "b_PlayRecord":
					SpHrView_TransferFuncPlayRec(1)
					break
				case "b_samplingrate":
					SVAR rateList = $SpHrView_df_transfer_globals+"rateList"
					PopupContextualMenu rateList
					if(v_flag)
						NVAR Snd_fs = $SpHrView_df_transfer_globals+"Snd_fs"
						Snd_fs = str2num(StringFromList(v_flag-1, rateList))
					endif
					break
					
				case "b_SaveImpulseFile":
					wave/Z Impulses = $SpHrView_df_transfer+"Impulses"
					if(waveexists(Impulses))
						Save/I/C Impulses
					else
						DoAlert/T="Warning" 0, "Wave "+SpHrView_df_transfer+"Impulses" + " is missing"
					endif
					break
					
				case "b_SaveRecordedFile":
					wave/Z RecordedSweep = $SpHrView_df_transfer+"RecordedSweep"
					if(waveexists(RecordedSweep))
						Save/I/C RecordedSweep
					else
						DoAlert/T="Warning" 0, "Wave "+SpHrView_df_transfer+"RecordedSweep" + " is missing"
					endif
					break
				
					
				case "b_SaveImpulseWave":
					wave Impulses = $SpHrView_df_transfer+"Impulses"
					if(waveexists(Impulses))
						WaveNameStr = "Impulses_Untitled"
						Prompt WaveNameStr, "Enter name of wave: "		// Set prompt for x param
						DoPrompt "Wave Name", WaveNameStr
						if (V_Flag)
							return -1								// User canceled
						endif
						WaveNameStr = cleanupName(WaveNameStr,0)
						wave/Z w = $SpHrView_df_transfer_impulses+WaveNameStr
						if(waveexists(w))
							DoAlert/T="Warning" 1, "Wave:  '" + WaveNameStr+"'  already exists!\rOverwrite this wave?\r\r" + "Full path to wave:\r" + SpHrView_df_transfer_impulses + WaveNameStr
							if(v_flag != 1)
								return 0
							endif
						endif
						Duplicate/O Impulses, $SpHrView_df_transfer_impulses+WaveNameStr
					else
						DoAlert/T="Warning" 0, "Wave "+SpHrView_df_transfer+"Impulses" + " is missing"
					endif
					break
					
				case "b_SaveRecordedWave":
					wave RecordedSweep = $SpHrView_df_transfer+"RecordedSweep"
					if(waveexists(RecordedSweep))
						WaveNameStr = "Recording_Untitled"
						Prompt WaveNameStr, "Enter name of wave: "		// Set prompt for x param
						DoPrompt "Wave Name", WaveNameStr
						if (V_Flag)
							return -1								// User canceled
						endif
						WaveNameStr = cleanupName(WaveNameStr,0)
						wave/Z w = $SpHrView_df_waves+WaveNameStr
						if(waveexists(w))
							DoAlert/T="Warning" 1, "Wave:  '" + WaveNameStr+"'  already exists!\rOverwrite this wave?\r\r" + "Full path to wave:\r" + SpHrView_df_transfer_impulses + WaveNameStr
							if(v_flag != 1)
								return 0
							endif
						endif
						Duplicate/O RecordedSweep, $SpHrView_df_waves+WaveNameStr
					else
						DoAlert/T="Warning" 0, "Wave "+SpHrView_df_transfer+"RecordedSweep" + " is missing"
					endif
					break
					
				case "b_swapFreqs":
					NVAR StartFreq = $SpHrView_df_transfer_globals+"StartFreq"
					NVAR EndFreq = $SpHrView_df_transfer_globals+"EndFreq"
					variable temp = StartFreq
					StartFreq = EndFreq
					EndFreq = temp
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

Function SpHrView_TransferFuncPlayRec(variable doRec)
	DFREF saveDFR = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_transfer
	NVAR fs = $SpHrView_df_transfer_globals+"Snd_fs"
	NVAR Duration = $SpHrView_df_transfer_globals+"Duration"
	NVAR FrontBuff_ms = $SpHrView_df_transfer_globals+"FrontBuff_ms"
	NVAR BackBuff_ms = $SpHrView_df_transfer_globals+"BackBuff_ms"
	NVAR StartFreq = $SpHrView_df_transfer_globals+"StartFreq"
	NVAR EndFreq = $SpHrView_df_transfer_globals+"EndFreq"
	NVAR SoundInChannels = $SpHrView_df_transfer_globals+"SoundInChannels"
	variable InputChannels = SoundInChannels // may change InputChannels
	NVAR DisplayRecording = $SpHrView_df_transfer_globals+"DisplayRecording"
	NVAR DisplaySweep = $SpHrView_df_transfer_globals+"DisplaySweep"
	NVAR DisplayInverseFilter = $SpHrView_df_transfer_globals+"DisplayInverseFilter"
	NVAR DisplayImpulses = $SpHrView_df_transfer_globals+"DisplayImpulses"
	NVAR DisplaySpectra = $SpHrView_df_transfer_globals+"DisplaySpectra"
	if(InputChannels == 1)
		InputChannels = 0
	endif
	
	SpHr_SignalSweep /Type="log" /Sec=(Duration)/Ramp=5/Buff={FrontBuff_ms,BackBuff_ms} fs, StartFreq, EndFreq, BufferedSweep
	
	if(!doRec)
		PlaySound/A BufferedSweep
		return 0
	endif
	
	// make sure sound system is awake
	Make/O/N=(1024,InputChannels)/FREE Signal_init=0, rec_init=0
	SetScale/P x, 0, dimdelta(BufferedSweep,0), "s", Signal_init, rec_init
	PlaySound/A Signal_init
	SoundInRecord/Z rec_init
	sleep/S (dimsize(Signal_init,0)*dimdelta(Signal_init,0))+0.5
	
	Make/O/N=(dimsize(BufferedSweep,0), InputChannels) RecordedSweep
	SetScale/P x, 0, dimdelta(BufferedSweep,0), "s", RecordedSweep
	SetScale d 0,0,"Pa", RecordedSweep
	// Play
	PlaySound/A BufferedSweep
	// Record
	SoundInRecord RecordedSweep
	sleep/S 0.5
	
	if(DisplayRecording)
		SpHrView_RecordedSweepWindow()
	endif
	if(DisplaySweep)
		SpHrView_SweepWindow()
	endif
	
	SpHr_SignalSweep /Type="log" /Sec=(Duration)/Ramp=5/Inv=SweepInverseFilter fs, StartFreq, EndFreq, Sweep
	
	if(InputChannels>=2)
		Redimension/N=(-1,2) Sweep, BufferedSweep, SweepInverseFilter
		Sweep[][1] = Sweep[p][0]
		BufferedSweep[][1] = BufferedSweep[p][0]
		SweepInverseFilter[][1] = SweepInverseFilter[p][0]
	endif
	
	// estimate system delay and align
	variable numFreqs = 8
	variable lowFreq, highFreq
	if(StartFreq < EndFreq)
		lowFreq = StartFreq
		highFreq = EndFreq
	else
		lowFreq = EndFreq
		highFreq = StartFreq
	endif
	SpHr_FilterCenterFreqs/Type=3 lowFreq, highFreq, numFreqs, CFs // linear spacing
	SpHr_FilterBankBiquadCoefs fs, CFs, coefs
	SpHr_FilterBankBiquad/Dest=SweepBank CFs, coefs, BufferedSweep // BufferedSweep // Sweep
	SpHr_FilterBankBiquad/Dest=RecordedSweepBank CFs, coefs, RecordedSweep
	killwaves/Z CFs, coefs
	
	//return 0
	
	SpHr_Correlate/Env=1/AE={CorrelAvgPeaks} RecordedSweepBank, SweepBank // /A=CorrelAvg
	killwaves/Z RecordedSweepBank, SweepBank
	
	//return 0
	
	Variable lagTime, SystemDelayPnts
		// use shortest lag if using 2 channels
	if(InputChannels == 2)
		if(abs(CorrelAvgPeaks[0][0]) < abs(CorrelAvgPeaks[1][0]))
			lagTime = (CorrelAvgPeaks[0][0])
		else
			lagTime = (CorrelAvgPeaks[1][0])
		endif		
		SystemDelayPnts = round(abs(lagTime)/(1/fs))
	else
		lagTime = CorrelAvgPeaks[0][0]
		SystemDelayPnts = round(abs(lagTime)/(1/fs))
	endif
	killwaves/Z CorrelAvgPeaks

	if(lagTime < 0)
		InsertPoints 0, SystemDelayPnts, RecordedSweep
	else
		DeletePoints 0, SystemDelayPnts, RecordedSweep
	endif
	Redimension/N=(dimsize(BufferedSweep,0), -1) RecordedSweep
	
	//if(!DisplayImpulses && !DisplaySpectra)
	//	return 0
	//endif
	
	// get impuluse(s)
	SpHr_Convolve/CX=1/Dest=Impulses RecordedSweep, SweepInverseFilter
	
	// adjust impulse scale for front buffer
	SetScale/P x, dimoffset(Impulses,0)+(FrontBuff_ms/1000), dimdelta(Impulses,0), "s", Impulses
	
	// crop impluse time(s)
	// start
	NVAR CropStart = $SpHrView_df_transfer_globals+"CropStart"
	NVAR ImpulseStartTime = $SpHrView_df_transfer_globals+"ImpulseStartTime"
	if(CropStart)
		variable pnt = x2pnt(Impulses,ImpulseStartTime/1000)
		DeletePoints 0, pnt, Impulses
		SetScale/P x, -abs(ImpulseStartTime/1000), dimdelta(Impulses,0), "s", Impulses
		SetScale d 0,0,"Pa", Impulses
	endif
	// end
	NVAR CropEnd = $SpHrView_df_transfer_globals+"CropEnd"
	NVAR ImpulseEndTime = $SpHrView_df_transfer_globals+"ImpulseEndTime"
	if(CropEnd)
		pnt = x2pnt(Impulses,ImpulseEndTime/1000)
		//DeletePoints pnt, dimsize(Impulses,0), Impulses
		Redimension/N=(pnt,-1) Impulses
	endif
	
	// Ramp impuluse(s)
	NVAR WindowStart = $SpHrView_df_transfer_globals+"WindowStart"
	NVAR WindowEnd = $SpHrView_df_transfer_globals+"WindowEnd"
	NVAR ImpulseStartWindow = $SpHrView_df_transfer_globals+"ImpulseStartWindow"
	NVAR ImpulseEndWindow = $SpHrView_df_transfer_globals+"ImpulseEndWindow"
		// Impulses = 1
	if(WindowStart && WindowEnd)
		SpHr_RampOnOff ImpulseStartWindow, ImpulseEndWindow, Impulses
	elseif(!WindowStart && WindowEnd)
		SpHr_RampOnOff 0, ImpulseEndWindow, Impulses
	elseif(WindowStart && !WindowEnd)
		SpHr_RampOnOff ImpulseStartWindow, 0, Impulses
	endif
	
	// Set notes for Impulse wave
	string noteStr=""
	noteStr = "Date="+Secs2Date(DateTime,-2) + ";Time="+Secs2Time(DateTime,2)
	Note/K Impulses, noteStr
	
	SVAR InputDeviceName = $SpHrView_df_transfer_globals+"InputDeviceName"
	noteStr = "InputDevice="+InputDeviceName+";"
	Note Impulses, noteStr
	
	SPrintf noteStr, "fs=%g;numInputs=%d;", fs, InputChannels
	Note Impulses, noteStr
	SPrintf noteStr, "F1=%g;F2=%g;Duration=%g", StartFreq, EndFreq, Duration
	Note Impulses, noteStr
	
	if(CropStart && CropEnd && WindowStart && WindowEnd)
		SPrintf noteStr, "FrontBuffer=%g;FrontWindow=%g;", ImpulseStartTime, ImpulseStartWindow
		Note Impulses, noteStr
		SPrintf noteStr, "BackBuffer=%g;BackWindow=%g;", ImpulseEndTime, ImpulseEndWindow
		Note Impulses, noteStr
	else
		noteStr = "Impulse not fully cropped or windowed"
		Note Impulses, noteStr
	endif
	
	
	// Impulse Spectra
	SpHr_DFTSpectrum/dB=1/Oct=24/Mag=SweepImpulseSpectra Impulses
	
	if(DisplayInverseFilter)
		SpHrView_SweepInverseFilterWindow()
	else
		//killwaves/Z SweepInverseFilter
	endif
		
	if(DisplayImpulses)
		SpHrView_SweepImpulsesWindow()
	endif
	if(DisplaySpectra)
		SpHrView_SweepImpulseSpectraWindow()
	endif
		
	
	SetDataFolder saveDFR
End


// PNG: width= 105, height= 15
Picture SpHrView_pict_circlePlayRecord
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!&.!!!!`#Qau+!!B!BK)blTCM?aMFBD723b*)F/n&:/@V%0:D/]IT!!!
	!]5Bq-U@r#Xt+C\npBl6'Xn%6=Y+DG@L,$-Wr0MQLRAS>dKH?*RqH;7FLH>@"NA0O6%$9WPoG\qD:A
	TV?'G\q87F#nP/,%5##@V%T<F#n/LFC?gH+F#qlD/aT44X+!&:`r,DEb-@N/i>78,!H>V4E=tE3`8@
	8+F%a>DK@jZA7dtKBQS?83\N.1GBYZ`1G3TdB.ku"3B8`H1+tC</TPB6/TZ2TFCBDGDK@$H4piDT4E
	=tE3^dP#@rc:&FD5Z2+ED")3alK9F`]bj+poG8+<YoGCi4;TA7IbHBQS?83\N.*F`M:NDfTJDA7I9(
	Ch7HpDKKqQ0eP16+poG8+<YoGCi4;TAU%p$4X+Q]FDs8o05bh`@:X:cAM.J2D(fIFBkB"Q/heq&$6U
	H6+F%a>DK@j\BkCs?,%u(?E&oX*DK@F=A8bpg/n8g:06Co?AhPkk0J=UW+<VdLG\q87F#n8RDffo=B
	Q%iQ,%u(?E&oX*DK@F=A8bpg/n8g:05tTCFDl56Df@?a/heq&$6UH6+F%a>DK@j`D/_W^BQS?83\N.
	(F"Um3Ddm91@rH3;G[YPE0eP.5+poG8+<YoGCi4;TG\qCo9g'\6FEDI_0/%3a/n&:/@V%0%Df%.P@;
	mkS/heqqD(d(?+<VdLG\q87F#nAa7<Wc9,%u(?E&oX*DK@F=A8bpg/n8g:06goE0/5(506:*<E+L\#
	ATMd4EarZLG%GQ5,9dI;+<Y6?BkBD&Bm=3*=@?k?ASuR'Df.]^1c%*8$6UH6AU%p$3`'O8ASbL;Bl.
	F!F(oQ14X)d?+poG8+D#e3Ai`:tCi=>mE+*6f4X)U)$6UH6FD5?!3_=0sB4Y.YA9;KJ+uD)8+poG8+
	EV=0Ai`M#@:s.FASu.&BK[Ot1Em/K+<Yc?AnF)%ATMd+F`_>9DH1RgF?Lg'+poG8+EV=0Aia$lATMd
	+F`_>9DET1"0JG+5+poG8+EV=0Aia'mATMd+F`_>9DET1"0JG+5+poG8+E2"4FDl56Df@a)Des?49l
	FQR4X)[+$6UH6E+j03Df]T1E&pQX6VqfAAnc'm4X+rF7m\=i77/sf3ArcI1+k:*$6UH6G\qC\9lFQV
	ApHdQFCdTZ1,(I;/MK"50eu]_1HI6E3\W<@0JkgE0HpiH+<YoGE&p^)FCA]gFC@RGFCdTZ1,(I;/MK
	"50eu]_1HI6E3\W<@0JkgE0Hr[f+<VdhA7IZBBln0&4piDT+<WdeA7dkgCihiJ+<VdL+?X[TAiaaB+
	F%a>3c&5BB0@G`/S&F)@<HC.,!KIYFEDG=+EV13E,8s)AKW]bGVV.$1H[=2A7dl=Bg)kH+<Vdh061T
	83^IS(4piDT+?V;f@le^GFDPMP$6UH64Et^^9hdZ>BleB7Ed9hU+<VdL4E=tE3`BU04piDT+<VdhEb
	&cCChX5!+<VdL+EMWnG'#cR@ruF'DET1_Ec5K2@qB0p$6UH6+<VeJF@C@p3cf4OFE_/6AO0s?DKJ#c
	6"X_CDJ=3<+@Kd^BkM<pEZcc)@<*JH0d&5*1,(.$$6UH6+<VeJF@C@p3d57KDET1!0JY=60Jk@91/o
	ta3\`<L0JY461cd<E+tl+e+<VdL4>1_cAi`k%EBX^P+<Wd"G\qCo9fbgZF*)>@Gs2QX+?V;tA7dkjA
	TM@%BlJ0.Df.`G+?V;tA7dl#6q0?I4>1q?G\qD:ATV?E$9UOtE+*6lATT&+DIbmSE[!_3@GWUt!!!%
	NBgY6aF&GLp+A#!h2DI3M2D$[:0`V2%O`cf4.n!5Ka$0cf(`BT_Y]pJT)%BLX)eGh=->ADofZ_#(V<
	5ha]pU-MAZI5anS>sJ^fFSg7AOI*@81RbF5G-j8ES^Q4eMVbpRLY7(Mr&hRPc!GONeO4:r'9Y7hug9
	1CZ<_ih\f<)VWif9TY_:ECZQ7DqnBqg?8ICQ?^8CTrpt3<7UVmn>(#BV4^u(/_m>&UN,6k@@&S_'Y_
	!I=IEMI:6kXu@Qb\L#FeXo8c@Z$036PJ<`i['D`1Nn/,-;Ti^i.mAgR/,G@M1.J00Io-5aeD+<rZ=E
	&5F_XJh9M.HO?hQX!O&NE((d8Wfm#9@T\8:In0:Srf(Cea!1l^)BT3:q=0T%ujA^q:5fE/bn0Zc-Jc
	+r'#`/8%?W?oCE2@r%@hX<U8Ir[WORAXD5!\38]7h#;)rG92`I5/X_:D'53@ja#,\,Q?uZckP-+8C[
	tJ,Ye(/=EkZDT#>\fAgBZJ(c2[hE$#iF<ErZ1m,6.]r,6<%c5B[#,+94u$5u`*!m<r\bAD)+^>O'bQ
	0.cO%EUP7-,b*'8H[<,l^kusG[[]=.@LJ3T3/[h`"q2?W@g)mB,S#tZrh0;u*#l+n!T%37P+YK(Yq4
	*m:N+]Fe*LrfpL<OUg\:G[IN2(M,-/d7egknoi:fV&;-2ZPc?o:IpRH<N4U4I_@:qR;):!2U7Nc%2"
	.\<YB/mf0M_c)nL,5@Y$r(.lKfP*?0MPKtFbGk$Ag7S5,Y<S1Cc35(OuSJG350*q0YV7i0Kr/rP,-?
	747W+C-\B0C0)4>8+E^gJTBN*R+f+FMrKWhKWG,OA3cZ<;4T>>`"u#]>p]Nm)pbM/ee7C?C"/`#H2+
	o9B7<$7(bL&`nWG,P8<2s!"4O;3VX=/t#-jUuPIc<<sdb=)3QcnL\GXA*#,SKWl4=uG-(e2&@Y#Hkj
	a%;GiWiE)!L1/T,X#Q,<'P4c.SPH*;m=t.Z5$tq*<S-nAW<<lc+KDH1VHNp"HnC,kiG^dAJp=uc+$%
	mqKK,!m<%>:Z9b^%kA=BC_J#*96f^3kWC4<o^&rD9.+hQReiY6tePfr/lXiBtFWiE?U!u'ra+:,jCE
	7jQ$LVr*FqnqB3,SG)V*:4*Mo/o\l#Qb85qP*s.=F1O5_s)9[7,`*pF[9=_S8kZX>f/-XXt`XLQK@f
	4p"eG0O:A)6F^8!Ipd'>WVC9T!ZXF[Ts+q;Ea5`AIPG(gOfj9FFdPPf^F)uDDF8qudFWKfPKFgE\(k
	rV*!TVcJ$:6-\%Soen7AJ7?[ic5mCMR^$C[6KnR\lLMM%G9XLZ>nhB%nsEL1.>hChd_M#XMcGkdk(Q
	Q]oO%X]@f]!Q-U!!et-\BnFS8AchlJhLOF"*uPq6)&[G+h=0P;__;LfVUsDM8u5F6E;5Z*+M_%\qi7
	DLrSuOH[\9Cg6mCU_HCKo/J"gto]Y;*dWDj?8!oK+c8V@YR$lH3k6!c"1E=Mnm=1Mq>MDS$`T?jFak
	sD'DZ71YtP,-RkoQn2WYLht/3Lrnf(g-$kVMV#q8J:qS3'fNa#D9k/_81\R!LM!;J,F>t]XPA:bN3+
	D.#L>&K[]2XGW[[obN4=7#Tf;3WfeXX`s6OF+:+UcfAGIM3&oQ0,=Mlh+Ltb&1ej?n,Y<d/iUck-*`
	%3YrIWO3&Od;e7M'*<?smAM_!Q;Loq&:>,Y<?0H^RQ^AQdJ=i@ERJG.638YLMV&_!39Q!o+m^1PhbR
	b;/c/6[F')L5(FlZ.OL1>ZK<r3h=B'C:+t=YNn5<;(e39^+/Jm4,a[k]e%A(`s3[>Cpn"63]K)9O2i
	],C+'?]`%]Da6!tlJ`,<%_'SYR,#QbZrU3Jqb==c?e1t:"'F5f_W[rKEq;ra<X.T2SoqtKP\X&Xh7U
	+!EQc0P$t9M>j(%0_AMVpj<Mg]RGdY3R(J'afXo.O>?%lip4_F)Glu^V:lAoV4p;ZIpf,\6LYuh<mZ
	T0etKs%YpTMk7Kd!r"JS4G'8%RSqjGoB!]b-cAmjBlo,em=aoku!SAM;LsP6')]K_6)$pEIXU)f8I'
	Hf/h;.1<*hLO5E6QYSK-6JmD6&c?A0*sa$l_(IZEc"l$k/)UO2p:#T?db3IktLB_b_&$VJ./Nh$f7_
	j7Dm6*fgQDYYqImi5#NbPrOfaH+q&.Ir&r&YHQ:@pOfnh2SNmoMP@=+S&o3hl*Ok90Rbu$?<g_ti*-
	p'Kh%1FJTo3iN1q^S;/^BVfBYSG,/Z*6<NXedFeZ`@O[oU%ZL3-FK/9[$-J'eFOd`07md@6]YJZs5l
	ZsrnR[fo7n*bJ4h-^l)#[g*3>g?k8eS8)a<Iu@iqtiMt:F_W]X^9I5T?fGN0ekXI#W-FII/"?)C,ML
	*QHNi2P":ap7\`m\;isU=fC`)m[BZt;)jIMk_-h4k@U`_Q^uOsl4-;);eS6t4:7'.8fMR!BHRC0gK>
	HKHTI<s/?!COVB!YlM:_YcH3&ifS((Q*?%S'`@VQT6:+QW.iCc9IVS8"F0[sN4=P\&PEpc+<.p\.HW
	9LL':Qcmq1pkE45U6s9iF1E:.0OPhH&rB"#iH.,+Sq$:i74+4LM/5*)i8@ueOar;BHI]U0_aNa2Rce
	tF1gM`cf\"jGn8Q"Ucn=:9CCMZV.b?^]Oh.^Q^OO!t.]KFOWDm?3fYYMB^qbK<iAgAF_8]f^eM]<D$
	AAPAj'Z'6ZL%2Y()@ZT7Rgd@3UpSuL;C1^`l83b%8NgBH?qWEcPML.:"-,mC*XDm6(C\_-Z</>oieK
	+\[bik'JKr:3#o/(^YDXe/sgqUjG>G2JfolD3&<?H1GdKg-jM/_a^iuRf"`_u,Y:iI=dI+Ka62UI,*
	>DD8!"!"<E0E\68FtiGok#r5TnG*f=/<$#qhel#a:I9kK]V+LVoYjP0IV)JjcB]KnT]7MdI[R\99^?
	`3kV),f;;H*]sE-:"-J;R@3=(BtrPDQ'G=Le_YZA4\:glDRX$k,Y?k(FXn)]=J]AH?-d%:)]K`(1'i
	I3#5m!#eBH-1+:tbL&C;G%>?g%;9MJ6@NK(?R<E]RD0[f4X-_Gd\Wi@0I*fjrREH,Z*aJtG/__@7qj
	P@Flh/9^XDJ'euC2.L'ERCfU&4/o!K[%cdF?0j33).kD?$Ep:91qnnRX9"`^Tk&+c"7F&#Qt48Za8%
	moOW:JB3]3aQ?/&`l;t`HZ@C!%m(Ic?Gb/PNf&:RQ]68%;HJ7b0>:$9tM@Udm<"t$p(dS_KcF;JCO\
	!Ej+?De"JO%$<G!CKU*)7a^6+Cssb1tcWlhM*o^h0nV!eG;M/;bXlET#H!Z");uXT0NO[^PnrfeZ&o
	.[A0Oq"V74f6Vdj/QC<#<%=;0,)"D/1`ROOaei&M4ZIYG6mDGS5+g0=Y[4O-NAo_Xn*b1>N-"!8*"E
	3ulO:\H=7^Z!eh2msEQ5E;@6)q7W$3+6H2mj;78]jp3Z&:>p=a<\rV[KE?51AjGONDsFM$?fDVVJNm
	=HIrWMkN?)olq?IIrZ)lhPqbn+XHMDR\BVe'hg;M21-$bN4W#@]HGK*(SJb@2KaeE\'o5gtge7rqs=
	$."WQm/1b&=Gjtp)CWuiLPE-eM#60*hDXRGuoAKZ"C,L7FQHY$og=t@TV,H6mHTW@rAJWEd#>()05e
	uj_1CV1mI&0_lA(K,YP%S$8(Y3Wo,Uu^@[!VQ8$%bFkFFSf?p$:3SObe^L`\<>V%j'(Sc*D.>(qb.c
	k2>0*:JVKh/i\lD"9-T\S>rU`n)"oaJ`A`29+XOSkBU@b]4BpSCpn:BCmnt4;IN]D+VY9;:-hMsKQ_
	N^Sj%QPMB^6LQS)R4S""I[SJFt_cfZB5H6c&p:"+0)Eu-K9N(0SY@$%7"fZ_0fk^9,\h;(jkrD](o;
	&#q+Vk$R1Un^;_MtLe(3::t%K>A\NE@f:b\j*U4j;HQE%'-nh[YiRWrPK.1XOV&B9o$XFRl>8?66Z,
	9-)pOKTDYk4jpBao'R<$3a#)&]Nur#@=e=e*MRmrr<]hFYN8dAR!Xp`Kb8u_B_t\po(_7]P/<ri33,
	[Wf4ZkiC(e1o:Xjq/#73,\Pa=n$kf(++cd3JKlhl?\%s%._7]DVX)C*mF7KHZ#2,]T?XB=$8XToc-c
	%)Z$o't;q7@3DEb=I+C5pbCE^bKJ)8Za9>_,E&gcacIi@i[6tMcnt.hrVMa'Yq17igpr=Z01+kCPY7
	iN&HkB2KQoW9Tuk)($lGuQ_d'j!EG9-d\$m/kQ*;eYNCFl*,#&a"F\GdI_dKkjWmq[(,!eK*%U%(ZP
	C0#a^aemOrH1D8Z'^5?I5;kZ2%A%*.$'k-%C6ZA$2)oNbV]ePbV1_F6TgFW1oaRg9d?6"WbRtL4:6P
	-^9`K.mB'HCd,sI;X@SDmF7nfu^Y`DugMa0Cj4E_tg4+MJ+5cF3o?/ps^,'fm"Ihb>S^(iRZ7HXlET
	>&uBJ`om.uL&:()@=raQN0\lg'#7584I44+:IsXf\^1/ggpVh/9uJjU/Z4l02nVdo*0pk2rPun*fZ2
	rP@q3KDXO".Y%H\_S(<eqRSuA#6r*qMAdOZc-?(AIQR0X5ND%p4(<AjU<];kV^P.iQeU0F#Qt2V(8M
	\dZ4-G4X]Cp^SNVK%r-is):kg8u]3u4`JKCo56ac;MC79>?mTkk5b2r3Hf\"'X_L=W_$jP%M5QBrpU
	M@3J]"0pe5C_[f+\gFW4F&,B'#QCZ1Ga)nF)Vk7DJfqNFfVRD91qp-CE;_%W`3ql_'B=9$4[k:p'lf
	`!]cgmj"UW:+rEFg\H[]&g"G&>5Q8/nX0&P-?FXsJX-l4CF01$DBfo,JKZET#Zrkt(&ju$9j;Naa`c
	7qT_#se*A_\uD"()MX&Og+\Qf7[KToa+u`qs1dcF*AI84?As4aI8*+)]oGe^rE^T2+:iIfJ6r]<WSE
	B:aU$j6Q/HhHpUA7GO$VG<*PpmbPJ[$,]iJ?;2[X#PJpr,^hj`-Bj#!qVL)cPF:Wt,XKS[/Bb61p=d
	<t!!iY2Vdcn:`^4T^01c^i2S8t_ra+`rIS72#?d!&D/2nEn1lY[5=^,CNo:>Forh7`ol^jUlRi<7W<
	6f#0&6/)Q]P7Y(d-Bn2Thq]<C@QQHb*=K[+'qhk<)R_d+:OAHb=#Ks2%Sgm8-6h8<Ku7mgQ9&5om1Q
	3\?V[J(l%!/me(mC2I!FgUhSQCnXMRCD09tYS'BA<*Jm1e?+X-]gYHbVa5b;'Ef`L*M*u<9YY#(GrZ
	&,l0NlD;^#=>u6F`m)U.A=cHF9b5SRh<p)A'jGE"iW_NoO@s='%>hUUeAgQGm![,F)6/[OSE,%c-(d
	gW9A>"hA4i8YiB]qDsqai.?Jp)V-=o#0/FQ0Q9H(=;I1FZ=2[/2pqQV3ojH`.ag#0qtKQc/6aTh()@
	[_VPXj9dgu!k]UOAZ0#tE)rPtCWr4M[W6[FK8_^'6Gq__+>j,ZF-<iabcF`MMCDJlVN,S#UIm<0;(7
	@rZ_%`\nNL(J\/pi3E:X&LQiF*#gZdg'anHbJ9`4thefan+6Ts8M_Y^*Gjig;`o>_#;5Z0TR7ePEW-
	k?J53a\(c<p?_L:-"Ms&9KYubd\D&i7^Nf98Q=YrJDVi$OpiV.Fr5!m@Qp:rm<)IegB%SM`3qe\H=a
	or&M5mSEVVSn11DYlhr[0[cqg:T;>pIO5:J]:C%*;\TpAF`-8Wj@2\9[].J_0#RGZ7\>Bj)4iW8$^\
	jm[Z3+i0Uue9/E);af#T3c.S4&'&[Na2@L2<-*l7@U1I1E@f^$`3D<!K;ZS*h58e[)J=#VJ_;dTYn=
	HHODD@?7Q$1XhSph?p$i?A.#uan7Ni%Ao:#fPE35*J7OF<JPtHWN7&//ol026W'O%:3E;L/qik%GF@
	t4/]X+f*!F"jZFC+bj:,g#*:bg"Bio3uYY*ul9elg)GBChIBN'ZJWl!9A/\\-M$fqe'MQ%L@[HGY_,
	]f[q%DC%O(YEqZP@eLQo.Vnau[r6BMpDVn4ffq1PJLk^NRp@e4SpOaT?+Ili*A!'G3+dlhPAu;:$3X
	j91($pWee?a";>Lh5uJ%W4u-3urC=<&8!4!ICcU8Z2U8[oeNZ*CRKG"[bGg=fF=L$/8QkK`bop)Xcu
	+Gd;-0tZ90eoc0dL!t-Jq@Pu]VQ3nNf@N$CCUR7mMOL-.`c3!$-T1e2*T@3,1C98=QqdJ?WkM*KoT'
	d>80%)lBd0%cFM6KZ0FW#YU3g[68Ys#7[8p;'Y3Y=o)G4mP\oapR1cH/KmhO]kr`@^dr4g#EhX5`Re
	p*O3HaoNA?+=hs(^qHKk32"W78\XZA29R]ZJTe1hqm4EC3RkEbKU1S\(fsCks%2"4Mf`!oA;m#MuVd
	Dm!Sct:j1qSE#]r?*@1tX&;4"4FTTO%1b+b_5+`Vo+WLt>&rF%j!/.,1X7R*U\uY6+RaCOMEu_ZqZQ
	mZ!g[.1jah_A(C2+*nYhe,@qG8m?BJ9jYc"9CA$j6Xlj5URPn[RBkL5&_q+a[kA\49DP[5GBtDRQ_G
	IfK'41koF8S"l1X)PRnJ@2L=QjJ4r,<0J'!U)T(Lc7?lAdJj,LnT_.A^b1XUhI;W(fs.&_Xf=o_$<'
	q^*flr[ME%ZC^UN`[5P=G?IGrpfG`mDoi*F7+:_^n7ocCX/h@SXb3#7DqKXSA!gY*<#NiU]F+NVpV:
	udBDb5^dWNB_[7qQS4pMT*_&qe*Njq3TA"2^hIR@G(;I2UCuccWGs:R`T(QMG;=>c(-D2)_B,JjGP$
	i8(!S"Ig(Zs:r[6Y@Gt=c&HjgC1#[;f:".nPgM;4s,(uONHNlFFc_!GZ9G!`W#8G-c70]$t0^r&!AX
	LINiV:L8Lr\O19bVVOk;2Nrk?L@WU)illD/7C$X%B^#RVNc&cKBg\KU=0@JsArFGLp$$[CVs<[^HU^
	?l3L'I(YYb#?G?LSc/9TDdNGks6$#%OSPPe;'qJu?.TsKR@Kdh*I1'GKBFQeLhhg@#\@UOI"6<"%Fk
	g*a2],'P)u^3rL8;SI.NgJfoQ"8jSA(ZrDrp^dtSB)&B),#k9u4NAN?7)>0&Wg&6P7KFWF)$SN+"d@
	7Vd<U9h@%XR7NaU+V$?R\l@nAV?/slQ$o`CW.0ug[/s!=Y(>bQ:"D0cSH!)W[\]eCFk:EUK$MhDH]F
	&1lYjLk;f^(h`@=R\fc%2j1>e/("Z.7j=,Q6\Xlca26EaF5Bo2l%bKVW!7k[23WVd)][^aCT=9nt/^
	[$bP]Sr<bAoQ^NlU2I4`r!*LA9ZWs4^=spF!]M^V9J_p1$Ng[^W_G<V:T!K7ee.P1f<u6Z=W@aY^?,
	%fkmQW2f.UHCG5X5C?T2g=P$iV?NX#[FF[C%b"r;\D;&$Z'`l#FmDS)X]D-51K5R^HpE\@ZB"8Zr5H
	+M/Sd,3>)8mYeQH&n!CB+<2CO.,b.jt,C/?^6FEFnS6p!K?V86CVa86:?Q:a%4+ukTMKME)3;Q43c]
	U!UJiJ;[()rsK:8ClkU/&aB)kOE#dF]hn!)^.)(ZB/iR]qBa:&#GK0a2_E;G$7:&WVh&,cS,=KjY)&
	Z95El40CpD5,<ha'`IAB@PTU7'k?Bd4$\\YHbShKBD1'e7U&9E>cR1fH3U0I9'E@)WhKej$:"($[TV
	n]c/O<Q+IDbMhI]@huBV\[LQ<*P+(Sqp/Y(MUJIJe%D9MlL'dqJ9QUKB&g,7)J%?&/e"3::B]\lrtG
	\"QX>m<1k(U"O'A!P63,P;#"S90/9/ZCe.mWbUhBk8K(`q7ME956/aoIN1"!s![8Tm<BA&&-@@(DRU
	,d%U+<]84WM8o&'5ObeO,Fid\/BC2K(n&-u)HV.j+kiVu6<!"B,:#6u2[3H-'g7"=ftXZo):^eIc(g
	=PP/\`)EMK0bYGiRX>Adm;Mt78^q\E9g22*P+rmVMN<Q8FON(\%do<Kf5D0ibPAgWC:1+;Q4>u#'Zo
	5rPGS5a8u&j/AJmSODH1666NI2.\5Lr[ir?-2%7B]R*fue-b9qM%(:ABd;lBu*/CqIC=aX8*"PXSdq
	%rk\TYWCKJ9rF(iU"b]n^GB/7Lh3+9(=k5P[tPkEjcBCT>dF0b:,S&OP#_p"7=egd<hke*$<[:AQim
	m!nd@VP5pB.k>"VqJ"pKNuND3OB`5'Yp>C+>-1*rKtb:80'2N[3:72:j-SaehOV$8<ck0,a*D?3#fg
	pfQ<BM:p;\`-6L&TtILMTXU+Vgp!/_oS4C;g^<C#mm$FX.oh<An'k-B]<6HgjaY,uapB;1]8(Q5hY@
	47W4d`.#MR+F1Z6GX;;*NpY1/KVg6:aji\H`79@s'tYMh#?I'J,4Zpl.sS>A%r.8;,DG4<i[5[=b--
	VW6NjhCFFOVr8"t4J,825X4>@:ml!"ni%caAj)C79q-<S`?]2IVS\VHI),KAup*-?07.#5G$LA?Ea'
	0]k\Qm-.bV4uX/Cle"7ankiIdUVQ\m33ZXs'-:/sk>[-:nhr,]l6%BlX['E5p#$O_a::H;1N41=n3K
	_(Eph2uol$-C2i@E31E$WN)u$l[_=ZF=MJLjMLAfs5*F`]`*u+qi=WslA/^Ua)S%-b&NqlSFm+b+GB
	ZAImY#AHL\kZa;ek%_bfdXD3d`u<u7O)C$/FB1i8[.oN=>Jb$D;IcJ.s:LhCEVXm&@?/<V9p,[hPP=
	Npmf)/V99ZW_"_daOAsY`g?l\;%^U5nGZ8M^1l-8(%-;Vp@5Q1h\taAm\b@f6E!>=OYT)/_?(0FjG`
	Z0T<e$LS@nf[GsIRFOr>Xj-9E\+WPoYkNeK7ipp1MqC<Y"5&@Y1+[-')jj=/sH"tebRgGhUm$Lg$E#
	\kc^0W:S)Ti,,L!Z`o_c`Wh2l,K(Eard81c"LC1[J7J9hUr)9ERWZ*r>@H4WqqRYkAtWas2R`F]ZI7
	%)\U[<u7C*aKfW=ML8Q3__I[(Z7g#I2Um^2/4p;\L+>kt<@!<-+c<RYCFr\OYgBL(0A%HUEoCde^@p
	-ehYOeOr7l1,p[A]$NC(P/UPFFZb#$`ji=D0DSSip8B0K\'eXCc#(kt>#dgj("H=q!VY1^on-\H6!Z
	hE`8Z'q!,%@o7Ng]tF3*sT*7!C7VbSk)knrQ-`%F(/6=V1@Sc4B9)LG_^OS>VepGC9Od=ld+,ESC/N
	'e3^=3Y!r9S#WeNBE31E$$%I1KAA^)0@7W/L5=tgqb$V7Jb2<->An?%u<'W>]a&Q,&>1u#=RgF,F`0
	o^0AQ`Q2L3fP2mWG*e!r<KP6rDIWl6D$;A<9:3SUHAYc-"]A6GY%"X(b'4;A[8h[l4d5FQ\PR!K\-F
	2mJjfMZf$dq*Zm,CD_dL*:ZdQNEY(52HZC7'%]n;_q@UZ'#i/,G!(7>%7Ab:2BU@VPa'UG=oC"'nBi
	?EO!)rElIZY'#_M)]1I8TrQMj13+UANA)itNLE<tQG1'M>,*61/*3+s$@S$K/5+/nr7T:eUf$-$.Y6
	+Uu$!h-5D&-TRnG<X3N__m'#.e_$Vm.2iLm;tN3C9hui`O4_b.i@]NIkr)'E>2d"Q5?gFrVo*0N)n`
	(qKlp[&F^CH)]Uf*":+Z/d\1!"#44o4d*mTLkZj]hSh[s1+3Mi]KmUIno3&!eVI/^i">lT4-jX;4[F
	&'/CBu)_.aa`mL\"pPJ,B7Ma>L'r4#b#1Xi[?H`O4`+A3C--q/uYe&N7LE)fNJ'J)e]DPsch$^NL9,
	BOEP84\5MVc+^G7'!^YGRmM80D,%SSfCKM6GI$M&V/q:]`'>0>DH<g=eX/G,1;.]N&c-`&"2h>b('O
	2i&'dM)VdLdnRas;TA#\;Y.WA+VV9Jms95u6^&rG^fE=LT;h>VAVWff8Ek?Mdo7D_s2nG7*^+>>!m6
	C6*]T*kPHde(u_q"WfY#WS#':%R""16NnSP14WfR37NP&W@jDl2jM]R)e6$pt6bmhMNGrU2EEN_uHV
	@6T+W_?i%:na8FP$Y71sO*f_Ltqf\3hJRR2DSQ^<1D]8PJ.$Gd0/h7W:e+su.(^M8jAm)&V?P_7TM%
	j!iY#gl/ANDR%D6@='ltHL!8s1K*4'kA0m<4X<$%fCoU<1f=(m3S:p7'GgY>/@D*?>SIBe9D::N7A0
	ISrZE#\Or!8N%7#O[tRO\'`HZ<^)X:KL-+OJU:[AHJ?!*mIbea?AboOV\]/"(ojZ=fF9`T$XZ&H[C<
	Wsj`34`US90RC!s81liM_<__@f5lFMF(JVbKTc0$=Q.i$ME['#HK\Rc0sh4P?@TBKj+9rs5+:C;9]5
	12MJqW;L2%<FK?8`JPgZmkKl.$<_`CG"2,2OiN?=&meNMl9-u^^.F6^56b,[-sn/cS&$g:*/5u%.6,
	9S^(a6r7gU^MP?+B`&TtTq#0#IOX<@BZa8%m[%TJ+[[<d;98XMg!eFT/#F0a5+bR__9tnPYhTpkRd,
	hg]K7eeH[(_.!9j*dWdHIj(,o@&f;CT,!d^U2^1hpGVXi$6C>p>)ZmgpWfMP?t^<d"7%AZd0il*=Xj
	U`aMGe!&7EMl4TQbj!%OU3l(WVJ2>1A_%3k[:D]e*"HYu"+/*\*k=-rN5uIp`gt2A.hbtsM*s$74PB
	j7<L)`mdkolupTa,LTN`I:pb5G6n="OmpNtbAKDqW1l;W^S:E'XdAnN(Vc-USREk%S#8@JohAnMqbc
	h#u)m>GD?Bq!Ss._!A#nA($;)h!-_E)#_UIo<>t1egm_V3R(3oc8bH\9a(_r?-\XK;-Gih*HFYbJf6
	l)eS#dU'U/!rI2L-.e5F,-<,]Ao*S'Q_`bnoG>=WC\AP"e4B0q\lr[9,4W[8+ergd4om1D])U#FdWc
	k?r-B@TWh"$d9^3"Yro(;QrZeK's!.24iR_`D$fWYEG;1?9p<i\Ob_8@2t:(Y+Rq4%"8^<re%N>iu.
	=^5dQEC%on__@P9Bl+M[F%k3Y;Q0e1X?%Hp!"_J^<9<)25+G9i?!4behr!>Ng49&!WiEL:I//.RQci
	qW7]C.mpdkGc((Wu)5u^g8qA0Bc5h^$h+@%h6U!t5r:#"g?f8#)!>.Ym_e982eSXg8B9\oXA>9=N(@
	eo$SoW=Iu2&nQ$HYkUqV+i)YEIj]&9=T.-F)W*(P=,rbY"95f:(T:WDb4mGim'3+_>AqUf5<2n0"-5
	2QsgiAlI<2\_%-W/WC%omL-P0'9a/9+`GT33BJ(+qdm";t1CjF::"+(X%.ueGT8k%MBJaJ1fq=L1k6
	(.-WLK0Tb2Y^7I.-gu`S@^;2f=S7Njd#A?.ZF9XKLl<^]2qm%N'rZ3$&Qb$%cPf[](C3LtNf$CR_3u
	7Rn\_;B61`8>T4qY.IpkX6fIrF%r`n>Mn?7eo-%rHbj""HL9g7c/JDth=5D/9ma]r<cP7<5X&/)ehe
	tBahW4>ql(ONNpLu>4.d%2rke5[e8EE@ZU"uGejmenZZ,dc*"KIKqfk5@\(`3#g=t@tpk!H)n,;GLS
	H$lY%',:0n%SD=aGh,JabG4E2&7@B^YO6lkj@bkp7'Ffe#Hj04Ic2:<0+ZS;W?+?g9TCJR\mK16WL0
	e78[#a*?6&81$T[8`Ls[Va)MOlZ2+>r+Q0NYCGQQKc7mk9QQ+Ws[9=%)+S>F&2_>p%5.\Zi?.2a@Vm
	MC@3Bi9qii`X>nD7K(k=J=;2M>N-n*WY87]bTQ9CKs\E'R`63h:0%jiA$i)UUY/S_ru9c>QtdNA(5)
	(G;mr^2W%_c";STmO\nm#XZ(,(.BqrNODqHen27>+I,QqR395?V;PFhJlu:%Pb+CerT$bG5C^D@LS?
	1hWDo$l3=lHu=T12m$9=YP__C@U]/tS;)7A:(lMmaBL[pE42r8rBc2%0\=Sd^uL%uX]==ceehsg,SM
	b!1-TKtA"#_K:?N6%<V8nj%M^Ge+K`l8o0#pGPZkM]<5,*B+6E1,fm'<>U9@5e(dFMaI@gu%q_b,(A
	)LMN^3-O5:TAu2>;>'.;+jBAV=aQVh9Pj7Y?;hrP<I9uaIDu+qShd-5*?i7NC]QEBNpk-r8a>?WDXH
	";++nYEk;o:9l6%C;J/O$tc^]6f#n>7-'J@+!armedSI/1rd\_F.9>PNnZ=M0'o=`)R:eQ#ULBprp4
	OCJ<&=V4!2>ctsGCJ:$ss44d\9OAcEI-Zd$VP^5`RTVHcL!lbKP1_MtLu134Nh2i6(2*Ad4CG^117;
	_f3LijXY>;XQh0rNUW'9,*ltjQmU?aj^:iNsMLRWU6#ckJ49'Xu7Droj!%AL[#YMFI=\OMb3ck+f4j
	.fWtW9V&_@;^)46E[_r,(2X^I@6ZXY;L-N?T2<kD=;ECg>u]=#G:kub=g%;E4BdU3#iF,f:UCP>N$M
	qe)bUQQK:X_1,(E/]#t's0SE7`SShJ)e)Bi8J+Zm^Rl=3b94*WI\,PL(SH%C-)BLbD(rij4FR0Ek_e
	f^#/4bqW`e5nd[bCg#?t]b6@s?ea&afslV_>gTg)JE#'cn".nY)X"OsEInKPn'8%_06e/rF,K>TOb#
	Bg&q^+Ge.]8Wk?ESj/quX"+h9^i?73N'[W?ol<'Hd$8CuRMd$C</TjaR\kjpL+M[F"6c7j&hW2ekL8
	<`O4ZtjgqL,%\Fdpac0fNfnbA\(G4+Qn7I`HD>8-W"eoIlmBdnoX*#um\lX&J74C5r&H1gdDmf2s0^
	[gjniTL4!b0!=-(-Ckn5sI8fj'(^)#X+GP<<\;XonT4hC1LiXlH:9SMcNfbnFJlZF6f.FJ]H%0oONF
	HQP[C*Kk=p]oHlIj;q!Q_a$9*!%2"Rhj72mN3ZqoL`akM)GINhbb2U4rS[\&]:JW(Xjp;.l_o'AONc
	/cO22fW/6.>K&qRW],G.To"8@5cBKL-jG31*=7RL,]:b"ef(hVMS4K7aXITnf&[-_bkMb*?QVX=VQ/
	Gk"+#<Iu@I'r2Rd_DOb][f9sOpc$/u[J0(?hYrYq>+njiW)37sB1MFKQ;j:8s#Q@3=BMetCMhk=(`4
	+Bh;,26X3[A#$O26c<;d5u^q;]-ZbE&;SlhSQRL4hf7iW\+_r9J/hJ>*^^:kYJ7ZTOsXB)BKH[CXE\
	D?T3iSet3BO>+R4<?-kMofKb&r@TW=Y.`CP=`X[YPnF7Y:VE^s6,NJ;@.8SV&FN(AU_Ai9>@iSldk.
	XA2BF@j#G,<Ful1N-8trd!C781',(pG8uL.ee'l@KKSFBc27`LXBVTD\B.o:*NZN2*!lsi1WMr4X!/
	>$j'%7\PRARO*\S2n8,Y@V!o?I3+B#&CKO[p1E76qL5l1E3Dbi)(Gh;)'FKepsa)<RgB(IEpo`sG!:
	Z&8)pS!EsOf<8LtQH*L(Ahod-hL=Y6T5X8dU0X:>CFZO=[VOY+DR=n8lIq:9f1^d@>-!?7D9Qn,[op
	^>22mMej^GR.]X#rpY*!)il;q3f.7"Mo^H\@:bH_Q0#m]>=#Pene9tq'cp)EuiNAl>NnFQ[)mN:=i\
	MZ)riUcl*X]Z2D"q--:,Y=H%2*6uaH2aS"5*@1JVWjA2PnS7)L&6.L63.3Ld+q,fPXgcU$%frHbjrp
	!m*GS<IcCWFRb5VL.m6cAN`XQ!ci<@X#/C)tCTkK:1,@;E:dPl8%mTtgP,*-/I-K(U[T]fA%YqqF8J
	3#XL-hTO2m,cOUuFUXK'5A/n=Y;CYk(6F04)9sXhg$H.\"[p(^>X<I6O:TlYZFoF?7+?Yp#"!fB^+t
	q=4j?iuJVb/BbG)NT^+F0^*\lpJ4a@QWI6iiIejfqmJX'.uBuO#]>fAjfS*2ni37Kjn\]4jMd#nTl>
	M!?flC>p%<H1Q<c,peZ2b'pTah?EfEF+MuTI*]##tcq\aBJm'ZL=0/!:'Ec$(^0fr%g36cG<%(!>q&
	OL0;>5]kA=+\7=f<07QCMg"@%)\T1VkPTiC/kfJ^YaNk&!'>D^:X,F-ZZ@,.pm:Gjl\8ogtPofSt)>
	Cq0XdZC`k!Fh/k]KLK>rlBQ\I*>V:(;2`s3MiPL;4SSpJ-%TQN0eLJbJqh^:D>$kW4Vpg5?KYuc6Pn
	gLS)]T_%]_NoG@m@ul317ru#U(/mY5u[`6kG/"9o:\gT'H7OmP"R6lPG"Yl,S:$IW7(Z$oY4^Bpqhi
	4tW,E%)WJ`%PB@dG,^)q5nJT4+IgVBL>(TMJn7oI9j))hSbI;dD<&LOKo:j*H[C)[%S&%m#fgqWcNe
	!]<i\AH4brWL(l#SJpF_-dY?eb`ZLa&:X$olr=Z.%Re_c`:#k,P.^7m%;Ed;A5.?R024W\Mub#6_i[
	Aka7K-7r5W6^>41(_kBK2\!>\_Mg0`A93PFQq6+f[pf$S*0SX557BCeOl[DZEaNP&A*Uj\],kc`b7^
	sMU.06mh\HF,F9/u9htW-F3PX4idj@X@2P#GOsKr6!s]=V&OIg2\nNXKYu.Y&p$OI"Eh-Q@R[]r<*n
	Tn)USO%u4a?Ih;Q71^mY_l77O!?:NAnaerp0I]29^e.%pg;!+-/f6-[;PhW]k_!,b7]B>crd7L&H*K
	9lk/gD1-i'kBX\qrl,mq1eW73"gT[DHM".4.(ENbM\e&?o^j8Z[^NW94aRB%'5l^A%3ZmF@2L!)PmR
	]gku2#3;pFKJN\beu;&&TuM0PO/*tJekp\+<8_r7Aoa!68#a^d/pjE]ToZ[j6)]ks(]k<Jt@@oZN!7
	RlP&k4W`%Gs;8mg8@P78IQo"L5*?Img4S]%)[Jbd79oZp@\)a\8YlkD'b)!rVQ>ieS;3eS!f[I?=*%
	F8[D5A4F6Wj;%sRpU+!o5mbPKV-kodL*?6'3HM)#99+h"]cns!H.N6>FP%:RB1/5Y?$k1T>bb-B.iJ
	@1!`8bjPhg]_O@M#>5WR*Ib0emZ'>IZ7)_)4fNSn=SVP.?B1o)C5lb,(9<ZA'JE<Aek=RAPOsNf[!I
	#"A>SU!qt(gF<iR0M.GaJ,ardG?.aFp.'':7=k=?S]%aD=h]7W/kseIQYU/DV&o'Xf[tHFoI4UYn%A
	8;f[s;]C./\e]6E^a*#Ak`<`W7'Q@8PtTqWs#9',3[RkliVNLRC#V!C&EYb=>f8Wn%2^VpX%"OT'.=
	=h2Xc8u[Q'4'.LnB>Z`CG0#,^i=aBEcQJP])JY*Bo8WU;:Mj7p!MFJh0nkT#sr<=BP;jD*Z/[]k4cU
	a4bdA^L5A;BDnYgELRtJZRN!1bX/i9\NfT0Zc-4De]6CG`/dm8=\(gB,\^N+2At#U"l``RHoH_qFBN
	ia[LOg$5rqF07X]Al<]_0i[D&llO(h[*#l!/#Md0<D]j`EQB&:e1+T?gT"kej^ln81K^:^E]@2!ss^
	*=g-Pk%6B:<)lq]UoRS#NfT6FI^h[@PhYRXPtHWNeAK]Sa"8hnS3(qtfsV=H$!r5,j`7n:%YqsQUj2
	&'^o><,g7#%AJfn=/Eb/]hiY798iA_ka"E[p^(^?Emf*bpD3,Z4[4)a'X%TKQ2@i.[4P[H2DWp53A&
	&?)']A;)B?Z5&G@4QpPKG]H;Bf_U(d52;Fgt/`:n*0'siadlk8Vb1ac**[5-WD.&7;>+"*^"ZI/uQV
	G\9@7CQVM6)j,@b,SSKTMOVigu]j@no>:^OMNiU]FE"6i=ELO`*Er#Z_35I-]Mks<#PtFW2<%Je<?G
	$Ob<p-gI3"Gc7E)BDoUeE*k5<loi[+#O,atSe)e^a?cF[]/b`T#=odY[!G-NH`$O[qm+AX]KiekRJ'
	Z1&0m+GatrCml\a8Y5*(2`GH>#.8LKp?a*s=1]+LK0`?KBpr(&Q!kM8f3a%+F\$'-I/F).,jFaQ]@C
	X24LNdIHgLs6(7jBBCF=Xd>W\X:`,XSuot"B\L1*'%.i+%$S'C!`=@G4ia.YNo*0j-Z"9i6f?bUper
	-8ZG2?Z=ug=i98=qnqgc^rhWpA+O[<pCO1"b[H-EM%^Vl/(8<Pg<N:>+2"Qm8>C"AfsZ#(^t/J`c57
	[;M"=_MoRTT!),Mm0L2>QZ<T#K]jC/e)`K<I/#I$HbN2\QV5e8gRsk"sg%b@;o^m7gV2,W^Eh;aU]-
	djcbHam@oB+;e>q2=c"O=O.(+eY&#ur8EfhnX(mq5d(&;3r\"G):CK0-1XQcm41hS$V9A26/%1pjp.
	gg]ZJ<j9'Bp\=:=Ek0Ec4RCDaGb!eRQ=U/L5j)HI*Eu`PT7$29\=C.5_r<ahr<T&E$8*?ODjAN-i&b
	,M9P%A`hVO!%kF[.mM("8QoCGI_X0i<p@7V`/E(e5.9dkco-kn?D45<Ke`"3(1iPUH*gY6[r*KS7E?
	b[(.-7g%L/Y$g_F]cqsP8r0o!5gM:T0Fuo8qpb6*0/._ahGL'")2Of!E''Of@Q^X(WUM(_ZYTX^Y`t
	=gW.@TLPCPPFeraTR\heJZa$ctcf]FVQR']3E%EXq#6KI9NK(Llj5]/r`2/ZS.[<V3&4r%t?LRIUo"
	`&V95KiOI<J[QR7op]0,BJ*O,o&_Th\8cT'(1*gr@?g/Bb=MQV&PDa!6_&bN0(d_\3[sd:fnNPq%Z#
	Thp7iN3Y!-Ou,!PFQ((?78]5i,ZJ<17\aq8gS#bcHleFm'h&];@<-C/EH0AmI;r_dEUAR=]#6H6<$[
	3,Z7Mp&l-7IIW3G?+]48+unF1D/]=O^b]=Y\c>RkoWNoP'uT9tu_+!6gpBle?Z@2OS=J9<(],=c)a3
	"/ee1RUotSV@+[a)OqT"g1E."JMhF#QhR>";r'p#m:<H9r'10.[A0<E\J*Oo*D%HM2?<:c5i%5I%!;
	+d58PI\XC]S%E!]7Vum,(!s8XB\_[Uc8u7\CV;OlFJqL=1$smWpPM+^Fdrh3(#bTGpf@Mr<1t4.S)2
	?htfmu]/Y3Ro=qg?,g\[b*P!F[O6@CEVQi'oLk;*LT_"?Ep0aoOf.e`uSjDP^5C"@:KnR\lk-kj7[&
	aN2Fq1&2dJN,+1Pkpk;d;3;^NhdY8L10H.Dl_RC0_-hbBIl&'d3h:4"<H%a@-//jG0FS&L@lBgY(RT
	fW(]F=2F)uF$rVc6.Gb/8L?dIhSodfRJ\t5K7QcisHfq:4Nrp`_3K^fq7ali9"Q,,UX86EQEiPL/G>
	S,)W$Ng"krI06GIt'!bF#okR07r6Z@6pRKqdDd:H[Fr=kPP/R?`rpD(f9.6@q,TJk[PH!;,f(^lg*k
	himRL$MLUpD&$OJ:$sAl(KgI,MJOQR(\gWF=5tk'M:JXbRX]DA9SkLJFRYL?(P)R@h]tM*eZpJ9(r@
	bK@4sXlR1)XcF"?)'E(_[PrkKfbbO=G&`"pb90KSe^Y3!a!"fjOX+-SeE&rmbO^/f'AuCmJYcrIR^<
	:<q)-[*h+sP]=^Ne)Zs/.M;a-S]PF5q7IgUn.H%ZE&\gILSN_a%EJWLWN$:C?BFA$bHfU]m-3aP)]O
	^Uis,tn__C!NkAiSSkOHju3?4-n!oq0'K2ag5QD%cFU*u0M5o$T.Z*D$W^1M(joad/-jTIjL.?4sMs
	$UuqT]?mp@ejGa<IiX%`F:Qm:0/n[YkTm\UF6J&Xn,scc+jX>V$Y.I-kmR<)B'Y<%Y[4SBE7d5dciS
	B!D$n1fjK2(`-'=9(^;Mt78Zb-1eeW9aan:g3]burSj!EX.[AHHFK&4@6#[f.>*/[r$%cP%7N`6Z*#
	o*nkuHKdp>_*IVo(BM<?qPf"HfIMSK0C+5ir/iec-;9`%W_^%ZcX1%Lil`<JMg=#?J`BIiq+bnI\)Q
	C""m@J\]@J#gg)H!sIQFIR/M.7na'9V8,#-)B&qc!j=B(p&hTIp\+=/kqmbto!99Co`\FsZ``W"/8L
	\D;:NCs5U)?Sl014^K8*]BH[C+$;h>R\-YCre-]AS)(R192ArF^iM*qC`LUH,7lFP9Hm!_7Y?QBmbF
	JZ";Z7ObtpAY'jdF$@uhVM#%H`)fYY'$4L>u?`7\eoCs!i#%=U;0f[@2Nej%Yk-]l[lqq'P_J>&ck1
	Y,Y8ur62n)68ZJeBH&lAN5nK-9n+D!8I)q7`p[kmH#g9N8Z2bQSk.,uj"$m!>!)3#_:-UfCW!o9\)2
	;R&((Ng,*>5:cJfp_uVU+E5#(qIl_!,sLI^BIE$reie(e0Z6:pp2@\(PRMQcn^f_Y>aMlNAT,TO]2j
	Rc?A0I]t`gSd%mLG.U$O^YeL#\L]if5#%PeXa===U5D'`HGBU]J72R926\cEY,gRb:-jtC#B`Uh$/f
	Rc&*gjfiu^A1O5`'@/C(b;+\kg%T&C)j35^dsr]f1)cZ"UpSA?]Q5F;F5;!!QgB`J,5!(fUS7'8jaJ
	c
	ASCII85End
End






Function SpHrView_RecordedSweepWindow()
	wave RecordedSweep = $SpHrView_df_transfer+"RecordedSweep"
	variable top = 320
	KillWindow/Z RecordedSweepWindow
	Display/K=1/W=(300,top,300+700,top+200) as "Recorded Sweep"
	DoWindow/C RecordedSweepWindow
	
	ControlBar 37
	String WinNameStr = "RecordedSweepWindow"
	NewPanel/HOST=$WinNameStr/FG=(FL,FT,GR,GT)/HOST=# 
	RenameWindow #,PTop
	
	Button b_SaveRecordedFile,pos={50.00,7.5},size={80.00,21.00},fSize=10,title="Save to file"
	Button b_SaveRecordedFile,proc=SpHrView_TransferFunc_ButtonProc,help={"Save impulse to Igor binary wave"}
	
	Button b_SaveRecordedWave,pos={150.00,7.5},size={190.00,21.00},fSize=10,title="Copy recordings to datafolder"
	Button b_SaveRecordedWave,proc=SpHrView_TransferFunc_ButtonProc,help={"Copy recordings wave to root:SpHrView:Transfer:ImpulseWaves datafolder"}
	
	SetActiveSubwindow $WinNameStr //##
	ModifyGraph/W=$WinNameStr cbRGB=(65534,65534,65534)
	
	if(dimsize(RecordedSweep,1)>=2)
		AppendToGraph RecordedSweep[][0] /TN=left
		AppendToGraph RecordedSweep[][1] /TN=right
		ModifyGraph rgb(left)=(0,0,65535)
	else
		AppendToGraph RecordedSweep
		ModifyGraph rgb=(0,0,0)
	endif
	ModifyGraph margin(left)=50,margin(bottom)=36,width=720,height=144
	ModifyGraph lSize=0.5
	ModifyGraph fSize=10,lblMargin=5,standoff=0
	ModifyGraph axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=4,btThick=0.5,stLen=2,stThick=0.5
	Label left "amplitude (\\U)"
	Label bottom "time (\\U)"
	SetAxis/A=2/E=2 left
	
	DoUpdate/W=RecordedSweepWindow // for Igor 9
	SetWindow RecordedSweepWindow, hook(TransferFuncHook)=SpHrView_TransferFunc_WindowHook
	SetWindow RecordedSweepWindow note="PACKAGE:SpHrView;GROUP:TransferFunc"
End
Function SpHrView_SweepWindow()
	wave BufferedSweep = $SpHrView_df_transfer+"BufferedSweep"
	variable top = 60
	KillWindow/Z SweepWindow
	Display/K=1/W=(300,top,300+700,top+200) BufferedSweep as "Sweep"
	DoWindow/C SweepWindow
	ModifyGraph margin(left)=50,margin(bottom)=36,width=720,height=144
	ModifyGraph lSize=0.5,rgb=(0,0,0)
	ModifyGraph fSize=10,lblMargin=5,standoff=0
	ModifyGraph axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=4,btThick=0.5,stLen=2,stThick=0.5
	Label left "amplitude (\\U)"
	Label bottom "time (\\U)"
	SetAxis/A=2/E=2 left
	DoUpdate/W=SweepWindow // for Igor 9
	SetWindow SweepWindow, hook(TransferFuncHook)=SpHrView_TransferFunc_WindowHook
	SetWindow SweepWindow note="PACKAGE:SpHrView;GROUP:TransferFunc"
End
Function SpHrView_SweepInverseFilterWindow()
	wave SweepInverseFilter = $SpHrView_df_transfer+"SweepInverseFilter"
	variable top = 60
	KillWindow/Z SweepInverseFilterWindow
	Display/K=1/W=(300,top,300+700,top+200) SweepInverseFilter as "Sweep Inverse Filter"
	DoWindow/C SweepInverseFilterWindow
	ModifyGraph margin(left)=50,margin(bottom)=36,width=720,height=144
	ModifyGraph lSize=0.5,rgb=(0,0,0)
	ModifyGraph fSize=10,lblMargin=5,standoff=0
	ModifyGraph axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=4,btThick=0.5,stLen=2,stThick=0.5
	Label left "amplitude (\\U)"
	Label bottom "time (\\U)"
	SetAxis/A=2/E=2 left
	DoUpdate/W=SweepInverseFilterWindow // for Igor 9
	SetWindow SweepInverseFilterWindow, hook(TransferFuncHook)=SpHrView_TransferFunc_WindowHook
	SetWindow SweepInverseFilterWindow note="PACKAGE:SpHrView;GROUP:TransferFunc"
End

Function SpHrView_SweepImpulsesWindow()
	wave Impulses = $SpHrView_df_transfer+"Impulses"
	variable top = 60
	KillWindow/Z SweepImpulsesWindow
	Display/K=1/W=(300,top,300+700,top+200) Impulses as "Sweep Impulses"
	DoWindow/C SweepImpulsesWindow
	ControlBar 37
	String WinNameStr = "SweepImpulsesWindow"
	NewPanel/HOST=$WinNameStr/FG=(FL,FT,GR,GT)/HOST=# 
	RenameWindow #,PTop
	
	Button b_SaveImpulseFile,pos={50.00,7.5},size={80.00,21.00},fSize=10,title="Save to file"
	Button b_SaveImpulseFile,proc=SpHrView_TransferFunc_ButtonProc,help={"Save impulse to Igor binary wave"}
	
	Button b_SaveImpulseWave,pos={150.00,7.5},size={190.00,21.00},fSize=10,title="Copy to ImpulseWaves datafolder"
	Button b_SaveImpulseWave,proc=SpHrView_TransferFunc_ButtonProc,help={"Copy impulse to root:SpHrView:Transfer:ImpulseWaves datafolder"}
	
	SetActiveSubwindow $WinNameStr //##
	ModifyGraph/W=$WinNameStr cbRGB=(65534,65534,65534)
	
	if(dimsize(Impulses,1)>=2)
		AppendToGraph Impulses[][0] /TN=left
		AppendToGraph Impulses[][1] /TN=right
		ModifyGraph rgb(left)=(0,0,65535)
	else
		AppendToGraph Impulses
		ModifyGraph rgb=(0,0,0)
	endif
	ModifyGraph margin(left)=50,margin(bottom)=36,width=720,height=144
	ModifyGraph lSize=0.5
	ModifyGraph fSize=10,lblMargin=5,standoff=0
	ModifyGraph axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=4,btThick=0.5,stLen=2,stThick=0.5
	Label left "amplitude (\\U)"
	Label bottom "time (\\U)"
	SetAxis/A=2/E=2 left
	DoUpdate/W=SweepImpulsesWindow // for Igor 9
	SetWindow SweepImpulsesWindow, hook(TransferFuncHook)=SpHrView_TransferFunc_WindowHook
	SetWindow SweepImpulsesWindow note="PACKAGE:SpHrView;GROUP:TransferFunc"
End
Function SpHrView_SweepImpulseSpectraWindow()
	wave SweepImpulseSpectra = $SpHrView_df_transfer+"SweepImpulseSpectra"
	variable top = 320
	KillWindow/Z SweepImpulseSpectraWindow
	Display/K=1/W=(300,top,300+700,top+200) as "Sweep Impulse Spectra"
	DoWindow/C SweepImpulseSpectraWindow
	if(dimsize(SweepImpulseSpectra,1)>=2)
		AppendToGraph SweepImpulseSpectra[][0] /TN=left
		AppendToGraph SweepImpulseSpectra[][1] /TN=right
		ModifyGraph rgb(left)=(0,0,65535)
	else
		AppendToGraph SweepImpulseSpectra
		ModifyGraph rgb=(0,0,0)
	endif
	ModifyGraph margin(left)=50,margin(bottom)=36,width=720,height=144
	ModifyGraph lSize=0.75
	ModifyGraph fSize=10,lblMargin=5,standoff=0
	ModifyGraph axThick=0.5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=4,btThick=0.5,stLen=2,stThick=0.5
	Label left "amplitude (\\U)"
	Label bottom "frequency (\\U)"
	ModifyGraph log(bottom)=1
	SetAxis/A=2 left
	NVAR StartFreq = $SpHrView_df_transfer_globals+"StartFreq"
	NVAR EndFreq = $SpHrView_df_transfer_globals+"EndFreq"
	SetAxis bottom min(StartFreq,EndFreq), max(StartFreq,EndFreq)
	DoUpdate/W=SweepImpulseSpectraWindow // for Igor 9
	SetWindow SweepImpulseSpectraWindow, hook(TransferFuncHook)=SpHrView_TransferFunc_WindowHook
	SetWindow SweepImpulseSpectraWindow note="PACKAGE:SpHrView;GROUP:TransferFunc"
End

