﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// SpHrView_BinauralSignal

// light red (65535,49151,49151,32768)
// light green (2,39321,1,13107)

StrConstant SpHrView_df_signals = "root:SpHrView:Signals:"
StrConstant SpHrView_df_signalGlobals = "root:SpHrView:Signals:Globals:"
Constant kFreqAxisLow = 20 		// default low frequency for graphs
//Constant kFreqAxisHigh = 10000	// default high frequency for graphs

// =================================================================================================
// SpHrView_SignalMakeSound(), Generate signal 
// Build string for calling the SpHrView_SignalBinaural operation using the Execute operation
// =================================================================================================
function SpHrView_SignalMakeSound(playSignal)
	variable playSignal
	DFREF dfSav = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_signals
	
	String functionStr// = "SpHr_SignalBinaural"
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	variable InterpVal
	String DF
	
	NVAR makeAmbient = $SpHrView_df_signalGlobals+"makeAmbient"
	NVAR makeNoise1 = $SpHrView_df_signalGlobals+"makeNoise1"
	NVAR makeNoise2 = $SpHrView_df_signalGlobals+"makeNoise2"
	NVAR same12NoiseSeeds = $SpHrView_df_signalGlobals+"same12NoiseSeeds"
	NVAR makeTones = $SpHrView_df_signalGlobals+"makeTones"
	variable NumSignals = makeAmbient + makeNoise1 + makeNoise2 + makeTones
	
	// for /NS flag, Mersenne Twister pseudorandom number generator seeds
	variable noiseSeed0= 0, noiseSeed1= 0, noiseSeed2= 0
	
	// Ambient - index 0
	// Noise1 - index 1
	// Noise2 - index 2
	// Tones - index 3
	
	SVAR WavesName = $SpHrView_df_signalGlobals+"WavesName"
	NVAR PrintCommands = $SpHrView_df_signalGlobals+"PrintCommands"
	if(PrintCommands)
		Printf "\rBinaural Signal: %s\r", WavesName
	endif
	
	// get signal generation sequence, shortest first, longest last (although not necessary)
	Make/O/N=(4) SignalSequence, SignalSeconds // / FREE
	SignalSequence = NaN
	variable i
	for(i=0;i<4;++i)
		if( (i==0 && makeAmbient) || (i==1 && makeNoise1) || (i==2 && makeNoise2) || (i==3 && makeTones)  )
			// Duration flags
			wave Durations = $SpHrView_df_signalGlobals+"Durations"
			wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
			strswitch(DurationUnits[i])
				case "ms":
					SignalSeconds[i] = Durations[i]/1000
					SignalSequence[i] = i
					break
				case "sec":
				case "seconds":
					SignalSeconds[i] = Durations[i]
					SignalSequence[i] = i
					break
				case "points":
				case "pnts":
				case "rows":
					SignalSeconds[i] = Durations[i] / fs
					SignalSequence[i] = i
					break
				default:
					Doalert/T="Signal Error" 0, "Duration units missing"
					return 0
			endswitch
		endif
	endfor
	
	// sort by signal length so the longest is made last
	Sort SignalSeconds, SignalSeconds, SignalSequence
	killwaves/Z SignalSeconds
	
	////////////////////////////////////////
	variable SigInd, cnt=0
	for(i=0;i<dimsize(SignalSequence,0);++i)
		
		SigInd = SignalSequence[i]
			
		if(numtype(SigInd)==0)
		
			functionStr = "SpHr_SignalBinaural"
			
			if(cnt > 0)
				functionStr += "/Sum"
			endif
			cnt += 1
			
			// Duration flag
			wave Durations = $SpHrView_df_signalGlobals+"Durations"
			wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
			strswitch(DurationUnits[SigInd])
				case "ms":
					functionStr += "/ms="+num2str(Durations[SigInd])
					break
				case "sec":
				case "seconds":
					functionStr += "/sec="+num2str(Durations[SigInd])
					break
				case "points":
				case "pnts":
				case "rows":
					functionStr += "/ROWS="+num2str(Durations[SigInd])
					break
				default:
					Doalert/T="Signal Error" 0, "Duration units missing"
					return 0
			endswitch
			
			// Buffer flag
			wave OnsetBuffers_sec = $SpHrView_df_signalGlobals+"OnsetBuffers_sec"
			wave OnsetDelays_ms = $SpHrView_df_signalGlobals+"OnsetDelays_ms"
			wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
			wave OffsetDelays_ms = $SpHrView_df_signalGlobals+"OffsetDelays_ms"
			variable OnsetBuffer_ms = abs(OnsetDelays_ms[SigInd]) + abs(OnsetBuffers_sec[SigInd]*1000.0)
			variable OffsetBuffer_ms = abs(OffsetDelays_ms[SigInd]) + abs(OffsetBuffers_sec[SigInd]*1000.0)
			if(OnsetBuffer_ms > 0 && OnsetBuffer_ms == OffsetBuffer_ms)
				functionStr += "/Buff="+num2str(OnsetBuffer_ms)
			elseif(OnsetBuffer_ms > 0 || OffsetBuffer_ms > 0)
				functionStr += "/Buff={"+num2str(OnsetBuffer_ms)+","+num2str(OffsetBuffer_ms)+"}"
			endif
			
			// Ramp flag
			wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
			wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
			if(OnRamps_ms[SigInd] == OffRamps_ms[SigInd])
				functionStr += "/Ramp="+num2str(OnRamps_ms[SigInd])
			else
				functionStr += "/Ramp={"+num2str(OnRamps_ms[SigInd])+","+num2str(OffRamps_ms[SigInd])+"}"
			endif
			
			// Amplitude flag
			wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
			wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
			strswitch(AmplitudeUnits[SigInd])
				case "dB":
				case "dB re:20µPa":
					functionStr += "/dB="+num2str(Amplitudes[SigInd])
					break
				case "Pa":
				case "V":
					functionStr += "/Pa="+num2str(Amplitudes[SigInd])
					break
			endswitch
			
			////////////////////
			// Tone or Sweep flags
			if(SigInd == 3)
			
				NVAR F1 = $SpHrView_df_signalGlobals+"F1"
				SVAR toneType = $SpHrView_df_signalGlobals+"toneType"
				DF = SpHrView_df_signalGlobals+"Tones:"
				NVAR useDFTforTones = $DF+"useDFTforTones"
				//NVAR DFTPhiShift = $DF+"DFTPhiShift"
				strswitch(toneType)
				
					case "Log Sweep":
						NVAR SweepF2 = $SpHrView_df_signalGlobals+"SweepF2"
						functionStr += "/Swp={"+num2str(F1)+","+num2str(SweepF2)+",\"Log\"}" // Log sweep
						break
					case "Linear Sweep":
						NVAR SweepF2 = $SpHrView_df_signalGlobals+"SweepF2"
						functionStr += "/Swp={"+num2str(F1)+","+num2str(SweepF2)+",\"Linear\"}" // Linear sweep
						break
					case "Linear 'Pink' Sweep":
						NVAR SweepF2 = $SpHrView_df_signalGlobals+"SweepF2"
						functionStr += "/Swp={"+num2str(F1)+","+num2str(SweepF2)+",\"Linear 1/f\"}" // Linear sweep with 1/frequency spectrum
						break
						
					case "Pure Tone":
						functionStr += "/Tone="+num2str(F1)
						break
						
					Default:
						if(waveexists($DF+"toneFreqs") && waveexists($DF+"toneAmps") && waveexists($DF+"tonePhis"))
							functionStr += "/Tone={0,"+DF+"toneFreqs,"+DF+"toneAmps,"+DF+"tonePhis}"  // /Tone={reserved, FreqSrcWave, [AmpSrcWave, PhiSrcWave]}
						else
							DoAlert/T="Warning" 0, "Wave missing: "+DF+"toneFreqs or "+DF+"toneAmps or "+DF+"tonePhis"
						endif
						break
				endswitch
				
			endif
				
			////////////////////
			// Tone(s) or Noise flags
			
			if(SigInd < 3)
			
				// noise Seeds
				if(SigInd == 1 || SigInd == 2)
					if(makeNoise1 && makeNoise2 && same12NoiseSeeds)
						if(noiseSeed0 > 0) // && noiseSeed1>0 && noiseSeed2>0
							functionStr += "/NS={"+num2istr(noiseSeed0)+","+num2istr(noiseSeed1)+","+num2istr(noiseSeed2)+"}"
						endif
					endif
				endif
								
				// Highpass filter flag
				wave HighPassFreqs = $SpHrView_df_signalGlobals+"HighPassFreqs"
				if(HighPassFreqs[SigInd] > 1)
					wave HighPassOrders = $SpHrView_df_signalGlobals+"HighPassOrders"
					if(HighPassOrders[SigInd] == 2) // default order
						functionStr += "/HP="+num2str(HighPassFreqs[SigInd])
					else
						functionStr += "/HP={"+num2str(HighPassFreqs[SigInd])+","+num2str(HighPassOrders[SigInd])+"}"
					endif
				endif
				// Lowpass filter flag
				wave LowPassFreqs = $SpHrView_df_signalGlobals+"LowPassFreqs"
				if(LowPassFreqs[SigInd] < fs*0.5)
					wave LowPassOrders = $SpHrView_df_signalGlobals+"LowPassOrders"
					if(LowPassOrders[SigInd] == 2) // default order
						functionStr += "/LP="+num2str(LowPassFreqs[SigInd])
					else
						functionStr += "/LP={"+num2str(LowPassFreqs[SigInd])+","+num2str(LowPassOrders[SigInd])+"}"
					endif
				endif
				
				// Spectrum flag
				wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
				DF = SpHrView_df_signalGlobals+"Spectrum:"
				strswitch(SpectrumShape[SigInd])
					case "flat":
					case "white":
						//functionStr += "/Sp=0"	// flat
						break
					case "1 / Frequency":
					case "1 / Freq":
					case "1 / f":
					case "pink":
						functionStr += "/Sp=1" // 1 / frequency
						break
						
					case "Octave Band":
					case "Range":
					case "Arbitrary":
						wave/Z Spectrum = $SpHrView_df_signals+"Spectrum_"+num2str(SigInd)
						if(!waveexists(Spectrum))
							DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"Spectrum_"+num2str(SigInd)+"  is missing"
							return 0
						endif
						wave/T/Z InterpType = $DF+"InterpType"
						if(!waveexists(InterpType))
							DoAlert/T="Warning" 0, "Wave  "+DF+"InterpType  is missing"
							return 0
						endif
						strswitch(InterpType[SigInd])
							case "Linear":
								functionStr += "/Sp={1,0,"+SpHrView_df_signals+"Spectrum_"+num2str(SigInd)+"}" // Interpolation type, dB scale, 
								break
							case "Quadratic":
								functionStr += "/Sp={4,0,"+SpHrView_df_signals+"Spectrum_"+num2str(SigInd)+"}"
								break
							case "Spline":
								functionStr += "/Sp={2,0,"+SpHrView_df_signals+"Spectrum_"+num2str(SigInd)+"}"
								break
						endswitch
						break
						
					default:
						functionStr += "/Sp=1"	// 1 / frequency
						break
				endswitch
				// Spectrum for right channel
				wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
				if(!LinkSpectrumChannels[0])
					wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
					strswitch(SpectrumShape2[SigInd])
						case "flat":
						case "white":
							//functionStr += "/Sp2=0"	// flat
							break
						case "1 / Frequency":
						case "1 / Freq":
						case "1 / f":
						case "pink":
							functionStr += "/Sp2=1" // 1 / frequency
							break
							
						case "Octave Band":
						case "Range":
						case "Arbitrary":
							wave/Z Spectrum2 = $SpHrView_df_signals+"Spectrum2_"+num2str(SigInd)
							if(!waveexists(Spectrum2))
								DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"Spectrum2_"+num2str(SigInd)+"  is missing"
								return 0
							endif
							wave/T/Z InterpType2 = $DF+"InterpType2"
							if(!waveexists(InterpType2))
								DoAlert/T="Warning" 0, "Wave  "+DF+"InterpType2  is missing"
								return 0
							endif
							strswitch(InterpType2[SigInd])
								case "Linear":
									functionStr += "/Sp2={1,0,"+SpHrView_df_signals+"Spectrum2_"+num2str(SigInd)+"}" // Interpolation type, dB scale, 
									break
								case "Quadratic":
									functionStr += "/Sp2={4,0,"+SpHrView_df_signals+"Spectrum2_"+num2str(SigInd)+"}"
									break
								case "Spline":
									functionStr += "/Sp2={2,0,"+SpHrView_df_signals+"Spectrum2_"+num2str(SigInd)+"}"
									break
							endswitch
							break
						default:
							functionStr += "/Sp2=1"	// 1 / frequency
							break
					endswitch
				endif
				
				// Independence flag
				wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
				DF = SpHrView_df_signalGlobals+"Independence:"
				strswitch(Independence[SigInd])
					case "Index":
						wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
						if(!waveexists(IndependenceVal))
							DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"IndependenceVal  is missing"
						endif
						if(IndependenceVal[SigInd] > 0)
							functionStr += "/Ind="+num2str(IndependenceVal[SigInd])	// independence value
						endif
						break
					default:
						wave/T/Z InterpType = $DF+"InterpType"
						if(!waveexists(InterpType))
							DoAlert/T="Warning" 0, "Wave  "+DF+"InterpType  is missing"
							return 0
						endif
						InterpVal = 1
						strswitch(InterpType[SigInd])
							case "Linear":
								InterpVal = 1
								break
							case "Quadratic":
								InterpVal = 4
								break
							case "Spline":
								InterpVal = 2
								break
						endswitch
						wave/Z Independence_N = $SpHrView_df_signals+"Independence_"+num2str(SigInd)
						//if(!waveexists(Independence_N))
						//	DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"Independence_"+num2str(SigInd)+"  is missing"
						//	return 0
						//endif
						if(waveexists(Independence_N))
							functionStr += "/Ind={"+num2str(InterpVal)+",0,"+SpHrView_df_signals+"Independence_"+num2str(SigInd)+"}"
						endif
				endswitch
				
				// Phase inversion flag
				wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
				DF = SpHrView_df_signalGlobals+"PhaseInversion:"
				strswitch(PhaseInversion[SigInd])
					case "Normal Phase":
						//functionStr += "/INVP=0"
						break
					case "Inverted Phase":
						functionStr += "/INVP=1"
						break
					default:
						wave/Z PhaseInversion_N = $SpHrView_df_signals+"PhaseInversion_"+num2str(SigInd)
						//if(!waveexists(PhaseInversion_N))
						//	DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"PhaseInversion_"+num2str(SigInd)+"  is missing"
						//	return 0
						//endif
						if(waveexists(PhaseInversion_N))
							functionStr += "/InvP={1,"+SpHrView_df_signals+"PhaseInversion_"+num2str(SigInd)+"}"
						endif
				endswitch
						
			endif
						
			//////////////////////////////////////////
			// interaural cue flags
			
			// ITD flag
			wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
			if(!waveexists(ITD))
				DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"ITD  is missing"
			endif
			wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
			if(!waveexists(ITDVal))
				DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"ITDVal  is missing"
			endif
			DF = SpHrView_df_signalGlobals+"ITD:"
			strswitch(ITD[SigInd])
				case "ITD (nearest point)":
					if(ITDVal[SigInd] != 0)
						functionStr += "/ITD="+num2str(ITDVal[SigInd])	// ITD value in ms
					endif
					break
				case "ITD (phase shift)":
					if(ITDVal[SigInd] != 0)
						functionStr += "/ITD={"+num2str(ITDVal[SigInd])+",1}"	// ITD value in ms
					endif
					break
				case "ITD (ramped phase shift)":
					if(ITDVal[SigInd] != 0)
						functionStr += "/ITD={"+num2str(ITDVal[SigInd])+",2}"	// ITD value in ms
					endif
					break
					
				default:
					wave/T/Z InterpType = $DF+"InterpType"
					if(!waveexists(InterpType))
						DoAlert/T="Warning" 0, "Wave  "+DF+"InterpType  is missing"
						return 0
					endif
					InterpVal = 1
					strswitch(InterpType[SigInd])
						case "Linear":
							InterpVal = 1
							break
						case "Quadratic":
							InterpVal = 4
							break
						case "Spline":
							InterpVal = 2
							break
					endswitch
					wave/Z ITD_0 = $SpHrView_df_signals+"ITD_"+num2str(SigInd)
					if(!waveexists(ITD_0))
						DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"ITD_"+num2str(SigInd)+"  is missing"
						return 0
					endif
					// /ITD={interpType , units, ITDSrcWave [, FreqSrcWave]}
					functionStr += "/ITD={"+num2str(InterpVal)+",0,"+SpHrView_df_signals+"ITD_"+num2str(SigInd)+"}" // units are sec
			endswitch
			
			// IPD flag
			wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
			if(!waveexists(IPD))
				DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"IPD  is missing"
			endif
			wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
			if(!waveexists(IPDVal))
				DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"IPDVal  is missing"
			endif
			DF = SpHrView_df_signalGlobals+"IPD:"
			strswitch(IPD[SigInd])
				case "IPD":
					if(IPDVal[SigInd] != 0.0)
						functionStr += "/IPD={"+num2str(IPDVal[SigInd])+",1}"	// pass 1 for IPD value in cycles
					endif
					break
				default:
					wave/T/Z InterpType = $DF+"InterpType"
					if(!waveexists(InterpType))
						DoAlert/T="Warning" 0, "Wave  "+DF+"InterpType  is missing"
						return 0
					endif
					InterpVal = 1
					strswitch(InterpType[SigInd])
						case "Linear":
							InterpVal = 1
							break
						case "Quadratic":
							InterpVal = 4
							break
						case "Spline":
							InterpVal = 2
							break
					endswitch
					wave/Z IPD_0 = $SpHrView_df_signals+"IPD_"+num2str(SigInd)
					if(!waveexists(IPD_0))
						DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"IPD_"+num2str(SigInd)+"  is missing"
						return 0
					endif
					// /IPD={interpType , units, IPDSrcWave [, FreqSrcWave]}
					functionStr += "/IPD={"+num2str(InterpVal)+",1,"+SpHrView_df_signals+"IPD_"+num2str(SigInd)+"}" // units are cycles
			endswitch
			
			// ILD flag
			wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
			if(!waveexists(ILD))
				DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"ILD  is missing"
			endif
			wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
			if(!waveexists(ILDVal))
				DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signalGlobals+"ILDVal  is missing"
			endif
			DF = SpHrView_df_signalGlobals+"ILD:"
			strswitch(ILD[SigInd])
				case "ILD":
					if(ILDVal[SigInd] != 0.0)
						functionStr += "/ILD={"+num2str(ILDVal[SigInd])+",0}"	// ILD value in dB
					endif
					break
				default:
					wave/T/Z InterpType = $DF+"InterpType"
					if(!waveexists(InterpType))
						DoAlert/T="Warning" 0, "Wave  "+DF+"InterpType  is missing"
						return 0
					endif
					InterpVal = 1
					strswitch(InterpType[SigInd])
						case "Linear":
							InterpVal = 1
							break
						case "Quadratic":
							InterpVal = 4
							break
						case "Spline":
							InterpVal = 2
							break
					endswitch
					wave/Z ILD_0 = $SpHrView_df_signals+"ILD_"+num2str(SigInd)
					if(!waveexists(ILD_0))
						DoAlert/T="Warning" 0, "Wave  "+SpHrView_df_signals+"ILD_"+num2str(SigInd)+"  is missing"
						return 0
					endif
					// /ILD={interpType , units, ILDSrcWave [, FreqSrcWave]}
					functionStr += "/ILD={"+num2str(InterpVal)+",0,"+SpHrView_df_signals+"ILD_"+num2str(SigInd)+"}" // units are sec
			endswitch
				
			// DFT flags
			// force DFT when making tones
			NVAR makeDFTwaves = $SpHrView_df_signalGlobals+"makeDFTwaves"
			if(makeDFTwaves)
				useDFTforTones = 1
			endif
			if(!makeDFTwaves && (SigInd == 3 && useDFTforTones))
					functionStr += "/DFT=1"
			endif
			// make DFT waves when making noises
			if(makeDFTwaves) // SigInd != 3 && 
				functionStr += "/DFT={1,"+SpHrView_df_signals+"DFT}"
			endif
			
			//  HRIR
			if(strlen(OperationList("SpHr_HRIR*", ";", "external")) < 2)
				Print "SpHr_HRIR operation provided by HRIR-64.xop"
			else
				wave/T HRIR_DataSet = $SpHrView_df_signalGlobals+"HRIR_DataSet"
				wave useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC" 			// HRIR_useCDFC[SigInd]
				wave/T HPCFStr = $SpHrView_df_signalGlobals+"HRIR_HPCFStr"		// HRIR_HPCFStr[SigInd]
				variable DataSet = -1
				strswitch(HRIR_DataSet[SigInd])
					case "2° - NeumannKU100":
						DataSet = 1
						break
					case "360° microphone stand - NeumannKU100":
						DataSet = 2
						break
					case "360° full rotation mount - NeumannKU100":
						DataSet = 3
						break
					case "Lebedev 2354 - NeumannKU100":
						DataSet = 4
						break
					case "Lebedev 2702 - NeumannKU100":
						DataSet = 5
						break
					case "No HRIR":
						break
					default:
						break
				endswitch
				if(DataSet > 0) // ignore if DataSet = 0, Ambient signal
					if(fs != 48000)
						Printf "Signal must be sampled at 48,000 Hz when using HRIRs. Filter frequencies for current signal may be incorrect."
						fs = 48000
					endif
					wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
					wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
					// using Execute in case HRIR-64.xop is missing
					Execute "SpHr_HRIR /Deg={"+num2str(HRIR_Az[SigInd])+","+num2str(HRIR_El[SigInd])+"} /CDFC=("+num2str(useCDFC[SigInd])+")/HPCF=\""+HPCFStr[SigInd]+"\"/Imp=IRs DataSet=("+num2istr(DataSet)+")"
					//SpHr_HRIR /Deg={HRIR_Az[SigInd],HRIR_El[SigInd]} /CDFC=(useCDFC[SigInd]) /HPCF=HPCFStr[SigInd] /Imp=IRs DataSet=(DataSet)
					if(PrintCommands)
						Print "SpHr_HRIR /Deg={"+num2str(HRIR_Az[SigInd])+","+num2str(HRIR_El[SigInd])+"} /CDFC=("+num2str(useCDFC[SigInd])+")/HPCF=\""+HPCFStr[SigInd]+"\"/Imp=IRs DataSet=("+num2istr(DataSet)+")"
					endif
					functionStr += "/Conv=IRs"
					//Printf "Az=%g, El=%g, Index= %g" , V_Az, V_El, V_Index
				endif
			endif
			
			// BRIR
			wave/T BRIR_Location = $SpHrView_df_signalGlobals+"BRIR_Location"
			wave/T BRIR_Option = $SpHrView_df_signalGlobals+"BRIR_Option"
			wave/T BRIR_Source = $SpHrView_df_signalGlobals+"BRIR_Source"
			wave BRIR_Az = $SpHrView_df_signalGlobals+"BRIR_Az"
			
			String WaveStr = ""
			
			variable BRIRoption = 0
			variable BRIRsource = 0
			strswitch(BRIR_Option[SigInd])
				case "Microphone stand":
					BRIRoption = 1
					break
				case "Rotation mount":
					BRIRoption = 2
					break
					
//				default:
//					WaveStr = BRIR_Option[SigInd]
//					break

			endswitch
			strswitch(BRIR_Location[SigInd])
				case "Select BRIR wave":
					
					WaveStr = BRIR_Option[SigInd]
					wave w = $"root:SpHrView:Transfer:ImpulseWaves:"+WaveStr
					if(waveexists(w))
						//Printf "fs= %g, 1.0/dimdelta(w,0)= %g\r", fs, round(1.0/dimdelta(w,0))
						if(round(1.0/dimdelta(w,0)) != fs)
							fs = 1.0/dimdelta(w,0)
							Printf "Sampling rate changed to %g to match seleted BRIR.\r", fs
						endif
						Duplicate/O w, IRs
						functionStr += "/Conv=IRs"
					endif
					break
			
				case "Control Room 1 - NeumannKU100":
					if(strlen(OperationList("SpHr_BRIR_CR1_NeumannKU100", ";", "external")) < 2)
						Print "SpHr_BRIR_CR1_NeumannKU100 operation provided by BRIR_CR1_NeumannKU100-64.xop"
						break
					endif
					strswitch(BRIR_Source[SigInd])
						case "Left - Genelec8260a":
							BRIRsource = 1
							break
						case "Right - Genelec8260a":
							BRIRsource = 2
							break
					endswitch
					// using Execute in case SpHr_BRIR_CR1_NeumannKU100-64.xop is missing
					Execute "SpHr_BRIR_CR1_NeumannKU100 /Deg=("+num2str(BRIR_Az[SigInd])+")/CDFC=("+num2str(useCDFC[SigInd])+")/HPCF=\""+HPCFStr[SigInd]+"\"/Imp=IRs Source=("+num2istr(BRIRsource)+"), Mount=("+num2istr(BRIRoption)+")"
					//SpHr_BRIR_CR1_NeumannKU100/Deg=(BRIR_Az[SigInd]) /CDFC=(useCDFC[SigInd]) /HPCF=HPCFStr[SigInd]/Imp=IRs Source=(BRIRsource) , Mount=(BRIRoption)
					functionStr += "/Conv=IRs"
					if(fs != 48000)
						Printf "Signal must be sampled at 48,000 Hz when using Neumann BRIRs. Filter frequencies for current signal may be incorrect."
						fs = 48000
					endif
					break
				case "Control Room 7 - NeumannKU100":
					if(strlen(OperationList("SpHr_BRIR_CR7_NeumannKU100", ";", "external")) < 2)
						Print "SpHr_BRIR_CR7_NeumannKU100 operation provided by BRIR_CR7_NeumannKU100-64.xop"
						break
					endif
					strswitch(BRIR_Source[SigInd])
						case "Left - B&W803D":
							BRIRsource = 1
							break
						case "Right - B&W803D":
							BRIRsource = 2
							break
					endswitch
					// using Execute in case SpHr_BRIR_CR7_NeumannKU100-64.xop is missing
					Execute "SpHr_BRIR_CR7_NeumannKU100 /Deg=("+num2str(BRIR_Az[SigInd])+")/CDFC=("+num2str(useCDFC[SigInd])+")/HPCF=\""+HPCFStr[SigInd]+"\"/Imp=IRs Source=("+num2istr(BRIRsource)+"), Mount=("+num2istr(BRIRoption)+")"
					//SpHr_BRIR_CR7_NeumannKU100/Deg=(BRIR_Az[SigInd]) /CDFC=(useCDFC[SigInd]) /HPCF=HPCFStr[SigInd]/Imp=IRs Source=(BRIRsource) , Mount=(BRIRoption)
					functionStr += "/Conv=IRs"
					if(fs != 48000)
						Printf "Signal must be sampled at 48,000 Hz when using Neumann BRIRs. Filter frequencies for current signal may be incorrect."
						fs = 48000
					endif
					break
				case "Small Broadcast Studio - NeumannKU100":
					if(strlen(OperationList("SpHr_BRIR_SBS_NeumannKU100", ";", "external")) < 2)
						Print "SpHr_BRIR_SBS_NeumannKU100 operation provided by BRIR_SBS_NeumannKU100-64.xop"
						break
					endif
					strswitch(BRIR_Source[SigInd])
						case "Center - Sonic Ball":
							BRIRsource = 1
							break
						case "Center - AD Systems":
							BRIRsource = 2
							break
						case "Left - AD Systems":
							BRIRsource = 3
							break
						case "Right - AD Systems":
							BRIRsource = 4
							break
					endswitch
					// using Execute in case SpHr_BRIR_SBS_NeumannKU100-64.xop is missing
					Execute "SpHr_BRIR_SBS_NeumannKU100 /Deg=("+num2str(BRIR_Az[SigInd])+")/CDFC=("+num2str(useCDFC[SigInd])+")/HPCF=\""+HPCFStr[SigInd]+"\"/Imp=IRs Source=("+num2istr(BRIRsource)+"), Mount=("+num2istr(BRIRoption)+")"
					//SpHr_BRIR_SBS_NeumannKU100/Deg=(BRIR_Az[SigInd]) /CDFC=(useCDFC[SigInd]) /HPCF=HPCFStr[SigInd]/Imp=IRs Source=(BRIRsource) , Mount=(BRIRoption)
					functionStr += "/Conv=IRs"
					if(fs != 48000)
						Printf "Signal must be sampled at 48,000 Hz when using Neumann BRIRs. Filter frequencies for current signal may be incorrect."
						fs = 48000
					endif
					break
				case "Large Broadcast Studio - NeumannKU100":
					if(strlen(OperationList("SpHr_BRIR_LBS_NeumannKU100", ";", "external")) < 2)
						Print "SpHr_BRIR_LBS_NeumannKU100 operation provided by BRIR_LBS_NeumannKU100-64.xop"
						break
					endif
					strswitch(BRIR_Source[SigInd])
						case "Center - Sonic Ball":
							BRIRsource = 1
							break
						case "Center - AD Systems":
							BRIRsource = 2
							break
						case "Left - Sonic Ball":
							BRIRsource = 3
							break
						case "Left - AD Systems":
							BRIRsource = 4
							break
					endswitch
					// using Execute in case BRIR_LBS_NeumannKU100-64.xop is missing
					Execute "SpHr_BRIR_LBS_NeumannKU100 /Deg=("+num2str(BRIR_Az[SigInd])+")/CDFC=("+num2str(useCDFC[SigInd])+")/HPCF=\""+HPCFStr[SigInd]+"\"/Imp=IRs Source=("+num2istr(BRIRsource)+"), Mount=("+num2istr(BRIRoption)+")"
					//SpHr_BRIR_LBS_NeumannKU100/Deg=(BRIR_Az[SigInd]) /CDFC=(useCDFC[SigInd]) /HPCF=HPCFStr[SigInd]/Imp=IRs Source=(BRIRsource), Mount=(BRIRoption)
					functionStr += "/Conv=IRs"
					if(fs != 48000)
						Printf "Signal must be sampled at 48,000 Hz when using Neumann BRIRs. Filter frequencies for current signal may be incorrect."
						fs = 48000
					endif
					break
				default:
					break
			endswitch
		
					
			
			///////////////////
			// parameters
			
			// fs parameter
			functionStr += " "+num2str(fs)
			
			// wave name parameter
			SVAR WavesName = $SpHrView_df_signalGlobals+"WavesName"
			functionStr += ", "+WavesName
			if(PrintCommands)
				switch (SigInd)
					case 0:
						Print "//  Ambient signal:"
						break
					case 1:
						Print "//  Noise 1:"
						break
					case 2:
						Print "//  Noise 2:"
						break
					case 3:
						Print "//  Tone(s):"
						break
				endswitch
				Print functionStr
			endif
			
			Execute/Q functionStr
			
			NVAR/Z V_Seed=V_Seed
			NVAR/Z V_Seed2=V_Seed2
			NVAR/Z V_Seed3=V_Seed3
			
			if(SigInd == 1 || SigInd == 2)
				if(makeNoise1 && makeNoise2 && same12NoiseSeeds)
					noiseSeed0 = V_Seed
					noiseSeed1 = V_Seed2
					noiseSeed2 = V_Seed3
				endif
			endif
			
			
			if(PrintCommands)
				// for /NS flag, Mersenne Twister pseudorandom number generator seeds
				if(SigInd < 3)
					if(NVAR_Exists(V_Seed)) // && NVAR_Exists(V_Seed2) && NVAR_Exists(V_Seed3)
						if(noiseSeed0==0 || SigInd==0 || SigInd==2)
							Printf "//  /NS={%d,%d,%d} // internally generated seeds\r", V_Seed, V_Seed2, V_Seed3
						endif
					endif
				endif
				
				if(SigInd == 3)
					SVAR S_toneFreqs = S_toneFreqs
					Printf "//Frequencies: %s\r", S_toneFreqs
				endif
			endif
			
		endif 	// if(numtype(SigInd)==0)
		
	endfor 	// for(i=0;i<dimsize(SignalSequence,0);++i)
	killwaves/Z SignalSequence
	
	//killVariables/Z V_DFT

	wave/Z w = $WavesName
	if(waveexists(w) && playSignal)
		// limit to ±1 since getting artifacts on new MacBook Pro...
		wavestats/Q/M=1 w
		if(v_max > 1 || v_min < -1)
			w /= max(v_max, -v_min)
		endif
		PlaySound/A w
	endif
	
	NVAR View = $SpHrView_df_signalGlobals+"View"
	if(!playSignal && waveexists(w) && View)
		NVAR preserveWaveTemporarily = $SpHrView_df_globals+"preserveWaveTemporarily"
		preserveWaveTemporarily = 1
		killWindow/Z SpHrViewPeaksWindow
		SpHrView_LoadPeaksFromWave(ParseFilePath(0, WavesName, ":", 1, 0), -1 , -1, 0, 0)  // pass only the wave's name, assumed to be in SpHrView_df_waves
	endif
	
//	if(waveexists(w))
//		SpHr_DFTSpectrum/Mag=root:Spectrum/dB w ///Oct=3
//	endif
	SetDataFolder dfSav
end

function SpHrView_SignalDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	
	SpHrView_Defaults()
	String FolderStr
//	// Make "SpHrView" folder, SpHrView_df
//	if(!DataFolderExists(SpHrView_df))
//		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
	// Make "SpHrView:Signals" folder, SpHrView_df_signals
	if(!DataFolderExists(SpHrView_df_signals))
		FolderStr= SpHrView_df_signals[0,strlen(SpHrView_df_signals)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		FolderStr= SpHrView_df_signalGlobals[0,strlen(SpHrView_df_signalGlobals)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	
//	// Make "SpHrView:Signals:Globals:Independence" folder, SpHrView_df_signalGlobals+"Independence:"
//	String DF = SpHrView_df_signalGlobals+"Independence:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
//	//////// default conditions require Independence defaults ///////////////////
//	SpHrView_IndependenceDefaults()
//	
//	// Make "SpHrView:Signals:Globals:PhaseInversion" folder, SpHrView_df_signalGlobals+"PhaseInversion:"
//	DF = SpHrView_df_signalGlobals+"PhaseInversion:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
//	//////// default conditions require PhaseInversion defaults ///////////////////
//	SpHrView_PhaseInversionDefaults()
//	
//	// Make "SpHrView:Signals:Globals:Spectrum" folder, SpHrView_df_signalGlobals+"Spectrum:"
//	DF = SpHrView_df_signalGlobals+"Spectrum:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
//	//////// default conditions require Spectrum defaults ///////////////////
//	//SpHrView_SpectrumDefaults()
//	
//	// Make "SpHrView:Signals:Globals:ITD" folder, SpHrView_df_signalGlobals+"ITD:"
//	DF = SpHrView_df_signalGlobals+"ITD:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
//	// Make "SpHrView:Signals:Globals:IPD" folder, SpHrView_df_signalGlobals+"IPD:"
//	DF = SpHrView_df_signalGlobals+"IPD:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
//	// Make "SpHrView:Signals:Globals:ILD" folder, SpHrView_df_signalGlobals+"ILD:"
//	DF = SpHrView_df_signalGlobals+"ILD:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
//	
//	// Make "SpHrView:Signals:Globals:Tones" folder, SpHrView_df_signalGlobals+"Tones:"
//	DF = SpHrView_df_signalGlobals+"Tones:"
//	if(!DataFolderExists(DF))
//		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
//		NewDataFolder/O $FolderStr
//	endif
	
	
	SetDataFolder $SpHrView_df_signalGlobals
	
	QuerySoundManager(SpHrView_df_signalGlobals+"Audio")
	SVAR rateListStr = $SpHrView_df_signalGlobals+"Audio:rateList"
	
	string sval
	variable nval
	sval= StrVarOrDefault(SpHrView_df_signalGlobals+"WavesName", "root:SpHrView:Waves:Untitled")		// propotion of screen height used for display
	string/G WavesName = sval
	sval= StrVarOrDefault(SpHrView_df_signalGlobals+"PresetStr", "Default")		// Default preset string
	string/G PresetStr = sval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"View", 1)						// View wave in SpHrView
	variable/G View = nval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"PrintCommands", 0)			// print sound generation command(s)
	variable/G PrintCommands = nval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"KeepWindowOpen", 1)			// do not kill window when deactivated
	variable/G KeepWindowOpen = nval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"fs", 48000)						// Sampling rate (Hz)
	variable/G fs = nval
	string fsStr = num2str(fs)
	if(strsearch(rateListStr, fsStr, 0) < 0)
		fs = str2num(stringFromList(0, rateListStr))
	endif
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"makeAmbient", 1)			// make with background
	variable/G makeAmbient = nval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"makeNoise1", 1)			// make Noise 1
	variable/G makeNoise1 = nval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"makeNoise2", 0)			// make Noise 2
	variable/G makeNoise2 = nval
	
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"makeDFTwaves", 0)			// make DFT waves
	variable/G makeDFTwaves = nval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"same12NoiseSeeds", 1)	// seed for Noise 2 same as for Noise 1
	variable/G same12NoiseSeeds = nval
	
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"makeTones", 0)				// make Tones
	variable/G makeTones = nval

	sval= StrVarOrDefault(SpHrView_df_signalGlobals+"toneType", "Pure Tone")	// Tone or Tones
	string/G toneType = sval
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"F1", 100)			// Tone frequecy
	variable/G F1 = nval
	

	
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"logFreqAxis", 1)			// log frequency axis
	variable/G logFreqAxis = nval
		
	// waves
	if(!waveexists($SpHrView_df_signalGlobals+"Durations"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"Durations"; wave Durations=$SpHrView_df_signalGlobals+"Durations"
		Durations[0] = 3;			// sec, ambient
		Durations[1,3] = 1; 		// sec, Noise1, Noise2, Tones
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"DurationUnits"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"DurationUnits"; wave/T DurationUnits=$SpHrView_df_signalGlobals+"DurationUnits"
		DurationUnits[0] = "sec"
		DurationUnits[1,3] = "sec"
	endif
	

	if(!waveexists($SpHrView_df_signalGlobals+"Amplitudes"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"Amplitudes"; wave Amplitudes=$SpHrView_df_signalGlobals+"Amplitudes"
		if(1) // dB
			Amplitudes[0] = 30
			Amplitudes[1,3] = 70
		else	// Pa
			Amplitudes[0] = (10^(30/20))*0.00002
			Amplitudes[1,3] = (10^(70/20))*0.00002
		endif
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"AmplitudeUnits"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"AmplitudeUnits"; wave/T AmplitudeUnits=$SpHrView_df_signalGlobals+"AmplitudeUnits"
		AmplitudeUnits = "dB re:20µPa"
	endif

	if(!waveexists($SpHrView_df_signalGlobals+"OnRamps_ms"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"OnRamps_ms"; wave OnRamps_ms=$SpHrView_df_signalGlobals+"OnRamps_ms"
		OnRamps_ms = 20
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"OffRamps_ms"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"OffRamps_ms"; wave OffRamps_ms=$SpHrView_df_signalGlobals+"OffRamps_ms"
		OffRamps_ms = 20
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"OnsetBuffers_sec"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"OnsetBuffers_sec"; wave OnsetBuffers_sec=$SpHrView_df_signalGlobals+"OnsetBuffers_sec"
		OnsetBuffers_sec[0] = 0
		OnsetBuffers_sec[1,3] = 1
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"OnsetDelays_ms"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"OnsetDelays_ms"; wave OnsetDelays_ms=$SpHrView_df_signalGlobals+"OnsetDelays_ms"
		OnsetDelays_ms[0,1] = 0
		OnsetDelays_ms[2] = 0
		OnsetDelays_ms[3] = 0
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"OffsetBuffers_sec"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"OffsetBuffers_sec"; wave OffsetBuffers_sec=$SpHrView_df_signalGlobals+"OffsetBuffers_sec"
		OffsetBuffers_sec[0] = 0
		OffsetBuffers_sec[1,3] = 0
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"OffsetDelays_ms"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"OffsetDelays_ms"; wave OffsetDelays_ms=$SpHrView_df_signalGlobals+"OffsetDelays_ms"
		OffsetDelays_ms = 0
	endif
	

	if(!waveexists($SpHrView_df_signalGlobals+"SpectrumShape"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"SpectrumShape"; wave/T SpectrumShape=$SpHrView_df_signalGlobals+"SpectrumShape"
		SpectrumShape = "1 / Frequency"
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"SpectrumShape2"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"SpectrumShape2"; wave/T SpectrumShape2=$SpHrView_df_signalGlobals+"SpectrumShape2"
		SpectrumShape2 = "1 / Frequency"
	endif
	wave/Z LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
	if(!waveexists(LinkSpectrumChannels))
		Make/O/N=(4) $SpHrView_df_signalGlobals+"LinkSpectrumChannels"; wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
		LinkSpectrumChannels = 1;
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"LowPassFreqs"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"LowPassFreqs"; wave LowPassFreqs=$SpHrView_df_signalGlobals+"LowPassFreqs"
		LowPassFreqs = fs*0.5
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"LowPassOrders"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"LowPassOrders"; wave LowPassOrders=$SpHrView_df_signalGlobals+"LowPassOrders"
		LowPassOrders = 2
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"HighPassFreqs"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"HighPassFreqs"; wave HighPassFreqs=$SpHrView_df_signalGlobals+"HighPassFreqs"
		HighPassFreqs = 40
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"HighPassOrders"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"HighPassOrders"; wave HighPassOrders=$SpHrView_df_signalGlobals+"HighPassOrders"
		HighPassOrders = 2
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"Independence"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"Independence"; wave/T Independence=$SpHrView_df_signalGlobals+"Independence"
		Independence[0] = "Independence_Ambient"
		Independence[1,2] = "Index"
		Independence[3] = ""
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"IndependenceVal"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"IndependenceVal"; wave IndependenceVal=$SpHrView_df_signalGlobals+"IndependenceVal"
		IndependenceVal = 0
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"PhaseInversion"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"PhaseInversion"; wave/T PhaseInversion=$SpHrView_df_signalGlobals+"PhaseInversion"
		PhaseInversion[0] = "PhaseInverts_Ambient"
		PhaseInversion[1,3] = "Normal Phase"
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"ITD"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"ITD"; wave/T ITD=$SpHrView_df_signalGlobals+"ITD"
		ITD = "ITD (phase shift)"
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"ITDVal"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"ITDVal"; wave ITDVal=$SpHrView_df_signalGlobals+"ITDVal"
		ITDVal = 0
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"IPD"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"IPD"; wave/T IPD=$SpHrView_df_signalGlobals+"IPD"
		IPD = "IPD"
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"IPDVal"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"IPDVal"; wave IPDVal=$SpHrView_df_signalGlobals+"IPDVal"
		IPDVal = 0
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"ILD"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"ILD"; wave/T ILD=$SpHrView_df_signalGlobals+"ILD"
		ILD = "ILD"
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"ILDVal"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"ILDVal"; wave ILDVal=$SpHrView_df_signalGlobals+"ILDVal"
		ILDVal = 0
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"HRIR_DataSet"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"HRIR_DataSet"; wave/T HRIR_DataSet=$SpHrView_df_signalGlobals+"HRIR_DataSet"
		HRIR_DataSet[0] = "No HRIR"
		HRIR_DataSet[1] = "No HRIR"
		HRIR_DataSet[2] = "No HRIR"
		HRIR_DataSet[3] = "No HRIR"
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"HRIR_Az"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"HRIR_Az"; wave HRIR_Az=$SpHrView_df_signalGlobals+"HRIR_Az"
		HRIR_Az = 0
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"HRIR_EL"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"HRIR_EL"; wave HRIR_EL=$SpHrView_df_signalGlobals+"HRIR_EL"
		HRIR_EL = 0
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"BRIR_Location"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"BRIR_Location"; wave/T BRIR_Location=$SpHrView_df_signalGlobals+"BRIR_Location"
		BRIR_Location[0] = "No BRIR"
		BRIR_Location[1] = "No BRIR"
		BRIR_Location[2] = "No BRIR"
		BRIR_Location[3] = "No BRIR"
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"BRIR_Source"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"BRIR_Source"; wave/T BRIR_Source=$SpHrView_df_signalGlobals+"BRIR_Source"
		BRIR_Source[0] = ""
		BRIR_Source[1] = ""
		BRIR_Source[2] = ""
		BRIR_Source[3] = ""
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"BRIR_Option"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"BRIR_Option"; wave/T BRIR_Option=$SpHrView_df_signalGlobals+"BRIR_Option"
		BRIR_Option[0] = ""
		BRIR_Option[1] = ""
		BRIR_Option[2] = ""
		BRIR_Option[3] = ""
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"BRIR_Az"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"BRIR_Az"; wave BRIR_Az=$SpHrView_df_signalGlobals+"BRIR_Az"
		BRIR_Az = 0
	endif
	
	if(!waveexists($SpHrView_df_signalGlobals+"HRIR_useCDFC"))
		Make/O/N=4 $SpHrView_df_signalGlobals+"HRIR_useCDFC"; wave HRIR_useCDFC=$SpHrView_df_signalGlobals+"HRIR_useCDFC"
		HRIR_useCDFC = 0
		HRIR_useCDFC[1,3] = 1
	endif
	if(!waveexists($SpHrView_df_signalGlobals+"HRIR_HPCFStr"))
		Make/T/O/N=4 $SpHrView_df_signalGlobals+"HRIR_HPCFStr"; wave/T HRIR_HPCFStr=$SpHrView_df_signalGlobals+"HRIR_HPCFStr"
		HRIR_HPCFStr[0] = "No headphone compensation"
		HRIR_HPCFStr[1] = "No headphone compensation"
		HRIR_HPCFStr[2] = "No headphone compensation"
		HRIR_HPCFStr[3] = "No headphone compensation"
	endif
	
	
	
	// Make "SpHrView:Signals:Globals:Independence" folder, SpHrView_df_signalGlobals+"Independence:"
	String DF = SpHrView_df_signalGlobals+"Independence:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	//////// default conditions require Independence defaults ///////////////////
	SpHrView_IndependenceDefaults()
	
	// Make "SpHrView:Signals:Globals:PhaseInversion" folder, SpHrView_df_signalGlobals+"PhaseInversion:"
	DF = SpHrView_df_signalGlobals+"PhaseInversion:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	//////// default conditions require PhaseInversion defaults ///////////////////
	SpHrView_PhaseInversionDefaults()
	
	// Make "SpHrView:Signals:Globals:Spectrum" folder, SpHrView_df_signalGlobals+"Spectrum:"
	DF = SpHrView_df_signalGlobals+"Spectrum:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	//////// default conditions require Spectrum defaults ///////////////////
	//SpHrView_SpectrumDefaults()
	
	// Make "SpHrView:Signals:Globals:ITD" folder, SpHrView_df_signalGlobals+"ITD:"
	DF = SpHrView_df_signalGlobals+"ITD:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Signals:Globals:IPD" folder, SpHrView_df_signalGlobals+"IPD:"
	DF = SpHrView_df_signalGlobals+"IPD:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Signals:Globals:ILD" folder, SpHrView_df_signalGlobals+"ILD:"
	DF = SpHrView_df_signalGlobals+"ILD:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	
	// Make "SpHrView:Signals:Globals:Tones" folder, SpHrView_df_signalGlobals+"Tones:"
	DF = SpHrView_df_signalGlobals+"Tones:"
	if(!DataFolderExists(DF))
		FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	
	
	
	
	
	SetDataFolder $SpHrView_df_signals
	
	//////////////   Ambient Independence indicies    //////////////////////////
	
//	//   DailyBinauralLog, average IPD distribtuions 9:03 AM 8/3/19   //
//	wave/Z Independence_Ambient = $SpHrView_df_signals+"Independence_Ambient"
//	if(!waveexists(Independence_Ambient))
//		Make/O/N=(64,3) Independence_Ambient
//		Independence_Ambient[][0]={0.052,0.081,0.112,0.148,0.199,0.256,0.283,0.260,0.242,0.253,0.279,0.294,0.307,0.330,0.354,0.384,0.418,0.449,0.488,0.537,0.575,0.629,0.691,0.817,0.543,0.505,0.478,0.464,0.470,0.493,0.518,0.556,0.648,0.751,0.686,0.665,0.665,0.668,0.669,0.704,0.801,0.614,0.629,0.630,0.642,0.880,0.813,0.777,0.766,0.769,0.792,0.843,0.874,0.863,0.837,0.813,0.807,0.802,0.784,0.757,0.735,0.716,0.696,0.688}
//		Independence_Ambient[][1]={20,32,45,59,73,88,104,120,137,156,175,195,216,238,261,285,310,337,365,395,425,458,492,528,565,604,646,689,734,782,832,885,940,998,1058,1122,1189,1259,1333,1410,1491,1577,1666,1760,1858,1962,2070,2184,2304,2429,2561,2699,2844,2996,3156,3323,3499,3684,3877,4081,4294,4518,4753,5000}
//		Independence_Ambient[][2]={0.000,0.000,0.000,0.000,0.000,0.000,0.001,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.001,0.001,0.002,0.002,0.004,0.006,0.011,0.027,0.214,0.051,0.033,0.022,0.017,0.018,0.025,0.038,0.042,0.425,0.076,0.032,0.028,0.030,0.032,0.033,0.051,0.304,0.381,0.481,0.323,0.278,0.446,0.157,0.090,0.073,0.072,0.089,0.183,0.340,0.287,0.189,0.127,0.115,0.113,0.098,0.069,0.051,0.041,0.032,0.028}
//		SetDimLabel 1, 0, Index, Independence_Ambient
//		SetDimLabel 1, 1, Hz, Independence_Ambient
//		SetDimLabel 1, 2, Sigma, Independence_Ambient
//	endif
//	//////////////  Ambient Phase inversions    //////////////////////////
//	//   DailyBinauralLog, average phase inversions 9:03 AM 8/3/19   //
//	wave/Z PhaseInverts_Ambient = $SpHrView_df_signals+"PhaseInverts_Ambient"
//	if(!waveexists(PhaseInverts_Ambient))
//		Make/O/N=(64,2) PhaseInverts_Ambient
//		PhaseInverts_Ambient[][0]={0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,1.000,1.000,1.000,1.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000}
//		PhaseInverts_Ambient[][1]={20,32,45,59,73,88,104,120,137,156,175,195,216,238,261,285,310,337,365,395,425,458,492,528,565,604,646,689,734,782,832,885,940,998,1058,1122,1189,1259,1333,1410,1491,1577,1666,1760,1858,1962,2070,2184,2304,2429,2561,2699,2844,2996,3156,3323,3499,3684,3877,4081,4294,4518,4753,5000}
//		SetDimLabel 1, 0, Invert, PhaseInverts_Ambient
//		SetDimLabel 1, 1, Hz, PhaseInverts_Ambient
//	endif

	//   DailyBinauralLog, average IPD distribtuions 2:00 PM 5/27/20   //
	//////////////   Independence indicies    //////////////////////////
	wave/Z Independence_Ambient = $SpHrView_df_signals+"Independence_Ambient"
	if(!waveexists(Independence_Ambient))
		Make/O/N=(80,3) Independence_Ambient
		Independence_Ambient[][0]={0.066,0.068,0.071,0.073,0.077,0.080,0.084,0.088,0.092,0.097,0.104,0.109,0.115,0.122,0.130,0.137,0.145,0.155,0.166,0.180,0.196,0.215,0.234,0.249,0.258,0.261,0.257,0.248,0.238,0.231,0.229,0.233,0.241,0.251,0.264,0.276,0.289,0.300,0.312,0.327,0.347,0.367,0.389,0.408,0.430,0.455,0.493,0.537,0.572,0.621,0.700,0.690,0.540,0.489,0.464,0.454,0.458,0.471,0.487,0.518,0.579,0.818,0.676,0.624,0.612,0.625,0.651,0.695,0.800,0.714,0.682,0.700,0.939,0.834,0.810,0.801,0.826,0.877,0.811,0.718}
		Independence_Ambient[][1]={20,21,23,24,26,27,29,31,33,35,38,40,43,46,49,52,55,59,63,67,71,76,81,86,92,98,104,111,118,126,134,143,152,162,173,184,196,209,223,237,253,269,287,306,326,347,370,394,420,447,477,508,541,577,614,655,698,743,792,844,899,958,1021,1087,1159,1234,1315,1401,1493,1591,1695,1806,1924,2050,2185,2328,2480,2643,2816,3000}
		Independence_Ambient[][2]={0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.001,0.001,0.001,0.001,0.001,0.002,0.004,0.006,0.009,0.019,0.059,8.998,0.132,0.041,0.025,0.019,0.020,0.024,0.030,0.049,0.126,0.276,0.030,0.016,0.014,0.018,0.027,0.058,0.390,9.767,3.297,5.165,5.411,0.385,1.408,0.185,0.409,1.747,304.591,10.830}
		SetDimLabel 1, 0, Index, Independence_Ambient
		SetDimLabel 1, 1, Hz, Independence_Ambient
		SetDimLabel 1, 2, Sigma, Independence_Ambient
	endif
	//////////////  Ambient Phase inversions    //////////////////////////
	wave/Z PhaseInverts_Ambient = $SpHrView_df_signals+"PhaseInverts_Ambient"
	if(!waveexists(PhaseInverts_Ambient))
		Make/O/N=(80,2) PhaseInverts_Ambient
		PhaseInverts_Ambient[][0]={0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,1.000,1.000,1.000,0.000,0.000,0.000,0.000,0.000,0.000,1.000,1.000}
		PhaseInverts_Ambient[][1]={20,21,23,24,26,27,29,31,33,35,38,40,43,46,49,52,55,59,63,67,71,76,81,86,92,98,104,111,118,126,134,143,152,162,173,184,196,209,223,237,253,269,287,306,326,347,370,394,420,447,477,508,541,577,614,655,698,743,792,844,899,958,1021,1087,1159,1234,1315,1401,1493,1591,1695,1806,1924,2050,2185,2328,2480,2643,2816,3000}
		SetDimLabel 1, 0, Invert, PhaseInverts_Ambient
		SetDimLabel 1, 1, Hz, PhaseInverts_Ambient
	endif
	
	
	SetDataFolder dfSav
End


Function SpHrView_SignalPresets(String PresetString)
	// kill data folder to re-set defaults
	int tabVal
	DoWindow SpHrView_BinauralSignal_Window
 	if(V_Flag)
		ControlInfo/W=SpHrView_BinauralSignal_Window tab_signal
		tabVal = V_Value
	else
		tabVal = 1
	endif
	KillWindow/Z SpHrView_BinauralSignal_Window
	KillDataFolder $SpHrView_df_signalGlobals
	SpHrView_SignalDefaults()
	SVAR PresetStr=$SpHrView_df_signalGlobals+"PresetStr"
	PresetStr = PresetString
	strSwitch(PresetStr)
		case "Lead Lag pair, ±.25 ms ITD, 2.5 ms Lag":
			NVAR makeNoise1 = $SpHrView_df_signalGlobals+"makeNoise1"
			makeNoise1 = 1
			NVAR makeNoise2 = $SpHrView_df_signalGlobals+"makeNoise2"
			makeNoise2 = 2
			wave Durations = $SpHrView_df_signalGlobals+"Durations"
			Durations[0] = 5
			Durations[1,2] = 3
			wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
			ITD = "ITD (ramped phase shift)"
			wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
			ITDVal[1] = 0.25
			ITDVal[2] = -0.25
			wave OnsetDelays_ms=$SpHrView_df_signalGlobals+"OnsetDelays_ms"
			OnsetDelays_ms[1] = 0
			OnsetDelays_ms[2] = 2.5		
			break
			
		case "+0.25 ms ITD":
			wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
			ITDVal[1] = 0.25
			break
			
		case "Head related impulse response (HRIR)":
			wave/T HRIR_DataSet=$SpHrView_df_signalGlobals+"HRIR_DataSet"
			HRIR_DataSet[0] = "No HRIR"
			HRIR_DataSet[1] = "2° - NeumannKU100"
			HRIR_DataSet[2] = "2° - NeumannKU100"
			HRIR_DataSet[3] = "2° - NeumannKU100"
			break
			
		case "Binaural room impulse response (BRIR)":
			wave/T BRIR_Location=$SpHrView_df_signalGlobals+"BRIR_Location"
			BRIR_Location[0] = "No BRIR"
			BRIR_Location[1] = "Control Room 7 - NeumannKU100"
			BRIR_Location[2] = "Control Room 7 - NeumannKU100"
			BRIR_Location[3] = "Control Room 7 - NeumannKU100"
			wave/T BRIR_Source=$SpHrView_df_signalGlobals+"BRIR_Source"
			BRIR_Source[0] = ""
			BRIR_Source[1] = "Left - B&W803D"
			BRIR_Source[2] = "Right - B&W803D"
			BRIR_Source[3] = "Left - B&W803D"
			wave/T BRIR_Option=$SpHrView_df_signalGlobals+"BRIR_Option"
			BRIR_Option[0] = ""
			BRIR_Option[1] = "Microphone stand"
			BRIR_Option[2] = "Microphone stand"
			BRIR_Option[3] = "Microphone stand"
			wave BRIR_Az = $SpHrView_df_signalGlobals+"BRIR_Az"
			BRIR_Az[0] = 0
			BRIR_Az[1] = 0
			BRIR_Az[2] = 0
			BRIR_Az[3] = 0
			wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
			OffsetBuffers_sec[0] = 0
			OffsetBuffers_sec[1] = 2
			OffsetBuffers_sec[2] = 2
			OffsetBuffers_sec[3] = 2
			break
	  
		default:
			break
	endSwitch
	SpHrView_BinauralSignalWindow(tabVal)
End

// Add list of waves in SpHrView_df_signals
function/T SpHrView_SignalWaveList()
	// wavelist
	DFREF dfSav = GetDataFolderDFR()	
	SetDataFolder $SpHrView_df_signals
	String WaveSelectStr = WaveList("*",";","")
	WaveSelectStr = removeFromList("ILD_0", WaveSelectStr)
	WaveSelectStr = removeFromList("IPD_0", WaveSelectStr)
	WaveSelectStr = removeFromList("ITD_0", WaveSelectStr)
	WaveSelectStr = removeFromList("PhaseInversion_0", WaveSelectStr)
	WaveSelectStr = removeFromList("Independence_0", WaveSelectStr)
	WaveSelectStr = removeFromList("Spectrum_0", WaveSelectStr)
	WaveSelectStr = removeFromList("Spectrum2_0", WaveSelectStr)
	WaveSelectStr = removeFromList("ILD_1", WaveSelectStr)
	WaveSelectStr = removeFromList("IPD_1", WaveSelectStr)
	WaveSelectStr = removeFromList("ITD_1", WaveSelectStr)
	WaveSelectStr = removeFromList("PhaseInversion_1", WaveSelectStr)
	WaveSelectStr = removeFromList("Independence_1", WaveSelectStr)
	WaveSelectStr = removeFromList("Spectrum_1", WaveSelectStr)
	WaveSelectStr = removeFromList("Spectrum2_1", WaveSelectStr)
	WaveSelectStr = removeFromList("ILD_2", WaveSelectStr)
	WaveSelectStr = removeFromList("IPD_2", WaveSelectStr)
	WaveSelectStr = removeFromList("ITD_2", WaveSelectStr)
	WaveSelectStr = removeFromList("PhaseInversion_2", WaveSelectStr)
	WaveSelectStr = removeFromList("Independence_2", WaveSelectStr)
	WaveSelectStr = removeFromList("Spectrum_2", WaveSelectStr)
	WaveSelectStr = removeFromList("Spectrum2_2", WaveSelectStr)
	WaveSelectStr = removeFromList("PhaseInversion_3", WaveSelectStr)
	WaveSelectStr = removeFromList("ILD_3", WaveSelectStr)
	WaveSelectStr = removeFromList("IPD_3", WaveSelectStr)
	WaveSelectStr = removeFromList("ITD_3", WaveSelectStr)
	SetDataFolder dfSav
	return WaveSelectStr
End

// =================================================================================================
// SpHrView_BinauralSignalWindow(variable tabVal)
// General binaural signal controls
// call the following to fill in specific signal controls:
// SpHrView_BinauralSignalWindow_NoiseAmb()
// SpHrView_BinauralSignalWindow_Noise1()
// SpHrView_BinauralSignalWindow_Noise2()
// use SpHrView_BinauralSignalWindowTabSelect(tab) to select controls for specific signals
// SpHrView_BinauralSignalWindowColors(index) is called to shade SetVariable controls.
// =================================================================================================
Function SpHrView_BinauralSignalWindow(variable tabVal)
	if(tabVal<0)
		DoWindow SpHrView_BinauralSignal_Window
	 	if(V_Flag)
			ControlInfo/W=SpHrView_BinauralSignal_Window tab_signal
			tabVal = V_Value
		else
			tabVal = 1
		endif
	endif
	
	SpHrView_SignalDefaults()
	SpHrView_ToneDefaults()
	
	// value checks
	if(!DataFolderExists("root:SpHrView:Transfer:ImpulseWaves"))
		wave/T BRIR_Option = $SpHrView_df_signalGlobals+"BRIR_Option"
		wave/T BRIR_Location = $SpHrView_df_signalGlobals+"BRIR_Location"
		if(stringmatch(BRIR_Location[1], "Select BRIR wave"))
			BRIR_Location[0] = "No BRIR"
			BRIR_Option[0] = ""
		endif
		if(stringmatch(BRIR_Location[1], "Select BRIR wave"))
			BRIR_Location[1] = "No BRIR"
			BRIR_Option[1] = ""
		endif
		if(stringmatch(BRIR_Location[2], "Select BRIR wave"))
			BRIR_Location[2] = "No BRIR"
			BRIR_Option[2] = ""
		endif
		if(stringmatch(BRIR_Location[3], "Select BRIR wave"))
			BRIR_Location[3] = "No BRIR"
			BRIR_Option[3] = ""
		endif
	else
		wave BRIR_Az = $SpHrView_df_signalGlobals+"BRIR_Az"
		BRIR_Az = (numtype(BRIR_Az) == 2) ? 0 : BRIR_Az 
	endif
	
	
		
	NVAR View = $SpHrView_df_signalGlobals+"View"
	SVAR WavesName = $SpHrView_df_signalGlobals+"WavesName"
	NVAR makeAmbient = $SpHrView_df_signalGlobals+"makeAmbient"
	NVAR makeNoise1 = $SpHrView_df_signalGlobals+"makeNoise1"
	NVAR makeNoise2 = $SpHrView_df_signalGlobals+"makeNoise2"
	NVAR same12NoiseSeeds = $SpHrView_df_signalGlobals+"same12NoiseSeeds"
	NVAR makeTones = $SpHrView_df_signalGlobals+"makeTones"
	NVAR PrintCommands = $SpHrView_df_signalGlobals+"PrintCommands"
	NVAR makeDFTwaves = $SpHrView_df_signalGlobals+"makeDFTwaves"
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	variable winHeight = 900
	variable winWidth = 293
	KillWindow/Z SpHrView_BinauralSignal_Window
	NewPanel /K=1 /W=(10,50,10+winWidth,50+winHeight) as "Binaural Signal"
	DoWindow/C SpHrView_BinauralSignal_Window
	variable top = 7
	Button b_play,pos={85.00,top},size={80.00,22},proc=SpHrView_Signal_ButtonProc,title="Play"
	Button b_play,fSize=12
	Button b_make,pos={185.00,top},size={80.00,22},proc=SpHrView_Signal_ButtonProc,title="Make"
	Button b_make,fSize=12
	CheckBox ck_SpHrView,pos={13.00,top+4},size={40.00,30},proc=SpHrView_Signal_CheckProc,title="View"
	CheckBox ck_SpHrView,fSize=10,variable= View
	top += 37
	GroupBox gb_wave,pos={6.00,top-6},size={280.00,55.00},fSize=10,fStyle=1
	SetVariable sv_WaveName,pos={13.00,top},size={268.00,16.00},bodyWidth=205,proc=SpHrView_Signal_SetVarProc,title="\\f01Destination",fstyle=0
	SetVariable sv_WaveName,help={"Name of generated wave in datafolder 'Waves'"}
	SetVariable sv_WaveName,fSize=10,value= WavesName
	top += 23
	CheckBox ck_makeDFTwaves,pos={13.00,top+0},size={40.00,30},proc=SpHrView_Signal_CheckProc,title="save DFTs"
	CheckBox ck_makeDFTwaves,fSize=10,variable= makeDFTwaves
	SetVariable sv_samplingRate,pos={100.00,top},size={157.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Sampling Rate (Hz)"
	SetVariable sv_samplingRate,fSize=10
	SetVariable sv_samplingRate,limits={22050,192000,0},value= fs
	Button b_sampleRatePopUp,pos={260.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_sampleRatePopUp,picture= ProcGlobal#SpHrView_pict_PopUpList
	
	top += 35
	GroupBox gb_preset,pos={6.00,top-7},size={280.00,30.00},title=""
	SetVariable sv_Preset pos={15.00,top},size={242,16.00},bodyWidth=205,proc=SpHrView_Signal_SetVarProc,title="\\f01Preset"
	SetVariable sv_Preset fSize=10, fstyle=0,noedit=1,variable= $SpHrView_df_signalGlobals+"PresetStr"

	Button b_PresetPopUp,pos={260.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_PresetPopUp,picture= ProcGlobal#SpHrView_pict_PopUpList
	
	top += 52
	GroupBox gb_signals,pos={6.00,top-25},size={280.00,70.00},title="Signals to be generated"
	GroupBox gb_signals,fSize=10,fStyle=1
	CheckBox ck_signalAmbient,pos={20.00,top},size={57.00,16.00},proc=SpHrView_Signal_CheckProc,title="Ambient"
	CheckBox ck_signalAmbient,fSize=10,fstyle=1
	CheckBox ck_signalAmbient,variable= makeAmbient
	CheckBox ck_noise_Noise1,pos={94.00,top},size={52.00,16.00},proc=SpHrView_Signal_CheckProc,title="Noise 1"
	CheckBox ck_noise_Noise1,fSize=10,fstyle=1
	CheckBox ck_noise_Noise1,variable= makeNoise1
	CheckBox ck_noise_Noise2,pos={157.00,top},size={53.00,16.00},proc=SpHrView_Signal_CheckProc,title="Noise 2"
	CheckBox ck_noise_Noise2,fSize=10,fstyle=1
	CheckBox ck_noise_Noise2,variable= makeNoise2
	CheckBox ck_tones,pos={228.00,top},size={47.00,16.00},proc=SpHrView_Signal_CheckProc,title="Tones"
	CheckBox ck_tones,fSize=10,fstyle=1,variable= makeTones
	top += 23
	CheckBox ck_noise_Noise2SameAsNoise1,pos={50,top},size={53.00,16.00},proc=SpHrView_Signal_CheckProc,title="Use same seeds for Noises 1 & 2"
	CheckBox ck_noise_Noise2SameAsNoise1,fSize=10,fstyle=0
	CheckBox ck_noise_Noise2SameAsNoise1,variable= same12NoiseSeeds
	if(makeNoise1 && makeNoise2)
		CheckBox ck_noise_Noise2SameAsNoise1, disable = 0
	else
		CheckBox ck_noise_Noise2SameAsNoise1, disable = 2
	endif
	
	top += 30
	TabControl tab_signal,pos={6.00,top},size={280.00,20.00},proc=SpHrView_Signal_TabProc, fstyle=1
	TabControl tab_signal,tabLabel(0)=" Ambient",tabLabel(1)="Noise 1 "
	TabControl tab_signal,tabLabel(2)="Noise 2",tabLabel(3)="Tones",value= tabVal
	
	
	// bottom checks
	CheckBox ck_printCmds,pos={20.00,winHeight-23},size={135.00,16.00},proc=SpHrView_Signal_CheckProc,title="Print commands"
	CheckBox ck_printCmds,fSize=9
	CheckBox ck_printCmds,variable= PrintCommands
	
	CheckBox ck_keepWindowOpen title="📌 ",fSize=11,pos={240,winHeight-23},side=0,variable=$SpHrView_df_signalGlobals+"KeepWindowOpen"
	CheckBox ck_keepWindowOpen help={"Keep window open"}
	
	SpHrView_BinauralSignalWindow_NoiseAmb()
	SpHrView_BinauralSignalWindow_Noise1()
	SpHrView_BinauralSignalWindow_Noise2()
	SpHrView_BinauralSignalWindow_Tones()

	SpHrView_BinauralSignalWindowTabSelect(tabVal)
	
	SpHrView_BinauralSignalWindowColors(tabVal)
	
	DoUpdate/W=SpHrView_BinauralSignal_Window // for Igor 9
	SetWindow SpHrView_BinauralSignal_Window, hook(BinauralSignalWindowHook)=SpHrView_BinauralSignalWindowHook
	SetWindow SpHrView_BinauralSignal_Window note="PACKAGE:SpHrView;GROUP:BinauralSignal;"
End

Function SpHrView_BinauralSignalWindowHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0	
	switch(s.eventCode)
		//////////////
		// deactivate
		case 1:
			strswitch(s.winName)
				case "SpHrView_BinauralSignal_Window":
					//ControlInfo/W=SpHrView_BinauralSignal_Window ck_keepWindowOpen
					//if(!V_Value)
					NVAR KeepWindowOpen = $SpHrView_df_signalGlobals+"KeepWindowOpen"
					if(!KeepWindowOpen)
						Execute/Z/Q/P "killwindow/Z SpHrView_BinauralSignal_Window"
						hookResult=1
					endif
					break
			endswitch
			break
	endswitch
End

Function SpHrView_BinauralSignalWindow_NoiseAmb()
	variable SigInd = 0
	wave Durations = $SpHrView_df_signalGlobals+"Durations"
	wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
	wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
	wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
	wave OnsetBuffers_sec = $SpHrView_df_signalGlobals+"OnsetBuffers_sec"
	wave OnsetDelays_ms = $SpHrView_df_signalGlobals+"OnsetDelays_ms"
	wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
	wave OffsetDelays_ms = $SpHrView_df_signalGlobals+"OffsetDelays_ms"
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
	wave HighPassFreqs = $SpHrView_df_signalGlobals+"HighPassFreqs"
	wave HighPassOrders = $SpHrView_df_signalGlobals+"HighPassOrders"
	wave LowPassFreqs = $SpHrView_df_signalGlobals+"LowPassFreqs"
	wave LowPassOrders = $SpHrView_df_signalGlobals+"LowPassOrders"
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	
	DoWindow/F SpHrView_BinauralSignal_Window
	variable top = 230.00
	GroupBox gb_durationsAmplitudes_NoiseAmb,pos={6.00,top},size={280.00,55.00}
	top += 10
	SetVariable sv_duration_NoiseAmb,pos={32.00,top},size={113.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Duration"
	SetVariable sv_duration_NoiseAmb,fSize=10,fStyle=0
	SetVariable sv_duration_NoiseAmb,limits={0,1e+10,0.1},value= Durations[SigInd]
	SetVariable sv_durationUnits_NoiseAmb,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_durationUnits_NoiseAmb,fSize=10
	SetVariable sv_durationUnits_NoiseAmb,value= DurationUnits[SigInd],noedit= 1
	Button b_durationUnits_NoiseAmb,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_durationUnits_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 21
	SetVariable sv_amplitude_NoiseAmb,pos={23.00,top},size={122.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Amplitude"
	SetVariable sv_amplitude_NoiseAmb,fSize=10
	SetVariable sv_amplitude_NoiseAmb,limits={0,120,6},value= Amplitudes[SigInd]
	SetVariable sv_amplitudeUnits_NoiseAmb,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_amplitudeUnits_NoiseAmb,fSize=10
	SetVariable sv_amplitudeUnits_NoiseAmb,value= AmplitudeUnits[SigInd],noedit= 1
	Button b_amplitudeUnits_NoiseAmb,pos={258.00,top-2},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_amplitudeUnits_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 30
	GroupBox gb_ramps_NoiseAmb,pos={6.00,top-6},size={280.00,30.00},fSize=10
	GroupBox gb_ramps_NoiseAmb,fStyle=1
	SetVariable sv_onRamp_NoiseAmb,pos={29.00,top},size={126.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Ramp:  On \\f00(ms)"
	SetVariable sv_onRamp_NoiseAmb,fSize=10
	SetVariable sv_onRamp_NoiseAmb,limits={0,1000,1},value= OnRamps_ms[SigInd]
	SetVariable sv_offRamp_NoiseAmb,pos={172.00,top},size={90.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Off \\f00(ms)"
	SetVariable sv_offRamp_NoiseAmb,fSize=10
	SetVariable sv_offRamp_NoiseAmb,limits={0,1000,1},value= OffRamps_ms[SigInd]
	top += 50
	// not allowing HRIRs to be applied to Ambient signals
	GroupBox gb_HRIR_NoiseAmb,pos={6.00,top-21},size={280.00,43.00},title="Head related impulse response (HRIR)"
	GroupBox gb_HRIR_NoiseAmb,fSize=10,fStyle=1,disable=2
	// not allowing BRIRs to be applied to Ambient signals
	top += 43
	GroupBox gb_BRIR_NoiseAmb,pos={6.00,top-21},size={280.00,65.00},title="Binarual Room impulse response (BRIR)"
	GroupBox gb_BRIR_NoiseAmb,fSize=10,fStyle=1,disable=2
	top += 50
	GroupBox gb_Filters_NoiseAmb,pos={6.00,top-5},size={280.00,27.00},title=""
	GroupBox gb_Filters_NoiseAmb,fSize=10,fStyle=1
	top += 48
	GroupBox gb_buffersDelays_NoiseAmb,pos={6.00,top-19},size={280.00,62.00},title="Timing"
	GroupBox gb_buffersDelays_NoiseAmb,fSize=10,fStyle=1,disable=2
	top += 20
	// also not allowing buffers to be applied to Ambient signals
//	SetVariable sv_onsetBuffer_NoiseAmb,pos={15.00,top},size={151.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Onset:\\f00  Buffer (sec)"
//	SetVariable sv_onsetBuffer_NoiseAmb,fSize=10,disable=2
//	SetVariable sv_onsetBuffer_NoiseAmb,limits={0,120,0.1},value= OnsetBuffers_sec[SigInd]
//	SetVariable sv_onsetDelay_NoiseAmb,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Delay\\f00 (ms)"
//	SetVariable sv_onsetDelay_NoiseAmb,fSize=10,disable=2
//	SetVariable sv_onsetDelay_NoiseAmb,limits={0,1000,0.05},value= OnsetDelays_ms[SigInd]
//	top += 35
//	SetVariable sv_offsetBuffer_NoiseAmb,pos={13.00,top},size={153.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Offset:  Buffer (sec)"
//	SetVariable sv_offsetBuffer_NoiseAmb,fSize=10,disable=2
//	SetVariable sv_offsetBuffer_NoiseAmb,limits={0,120,0.1},value= OffsetBuffers_sec[SigInd]
//	SetVariable sv_offsetDelay_NoiseAmb,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Delay (ms)"
//	SetVariable sv_offsetDelay_NoiseAmb,fSize=10,disable=2
//	SetVariable sv_offsetDelay_NoiseAmb,limits={0,1000,0.05},value= OffsetDelays_ms[SigInd]
	top += 48
	GroupBox gb_spectrum_NoiseAmb,pos={6.00,top-22},size={280.00,95.00},title="Spectrum"
	GroupBox gb_spectrum_NoiseAmb,fSize=10,fStyle=1
	SetVariable sv_Spectrum_NoiseAmb,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_Spectrum_NoiseAmb,fSize=10
	SetVariable sv_Spectrum_NoiseAmb,value= SpectrumShape[SigInd],noedit= 1
	Button b_spectrumPopup_NoiseAmb,pos={161.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_spectrumPopup_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_SpectrumViewEdit_NoiseAmb,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_SpectrumViewEdit_NoiseAmb,fSize=10
	top += 24
	SetVariable sv_highPassFreq_NoiseAmb,pos={17.00,top},size={187.00,16.00},bodyWidth=55,proc=SpHrView_Signal_SetVarProc,title="Highpass corner (0dB, Hz)"
	SetVariable sv_highPassFreq_NoiseAmb,help={"High pass corner (-0dB) frequency"}
	SetVariable sv_highPassFreq_NoiseAmb,fSize=10
	SetVariable sv_highPassFreq_NoiseAmb,limits={0,fs/2,10},value= HighPassFreqs[SigInd]
	SetVariable sv_highPassOrder_NoiseAmb,pos={211.00,top},size={67.00,16.00},bodyWidth=35,proc=SpHrView_Signal_SetVarProc,title="Order"
	SetVariable sv_highPassOrder_NoiseAmb,help={"Decline per octave = order * -6 dB. Pass 0 for brick-wall filter."}
	SetVariable sv_highPassOrder_NoiseAmb,fSize=10
	SetVariable sv_highPassOrder_NoiseAmb,limits={0,20,1},value= HighPassOrders[SigInd]
	top += 20
	SetVariable sv_lowPassFreq_NoiseAmb,pos={19.00,top},size={185.00,16.00},bodyWidth=55,proc=SpHrView_Signal_SetVarProc,title="Lowpass corner (0dB, Hz)"
	SetVariable sv_lowPassFreq_NoiseAmb,help={"High pass corner (-0dB) frequency"}
	SetVariable sv_lowPassFreq_NoiseAmb,fSize=10
	SetVariable sv_lowPassFreq_NoiseAmb,limits={0,fs/2,10},value= LowPassFreqs[SigInd]
	SetVariable sv_lowPassOrder_NoiseAmb,pos={210.00,top},size={67.00,16.00},bodyWidth=35,proc=SpHrView_Signal_SetVarProc,title="Order"
	SetVariable sv_lowPassOrder_NoiseAmb,help={"Decline per octave = order * -6 dB. Pass 0 for brick-wall filter."}
	SetVariable sv_lowPassOrder_NoiseAmb,fSize=10
	SetVariable sv_lowPassOrder_NoiseAmb,limits={0,20,1},value= LowPassOrders[SigInd]
	top += 54
	GroupBox gb_independence_NoiseAmb,pos={6.00,top-21},size={280.00,43.00},title="Binaural Independence                 Index"
	GroupBox gb_independence_NoiseAmb,fSize=10,fStyle=1
	SetVariable sv_Independence_NoiseAmb,pos={11.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_Independence_NoiseAmb,fSize=10
	SetVariable sv_Independence_NoiseAmb,value= Independence[SigInd],noedit= 1
	Button b_independencePopup_NoiseAmb,pos={162.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_independencePopup_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IndependenceVal_NoiseAmb,pos={182.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IndependenceVal_NoiseAmb,help={"Binaural independence. Option-click = 0.0"}
	SetVariable sv_IndependenceVal_NoiseAmb,fSize=10,format="%.2g"
	SetVariable sv_IndependenceVal_NoiseAmb,limits={0,1,0.05},value= IndependenceVal[SigInd]
	Button b_IndependenceViewEdit_NoiseAmb,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IndependenceViewEdit_NoiseAmb,fSize=10
	top += 44
	GroupBox gb_phaseInversion_NoiseAmb,pos={6.00,top-21},size={280.00,43.00},title="Phase inversion"
	GroupBox gb_phaseInversion_NoiseAmb,fSize=10,fStyle=1
	SetVariable sv_PhaseInversion_NoiseAmb,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_PhaseInversion_NoiseAmb,fSize=10,proc=SpHrView_Signal_SetVarProc
	SetVariable sv_PhaseInversion_NoiseAmb,value= PhaseInversion[SigInd]//,noedit= 1
	Button b_PhaseInversionPopup_NoiseAmb,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_PhaseInversionPopup_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_PhaseInversionViewEdit_NoiseAmb,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_PhaseInversionViewEdit_NoiseAmb,fSize=10
	top += 28
	GroupBox gb_InterauralDifferences_NoiseAmb,pos={6.00,top},size={280.00,145.00},title="\\f01Interaural Differences"
	GroupBox gb_InterauralDifferences_NoiseAmb,fSize=10,fStyle=0
	top += 37
	TitleBox tb_ITD_NoiseAmb,pos={21.00,top-15},size={204.00,13.00},title="\\f01Time                                                  ITD (ms)"
	TitleBox tb_ITD_NoiseAmb,fSize=10,frame=0
	SetVariable sv_ITD_NoiseAmb,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ITD_NoiseAmb,fSize=10
	SetVariable sv_ITD_NoiseAmb,value= ITD[SigInd],noedit= 1
	Button b_ITDPopup_NoiseAmb,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ITDPopup_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ITDVal_NoiseAmb,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ITDVal_NoiseAmb,help={"Interaural Time Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ITDVal_NoiseAmb,fSize=10,format="%.3g"
	SetVariable sv_ITDVal_NoiseAmb,limits={-9999,9999,0.05},value= ITDVal[SigInd]
	Button b_ITDViewEdit_NoiseAmb,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ITDViewEdit_NoiseAmb,fSize=10
	top += 38
	TitleBox tb_IPD_NoiseAmb,pos={21.00,top-15},size={220.00,13.00},title="\\f01Phase                                               IPD (cycles)"
	TitleBox tb_IPD_NoiseAmb,fSize=10,frame=0
	SetVariable sv_IPD_NoiseAmb,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_IPD_NoiseAmb,fSize=10
	SetVariable sv_IPD_NoiseAmb,value= IPD[SigInd],noedit= 1
	Button b_IPDPopup_NoiseAmb,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_IPDPopup_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IPDVal_NoiseAmb,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IPDVal_NoiseAmb,help={"Interaural Phase Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_IPDVal_NoiseAmb,fSize=10,format="%.3g"
	SetVariable sv_IPDVal_NoiseAmb,limits={-2,2,0.01},value= IPDVal[SigInd]
	Button b_IPDViewEdit_NoiseAmb,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IPDViewEdit_NoiseAmb,fSize=10
	top += 38
	TitleBox tb_ILD_NoiseAmb,pos={21.00,top-15},size={201.00,13.00},title="\\f01Level                                                 ILD (dB)"
	TitleBox tb_ILD_NoiseAmb,fSize=10,frame=0
	SetVariable sv_ILD_NoiseAmb,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ILD_NoiseAmb,fSize=10
	SetVariable sv_ILD_NoiseAmb,value= ILD[SigInd],noedit= 1
	Button b_ILDPopup_NoiseAmb,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ILDPopup_NoiseAmb,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ILDVal_NoiseAmb,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ILDVal_NoiseAmb,help={"Interaural Level Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ILDVal_NoiseAmb,fSize=10,format="%.3g"
	SetVariable sv_ILDVal_NoiseAmb,limits={-60,60,3},value= ILDVal[SigInd],live= 1
	Button b_ILDViewEdit_NoiseAmb,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ILDViewEdit_NoiseAmb,fSize=10
	
	if(stringmatch(PhaseInversion[SigInd], "PhaseInverts_Ambient"))
		SetVariable sv_PhaseInversion_NoiseAmb,valueBackColor=(49151,53155,65535,32768)
	endif
	
	strswitch(Independence[SigInd])
		case "Independence_Ambient":
			SetVariable sv_Independence_NoiseAmb,valueBackColor=(49151,53155,65535,32768)
			break
		case "Index":
			SetVariable sv_IndependenceVal_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable=2
			Button b_IndependenceViewEdit_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable=2
			break
		default:
			SetVariable sv_IndependenceVal_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable=0
			Button b_IndependenceViewEdit_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable=0
			break
	endswitch
	
End

Function SpHrView_BinauralSignalWindow_Noise1()
	variable SigInd = 1
	wave Durations = $SpHrView_df_signalGlobals+"Durations"
	wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
	wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
	wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
	wave OnsetBuffers_sec = $SpHrView_df_signalGlobals+"OnsetBuffers_sec"
	wave OnsetDelays_ms = $SpHrView_df_signalGlobals+"OnsetDelays_ms"
	wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
	wave OffsetDelays_ms = $SpHrView_df_signalGlobals+"OffsetDelays_ms"
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
	wave HighPassFreqs = $SpHrView_df_signalGlobals+"HighPassFreqs"
	wave HighPassOrders = $SpHrView_df_signalGlobals+"HighPassOrders"
	wave LowPassFreqs = $SpHrView_df_signalGlobals+"LowPassFreqs"
	wave LowPassOrders = $SpHrView_df_signalGlobals+"LowPassOrders"
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	wave/T HRIR_DataSet = $SpHrView_df_signalGlobals+"HRIR_DataSet"
	wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
	wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
	wave HRIR_useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC"
	wave/T HRIR_HPCFStr = $SpHrView_df_signalGlobals+"HRIR_HPCFStr"
	wave/T BRIR_Location = $SpHrView_df_signalGlobals+"BRIR_Location"
	wave BRIR_Az = $SpHrView_df_signalGlobals+"BRIR_Az"
	wave/T BRIR_Source = $SpHrView_df_signalGlobals+"BRIR_Source"
	wave/T BRIR_Option = $SpHrView_df_signalGlobals+"BRIR_Option"
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	
	DoWindow/F SpHrView_BinauralSignal_Window
	variable top = 230.00
	GroupBox gb_durationsAmplitudes_Noise1,pos={6.00,top},size={280.00,55.00}
	top += 10
	SetVariable sv_duration_Noise1,pos={32.00,top},size={113.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Duration"
	SetVariable sv_duration_Noise1,fSize=10,fStyle=0
	SetVariable sv_duration_Noise1,limits={0,1e+10,50},value= Durations[SigInd]
	SetVariable sv_durationUnits_Noise1,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_durationUnits_Noise1,fSize=10
	SetVariable sv_durationUnits_Noise1,value= DurationUnits[SigInd],noedit= 1
	Button b_durationUnits_Noise1,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_durationUnits_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 21
	SetVariable sv_amplitude_Noise1,pos={23.00,top},size={122.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Amplitude"
	SetVariable sv_amplitude_Noise1,fSize=10
	SetVariable sv_amplitude_Noise1,limits={0,120,6},value= Amplitudes[SigInd]
	SetVariable sv_amplitudeUnits_Noise1,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_amplitudeUnits_Noise1,fSize=10
	SetVariable sv_amplitudeUnits_Noise1,value= AmplitudeUnits[SigInd],noedit= 1
	Button b_amplitudeUnits_Noise1,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_amplitudeUnits_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 30
	GroupBox gb_ramps_Noise1,pos={6.00,top-6},size={280.00,30.00},fSize=10
	GroupBox gb_ramps_Noise1,fStyle=1
	SetVariable sv_onRamp_Noise1,pos={29.00,top},size={126.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Ramp:  On \\f00(ms)"
	SetVariable sv_onRamp_Noise1,fSize=10
	SetVariable sv_onRamp_Noise1,limits={0,1000,1},value= OnRamps_ms[SigInd]
	SetVariable sv_offRamp_Noise1,pos={172.00,top},size={90.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Off \\f00(ms)"
	SetVariable sv_offRamp_Noise1,fSize=10
	SetVariable sv_offRamp_Noise1,limits={0,1000,1},value= OffRamps_ms[SigInd]
	
	top += 50
	GroupBox gb_HRIR_Noise1,pos={6.00,top-21},size={280.00,43.00},title="Head related impulse response (HRIR)"
	GroupBox gb_HRIR_Noise1,fSize=10,fStyle=1
	SetVariable sv_HRIRDataSet_Noise1 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_HRIRDataSet_Noise1 fSize=10
	SetVariable sv_HRIRDataSet_Noise1 value= HRIR_DataSet[SigInd],noedit= 1
	Button b_HRIRDataSetPopup_Noise1,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_HRIRDataSetPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	variable azInc = 2, elInc = 2
	strswitch(HRIR_DataSet[SigInd])
		case "2° - NeumannKU100":
			azInc = 2
			elInc = 2
			break
		case "360° microphone stand - NeumannKU100":
			azInc = 1
			elInc = 1
			break
		case "360° full rotation mount - NeumannKU100":
			azInc = 1
			elInc = 1
			break
		case "Lebedev 2354 - NeumannKU100":
			azInc = 5 // ?
			elInc = 5 // ?
			break
		case "Lebedev 2702 - NeumannKU100":
			azInc = 5 // ?
			elInc = 5 // ?
			break
		default:
			break
	endswitch
	SetVariable sv_HRIR_Az_Noise1,pos={170.00,top},size={45.00,16.00},bodyWidth=45,proc=SpHrView_IR_SetVarProc,title="Az"
	SetVariable sv_HRIR_Az_Noise1,help={"Azimuth. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_HRIR_Az_Noise1,fSize=10,format="%.0f",limits={-180,180,azInc},value= HRIR_Az[SigInd],live= 1
	wave/T HRIR_DataSet=$SpHrView_df_signalGlobals+"HRIR_DataSet"
	SetVariable sv_HRIR_El_Noise1,pos={233.00,top},size={46.00,16.00},bodyWidth=46,proc=SpHrView_IR_SetVarProc,title="El"
	SetVariable sv_HRIR_El_Noise1,help={"Elevation. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_HRIR_El_Noise1,fSize=10,format="%.1f",limits={-90,90,elInc},value= HRIR_El[SigInd],live= 1
	wave/T HRIR_DataSet=$SpHrView_df_signalGlobals+"HRIR_DataSet"
	
	if(strlen(OperationList("SpHr_HRIR*", ";", "external")) < 2)
		HRIR_DataSet[SigInd] = "No HRIR"
		GroupBox gb_HRIR_Noise1 disable=2
		SetVariable sv_HRIRDataSet_Noise1 disable=2
		Button b_HRIRDataSetPopup_Noise1 disable=2
		SetVariable sv_HRIR_Az_Noise1 disable=2
		SetVariable sv_HRIR_El_Noise1 disable=2
	endif

	
	top += 43
	GroupBox gb_BRIR_Noise1,pos={6.00,top-21},size={280.00,65.00},title="Binarual Room impulse response (BRIR)"
	GroupBox gb_BRIR_Noise1,fSize=10,fStyle=1
	SetVariable sv_BRIR_Location_Noise1 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_BRIR_Location_Noise1 fSize=10
	SetVariable sv_BRIR_Location_Noise1 value= BRIR_Location[SigInd],noedit= 1
	Button b_BRIR_LocationPopup_Noise1,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_LocationPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_BRIR_Az_Noise1,pos={170.00,top},size={45.00,16.00},bodyWidth=45,proc=SpHrView_IR_SetVarProc,title="Az"
	SetVariable sv_BRIR_Az_Noise1,help={"Azimuth. Option-click: set to 0.0, Command-click: reverse sign"}
	//azInc = 1
	SetVariable sv_BRIR_Az_Noise1,fSize=10,format="%.0f",limits={-180,180,azInc},value= BRIR_Az[SigInd],live= 1
	top += 21
	SetVariable sv_BRIR_Option_Noise1 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_BRIR_Option_Noise1 fSize=10
	SetVariable sv_BRIR_Option_Noise1 value= BRIR_Option[SigInd],noedit= 1
	Button b_BRIR_OptionPopup_Noise1,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_OptionPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_BRIR_Source_Noise1 pos={155.00,top},size={110.00,16.00},bodyWidth=110,title=" "
	SetVariable sv_BRIR_Source_Noise1 fSize=10
	SetVariable sv_BRIR_Source_Noise1 value= BRIR_Source[SigInd],noedit= 1
	Button b_BRIR_SourcePopup_Noise1,pos={266.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_SourcePopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	
	if(strlen(OperationList("SpHr_BRIR*", ";", "external")) < 2)
		BRIR_Location[SigInd] = "No BRIR"
		GroupBox gb_BRIR_Noise1 disable=2
		SetVariable sv_BRIR_Location_Noise1 disable=2
		Button b_BRIR_LocationPopup_Noise1 disable=2
		SetVariable sv_BRIR_Az_Noise1 disable=2
		SetVariable sv_BRIR_Option_Noise1 disable=2
		Button b_BRIR_OptionPopup_Noise1 disable=2
		SetVariable sv_BRIR_Source_Noise1 disable=2
		Button b_BRIR_SourcePopup_Noise1 disable=2
	endif
	
	top += 29
	GroupBox gb_Filters_Noise1,pos={6.00,top-5},size={280.00,27.00},title=""
	GroupBox gb_Filters_Noise1,fSize=10,fStyle=1
	SetVariable sv_HRIR_HPCFStr_Noise1 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_HRIR_HPCFStr_Noise1 fSize=10, help={"Apply headphone compensation filter"}
	SetVariable sv_HRIR_HPCFStr_Noise1 value= HRIR_HPCFStr[SigInd],noedit= 1
	Button b_HRIR_HPCFStrPopup_Noise1,pos={131.00,top},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_HRIR_HPCFStrPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	CheckBox ck_HRIR_useCDFC_Noise1 pos={165.00,top+0},size={40.00,30},proc=SpHrView_Signal_CheckProc,title="Diffuse field filter"
	CheckBox ck_HRIR_useCDFC_Noise1 fSize=10, value= HRIR_useCDFC[SigInd], help={"Apply free-field compensation filter"}
	
	top += 48
	GroupBox gb_buffersDelays_Noise1,pos={6.00,top-19},size={280.00,62.00},title="Timing"
	GroupBox gb_buffersDelays_Noise1,fSize=10,fStyle=1
	SetVariable sv_onsetBuffer_Noise1,pos={15.00,top},size={151.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Onset:\\f00  Buffer (sec)"
	SetVariable sv_onsetBuffer_Noise1,fSize=10
	SetVariable sv_onsetBuffer_Noise1,limits={0,120,0.1},value= OnsetBuffers_sec[SigInd]
	SetVariable sv_onsetDelay_Noise1,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Delay\\f00 (ms)"
	SetVariable sv_onsetDelay_Noise1,fSize=10
	SetVariable sv_onsetDelay_Noise1,limits={0,1000,0.05},value= OnsetDelays_ms[SigInd]
	top += 20
	SetVariable sv_offsetBuffer_Noise1,pos={13.00,top},size={153.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Offset:  Buffer (sec)"
	SetVariable sv_offsetBuffer_Noise1,fSize=10
	SetVariable sv_offsetBuffer_Noise1,limits={0,120,0.10},value= OffsetBuffers_sec[SigInd]
	SetVariable sv_offsetDelay_Noise1,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Delay (ms)"
	SetVariable sv_offsetDelay_Noise1,fSize=10
	SetVariable sv_offsetDelay_Noise1,limits={0,1000,0.05},value= OffsetDelays_ms[SigInd]
	top += 48
	GroupBox gb_spectrum_Noise1,pos={6.00,top-22},size={280.00,95.00},title="Spectrum"
	GroupBox gb_spectrum_Noise1,fSize=10,fStyle=1
	SetVariable sv_Spectrum_Noise1,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_Spectrum_Noise1,fSize=10
	SetVariable sv_Spectrum_Noise1,value= SpectrumShape[SigInd],noedit= 1
	Button b_spectrumPopup_Noise1,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_spectrumPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_SpectrumViewEdit_Noise1,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_SpectrumViewEdit_Noise1,fSize=10
	top += 24
	SetVariable sv_highPassFreq_Noise1,pos={17.00,top},size={187.00,16.00},bodyWidth=55,proc=SpHrView_Signal_SetVarProc,title="Highpass corner (0dB, Hz)"
	SetVariable sv_highPassFreq_Noise1,help={"High pass corner (-0dB) frequency"}
	SetVariable sv_highPassFreq_Noise1,fSize=10
	SetVariable sv_highPassFreq_Noise1,limits={0,fs/2,10},value= HighPassFreqs[SigInd]
	SetVariable sv_highPassOrder_Noise1,pos={211.00,top},size={67.00,16.00},bodyWidth=35,proc=SpHrView_Signal_SetVarProc,title="Order"
	SetVariable sv_highPassOrder_Noise1,help={"Decline per octave = order * -6 dB. Pass 0 for brick-wall filter."}
	SetVariable sv_highPassOrder_Noise1,fSize=10
	SetVariable sv_highPassOrder_Noise1,limits={0,20,1},value= HighPassOrders[SigInd]
	top += 20
	SetVariable sv_lowPassFreq_Noise1,pos={19.00,top},size={185.00,16.00},bodyWidth=55,proc=SpHrView_Signal_SetVarProc,title="Lowpass corner (0dB, Hz)"
	SetVariable sv_lowPassFreq_Noise1,help={"High pass corner (-0dB) frequency"}
	SetVariable sv_lowPassFreq_Noise1,fSize=10
	SetVariable sv_lowPassFreq_Noise1,limits={0,fs/2,10},value= LowPassFreqs[SigInd]
	SetVariable sv_lowPassOrder_Noise1,pos={210.00,top},size={67.00,16.00},bodyWidth=35,proc=SpHrView_Signal_SetVarProc,title="Order"
	SetVariable sv_lowPassOrder_Noise1,help={"Decline per octave = order * -6 dB. Pass 0 for brick-wall filter."}
	SetVariable sv_lowPassOrder_Noise1,fSize=10
	SetVariable sv_lowPassOrder_Noise1,limits={0,20,1},value= LowPassOrders[SigInd]
		
	top += 54
	GroupBox gb_independence_Noise1,pos={6.00,top-21},size={280.00,43.00},title="Binaural Independence                 Index"
	GroupBox gb_independence_Noise1,fSize=10,fStyle=1
	SetVariable sv_Independence_Noise1,pos={11.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_Independence_Noise1,fSize=10
	SetVariable sv_Independence_Noise1,value= Independence[SigInd],noedit= 1
	Button b_independencePopup_Noise1,pos={162.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_independencePopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IndependenceVal_Noise1,pos={182.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IndependenceVal_Noise1,help={"Binaural independence. Option-click = 0.0"}
	SetVariable sv_IndependenceVal_Noise1,fSize=10,format="%.2g"
	SetVariable sv_IndependenceVal_Noise1,limits={0,1,0.05},value= IndependenceVal[SigInd]
	Button b_IndependenceViewEdit_Noise1,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IndependenceViewEdit_Noise1,fSize=10
	top += 44
	GroupBox gb_phaseInversion_Noise1,pos={6.00,top-21},size={280.00,43.00},title="Phase inversion"
	GroupBox gb_phaseInversion_Noise1,fSize=10,fStyle=1
	SetVariable sv_PhaseInversion_Noise1,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_PhaseInversion_Noise1,fSize=10,proc=SpHrView_Signal_SetVarProc
	SetVariable sv_PhaseInversion_Noise1,value= PhaseInversion[SigInd]//,noedit= 1
	Button b_PhaseInversionPopup_Noise1,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_PhaseInversionPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_PhaseInversionViewEdit_Noise1,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_PhaseInversionViewEdit_Noise1,fSize=10
	top += 28
	GroupBox gb_InterauralDifferences_Noise1,pos={6.00,top},size={280.00,145.00},title="\\f01Interaural Differences"
	GroupBox gb_InterauralDifferences_Noise1,fSize=10,fStyle=0
	top += 37
	TitleBox tb_ITD_Noise1,pos={21.00,top-15},size={204.00,13.00},title="\\f01Time                                                  ITD (ms)"
	TitleBox tb_ITD_Noise1,fSize=10,frame=0
	SetVariable sv_ITD_Noise1,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ITD_Noise1,fSize=10
	SetVariable sv_ITD_Noise1,value= ITD[SigInd],noedit= 1
	Button b_ITDPopup_Noise1,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ITDPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ITDVal_Noise1,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ITDVal_Noise1,help={"Interaural Time Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ITDVal_Noise1,fSize=10,format="%.3g"
	SetVariable sv_ITDVal_Noise1,limits={-9999,9999,0.05},value= ITDVal[SigInd]
	Button b_ITDViewEdit_Noise1,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ITDViewEdit_Noise1,fSize=10
	top += 38
	TitleBox tb_IPD_Noise1,pos={21.00,top-15},size={220.00,13.00},title="\\f01Phase                                               IPD (cycles)"
	TitleBox tb_IPD_Noise1,fSize=10,frame=0
	SetVariable sv_IPD_Noise1,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_IPD_Noise1,fSize=10
	SetVariable sv_IPD_Noise1,value= IPD[SigInd],noedit= 1
	Button b_IPDPopup_Noise1,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_IPDPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IPDVal_Noise1,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IPDVal_Noise1,help={"Interaural Phase Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_IPDVal_Noise1,fSize=10,format="%.3g"
	SetVariable sv_IPDVal_Noise1,limits={-2,2,0.01},value= IPDVal[SigInd]
	Button b_IPDViewEdit_Noise1,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IPDViewEdit_Noise1,fSize=10
	top += 38
	TitleBox tb_ILD_Noise1,pos={21.00,top-15},size={201.00,13.00},title="\\f01Level                                                 ILD (dB)"
	TitleBox tb_ILD_Noise1,fSize=10,frame=0
	SetVariable sv_ILD_Noise1,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ILD_Noise1,fSize=10
	SetVariable sv_ILD_Noise1,value= ILD[SigInd],noedit= 1
	Button b_ILDPopup_Noise1,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ILDPopup_Noise1,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ILDVal_Noise1,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ILDVal_Noise1,help={"Interaural Level Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ILDVal_Noise1,fSize=10,format="%.3g"
	SetVariable sv_ILDVal_Noise1,limits={-60,60,3},value= ILDVal[SigInd],live= 1
	Button b_ILDViewEdit_Noise1,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ILDViewEdit_Noise1,fSize=10
	if(stringmatch(PhaseInversion[SigInd], "PhaseInverts_Ambient"))
		SetVariable sv_PhaseInversion_Noise1,valueBackColor=(49151,53155,65535,32768)
	endif
	strswitch(Independence[SigInd])
		case "Independence_Ambient":
			SetVariable sv_Independence_Noise1,valueBackColor=(49151,53155,65535,32768)
			break
		case "Index":
			SetVariable sv_IndependenceVal_Noise1, win=SpHrView_BinauralSignal_Window, disable=2
			Button b_IndependenceViewEdit_Noise1, win=SpHrView_BinauralSignal_Window, disable=2
			break
		default:
			SetVariable sv_IndependenceVal_Noise1, win=SpHrView_BinauralSignal_Window, disable=0
			Button b_IndependenceViewEdit_Noise1, win=SpHrView_BinauralSignal_Window, disable=0
			break
	endswitch
End


Function SpHrView_BinauralSignalWindow_Noise2()
	variable SigInd = 2
	wave Durations = $SpHrView_df_signalGlobals+"Durations"
	wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
	wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
	wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
	wave OnsetBuffers_sec = $SpHrView_df_signalGlobals+"OnsetBuffers_sec"
	wave OnsetDelays_ms = $SpHrView_df_signalGlobals+"OnsetDelays_ms"
	wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
	wave OffsetDelays_ms = $SpHrView_df_signalGlobals+"OffsetDelays_ms"
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
	wave HighPassFreqs = $SpHrView_df_signalGlobals+"HighPassFreqs"
	wave HighPassOrders = $SpHrView_df_signalGlobals+"HighPassOrders"
	wave LowPassFreqs = $SpHrView_df_signalGlobals+"LowPassFreqs"
	wave LowPassOrders = $SpHrView_df_signalGlobals+"LowPassOrders"
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	wave/T HRIR_DataSet = $SpHrView_df_signalGlobals+"HRIR_DataSet"
	wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
	wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
	wave HRIR_useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC"
	wave/T HRIR_HPCFStr = $SpHrView_df_signalGlobals+"HRIR_HPCFStr"
	wave/T BRIR_Location = $SpHrView_df_signalGlobals+"BRIR_Location"
	wave BRIR_Az = $SpHrView_df_signalGlobals+"BRIR_Az"
	wave/T BRIR_Source = $SpHrView_df_signalGlobals+"BRIR_Source"
	wave/T BRIR_Option = $SpHrView_df_signalGlobals+"BRIR_Option"
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	
	DoWindow/F SpHrView_BinauralSignal_Window
	variable top = 230.00
	GroupBox gb_durationsAmplitudes_Noise2,pos={6.00,top},size={280.00,55.00}
	top += 10
	SetVariable sv_duration_Noise2,pos={32.00,top},size={113.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Duration"
	SetVariable sv_duration_Noise2,fSize=10,fStyle=0
	SetVariable sv_duration_Noise2,limits={0,1e+10,50},value= Durations[SigInd]
	SetVariable sv_durationUnits_Noise2,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_durationUnits_Noise2,fSize=10
	SetVariable sv_durationUnits_Noise2,value= DurationUnits[SigInd],noedit= 1
	Button b_durationUnits_Noise2,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_durationUnits_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 21
	SetVariable sv_amplitude_Noise2,pos={23.00,top},size={122.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Amplitude"
	SetVariable sv_amplitude_Noise2,fSize=10
	SetVariable sv_amplitude_Noise2,limits={0,120,6},value= Amplitudes[SigInd]
	SetVariable sv_amplitudeUnits_Noise2,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_amplitudeUnits_Noise2,fSize=10
	SetVariable sv_amplitudeUnits_Noise2,value= AmplitudeUnits[SigInd],noedit= 1
	Button b_amplitudeUnits_Noise2,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_amplitudeUnits_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 30
	GroupBox gb_ramps_Noise2,pos={6.00,top-6},size={280.00,30.00},fSize=10
	GroupBox gb_ramps_Noise2,fStyle=1
	SetVariable sv_onRamp_Noise2,pos={29.00,top},size={126.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Ramp:  On \\f00(ms)"
	SetVariable sv_onRamp_Noise2,fSize=10
	SetVariable sv_onRamp_Noise2,limits={0,1000,1},value= OnRamps_ms[SigInd]
	SetVariable sv_offRamp_Noise2,pos={172.00,top},size={90.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Off \\f00(ms)"
	SetVariable sv_offRamp_Noise2,fSize=10
	SetVariable sv_offRamp_Noise2,limits={0,1000,1},value= OffRamps_ms[SigInd]
	
	top += 50
	GroupBox gb_HRIR_Noise2,pos={6.00,top-21},size={280.00,43.00},title="Head related impulse response (HRIR)"
	GroupBox gb_HRIR_Noise2,fSize=10,fStyle=1
	SetVariable sv_HRIRDataSet_Noise2 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_HRIRDataSet_Noise2 fSize=10
	SetVariable sv_HRIRDataSet_Noise2 value= HRIR_DataSet[SigInd],noedit= 1
	Button b_HRIRDataSetPopup_Noise2,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_HRIRDataSetPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	variable azInc = 2, elInc = 2
	wave/T HRIR_DataSet=$SpHrView_df_signalGlobals+"HRIR_DataSet"
	strswitch(HRIR_DataSet[SigInd])
		case "2° - NeumannKU100":
			azInc = 2
			elInc = 2
			break
		case "360° microphone stand - NeumannKU100":
			azInc = 1
			elInc = 1
			break
		case "360° full rotation mount - NeumannKU100":
			azInc = 1
			elInc = 1
			break
		case "Lebedev 2354 - NeumannKU100":
			azInc = 5 // ?
			elInc = 5 // ?
			break
		case "Lebedev 2702 - NeumannKU100":
			azInc = 5 // ?
			elInc = 5 // ?
			break
		default:
			break
	endswitch
	SetVariable sv_HRIR_Az_Noise2,pos={170.00,top},size={45.00,16.00},bodyWidth=45,proc=SpHrView_IR_SetVarProc,title="Az"
	SetVariable sv_HRIR_Az_Noise2,help={"Azimuth. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_HRIR_Az_Noise2,fSize=10,format="%.0f",limits={-180,180,azInc},value= HRIR_Az[SigInd],live= 1
	
	SetVariable sv_HRIR_El_Noise2,pos={233.00,top},size={47.00,16.00},bodyWidth=47,proc=SpHrView_IR_SetVarProc,title="El"
	SetVariable sv_HRIR_El_Noise2,help={"Elevation. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_HRIR_El_Noise2,fSize=10,format="%.1f",limits={-90,90,elInc},value= HRIR_El[SigInd],live= 1
	
	if(strlen(OperationList("SpHr_HRIR*", ";", "external")) < 2)
		HRIR_DataSet[SigInd] = "No HRIR"
		GroupBox gb_HRIR_Noise2 disable=2
		SetVariable sv_HRIRDataSet_Noise2 disable=2
		Button b_HRIRDataSetPopup_Noise2 disable=2
		SetVariable sv_HRIR_Az_Noise2 disable=2
		SetVariable sv_HRIR_El_Noise2 disable=2
	endif
	
	top += 43
	GroupBox gb_BRIR_Noise2,pos={6.00,top-21},size={280.00,65.00},title="Binarual Room impulse response (BRIR)"
	GroupBox gb_BRIR_Noise2,fSize=10,fStyle=1
	SetVariable sv_BRIR_Location_Noise2 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_BRIR_Location_Noise2 fSize=10
	SetVariable sv_BRIR_Location_Noise2 value= BRIR_Location[SigInd],noedit= 1
	Button b_BRIR_LocationPopup_Noise2,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_LocationPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_BRIR_Az_Noise2,pos={170.00,top},size={45.00,16.00},bodyWidth=45,proc=SpHrView_IR_SetVarProc,title="Az"
	SetVariable sv_BRIR_Az_Noise2,help={"Azimuth. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_BRIR_Az_Noise2,fSize=10,format="%.0f",limits={-180,180,1},value= BRIR_Az[SigInd],live= 1
	
	top += 21
	SetVariable sv_BRIR_Option_Noise2 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_BRIR_Option_Noise2 fSize=10
	SetVariable sv_BRIR_Option_Noise2 value= BRIR_Option[SigInd],noedit= 1
	Button b_BRIR_OptionPopup_Noise2,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_OptionPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_BRIR_Source_Noise2 pos={155.00,top},size={110.00,16.00},bodyWidth=110,title=" "
	SetVariable sv_BRIR_Source_Noise2 fSize=10
	SetVariable sv_BRIR_Source_Noise2 value= BRIR_Source[SigInd],noedit= 1
	Button b_BRIR_SourcePopup_Noise2,pos={266.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_SourcePopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	
	if(strlen(OperationList("SpHr_BRIR*", ";", "external")) < 2)
		BRIR_Location[SigInd] = "No BRIR"
		GroupBox gb_BRIR_Noise2 disable=2
		SetVariable sv_BRIR_Location_Noise2 disable=2
		Button b_BRIR_LocationPopup_Noise2 disable=2
		SetVariable sv_BRIR_Az_Noise2 disable=2
		SetVariable sv_BRIR_Option_Noise2 disable=2
		Button b_BRIR_OptionPopup_Noise2 disable=2
		SetVariable sv_BRIR_Source_Noise2 disable=2
		Button b_BRIR_SourcePopup_Noise2 disable=2
	endif
	
	top += 29
	GroupBox gb_Filters_Noise2,pos={6.00,top-5},size={280.00,27.00},title=""
	GroupBox gb_Filters_Noise2,fSize=10,fStyle=1
	SetVariable sv_HRIR_HPCFStr_Noise2 pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_HRIR_HPCFStr_Noise2 fSize=10, help={"Apply headphone compensation filter"}
	SetVariable sv_HRIR_HPCFStr_Noise2 value= HRIR_HPCFStr[SigInd],noedit= 1
	Button b_HRIR_HPCFStrPopup_Noise2,pos={131.00,top},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_HRIR_HPCFStrPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	CheckBox ck_HRIR_useCDFC_Noise2 pos={165.00,top+0},size={40.00,30},proc=SpHrView_Signal_CheckProc,title="Diffuse field filter"
	CheckBox ck_HRIR_useCDFC_Noise2 fSize=10, value= HRIR_useCDFC[SigInd], help={"Apply free-field compensation filter"}
	top += 48
	GroupBox gb_buffersDelays_Noise2,pos={6.00,top-19},size={280.00,62.00},title="Timing"
	GroupBox gb_buffersDelays_Noise2,fSize=10,fStyle=1
	SetVariable sv_onsetBuffer_Noise2,pos={15.00,top},size={151.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Onset:\\f00  Buffer (sec)"
	SetVariable sv_onsetBuffer_Noise2,fSize=10
	SetVariable sv_onsetBuffer_Noise2,limits={0,120,0.1},value= OnsetBuffers_sec[SigInd]
	SetVariable sv_onsetDelay_Noise2,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Delay\\f00 (ms)"
	SetVariable sv_onsetDelay_Noise2,fSize=10
	SetVariable sv_onsetDelay_Noise2,limits={0,1000,0.05},value= OnsetDelays_ms[SigInd]
	top += 20
	SetVariable sv_offsetBuffer_Noise2,pos={13.00,top},size={153.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Offset:  Buffer (sec)"
	SetVariable sv_offsetBuffer_Noise2,fSize=10
	SetVariable sv_offsetBuffer_Noise2,limits={0,120,0.10},value= OffsetBuffers_sec[SigInd]
	SetVariable sv_offsetDelay_Noise2,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Delay (ms)"
	SetVariable sv_offsetDelay_Noise2,fSize=10
	SetVariable sv_offsetDelay_Noise2,limits={0,1000,0.05},value= OffsetDelays_ms[SigInd]
	top += 48
	GroupBox gb_spectrum_Noise2,pos={6.00,top-22},size={280.00,95.00},title="Spectrum"
	GroupBox gb_spectrum_Noise2,fSize=10,fStyle=1
	SetVariable sv_Spectrum_Noise2,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_Spectrum_Noise2,fSize=10
	SetVariable sv_Spectrum_Noise2,value= SpectrumShape[SigInd],noedit= 1
	Button b_spectrumPopup_Noise2,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_spectrumPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_SpectrumViewEdit_Noise2,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_SpectrumViewEdit_Noise2,fSize=10
	top += 24
	SetVariable sv_highPassFreq_Noise2,pos={17.00,top},size={187.00,16.00},bodyWidth=55,proc=SpHrView_Signal_SetVarProc,title="Highpass corner (0dB, Hz)"
	SetVariable sv_highPassFreq_Noise2,help={"High pass corner (-0dB) frequency"}
	SetVariable sv_highPassFreq_Noise2,fSize=10
	SetVariable sv_highPassFreq_Noise2,limits={0,fs/2,10},value= HighPassFreqs[SigInd]
	SetVariable sv_highPassOrder_Noise2,pos={211.00,top},size={67.00,16.00},bodyWidth=35,proc=SpHrView_Signal_SetVarProc,title="Order"
	SetVariable sv_highPassOrder_Noise2,help={"Decline per octave = order * -6 dB. Pass 0 for brick-wall filter."}
	SetVariable sv_highPassOrder_Noise2,fSize=10
	SetVariable sv_highPassOrder_Noise2,limits={0,20,1},value= HighPassOrders[SigInd]
	top += 20
	SetVariable sv_lowPassFreq_Noise2,pos={19.00,top},size={185.00,16.00},bodyWidth=55,proc=SpHrView_Signal_SetVarProc,title="Lowpass corner (0dB, Hz)"
	SetVariable sv_lowPassFreq_Noise2,help={"High pass corner (-0dB) frequency"}
	SetVariable sv_lowPassFreq_Noise2,fSize=10
	SetVariable sv_lowPassFreq_Noise2,limits={0,fs/2,10},value= LowPassFreqs[SigInd]
	SetVariable sv_lowPassOrder_Noise2,pos={210.00,top},size={67.00,16.00},bodyWidth=35,proc=SpHrView_Signal_SetVarProc,title="Order"
	SetVariable sv_lowPassOrder_Noise2,help={"Decline per octave = order * -6 dB. Pass 0 for brick-wall filter."}
	SetVariable sv_lowPassOrder_Noise2,fSize=10
	SetVariable sv_lowPassOrder_Noise2,limits={0,20,1},value= LowPassOrders[SigInd]
	top += 54
	GroupBox gb_independence_Noise2,pos={6.00,top-21},size={280.00,43.00},title="Binaural Independence                 Index"
	GroupBox gb_independence_Noise2,fSize=10,fStyle=1
	SetVariable sv_Independence_Noise2,pos={11.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_Independence_Noise2,fSize=10
	SetVariable sv_Independence_Noise2,value= Independence[SigInd],noedit= 1
	Button b_independencePopup_Noise2,pos={162.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_independencePopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IndependenceVal_Noise2,pos={182.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IndependenceVal_Noise2,help={"Binaural independence. Option-click = 0.0"}
	SetVariable sv_IndependenceVal_Noise2,fSize=10,format="%.2g"
	SetVariable sv_IndependenceVal_Noise2,limits={0,1,0.05},value= IndependenceVal[SigInd]
	Button b_IndependenceViewEdit_Noise2,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IndependenceViewEdit_Noise2,fSize=10
	top += 44
	GroupBox gb_phaseInversion_Noise2,pos={6.00,top-21},size={280.00,43.00},title="Phase inversion"
	GroupBox gb_phaseInversion_Noise2,fSize=10,fStyle=1
	SetVariable sv_PhaseInversion_Noise2,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_PhaseInversion_Noise2,fSize=10,proc=SpHrView_Signal_SetVarProc
	SetVariable sv_PhaseInversion_Noise2,value= PhaseInversion[SigInd]//,noedit= 1
	Button b_PhaseInversionPopup_Noise2,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_PhaseInversionPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_PhaseInversionViewEdit_Noise2,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_PhaseInversionViewEdit_Noise2,fSize=10
	top += 28
	GroupBox gb_InterauralDifferences_Noise2,pos={6.00,top},size={280.00,145.00},title="\\f01Interaural Differences"
	GroupBox gb_InterauralDifferences_Noise2,fSize=10,fStyle=0
	top += 37
	TitleBox tb_ITD_Noise2,pos={21.00,top-15},size={204.00,13.00},title="\\f01Time                                                  ITD (ms)"
	TitleBox tb_ITD_Noise2,fSize=10,frame=0
	SetVariable sv_ITD_Noise2,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ITD_Noise2,fSize=10
	SetVariable sv_ITD_Noise2,value= ITD[SigInd],noedit= 1
	Button b_ITDPopup_Noise2,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ITDPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ITDVal_Noise2,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ITDVal_Noise2,help={"Interaural Time Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ITDVal_Noise2,fSize=10,format="%.3g"
	SetVariable sv_ITDVal_Noise2,limits={-9999,9999,0.05},value= ITDVal[SigInd]
	Button b_ITDViewEdit_Noise2,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ITDViewEdit_Noise2,fSize=10
	top += 38
	TitleBox tb_IPD_Noise2,pos={21.00,top-15},size={220.00,13.00},title="\\f01Phase                                               IPD (cycles)"
	TitleBox tb_IPD_Noise2,fSize=10,frame=0
	SetVariable sv_IPD_Noise2,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_IPD_Noise2,fSize=10
	SetVariable sv_IPD_Noise2,value= IPD[SigInd],noedit= 1
	Button b_IPDPopup_Noise2,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_IPDPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IPDVal_Noise2,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IPDVal_Noise2,help={"Interaural Phase Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_IPDVal_Noise2,fSize=10,format="%.3g"
	SetVariable sv_IPDVal_Noise2,limits={-2,2,0.01},value= IPDVal[SigInd]
	Button b_IPDViewEdit_Noise2,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IPDViewEdit_Noise2,fSize=10
	top += 38
	TitleBox tb_ILD_Noise2,pos={21.00,top-15},size={201.00,13.00},title="\\f01Level                                                 ILD (dB)"
	TitleBox tb_ILD_Noise2,fSize=10,frame=0
	SetVariable sv_ILD_Noise2,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ILD_Noise2,fSize=10
	SetVariable sv_ILD_Noise2,value= ILD[SigInd],noedit= 1
	Button b_ILDPopup_Noise2,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ILDPopup_Noise2,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ILDVal_Noise2,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ILDVal_Noise2,help={"Interaural Level Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ILDVal_Noise2,fSize=10,format="%.3g"
	SetVariable sv_ILDVal_Noise2,limits={-60,60,3},value= ILDVal[SigInd],live= 1
	Button b_ILDViewEdit_Noise2,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ILDViewEdit_Noise2,fSize=10
	
	if(stringmatch(PhaseInversion[SigInd], "PhaseInverts_Ambient"))
		SetVariable sv_PhaseInversion_Noise2,valueBackColor=(49151,53155,65535,32768)
	endif
	
	strswitch(Independence[SigInd])
		case "Independence_Ambient":
			SetVariable sv_Independence_Noise2,valueBackColor=(49151,53155,65535,32768)
			break
		case "Index":
			SetVariable sv_IndependenceVal_Noise2, win=SpHrView_BinauralSignal_Window, disable=2
			Button b_IndependenceViewEdit_Noise2, win=SpHrView_BinauralSignal_Window, disable=2
			break
		default:
			SetVariable sv_IndependenceVal_Noise2, win=SpHrView_BinauralSignal_Window, disable=0
			Button b_IndependenceViewEdit_Noise2, win=SpHrView_BinauralSignal_Window, disable=0
			break
	endswitch
End

Function SpHrView_BinauralSignalWindow_Tones()
	variable SigInd = 3
	wave Durations = $SpHrView_df_signalGlobals+"Durations"
	wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
	wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
	wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
	wave OnsetBuffers_sec = $SpHrView_df_signalGlobals+"OnsetBuffers_sec"
	wave OnsetDelays_ms = $SpHrView_df_signalGlobals+"OnsetDelays_ms"
	wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
	wave OffsetDelays_ms = $SpHrView_df_signalGlobals+"OffsetDelays_ms"
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	wave/T HRIR_DataSet = $SpHrView_df_signalGlobals+"HRIR_DataSet"
	wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
	wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
	wave HRIR_useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC"
	wave/T HRIR_HPCFStr = $SpHrView_df_signalGlobals+"HRIR_HPCFStr"
	wave/T BRIR_Location = $SpHrView_df_signalGlobals+"BRIR_Location"
	wave BRIR_Az = $SpHrView_df_signalGlobals+"BRIR_Az"
	wave/T BRIR_Source = $SpHrView_df_signalGlobals+"BRIR_Source"
	wave/T BRIR_Option = $SpHrView_df_signalGlobals+"BRIR_Option"
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	
	SVAR toneType=$SpHrView_df_signalGlobals+"toneType"
	NVAR F1=$SpHrView_df_signalGlobals+"F1"
	NVAR sweepF2=$SpHrView_df_signalGlobals+"sweepF2"
	
	DoWindow/F SpHrView_BinauralSignal_Window
	variable top = 230.00
	GroupBox gb_durationsAmplitudes_Tones,pos={6.00,top},size={280.00,55.00}
	top += 10
	SetVariable sv_duration_Tones,pos={32.00,top},size={113.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Duration"
	SetVariable sv_duration_Tones,fSize=10,fStyle=0
	SetVariable sv_duration_Tones,limits={0,1e+10,50},value= Durations[SigInd]
	SetVariable sv_durationUnits_Tones,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_durationUnits_Tones,fSize=10
	SetVariable sv_durationUnits_Tones,value= DurationUnits[SigInd],noedit= 1
	Button b_durationUnits_Tones,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_durationUnits_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 21
	SetVariable sv_amplitude_Tones,pos={23.00,top},size={122.00,16.00},bodyWidth=65,proc=SpHrView_Signal_SetVarProc,title="\\f01Amplitude"
	SetVariable sv_amplitude_Tones,fSize=10
	SetVariable sv_amplitude_Tones,limits={0,120,6},value= Amplitudes[SigInd]
	SetVariable sv_amplitudeUnits_Tones,pos={182.00,top},size={75.00,16.00},bodyWidth=75,title="units"
	SetVariable sv_amplitudeUnits_Tones,fSize=10
	SetVariable sv_amplitudeUnits_Tones,value= AmplitudeUnits[SigInd],noedit= 1
	Button b_amplitudeUnits_Tones,pos={258.00,top-1},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_amplitudeUnits_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	top += 30
	GroupBox gb_ramps_Tones,pos={6.00,top-6},size={280.00,30.00},fSize=10
	GroupBox gb_ramps_Tones,fStyle=1
	SetVariable sv_onRamp_Tones,pos={29.00,top},size={126.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Ramp:  On \\f00(ms)"
	SetVariable sv_onRamp_Tones,fSize=10
	SetVariable sv_onRamp_Tones,limits={0,1000,1},value= OnRamps_ms[SigInd]
	SetVariable sv_offRamp_Tones,pos={172.00,top},size={90.00,16.00},bodyWidth=45,proc=SpHrView_Signal_SetVarProc,title="\\f01Off \\f00(ms)"
	SetVariable sv_offRamp_Tones,fSize=10
	SetVariable sv_offRamp_Tones,limits={0,1000,1},value= OffRamps_ms[SigInd]
	
	top += 50
	GroupBox gb_HRIR_Tones,pos={6.00,top-21},size={280.00,43.00},title="Head related impulse response (HRIR)"
	GroupBox gb_HRIR_Tones,fSize=10,fStyle=1
	SetVariable sv_HRIRDataSet_Tones pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_HRIRDataSet_Tones fSize=10
	SetVariable sv_HRIRDataSet_Tones value= HRIR_DataSet[SigInd],noedit= 1
	Button b_HRIRDataSetPopup_Tones,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_HRIRDataSetPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	variable azInc = 2, elInc = 2
	strswitch(HRIR_DataSet[SigInd])
		case "2° - NeumannKU100":
			azInc = 2
			elInc = 2
			break
		case "360° microphone stand - NeumannKU100":
			azInc = 1
			elInc = 1
			break
		case "360° full rotation mount - NeumannKU100":
			azInc = 1
			elInc = 1
			break
		case "Lebedev 2354 - NeumannKU100":
			azInc = 5 // ?
			elInc = 5 // ?
			break
		case "Lebedev 2702 - NeumannKU100":
			azInc = 5 // ?
			elInc = 5 // ?
			break
		default:
			break
	endswitch
	SetVariable sv_HRIR_Az_Tones,pos={170.00,top},size={45.00,16.00},bodyWidth=45,proc=SpHrView_IR_SetVarProc,title="Az"
	SetVariable sv_HRIR_Az_Tones,help={"Azimuth. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_HRIR_Az_Tones,fSize=10,format="%.0f",limits={-180,180,azInc},value= HRIR_Az[SigInd],live= 1
	SetVariable sv_HRIR_El_Tones,pos={233.00,top},size={47.00,16.00},bodyWidth=47,proc=SpHrView_IR_SetVarProc,title="El"
	SetVariable sv_HRIR_El_Tones,help={"Elevation. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_HRIR_El_Tones,fSize=10,format="%.1f",limits={-90,90,elInc},value= HRIR_El[SigInd],live= 1

	if(strlen(OperationList("SpHr_HRIR*", ";", "external")) < 2)
		HRIR_DataSet[SigInd] = "No HRIR"
		GroupBox gb_HRIR_Tones disable=2
		SetVariable sv_HRIRDataSet_Tones disable=2
		Button b_HRIRDataSetPopup_Tones disable=2
		SetVariable sv_HRIR_Az_Tones disable=2
		SetVariable sv_HRIR_El_Tones disable=2
	endif
	
	top += 43
	GroupBox gb_BRIR_Tones,pos={6.00,top-21},size={280.00,65.00},title="Binarual Room impulse response (BRIR)"
	GroupBox gb_BRIR_Tones,fSize=10,fStyle=1
	SetVariable sv_BRIR_Location_Tones pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_BRIR_Location_Tones fSize=10
	SetVariable sv_BRIR_Location_Tones value= BRIR_Location[SigInd],noedit= 1
	Button b_BRIR_LocationPopup_Tones,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_LocationPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_BRIR_Az_Tones,pos={170.00,top},size={45.00,16.00},bodyWidth=45,proc=SpHrView_IR_SetVarProc,title="Az"
	SetVariable sv_BRIR_Az_Tones,help={"Azimuth. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_BRIR_Az_Tones,fSize=10,format="%.0f",limits={-180,180,1},value= BRIR_Az[SigInd],live= 1
	
	top += 21
	SetVariable sv_BRIR_Option_Tones pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_BRIR_Option_Tones fSize=10
	SetVariable sv_BRIR_Option_Tones value= BRIR_Option[SigInd],noedit= 1
	Button b_BRIR_OptionPopup_Tones,pos={131.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_OptionPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_BRIR_Source_Tones pos={155.00,top},size={110.00,16.00},bodyWidth=110,title=" "
	SetVariable sv_BRIR_Source_Tones fSize=10
	SetVariable sv_BRIR_Source_Tones value= BRIR_Source[SigInd],noedit= 1
	Button b_BRIR_SourcePopup_Tones,pos={266.00,top-0.5},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_BRIR_SourcePopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	
	if(strlen(OperationList("SpHr_BRIR*", ";", "external")) < 2)
		BRIR_Location[SigInd] = "No BRIR"
		GroupBox gb_BRIR_Tones disable=2
		SetVariable sv_BRIR_Location_Tones disable=2
		Button b_BRIR_LocationPopup_Tones disable=2
		SetVariable sv_BRIR_Az_Tones disable=2
		SetVariable sv_BRIR_Option_Tones disable=2
		Button b_BRIR_OptionPopup_Tones disable=2
		SetVariable sv_BRIR_Source_Tones disable=2
		Button b_BRIR_SourcePopup_Tones disable=2
	endif
	
	top += 29
	GroupBox gb_Filters_Tones,pos={6.00,top-5},size={280.00,27.00},title=""
	GroupBox gb_Filters_Tones,fSize=10,fStyle=1
	SetVariable sv_HRIR_HPCFStr_Tones pos={10.00,top},size={120.00,16.00},bodyWidth=120,title=" "
	SetVariable sv_HRIR_HPCFStr_Tones fSize=10, help={"Apply headphone compensation filter"}
	SetVariable sv_HRIR_HPCFStr_Tones value= HRIR_HPCFStr[SigInd],noedit= 1
	Button b_HRIR_HPCFStrPopup_Tones,pos={131.00,top},size={15.00,16.00},proc=SpHrView_IR_ButtonProc,title=""
	Button b_HRIR_HPCFStrPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	CheckBox ck_HRIR_useCDFC_Tones pos={165.00,top+0},size={40.00,30},proc=SpHrView_Signal_CheckProc,title="Diffuse field filter"
	CheckBox ck_HRIR_useCDFC_Tones fSize=10, value= HRIR_useCDFC[SigInd], help={"Apply free-field compensation filter"}	
	
	top += 48
	GroupBox gb_buffersDelays_Tones,pos={6.00,top-19},size={280.00,62.00},title="Timing"
	GroupBox gb_buffersDelays_Tones,fSize=10,fStyle=1
	SetVariable sv_onsetBuffer_Tones,pos={15.00,top},size={151.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Onset:\\f00  Buffer (sec)"
	SetVariable sv_onsetBuffer_Tones,fSize=10
	SetVariable sv_onsetBuffer_Tones,limits={0,120,0.1},value= OnsetBuffers_sec[SigInd]
	SetVariable sv_onsetDelay_Tones,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="\\f01Delay\\f00 (ms)"
	SetVariable sv_onsetDelay_Tones,fSize=10
	SetVariable sv_onsetDelay_Tones,limits={0,1000,0.05},value= OnsetDelays_ms[SigInd]
	top += 20
	SetVariable sv_offsetBuffer_Tones,pos={13.00,top},size={153.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Offset:  Buffer (sec)"
	SetVariable sv_offsetBuffer_Tones,fSize=10
	SetVariable sv_offsetBuffer_Tones,limits={0,120,0.10},value= OffsetBuffers_sec[SigInd]
	SetVariable sv_offsetDelay_Tones,pos={173.00,top},size={106.00,16.00},bodyWidth=50,proc=SpHrView_Signal_SetVarProc,title="Delay (ms)"
	SetVariable sv_offsetDelay_Tones,fSize=10
	SetVariable sv_offsetDelay_Tones,limits={0,1000,0.05},value= OffsetDelays_ms[SigInd]
	
	top += 80
	GroupBox gb_toneType_Tones,pos={6.00,top-40},size={280.00,90.00},title="Sweep, Tones or Pure-Tone"
	GroupBox gb_toneType_Tones,fSize=10,fStyle=1
	SetVariable sv_toneType_Tones,pos={10.00,top},size={140.00,16.00},bodyWidth=140,title=" "
	SetVariable sv_toneType_Tones,fSize=10
	SetVariable sv_toneType_Tones,value= toneType,noedit= 1
	Button b_toneTypePopup_Tones,pos={151.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_toneTypePopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	TitleBox tb_toneHz_Tones,pos={175.00,top-17},size={204.00,13.00},title="\\f01Frequency (Hz)"
	TitleBox tb_toneHz_Tones,fSize=10,frame=0
	SetVariable sv_F1_Tones,pos={173.00,top},size={56.00,16.00},bodyWidth=60,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_F1_Tones,help={"F1, fundemental frequency or pure-tone frequency"}
	SetVariable sv_F1_Tones,fSize=10,format="%.1d"
	SetVariable sv_F1_Tones,limits={20,fs/2,100},value= F1
	Button b_toneViewEdit_Tones,pos={240.00,top-2},size={35.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_toneViewEdit_Tones,fSize=10
	
	strswitch(toneType)
		case "Log Sweep":
		case "Linear Sweep":
		case "Linear 'Pink' Sweep":
			TitleBox tb_toneHz_Tones, pos={175.00,top-17}, title="\\f01F1 & F2\\f00 (Hz)"
			break
		case "Pure Tone":
		case "Pure-Tone":
			TitleBox tb_toneHz_Tones, pos={175.00,top-17}, title="\\f01Frequency\\f00 (Hz)"
			break
		case "SideBands":
			TitleBox tb_toneHz_Tones, pos={175.00,top-17}, title="\\f01Frequency\\f00 (Hz)"
			break
		case "Sawtooth wave harmonics":
		case "Square wave harmonics":
		case "Triange wave harmonics":
		case "harmonics":
			TitleBox tb_toneHz_Tones, pos={175.00,top-17}, title="\\f01F1\\f00 (Hz)"
			break
	endswitch
	
	top += 20
	SetVariable sv_F2_Tones,pos={173.00,top},size={56.00,16.00},bodyWidth=60,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_F2_Tones,help={"Ending frequency (Hz)"}
	SetVariable sv_F2_Tones,fSize=10,format="%.1d"
	SetVariable sv_F2_Tones,limits={20,fs/2,100},value= sweepF2, disable=3
	
	strswitch(toneType)
		case "Log sweep":
		case "Linear Sweep":
		case "Linear 'Pink' Sweep":
			SetVariable sv_F2_Tones, disable=0
			break
		default:
			break
	endswitch
	
	top += 95
	GroupBox gb_phaseInversion_Tones,pos={6.00,top-21},size={280.00,43.00},title="Phase inversion"
	GroupBox gb_phaseInversion_Tones,fSize=10,fStyle=1
	SetVariable sv_PhaseInversion_Tones,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_PhaseInversion_Tones,fSize=10,proc=SpHrView_Signal_SetVarProc
	SetVariable sv_PhaseInversion_Tones,value= PhaseInversion[SigInd]//,noedit= 1
	Button b_PhaseInversionPopup_Tones,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_PhaseInversionPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	Button b_PhaseInversionViewEdit_Tones,pos={245.00,top-2},size={35.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_PhaseInversionViewEdit_Tones,fSize=10
	top += 28
	GroupBox gb_InterauralDifferences_Tones,pos={6.00,top},size={280.00,145.00},title="\\f01Interaural Differences"
	GroupBox gb_InterauralDifferences_Tones,fSize=10,fStyle=0
	top += 37
	TitleBox tb_ITD_Tones,pos={21.00,top-15},size={204.00,13.00},title="\\f01Time                                                  ITD (ms)"
	TitleBox tb_ITD_Tones,fSize=10,frame=0
	SetVariable sv_ITD_Tones,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ITD_Tones,fSize=10
	SetVariable sv_ITD_Tones,value= ITD[SigInd],noedit= 1
	Button b_ITDPopup_Tones,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ITDPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ITDVal_Tones,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ITDVal_Tones,help={"Interaural Time Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ITDVal_Tones,fSize=10,format="%.3g"
	SetVariable sv_ITDVal_Tones,limits={-9999,9999,0.05},value= ITDVal[SigInd]
	Button b_ITDViewEdit_Tones,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ITDViewEdit_Tones,fSize=10
	top += 38
	TitleBox tb_IPD_Tones,pos={21.00,top-15},size={220.00,13.00},title="\\f01Phase                                               IPD (cycles)"
	TitleBox tb_IPD_Tones,fSize=10,frame=0
	SetVariable sv_IPD_Tones,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_IPD_Tones,fSize=10
	SetVariable sv_IPD_Tones,value= IPD[SigInd],noedit= 1
	Button b_IPDPopup_Tones,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_IPDPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_IPDVal_Tones,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_IPDVal_Tones,help={"Interaural Phase Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_IPDVal_Tones,fSize=10,format="%.3g"
	SetVariable sv_IPDVal_Tones,limits={-2,2,0.01},value= IPDVal[SigInd]
	Button b_IPDViewEdit_Tones,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_IPDViewEdit_Tones,fSize=10
	top += 38
	TitleBox tb_ILD_Tones,pos={21.00,top-15},size={201.00,13.00},title="\\f01Level                                                 ILD (dB)"
	TitleBox tb_ILD_Tones,fSize=10,frame=0
	SetVariable sv_ILD_Tones,pos={10.00,top},size={150.00,16.00},bodyWidth=150,title=" "
	SetVariable sv_ILD_Tones,fSize=10
	SetVariable sv_ILD_Tones,value= ILD[SigInd],noedit= 1
	Button b_ILDPopup_Tones,pos={161.00,top},size={15.00,16.00},proc=SpHrView_Signal_ButtonProc,title=""
	Button b_ILDPopup_Tones,picture= ProcGlobal#SpHrView_pict_PopUpList
	SetVariable sv_ILDVal_Tones,pos={181.00,top},size={54.00,16.00},bodyWidth=54,proc=SpHrView_Signal_SetVarProc,title=" "
	SetVariable sv_ILDVal_Tones,help={"Interaural Level Difference. Option-click: set to 0.0, Command-click: reverse sign"}
	SetVariable sv_ILDVal_Tones,fSize=10,format="%.3g"
	SetVariable sv_ILDVal_Tones,limits={-60,60,3},value= ILDVal[SigInd],live= 1
	Button b_ILDViewEdit_Tones,pos={240.00,top-2},size={40.00,20.00},proc=SpHrView_Signal_ButtonProc,title="Edit"
	Button b_ILDViewEdit_Tones,fSize=10
End


Function SpHrView_Signal_TabProc(tca) : TabControl
	STRUCT WMTabControlAction &tca
		switch( tca.eventCode )
			case 2: // mouse up
				Variable tab = tca.tab
				strswitch( tca.win )
					case "SpHrView_BinauralSignal_Window":
						SpHrView_BinauralSignalWindowTabSelect(tab)
						break
				endswitch				
				break
			case -1: // control being killed
				break
		endswitch

	return 0
End

function SpHrView_BinauralSignalWindowTabSelect(tab)
	variable tab
	string suffixStr=""
	switch(tab)
		case 0:
			suffixStr =  "_NoiseAmb"
			break
		case 1:
			suffixStr =  "_Noise1"
			break
		case 2:
			suffixStr =  "_Noise2"
			break
		case 3:
			suffixStr =  "_Tones"
			break
	endswitch
	String controlList// = ControlNameList("SpHrView_BinauralSignal_Window", ";", "*"+suffixStr)
	variable i, disableState=0
	string str
	//GroupBoxes
	controlList = ControlNameList("SpHrView_BinauralSignal_Window", ";", "gb_*_NoiseAmb")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "gb_*_Noise1")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "gb_*_Noise2")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "gb_*_Tones")
	for(i=0;i<ItemsInList(controlList);++i)
		str = StringFromList(i, controlList)
		ControlInfo/W=SpHrView_BinauralSignal_Window $str
		disableState = V_disable
		GroupBox $str, win=SpHrView_BinauralSignal_Window, disable= 1
		if(StringMatch(str, "*"+suffixStr ))
			if(disableState==2)
				GroupBox $str, win=SpHrView_BinauralSignal_Window, disable= 2
			else //if(disableState==0)
				GroupBox $str, win=SpHrView_BinauralSignal_Window, disable= 0
			endif
		endif
	endfor
	//Buttons
	controlList = ControlNameList("SpHrView_BinauralSignal_Window", ";", "b_*_NoiseAmb")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "b_*_Noise1")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "b_*_Noise2")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "b_*_Tones")
	for(i=0;i<ItemsInList(controlList);++i)
		str = StringFromList(i, controlList)
		ControlInfo/W=SpHrView_BinauralSignal_Window $str
		disableState = V_disable
		Button $str, win=SpHrView_BinauralSignal_Window, disable= 1
		if(StringMatch(str, "*"+suffixStr ))
			ControlInfo/W=SpHrView_BinauralSignal_Window $str
			if(disableState==2)
				Button $str, win=SpHrView_BinauralSignal_Window, disable= 2
			else
				Button $str, win=SpHrView_BinauralSignal_Window, disable= 0
			endif
		endif
	endfor
	//Set Variables
	controlList = ControlNameList("SpHrView_BinauralSignal_Window", ";", "sv_*_NoiseAmb")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "sv_*_Noise1")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "sv_*_Noise2")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "sv_*_Tones")
	for(i=0;i<ItemsInList(controlList);++i)
		str = StringFromList(i, controlList)
		ControlInfo/W=SpHrView_BinauralSignal_Window $str
		disableState = V_disable
		SetVariable $str, win=SpHrView_BinauralSignal_Window, disable= 1
		if(StringMatch(str, "*"+suffixStr ))
			ControlInfo/W=SpHrView_BinauralSignal_Window $str
			if(disableState==2)
				SetVariable $str, win=SpHrView_BinauralSignal_Window, disable= 2
			elseif(disableState==3)
				SetVariable $str, win=SpHrView_BinauralSignal_Window, disable= 3
			else
				SetVariable $str, win=SpHrView_BinauralSignal_Window, disable= 0
			endif
		endif
	endfor
	//Check Boxes
	controlList = ControlNameList("SpHrView_BinauralSignal_Window", ";", "ck_*_NoiseAmb")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "ck_*_Noise1")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "ck_*_Noise2")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "ck_*_Tones")
	for(i=0;i<ItemsInList(controlList);++i)
		str = StringFromList(i, controlList)
		if(stringmatch(str, "ck_noise_Noise1") || stringmatch(str, "ck_noise_Noise2") || stringmatch(str, "ck_noise_Tones"))
			// skip these check boxes
		else
			ControlInfo/W=SpHrView_BinauralSignal_Window $str
			disableState = V_disable
			CheckBox $str, win=SpHrView_BinauralSignal_Window, disable= 1
			if(StringMatch(str, "*"+suffixStr ))
				ControlInfo/W=SpHrView_BinauralSignal_Window $str
				if(disableState==2)
					CheckBox $str, win=SpHrView_BinauralSignal_Window, disable= 2
				else
					CheckBox $str, win=SpHrView_BinauralSignal_Window, disable= 0
				endif
			endif
		endif
	endfor
	//TitleBoxes
	controlList = ControlNameList("SpHrView_BinauralSignal_Window", ";", "tb_*_NoiseAmb")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "tb_*_Noise1")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "tb_*_Noise2")
	controlList += ControlNameList("SpHrView_BinauralSignal_Window", ";", "tb_*_Tones")
	for(i=0;i<ItemsInList(controlList);++i)
		str = StringFromList(i, controlList)
		ControlInfo/W=SpHrView_BinauralSignal_Window $str
		disableState = V_disable
		TitleBox $str, win=SpHrView_BinauralSignal_Window, disable= 1
		if(StringMatch(str, "*"+suffixStr ))
			ControlInfo/W=SpHrView_BinauralSignal_Window $str
			if(disableState==2)
				TitleBox $str, win=SpHrView_BinauralSignal_Window, disable= 2
			else
				TitleBox $str, win=SpHrView_BinauralSignal_Window, disable= 0
			endif
		endif
	endfor
	SpHrView_BinauralSignalWindowColors(-1)
	
	// Color signal generation checks if unchecked when corresponding tab selected
	NVAR makeAmbient = $SpHrView_df_signalGlobals+"makeAmbient"
	NVAR makeNoise1 = $SpHrView_df_signalGlobals+"makeNoise1"
	NVAR makeNoise2 = $SpHrView_df_signalGlobals+"makeNoise2"
	NVAR makeTones = $SpHrView_df_signalGlobals+"makeTones"
	strswitch(suffixStr)
		case "_NoiseAmb":
			if(makeAmbient)
				CheckBox ck_signalAmbient, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			else
				CheckBox ck_signalAmbient, win=SpHrView_BinauralSignal_Window, fColor=(52428,1,1)//,fSize=11
			endif
			CheckBox ck_noise_Noise1, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_noise_Noise2, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_tones, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			break
			break
		case "_Noise1":
			if(makeNoise1)
				CheckBox ck_noise_Noise1, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			else
				CheckBox ck_noise_Noise1, win=SpHrView_BinauralSignal_Window, fColor=(52428,1,1)//,fSize=11
			endif
			CheckBox ck_signalAmbient, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_noise_Noise2, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_tones, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			break
		case "_Noise2":
			if(makeNoise2)
				CheckBox ck_noise_Noise2, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			else
				CheckBox ck_noise_Noise2, win=SpHrView_BinauralSignal_Window, fColor=(52428,1,1)//,fSize=11
			endif
			CheckBox ck_signalAmbient, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_noise_Noise1, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_tones, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			break
		case "_Tones":
			if(makeTones)
				CheckBox ck_tones, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			else
				CheckBox ck_tones, win=SpHrView_BinauralSignal_Window, fColor=(52428,1,1)//,fSize=11
			endif
			CheckBox ck_signalAmbient, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_noise_Noise1, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			CheckBox ck_noise_Noise2, win=SpHrView_BinauralSignal_Window, fColor=(0,0,0)//,fSize=10
			break
	endswitch
	
	switch(tab)
		case 0: // "_NoiseAmb"
			GroupBox gb_HRIR_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable= 2
			GroupBox gb_BRIR_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable= 2
			GroupBox gb_buffersDelays_NoiseAmb, win=SpHrView_BinauralSignal_Window, disable= 2
			break
		case 1:	// "_Noise1"
			break
		case 2:	// "_Noise2"
			break
		case 3: 	// "_Tones"
			break
	endswitch
End

// =================================================================================================
// SpHrView_BinauralSignalWindowColors(index)
// Shade SetVariable boxes 
// white/blank: value is zero or typical
// greenish: value is non-zero of non-typical
// reddish: values for Noise 1 and 2 differ
// blueish: shaped values but typical
// =================================================================================================
Function SpHrView_BinauralSignalWindowColors(index)
	variable index
	DoWindow SpHrView_BinauralSignal_Window
	if(!v_flag)
		return 0
	endif
	if(index < 0)
		ControlInfo/W=SpHrView_BinauralSignal_Window tab_signal
		index = V_Value
	endif
	wave Durations = $SpHrView_df_signalGlobals+"Durations"
	wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
	wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
	wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
	wave OnsetBuffers_sec = $SpHrView_df_signalGlobals+"OnsetBuffers_sec"
	wave OnsetDelays_ms = $SpHrView_df_signalGlobals+"OnsetDelays_ms"
	wave OffsetBuffers_sec = $SpHrView_df_signalGlobals+"OffsetBuffers_sec"
	wave OffsetDelays_ms = $SpHrView_df_signalGlobals+"OffsetDelays_ms"
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
	wave HighPassFreqs = $SpHrView_df_signalGlobals+"HighPassFreqs"
	wave HighPassOrders = $SpHrView_df_signalGlobals+"HighPassOrders"
	wave LowPassFreqs = $SpHrView_df_signalGlobals+"LowPassFreqs"
	wave LowPassOrders = $SpHrView_df_signalGlobals+"LowPassOrders"
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	
	NVAR makeNoise1 = $SpHrView_df_signalGlobals+"makeNoise1"
	NVAR makeNoise2 = $SpHrView_df_signalGlobals+"makeNoise2"
	variable makingNoise1and2 = 0
	if(makeNoise1 && makeNoise2)
		makingNoise1and2 = 1
	endif
	
	String CtrlSuffix
	switch(index)
		case 0:
			CtrlSuffix = "NoiseAmb"
			break
		case 1:
			CtrlSuffix = "Noise1"
			break
		case 2:
			CtrlSuffix = "Noise2"
			break
		case 3:
			CtrlSuffix = "Tones"
			return 0
			break
	Endswitch
	
	// ILD
	if(makingNoise1and2 && (index==1 || index==2))
		if(ILDVal[1]==0 && ILDVal[2]==0)
			SetVariable sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		elseif(ILDVal[1] == ILDVal[2]) 	// Shade greenish if equal
			SetVariable sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
			SetVariable sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		else										// Shade reddish if not equal
			SetVariable sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else
		if(ILDVal[index] == 0)																		// No Shade if 0
			SetVariable $"sv_ILDVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade greenish if not 0
			SetVariable $"sv_ILDVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		endif
	endif
	
	// ITD
	if(makingNoise1and2 && (index==1 || index==2))
		if(ITDVal[1]==0 && ITDVal[2]==0)
			SetVariable sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		elseif(ITDVal[1] == ITDVal[2]) 																// Shade greenish if equal
			SetVariable sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
			SetVariable sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		else																									// Shade reddish if not equal
			SetVariable sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else
		if(ITDVal[index] == 0)																		// No Shade if 0
			SetVariable $"sv_ITDVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade greenish if not 0
			SetVariable $"sv_ITDVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		endif
	endif

	// IPD
	if(makingNoise1and2 && (index==1 || index==2))
		if(IPDVal[1]==0 && IPDVal[2]==0)
			SetVariable sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		elseif(IPDVal[1] == IPDVal[2]) 																// Shade greenish if equal
			SetVariable sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
			SetVariable sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		else																									// Shade reddish if not equal
			SetVariable sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else
		if(IPDVal[index] == 0)																		/// No Shade if 0
			SetVariable $"sv_IPDVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade greenish if not 0
			SetVariable $"sv_IPDVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		endif
	endif
	
	// Independence
	if(makingNoise1and2 && (index==1 || index==2))
		if(IndependenceVal[1]==0 && IndependenceVal[2]==0)
			SetVariable sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		elseif(IndependenceVal[1] == IndependenceVal[2]) 													// Shade greenish if equal
			SetVariable sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
			SetVariable sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		else																												// Shade reddish if not equal
			SetVariable sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else
		if(IndependenceVal[index] == 0)																		// No Shade if 0
			SetVariable $"sv_IndependenceVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																											// Shade greenish if not 0
			SetVariable $"sv_IndependenceVal_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
		endif
	endif
	
	// Duration
	if(makingNoise1and2 && (index==1 || index==2))
		if(Durations[1] == Durations[2])
			SetVariable sv_Duration_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_Duration_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_Duration_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_Duration_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_Duration_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	
	// Amplitudes
	if(makingNoise1and2 && (index==1 || index==2))
		if(Amplitudes[1] == Amplitudes[2])
			SetVariable sv_amplitude_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_amplitude_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_amplitude_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_amplitude_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_Amplitude_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	
	// OnRamps_ms
	if(makingNoise1and2 && (index==1 || index==2))
		if(OnRamps_ms[1] == OnRamps_ms[2])
			SetVariable sv_onRamp_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_onRamp_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_onRamp_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_onRamp_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_onRamp_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	// OffRamps_ms
	if(makingNoise1and2 && (index==1 || index==2))
		if(OffRamps_ms[1] == OffRamps_ms[2])
			SetVariable sv_OffRamp_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_OffRamp_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_OffRamp_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_OffRamp_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_OffRamp_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	
	// OnsetBuffers_sec
	if(makingNoise1and2 && (index==1 || index==2))
		if(OnsetBuffers_sec[1] == OnsetBuffers_sec[2])
			SetVariable sv_onsetBuffer_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_onsetBuffer_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_onsetBuffer_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_onsetBuffer_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	elseif (index==3)																				// No Shade if 0
		SetVariable $"sv_onsetBuffer_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	// OffsetBuffers_sec
	if(makingNoise1and2 && (index==1 || index==2))
		if(OffsetBuffers_sec[1] == OffsetBuffers_sec[2])
			SetVariable sv_offsetBuffer_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_offsetBuffer_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_offsetBuffer_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_offsetBuffer_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	elseif (index==3)																				// No Shade if 0
		Print CtrlSuffix
		SetVariable $"sv_OffsetBuffer_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	
	// OnsetDelays_ms
	if(makingNoise1and2 && (index==1 || index==2))
		if(OnsetDelays_ms[1] == OnsetDelays_ms[2])
			SetVariable sv_onsetDelay_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_onsetDelay_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_onsetDelay_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_onsetDelay_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	elseif (index==3)															// No Shade if 0
		SetVariable $"sv_onsetDelay_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	// OffsetDelays_ms
	if(makingNoise1and2 && (index==1 || index==2))
		if(OffsetDelays_ms[1] == OffsetDelays_ms[2])
			SetVariable sv_offsetDelay_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_offsetDelay_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_offsetDelay_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_offsetDelay_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	elseif (index==3)																				// No Shade if 0
		SetVariable $"sv_offsetDelay_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif

	//HighPassFreqs
	if(makingNoise1and2 && (index==1 || index==2))
		if(HighPassFreqs[1] == HighPassFreqs[2])
			SetVariable sv_highPassFreq_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_highPassFreq_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_highPassFreq_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_highPassFreq_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_highPassFreq_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	//HighPassOrders
	if(makingNoise1and2 && (index==1 || index==2))
		if(HighPassOrders[1] == HighPassOrders[2])
			SetVariable sv_highPassOrder_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_highPassOrder_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_highPassOrder_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_highPassOrder_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_highPassOrder_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	
	//LowPassFreqs
	if(makingNoise1and2 && (index==1 || index==2))
		if(LowPassFreqs[1] == LowPassFreqs[2])
			SetVariable sv_lowPassFreq_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_lowPassFreq_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_lowPassFreq_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_lowPassFreq_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_lowPassFreq_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	//LowPassOrders
	if(makingNoise1and2 && (index==1 || index==2))
		if(LowPassOrders[1] == LowPassOrders[2])
			SetVariable sv_lowPassOrder_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_lowPassOrder_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_lowPassOrder_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_lowPassOrder_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_lowPassOrder_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif
	
	//SpectrumShape
	if(makingNoise1and2 && (index==1 || index==2))
		if(stringmatch(SpectrumShape[1], SpectrumShape[2]))
			SetVariable sv_Spectrum_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
			SetVariable sv_Spectrum_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
		else																								// Shade reddish if not equal
			SetVariable sv_Spectrum_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
			SetVariable sv_Spectrum_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(65535,49151,49151,32768)
		endif
	else																			// No Shade if 0
		SetVariable $"sv_lowPassOrder_"+CtrlSuffix win=SpHrView_BinauralSignal_Window, valueBackColor=0
	endif

End

Function SpHrView_Signal_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	variable down = 0
	switch( sva.eventCode )
		case 9: // mouse down
			down = 1
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			Variable forceUpdate = 0
			Variable dval = sva.dval
			//if(numtype(dval)!= 0)
			//	dval = 1
			//endif
			//Printf "sva.eventMod= %g\r", sva.eventMod
			if(sva.eventMod == 5)  // option key: Set to zero
				dval = 0
				forceUpdate = 1
			endif
			if(sva.eventMod == 9)  // command key: Reverse sign
				dval = -dval
				forceUpdate = 1
			endif
			String sval = sva.sval
			String nameStr = sva.vName
			wave/Z w = sva.svWave
			Variable index = sva.rowIndex
			string str

			strswitch( sva.ctrlName )
//				case "sv_HRIR_Az_Noise1":
//					wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
//					HRIR_Az[1] = dval
//					SpHr_HRIR_DisplaylocationsUpdate()
//					break
//				case "sv_HRIR_El_Noise1":
//					wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
//					HRIR_El[1] = dval
//					SpHr_HRIR_DisplaylocationsUpdate()
//					break
//				case "sv_HRIR_Az_Noise2":
//					wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
//					HRIR_Az[2] = dval
//					SpHr_HRIR_DisplaylocationsUpdate()
//					break
//				case "sv_HRIR_El_Noise2":
//					wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
//					HRIR_El[2] = dval
//					SpHr_HRIR_DisplaylocationsUpdate()
//					break
//				case "sv_HRIR_Az_Tones":
//					wave HRIR_Az = $SpHrView_df_signalGlobals+"HRIR_Az"
//					HRIR_Az[3] = dval
//					SpHr_HRIR_DisplaylocationsUpdate()
//					break
//				case "sv_HRIR_El_Tones":
//					wave HRIR_El = $SpHrView_df_signalGlobals+"HRIR_El"
//					HRIR_El[3] = dval
//					SpHr_HRIR_DisplaylocationsUpdate()
//					break
				
				case "sv_F1_Tones":
					String DF = SpHrView_df_signalGlobals+"Tones:"
					wave/Z toneFreqs = $DF+"toneFreqs"
					if(!waveexists(toneFreqs))
						SpHrView_ToneDefaults()
					endif
					SpHrView_TonesEditMakePoints()
					break
					
				case "sv_F2_Tones":
				
					break
			
				case "sv_ILDVal_NoiseAmb":
				case "sv_ILDVal_Noise1":
				case "sv_ILDVal_Noise2":	
				case "sv_ILDVal_Tones":		
					//SpHrView_ILDEditMakePoints(index)
					SpHrView_ILDSetVarHandler(sva.ctrlName, index, dval) 
					break
					
				case "sv_IPDVal_NoiseAmb":
				case "sv_IPDVal_Noise1":
				case "sv_IPDVal_Noise2":		
				case "sv_IPDVal_Tones":		
					//SpHrView_IPDEditMakePoints(index)
					SpHrView_IPDSetVarHandler(sva.ctrlName, index, dval) 
					break
					
				case "sv_ITDVal_NoiseAmb":
				case "sv_ITDVal_Noise1":
				case "sv_ITDVal_Noise2":		
				case "sv_ITDVal_Tones":	
					//SpHrView_ITDEditMakePoints(index)
					SpHrView_ITDSetVarHandler(sva.ctrlName, index, dval) 
					break
					
				case "sv_IndependenceVal_NoiseAmb":
				case "sv_IndependenceVal_Noise1":
				case "sv_IndependenceVal_Noise2":		
					//SpHrView_IndependenceEditMakePoints(index)
					SpHrView_IndependenceSetVarHandler(sva.ctrlName, index, dval) 
					break
					
				case "sv_WaveName":
					SVAR WavesName = $SpHrView_df_signalGlobals+"WavesName"
					// cleanup name if no path
					if(strlen(ParseFilePath(1, WavesName, ":", 1, 0)) < 1) // path to folder containing the wave
						WavesName = cleanupName(WavesName,0)
					else
						// use path
						WavesName = ParseFilePath(1, WavesName, ":", 1, 0)+cleanupName(ParseFilePath(0, WavesName, ":", 1, 0),0)
					endif
					break
					
				case "sv_samplingRate":
					NVAR fs = $SpHrView_df_signalGlobals+"fs"
					SVAR/Z rateListStr = $SpHrView_df_signalGlobals+"Audio:rateList"
					str = num2str(fs)
					if(FindListItem(str, rateListStr) < 0)
						fs = str2num(StringFromList(1, rateListStr)) // get second in list, whatever that is..
					endif
					break
					
				if(down)
					return 0
				endif
					
				case "sv_duration_NoiseAmb":
				case "sv_duration_Noise1":
				case "sv_duration_Noise2":
				case "sv_duration_Tones":
				
				case "sv_onRamp_NoiseAmb":
				case "sv_onRamp_Noise1":
				case "sv_onRamp_Noise2":
				case "sv_onRamp_Tones":
				
				case "sv_offRamp_NoiseAmb":
				case "sv_offRamp_Noise1":
				case "sv_offRamp_Noise2":
				case "sv_offRamp_Tones":
				
					if(waveexists(w))
						wave OnRamps_ms = $SpHrView_df_signalGlobals+"OnRamps_ms"
						wave OffRamps_ms = $SpHrView_df_signalGlobals+"OffRamps_ms"
						wave Durations = $SpHrView_df_signalGlobals+"Durations"
						wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
						variable Dur = Durations[index] // ms
						if(stringmatch(DurationUnits[index], "sec"))
							Dur *= 1000	// convert to ms
						elseif(stringmatch(DurationUnits[index], "points"))
							NVAR fs = $SpHrView_df_signalGlobals+"fs"
							Dur /= fs; Dur *= 1000 // convert to ms
						endif
						if(OnRamps_ms[index] + OffRamps_ms[index] > Dur)
							OnRamps_ms[index] = Dur * 0.5
							OffRamps_ms[index] = Dur * 0.5
						endif
						SpHrView_BinauralSignalWindowColors(index)
					endif
										
					break
					
				case "sv_amplitude_NoiseAmb":
				case "sv_amplitude_Noise1":
				case "sv_amplitude_Noise2":
				case "sv_amplitude_Tones":
				case "sv_Spectrum_NoiseAmb":
				case "sv_Spectrum_Noise1":
				case "sv_Spectrum_Noise2":
				case "sv_highPassFreq_NoiseAmb":
				case "sv_highPassFreq_Noise1":
				case "sv_highPassFreq_Noise2":
				case "sv_lowPassFreq_NoiseAmb":
				case "sv_lowPassFreq_Noise1":
				case "sv_lowPassFreq_Noise2":
				case "sv_highPassOrder_NoiseAmb":
				case "sv_highPassOrder_Noise1":
				case "sv_highPassOrder_Noise2":
				case "sv_lowPassOrder_NoiseAmb":
				case "sv_lowPassOrder_Noise1":
				case "sv_lowPassOrder_Noise2":
					SpHrView_BinauralSignalWindowColors(index)
					break
					
				case "sv_PhaseInversion_NoiseAmb":
				case "sv_PhaseInversion_Noise1":
				case "sv_PhaseInversion_Noise2":
				case "sv_PhaseInversion_Tones":
					if(sva.eventCode == 9)
						SpHrView_PhaseInversionSetVarHandler(sva.ctrlName, index, dval, sva.eventMod) 
						forceUpdate = 1
					endif
					break
				
			endswitch
			break
		case -1: // control being killed
			break
	endswitch
	
	if(forceUpdate)	
		ControlUpdate/W=$sva.win $sva.ctrlName
	endif
	return 0
End

Function SpHrView_Signal_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 2: // mouse up
			variable index
			String DF //, str="", popupListStr=""
			String checked= "\\M0:!" + num2char(18) + ":" // checkmark code
			// click code here
			strswitch( ba.ctrlName )
				case "b_PresetPopUp":
					String PresetListStr = "Default;-;Head related impulse response (HRIR);Binaural room impulse response (BRIR);-;+0.25 ms ITD;Lead Lag pair, ±.25 ms ITD, 2.5 ms Lag;"
					PopupContextualMenu PresetListStr
					if(V_flag>=1)
						SVAR PresetStr=$SpHrView_df_signalGlobals+"PresetStr"
						PresetStr = S_Selection
						SpHrView_SignalPresets(PresetStr)
					endif
					break
					
				case "b_ILDViewEdit_NoiseAmb":
				case "b_IPDViewEdit_NoiseAmb":
				case "b_ITDViewEdit_NoiseAmb":
				case "b_PhaseInversionViewEdit_NoiseAmb":
				case "b_IndependenceViewEdit_NoiseAmb":
				case "b_SpectrumViewEdit_NoiseAmb":
				case "b_durationUnits_NoiseAmb":
				case "b_amplitudeUnits_NoiseAmb":
					index = 0
					break
				case "b_ILDViewEdit_Noise1":
				case "b_IPDViewEdit_Noise1":
				case "b_ITDViewEdit_Noise1":
				case "b_PhaseInversionViewEdit_Noise1":
				case "b_IndependenceViewEdit_Noise1":
				case "b_SpectrumViewEdit_Noise1":
				case "b_durationUnits_Noise1":
				case "b_amplitudeUnits_Noise1":
					index = 1
					break
				case "b_ILDViewEdit_Noise2":
				case "b_IPDViewEdit_Noise2":
				case "b_ITDViewEdit_Noise2":
				case "b_PhaseInversionViewEdit_Noise2":
				case "b_IndependenceViewEdit_Noise2":
				case "b_SpectrumViewEdit_Noise2":
				case "b_durationUnits_Noise2":
				case "b_amplitudeUnits_Noise2":
					index = 2
					break
				case "b_ILDViewEdit_Tones":
				case "b_IPDViewEdit_Tones":
				case "b_ITDViewEdit_Tones":
				case "b_PhaseInversionViewEdit_Tones":
				case "b_IndependenceViewEdit_Tones":
				case "b_SpectrumViewEdit_Tones":
				case "b_durationUnits_Tones":
				case "b_amplitudeUnits_Tones":
					index = 3
					break
			endswitch
			
			strswitch( ba.ctrlName )
			
				// ILD - Edit Button
				case "b_ILDViewEdit_NoiseAmb":
				case "b_ILDViewEdit_Noise1":
				case "b_ILDViewEdit_Noise2":
				case "b_ILDViewEdit_Tones":
					wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
					if(stringmatch(ILD[index], "Disabled"))
						ILD[index] = "ILD"
					endif
					DoWindow SpHrView_ILDEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_ILDEdit_Window
					else
						SpHrView_ILDEditWindow(index)
					endif
					break
				// ILD - PopupContextualMenu Button
				case "b_ILDPopup_NoiseAmb":
				case "b_ILDPopup_Noise1":
				case "b_ILDPopup_Noise2":
				case "b_ILDPopup_Tones":
					SpHrView_ILDButtonPopUpHandler(ba.ctrlName, -1) 
					break
			
				// IPD - Edit Button
				case "b_IPDViewEdit_NoiseAmb":
				case "b_IPDViewEdit_Noise1":
				case "b_IPDViewEdit_Noise2":
				case "b_IPDViewEdit_Tones":
					wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
					if(stringmatch(IPD[index], "Disabled"))
						IPD[index] = "IPD"
					endif
					DoWindow SpHrView_IPDEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_IPDEdit_Window
					else
						SpHrView_IPDEditWindow(index)
					endif
					break
				// IPD - PopupContextualMenu Button
				case "b_IPDPopup_NoiseAmb":
				case "b_IPDPopup_Noise1":
				case "b_IPDPopup_Noise2":
				case "b_IPDPopup_Tones":
					SpHrView_IPDButtonPopUpHandler(ba.ctrlName, -1) 
					break
						
				// ITD - Edit Button
				case "b_ITDViewEdit_NoiseAmb":
				case "b_ITDViewEdit_Noise1":
				case "b_ITDViewEdit_Noise2":
				case "b_ITDViewEdit_Tones":
					wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
					if(stringmatch(ITD[index], "Disabled"))
						ITD[index] = "ITD (phase shift)"
					endif
					DoWindow SpHrView_ITDEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_ITDEdit_Window
					else
						SpHrView_ITDEditWindow(index)
					endif
					break
				// ITD - PopupContextualMenu Button
				case "b_ITDPopup_NoiseAmb":
				case "b_ITDPopup_Noise1":
				case "b_ITDPopup_Noise2":
				case "b_ITDPopup_Tones":
					SpHrView_ITDButtonPopUpHandler(ba.ctrlName, -1) 
					break
				
				// Phase inversion - Edit Button
				case "b_PhaseInversionViewEdit_NoiseAmb":
				case "b_PhaseInversionViewEdit_Noise1":
				case "b_PhaseInversionViewEdit_Noise2":
				case "b_PhaseInversionViewEdit_Tones":
					DoWindow SpHrView_PhaseInversionEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_PhaseInversionEdit_Window
					else
						SpHrView_PhaseInversionEditWindow(index)
					endif
					break
				// Phase inversion - PopupContextualMenu Button
				case "b_PhaseInversionPopup_NoiseAmb":
				case "b_PhaseInversionPopup_Noise1":
				case "b_PhaseInversionPopup_Noise2":
				case "b_PhaseInversionPopup_Tones":
					SpHrView_PhaseInversionButtonPopUpHandler(ba.ctrlName, -1) 
					break
					
				// Independence - Edit Button
				case "b_IndependenceViewEdit_NoiseAmb":
				case "b_IndependenceViewEdit_Noise1":
				case "b_IndependenceViewEdit_Noise2":
					DoWindow SpHrView_IndependenceEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_IndependenceEdit_Window
					else
						SpHrView_IndependenceEditWindow(index)
					endif
					break
				// PopupContextualMenu - PopupContextualMenu Button
				case "b_independencePopup_NoiseAmb":
				case "b_independencePopup_Noise1":
				case "b_independencePopup_Noise2":
					SpHrView_IndependenceButtonPopUpHandler(ba.ctrlName, -1) 
					break
					
				// Spectrum - Edit Button
				case "b_SpectrumViewEdit_NoiseAmb":
				case "b_SpectrumViewEdit_Noise1":
				case "b_SpectrumViewEdit_Noise2":
					DoWindow SpHrView_SpectrumEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_SpectrumEdit_Window
					else
						SpHrView_SpectrumEditWindow(index)
					endif
					break
				// PopupContextualMenu - PopupContextualMenu Button
				case "b_spectrumPopup_NoiseAmb":
				case "b_spectrumPopup_Noise1":
				case "b_spectrumPopup_Noise2":
					SpHrView_SpectrumButtonPopUpHandler(ba.ctrlName, -1) 
					break
					
					
				// General Signal Button and PopupContextualMenu Buttons
				case "b_durationUnits_NoiseAmb":
				case "b_durationUnits_Noise1":
				case "b_durationUnits_Noise2":
				case "b_durationUnits_Tones":
					wave/T DurationUnits = $SpHrView_df_signalGlobals+"DurationUnits"
					PopupContextualMenu "ms;sec;points"
					if(V_flag>=1)
						// convert durations if necessary
						if(!stringmatch(DurationUnits[index],S_selection))
							wave Durations = $SpHrView_df_signalGlobals+"Durations"
							if(stringmatch(DurationUnits[index],"ms") && stringmatch(S_selection,"sec"))
								//DurationUnits[index] = S_selection
								Durations[index] /= 1000
								if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
									SetVariable sv_duration_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
								if(strsearch(ba.ctrlName,"_Noise1",0)>=0)
									SetVariable sv_duration_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
								if(strsearch(ba.ctrlName,"_Noise2",0)>=0)
									SetVariable sv_duration_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
								if(strsearch(ba.ctrlName,"_Tones",0)>=0)
									SetVariable sv_duration_Tones, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
							elseif(stringmatch(DurationUnits[index],"sec") && stringmatch(S_selection,"ms"))
								//DurationUnits[index] = S_selection
								Durations[index] *= 1000
								if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
									SetVariable sv_duration_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
								if(strsearch(ba.ctrlName,"_Noise1",0)>=0)
									SetVariable sv_duration_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
								if(strsearch(ba.ctrlName,"_Noise2",0)>=0)
									SetVariable sv_duration_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
								if(strsearch(ba.ctrlName,"_Tones",0)>=0)
									SetVariable sv_duration_Tones, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
							elseif(stringmatch(DurationUnits[index],"sec") && stringmatch(S_selection,"points"))
								NVAR fs = $SpHrView_df_signalGlobals+"fs"
								//DurationUnits[index] = S_selection
								Durations[index] *= fs
								if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
									SetVariable sv_duration_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
								if(strsearch(ba.ctrlName,"_Noise1",0)>=0)
									SetVariable sv_duration_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
								if(strsearch(ba.ctrlName,"_Noise2",0)>=0)
									SetVariable sv_duration_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
								if(strsearch(ba.ctrlName,"_Tones",0)>=0)
									SetVariable sv_duration_Tones, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
							elseif(stringmatch(DurationUnits[index],"ms") && stringmatch(S_selection,"points"))
								NVAR fs = $SpHrView_df_signalGlobals+"fs"
								//DurationUnits[index] = S_selection
								Durations[index] *= (fs / 1000)
								if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
									SetVariable sv_duration_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
								if(strsearch(ba.ctrlName,"_Noise1",0)>=0)
									SetVariable sv_duration_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
								if(strsearch(ba.ctrlName,"_Noise2",0)>=0)
									SetVariable sv_duration_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
								if(strsearch(ba.ctrlName,"_Tones",0)>=0)
									SetVariable sv_duration_Tones, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,1000}
								endif
							elseif(stringmatch(DurationUnits[index],"points") && stringmatch(S_selection,"sec"))
								NVAR fs = $SpHrView_df_signalGlobals+"fs"
								//DurationUnits[index] = S_selection
								Durations[index] /= fs
								if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
									SetVariable sv_duration_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
								if(strsearch(ba.ctrlName,"_Noise1",0)>=0)
									SetVariable sv_duration_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
								if(strsearch(ba.ctrlName,"_Noise2",0)>=0)
									SetVariable sv_duration_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
								if(strsearch(ba.ctrlName,"_Tones",0)>=0)
									SetVariable sv_duration_Tones, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,0.1}
								endif
							elseif(stringmatch(DurationUnits[index],"points") && stringmatch(S_selection,"ms"))
								NVAR fs = $SpHrView_df_signalGlobals+"fs"
								//DurationUnits[index] = S_selection
								Durations[index] /= (fs / 1000)
								if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
									SetVariable sv_duration_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
								if(strsearch(ba.ctrlName,"_Noise1",0)>=0)
									SetVariable sv_duration_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
								if(strsearch(ba.ctrlName,"_Noise2",0)>=0)
									SetVariable sv_duration_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
								if(strsearch(ba.ctrlName,"_Tones",0)>=0)
									SetVariable sv_duration_Tones, win=SpHrView_BinauralSignal_Window, limits={0,1e+10,10}
								endif
							endif
						endif
						DurationUnits[index] = S_selection
					endif
					break
					
				case "b_make":
					SpHrView_SignalMakeSound(0)
					break
				case "b_play":
					SpHrView_SignalMakeSound(1)
					break
									
				case "b_sampleRatePopUp":			
					NVAR fs = $SpHrView_df_signalGlobals+"fs"
					SVAR/Z rateListStr = $SpHrView_df_signalGlobals+"Audio:rateList"
					if(SVAR_Exists(rateListStr))
						PopupContextualMenu rateListStr
						if(V_flag>=1)
							fs = str2num(S_selection)
						endif
					endif
					break
					
				case "b_amplitudeUnits_NoiseAmb":
				case "b_amplitudeUnits_Noise1":
				case "b_amplitudeUnits_Noise2":
				case "b_amplitudeUnits_Tones":
					wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
					PopupContextualMenu "dB re:20µPa;Pa"
					if(V_flag>=1)
						// convert amplitudes if necessary
						if(!stringmatch(AmplitudeUnits[index],S_selection))
							wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
							if(stringmatch(AmplitudeUnits[index],"dB re:20µPa") && stringmatch(S_selection,"Pa"))
									Amplitudes[index] = (10^(Amplitudes[index]/20))*0.00002			// assuming re 20 µPa
									Amplitudes[index] = round(Amplitudes[index]*10000)/10000			// round with precision
									if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
										SetVariable sv_amplitude_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,2,0.02}
									elseif(strsearch(ba.ctrlName,"_Noise1",0)>=0)
										SetVariable sv_amplitude_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,2,0.02}
									elseif(strsearch(ba.ctrlName,"_Noise2",0)>=0)
										SetVariable sv_amplitude_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,2,0.02}
									elseif(strsearch(ba.ctrlName,"_Tones",0)>=0)
										SetVariable sv_amplitude_Tones, win=SpHrView_BinauralSignal_Window, limits={0,2,0.02}
									endif
							elseif(stringmatch(AmplitudeUnits[index],"Pa") && stringmatch(S_selection,"dB re:20µPa"))
									Amplitudes[index] = 20*log(Amplitudes[index]/0.00002) 				// re 20 µPa
									Amplitudes[index] = round(Amplitudes[index]*10)/10					// round with precision
									if(strsearch(ba.ctrlName,"_NoiseAmb",0)>=0)
										SetVariable sv_amplitude_NoiseAmb, win=SpHrView_BinauralSignal_Window, limits={0,120,6}
									elseif(strsearch(ba.ctrlName,"_Noise1",0)>=0)
										SetVariable sv_amplitude_Noise1, win=SpHrView_BinauralSignal_Window, limits={0,120,6}
									elseif(strsearch(ba.ctrlName,"_Noise2",0)>=0)
										SetVariable sv_amplitude_Noise2, win=SpHrView_BinauralSignal_Window, limits={0,120,6}
									elseif(strsearch(ba.ctrlName,"_Tones",0)>=0)
										SetVariable sv_amplitude_Tones, win=SpHrView_BinauralSignal_Window, limits={0,120,6}
									endif
							endif
							AmplitudeUnits[index] = S_selection
						endif
					endif
					break
					
				case "b_toneTypePopup_Tones":
					SpHrView_ToneButtonPopUpHandler("", 0)
					break
					
				case "b_toneViewEdit_Tones":
					DoWindow SpHrView_TonesEdit_Window
					if(v_flag)
						killwindow/Z SpHrView_TonesEdit_Window
					else
						SpHrView_TonesEditWindow()
					endif
					break
				
			endswitch
			SpHrView_BinauralSignalWindowColors(index)
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

Function SpHrView_Signal_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			Variable checked = cba.checked
			strswitch( cba.ctrlName )
				case "ck_noise_Noise1":
				case "ck_noise_Noise2":
					NVAR makeNoise1 = $SpHrView_df_signalGlobals+"makeNoise1"
					NVAR makeNoise2 = $SpHrView_df_signalGlobals+"makeNoise2"
					NVAR same12NoiseSeeds = $SpHrView_df_signalGlobals+"same12NoiseSeeds"
					if(makeNoise1 && makeNoise2)
						CheckBox ck_noise_Noise2SameAsNoise1, win=$cba.win, disable = 0
					else
						CheckBox ck_noise_Noise2SameAsNoise1, win=$cba.win, disable = 2
					endif
					SpHrView_BinauralSignalWindowColors(-1)
					if(checked)
						CheckBox $cba.ctrlName fColor=(0,0,0)
					endif
					break
				case "ck_signalAmbient":
				case "ck_tones":
					if(checked)
						CheckBox $cba.ctrlName fColor=(0,0,0)
					endif
					break
					
				case "ck_HRIR_useCDFC_Noise1":
					wave HRIR_useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC"
					HRIR_useCDFC[1] = checked
					break
				case "ck_HRIR_useCDFC_Noise2":
					wave HRIR_useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC"
					HRIR_useCDFC[2] = checked
					break
				case "ck_HRIR_useCDFC_Tones":
					wave HRIR_useCDFC = $SpHrView_df_signalGlobals+"HRIR_useCDFC"
					HRIR_useCDFC[3] = checked
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End


// =================================================================================================
// SpHrView_SignalRedimensionEditPoints(SelPnts, xwave, ywave), Redimensions and interpolates xwave, ywave
//		when choosing between 6, 11, or 21 point lengths
// =================================================================================================
function SpHrView_SignalRedimensionEditPoints(SelPnts, xwave, ywave)
	variable SelPnts
	wave xwave, ywave
	variable OrigPnts = dimsize(xwave,0)
	if(OrigPnts > 22)
		DoAlert/T="Can't Redimension wave" 0, "Wave Redimension limited to <= 22 points."
		return OrigPnts
	endif
	if(OrigPnts != dimsize(ywave,0))
		return OrigPnts
	endif
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	Duplicate/O xwave, xwave_copy
	Duplicate/O ywave, ywave_copy
	variable NewPnts, i, r, pnt // 6, 11, 21 // NewPnts = OrigYPnts+(OrigYPnts-1) or = (OrigPnts+1)/2
	if(SelPnts==6)
		NewPnts = 6
	elseif(SelPnts==11)
		NewPnts = 11
	elseif(SelPnts==21)
		NewPnts = 21
	elseif(SpHr_equal(SelPnts,OrigPnts,0.001))
		return OrigPnts
	elseif(SpHr_equal(SelPnts,5.9,0.001))
		NewPnts = 21 // loop 
	elseif(SpHr_equal(SelPnts,6.1,0.001))
		NewPnts = 11
	elseif(SpHr_equal(SelPnts,11.1,0.001))
		NewPnts = 21
	elseif(SpHr_equal(SelPnts,20.9,0.001))
		NewPnts = 11
	elseif(SpHr_equal(SelPnts,10.9,0.001))
		NewPnts = 6
	elseif(SpHr_equal(SelPnts,21.1,0.001))
		NewPnts = 6	// loop
	elseif(SelPnts < 5.9)
		NewPnts = 6
	elseif(SelPnts >= 9 && SelPnts <= 16)
		NewPnts = 11
	elseif(SelPnts > 16)
		NewPnts = 21
	endif
	if((OrigPnts == 6 && NewPnts == 11) || (OrigPnts == 11 && NewPnts == 21))
		NewPnts = OrigPnts + (OrigPnts - 1)
		redimension/N=(NewPnts) xwave, ywave
		for(i=0;i<OrigPnts;++i)
			pnt = i*2
			xwave[pnt] = xwave_copy[i]
			ywave[pnt] = ywave_copy[i]
			if(i<OrigPnts-1)
				xwave[pnt+1] = 10^( (log(xwave_copy[i]) + log(xwave_copy[i+1])) / 2 )
				ywave[pnt+1] = (ywave_copy[i] + ywave_copy[i+1]) / 2
			endif
		endfor
	elseif(OrigPnts == 6 && NewPnts == 21)
		for(r=0;r<2;++r)
			NewPnts = OrigPnts + (OrigPnts - 1)
			redimension/N=(NewPnts) xwave, ywave
			for(i=0;i<OrigPnts;++i)
				pnt = i*2
				xwave[pnt] = xwave_copy[i]
				ywave[pnt] = ywave_copy[i]
				if(i<OrigPnts-1)
					xwave[pnt+1] = 10^( (log(xwave_copy[i]) + log(xwave_copy[i+1])) / 2 )
					ywave[pnt+1] = (ywave_copy[i]+ywave_copy[i+1]) / 2
				endif
			endfor
			Duplicate/O xwave, xwave_copy
			Duplicate/O ywave, ywave_copy
			OrigPnts = dimsize(xwave,0)
		endfor
	elseif((OrigPnts == 11 && NewPnts == 6) || (OrigPnts==21 && NewPnts == 11))
		NewPnts = (OrigPnts + 1)/2
		redimension/N=(NewPnts) xwave, ywave
		for(i=0;i<NewPnts;++i)
			pnt = i*2
			xwave[i] = xwave_copy[pnt]
			ywave[i] = ywave_copy[pnt]
		endfor
		
	elseif(OrigPnts == 21 && NewPnts == 6)
		for(r=0;r<2;++r)
			NewPnts = (OrigPnts + 1)/2
			redimension/N=(NewPnts) xwave, ywave
			for(i=0;i<NewPnts;++i)
				pnt = i*2
				xwave[i] = xwave_copy[pnt]
				ywave[i] = ywave_copy[pnt]
			endfor
			Duplicate/O xwave, xwave_copy
			Duplicate/O ywave, ywave_copy
			OrigPnts = dimsize(xwave,0)
		endfor
	endif
	killwaves/Z xwave_copy, ywave_copy	
	return NewPnts
End



function SpHrView_SignalInterpEditPoints(ywave, xwave, destSpec, interpTypeStr)
	wave ywave, xwave
	string destSpec, interpTypeStr
	strswitch(interpTypeStr)
		case "Linear":
			SpHr_Interpolate /Log/D=5 /T=1 /Dest=$destSpec ywave, xwave
			break
		case "Spline":
			SpHr_Interpolate /Log/D=5 /T=2 /Dest=$destSpec ywave, xwave // xwave_log
			break
		case "Quadratic":
			SpHr_Interpolate /Log/D=5 /T=4 /Dest=$destSpec ywave, xwave
			break
	endswitch
	wave w = $destSpec
	Wavestats/Q w
	if(V_numNans > 0)
		DoAlert/T="Warning" 0, "Error while computing "+interpTypeStr+" interpolation. Increase the number of points or choose another type of interpolation"
	endif
End


// =================================================================================================
// SpHrView_SpectrumDefaults(SignalIndex), Spectrum Defaults
// =================================================================================================
function SpHrView_SpectrumDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	// Make "SpHrView:Signals:Globals:SpectrumShape" folder, SpHrView_df_signalGlobals+"Spectrum:"
	String DF = SpHrView_df_signalGlobals+"Spectrum:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/Z Spectrum_LowHz = $DF+"Spectrum_LowHz"
	if(!waveexists(Spectrum_LowHz))
		Make/O/N=(4) $DF+"Spectrum_LowHz"; wave Spectrum_LowHz = $DF+"Spectrum_LowHz"
		Spectrum_LowHz = 50;
	endif
	wave/Z Spectrum_LowHz2 = $DF+"Spectrum_LowHz2"
	if(!waveexists(Spectrum_LowHz2))
		Make/O/N=(4) $DF+"Spectrum_LowHz2"; wave Spectrum_LowHz2 = $DF+"Spectrum_LowHz2"
		Spectrum_LowHz2 = 50;
	endif
	wave/Z Spectrum_HighHz = $DF+"Spectrum_HighHz"
	if(!waveexists(Spectrum_HighHz))
		Make/O/N=(4) $DF+"Spectrum_HighHz"; wave Spectrum_HighHz = $DF+"Spectrum_HighHz"
		Spectrum_HighHz = 1500;
	endif
	wave/Z Spectrum_HighHz2 = $DF+"Spectrum_HighHz2"
	if(!waveexists(Spectrum_HighHz2))
		Make/O/N=(4) $DF+"Spectrum_HighHz2"; wave Spectrum_HighHz2 = $DF+"Spectrum_HighHz2"
		Spectrum_HighHz2 = 1500;
	endif
	wave/Z Spectrum_RefHz = $DF+"Spectrum_RefHz"
	if(!waveexists(Spectrum_RefHz))
		Make/O/N=(4) $DF+"Spectrum_RefHz"; wave Spectrum_RefHz = $DF+"Spectrum_RefHz"
		Spectrum_RefHz = 1000;
	endif
	wave/Z Spectrum_RefHz2 = $DF+"Spectrum_RefHz2"
	if(!waveexists(Spectrum_RefHz2))
		Make/O/N=(4) $DF+"Spectrum_RefHz2"; wave Spectrum_RefHz2 = $DF+"Spectrum_RefHz2"
		Spectrum_RefHz2 = 1000;
	endif
	wave/Z Spectrum_Hz = $DF+"Spectrum_Hz"
	if(!waveexists(Spectrum_Hz))
		Make/O/N=(4) $DF+"Spectrum_Hz"; wave Spectrum_Hz = $DF+"Spectrum_Hz"
		Spectrum_Hz = 1000;
	endif
	wave/Z Spectrum_Hz2 = $DF+"Spectrum_Hz2"
	if(!waveexists(Spectrum_Hz2))
		Make/O/N=(4) $DF+"Spectrum_Hz2"; wave Spectrum_Hz2 = $DF+"Spectrum_Hz2"
		Spectrum_Hz2 = 1000;
	endif
	wave/Z OctaveDenominator = $DF+"OctaveDenominator"
	if(!waveexists(OctaveDenominator))
		Make/O/N=(4) $DF+"OctaveDenominator"; wave OctaveDenominator = $DF+"OctaveDenominator"
		OctaveDenominator = 3;
	endif
	wave/Z OctaveDenominator2 = $DF+"OctaveDenominator2"
	if(!waveexists(OctaveDenominator2))
		Make/O/N=(4) $DF+"OctaveDenominator2"; wave OctaveDenominator2 = $DF+"OctaveDenominator2"
		OctaveDenominator2 = 3;
	endif
	wave/Z NumberEditPoints = $DF+"NumberEditPoints"
	if(!waveexists(NumberEditPoints))
		Make/O/N=(4) $DF+"NumberEditPoints"; wave NumberEditPoints = $DF+"NumberEditPoints"
		NumberEditPoints = 6;
	endif
	wave/Z NumberEditPoints2 = $DF+"NumberEditPoints2"
	if(!waveexists(NumberEditPoints2))
		Make/O/N=(4) $DF+"NumberEditPoints2"; wave NumberEditPoints2 = $DF+"NumberEditPoints2"
		NumberEditPoints2 = 6;
	endif
	
	wave/T/Z InterpType = $DF+"InterpType"
	if(!waveexists(InterpType))
		Make/T/O/N=(4) $DF+"InterpType"; wave/T InterpType = $DF+"InterpType"
		InterpType = "Linear";
	endif
	wave/T/Z InterpType2 = $DF+"InterpType2"
	if(!waveexists(InterpType2))
		Make/T/O/N=(4) $DF+"InterpType2"; wave/T InterpType2 = $DF+"InterpType2"
		InterpType2 = "Linear";
	endif
	
	wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
	wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
	if(!waveexists(Spectrum_Edit_Xs) || !waveexists(Spectrum_Edit_Ys))
		Make/O/N=(6) Spectrum_Edit_Xs, Spectrum_Edit_Ys
		SetScale/I x, 0, fs/2, "Hz" Spectrum_Edit_Xs
		SetScale d 0,0,"Hz", Spectrum_Edit_Xs
		SetScale d 0,0,"dB", Spectrum_Edit_Ys // !! units are dB or Pa/V, defaults to dB
	endif
	
	//wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
	//if( !LinkSpectrumChannels[SignalIndex] && (!waveexists(Spectrum_Edit_Xs2) || !waveexists(Spectrum_Edit_Ys2)) )
	wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
	wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
	if(!waveexists(Spectrum_Edit_Xs2) || !waveexists(Spectrum_Edit_Ys2) )
		Make/O/N=(6) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2
		SetScale/I x, 0, fs/2, "Hz" Spectrum_Edit_Xs2
		SetScale d 0,0,"Hz", Spectrum_Edit_Xs2
		SetScale d 0,0,"dB", Spectrum_Edit_Ys2 // !! units are dB or Pa/V, defaults to dB
	endif
	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_SpectrumEditWindow(SignalIndex), Spectrum window
// =================================================================================================
function SpHrView_SpectrumEditWindow(SignalIndex)
	variable SignalIndex // 0: bkgrd, 1: Noise_Noise1, 2:Noise_Noise2, 3: Tones
	DFREF dfSav = GetDataFolderDFR()	
	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"	
	wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"

	String DF = SpHrView_df_signalGlobals+"Spectrum:"
	SetDataFolder $DF
	
	wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
	if(!waveexists(Spectrum_Edit_Xs))
		SpHrView_SpectrumDefaults()
	endif
	
	wave Spectrum_LowHz = $DF+"Spectrum_LowHz"
	wave Spectrum_LowHz2 = $DF+"Spectrum_LowHz2"
	wave Spectrum_HighHz = $DF+"Spectrum_HighHz"
	wave Spectrum_HighHz2 = $DF+"Spectrum_HighHz2"
	wave Spectrum_RefHz = $DF+"Spectrum_RefHz"
	wave Spectrum_RefHz2 = $DF+"Spectrum_RefHz2"
	wave Spectrum_Hz = $DF+"Spectrum_Hz"
	wave Spectrum_Hz2 = $DF+"Spectrum_Hz2"
	wave OctaveDenominator = $DF+"OctaveDenominator"
	wave OctaveDenominator2 = $DF+"OctaveDenominator2"
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave NumberEditPoints2 = $DF+"NumberEditPoints2"
	wave/T InterpType = $DF+"InterpType"
	wave/T InterpType2 = $DF+"InterpType2"
	wave Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
	wave Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
	wave Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
	wave Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
	
	SpHrView_SpectrumEditMakePoints(SignalIndex)

	variable winHght = 400
	variable winWdth = 770
	KillWindow/Z SpHrView_SpectrumEdit_Window
	Display/K=1/W=(330,110,330+winWdth,110+winHght) as "Spectrum"
	DoWindow/C SpHrView_SpectrumEdit_Window
	//ModifyGraph width=610,height=288
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=18,margin(top)=7
	if(LinkSpectrumChannels[SignalIndex])
		ControlBar 27
	else
		ControlBar 50
	endif
	Variable leftPos = 35
	Variable ch1Top = 3
	Variable ch2Top = 25
	
	leftPos = 3
	TitleBox tb_left title=" 🔗",fSize=10, pos={leftPos,ch1Top+5},frame=0
	
	leftPos = 40
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={83,20},pos={leftPos,ch1Top+3},bodyWidth=95,fSize=9,value=SpectrumShape[SignalIndex]
	//SetVariable sv_Values proc=DFTSignalCues_ButtonProc
	Button b_ValuesPopUp title="",pos={leftPos+83,ch1Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 105
	Button b_Edit title="✏️",pos={leftPos,ch1Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 10
	SetVariable sv_Pnts title="Points",focusRing=0,size={83,20},pos={leftPos,ch1Top+3},bodyWidth=30,value=NumberEditPoints[SignalIndex]
	SetVariable sv_Pnts limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 60
	GroupBox g_SpectrumVariables pos={leftPos+29,ch1Top-1},size={435,22}
	TitleBox tb_Hz title="Hz:",pos={leftPos+35,ch1Top+5},frame=0
	
	SetVariable sv_Spectrum_RefHz title="Re:",focusRing=0,size={125,20},pos={leftPos, ch1Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_RefHz[SignalIndex],limits={0,20000,100}
	
	leftPos += 90
	SetVariable sv_Spectrum_Hz title="Center",focusRing=0,size={123,20},pos={leftPos, ch1Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_Hz[SignalIndex],limits={0,20000,100}
	SetVariable sv_OctaveDenominator title="Octave 1/",focusRing=0,size={120,20},pos={leftPos+90,ch1Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator[SignalIndex],limits={0.1,48,1}
	
	leftPos += 175
	SetVariable sv_Spectrum_LowHz title="Range:",focusRing=0,size={123,20},pos={leftPos+0, ch1Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_LowHz[SignalIndex],limits={0,10000,100}
	SetVariable sv_Spectrum_HighHz title="to",focusRing=0,size={125,20},pos={leftPos+65, ch1Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_HighHz[SignalIndex],limits={0,96000,100}

	leftPos += 135
	SetVariable sv_InterpType title=" ",focusRing=0,size={123,20},pos={leftPos+0, ch1Top+3},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc,value=InterpType[SignalIndex]
	Button b_InterpTypePopUp title="",pos={leftPos+123,ch1Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	
	strswitch(SpectrumShape[SignalIndex])
		case "Flat":
			//SetVariable sv_Pnts disable=2
			SetVariable sv_Spectrum_RefHz disable=2
			SetVariable sv_Spectrum_LowHz disable=2
			SetVariable sv_Spectrum_HighHz disable=2
			SetVariable sv_Spectrum_Hz disable=2
			SetVariable sv_OctaveDenominator disable=2
			SetVariable sv_InterpType disable=2
			Button b_InterpTypePopUp disable=2
			break
		case "1 / Frequency":
			//SetVariable sv_Pnts disable=2
			SetVariable sv_Spectrum_RefHz disable=0
			SetVariable sv_Spectrum_LowHz disable=2
			SetVariable sv_Spectrum_HighHz disable=2
			SetVariable sv_Spectrum_Hz disable=2
			SetVariable sv_OctaveDenominator disable=2
			SetVariable sv_InterpType disable=2
			Button b_InterpTypePopUp disable=2
			break
		case "Octave Band":
		case "Octave Notch":
			//SetVariable sv_Pnts disable=2
			SetVariable sv_Spectrum_RefHz disable=2
			SetVariable sv_Spectrum_LowHz disable=2
			SetVariable sv_Spectrum_HighHz disable=2
			SetVariable sv_Spectrum_Hz disable=0
			SetVariable sv_OctaveDenominator disable=0
			SetVariable sv_InterpType disable=2
			Button b_InterpTypePopUp disable=2
			break
		case "Range":
			//SetVariable sv_Pnts disable=2
			SetVariable sv_Spectrum_RefHz disable=2
			SetVariable sv_Spectrum_LowHz disable=0
			SetVariable sv_Spectrum_HighHz disable=0
			SetVariable sv_Spectrum_Hz disable=2
			SetVariable sv_OctaveDenominator disable=2
			SetVariable sv_InterpType disable=2
			Button b_InterpTypePopUp disable=2
			break
		case "Arbitrary":
			//SetVariable sv_Pnts disable=2
			SetVariable sv_Spectrum_RefHz disable=2
			SetVariable sv_Spectrum_LowHz disable=2
			SetVariable sv_Spectrum_HighHz disable=2
			SetVariable sv_Spectrum_Hz disable=2
			SetVariable sv_OctaveDenominator disable=2
			SetVariable sv_InterpType disable=0
			Button b_InterpTypePopUp disable=0
			break
		default:
			SetVariable sv_Spectrum_LowHz disable=0
			SetVariable sv_Spectrum_HighHz disable=0
			SetVariable sv_Spectrum_RefHz disable=0
			SetVariable sv_Spectrum_Hz disable=0
			SetVariable sv_OctaveDenominator disable=0
			SetVariable sv_InterpType disable=0
			Button b_InterpTypePopUp disable=0
			break
	endswitch


	wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
	if(!LinkSpectrumChannels[SignalIndex])
	
		wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
		
		leftPos = 3
		TitleBox tb_left title="Left",pos={leftPos,ch1Top+5},frame=0
		TitleBox tb_right title="Right",pos={leftPos,ch2Top+5},frame=0
	
		leftPos = 40
		SetVariable sv_Values2 title=" ",noedit=1,focusRing=0,size={83,20},pos={leftPos,ch2Top+3},bodyWidth=95,fSize=9,value=SpectrumShape2[SignalIndex]
		//SetVariable sv_Values2 proc=DFTSignalCues_ButtonProc
		Button b_ValuesPopUp2 title="",pos={leftPos+83,ch2Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
		
		leftPos += 105
		Button b_Edit2 title="✏️",pos={leftPos,ch2Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
		
		leftPos += 10
		SetVariable sv_Pnts2 title="Points",focusRing=0,size={83,20},pos={leftPos,ch2Top+3},bodyWidth=30,value=NumberEditPoints2[SignalIndex]
		SetVariable sv_Pnts2 limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
		
		leftPos += 60
		GroupBox g_SpectrumVariables2 pos={leftPos+29,ch2Top-1},size={435,22}
		TitleBox tb_Hz2 title="Hz:",pos={leftPos+35,ch2Top+5},frame=0
		
		SetVariable sv_Spectrum_RefHz2 title="Re:",focusRing=0,size={125,20},pos={leftPos+0, ch2Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_RefHz2[SignalIndex],limits={0,20000,100}
		
		leftPos += 90
		SetVariable sv_Spectrum_Hz2 title="Center",focusRing=0,size={123,20},pos={leftPos, ch2Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_Hz2[SignalIndex],limits={0,20000,100}
		SetVariable sv_OctaveDenominator2 title="Octave 1 /",focusRing=0,size={120,20},pos={leftPos+90, ch2Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator2[SignalIndex],limits={0.1,48,1}
	
		leftPos += 175
		SetVariable sv_Spectrum_LowHz2 title="Range:",focusRing=0,size={123,20},pos={leftPos+0, ch2Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_LowHz2[SignalIndex],limits={0,10000,100}
		SetVariable sv_Spectrum_HighHz2 title="to",focusRing=0,size={125,20},pos={leftPos+65, ch2Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=Spectrum_HighHz2[SignalIndex],limits={0,96000,100}

		leftPos += 135
		SetVariable sv_InterpType2 title=" ",focusRing=0,size={123,20},pos={leftPos+0, ch2Top+3},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc,value=InterpType2[SignalIndex]
		Button b_InterpTypePopUp2 title="",pos={leftPos+123,ch2Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
		strswitch(SpectrumShape2[SignalIndex])
			case "Flat":
				//SetVariable sv_Pnts2 disable=2
				SetVariable sv_Spectrum_RefHz2 disable=2
				SetVariable sv_Spectrum_LowHz2 disable=2
				SetVariable sv_Spectrum_HighHz2 disable=2
				SetVariable sv_Spectrum_Hz2 disable=2
				SetVariable sv_OctaveDenominator2 disable=2
				SetVariable sv_InterpType2 disable=2
				Button b_InterpTypePopUp2 disable=2
				break
			case "1 / Frequency":
				//SetVariable sv_Pnts disable=2
				SetVariable sv_Spectrum_RefHz2 disable=0
				SetVariable sv_Spectrum_LowHz2 disable=2
				SetVariable sv_Spectrum_HighHz2 disable=2
				SetVariable sv_Spectrum_Hz2 disable=2
				SetVariable sv_OctaveDenominator2 disable=2
				SetVariable sv_InterpType2 disable=2
				Button b_InterpTypePopUp2 disable=2
				break
			case "Octave Band":
			case "Octave Notch":
				//SetVariable sv_Pnts disable=2
				SetVariable sv_Spectrum_RefHz2 disable=2
				SetVariable sv_Spectrum_LowHz2 disable=2
				SetVariable sv_Spectrum_HighHz2 disable=2
				SetVariable sv_Spectrum_Hz2 disable=0
				SetVariable sv_OctaveDenominator2 disable=0
				SetVariable sv_InterpType2 disable=2
				Button b_InterpTypePopUp2 disable=2
				break
			case "Range":
				//SetVariable sv_Pnts disable=2
				SetVariable sv_Spectrum_RefHz2 disable=2
				SetVariable sv_Spectrum_LowHz2 disable=0
				SetVariable sv_Spectrum_HighHz2 disable=0
				SetVariable sv_Spectrum_Hz2 disable=2
				SetVariable sv_OctaveDenominator2 disable=2
				SetVariable sv_InterpType2 disable=2
				Button b_InterpTypePopUp2 disable=2
				break
			case "Arbitrary":
				//SetVariable sv_Pnts disable=2
				SetVariable sv_Spectrum_RefHz2 disable=2
				SetVariable sv_Spectrum_LowHz2 disable=2
				SetVariable sv_Spectrum_HighHz2 disable=2
				SetVariable sv_Spectrum_Hz2 disable=2
				SetVariable sv_OctaveDenominator2 disable=2
				SetVariable sv_InterpType2 disable=0
				Button b_InterpTypePopUp2 disable=0
				break
			default:
				SetVariable sv_Spectrum_LowHz2 disable=0
				SetVariable sv_Spectrum_HighHz2 disable=0
				SetVariable sv_Spectrum_RefHz2 disable=0
				SetVariable sv_Spectrum_Hz2 disable=0
				SetVariable sv_OctaveDenominator2 disable=0
				SetVariable sv_InterpType2 disable=0
				Button b_InterpTypePopUp2 disable=0
				break
		endswitch
		
	endif	// if(!LinkSpectrumChannels[SignalIndex])

	AppendToGraph Spectrum_Edit_Ys vs Spectrum_Edit_Xs
	ModifyGraph mode(Spectrum_Edit_Ys)=4,marker(Spectrum_Edit_Ys)=19,msize(Spectrum_Edit_Ys)=2.5,lsize(Spectrum_Edit_Ys)=1,rgb(Spectrum_Edit_Ys)=(0,0,0)
	
	if(!LinkSpectrumChannels[SignalIndex])
		AppendToGraph Spectrum_Edit_Ys2 vs Spectrum_Edit_Xs2
		ModifyGraph rgb(Spectrum_Edit_Ys)=(0,0,65535)
		ModifyGraph mode(Spectrum_Edit_Ys2)=4,marker(Spectrum_Edit_Ys2)=19,msize(Spectrum_Edit_Ys2)=2.5,lsize(Spectrum_Edit_Ys2)=1,rgb(Spectrum_Edit_Ys2)=(65535,0,0)
	endif
	
	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid=1, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	ModifyGraph lblMargin(left)=10
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5
	Label left "Spectrum Scale (\\U)"
	ModifyGraph tickEnab(left)={-50,inf}
	
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
	else
		ModifyGraph log=0
	endif
	SetAxis/A/N=0/E=1/W=SpHrView_SpectrumEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
	Label bottom "Frequency (\\U)"
	
	wavestats/Q/M=1 Spectrum_Edit_Ys
	variable Yrange = v_max-v_min
	if(!LinkSpectrumChannels[SignalIndex])
		wavestats/Q/M=1 Spectrum_Edit_Ys2
	 	Yrange = max(Yrange, v_max-v_min)
	endif
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
	wave/T/Z SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
	if(Yrange < 6)
		SetAxis/W=SpHrView_SpectrumEdit_Window left -6,6
	else
		strswitch(SpectrumShape[SignalIndex])
			case "Flat":
			case "1 / Frequency":
				SetAxis/A=2/N=0/E=2 left
				break
			default:
				SetAxis/A=2/E=0 left
		endswitch
		strswitch(SpectrumShape2[SignalIndex])
			case "Flat":
			case "1 / Frequency":
				SetAxis/A=2/N=0/E=2 left
				break
		endswitch
	endif
	
	//  append interpolated lines if they are being used
	wave/T SpectrumInterp = $DF+"SpectrumInterp"
	strswitch(SpectrumShape[SignalIndex])
		case "Flat":
		case "1 / Frequency":
			break
		default:
			if(waveexists(SpectrumInterp))
				appendtograph SpectrumInterp
				ModifyGraph lsize(SpectrumInterp)=4,rgb(SpectrumInterp)=(0,0,65535,13107)
			endif
	endswitch
	wave/T/Z SpectrumInterp2 = $DF+"SpectrumInterp2"
	strswitch(SpectrumShape[SignalIndex])
		case "Flat":
		case "1 / Frequency":
			break
		default:
			if( waveexists(SpectrumInterp2) && !LinkSpectrumChannels[SignalIndex])
				appendtograph SpectrumInterp2
				ModifyGraph lsize(SpectrumInterp2)=4,rgb(SpectrumInterp2)=(65535,0,0,13107)
			endif
	endswitch
	
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=fs/2,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=20,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc

	SetWindow SpHrView_SpectrumEdit_Window userdata(SignalData)="SIGNALPARAM:Spectrum;SIGNALNUM:"+num2str(SignalIndex)
	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_SpectrumSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_SpectrumSetVarHandler(string ctrlName, variable index, variable dval) 
	String DF = SpHrView_df_signalGlobals+"Spectrum:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		case "sv_Pnts":
			wave/Z Xs = $DF+"Spectrum_Edit_Xs"
			if(!waveexists(Xs))
				SpHrView_SpectrumDefaults()
			endif
			wave/Z Xs = $DF+"Spectrum_Edit_Xs"
			wave/Z Ys = $DF+"Spectrum_Edit_Ys"
			wave NumberEditPoints = $DF+"NumberEditPoints"
			NumberEditPoints[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs, Ys)						
			SpHrView_SpectrumEditMakePoints(index)	// ??
			SetAxis/A/N=0/E=1/W=SpHrView_SpectrumEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
			wave/T InterpType = $DF+"InterpType"
			SpHrView_SignalInterpEditPoints(Ys, Xs, DF+"SpectrumInterp", InterpType[index])
			break
			
		case "sv_Pnts2":
			wave/Z Xs2 = $DF+"Spectrum_Edit_Xs2"
			wave/Z Ys2 = $DF+"Spectrum_Edit_Ys2"
			wave NumberEditPoints2 = $DF+"NumberEditPoints2"
			NumberEditPoints2[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs2, Ys2)
			SpHrView_SpectrumEditMakePoints(index) // ??
			SetAxis/A/N=0/E=1/W=SpHrView_SpectrumEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
			wave/T InterpType2 = $DF+"InterpType2"
			SpHrView_SignalInterpEditPoints(Ys2, Xs2, DF+"SpectrumInterp2", InterpType2[index])
			break
			
		case "sv_Spectrum_RefHz":
		case "sv_Spectrum_LowHz":
		case "sv_Spectrum_HighHz":
		case "sv_Spectrum_Hz":
		case "sv_OctaveDenominator":
		case "sv_Spectrum_RefHz2":
		case "sv_Spectrum_LowHz2":
		case "sv_Spectrum_HighHz2":
		case "sv_Spectrum_Hz2":
		case "sv_OctaveDenominator2":
		
			//wave NumberEditPoints = $DF+"NumberEditPoints"
			wave Spectrum_RefHz = $DF+"Spectrum_RefHz"
			wave Spectrum_LowHz = $DF+"Spectrum_LowHz"
			wave Spectrum_HighHz = $DF+"Spectrum_HighHz"
			if(Spectrum_HighHz[index] > fs*0.5)
					Spectrum_HighHz[index] = fs*0.5
			endif
			if(Spectrum_LowHz[index] < 1)
					Spectrum_LowHz[index] = 1
			endif
			if(Spectrum_RefHz[index] > fs*0.5)
					Spectrum_RefHz[index] = 10000
			endif
			variable tempVar
			if(Spectrum_HighHz[index] < Spectrum_LowHz[index])
				tempVar = Spectrum_LowHz[index]
				Spectrum_LowHz[index] = Spectrum_HighHz[index]
				Spectrum_HighHz[index] = tempVar
				endif
				
			SpHrView_SpectrumEditMakePoints(index)
			SetAxis/A/N=0/E=1/W=SpHrView_SpectrumEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
			break
	endswitch		// strswitch( sva.ctrlName )
End

// =================================================================================================
// SpHrView_SpectrumButtonPopUpHandler(), Spectrum popup button handler
// =================================================================================================
function SpHrView_SpectrumButtonPopUpHandler(string ctrlName, variable index) 
	String DF = SpHrView_df_signalGlobals+"Spectrum:"
	String checked= "\\M0:!" + num2char(18) + ":" // checkmark code
	String str, SpectrumMenuStr
	String InterpTypeMenuStr = "Linear;Quadratic;Spline"
	Variable ch = 1 // 1 for left channel spectrum, 2 for right channel spectrum
	wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
	variable var
	
	strswitch( ctrlName )
		case "b_Edit2":					// for right channel spectrum
		case "b_ValuesPopUp2": 		// for right channel spectrum
		case "b_InterpTypePopUp2":	// for right channel spectrum
			ch = 2	// else ch = 1
			break
			
		case "b_spectrumPopup_NoiseAmb":	// SpHrView_BinauralSignal_Window
			index = 0
			break
		case "b_spectrumPopup_Noise1":		// SpHrView_BinauralSignal_Window
			index = 1
			break
		case "b_spectrumPopup_Noise2":		// SpHrView_BinauralSignal_Window
			index = 2
			break
	endswitch
	
	strswitch( ctrlName )
		//////////////////////////
		// SpHrView_BinauralSignal_Window
		//////////////////////////
		case "b_spectrumPopup_NoiseAmb":
		case "b_spectrumPopup_Noise1":
		case "b_spectrumPopup_Noise2":
			wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
			SpectrumMenuStr = "Flat;\\M01 / Frequency;Octave Band;Octave Notch;Range;-;"
			DF = SpHrView_df_signalGlobals+"Spectrum:"
			if(LinkSpectrumChannels[index])
				SpectrumMenuStr += checked+"Channels Linked;"
			else
				SpectrumMenuStr += "Channels Linked;"
			endif
			SpectrumMenuStr += "-;View Table;"
			// Add list of waves in SpHrView_df_signals
			SpectrumMenuStr += "-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(SpectrumShape[index], SpectrumMenuStr,";",0,0)
			if(var < 0) // look for special character interpretation...
				var = WhichListItem("\M01 / Frequency", SpectrumMenuStr,";",0,0)
			endif
			if(var >= 0)
				SpectrumMenuStr = RemoveListItem(var, SpectrumMenuStr) 
				SpectrumMenuStr = AddListItem(checked+SpectrumShape[index], SpectrumMenuStr, ";", var)
			endif
			PopupContextualMenu SpectrumMenuStr
			if(V_flag>=1)
				// toggle Channels Linked
				strswitch(S_selection)
					case "Channels Linked":
						LinkSpectrumChannels[index] = !LinkSpectrumChannels[index]
						if(LinkSpectrumChannels[index])
							wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
							SpectrumShape2[index] = SpectrumShape[index]
						endif
						SpHrView_SpectrumEditWindow(index)
						break
						
					case "View Table":
						wave/Z Xs = $DF+"Spectrum_Edit_Xs"
						if(!waveexists(Xs))
							SpHrView_SpectrumDefaults()
						endif
						wave Xs = $DF+"Spectrum_Edit_Ys", Ys = $DF+"Spectrum_Edit_Xs"
						if(waveexists(Ys) && waveexists(Xs))
							KillWindow/Z SpectrumValues
							Edit/K=1/W=(15,50,450,500) Ys, Xs as "Spectrum Values"
							DoWindow/C SpectrumValues
							if(!LinkSpectrumChannels[index])
								wave Xs2 = $DF+"Spectrum_Edit_Ys2", Ys2 = $DF+"Spectrum_Edit_Xs2"
								if(waveexists(Ys) && waveexists(Xs))
									AppendToTable Ys2, Xs2
								endif
							endif
						endif
						break
						
					case "Octave Band":
					case "Octave Notch":
					case "Range":
					case "Arbtrary":
						SpectrumShape[index] = S_selection
						SpHrView_SpectrumEditWindow(0)
						SetVariable sv_Spectrum_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						break
						
					case "Flat":
					case "1 / Frequency":
						SpectrumShape[index] = S_selection
						SetVariable sv_Spectrum_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=0
						dowindow SpHrView_SpectrumEdit_Window
						if(v_flag || !LinkSpectrumChannels[index])
							SpHrView_SpectrumEditWindow(0)
						endif
						break

				endswitch
			endif
			SpHrView_BinauralSignalWindowColors(index)
			break
	
		////////////////////////////////
		// SpHrView_SpectrumEdit_Window
		////////////////////////////////
		case "b_Edit":
			Button b_Edit rename=b_QuitEdit, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_SpectrumEdit_Window Spectrum_Edit_Ys
			wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
			SpectrumShape[index] = "Arbitrary"
			Button b_InterpTypePopUp win=SpHrView_SpectrumEdit_Window, disable=0
			SetVariable sv_InterpType win=SpHrView_SpectrumEdit_Window, disable=0
			/// get interpolation if Quadratic
			DF = SpHrView_df_signalGlobals+"Spectrum:"
			wave/T InterpType = $DF+"InterpType"
			wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
			wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
			SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys, Spectrum_Edit_Xs, DF+"SpectrumInterp", InterpType[index])
			break
		case "b_QuitEdit":
			Button b_QuitEdit rename=b_Edit, title="✏️"
			GraphNormal/W=SpHrView_SpectrumEdit_Window
			//wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
			//SpectrumShape[index] = "Arbitrary"
			DF = SpHrView_df_signalGlobals+"Spectrum:"
			wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
			wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
			if( waveexists(Spectrum_Edit_Xs) && waveexists(Spectrum_Edit_Ys) )
				Spectrum_Edit_Xs = round(Spectrum_Edit_Xs) // round to 1 Hz
				/// get interpolation
				wave/T InterpType = $DF+"InterpType"
				wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
				wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
				SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys, Spectrum_Edit_Xs, DF+"SpectrumInterp2", InterpType[index])
				///
				//SpHrView_SpectrumEditMakePoints(index)
				SpHrView_SpectrumEditWindow(index)
			endif
			break
		// 2
		case "b_Edit2":
			Button b_Edit2 rename=b_QuitEdit2, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_SpectrumEdit_Window Spectrum_Edit_Ys2
			wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
			SpectrumShape2[index] = "Arbitrary"
			Button b_InterpTypePopUp2 win=SpHrView_SpectrumEdit_Window, disable=0
			SetVariable sv_InterpType2 win=SpHrView_SpectrumEdit_Window, disable=0
			/// get interpolation
			DF = SpHrView_df_signalGlobals+"Spectrum:"
			wave/T InterpType2 = $DF+"InterpType2"
			wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
			wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
			SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys2, Spectrum_Edit_Xs2, DF+"SpectrumInterp2", InterpType2[index])
			break
		case "b_QuitEdit2":
			Button b_QuitEdit2 rename=b_Edit2, title="✏️"
			GraphNormal/W=SpHrView_SpectrumEdit_Window
			//wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
			//SpectrumShape2[index] = "Arbitrary"
			DF = SpHrView_df_signalGlobals+"Spectrum:"
			wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
			wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
			if( waveexists(Spectrum_Edit_Xs2) && waveexists(Spectrum_Edit_Ys2) )
				Spectrum_Edit_Xs2 = round(Spectrum_Edit_Xs2) // round to 1 Hz
				/// get interpolation
				wave/T InterpType2 = $DF+"InterpType2"
				wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
				wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
				SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys2, Spectrum_Edit_Xs2, DF+"SpectrumInterp2", InterpType2[index])
				///
				//SpHrView_SpectrumEditMakePoints(index)
				SpHrView_SpectrumEditWindow(index)
			endif
			break
			
			// Spectrum window pop-up
		case "b_ValuesPopUp":
		case "b_ValuesPopUp2":	// for right channel spectrum
			// build pop-up menu
			wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
			SpectrumMenuStr = "Flat;\\M01 / Frequency;Octave Band;Octave Notch;Range;-;"
			DF = SpHrView_df_signalGlobals+"Spectrum:"
			wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
			if(LinkSpectrumChannels[index])
				SpectrumMenuStr += checked+"Channels Linked;"
			else
				SpectrumMenuStr += "Channels Linked;"
			endif
			SpectrumMenuStr += "-;View Table;"
		
			if(ch==1 && stringmatch(SpectrumShape[index], "Arbitrary"))
				SpectrumMenuStr += "-;Save Arbitrary Wave;"
			endif
			wave/T/Z SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
			if(waveexists(SpectrumShape2))
				if(ch==2 && stringmatch(SpectrumShape2[index], "Arbitrary"))
					SpectrumMenuStr += "-;Save Arbitrary Wave;"
				endif
			endif
			// Add list of waves in SpHrView_df_signals
			SpectrumMenuStr += "-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(SpectrumShape[index], SpectrumMenuStr,";",0,0)
			if(var < 0) // look for special character interpretation...
				var = WhichListItem("\M01 / Frequency", SpectrumMenuStr,";",0,0)
			endif
			if(var >= 0)
				SpectrumMenuStr = RemoveListItem(var, SpectrumMenuStr) 
				SpectrumMenuStr = AddListItem(checked+SpectrumShape[index], SpectrumMenuStr, ";", var)
			endif
			// menu
			PopupContextualMenu SpectrumMenuStr
			if(V_flag>=1)
				// process selection
				strswitch(S_selection)
					case "Channels Linked":
						LinkSpectrumChannels[index] = !LinkSpectrumChannels[index]
						if(LinkSpectrumChannels[index])
							wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
							SpectrumShape2[index] = SpectrumShape[index]
						endif
						break
					// View Table
					case "View Table":
						wave Xs = $DF+"Spectrum_Edit_Ys", Ys = $DF+"Spectrum_Edit_Xs"
						if(waveexists(Ys) && waveexists(Xs))
							KillWindow/Z SpectrumValues
							Edit/K=1/W=(15,50,450,500) Ys, Xs as "Spectrum Values"
							DoWindow/C SpectrumValues
							if(!LinkSpectrumChannels[index])
								wave Xs2 = $DF+"Spectrum_Edit_Ys2", Ys2 = $DF+"Spectrum_Edit_Xs2"
								if(waveexists(Ys) && waveexists(Xs))
									AppendToTable Ys2, Xs2
								endif
							endif
						endif
						break
					
					case "Save Arbitrary Wave":
						if(ch==1)// && stringmatch(S_selection, "Save Arbitrary Wave"))
							str = "Spectrum_untitled"
							Prompt str, "Wave Name:"
							DoPrompt "Save Spectrum Wave?", str
							if(!V_Flag)
								wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
								wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
								Duplicate/O Spectrum_Edit_Ys, $SpHrView_df_signals+str
								wave savedWave = $SpHrView_df_signals+str
								Redimension/N=(-1,2) savedWave
								savedWave[][1] = Spectrum_Edit_Xs[p]
							endif

						elseif(ch==2) // && stringmatch(S_selection, "Save Arbitrary Wave"))
							str = "Spectrum2_untitled"
							Prompt str, "Wave Name:"
							DoPrompt "Save Spectrum (channel 2)", str
							if(!V_Flag)
								wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
								wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
								Duplicate/O Spectrum_Edit_Ys2, $SpHrView_df_signals+str
								wave savedWave = $SpHrView_df_signals+str
								Redimension/N=(-1,2) savedWave
								savedWave[][1] = Spectrum_Edit_Xs2[p]
							endif
						endif
						break
					
					case "Flat":
					case "1 / Frequency":
					case "Octave Band":
					case "Octave Notch":
					case "Range":
					
						if(ch==1)
							SpectrumShape[index] = S_selection
							//Print SpectrumShape[index]
							
						elseif(ch==2)
							wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
							SpectrumShape2[index] = S_selection
						endif
						break
							
						// must be saved wave if S_selection not recognized...
					default:
						// load saved wave into Spectrum_Edit_Xs amd Spectrum_Edit_Ys
						wave/Z w = $SpHrView_df_signals+S_selection
						if(ch==1 && waveexists(w))
							SpectrumShape[index] = S_selection
							wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
							wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
							wave NumberEditPoints = $DF+"NumberEditPoints"
							NumberEditPoints[index] = dimsize(w,0)
							redimension/N=(NumberEditPoints[index]) Spectrum_Edit_Ys, Spectrum_Edit_Xs
							Spectrum_Edit_Ys = w[p][0]
							Spectrum_Edit_Xs = w[p][1]
						elseif(ch==2 && waveexists(w))
							SpectrumShape2[index] = S_selection
							wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
							wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
							wave NumberEditPoints2 = $DF+"NumberEditPoints2"
							NumberEditPoints2[index] = dimsize(w,0)
							redimension/N=(NumberEditPoints2[index]) Spectrum_Edit_Ys2, Spectrum_Edit_Xs2
							Spectrum_Edit_Ys2 = w[p][0]
							Spectrum_Edit_Xs2 = w[p][1]
						endif
								
					endswitch
					
				SpHrView_SpectrumEditWindow(index)
				
			endif // PopupContextualMenu SpectrumMenuStr  if(V_flag>=1)
			break	// case "b_ValuesPopUp":  case "b_ValuesPopUp2":	// for right channel spectrum
			
		case "b_InterpTypePopUp":
		case "b_InterpTypePopUp2":
			PopupContextualMenu InterpTypeMenuStr
			if(V_flag>=1)
				DF = SpHrView_df_signalGlobals+"Spectrum:"
				if(ch==1)
					wave/T InterpType = $DF+"InterpType"
					InterpType[index] = S_selection
					wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
					wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
					SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys, Spectrum_Edit_Xs, DF+"SpectrumInterp", InterpType[index])
				elseif(ch==2)
					wave/T InterpType2 = $DF+"InterpType2"
					InterpType2[index] = S_selection
					wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
					wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
					SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys2, Spectrum_Edit_Xs2, DF+"SpectrumInterp2", InterpType2[index])
				endif
			endif
			break
			
	endswitch // strswitch( ba.ctrlName )
End

// =================================================================================================
// SpHrView_IndependenceDefaults(SignalIndex), Independence Defaults
// =================================================================================================
function SpHrView_IndependenceDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	// Make "SpHrView:Signals:Globals:Independence" folder, SpHrView_df_signalGlobals+"Independence:"
	String DF = SpHrView_df_signalGlobals+"Independence:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	//if(!NVAR_Exists(fs))
		//SpHrView_SignalDefaults()
	//endif
	//NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	
	wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
	if(!waveexists(Independence_Edit_Xs) || !waveexists(Independence_Edit_Ys))
		Make/O/N=(6) Independence_Edit_Xs, Independence_Edit_Ys
		SetScale/I x, 0, fs/2, "Hz" Independence_Edit_Xs
		SetScale d 0,0,"Hz", Independence_Edit_Xs
		SetScale d 0,0,"Index", Independence_Edit_Ys
	endif
	
	wave/Z Independence_Hz = $DF+"Independence_Hz"
	if(!waveexists(Independence_Hz))
		Make/O/N=(4) $DF+"Independence_Hz"; wave Independence_Hz = $DF+"Independence_Hz"
		Independence_Hz = 1000;
	endif
	wave/Z OctaveDenominator = $DF+"OctaveDenominator"
	if(!waveexists(OctaveDenominator))
		Make/O/N=(4) $DF+"OctaveDenominator"; wave OctaveDenominator = $DF+"OctaveDenominator"
		OctaveDenominator = 3;
	endif
	wave/Z Independence_LowHz = $DF+"Independence_LowHz"
	if(!waveexists(Independence_LowHz))
		Make/O/N=(4) $DF+"Independence_LowHz"; wave Independence_LowHz = $DF+"Independence_LowHz"
		Independence_LowHz = 50;
	endif
	wave/Z Independence_HighHz = $DF+"Independence_HighHz"
	if(!waveexists(Independence_HighHz))
		Make/O/N=(4) $DF+"Independence_HighHz"; wave Independence_HighHz = $DF+"Independence_HighHz"
		Independence_HighHz = 1500;
	endif
	
	wave/T/Z InterpType = $DF+"InterpType"
	if(!waveexists(InterpType))
		Make/T/O/N=(4) $DF+"InterpType"; wave/T InterpType = $DF+"InterpType"
		InterpType = "Linear";
	endif
	wave/Z NumberEditPoints = $DF+"NumberEditPoints"
	if(!waveexists(NumberEditPoints))
		Make/O/N=(4) $DF+"NumberEditPoints"; wave NumberEditPoints = $DF+"NumberEditPoints"
		NumberEditPoints = 6;
	endif
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_IndependenceEditWindow(SignalIndex), Independence window
// =================================================================================================
function SpHrView_IndependenceEditWindow(SignalIndex)
	variable SignalIndex // 0: bkgrd, 1: Noise_Noise1, 2:Noise_Noise2, 3: Tones
	DFREF dfSav = GetDataFolderDFR()	
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	String DF = SpHrView_df_signalGlobals+"Independence:"
	wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	if(!waveexists(Independence_Edit_Xs))
		SpHrView_IndependenceDefaults()
	endif
	wave Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	wave Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
	wave Independence_Hz = $DF+"Independence_Hz"
	wave OctaveDenominator = $DF+"OctaveDenominator"
	wave Independence_LowHz = $DF+"Independence_LowHz"
	wave Independence_HighHz = $DF+"Independence_HighHz"
	wave/T InterpType = $DF+"InterpType"
	wave NumberEditPoints = $DF+"NumberEditPoints"
	
	SpHrView_IndependenceEditMakePoints(SignalIndex)
	
	SpHrView_SignalInterpEditPoints(Independence_Edit_Ys, Independence_Edit_Xs, DF+"IndependenceInterp", InterpType[SignalIndex])

	variable winHght = 400
	variable winWdth = 770
	KillWindow/Z SpHrView_IndependenceEdit_Window
	Display/K=1/W=(310,90,310+winWdth,90+winHght) as "Independence"
	DoWindow/C SpHrView_IndependenceEdit_Window
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=18,margin(top)=7
	ControlBar 27
	
	variable Top = 3, leftPos = 5
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={130,20},pos={leftPos,Top+3},bodyWidth=130,fSize=9,value=Independence[SignalIndex]
	Button b_ValuesPopUp title="",pos={leftPos+130,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 155
	Button b_Edit title="✏️",pos={leftPos,Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 10
	SetVariable sv_Pnts title="Points",focusRing=0,size={83,20},pos={leftPos,Top+3},bodyWidth=30,value=NumberEditPoints[SignalIndex]
	SetVariable sv_Pnts limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 60
	GroupBox g_IndependenceVariables pos={leftPos+29,Top-1},size={425,22}
	
	leftPos += 66
	SetVariable sv_IndependenceVal,title="Index", pos={leftPos,Top+3},size={45.00,16.00},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc
	SetVariable sv_IndependenceVal,help={"Binaural independence."},fSize=10
	SetVariable sv_IndependenceVal,limits={0,1,0.05},value= root:SpHrView:Signals:Globals:IndependenceVal[SignalIndex]
	
	leftPos += 13
	SetVariable sv_Independence_Hz title="Center",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=Independence_Hz[SignalIndex],limits={0,20000,100}
	SetVariable sv_OctaveDenominator title="Octave 1/",focusRing=0,size={120,20},pos={leftPos+87,Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator[SignalIndex],limits={0.1,48,1}
	
	leftPos += 175
	SetVariable sv_Independence_LowHz title="Range:",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=Independence_LowHz[SignalIndex],limits={0,10000,100}
	SetVariable sv_Independence_HighHz title="to",focusRing=0,size={125,20},pos={leftPos+64, Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=Independence_HighHz[SignalIndex],limits={0,96000,100}

	leftPos += 135
	SetVariable sv_InterpType title=" ",focusRing=0,size={123,20},pos={leftPos+0, Top+3},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc,value=InterpType[SignalIndex]
	Button b_InterpTypePopUp title="",pos={leftPos+123,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	AppendToGraph Independence_Edit_Ys vs Independence_Edit_Xs
	ModifyGraph mode(Independence_Edit_Ys)=4,marker(Independence_Edit_Ys)=19,msize(Independence_Edit_Ys)=2.5,lsize(Independence_Edit_Ys)=1,rgb(Independence_Edit_Ys)=(0,0,0)

	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid=1, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	ModifyGraph lblMargin(left)=10
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5
	Label left "Independence (\\U)"
	ModifyGraph tickEnab(left)={-50,inf}
	
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
		Label bottom "Frequency\\u#2"
	else
		ModifyGraph log=0
		Label bottom "Frequency (\\U)"
	endif
	SetAxis/A/N=0/E=1/W=SpHrView_IndependenceEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
	
	SetAxis/W=SpHrView_IndependenceEdit_Window left -0.05, 1.05
	
	//  append interpolated lines if they are being used
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	wave/Z IndependenceInterp = $DF+"IndependenceInterp"
	if(!Stringmatch(Independence[SignalIndex], "Index") && waveexists(IndependenceInterp))
		appendtograph IndependenceInterp
		ModifyGraph lsize(IndependenceInterp)=4,rgb(IndependenceInterp)=(0,0,65535,13107)
	endif
	
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=fs/2,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=20,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc
	
	strswitch(Independence[SignalIndex])
		case "Index":
			SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_IndependenceVal win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_Independence_Hz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_LowHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_HighHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_IndependenceEdit_Window, disable=2
			Button/Z b_InterpTypePopUp win=SpHrView_IndependenceEdit_Window, disable=2
			break
		case "Independent Octave Band":
		case "Dependent Octave Band":
			SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_IndependenceVal win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_Independence_Hz win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_Independence_LowHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_HighHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_IndependenceEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_IndependenceEdit_Window, disable=0
			break
		case "Independent Range":
		case "Dependent Range":
			SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_IndependenceVal win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_Independence_Hz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_LowHz win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_Independence_HighHz win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_InterpType win=SpHrView_IndependenceEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_IndependenceEdit_Window, disable=0
			break
						
		case "Arbitrary":
			SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_IndependenceVal win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_Independence_Hz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_LowHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_HighHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_IndependenceEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_IndependenceEdit_Window, disable=0
			break

		default:
			SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable/Z sv_IndependenceVal win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_Hz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_LowHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable/Z sv_Independence_HighHz win=SpHrView_IndependenceEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_IndependenceEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_IndependenceEdit_Window, disable=0
			break
	endswitch
	
	SetWindow SpHrView_IndependenceEdit_Window userdata(SignalData)="SIGNALPARAM:Independence;SIGNALNUM:"+num2str(SignalIndex)

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_IndependenceSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_IndependenceSetVarHandler(string ctrlName, variable index, variable dval) 
	String DF = SpHrView_df_signalGlobals+"Independence:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		case "sv_Pnts":
			wave/Z Xs = $DF+"Independence_Edit_Xs"
			if(!waveexists(Xs))
				SpHrView_IndependenceDefaults()
			endif
			wave Xs = $DF+"Independence_Edit_Xs"
			wave Ys = $DF+"Independence_Edit_Ys"
			wave NumberEditPoints = $DF+"NumberEditPoints"
			NumberEditPoints[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs, Ys)						
			SpHrView_IndependenceEditMakePoints(index)	// ??
			SetAxis/A/N=0/E=1/W=SpHrView_IndependenceEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
			wave/T InterpType = $DF+"InterpType"
			SpHrView_SignalInterpEditPoints(Ys, Xs, DF+"IndependenceInterp", InterpType[index])
			break
			
		case "sv_IndependenceVal":	
		case "sv_IndependenceVal_NoiseAmb":
		case "sv_IndependenceVal_Noise1":	
		case "sv_IndependenceVal_Noise2":	
			
			wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
			if(SpHr_Equal(dval, 0, 0.000001))
				IndependenceVal[index] = 0.0
			else
				IndependenceVal[index] = dval
			endif
			
			SpHrView_BinauralSignalWindowColors(index)
			
			SpHrView_IndependenceEditMakePoints(index)
			break
			

		case "sv_Independence_LowHz":
		case "sv_Independence_HighHz":
		case "sv_Independence_Hz":
		case "sv_OctaveDenominator":
		
			wave Independence_LowHz = $DF+"Independence_LowHz"
			wave Independence_HighHz = $DF+"Independence_HighHz"
			if(Independence_HighHz[index] > fs*0.5)
					Independence_HighHz[index] = fs*0.5
			endif
			if(Independence_LowHz[index] < 1)
					Independence_LowHz[index] = 1
			endif
			variable tempVar
			if(Independence_HighHz[index] < Independence_LowHz[index])
				tempVar = Independence_LowHz[index]
				Independence_LowHz[index] = Independence_HighHz[index]
				Independence_HighHz[index] = tempVar
			endif
				
			SpHrView_IndependenceEditMakePoints(index)
			SetAxis/A/N=0/E=1/W=SpHrView_IndependenceEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
	endswitch	
End

// =================================================================================================
// SpHrView_IndependenceButtonPopUpHandler(), Independence popup button handler
// =================================================================================================
function SpHrView_IndependenceButtonPopUpHandler(string ctrlName, variable index) 
	String DF = SpHrView_df_signalGlobals+"Independence:"
	String checked= "\\M0:!" + num2char(18) + ":" // checkmark coder
	String str, MenuStr
	Variable var
	String InterpTypeMenuStr = "Linear;Quadratic;Spline"
	variable DoPopupContextualMenu = 0
	
	wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	if(!waveexists(Independence_Edit_Xs))
		SpHrView_IndependenceDefaults()
	endif
	
	strswitch(ctrlName)
		case "b_independencePopup_NoiseAmb":			// SpHrView_BinauralSignal_Window
			index = 0
			break
		case "b_independencePopup_Noise1":			// SpHrView_BinauralSignal_Window
			index = 1
			break
		case "b_independencePopup_Noise2":			// SpHrView_BinauralSignal_Window
			index = 2
			break
	endswitch
	
	strswitch(ctrlName)
		//////////////////////////
		// SpHrView_BinauralSignal_Window
		case "b_independencePopup_NoiseAmb":
		case "b_independencePopup_Noise1":
		case "b_independencePopup_Noise2":
			wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
			MenuStr = "Index;-;Independent Octave Band;Dependent Octave Band;Independent Range;Dependent Range;"
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(Independence[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+Independence[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break		
			
		////////////////////////////////////
		// SpHrView_IndependenceEdit_Window
		case "b_Edit":
			Button b_Edit rename=b_QuitEdit, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_IndependenceEdit_Window Independence_Edit_Ys
			wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
			Independence[index] = "Arbitrary"
			Button b_InterpTypePopUp win=SpHrView_IndependenceEdit_Window, disable=0
			SetVariable sv_InterpType win=SpHrView_IndependenceEdit_Window, disable=0
			/// get interpolation if Quadratic
			wave/T InterpType = $DF+"InterpType"
			wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
			wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
			SpHrView_SignalInterpEditPoints(Independence_Edit_Ys, Independence_Edit_Xs, DF+"Independence", InterpType[index])
			break
		case "b_QuitEdit":
			Button b_QuitEdit rename=b_Edit, title="✏️"
			GraphNormal/W=SpHrView_IndependenceEdit_Window
			wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
			wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
			if( waveexists(Independence_Edit_Xs) && waveexists(Independence_Edit_Ys) )
				//Independence_Edit_Xs = round(Independence_Edit_Xs) // round to 1 Hz?
				/// get interpolation
				wave/T InterpType = $DF+"InterpType"
				wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
				wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"	
				SpHrView_SignalInterpEditPoints(Independence_Edit_Ys, Independence_Edit_Xs, DF+"IndependenceInterp", InterpType[index])
				///
				//SpHrView_IndependenceEditMakePoints(index)
				SpHrView_IndependenceEditWindow(index)
			endif
			break
			
		case "b_ValuesPopUp":
			wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
			MenuStr = "Index;-;Independent Octave Band;Dependent Octave Band;Independent Range;Dependent Range;"
			if(stringmatch(Independence[index], "Arbitrary"))
				MenuStr += ";-;Save Arbitrary Wave;"
			endif
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(Independence[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+Independence[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break
			
		case "b_InterpTypePopUp":
			PopupContextualMenu InterpTypeMenuStr
			if(V_flag>=1)
				DF = SpHrView_df_signalGlobals+"Independence:"
				wave/T InterpType = $DF+"InterpType"
				InterpType[index] = S_selection
				wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
				wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
				SpHrView_SignalInterpEditPoints(Independence_Edit_Ys, Independence_Edit_Xs, DF+"IndependenceInterp", InterpType[index])							
			endif
			break
	endswitch
	
	if(DoPopupContextualMenu)
		PopupContextualMenu MenuStr
			if(V_flag>=1)
				strswitch(S_selection)
					case "Save Arbitrary Wave":
						str = "Independence_untitled"
						Prompt str, "Wave Name:"
						DoPrompt "Save Independence Indicies Wave?", str
						if(!V_Flag)
							wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
							wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
							Duplicate/O Independence_Edit_Ys, $SpHrView_df_signals+str
							wave savedWave = $SpHrView_df_signals+str
							Redimension/N=(-1,2) savedWave
							savedWave[][1] = Independence_Edit_Xs[p]
						endif
						break
						
					case "Index":
						if(index==0)
							SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_Independence_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==1)
							SetVariable/Z sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_Independence_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==2)
							SetVariable/Z sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_Independence_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						endif
						Independence[index] = S_selection
						SpHrView_IndependenceEditMakePoints(index)
						break
					
					case "Independent Octave Band":
					case "Independent Range":
					case "Dependent Octave Band":
					case "Dependent Range":
						if(index==0)
							SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						Independence[index] = S_selection
						SpHrView_IndependenceEditMakePoints(index)
						SpHrView_IndependenceEditWindow(index)
						break
						
					case "Independence_Ambient":
						if(index==0)
							SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
						elseif(index==1)
							SetVariable/Z sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_Independence_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
						elseif(index==2)
							SetVariable/Z sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_Independence_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
						endif
						Independence[index] = S_selection
						SpHrView_IndependenceEditMakePoints(index)
						SpHrView_IndependenceEditWindow(index)
						break

					default:
						if(index==0)
							SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_IndependenceVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_IndependenceVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_Independence_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						Independence[index] = S_selection
						SpHrView_IndependenceEditMakePoints(index)
						SpHrView_IndependenceEditWindow(index)
						break
				endswitch
			endif	// if(V_flag>=1)
	endif // if(DoPopupContextualMenu)
End

// =================================================================================================
// SpHrView_IndependenceEditMakePoints(SignalIndex), Get Independence shapes
// =================================================================================================
function SpHrView_IndependenceEditMakePoints(SignalIndex)
	variable SignalIndex
	DFREF dfSav= GetdataFolderDFR()
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	wave/T Independence = $SpHrView_df_signalGlobals+"Independence"
	String DF = SpHrView_df_signalGlobals+"Independence:"
	SetDataFolder $DF
	wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	if(!waveexists(Independence_Edit_Xs))
		SpHrView_IndependenceDefaults()
	endif
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
	wave Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
	wave IndependenceVal = $SpHrView_df_signalGlobals+"IndependenceVal"
	wave/T InterpType = $DF+"InterpType"
	
	strswitch(Independence[SignalIndex])
		case "Index":
			if(!waveexists(Independence_Edit_Xs) || !waveexists(Independence_Edit_Ys))
				Make/O/N=(NumberEditPoints[SignalIndex]) Independence_Edit_Xs, Independence_Edit_Ys
			endif
			if(NumberEditPoints[SignalIndex] > 21 || NumberEditPoints[SignalIndex] < 6)
					NumberEditPoints[SignalIndex] = 6
				endif
			if(dimsize(Independence_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(Independence_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
				Redimension/N=(NumberEditPoints[SignalIndex]) Independence_Edit_Xs, Independence_Edit_Ys
			endif
			SetScale/I x, log(1), log(fs/2), "Hz" Independence_Edit_Xs
			SetScale d 0,0,"Hz", Independence_Edit_Xs
			SetScale d 0,0,"Index", Independence_Edit_Ys 
			Independence_Edit_Xs = 10^x
			Independence_Edit_Ys = IndependenceVal[SignalIndex]
			//InterpType[SignalIndex] = "Linear"
			SpHrView_SignalInterpEditPoints(Independence_Edit_Ys, Independence_Edit_Xs, DF+"IndependenceInterp", InterpType[SignalIndex])
			break
			
		case "Independent Octave Band":
		case "Dependent Octave Band":
			wave Independence_Hz = $DF+"Independence_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) Independence_Edit_Xs, Independence_Edit_Ys//, Independence_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" Independence_Edit_Xs
			SetScale d 0,0,"Hz", Independence_Edit_Xs
			SetScale d 0,0,"index", Independence_Edit_Ys
	
			variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = Independence_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = Independence_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			strswitch(Independence[SignalIndex])
				case "Independent Octave Band":
					Independence_Edit_Ys[0,1] = 0
					Independence_Edit_Ys[2,3] = IndependenceVal[SignalIndex]
					Independence_Edit_Ys[4,5] = 0
					break
				case "Dependent Octave Band":
					Independence_Edit_Ys[0,1] = IndependenceVal[SignalIndex]
					Independence_Edit_Ys[2,3] = 0
					Independence_Edit_Ys[4,5] = IndependenceVal[SignalIndex]
					break
			endswitch
			// Xs
			Independence_Edit_Xs[0] = min(OctaveHzLow,1)
			Independence_Edit_Xs[1] = max(1,OctaveHzLow)
			Independence_Edit_Xs[2] = max(1,OctaveHzLow)
			Independence_Edit_Xs[3] = OctaveHzHigh
			Independence_Edit_Xs[4] = OctaveHzHigh
			Independence_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				Independence_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				Independence_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, Independence_Edit_Xs, Independence_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, Independence_Edit_Xs, Independence_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
		
		case "Independent Range":	
		case "Dependent Range":
			wave Independence_LowHz = $DF+"Independence_LowHz"
			wave Independence_HighHz = $DF+"Independence_HighHz"
	
			Make/O/N=(6) Independence_Edit_Xs, Independence_Edit_Ys//, Independence_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" Independence_Edit_Xs
			SetScale d 0,0,"Hz", Independence_Edit_Xs
			SetScale d 0,0,"index", Independence_Edit_Ys
			// Ys
			strswitch(Independence[SignalIndex])
				case "Independent Range":
					Independence_Edit_Ys[0,1] = 0
					Independence_Edit_Ys[2,3] = IndependenceVal[SignalIndex]
					Independence_Edit_Ys[4,5] = 0
					break
				case "Dependent Range":
					Independence_Edit_Ys[0,1] = IndependenceVal[SignalIndex]
					Independence_Edit_Ys[2,3] = 0
					Independence_Edit_Ys[4,5] = IndependenceVal[SignalIndex]
					break
			endswitch
			// Xs
			Independence_Edit_Xs[0] = min(Independence_LowHz[SignalIndex],1)
			Independence_Edit_Xs[1] = max(1,Independence_LowHz[SignalIndex])
			Independence_Edit_Xs[2] = max(1,Independence_LowHz[SignalIndex])
			Independence_Edit_Xs[3] = Independence_HighHz[SignalIndex]
			Independence_Edit_Xs[4] = Independence_HighHz[SignalIndex]
			Independence_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				Independence_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				Independence_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, Independence_Edit_Xs, Independence_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, Independence_Edit_Xs, Independence_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
			
		default:
			// load saved wave into Spectrum_Edit_Xs amd Spectrum_Edit_Ys
			wave/Z w = $SpHrView_df_signals+Independence[SignalIndex]
			if(waveexists(w))
				//Independence[index] = S_selection
				wave/Z Independence_Edit_Xs = $DF+"Independence_Edit_Xs"
				wave/Z Independence_Edit_Ys = $DF+"Independence_Edit_Ys"
				wave NumberEditPoints = $DF+"NumberEditPoints"
				NumberEditPoints[SignalIndex] = dimsize(w,0)
				redimension/N=(NumberEditPoints[SignalIndex]) Independence_Edit_Ys, Independence_Edit_Xs
				Independence_Edit_Ys = w[p][0]
				Independence_Edit_Xs = w[p][1]
				// add point at fs/2, although unnecessary
				if(Independence_Edit_Xs[dimsize(w,0)-1] < fs/2)
					redimension/N=(dimsize(w,0)+1) Independence_Edit_Xs
					Independence_Edit_Xs[dimsize(w,0)] = fs/2
					redimension/N=(dimsize(w,0)+1) Independence_Edit_Ys
					Independence_Edit_Ys[dimsize(w,0)] = Independence_Edit_Ys[dimsize(w,0)-1]
				endif
				// add point at frequency = 1, although unnecessary
				if(Independence_Edit_Xs[0] > 1)
					insertPoints 0, 1, Independence_Edit_Xs
					Independence_Edit_Xs[0] = 1
					insertPoints 0, 1, Independence_Edit_Ys
					Independence_Edit_Ys[0] = Independence_Edit_Ys[1]
				endif
			
				SetVariable/Z sv_IndependenceVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
				if(!stringmatch(Independence[SignalIndex] , "Independence_Ambient"))
					SetVariable/Z sv_Independence_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
				endif
			endif
			break
			
	endswitch
	
	// Write Independence values wave to be used by SpHr_SignalBinaural
	Make/O/N=(dimsize(Independence_Edit_Ys,0), 2) $SpHrView_df_signals+"Independence_"+num2str(SignalIndex)
	wave IndependenceWave = $SpHrView_df_signals+"Independence_"+num2str(SignalIndex)
	IndependenceWave[][0] = Independence_Edit_Ys[p]
	IndependenceWave[][1] = Independence_Edit_Xs[p]
	IndependenceWave[][0] = (IndependenceWave[p][0] < 0) ? 0 : IndependenceWave[p][0]
	IndependenceWave[][0] = (IndependenceWave[p][0] > 1) ? 1 : IndependenceWave[p][0]

	SetDataFolder dfSav
end

Function SpHrView_EditWindow_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 2: // mouse up
			String SignalStr = GetUserData(ba.win, "", "SignalData" )
			String paramStr = StringByKey("SIGNALPARAM", SignalStr)
			Variable index = NumberByKey("SIGNALNUM", SignalStr)
			Variable ch = 1 // 2 for right channel spectrum
			String checked= "\\M0:!" + num2char(18) + ":" // checkmark code
			String InterpTypeMenuStr = "Linear;Quadratic;Spline"
			String DF, str

			strswitch( paramStr )
				// ILD
				case "ILD":
					SpHrView_ILDButtonPopUpHandler(ba.ctrlName, index) 
					break
					
				// IPD
				case "IPD":
					SpHrView_IPDButtonPopUpHandler(ba.ctrlName, index) 
					break
					
				// ITD
				case "ITD":
					SpHrView_ITDButtonPopUpHandler(ba.ctrlName, index) 
					break
			
				// PhaseInversion
				case "PhaseInversion":
					SpHrView_PhaseInversionButtonPopUpHandler(ba.ctrlName, index) 
					break
					
				// Independence
				case "Independence":
					SpHrView_IndependenceButtonPopUpHandler(ba.ctrlName, index) 
					break
		
				// Spectrum
				case "Spectrum":
					SpHrView_SpectrumButtonPopUpHandler(ba.ctrlName, index) 
					break
					
				// Tones
				case "Tones":
					SpHrView_ToneButtonPopUpHandler(ba.ctrlName, index) 
					break
					
				endswitch // strswitch( paramStr )  paramStr = "Spectrum"
			break
		case -1: 		// control being killed
			break
	endswitch		// switch( ba.eventCode )

	return 0
End

Function SpHrView_EditWindow_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
		case 9: // mouse down
			Variable forceUpdate = 0
			Variable dval = sva.dval
			//Printf "sva.eventMod= %g\r", sva.eventMod
			if(sva.eventMod == 5)  // option key: Set to zero
				dval = 0
				forceUpdate = 1
			endif
			if(sva.eventMod == 9)  // command key: Reverse sign
				dval = -dval
				forceUpdate = 1
			endif
			String SignalStr = GetUserData(sva.win, "", "SignalData" )
			String paramStr = StringByKey("SIGNALPARAM", SignalStr)
			Variable index = sva.rowIndex
			strswitch( paramStr )
				case "ILD":
					SpHrView_ILDSetVarHandler(sva.ctrlName, index, dval) 
					break
				case "IPD":
					SpHrView_IPDSetVarHandler(sva.ctrlName, index, dval) 
					break
				case "ITD":
					SpHrView_ITDSetVarHandler(sva.ctrlName, index, dval) 
					break
				case "PhaseInversion":
					SpHrView_PhaseInversionSetVarHandler(sva.ctrlName, index, dval, NaN) 
					break
				case "Independence":
					SpHrView_IndependenceSetVarHandler(sva.ctrlName, index, dval) 
					break
				case "Spectrum":
					SpHrView_SpectrumSetVarHandler(sva.ctrlName, index, dval) 
					break
				case "Tones":
					SpHrView_TonesSetVarHandler(sva.ctrlName, index, dval) 
				break
			endswitch		// strswitch( paramStr )
			SpHrView_BinauralSignalWindowColors(index)
			break
		case -1: // control being killed
			break
	endswitch
	if(forceUpdate)	
		ControlUpdate/W=$sva.win $sva.ctrlName
	endif
	return 0
End

Function SpHrView_EditWindow_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			Variable checked = cba.checked
			strswitch( cba.ctrlName )
				case "ck_logFreqAxis":
					if(checked)
						ModifyGraph log(bottom)=1,loglinear(bottom)=1
						NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
					else
						ModifyGraph log=0
					endif
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

// =================================================================================================
// SpHrView_SpectrumEditMakePoints(SignalIndex), Get Spectrum shapes
// =================================================================================================
function SpHrView_SpectrumEditMakePoints(SignalIndex)
	variable SignalIndex
	DFREF dfSav= GetdataFolderDFR()
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	wave/T SpectrumShape = $SpHrView_df_signalGlobals+"SpectrumShape"
	String DF = SpHrView_df_signalGlobals+"Spectrum:"
	SetDataFolder $DF
	wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
	if(!waveexists(Spectrum_Edit_Xs))
		SpHrView_SpectrumDefaults()
	endif
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave Spectrum_LowHz = $DF+"Spectrum_LowHz"
	wave Spectrum_HighHz = $DF+"Spectrum_HighHz"
	wave Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
	wave Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
	wave/T InterpType = $DF+"InterpType"
	
	variable width, OctaveHzLow, OctaveHzHigh
	
	strswitch(SpectrumShape[SignalIndex])
		case "Flat":
				wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
				wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
				if(!waveexists(Spectrum_Edit_Xs) || !waveexists(Spectrum_Edit_Ys))
					Make/O/N=(NumberEditPoints[SignalIndex]) Spectrum_Edit_Xs, Spectrum_Edit_Ys
				endif
				if(dimsize(Spectrum_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(Spectrum_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
					Redimension/N=(NumberEditPoints[SignalIndex]) Spectrum_Edit_Xs, Spectrum_Edit_Ys
				endif
				SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs
				//SetScale/I x, 1.30103, 4.30103, "Hz" Spectrum_Edit_Xs
				//SetScale/I x, log(Spectrum_LowHz[SignalIndex]), log(Spectrum_HighHz[SignalIndex]), "Hz" Spectrum_Edit_Xs
				SetScale d 0,0,"Hz", Spectrum_Edit_Xs
				SetScale d 0,0,"dB", Spectrum_Edit_Ys 
				Spectrum_Edit_Xs = 10^x
				Spectrum_Edit_Ys = 0
				InterpType[SignalIndex] = "Linear"
				break
				
		case "1 / Frequency":
				wave/Z Spectrum_Edit_Xs = $DF+"Spectrum_Edit_Xs"
				wave/Z Spectrum_Edit_Ys = $DF+"Spectrum_Edit_Ys"
				if(!waveexists(Spectrum_Edit_Xs) || !waveexists(Spectrum_Edit_Ys))
					Make/O/N=(NumberEditPoints[SignalIndex]) Spectrum_Edit_Xs, Spectrum_Edit_Ys
				endif
				if(dimsize(Spectrum_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(Spectrum_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
					Redimension/N=(NumberEditPoints[SignalIndex]) Spectrum_Edit_Xs, Spectrum_Edit_Ys
				endif
				SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs
				//SetScale/I x, 1.30103, 4.30103, "Hz" Spectrum_Edit_Xs
				//SetScale/I x, log(Spectrum_LowHz[SignalIndex]), log(Spectrum_HighHz[SignalIndex]), "Hz" Spectrum_Edit_Xs
				SetScale d 0,0,"Hz", Spectrum_Edit_Xs
				SetScale d 0,0,"dB", Spectrum_Edit_Ys 
				Spectrum_Edit_Xs = 10^x
				wave Spectrum_RefHz = $DF+"Spectrum_RefHz"
				if(0)// linear scale
					Spectrum_Edit_Ys = Spectrum_RefHz[SignalIndex]/Spectrum_Edit_Xs[p]  // re: reference Hz
				else // log scale
					Spectrum_Edit_Ys = 20*log(Spectrum_RefHz[SignalIndex]/Spectrum_Edit_Xs[p])  // re: reference Hz
				endif
				InterpType[SignalIndex] = "Linear"
				break

		case "Octave Band":
		
			wave Spectrum_Hz = $DF+"Spectrum_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) Spectrum_Edit_Xs, Spectrum_Edit_Ys//, Spectrum_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs
			SetScale d 0,0,"Hz", Spectrum_Edit_Xs
			SetScale d 0,0,"dB", Spectrum_Edit_Ys // !! units are dB or Pa or V, defaults to dB
			//SetScale d 0,0,"dB", Spectrum_Edit_Xs // !! units are dB or Pa or V, defaults to dB
			//SetScale/P x, 0, 1, "dB" Spectrum_Edit_Ys
	
			//variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = Spectrum_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = Spectrum_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			Spectrum_Edit_Ys[0,1] = -61
			Spectrum_Edit_Ys[2,3] = 0 // 1 for Signal_dB_re20uPa
			Spectrum_Edit_Ys[4,5] = -61
			// Xs
			Spectrum_Edit_Xs[0] = min(OctaveHzLow,1)
			Spectrum_Edit_Xs[1] = max(1,OctaveHzLow)
			Spectrum_Edit_Xs[2] = max(1,OctaveHzLow)
			Spectrum_Edit_Xs[3] = OctaveHzHigh
			Spectrum_Edit_Xs[4] = OctaveHzHigh
			Spectrum_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				Spectrum_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				Spectrum_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, Spectrum_Edit_Xs, Spectrum_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, Spectrum_Edit_Xs, Spectrum_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
			
		case "Octave Notch":
		
			wave Spectrum_Hz = $DF+"Spectrum_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) Spectrum_Edit_Xs, Spectrum_Edit_Ys//, Spectrum_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs
			SetScale d 0,0,"Hz", Spectrum_Edit_Xs
			SetScale d 0,0,"dB", Spectrum_Edit_Ys // !! units are dB or Pa or V, defaults to dB
			//SetScale d 0,0,"dB", Spectrum_Edit_Xs // !! units are dB or Pa or V, defaults to dB
			//SetScale/P x, 0, 1, "dB" Spectrum_Edit_Ys
	
			//variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = Spectrum_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = Spectrum_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			Spectrum_Edit_Ys[0,1] = 0 // -61
			Spectrum_Edit_Ys[2,3] = -61 // 0 // 1 for Signal_dB_re20uPa
			Spectrum_Edit_Ys[4,5] = 0 // -61
			// Xs
			Spectrum_Edit_Xs[0] = min(OctaveHzLow,1)
			Spectrum_Edit_Xs[1] = max(1,OctaveHzLow)
			Spectrum_Edit_Xs[2] = max(1,OctaveHzLow)
			Spectrum_Edit_Xs[3] = OctaveHzHigh
			Spectrum_Edit_Xs[4] = OctaveHzHigh
			Spectrum_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				Spectrum_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				Spectrum_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, Spectrum_Edit_Xs, Spectrum_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, Spectrum_Edit_Xs, Spectrum_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
			
		case "Range":
			wave Spectrum_LowHz = $DF+"Spectrum_LowHz"
			wave Spectrum_HighHz = $DF+"Spectrum_HighHz"
	
			Make/O/N=(6) Spectrum_Edit_Xs, Spectrum_Edit_Ys//, Spectrum_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs
			SetScale d 0,0,"Hz", Spectrum_Edit_Xs
			SetScale d 0,0,"dB", Spectrum_Edit_Ys // !! units are dB or Pa or V, defaults to dB
			// Ys
			Spectrum_Edit_Ys[0,1] = -61
			Spectrum_Edit_Ys[2,3] = 0 // 1 for Signal_dB_re20uPa
			Spectrum_Edit_Ys[4,5] = -61
			// Xs
			Spectrum_Edit_Xs[0] = min(Spectrum_LowHz[SignalIndex],1)
			Spectrum_Edit_Xs[1] = max(1,Spectrum_LowHz[SignalIndex])
			Spectrum_Edit_Xs[2] = max(1,Spectrum_LowHz[SignalIndex])
			Spectrum_Edit_Xs[3] = Spectrum_HighHz[SignalIndex]
			Spectrum_Edit_Xs[4] = Spectrum_HighHz[SignalIndex]
			Spectrum_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				Spectrum_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				Spectrum_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, Spectrum_Edit_Xs, Spectrum_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, Spectrum_Edit_Xs, Spectrum_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
	endswitch
	
	SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys, Spectrum_Edit_Xs, DF+"SpectrumInterp", InterpType[SignalIndex])
	
	
	wave LinkSpectrumChannels = $SpHrView_df_signalGlobals+"LinkSpectrumChannels"
	if(!LinkSpectrumChannels[SignalIndex])
		wave/T SpectrumShape2 = $SpHrView_df_signalGlobals+"SpectrumShape2"
		wave NumberEditPoints2 = $DF+"NumberEditPoints2"
		wave Spectrum_LowHz2 = $DF+"Spectrum_LowHz2"
		wave Spectrum_HighHz2 = $DF+"Spectrum_HighHz2"
		wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
		wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
		wave/T InterpType2 = $DF+"InterpType2"
		
		if(NumberEditPoints2[SignalIndex] < 6)
			NumberEditPoints2[SignalIndex] = 6
		endif
		if(NumberEditPoints2[SignalIndex] > 6 && NumberEditPoints2[SignalIndex]<11)
			NumberEditPoints2[SignalIndex] = 11
		endif
		if(NumberEditPoints2[SignalIndex] > 11)
			NumberEditPoints2[SignalIndex] = 21
		endif
		
		strswitch(SpectrumShape2[SignalIndex])
			case "Flat":
					wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
					wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
					if(!waveexists(Spectrum_Edit_Xs2) || !waveexists(Spectrum_Edit_Ys2))
						Make/O/N=(NumberEditPoints2[SignalIndex]) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2
					endif
					if(dimsize(Spectrum_Edit_Xs2,0)!=NumberEditPoints2[SignalIndex] || dimsize(Spectrum_Edit_Ys2,0)!=NumberEditPoints2[SignalIndex])
						Redimension/N=(NumberEditPoints2[SignalIndex]) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2
					endif
					SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs2
					//SetScale/I x, 1.30103, 4.30103, "Hz" Spectrum_Edit_Xs
					//SetScale/I x, log(Spectrum_LowHz[SignalIndex]), log(Spectrum_HighHz[SignalIndex]), "Hz" Spectrum_Edit_Xs
					SetScale d 0,0,"Hz", Spectrum_Edit_Xs2
					SetScale d 0,0,"dB", Spectrum_Edit_Ys2
					Spectrum_Edit_Xs2 = 10^x
					Spectrum_Edit_Ys2 = 0
					InterpType2[SignalIndex] = "Linear"
					break
					
			case "1 / Frequency":
					wave/Z Spectrum_Edit_Xs2 = $DF+"Spectrum_Edit_Xs2"
					wave/Z Spectrum_Edit_Ys2 = $DF+"Spectrum_Edit_Ys2"
					if(!waveexists(Spectrum_Edit_Xs2) || !waveexists(Spectrum_Edit_Ys2))
						Make/O/N=(NumberEditPoints2[SignalIndex]) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2
					endif
					if(dimsize(Spectrum_Edit_Xs2,0)!=NumberEditPoints2[SignalIndex] || dimsize(Spectrum_Edit_Ys2,0)!=NumberEditPoints2[SignalIndex])
						Redimension/N=(NumberEditPoints2[SignalIndex]) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2
					endif
					SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs2
					//SetScale/I x, 1.30103, 4.30103, "Hz" Spectrum_Edit_Xs2
					//SetScale/I x, log(Spectrum_LowHz2[SignalIndex]), log(Spectrum_HighHz2[SignalIndex]), "Hz" Spectrum_Edit_Xs2
					SetScale d 0,0,"Hz", Spectrum_Edit_Xs2
					SetScale d 0,0,"dB", Spectrum_Edit_Ys2
					Spectrum_Edit_Xs2 = 10^x
					wave Spectrum_RefHz2 = $DF+"Spectrum_RefHz2"
					if(0)// linear scale
						Spectrum_Edit_Ys2 = Spectrum_RefHz2[SignalIndex]/Spectrum_Edit_Xs2[p]  // re: reference Hz
					else // log scale
						Spectrum_Edit_Ys2 = 20*log(Spectrum_RefHz2[SignalIndex]/Spectrum_Edit_Xs2[p])  // re: reference Hz
					endif
					InterpType2[SignalIndex] = "Linear"
					break

			case "Octave Band":
			
				wave Spectrum_Hz2 = $DF+"Spectrum_Hz2"
				wave OctaveDenominator2 = $DF+"OctaveDenominator2"
		
				Make/O/N=(6) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2//, Spectrum_Edit_Labels
				SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs2
				SetScale d 0,0,"Hz", Spectrum_Edit_Xs2
				SetScale d 0,0,"dB", Spectrum_Edit_Ys2 // !! units are dB or Pa or V, defaults to dB
				//SetScale d 0,0,"dB", Spectrum_Edit_Xs2 // !! units are dB or Pa or V, defaults to dB
				//SetScale/P x, 0, 1, "dB" Spectrum_Edit_Ys2
		
				//variable width, OctaveHzLow, OctaveHzHigh
				width = 10^(3.0103/(10*OctaveDenominator2[SignalIndex]))
				OctaveHzLow = Spectrum_Hz2[SignalIndex] / width
				OctaveHzLow = max(0,OctaveHzLow)
			   OctaveHzHigh = Spectrum_Hz2[SignalIndex] * width
			   OctaveHzHigh = min(fs/2, OctaveHzHigh)
				// Ys
				Spectrum_Edit_Ys2[0,1] = -61
				Spectrum_Edit_Ys2[2,3] = 0 // 1 for Signal_dB_re20uPa
				Spectrum_Edit_Ys2[4,5] = -61
				// Xs
				Spectrum_Edit_Xs2[0] = min(OctaveHzLow,1)
				Spectrum_Edit_Xs2[1] = max(1,OctaveHzLow)
				Spectrum_Edit_Xs2[2] = max(1,OctaveHzLow)
				Spectrum_Edit_Xs2[3] = OctaveHzHigh
				Spectrum_Edit_Xs2[4] = OctaveHzHigh
				Spectrum_Edit_Xs2[5] = fs/2
				if(NumberEditPoints2[SignalIndex] >= 10)
					Spectrum_Edit_Xs2[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
					Spectrum_Edit_Xs2[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
				endif

				if(NumberEditPoints2[SignalIndex] == 11)
					SpHrView_SignalRedimensionEditPoints(11, Spectrum_Edit_Xs2, Spectrum_Edit_Ys2)
				elseif(NumberEditPoints2[SignalIndex] == 21)
					SpHrView_SignalRedimensionEditPoints(21, Spectrum_Edit_Xs2, Spectrum_Edit_Ys2)
				endif
				InterpType2[SignalIndex] = "Linear"
				break
				
			case "Octave Notch":
			
				wave Spectrum_Hz2 = $DF+"Spectrum_Hz2"
				wave OctaveDenominator2 = $DF+"OctaveDenominator2"
		
				Make/O/N=(6) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2//, Spectrum_Edit_Labels
				SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs2
				SetScale d 0,0,"Hz", Spectrum_Edit_Xs2
				SetScale d 0,0,"dB", Spectrum_Edit_Ys2 // !! units are dB or Pa or V, defaults to dB
				//SetScale d 0,0,"dB", Spectrum_Edit_Xs2 // !! units are dB or Pa or V, defaults to dB
				//SetScale/P x, 0, 1, "dB" Spectrum_Edit_Ys2
		
				//variable width, OctaveHzLow, OctaveHzHigh
				width = 10^(3.0103/(10*OctaveDenominator2[SignalIndex]))
				OctaveHzLow = Spectrum_Hz2[SignalIndex] / width
				OctaveHzLow = max(0,OctaveHzLow)
			   OctaveHzHigh = Spectrum_Hz2[SignalIndex] * width
			   OctaveHzHigh = min(fs/2, OctaveHzHigh)
				// Ys
				Spectrum_Edit_Ys2[0,1] = 0 // -61
				Spectrum_Edit_Ys2[2,3] = -61 // 0 // 1 for Signal_dB_re20uPa
				Spectrum_Edit_Ys2[4,5] = 0 // -61
				// Xs
				Spectrum_Edit_Xs2[0] = min(OctaveHzLow,1)
				Spectrum_Edit_Xs2[1] = max(1,OctaveHzLow)
				Spectrum_Edit_Xs2[2] = max(1,OctaveHzLow)
				Spectrum_Edit_Xs2[3] = OctaveHzHigh
				Spectrum_Edit_Xs2[4] = OctaveHzHigh
				Spectrum_Edit_Xs2[5] = fs/2
				if(NumberEditPoints2[SignalIndex] >= 10)
					Spectrum_Edit_Xs2[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
					Spectrum_Edit_Xs2[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
				endif

				if(NumberEditPoints2[SignalIndex] == 11)
					SpHrView_SignalRedimensionEditPoints(11, Spectrum_Edit_Xs2, Spectrum_Edit_Ys2)
				elseif(NumberEditPoints2[SignalIndex] == 21)
					SpHrView_SignalRedimensionEditPoints(21, Spectrum_Edit_Xs2, Spectrum_Edit_Ys2)
				endif
				InterpType2[SignalIndex] = "Linear"
				break
				
			case "Range":
				wave Spectrum_LowHz2 = $DF+"Spectrum_LowHz2"
				wave Spectrum_HighHz2 = $DF+"Spectrum_HighHz2"
		
				Make/O/N=(6) Spectrum_Edit_Xs2, Spectrum_Edit_Ys2
				SetScale/I x, log(1), log(fs/2), "Hz" Spectrum_Edit_Xs2
				SetScale d 0,0,"Hz", Spectrum_Edit_Xs2
				SetScale d 0,0,"dB", Spectrum_Edit_Ys2 // !! units are dB or Pa or V, defaults to dB
				// Ys
				Spectrum_Edit_Ys2[0,1] = -61
				Spectrum_Edit_Ys2[2,3] = 0 // 1 for Signal_dB_re20uPa
				Spectrum_Edit_Ys2[4,5] = -61
				// Xs
				Spectrum_Edit_Xs2[0] = min(Spectrum_LowHz2[SignalIndex],1)
				Spectrum_Edit_Xs2[1] = max(1,Spectrum_LowHz2[SignalIndex])
				Spectrum_Edit_Xs2[2] = max(1,Spectrum_LowHz2[SignalIndex])
				Spectrum_Edit_Xs2[3] = Spectrum_HighHz2[SignalIndex]
				Spectrum_Edit_Xs2[4] = Spectrum_HighHz2[SignalIndex]
				Spectrum_Edit_Xs2[5] = fs/2
				if(NumberEditPoints2[SignalIndex] >= 10)
					Spectrum_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
					Spectrum_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
				endif
				
				if(NumberEditPoints2[SignalIndex] == 11)
					SpHrView_SignalRedimensionEditPoints(11, Spectrum_Edit_Xs2, Spectrum_Edit_Ys2)
				elseif(NumberEditPoints2[SignalIndex] == 21)
					SpHrView_SignalRedimensionEditPoints(21, Spectrum_Edit_Xs2, Spectrum_Edit_Ys2)
				endif
				InterpType2[SignalIndex] = "Linear"
				break
		endswitch
		
		SpHrView_SignalInterpEditPoints(Spectrum_Edit_Ys2, Spectrum_Edit_Xs2, DF+"SpectrumInterp2", InterpType2[SignalIndex])

	endif
	
	DoWindow SpHrView_SpectrumEdit_Window
	if(v_flag)	
	
		if(stringmatch(SpectrumShape[SignalIndex],"Flat") && (LinkSpectrumChannels[SignalIndex] || stringmatch(SpectrumShape2[SignalIndex],"Flat")) )
			SetAxis/W=SpHrView_SpectrumEdit_Window left -6,6
		else
			SetAxis/A/E=2 left
		endif
	endif
	
	// Write SpectrumShape to be used by SpHr_SignalBinaural
	Make/O/N=(dimsize(Spectrum_Edit_Ys,0), 2) $SpHrView_df_signals+"Spectrum_"+num2str(SignalIndex)
	wave SpectrumWave = $SpHrView_df_signals+"Spectrum_"+num2str(SignalIndex)
	SpectrumWave[][0] = Spectrum_Edit_Ys[p]
	SpectrumWave[][1] = Spectrum_Edit_Xs[p]
	SpectrumWave[][0] = (SpectrumWave[p][0] < -60) ? -120 : SpectrumWave[p][0]
	
	if(!LinkSpectrumChannels[SignalIndex])
		Make/O/N=(dimsize(Spectrum_Edit_Ys2,0), 2) $SpHrView_df_signals+"Spectrum2_"+num2str(SignalIndex)
		wave SpectrumWave2 = $SpHrView_df_signals+"Spectrum2_"+num2str(SignalIndex)
		SpectrumWave2[][0] = Spectrum_Edit_Ys2[p]
		SpectrumWave2[][1] = Spectrum_Edit_Xs2[p]
		SpectrumWave2[][0] = (SpectrumWave2[p][0] < -60) ? -120 : SpectrumWave2[p][0]
	endif
	
	SetDataFolder dfSav
end


// =================================================================================================
// SpHrView_PhaseInversionButtonPopUpHandler(), Phase inversion popup button handler
// =================================================================================================
function SpHrView_PhaseInversionButtonPopUpHandler(string ctrlName, variable index) 
	String DF = SpHrView_df_signalGlobals+"PhaseInversion:"
	String checked= "\\M0:!" + num2char(18) + ":" // checkmark coder
	String str, MenuStr
	Variable var
	variable DoPopupContextualMenu = 0

	strswitch(ctrlName)
		case "b_PhaseInversionPopup_NoiseAmb":		// SpHrView_BinauralSignal_Window
			index = 0
			break
		case "b_PhaseInversionPopup_Noise1":			// SpHrView_BinauralSignal_Window
			index = 1
			break
		case "b_PhaseInversionPopup_Noise2":			// SpHrView_BinauralSignal_Window
			index = 2
			break
		case "b_PhaseInversionPopup_Tones":			// SpHrView_BinauralSignal_Window
			index = 3
			break
	endswitch
	
	wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
	if(!waveexists(PhaseInversion_Edit_Xs))
		SpHrView_PhaseInversionDefaults()
	endif
	
	strswitch(ctrlName)
		//////////////////////////
		// SpHrView_BinauralSignal_Window
		case "b_PhaseInversionPopup_NoiseAmb":
		case "b_PhaseInversionPopup_Noise1":
		case "b_PhaseInversionPopup_Noise2":
		case "b_PhaseInversionPopup_Tones":
			wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
			MenuStr = "Normal Phase;Inverted Phase;-;Inverted Octave Band;Uninverted Octave Band;Inverted Range;Uninverted Range;"
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(PhaseInversion[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+PhaseInversion[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break		
			
		////////////////////////////////////
		// SpHrView_PhaseInversionEdit_Window
		case "b_Edit":
			Button b_Edit rename=b_QuitEdit, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_PhaseInversionEdit_Window PhaseInversion_Edit_Ys
			wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
			PhaseInversion[index] = "Arbitrary"
			/// get interpolation if Quadratic
			wave/T InterpType = $DF+"InterpType"
			wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
			wave/Z PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
			//SpHrView_SignalInterpEditPoints(PhaseInversion_Edit_Ys, PhaseInversion_Edit_Xs, DF+"PhaseInversion", InterpType[index])
			break
		case "b_QuitEdit":
			Button b_QuitEdit rename=b_Edit, title="✏️"
			GraphNormal/W=SpHrView_PhaseInversionEdit_Window
			wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
			wave/Z PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
			if( waveexists(PhaseInversion_Edit_Xs) && waveexists(PhaseInversion_Edit_Ys) )
				//PhaseInversion_Edit_Xs = round(PhaseInversion_Edit_Xs) // round to 1 Hz?
				/// get interpolation
				wave/T InterpType = $DF+"InterpType"
				wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
				wave/Z PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
				
				PhaseInversion_Edit_Ys = (PhaseInversion_Edit_Ys > 1) ? 1 : PhaseInversion_Edit_Ys
				PhaseInversion_Edit_Ys = (PhaseInversion_Edit_Ys < 0) ? 0 : PhaseInversion_Edit_Ys
				
				SpHrView_PhaseInversionEditMakePoints(index)
				SpHrView_PhaseInversionEditWindow(index)
			endif
			break
			
		case "b_ValuesPopUp":
			wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
			MenuStr = "Normal Phase;Inverted Phase;-;Inverted Octave Band;Uninverted Octave Band;Inverted Range;Uninverted Range;"
			if(stringmatch(PhaseInversion[index], "Arbitrary"))
				MenuStr += ";-;Save Arbitrary Wave;"
			endif
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(PhaseInversion[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+PhaseInversion[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break
		
	endswitch
	
	if(DoPopupContextualMenu)
		PopupContextualMenu MenuStr
			if(V_flag>=1)
				strswitch(S_selection)
					case "Save Arbitrary Wave":
						str = "PhaseInversion_untitled"
						Prompt str, "Wave Name:"
						DoPrompt "Save Phase Inversion Wave?", str
						if(!V_Flag)
							wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
							wave/Z PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
							Duplicate/O PhaseInversion_Edit_Ys, $SpHrView_df_signals+str
							wave savedWave = $SpHrView_df_signals+str
							Redimension/N=(-1,2) savedWave
							savedWave[][1] = PhaseInversion_Edit_Xs[p]
						endif
						break
						
					case "Normal Phase":
						if(index==0)
							SetVariable/Z sv_PhaseInversion_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==1)
							SetVariable/Z sv_PhaseInversion_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==2)
							SetVariable/Z sv_PhaseInversion_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						endif
						PhaseInversion[index] = S_selection
						SpHrView_PhaseInversionEditMakePoints(index)
						break
						
					case "Inverted Phase":
						if(index==0)
							SetVariable sv_PhaseInversion_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_PhaseInversion_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_PhaseInversion_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						PhaseInversion[index] = S_selection
						SpHrView_PhaseInversionEditMakePoints(index)
						break
						
					case "PhaseInverts_Ambient":
						if(index==0)
							SetVariable/Z sv_PhaseInversion_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
						elseif(index==1)
							SetVariable/Z sv_PhaseInversion_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
						elseif(index==2)
							SetVariable/Z sv_PhaseInversion_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
						endif
						PhaseInversion[index] = S_selection
						SpHrView_PhaseInversionEditMakePoints(index)
						SpHrView_PhaseInversionEditWindow(index)
						break
					
					case "Inverted Octave Band":
					case "Inverted Range":
					case "Uninverted Octave Band":
					case "Uninverted Range":
						if(index==0)
							SetVariable/Z sv_PhaseInversion_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_PhaseInversion_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_PhaseInversion_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						PhaseInversion[index] = S_selection
						SpHrView_PhaseInversionEditMakePoints(index)
						SpHrView_PhaseInversionEditWindow(index)
						break
						
					default:
						if(index==0)
							SetVariable/Z sv_PhaseInversion_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_PhaseInversion_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_PhaseInversion_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						PhaseInversion[index] = S_selection
						SpHrView_PhaseInversionEditMakePoints(index)
						SpHrView_PhaseInversionEditWindow(index)
						break
				endswitch
			endif	// if(V_flag>=1)
	endif // if(DoPopupContextualMenu)

End


// =================================================================================================
// SpHrView_PhaseInversionDefaults(SignalIndex), Phase Defaults
// =================================================================================================
function SpHrView_PhaseInversionDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	// Make "SpHrView:Signals:Globals:PhaseInversion" folder, SpHrView_df_signalGlobals+"PhaseInversion:"
	String DF = SpHrView_df_signalGlobals+"PhaseInversion:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	//wave PhaseInversionVal = $SpHrView_df_signalGlobals+"PhaseInversionVal"
	
	wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
	wave/Z PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
	if(!waveexists(PhaseInversion_Edit_Xs) || !waveexists(PhaseInversion_Edit_Ys))
		Make/O/N=(6) PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys
		SetScale/I x, 0, fs/2, "Hz" PhaseInversion_Edit_Xs
		SetScale d 0,0,"Hz", PhaseInversion_Edit_Xs
		SetScale d 0,0,"", PhaseInversion_Edit_Ys
	endif
	
	wave/Z PhaseInversion_Hz = $DF+"PhaseInversion_Hz"
	if(!waveexists(PhaseInversion_Hz))
		Make/O/N=(4) $DF+"PhaseInversion_Hz"; wave PhaseInversion_Hz = $DF+"PhaseInversion_Hz"
		PhaseInversion_Hz = 1000;
	endif
	wave/Z OctaveDenominator = $DF+"OctaveDenominator"
	if(!waveexists(OctaveDenominator))
		Make/O/N=(4) $DF+"OctaveDenominator"; wave OctaveDenominator = $DF+"OctaveDenominator"
		OctaveDenominator = 3;
	endif
	wave/Z PhaseInversion_LowHz = $DF+"PhaseInversion_LowHz"
	if(!waveexists(PhaseInversion_LowHz))
		Make/O/N=(4) $DF+"PhaseInversion_LowHz"; wave PhaseInversion_LowHz = $DF+"PhaseInversion_LowHz"
		PhaseInversion_LowHz = 50;
	endif
	wave/Z PhaseInversion_HighHz = $DF+"PhaseInversion_HighHz"
	if(!waveexists(PhaseInversion_HighHz))
		Make/O/N=(4) $DF+"PhaseInversion_HighHz"; wave PhaseInversion_HighHz = $DF+"PhaseInversion_HighHz"
		PhaseInversion_HighHz = 1500;
	endif
	wave/Z NumberEditPoints = $DF+"NumberEditPoints"
	if(!waveexists(NumberEditPoints))
		Make/O/N=(4) $DF+"NumberEditPoints"; wave NumberEditPoints = $DF+"NumberEditPoints"
		NumberEditPoints = 6;
	endif
	
	// ??
	wave/T/Z InterpType = $DF+"InterpType"
	if(!waveexists(InterpType))
		Make/T/O/N=(4) $DF+"InterpType"; wave/T InterpType = $DF+"InterpType"
		InterpType = "Linear";
	endif
	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_PhaseInversionEditWindow(SignalIndex), Phase inversion window
// =================================================================================================
function SpHrView_PhaseInversionEditWindow(SignalIndex)
	variable SignalIndex // 0: bkgrd, 1: Noise_Noise1, 2:Noise_Noise2, 3: Tones
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	String DF = SpHrView_df_signalGlobals+"PhaseInversion:"
	SetDataFolder $DF
	wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
	if(!waveexists(PhaseInversion_Edit_Xs))
		SpHrView_PhaseInversionDefaults()
	endif
	wave PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
	wave PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
	wave PhaseInversion_Hz = $DF+"PhaseInversion_Hz"
	wave OctaveDenominator = $DF+"OctaveDenominator"
	wave PhaseInversion_LowHz = $DF+"PhaseInversion_LowHz"
	wave PhaseInversion_HighHz = $DF+"PhaseInversion_HighHz"
	
	SpHrView_PhaseInversionEditMakePoints(SignalIndex)
	
	variable winHght = 200
	variable winWdth = 770
	KillWindow/Z SpHrView_PhaseInversionEdit_Window
	Display/K=1/W=(310,520,310+winWdth,520+winHght) as "PhaseInversion"
	DoWindow/C SpHrView_PhaseInversionEdit_Window
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=18,margin(top)=7
	ControlBar 27
	
	variable Top = 3, leftPos = 20
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={130,20},pos={leftPos,Top+3},bodyWidth=130,fSize=9,value=PhaseInversion[SignalIndex]
	Button b_ValuesPopUp title="",pos={leftPos+130,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 160
	Button b_Edit title="✏️",pos={leftPos,Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 10
	SetVariable sv_Pnts title="Points",focusRing=0,size={83,20},pos={leftPos,Top+3},bodyWidth=30,value=NumberEditPoints[SignalIndex]
	SetVariable sv_Pnts limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 70
	GroupBox g_PhaseInversionVariables pos={leftPos+29,Top-1},size={370,22}
	SetVariable sv_PhaseInversion_Hz title="Center",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=PhaseInversion_Hz[SignalIndex],limits={0,20000,100}
	SetVariable sv_OctaveDenominator title="Octave 1/",focusRing=0,size={120,20},pos={leftPos+87,Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator[SignalIndex],limits={0.1,48,1}
	
	leftPos += 190
	SetVariable sv_PhaseInversion_LowHz title="Range:",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=PhaseInversion_LowHz[SignalIndex],limits={0,10000,100}
	SetVariable sv_PhaseInversion_HighHz title="to",focusRing=0,size={125,20},pos={leftPos+64, Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=PhaseInversion_HighHz[SignalIndex],limits={0,96000,100}

	AppendToGraph PhaseInversion_Edit_Ys vs PhaseInversion_Edit_Xs
	ModifyGraph mode(PhaseInversion_Edit_Ys)=4,marker(PhaseInversion_Edit_Ys)=19,msize(PhaseInversion_Edit_Ys)=2.5,lsize(PhaseInversion_Edit_Ys)=1,rgb(PhaseInversion_Edit_Ys)=(0,0,0)

	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid(bottom)=1, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	ModifyGraph lblMargin(left)=2
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5
	Label left "Phase inversion (\\U)\\sb+08\rUninverted ←    → Inverted    "
	
	SetDrawEnv ycoord= left,dash= 2;DelayUpdate
	DrawLine 0,0.5,1,0.5
		
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
		Label bottom "Frequency\\u#2"
	else
		ModifyGraph log=0
		Label/W=SpHrView_PhaseInversionEdit_Window bottom "Frequency (\\U)"
	endif
	SetAxis/A/N=0/E=1/W=SpHrView_PhaseInversionEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
	
	SetAxis/W=SpHrView_PhaseInversionEdit_Window left -0.05, 1.05
	ModifyGraph/W=SpHrView_PhaseInversionEdit_Window manTick(left)={0,1,0,1},manMinor(left)={0,50}
		
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=fs/2,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=20,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc

	strswitch(PhaseInversion[SignalIndex])
		case "Normal Phase":
		case "Inverted Phase":
			SetVariable/Z sv_Pnts win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_Hz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_LowHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_HighHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			break
		case "Inverted Octave Band":
		case "Uninverted Octave Band":
			SetVariable/Z sv_Pnts win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_Hz win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_OctaveDenominator win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_LowHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_HighHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			break
		case "Inverted Range":
		case "Uninverted Range":
			SetVariable/Z sv_Pnts win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_Hz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_LowHz win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_HighHz win=SpHrView_PhaseInversionEdit_Window, disable=0
			break
						
		case "Arbitrary":
			SetVariable/Z sv_Pnts win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_Hz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_LowHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_HighHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			break

		default:
			SetVariable/Z sv_Pnts win=SpHrView_PhaseInversionEdit_Window, disable=0
			SetVariable/Z sv_PhaseInversion_Hz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_LowHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			SetVariable/Z sv_PhaseInversion_HighHz win=SpHrView_PhaseInversionEdit_Window, disable=2
			break
	endswitch
	
	SetWindow SpHrView_PhaseInversionEdit_Window userdata(SignalData)="SIGNALPARAM:PhaseInversion;SIGNALNUM:"+num2str(SignalIndex)

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_PhaseInversionEditMakePoints(SignalIndex), Get PhaseInversion shapes
// =================================================================================================
function SpHrView_PhaseInversionEditMakePoints(SignalIndex)
	variable SignalIndex
	DFREF dfSav= GetdataFolderDFR()
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
	String DF = SpHrView_df_signalGlobals+"PhaseInversion:"
	SetDataFolder $DF
	wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
	if(!waveexists(PhaseInversion_Edit_Xs))
		SpHrView_PhaseInversionDefaults()
	endif
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
	wave PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
	//wave PhaseInversionVal = $DF+"PhaseInversionVal"
	wave/T InterpType = $DF+"InterpType"
	
	strswitch(PhaseInversion[SignalIndex])
		case "Normal Phase":
		case "Inverted Phase":
			if(!waveexists(PhaseInversion_Edit_Xs) || !waveexists(PhaseInversion_Edit_Ys))
				Make/O/N=(NumberEditPoints[SignalIndex]) PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys
			endif
			if(NumberEditPoints[SignalIndex] > 21 || NumberEditPoints[SignalIndex] < 6)
					NumberEditPoints[SignalIndex] = 6
				endif
			if(dimsize(PhaseInversion_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(PhaseInversion_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
				Redimension/N=(NumberEditPoints[SignalIndex]) PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys
			endif
			SetScale/I x, log(1), log(fs/2), "Hz" PhaseInversion_Edit_Xs
			SetScale d 0,0,"Hz", PhaseInversion_Edit_Xs
			SetScale d 0,0,"", PhaseInversion_Edit_Ys 
			PhaseInversion_Edit_Xs = 10^x
			strswitch(PhaseInversion[SignalIndex])
				case "Normal Phase":
					PhaseInversion_Edit_Ys = 0
					break
				case "Inverted Phase":
					PhaseInversion_Edit_Ys = 1
					break
			endswitch
			break
			
		case "Inverted Octave Band":
		case "Uninverted Octave Band":
			wave PhaseInversion_Hz = $DF+"PhaseInversion_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys//, PhaseInversion_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" PhaseInversion_Edit_Xs
			SetScale d 0,0,"Hz", PhaseInversion_Edit_Xs
			SetScale d 0,0,"", PhaseInversion_Edit_Ys
	
			variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = PhaseInversion_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = PhaseInversion_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			strswitch(PhaseInversion[SignalIndex])
				case "Inverted Octave Band":
					PhaseInversion_Edit_Ys[0,1] = 0
					PhaseInversion_Edit_Ys[2,3] = 1
					PhaseInversion_Edit_Ys[4,5] = 0
					break
				case "Uninverted Octave Band":
					PhaseInversion_Edit_Ys[0,1] = 1
					PhaseInversion_Edit_Ys[2,3] = 0
					PhaseInversion_Edit_Ys[4,5] = 1
					break
			endswitch
			// Xs
			PhaseInversion_Edit_Xs[0] = min(OctaveHzLow,1)
			PhaseInversion_Edit_Xs[1] = max(1,OctaveHzLow)
			PhaseInversion_Edit_Xs[2] = max(1,OctaveHzLow)
			PhaseInversion_Edit_Xs[3] = OctaveHzHigh
			PhaseInversion_Edit_Xs[4] = OctaveHzHigh
			PhaseInversion_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				PhaseInversion_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				PhaseInversion_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys)
			endif
			//InterpType[SignalIndex] = "Linear"
			break
		
		case "Inverted Range":	
		case "Uninverted Range":
			wave PhaseInversion_LowHz = $DF+"PhaseInversion_LowHz"
			wave PhaseInversion_HighHz = $DF+"PhaseInversion_HighHz"
	
			Make/O/N=(6) PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys//, PhaseInversion_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" PhaseInversion_Edit_Xs
			SetScale d 0,0,"Hz", PhaseInversion_Edit_Xs
			SetScale d 0,0,"", PhaseInversion_Edit_Ys
			// Ys
			strswitch(PhaseInversion[SignalIndex])
				case "Inverted Range":
					PhaseInversion_Edit_Ys[0,1] = 0
					PhaseInversion_Edit_Ys[2,3] = 1
					PhaseInversion_Edit_Ys[4,5] = 0
					break
				case "Uninverted Range":
					PhaseInversion_Edit_Ys[0,1] = 1
					PhaseInversion_Edit_Ys[2,3] = 0
					PhaseInversion_Edit_Ys[4,5] = 1
					break
			endswitch
			// Xs
			PhaseInversion_Edit_Xs[0] = min(PhaseInversion_LowHz[SignalIndex],1)
			PhaseInversion_Edit_Xs[1] = max(1,PhaseInversion_LowHz[SignalIndex])
			PhaseInversion_Edit_Xs[2] = max(1,PhaseInversion_LowHz[SignalIndex])
			PhaseInversion_Edit_Xs[3] = PhaseInversion_HighHz[SignalIndex]
			PhaseInversion_Edit_Xs[4] = PhaseInversion_HighHz[SignalIndex]
			PhaseInversion_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				PhaseInversion_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				PhaseInversion_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, PhaseInversion_Edit_Xs, PhaseInversion_Edit_Ys)
			endif
			//InterpType[SignalIndex] = "Linear"
			break
			
		default:
			// load saved wave into Spectrum_Edit_Xs amd Spectrum_Edit_Ys
			wave/Z w = $SpHrView_df_signals+PhaseInversion[SignalIndex]
			if(waveexists(w))
				//PhaseInversion[index] = S_selection
				wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
				wave/Z PhaseInversion_Edit_Ys = $DF+"PhaseInversion_Edit_Ys"
				wave NumberEditPoints = $DF+"NumberEditPoints"
				NumberEditPoints[SignalIndex] = dimsize(w,0)
				redimension/N=(NumberEditPoints[SignalIndex]) PhaseInversion_Edit_Ys, PhaseInversion_Edit_Xs
				PhaseInversion_Edit_Ys = w[p][0]
				PhaseInversion_Edit_Xs = w[p][1]
				// add point at fs/2, although unnecessary
				if(PhaseInversion_Edit_Xs[dimsize(w,0)-1] < fs/2)
					redimension/N=(dimsize(w,0)+1) PhaseInversion_Edit_Xs
					PhaseInversion_Edit_Xs[dimsize(w,0)] = fs/2
					redimension/N=(dimsize(w,0)+1) PhaseInversion_Edit_Ys
					PhaseInversion_Edit_Ys[dimsize(w,0)] = PhaseInversion_Edit_Ys[dimsize(w,0)-1]
				endif
				// add point at frequency = 1, although unnecessary
				if(PhaseInversion_Edit_Xs[0] > 1)
					insertPoints 0, 1, PhaseInversion_Edit_Xs
					PhaseInversion_Edit_Xs[0] = 1
					insertPoints 0, 1, PhaseInversion_Edit_Ys
					PhaseInversion_Edit_Ys[0] = PhaseInversion_Edit_Ys[1]
				endif
			
				if(!stringmatch(PhaseInversion[SignalIndex], "PhaseInverts_Ambient"))
					SetVariable/Z sv_PhaseInversion_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
				endif
				
			endif
			break
			
	endswitch
	

	// Write PhaseInversion values wave to be used by SpHr_SignalBinaural
	Make/O/N=(dimsize(PhaseInversion_Edit_Ys,0), 2) $SpHrView_df_signals+"PhaseInversion_"+num2str(SignalIndex)
	wave PhaseInversionWave = $SpHrView_df_signals+"PhaseInversion_"+num2str(SignalIndex)
	PhaseInversionWave[][0] = PhaseInversion_Edit_Ys[p]
	PhaseInversionWave[][1] = PhaseInversion_Edit_Xs[p]
	PhaseInversionWave[][0] = (PhaseInversionWave[p][0] < 0) ? 0 : PhaseInversionWave[p][0]
	PhaseInversionWave[][0] = (PhaseInversionWave[p][0] > 1) ? 1 : PhaseInversionWave[p][0]

	SetDataFolder dfSav
end

// =================================================================================================
// SpHrView_PhaseInversionSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_PhaseInversionSetVarHandler(string ctrlName, variable index, variable dval, variable eventMod) 
	String DF = SpHrView_df_signalGlobals+"PhaseInversion:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		
		case "sv_PhaseInversion_NoiseAmb":
		case "sv_PhaseInversion_Noise1":
		case "sv_PhaseInversion_Noise2":
			if(eventMod==5 || eventMod == 9)
				wave/T PhaseInversion = $SpHrView_df_signalGlobals+"PhaseInversion"
				//Printf "%s, %g, %g, %g, %s\r" ctrlName, index, dval, eventMod, PhaseInversion[index]
				strswitch(PhaseInversion[index])
					case "Normal Phase":
						PhaseInversion[index] = "Inverted Phase"
						break
					case "Inverted Phase":
						PhaseInversion[index] = "Normal Phase"
						break
				endswitch
			endif
			break
			
		case "sv_Pnts":
			wave/Z PhaseInversion_Edit_Xs = $DF+"PhaseInversion_Edit_Xs"
			if(!waveexists(PhaseInversion_Edit_Xs))
				SpHrView_PhaseInversionDefaults()
			endif
			wave Xs = $DF+"PhaseInversion_Edit_Xs"
			wave Ys = $DF+"PhaseInversion_Edit_Ys"
			wave NumberEditPoints = $DF+"NumberEditPoints"
			NumberEditPoints[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs, Ys)						
			SpHrView_PhaseInversionEditMakePoints(index)	// ??
			SetAxis/A/N=0/E=1/W=SpHrView_PhaseInversionEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
			break
//		case "sv_PhaseInversionVal":	
//			SpHrView_PhaseInversionEditMakePoints(index)
//			break
			

		case "sv_PhaseInversion_LowHz":
		case "sv_PhaseInversion_HighHz":
		case "sv_PhaseInversion_Hz":
		case "sv_OctaveDenominator":
		
			wave PhaseInversion_LowHz = $DF+"PhaseInversion_LowHz"
			wave PhaseInversion_HighHz = $DF+"PhaseInversion_HighHz"
			if(PhaseInversion_HighHz[index] > fs*0.5)
					PhaseInversion_HighHz[index] = fs*0.5
			endif
			if(PhaseInversion_LowHz[index] < 1)
					PhaseInversion_LowHz[index] = 1
			endif
			variable tempVar
			if(PhaseInversion_HighHz[index] < PhaseInversion_LowHz[index])
				tempVar = PhaseInversion_LowHz[index]
				PhaseInversion_LowHz[index] = PhaseInversion_HighHz[index]
				PhaseInversion_HighHz[index] = tempVar
			endif
				
			SpHrView_PhaseInversionEditMakePoints(index)
			SetAxis/A/N=0/E=1/W=SpHrView_PhaseInversionEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
	endswitch	
End


// =================================================================================================
// SpHrView_ITDButtonPopUpHandler(), ITD popup button handler
// =================================================================================================
function SpHrView_ITDButtonPopUpHandler(string ctrlName, variable index) 
	String DF = SpHrView_df_signalGlobals+"ITD:"
	String checked= "\\M0:!" + num2char(18) + ":" // checkmark coder
	String str, MenuStr
	Variable var
	String InterpTypeMenuStr = "Linear;Quadratic;Spline"
	variable DoPopupContextualMenu = 0
	
	wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
	if(!waveexists(ITD_Edit_Xs))
		SpHrView_ITDDefaults()
	endif
	
	strswitch(ctrlName)
		case "b_ITDPopup_NoiseAmb":			// SpHrView_BinauralSignal_Window
			index = 0
			break
		case "b_ITDPopup_Noise1":			// SpHrView_BinauralSignal_Window
			index = 1
			break
		case "b_ITDPopup_Noise2":			// SpHrView_BinauralSignal_Window
			index = 2
			break
		case "b_ITDPopup_Tones":			// SpHrView_BinauralSignal_Window
			index = 3
			break
	endswitch
	
	strswitch(ctrlName)
		//////////////////////////
		// SpHrView_BinauralSignal_Window
		case "b_ITDPopup_NoiseAmb":
		case "b_ITDPopup_Noise1":
		case "b_ITDPopup_Noise2":
		case "b_ITDPopup_Tones":
			wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
			MenuStr = "ITD (nearest point);ITD (phase shift);ITD (ramped phase shift);-;Shifted Octave Band;Zeroed Octave Band;Shifted Range;Zeroed Range;"
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(ITD[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+ITD[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break		
			
		////////////////////////////////////
		// SpHrView_ITDEdit_Window
		case "b_Edit":
			Button b_Edit rename=b_QuitEdit, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_ITDEdit_Window ITD_Edit_Ys
			wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
			ITD[index] = "Arbitrary"
			Button b_InterpTypePopUp win=SpHrView_ITDEdit_Window, disable=0
			SetVariable sv_InterpType win=SpHrView_ITDEdit_Window, disable=0
			/// get interpolation if Quadratic
			wave/T InterpType = $DF+"InterpType"
			wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
			wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
			SpHrView_SignalInterpEditPoints(ITD_Edit_Ys, ITD_Edit_Xs, DF+"ITD", InterpType[index])
			break
		case "b_QuitEdit":
			Button b_QuitEdit rename=b_Edit, title="✏️"
			GraphNormal/W=SpHrView_ITDEdit_Window
			wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
			wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
			if( waveexists(ITD_Edit_Xs) && waveexists(ITD_Edit_Ys) )
				//ITD_Edit_Xs = round(ITD_Edit_Xs) // round to 1 Hz?
				/// get interpolation
				wave/T InterpType = $DF+"InterpType"
				wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
				wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"	
				SpHrView_SignalInterpEditPoints(ITD_Edit_Ys, ITD_Edit_Xs, DF+"ITDInterp", InterpType[index])
				///
				//SpHrView_ITDEditMakePoints(index)
				SpHrView_ITDEditWindow(index)
			endif
			break
			
		case "b_ValuesPopUp":
			wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
			MenuStr = "ITD (nearest point);ITD (phase shift);ITD (ramped phase shift);-;Shifted Octave Band;Zeroed Octave Band;Shifted Range;Zeroed Range;"
			if(stringmatch(ITD[index], "Arbitrary"))
				MenuStr += ";-;Save Arbitrary Wave;"
			endif
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(ITD[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+ITD[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break
			
		case "b_InterpTypePopUp":
			PopupContextualMenu InterpTypeMenuStr
			if(V_flag>=1)
				DF = SpHrView_df_signalGlobals+"ITD:"
				wave/T InterpType = $DF+"InterpType"
				InterpType[index] = S_selection
				wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
				wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
				SpHrView_SignalInterpEditPoints(ITD_Edit_Ys, ITD_Edit_Xs, DF+"ITDInterp", InterpType[index])							
			endif
			break
	endswitch
	
	if(DoPopupContextualMenu)
		PopupContextualMenu MenuStr
			if(V_flag>=1)
				strswitch(S_selection)
					case "Save Arbitrary Wave":
						str = "ITD_untitled"
						Prompt str, "Wave Name:"
						DoPrompt "Save ITD Wave?", str
						if(!V_Flag)
							wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
							wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
							Duplicate/O ITD_Edit_Ys, $SpHrView_df_signals+str
							wave savedWave = $SpHrView_df_signals+str
							Redimension/N=(-1,2) savedWave
							savedWave[][1] = ITD_Edit_Xs[p]
						endif
						break
						
					case "ITD (nearest point)":
					case "ITD (phase shift)":
					case "ITD (ramped phase shift)":
						if(index==0)
							SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_ITD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==1)
							SetVariable/Z sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_ITD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==2)
							SetVariable/Z sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_ITD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						endif
						ITD[index] = S_selection
						SpHrView_ITDEditMakePoints(index)
						break
					
					case "Shifted Octave Band":
					case "Shifted Range":
					case "Zeroed Octave Band":
					case "Zeroed Range":
						if(index==0)
							//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ITD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							//SetVariable/Z sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ITD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							//SetVariable/Z sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ITD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						ITD[index] = S_selection
						SpHrView_ITDEditMakePoints(index)
						SpHrView_ITDEditWindow(index)
						break
						
//					case "ITD_Ambient":
//						if(index==0)
//							SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
//							SetVariable/Z sv_ITD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						elseif(index==1)
//							SetVariable/Z sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
//							SetVariable/Z sv_ITD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						elseif(index==2)
//							SetVariable/Z sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
//							SetVariable/Z sv_ITD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						endif
//						ITD[index] = S_selection
//						SpHrView_ITDEditMakePoints(index)
//						SpHrView_ITDEditWindow(index)
//						break

					default:
						if(index==0)
							SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ITD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_ITDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ITD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_ITDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ITD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						ITD[index] = S_selection
						SpHrView_ITDEditMakePoints(index)
						SpHrView_ITDEditWindow(index)
						break
				endswitch
			endif	// if(V_flag>=1)
	endif // if(DoPopupContextualMenu)
End


// =================================================================================================
// SpHrView_ITDDefaults(SignalIndex), ITD defaults
// =================================================================================================
function SpHrView_ITDDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	// Make "SpHrView:Signals:Globals:ITD" folder, SpHrView_df_signalGlobals+"ITD:"
	String DF = SpHrView_df_signalGlobals+"ITD:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	
	wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
	wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
	if(!waveexists(ITD_Edit_Xs) || !waveexists(ITD_Edit_Ys))
		Make/O/N=(6) ITD_Edit_Xs, ITD_Edit_Ys
		SetScale/I x, 0, fs/2, "Hz" ITD_Edit_Xs
		SetScale d 0,0,"Hz", ITD_Edit_Xs
		SetScale d 0,0,"s", ITD_Edit_Ys
	endif
	
	wave/Z ITD_Hz = $DF+"ITD_Hz"
	if(!waveexists(ITD_Hz))
		Make/O/N=(4) $DF+"ITD_Hz"; wave ITD_Hz = $DF+"ITD_Hz"
		ITD_Hz = 1000;
	endif
	wave/Z OctaveDenominator = $DF+"OctaveDenominator"
	if(!waveexists(OctaveDenominator))
		Make/O/N=(4) $DF+"OctaveDenominator"; wave OctaveDenominator = $DF+"OctaveDenominator"
		OctaveDenominator = 3;
	endif
	wave/Z ITD_LowHz = $DF+"ITD_LowHz"
	if(!waveexists(ITD_LowHz))
		Make/O/N=(4) $DF+"ITD_LowHz"; wave ITD_LowHz = $DF+"ITD_LowHz"
		ITD_LowHz = 50;
	endif
	wave/Z ITD_HighHz = $DF+"ITD_HighHz"
	if(!waveexists(ITD_HighHz))
		Make/O/N=(4) $DF+"ITD_HighHz"; wave ITD_HighHz = $DF+"ITD_HighHz"
		ITD_HighHz = 1500;
	endif
	
	wave/T/Z InterpType = $DF+"InterpType"
	if(!waveexists(InterpType))
		Make/T/O/N=(4) $DF+"InterpType"; wave/T InterpType = $DF+"InterpType"
		InterpType = "Linear";
	endif
	wave/Z NumberEditPoints = $DF+"NumberEditPoints"
	if(!waveexists(NumberEditPoints))
		Make/O/N=(4) $DF+"NumberEditPoints"; wave NumberEditPoints = $DF+"NumberEditPoints"
		NumberEditPoints = 6;
	endif
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_ITDEditWindow(SignalIndex), ITD window
// =================================================================================================
function SpHrView_ITDEditWindow(SignalIndex)
	variable SignalIndex // 0: bkgrd, 1: Noise_Noise1, 2:Noise_Noise2, 3: Tones
	DFREF dfSav = GetDataFolderDFR()	
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	String DF = SpHrView_df_signalGlobals+"ITD:"
	SetDataFolder $DF
	wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
	if(!waveexists(ITD_Edit_Xs))
		SpHrView_ITDDefaults()
	endif
	wave ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
	wave ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
	wave ITD_Hz = $DF+"ITD_Hz"
	wave OctaveDenominator = $DF+"OctaveDenominator"
	wave ITD_LowHz = $DF+"ITD_LowHz"
	wave ITD_HighHz = $DF+"ITD_HighHz"
	wave/T InterpType = $DF+"InterpType"
	wave NumberEditPoints = $DF+"NumberEditPoints"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	SpHrView_ITDEditMakePoints(SignalIndex)
	
	SpHrView_SignalInterpEditPoints(ITD_Edit_Ys, ITD_Edit_Xs, DF+"ITDInterp", InterpType[SignalIndex])

	variable winHght = 400
	variable winWdth = 770
	KillWindow/Z SpHrView_ITDEdit_Window
	Display/K=1/W=(310,90,310+winWdth,90+winHght) as "ITD"
	DoWindow/C SpHrView_ITDEdit_Window
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=18,margin(top)=7
	ControlBar 27
	
	variable Top = 3, leftPos = 5
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={130,20},pos={leftPos,Top+3},bodyWidth=130,fSize=9,value=ITD[SignalIndex]
	Button b_ValuesPopUp title="",pos={leftPos+130,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 155
	Button b_Edit title="✏️",pos={leftPos,Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 10
	SetVariable sv_Pnts title="Points",focusRing=0,size={83,20},pos={leftPos,Top+3},bodyWidth=30,value=NumberEditPoints[SignalIndex]
	SetVariable sv_Pnts limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 60
	GroupBox g_ITDVariables pos={leftPos+29,Top-1},size={432,22}
	
	leftPos += 82
	SetVariable sv_ITDVal,title="ITD (ms)", pos={leftPos,Top+3},size={45.00,16.00},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc
	SetVariable sv_ITDVal,help={"Binaural ITD."},fSize=10,format="%.3g"
	SetVariable sv_ITDVal,limits={-9999,9999,0.1},value= root:SpHrView:Signals:Globals:ITDVal[SignalIndex]
	
	leftPos += 10
	SetVariable sv_ITD_Hz title="Center",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=ITD_Hz[SignalIndex],limits={0,20000,100}
	SetVariable sv_OctaveDenominator title="Octave 1/",focusRing=0,size={120,20},pos={leftPos+85,Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator[SignalIndex],limits={0.1,48,1}
	
	leftPos += 173
	SetVariable sv_ITD_LowHz title="Range:",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=ITD_LowHz[SignalIndex],limits={0,10000,100}
	SetVariable sv_ITD_HighHz title="to",focusRing=0,size={125,20},pos={leftPos+64, Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=ITD_HighHz[SignalIndex],limits={0,96000,100}

	leftPos += 132
	SetVariable sv_InterpType title=" ",focusRing=0,size={123,20},pos={leftPos+0, Top+3},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc,value=InterpType[SignalIndex]
	Button b_InterpTypePopUp title="",pos={leftPos+123,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	AppendToGraph ITD_Edit_Ys vs ITD_Edit_Xs
	ModifyGraph mode(ITD_Edit_Ys)=4,marker(ITD_Edit_Ys)=19,msize(ITD_Edit_Ys)=2.5,lsize(ITD_Edit_Ys)=1,rgb(ITD_Edit_Ys)=(0,0,0)

	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid=1, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	ModifyGraph lblMargin(left)=3
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5
		
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
		Label bottom "Frequency\\u#2"
	else
		ModifyGraph log=0
		Label bottom "Frequency (\\U)"
	endif
	SetAxis/A/N=0/E=1/W=SpHrView_ITDEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
	
	wavestats/Q/M=1 ITD_Edit_Ys
	if(v_max == v_min || v_max-v_min < 0.00005)						// seconds
		SetAxis/Z/W=SpHrView_ITDEdit_Window left -0.0001,0.0001	// seconds
	else
		SetAxis/Z/W=SpHrView_ITDEdit_Window/A=2/N=2/E=2 left
	endif
	Label left "ITD (\\U)"
	ModifyGraph zero=1

	//  append interpolated lines if they are being used
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	wave/Z ITDInterp = $DF+"ITDInterp"
	if(!Stringmatch(ITD[SignalIndex], "ITD") && waveexists(ITDInterp))
		appendtograph ITDInterp
		ModifyGraph lsize(ITDInterp)=4,rgb(ITDInterp)=(0,0,65535,13107)
	endif
	
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=fs/2,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=20,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc
	
	strswitch(ITD[SignalIndex])
		case "ITD (nearest point)":
		case "ITD (phase shift)":
		case "ITD (ramped phase shift)":
			//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITDVal win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITD_Hz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_LowHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_HighHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_ITDEdit_Window, disable=2
			Button/Z b_InterpTypePopUp win=SpHrView_ITDEdit_Window, disable=2
			break
		case "Shifted Octave Band":
		case "Zeroed Octave Band":
			//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITDVal win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITD_Hz win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITD_LowHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_HighHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_ITDEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_ITDEdit_Window, disable=0
			break
		case "Shifted Range":
		case "Zeroed Range":
			//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITDVal win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITD_Hz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_LowHz win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITD_HighHz win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_InterpType win=SpHrView_ITDEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_ITDEdit_Window, disable=0
			break
						
		case "Arbitrary":
			//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITDVal win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITD_Hz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_LowHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_HighHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_ITDEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_ITDEdit_Window, disable=0
			break

		default:
			//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_ITDEdit_Window, disable=0
			SetVariable/Z sv_ITDVal win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_Hz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_LowHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable/Z sv_ITD_HighHz win=SpHrView_ITDEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_ITDEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_ITDEdit_Window, disable=0
			break
	endswitch
	
	SetWindow SpHrView_ITDEdit_Window userdata(SignalData)="SIGNALPARAM:ITD;SIGNALNUM:"+num2str(SignalIndex)

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_ITDEditMakePoints(SignalIndex), Get ITD shapes
// =================================================================================================
function SpHrView_ITDEditMakePoints(SignalIndex)
	variable SignalIndex
	DFREF dfSav= GetdataFolderDFR()
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
	String DF = SpHrView_df_signalGlobals+"ITD:"
	SetDataFolder $DF
	wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
	if(!waveexists(ITD_Edit_Xs))
		SpHrView_ITDDefaults()
	endif
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
	wave ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
	wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
	wave/T InterpType = $DF+"InterpType"
	
	strswitch(ITD[SignalIndex])
		case "ITD (nearest point)":
		case "ITD (phase shift)":
		case "ITD (ramped phase shift)":
			if(!waveexists(ITD_Edit_Xs) || !waveexists(ITD_Edit_Ys))
				Make/O/N=(NumberEditPoints[SignalIndex]) ITD_Edit_Xs, ITD_Edit_Ys
			endif
			if(NumberEditPoints[SignalIndex] > 21 || NumberEditPoints[SignalIndex] < 6)
					NumberEditPoints[SignalIndex] = 6
				endif
			if(dimsize(ITD_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(ITD_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
				Redimension/N=(NumberEditPoints[SignalIndex]) ITD_Edit_Xs, ITD_Edit_Ys
			endif
			SetScale/I x, log(1), log(fs/2), "Hz" ITD_Edit_Xs
			SetScale d 0,0,"Hz", ITD_Edit_Xs
			SetScale d 0,0,"s", ITD_Edit_Ys 
			ITD_Edit_Xs = 10^x
			ITD_Edit_Ys = ITDVal[SignalIndex]/1000 // ms to sec
			//InterpType[SignalIndex] = "Linear"
//			SpHrView_SignalInterpEditPoints(ITD_Edit_Ys, ITD_Edit_Xs, DF+"ITDInterp", InterpType[SignalIndex])
			break
			
		case "Shifted Octave Band":
		case "Zeroed Octave Band":
			wave ITD_Hz = $DF+"ITD_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) ITD_Edit_Xs, ITD_Edit_Ys//, ITD_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" ITD_Edit_Xs
			SetScale d 0,0,"Hz", ITD_Edit_Xs
			SetScale d 0,0,"s", ITD_Edit_Ys // !! units seconds
	
			variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = ITD_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = ITD_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			strswitch(ITD[SignalIndex])
				case "Shifted Octave Band":
					ITD_Edit_Ys[0,1] = 0
					ITD_Edit_Ys[2,3] = ITDVal[SignalIndex]/1000 // ms to sec
					ITD_Edit_Ys[4,5] = 0
					break
				case "Zeroed Octave Band":
					ITD_Edit_Ys[0,1] = ITDVal[SignalIndex]/1000 // ms to sec
					ITD_Edit_Ys[2,3] = 0
					ITD_Edit_Ys[4,5] = ITDVal[SignalIndex]/1000 // ms to sec
					break
			endswitch
			// Xs
			ITD_Edit_Xs[0] = min(OctaveHzLow,1)
			ITD_Edit_Xs[1] = max(1,OctaveHzLow)
			ITD_Edit_Xs[2] = max(1,OctaveHzLow)
			ITD_Edit_Xs[3] = OctaveHzHigh
			ITD_Edit_Xs[4] = OctaveHzHigh
			ITD_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				ITD_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				ITD_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, ITD_Edit_Xs, ITD_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, ITD_Edit_Xs, ITD_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
		
		case "Shifted Range":	
		case "Zeroed Range":
			wave ITD_LowHz = $DF+"ITD_LowHz"
			wave ITD_HighHz = $DF+"ITD_HighHz"
	
			Make/O/N=(6) ITD_Edit_Xs, ITD_Edit_Ys//, ITD_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" ITD_Edit_Xs
			SetScale d 0,0,"Hz", ITD_Edit_Xs
			SetScale d 0,0,"s", ITD_Edit_Ys // !! units are seconds
			// Ys
			strswitch(ITD[SignalIndex])
				case "Shifted Range":
					ITD_Edit_Ys[0,1] = 0
					ITD_Edit_Ys[2,3] = ITDVal[SignalIndex]/1000 // ms to sec
					ITD_Edit_Ys[4,5] = 0
					break
				case "Zeroed Range":
					ITD_Edit_Ys[0,1] = ITDVal[SignalIndex]/1000 // ms to sec
					ITD_Edit_Ys[2,3] = 0
					ITD_Edit_Ys[4,5] = ITDVal[SignalIndex]/1000 // ms to sec
					break
			endswitch
			// Xs
			ITD_Edit_Xs[0] = min(ITD_LowHz[SignalIndex],1)
			ITD_Edit_Xs[1] = max(1,ITD_LowHz[SignalIndex])
			ITD_Edit_Xs[2] = max(1,ITD_LowHz[SignalIndex])
			ITD_Edit_Xs[3] = ITD_HighHz[SignalIndex]
			ITD_Edit_Xs[4] = ITD_HighHz[SignalIndex]
			ITD_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				ITD_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				ITD_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, ITD_Edit_Xs, ITD_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, ITD_Edit_Xs, ITD_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
			
		default:
			// load saved wave into Spectrum_Edit_Xs amd Spectrum_Edit_Ys
			wave/Z w = $SpHrView_df_signals+ITD[SignalIndex]
			if(waveexists(w))
				//ITD[index] = S_selection
				wave/Z ITD_Edit_Xs = $DF+"ITD_Edit_Xs"
				wave/Z ITD_Edit_Ys = $DF+"ITD_Edit_Ys"
				wave NumberEditPoints = $DF+"NumberEditPoints"
				NumberEditPoints[SignalIndex] = dimsize(w,0)
				redimension/N=(NumberEditPoints[SignalIndex]) ITD_Edit_Ys, ITD_Edit_Xs
				ITD_Edit_Ys = w[p][0]
				ITD_Edit_Xs = w[p][1]
				// add point at fs/2, although unnecessary
				if(ITD_Edit_Xs[dimsize(w,0)-1] < fs/2)
					redimension/N=(dimsize(w,0)+1) ITD_Edit_Xs
					ITD_Edit_Xs[dimsize(w,0)] = fs/2
					redimension/N=(dimsize(w,0)+1) ITD_Edit_Ys
					ITD_Edit_Ys[dimsize(w,0)] = ITD_Edit_Ys[dimsize(w,0)-1]
				endif
				// add point at frequency = 1, although unnecessary
				if(ITD_Edit_Xs[0] > 1)
					insertPoints 0, 1, ITD_Edit_Xs
					ITD_Edit_Xs[0] = 1
					insertPoints 0, 1, ITD_Edit_Ys
					ITD_Edit_Ys[0] = ITD_Edit_Ys[1]
				endif
			
				//SetVariable/Z sv_ITDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
//				if(!stringmatch(ITD[SignalIndex] , "ITD_Ambient"))
//					SetVariable/Z sv_ITD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
//				endif
			endif
			break
			
	endswitch
	
	SpHrView_SignalInterpEditPoints(ITD_Edit_Ys, ITD_Edit_Xs, DF+"ITDInterp", InterpType[SignalIndex])
	
	// Write ITD values wave to be used by SpHr_SignalBinaural
	Make/O/N=(dimsize(ITD_Edit_Ys,0), 2) $SpHrView_df_signals+"ITD_"+num2str(SignalIndex)
	wave ITDWave = $SpHrView_df_signals+"ITD_"+num2str(SignalIndex)
	ITDWave[][0] = ITD_Edit_Ys[p]
	ITDWave[][1] = ITD_Edit_Xs[p]
	ITDWave[][0] = (ITDWave[p][0] < 0) ? 0 : ITDWave[p][0]
	ITDWave[][0] = (ITDWave[p][0] > 1) ? 1 : ITDWave[p][0]
	
	DoWindow SpHrView_ITDEdit_Window
	if(v_flag)
		wave/Z Ys = $DF+"ITD_Edit_Ys"
		wavestats/Q/M=1 Ys
		if(max(v_max,-v_min) < 0.000001)
			SetAxis/Z/W=SpHrView_ITDEdit_Window left -0.0001,0.0001
		else
			SetAxis/Z/W=SpHrView_ITDEdit_Window/A=2/N=2/E=2 left
		endif
	endif

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_ITDSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_ITDSetVarHandler(string ctrlName, variable index, variable dval) 
	String DF = SpHrView_df_signalGlobals+"ITD:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		case "sv_Pnts":
			wave/Z Xs = $DF+"ITD_Edit_Xs"
			if(!waveexists(Xs))
				SpHrView_ITDDefaults()
			endif
			wave Xs = $DF+"ITD_Edit_Xs"
			wave Ys = $DF+"ITD_Edit_Ys"
			wave NumberEditPoints = $DF+"NumberEditPoints"
			NumberEditPoints[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs, Ys)						
			SpHrView_ITDEditMakePoints(index)	// ??
			SetAxis/A/N=0/E=1/W=SpHrView_ITDEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
			wave/T InterpType = $DF+"InterpType"
			SpHrView_SignalInterpEditPoints(Ys, Xs, DF+"ITDInterp", InterpType[index])
			break
		case "sv_ITDVal":
		case "sv_ITDVal_NoiseAmb":
		case "sv_ITDVal_Noise1":
		case "sv_ITDVal_Noise2":
		case "sv_ITDVal_Tones":
			// color codes
			wave ITDVal = $SpHrView_df_signalGlobals+"ITDVal"
			// shade green if number is zero
			if(SpHr_Equal(dval, 0, 0.0000001))
				ITDVal[index] = 0.0
			else
				ITDVal[index] = dval
			endif
			
			SpHrView_BinauralSignalWindowColors(index)
						
			wave/T ITD = $SpHrView_df_signalGlobals+"ITD"
			if(stringmatch(ITD[index], "Disabled"))
				ITD[index] = "ITD (phase shift)"
			endif
			SpHrView_ITDEditMakePoints(index)
			break
			

		case "sv_ITD_LowHz":
		case "sv_ITD_HighHz":
		case "sv_ITD_Hz":
		case "sv_OctaveDenominator":
		
			wave ITD_LowHz = $DF+"ITD_LowHz"
			wave ITD_HighHz = $DF+"ITD_HighHz"
			if(ITD_HighHz[index] > fs*0.5)
					ITD_HighHz[index] = fs*0.5
			endif
			if(ITD_LowHz[index] < 1)
					ITD_LowHz[index] = 1
			endif
			variable tempVar
			if(ITD_HighHz[index] < ITD_LowHz[index])
				tempVar = ITD_LowHz[index]
				ITD_LowHz[index] = ITD_HighHz[index]
				ITD_HighHz[index] = tempVar
			endif
				
			SpHrView_ITDEditMakePoints(index)
			SetAxis/A/N=0/E=1/W=SpHrView_ITDEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
	endswitch	
End



// =================================================================================================
// SpHrView_IPDButtonPopUpHandler(), IPD popup button handler
// =================================================================================================
function SpHrView_IPDButtonPopUpHandler(string ctrlName, variable index) 
	String DF = SpHrView_df_signalGlobals+"IPD:"
	String checked= "\\M0:!" + num2char(18) + ":" // checkmark coder
	String str, MenuStr
	Variable var
	String InterpTypeMenuStr = "Linear;Quadratic;Spline"
	variable DoPopupContextualMenu = 0
	
	wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
	if(!waveexists(IPD_Edit_Xs))
		SpHrView_IPDDefaults()
	endif
	
	strswitch(ctrlName)
		case "b_IPDPopup_NoiseAmb":			// SpHrView_BinauralSignal_Window
			index = 0
			break
		case "b_IPDPopup_Noise1":			// SpHrView_BinauralSignal_Window
			index = 1
			break
		case "b_IPDPopup_Noise2":			// SpHrView_BinauralSignal_Window
			index = 2
			break
		case "b_IPDPopup_Tones":			// SpHrView_BinauralSignal_Window
			index = 3
			break
	endswitch
	
	strswitch(ctrlName)
		//////////////////////////
		// SpHrView_BinauralSignal_Window
		case "b_IPDPopup_NoiseAmb":
		case "b_IPDPopup_Noise1":
		case "b_IPDPopup_Noise2":
		case "b_IPDPopup_Tones":
			wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
			MenuStr = "IPD;-;Shifted Octave Band;Zeroed Octave Band;Shifted Range;Zeroed Range;"
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(IPD[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+IPD[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break		
			
		////////////////////////////////////
		// SpHrView_IPDEdit_Window
		case "b_Edit":
			Button b_Edit rename=b_QuitEdit, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_IPDEdit_Window IPD_Edit_Ys
			wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
			IPD[index] = "Arbitrary"
			Button b_InterpTypePopUp win=SpHrView_IPDEdit_Window, disable=0
			SetVariable sv_InterpType win=SpHrView_IPDEdit_Window, disable=0
			/// get interpolation if Quadratic
			wave/T InterpType = $DF+"InterpType"
			wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
			wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
			SpHrView_SignalInterpEditPoints(IPD_Edit_Ys, IPD_Edit_Xs, DF+"IPD", InterpType[index])
			break
		case "b_QuitEdit":
			Button b_QuitEdit rename=b_Edit, title="✏️"
			GraphNormal/W=SpHrView_IPDEdit_Window
			wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
			wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
			if( waveexists(IPD_Edit_Xs) && waveexists(IPD_Edit_Ys) )
				//IPD_Edit_Xs = round(IPD_Edit_Xs) // round to 1 Hz?
				/// get interpolation
				wave/T InterpType = $DF+"InterpType"
				wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
				wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"	
				SpHrView_SignalInterpEditPoints(IPD_Edit_Ys, IPD_Edit_Xs, DF+"IPDInterp", InterpType[index])
				///
				//SpHrView_IPDEditMakePoints(index)
				SpHrView_IPDEditWindow(index)
			endif
			break
			
		case "b_ValuesPopUp":
			wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
			MenuStr = "IPD;-;Shifted Octave Band;Zeroed Octave Band;Shifted Range;Zeroed Range;"
			if(stringmatch(IPD[index], "Arbitrary"))
				MenuStr += ";-;Save Arbitrary Wave;"
			endif
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(IPD[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+IPD[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break
			
		case "b_InterpTypePopUp":
			PopupContextualMenu InterpTypeMenuStr
			if(V_flag>=1)
				DF = SpHrView_df_signalGlobals+"IPD:"
				wave/T InterpType = $DF+"InterpType"
				InterpType[index] = S_selection
				wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
				wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
				SpHrView_SignalInterpEditPoints(IPD_Edit_Ys, IPD_Edit_Xs, DF+"IPDInterp", InterpType[index])							
			endif
			break
	endswitch
	
	if(DoPopupContextualMenu)
		PopupContextualMenu MenuStr
			if(V_flag>=1)
				strswitch(S_selection)
					case "Save Arbitrary Wave":
						str = "IPD_untitled"
						Prompt str, "Wave Name:"
						DoPrompt "Save IPD Wave?", str
						if(!V_Flag)
							wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
							wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
							Duplicate/O IPD_Edit_Ys, $SpHrView_df_signals+str
							wave savedWave = $SpHrView_df_signals+str
							Redimension/N=(-1,2) savedWave
							savedWave[][1] = IPD_Edit_Xs[p]
						endif
						break
						
					case "IPD":
						if(index==0)
							SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_IPD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==1)
							SetVariable/Z sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_IPD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==2)
							SetVariable/Z sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_IPD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						endif
						IPD[index] = S_selection
						SpHrView_IPDEditMakePoints(index)
						break
					
					case "Shifted Octave Band":
					case "Shifted Range":
					case "Zeroed Octave Band":
					case "Zeroed Range":
						if(index==0)
							//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_IPD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							//SetVariable/Z sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_IPD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							//SetVariable/Z sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_IPD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						IPD[index] = S_selection
						SpHrView_IPDEditMakePoints(index)
						SpHrView_IPDEditWindow(index)
						break
						
//					case "IPD_Ambient":
//						if(index==0)
//							SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
//							SetVariable/Z sv_IPD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						elseif(index==1)
//							SetVariable/Z sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
//							SetVariable/Z sv_IPD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						elseif(index==2)
//							SetVariable/Z sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
//							SetVariable/Z sv_IPD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						endif
//						IPD[index] = S_selection
//						SpHrView_IPDEditMakePoints(index)
//						SpHrView_IPDEditWindow(index)
//						break

					default:
						if(index==0)
							SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_IPD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_IPDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_IPD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_IPDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_IPD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						IPD[index] = S_selection
						SpHrView_IPDEditMakePoints(index)
						SpHrView_IPDEditWindow(index)
						break
				endswitch
			endif	// if(V_flag>=1)
	endif // if(DoPopupContextualMenu)
End


// =================================================================================================
// SpHrView_IPDDefaults(), IPD Defaults
// =================================================================================================
function SpHrView_IPDDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	// Make "SpHrView:Signals:Globals:IPD" folder, SpHrView_df_signalGlobals+"IPD:"
	String DF = SpHrView_df_signalGlobals+"IPD:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	
	wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
	wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
	if(!waveexists(IPD_Edit_Xs) || !waveexists(IPD_Edit_Ys))
		Make/O/N=(6) IPD_Edit_Xs, IPD_Edit_Ys
		SetScale/I x, 0, fs/2, "Hz" IPD_Edit_Xs
		SetScale d 0,0,"Hz", IPD_Edit_Xs
		SetScale d 0,0,"Cycles", IPD_Edit_Ys
	endif
	
	wave/Z IPD_Hz = $DF+"IPD_Hz"
	if(!waveexists(IPD_Hz))
		Make/O/N=(4) $DF+"IPD_Hz"; wave IPD_Hz = $DF+"IPD_Hz"
		IPD_Hz = 1000;
	endif
	wave/Z OctaveDenominator = $DF+"OctaveDenominator"
	if(!waveexists(OctaveDenominator))
		Make/O/N=(4) $DF+"OctaveDenominator"; wave OctaveDenominator = $DF+"OctaveDenominator"
		OctaveDenominator = 3;
	endif
	wave/Z IPD_LowHz = $DF+"IPD_LowHz"
	if(!waveexists(IPD_LowHz))
		Make/O/N=(4) $DF+"IPD_LowHz"; wave IPD_LowHz = $DF+"IPD_LowHz"
		IPD_LowHz = 50;
	endif
	wave/Z IPD_HighHz = $DF+"IPD_HighHz"
	if(!waveexists(IPD_HighHz))
		Make/O/N=(4) $DF+"IPD_HighHz"; wave IPD_HighHz = $DF+"IPD_HighHz"
		IPD_HighHz = 1500;
	endif
	
	wave/T/Z InterpType = $DF+"InterpType"
	if(!waveexists(InterpType))
		Make/T/O/N=(4) $DF+"InterpType"; wave/T InterpType = $DF+"InterpType"
		InterpType = "Linear";
	endif
	wave/Z NumberEditPoints = $DF+"NumberEditPoints"
	if(!waveexists(NumberEditPoints))
		Make/O/N=(4) $DF+"NumberEditPoints"; wave NumberEditPoints = $DF+"NumberEditPoints"
		NumberEditPoints = 6;
	endif	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_IPDEditWindow(SignalIndex), IPD window
// =================================================================================================
function SpHrView_IPDEditWindow(SignalIndex)
	variable SignalIndex // 0: bkgrd, 1: Noise_Noise1, 2:Noise_Noise2, 3: Tones
	DFREF dfSav= GetdataFolderDFR()
	String DF = SpHrView_df_signalGlobals+"IPD:"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
	if(!waveexists(IPD_Edit_Xs))
		SpHrView_IPDDefaults()
	endif
	wave IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
	wave IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
	wave IPD_Hz = $DF+"IPD_Hz"
	wave OctaveDenominator = $DF+"OctaveDenominator"
	wave IPD_LowHz = $DF+"IPD_LowHz"
	wave IPD_HighHz = $DF+"IPD_HighHz"
	wave/T InterpType = $DF+"InterpType"
	wave NumberEditPoints = $DF+"NumberEditPoints"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	SpHrView_IPDEditMakePoints(SignalIndex)
	SpHrView_SignalInterpEditPoints(IPD_Edit_Ys, IPD_Edit_Xs, DF+"IPDInterp", InterpType[SignalIndex])

	variable winHght = 400
	variable winWdth = 770
	KillWindow/Z SpHrView_IPDEdit_Window
	Display/K=1/W=(310,90,310+winWdth,90+winHght) as "IPD"
	DoWindow/C SpHrView_IPDEdit_Window
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=18,margin(top)=7
	ControlBar 27
	
	variable Top = 3, leftPos = 5
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={130,20},pos={leftPos,Top+3},bodyWidth=130,fSize=9,value=IPD[SignalIndex]
	Button b_ValuesPopUp title="",pos={leftPos+130,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 155
	Button b_Edit title="✏️",pos={leftPos,Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 10
	SetVariable sv_Pnts title="Points",focusRing=0,size={83,20},pos={leftPos,Top+3},bodyWidth=30,value=NumberEditPoints[SignalIndex]
	SetVariable sv_Pnts limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 60
	GroupBox g_IPDVariables pos={leftPos+29,Top-1},size={432,22}
	
	leftPos += 82
	SetVariable sv_IPDVal,title="IPD (ms)", pos={leftPos,Top+3},size={45.00,16.00},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc
	SetVariable sv_IPDVal,help={"Binaural IPD."},fSize=10,format="%.3g"
	SetVariable sv_IPDVal,limits={-2,2,0.01},value= root:SpHrView:Signals:Globals:IPDVal[SignalIndex]
	
	leftPos += 10
	SetVariable sv_IPD_Hz title="Center",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=IPD_Hz[SignalIndex],limits={0,20000,100}
	SetVariable sv_OctaveDenominator title="Octave 1/",focusRing=0,size={120,20},pos={leftPos+85,Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator[SignalIndex],limits={0.1,48,1}
	
	leftPos += 173
	SetVariable sv_IPD_LowHz title="Range:",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=IPD_LowHz[SignalIndex],limits={0,10000,100}
	SetVariable sv_IPD_HighHz title="to",focusRing=0,size={125,20},pos={leftPos+64, Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=IPD_HighHz[SignalIndex],limits={0,96000,100}

	leftPos += 132
	SetVariable sv_InterpType title=" ",focusRing=0,size={123,20},pos={leftPos+0, Top+3},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc,value=InterpType[SignalIndex]
	Button b_InterpTypePopUp title="",pos={leftPos+123,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	AppendToGraph IPD_Edit_Ys vs IPD_Edit_Xs
	ModifyGraph mode(IPD_Edit_Ys)=4,marker(IPD_Edit_Ys)=19,msize(IPD_Edit_Ys)=2.5,lsize(IPD_Edit_Ys)=1,rgb(IPD_Edit_Ys)=(0,0,0)

	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid=1, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	ModifyGraph lblMargin(left)=3
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5
		
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
		Label bottom "Frequency\\u#2"
	else
		ModifyGraph log=0
		Label bottom "Frequency (\\U)"
	endif
	SetAxis/A/N=0/E=1/W=SpHrView_IPDEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
	
	wavestats/Q/M=1 IPD_Edit_Ys
	if(v_max == v_min || v_max-v_min < 0.09)
		SetAxis/Z/W=SpHrView_IPDEdit_Window left -0.1,0.1
	else
		SetAxis/Z/W=SpHrView_IPDEdit_Window/A=2/N=2/E=2 left
	endif
	Label left "IPD (\\U)"
	ModifyGraph zero=1
	ModifyGraph lowTrip(left)=0.001

	//  append interpolated lines if they are being used
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	wave/Z IPDInterp = $DF+"IPDInterp"
	if(!Stringmatch(IPD[SignalIndex], "IPD") && waveexists(IPDInterp))
		appendtograph IPDInterp
		ModifyGraph lsize(IPDInterp)=4,rgb(IPDInterp)=(0,0,65535,13107)
	endif
	
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=fs/2,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=20,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc
	
	strswitch(IPD[SignalIndex])
		case "IPD":
			//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPDVal win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPD_Hz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_LowHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_HighHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_IPDEdit_Window, disable=2
			Button/Z b_InterpTypePopUp win=SpHrView_IPDEdit_Window, disable=2
			break
		case "Shifted Octave Band":
		case "Zeroed Octave Band":
			//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPDVal win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPD_Hz win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPD_LowHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_HighHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_IPDEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_IPDEdit_Window, disable=0
			break
		case "Shifted Range":
		case "Zeroed Range":
			//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPDVal win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPD_Hz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_LowHz win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPD_HighHz win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_InterpType win=SpHrView_IPDEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_IPDEdit_Window, disable=0
			break
						
		case "Arbitrary":
			//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPDVal win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPD_Hz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_LowHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_HighHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_IPDEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_IPDEdit_Window, disable=0
			break

		default:
			//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_IPDEdit_Window, disable=0
			SetVariable/Z sv_IPDVal win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_Hz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_LowHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable/Z sv_IPD_HighHz win=SpHrView_IPDEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_IPDEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_IPDEdit_Window, disable=0
			break
	endswitch
	
	SetWindow SpHrView_IPDEdit_Window userdata(SignalData)="SIGNALPARAM:IPD;SIGNALNUM:"+num2str(SignalIndex)

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_IPDEditMakePoints(SignalIndex), Get IPD shapes
// =================================================================================================
function SpHrView_IPDEditMakePoints(SignalIndex)
	variable SignalIndex
	DFREF dfSav= GetdataFolderDFR()
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
	String DF = SpHrView_df_signalGlobals+"IPD:"
	SetDataFolder $DF
	wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
	if(!waveexists(IPD_Edit_Xs))
		SpHrView_IPDDefaults()
	endif
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
	wave IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
	wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
	wave/T InterpType = $DF+"InterpType"
	
	strswitch(IPD[SignalIndex])
		case "IPD":
			if(!waveexists(IPD_Edit_Xs) || !waveexists(IPD_Edit_Ys))
				Make/O/N=(NumberEditPoints[SignalIndex]) IPD_Edit_Xs, IPD_Edit_Ys
			endif
			if(NumberEditPoints[SignalIndex] > 21 || NumberEditPoints[SignalIndex] < 6)
					NumberEditPoints[SignalIndex] = 6
				endif
			if(dimsize(IPD_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(IPD_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
				Redimension/N=(NumberEditPoints[SignalIndex]) IPD_Edit_Xs, IPD_Edit_Ys
			endif
			SetScale/I x, log(1), log(fs/2), "Hz" IPD_Edit_Xs
			SetScale d 0,0,"Hz", IPD_Edit_Xs
			SetScale d 0,0,"Cycles", IPD_Edit_Ys 
			IPD_Edit_Xs = 10^x
			IPD_Edit_Ys = IPDVal[SignalIndex]
			//InterpType[SignalIndex] = "Linear"
//			SpHrView_SignalInterpEditPoints(IPD_Edit_Ys, IPD_Edit_Xs, DF+"IPDInterp", InterpType[SignalIndex])
			break
			
		case "Shifted Octave Band":
		case "Zeroed Octave Band":
			wave IPD_Hz = $DF+"IPD_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) IPD_Edit_Xs, IPD_Edit_Ys//, IPD_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" IPD_Edit_Xs
			SetScale d 0,0,"Hz", IPD_Edit_Xs
			SetScale d 0,0,"Cycles", IPD_Edit_Ys // !! units are dB or Pa or V, defaults to dB
			//SetScale d 0,0,"dB", IPD_Edit_Xs // !! units are dB or Pa or V, defaults to dB
			//SetScale/P x, 0, 1, "dB" IPD_Edit_Ys
	
			variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = IPD_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = IPD_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			strswitch(IPD[SignalIndex])
				case "Shifted Octave Band":
					IPD_Edit_Ys[0,1] = 0
					IPD_Edit_Ys[2,3] = IPDVal[SignalIndex]
					IPD_Edit_Ys[4,5] = 0
					break
				case "Zeroed Octave Band":
					IPD_Edit_Ys[0,1] = IPDVal[SignalIndex]
					IPD_Edit_Ys[2,3] = 0
					IPD_Edit_Ys[4,5] = IPDVal[SignalIndex]
					break
			endswitch
			// Xs
			IPD_Edit_Xs[0] = min(OctaveHzLow,1)
			IPD_Edit_Xs[1] = max(1,OctaveHzLow)
			IPD_Edit_Xs[2] = max(1,OctaveHzLow)
			IPD_Edit_Xs[3] = OctaveHzHigh
			IPD_Edit_Xs[4] = OctaveHzHigh
			IPD_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				IPD_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				IPD_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, IPD_Edit_Xs, IPD_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, IPD_Edit_Xs, IPD_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
		
		case "Shifted Range":	
		case "Zeroed Range":
			wave IPD_LowHz = $DF+"IPD_LowHz"
			wave IPD_HighHz = $DF+"IPD_HighHz"
	
			Make/O/N=(6) IPD_Edit_Xs, IPD_Edit_Ys//, IPD_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" IPD_Edit_Xs
			SetScale d 0,0,"Hz", IPD_Edit_Xs
			SetScale d 0,0,"Cycles", IPD_Edit_Ys // !! units are dB or Pa or V, defaults to dB
			// Ys
			strswitch(IPD[SignalIndex])
				case "Shifted Range":
					IPD_Edit_Ys[0,1] = 0
					IPD_Edit_Ys[2,3] = IPDVal[SignalIndex]
					IPD_Edit_Ys[4,5] = 0
					break
				case "Zeroed Range":
					IPD_Edit_Ys[0,1] = IPDVal[SignalIndex]
					IPD_Edit_Ys[2,3] = 0
					IPD_Edit_Ys[4,5] = IPDVal[SignalIndex]
					break
			endswitch
			// Xs
			IPD_Edit_Xs[0] = min(IPD_LowHz[SignalIndex],1)
			IPD_Edit_Xs[1] = max(1,IPD_LowHz[SignalIndex])
			IPD_Edit_Xs[2] = max(1,IPD_LowHz[SignalIndex])
			IPD_Edit_Xs[3] = IPD_HighHz[SignalIndex]
			IPD_Edit_Xs[4] = IPD_HighHz[SignalIndex]
			IPD_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				IPD_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				IPD_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, IPD_Edit_Xs, IPD_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, IPD_Edit_Xs, IPD_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
			
		default:
			// load saved wave into Spectrum_Edit_Xs amd Spectrum_Edit_Ys
			wave/Z w = $SpHrView_df_signals+IPD[SignalIndex]
			if(waveexists(w))
				//IPD[index] = S_selection
				wave/Z IPD_Edit_Xs = $DF+"IPD_Edit_Xs"
				wave/Z IPD_Edit_Ys = $DF+"IPD_Edit_Ys"
				wave NumberEditPoints = $DF+"NumberEditPoints"
				NumberEditPoints[SignalIndex] = dimsize(w,0)
				redimension/N=(NumberEditPoints[SignalIndex]) IPD_Edit_Ys, IPD_Edit_Xs
				IPD_Edit_Ys = w[p][0]
				IPD_Edit_Xs = w[p][1]
				// add point at fs/2, although unnecessary
				if(IPD_Edit_Xs[dimsize(w,0)-1] < fs/2)
					redimension/N=(dimsize(w,0)+1) IPD_Edit_Xs
					IPD_Edit_Xs[dimsize(w,0)] = fs/2
					redimension/N=(dimsize(w,0)+1) IPD_Edit_Ys
					IPD_Edit_Ys[dimsize(w,0)] = IPD_Edit_Ys[dimsize(w,0)-1]
				endif
				// add point at frequency = 1, although unnecessary
				if(IPD_Edit_Xs[0] > 1)
					insertPoints 0, 1, IPD_Edit_Xs
					IPD_Edit_Xs[0] = 1
					insertPoints 0, 1, IPD_Edit_Ys
					IPD_Edit_Ys[0] = IPD_Edit_Ys[1]
				endif
			
				//SetVariable/Z sv_IPDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
//				if(!stringmatch(IPD[SignalIndex] , "IPD_Ambient"))
//					SetVariable/Z sv_IPD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
//				endif
			endif
			break
			
	endswitch
	
	SpHrView_SignalInterpEditPoints(IPD_Edit_Ys, IPD_Edit_Xs, DF+"IPDInterp", InterpType[SignalIndex])
	
	// Write IPD values wave to be used by SpHr_SignalBinaural
	Make/O/N=(dimsize(IPD_Edit_Ys,0), 2) $SpHrView_df_signals+"IPD_"+num2str(SignalIndex)
	wave IPDWave = $SpHrView_df_signals+"IPD_"+num2str(SignalIndex)
	IPDWave[][0] = IPD_Edit_Ys[p]
	IPDWave[][1] = IPD_Edit_Xs[p]
	IPDWave[][0] = (IPDWave[p][0] < 0) ? 0 : IPDWave[p][0]
	IPDWave[][0] = (IPDWave[p][0] > 1) ? 1 : IPDWave[p][0]
	
	DoWindow SpHrView_IPDEdit_Window
	if(v_flag)
		wave/Z Ys = $DF+"IPD_Edit_Ys"
		wavestats/Q/M=1 Ys
		if(max(v_max,-v_min) < 0.000001)
			SetAxis/Z/W=SpHrView_IPDEdit_Window left -0.0001,0.0001
		else
			SetAxis/Z/W=SpHrView_IPDEdit_Window/A=2/N=2/E=2 left
		endif
	endif

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_IPDSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_IPDSetVarHandler(string ctrlName, variable index, variable dval) 
	String DF = SpHrView_df_signalGlobals+"IPD:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		case "sv_Pnts":
			wave/Z Xs = $DF+"IPD_Edit_Xs"
			if(!waveexists(Xs))
				SpHrView_IPDDefaults()
			endif
			wave Xs = $DF+"IPD_Edit_Xs"
			wave Ys = $DF+"IPD_Edit_Ys"
			wave NumberEditPoints = $DF+"NumberEditPoints"
			NumberEditPoints[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs, Ys)						
			SpHrView_IPDEditMakePoints(index)	// ??
			SetAxis/A/N=0/E=1/W=SpHrView_IPDEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
			wave/T InterpType = $DF+"InterpType"
			SpHrView_SignalInterpEditPoints(Ys, Xs, DF+"IPDInterp", InterpType[index])
			break
		case "sv_IPDVal":
		case "sv_IPDVal_NoiseAmb":
		case "sv_IPDVal_Noise1":
		case "sv_IPDVal_Noise2":
			wave IPDVal = $SpHrView_df_signalGlobals+"IPDVal"
			if(SpHr_Equal(dval, 0, 0.000001))
				IPDVal[index] = 0.0
			else
				IPDVal[index] = dval
			endif
			
			SpHrView_BinauralSignalWindowColors(index)
			
			wave/T IPD = $SpHrView_df_signalGlobals+"IPD"
			if(stringmatch(IPD[index], "Disabled"))
				IPD[index] = "IPD"
			endif
			SpHrView_IPDEditMakePoints(index)
			break
			

		case "sv_IPD_LowHz":
		case "sv_IPD_HighHz":
		case "sv_IPD_Hz":
		case "sv_OctaveDenominator":
		
			wave IPD_LowHz = $DF+"IPD_LowHz"
			wave IPD_HighHz = $DF+"IPD_HighHz"
			if(IPD_HighHz[index] > fs*0.5)
					IPD_HighHz[index] = fs*0.5
			endif
			if(IPD_LowHz[index] < 1)
					IPD_LowHz[index] = 1
			endif
			variable tempVar
			if(IPD_HighHz[index] < IPD_LowHz[index])
				tempVar = IPD_LowHz[index]
				IPD_LowHz[index] = IPD_HighHz[index]
				IPD_HighHz[index] = tempVar
			endif
				
			SpHrView_IPDEditMakePoints(index)
			SetAxis/A/N=0/E=1/W=SpHrView_IPDEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
	endswitch	
End

// HELL


// =================================================================================================
// SpHrView_ILDButtonPopUpHandler(), ILD popup button handler
// =================================================================================================
function SpHrView_ILDButtonPopUpHandler(string ctrlName, variable index) 
	String DF = SpHrView_df_signalGlobals+"ILD:"
	String checked= "\\M0:!" + num2char(18) + ":" // checkmark coder
	String str, MenuStr
	Variable var
	String InterpTypeMenuStr = "Linear;Quadratic;Spline"
	variable DoPopupContextualMenu = 0
	
	wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
	if(!waveexists(ILD_Edit_Xs))
		SpHrView_ILDDefaults()
	endif
	
	strswitch(ctrlName)
		case "b_ILDPopup_NoiseAmb":			// SpHrView_BinauralSignal_Window
			index = 0
			break
		case "b_ILDPopup_Noise1":			// SpHrView_BinauralSignal_Window
			index = 1
			break
		case "b_ILDPopup_Noise2":			// SpHrView_BinauralSignal_Window
			index = 2
		case "b_ILDPopup_Tones":			// SpHrView_BinauralSignal_Window
			index = 3
			break
	endswitch
	
	strswitch(ctrlName)
		//////////////////////////
		// SpHrView_BinauralSignal_Window
		case "b_ILDPopup_NoiseAmb":
		case "b_ILDPopup_Noise1":
		case "b_ILDPopup_Noise2":
		case "b_ILDPopup_Tones":
			wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
			MenuStr = "ILD;-;Shifted Octave Band;Zeroed Octave Band;Shifted Range;Zeroed Range;"
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(ILD[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+ILD[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break		
			
		////////////////////////////////////
		// SpHrView_ILDEdit_Window
		case "b_Edit":
			Button b_Edit rename=b_QuitEdit, title="📌"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_ILDEdit_Window ILD_Edit_Ys
			wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
			ILD[index] = "Arbitrary"
			Button b_InterpTypePopUp win=SpHrView_ILDEdit_Window, disable=0
			SetVariable sv_InterpType win=SpHrView_ILDEdit_Window, disable=0
			/// get interpolation if Quadratic
			wave/T InterpType = $DF+"InterpType"
			wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
			wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
			SpHrView_SignalInterpEditPoints(ILD_Edit_Ys, ILD_Edit_Xs, DF+"ILD", InterpType[index])
			break
		case "b_QuitEdit":
			Button b_QuitEdit rename=b_Edit, title="✏️"
			GraphNormal/W=SpHrView_ILDEdit_Window
			wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
			wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
			if( waveexists(ILD_Edit_Xs) && waveexists(ILD_Edit_Ys) )
				//ILD_Edit_Xs = round(ILD_Edit_Xs) // round to 1 Hz?
				/// get interpolation
				wave/T InterpType = $DF+"InterpType"
				wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
				wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"	
				SpHrView_SignalInterpEditPoints(ILD_Edit_Ys, ILD_Edit_Xs, DF+"ILDInterp", InterpType[index])
				///
				//SpHrView_ILDEditMakePoints(index)
				SpHrView_ILDEditWindow(index)
			endif
			break
			
		case "b_ValuesPopUp":
			wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
			MenuStr = "ILD;-;Shifted Octave Band;Zeroed Octave Band;Shifted Range;Zeroed Range;"
			if(stringmatch(ILD[index], "Arbitrary"))
				MenuStr += ";-;Save Arbitrary Wave;"
			endif
			// Add list of waves in SpHrView_df_signals
			MenuStr += ";-;"+SpHrView_SignalWaveList()
			// Add check before current item
			var = WhichListItem(ILD[index], MenuStr,";",0,0)
			if(var >= 0)
				MenuStr = RemoveListItem(var, MenuStr) 
				MenuStr = AddListItem(checked+ILD[index], MenuStr, ";", var)
			endif
			DoPopupContextualMenu = 1
			break
			
		case "b_InterpTypePopUp":
			PopupContextualMenu InterpTypeMenuStr
			if(V_flag>=1)
				DF = SpHrView_df_signalGlobals+"ILD:"
				wave/T InterpType = $DF+"InterpType"
				InterpType[index] = S_selection
				wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
				wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
				SpHrView_SignalInterpEditPoints(ILD_Edit_Ys, ILD_Edit_Xs, DF+"ILDInterp", InterpType[index])							
			endif
			break
	endswitch
	
	if(DoPopupContextualMenu)
		PopupContextualMenu MenuStr
			if(V_flag>=1)
				strswitch(S_selection)
					case "Save Arbitrary Wave":
						str = "ILD_untitled"
						Prompt str, "Wave Name:"
						DoPrompt "Save ILD Wave?", str
						if(!V_Flag)
							wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
							wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
							Duplicate/O ILD_Edit_Ys, $SpHrView_df_signals+str
							wave savedWave = $SpHrView_df_signals+str
							Redimension/N=(-1,2) savedWave
							savedWave[][1] = ILD_Edit_Xs[p]
						endif
						break
						
					case "ILD":
						if(index==0)
							SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_ILD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==1)
							SetVariable/Z sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_ILD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						elseif(index==2)
							SetVariable/Z sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
							SetVariable/Z sv_ILD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=0
						endif
						ILD[index] = S_selection
						SpHrView_ILDEditMakePoints(index)
						break
					
					case "Shifted Octave Band":
					case "Shifted Range":
					case "Zeroed Octave Band":
					case "Zeroed Range":
						if(index==0)
							//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ILD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							//SetVariable/Z sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ILD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							//SetVariable/Z sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ILD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						ILD[index] = S_selection
						SpHrView_ILDEditMakePoints(index)
						SpHrView_ILDEditWindow(index)
						break
						
//					case "ILD_Ambient":
//						if(index==0)
//							SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
//							SetVariable/Z sv_ILD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						elseif(index==1)
//							SetVariable/Z sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=0
//							SetVariable/Z sv_ILD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						elseif(index==2)
//							SetVariable/Z sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=0
//							SetVariable/Z sv_ILD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(49151,53155,65535,32768)
//						endif
//						ILD[index] = S_selection
//						SpHrView_ILDEditMakePoints(index)
//						SpHrView_ILDEditWindow(index)
//						break

					default:
						if(index==0)
							SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ILD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==1)
							SetVariable/Z sv_ILDVal_Noise1 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ILD_Noise1 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						elseif(index==2)
							SetVariable/Z sv_ILDVal_Noise2 win=SpHrView_BinauralSignal_Window, disable=2
							SetVariable/Z sv_ILD_Noise2 win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
						endif
						ILD[index] = S_selection
						SpHrView_ILDEditMakePoints(index)
						SpHrView_ILDEditWindow(index)
						break
				endswitch
			endif	// if(V_flag>=1)
	endif // if(DoPopupContextualMenu)
End


// =================================================================================================
// SpHrView_ILDDefaults(), ILD Defaults
// =================================================================================================
function SpHrView_ILDDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	// Make "SpHrView:Signals:Globals:ILD" folder, SpHrView_df_signalGlobals+"ILD:"
	String DF = SpHrView_df_signalGlobals+"ILD:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	
	wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
	wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
	if(!waveexists(ILD_Edit_Xs) || !waveexists(ILD_Edit_Ys))
		Make/O/N=(6) ILD_Edit_Xs, ILD_Edit_Ys
		SetScale/I x, 0, fs/2, "Hz" ILD_Edit_Xs
		SetScale d 0,0,"Hz", ILD_Edit_Xs
		SetScale d 0,0,"dB", ILD_Edit_Ys
	endif
	
	wave/Z ILD_Hz = $DF+"ILD_Hz"
	if(!waveexists(ILD_Hz))
		Make/O/N=(4) $DF+"ILD_Hz"; wave ILD_Hz = $DF+"ILD_Hz"
		ILD_Hz = 1000;
	endif
	wave/Z OctaveDenominator = $DF+"OctaveDenominator"
	if(!waveexists(OctaveDenominator))
		Make/O/N=(4) $DF+"OctaveDenominator"; wave OctaveDenominator = $DF+"OctaveDenominator"
		OctaveDenominator = 3;
	endif
	wave/Z ILD_LowHz = $DF+"ILD_LowHz"
	if(!waveexists(ILD_LowHz))
		Make/O/N=(4) $DF+"ILD_LowHz"; wave ILD_LowHz = $DF+"ILD_LowHz"
		ILD_LowHz = 50;
	endif
	wave/Z ILD_HighHz = $DF+"ILD_HighHz"
	if(!waveexists(ILD_HighHz))
		Make/O/N=(4) $DF+"ILD_HighHz"; wave ILD_HighHz = $DF+"ILD_HighHz"
		ILD_HighHz = 1500;
	endif
	
	wave/T/Z InterpType = $DF+"InterpType"
	if(!waveexists(InterpType))
		Make/T/O/N=(4) $DF+"InterpType"; wave/T InterpType = $DF+"InterpType"
		InterpType = "Linear";
	endif
	wave/Z NumberEditPoints = $DF+"NumberEditPoints"
	if(!waveexists(NumberEditPoints))
		Make/O/N=(4) $DF+"NumberEditPoints"; wave NumberEditPoints = $DF+"NumberEditPoints"
		NumberEditPoints = 6;
	endif	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_ILDEditWindow(SignalIndex), ILD window
// =================================================================================================
function SpHrView_ILDEditWindow(SignalIndex)
	variable SignalIndex // 0: bkgrd, 1: Noise_Noise1, 2:Noise_Noise2, 3: Tones
	DFREF dfSav= GetdataFolderDFR()
	String DF = SpHrView_df_signalGlobals+"ILD:"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
	if(!waveexists(ILD_Edit_Xs))
		SpHrView_ILDDefaults()
	endif
	wave ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
	wave ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
	wave ILD_Hz = $DF+"ILD_Hz"
	wave OctaveDenominator = $DF+"OctaveDenominator"
	wave ILD_LowHz = $DF+"ILD_LowHz"
	wave ILD_HighHz = $DF+"ILD_HighHz"
	wave/T InterpType = $DF+"InterpType"
	wave NumberEditPoints = $DF+"NumberEditPoints"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	SpHrView_ILDEditMakePoints(SignalIndex)
	SpHrView_SignalInterpEditPoints(ILD_Edit_Ys, ILD_Edit_Xs, DF+"ILDInterp", InterpType[SignalIndex])

	variable winHght = 400
	variable winWdth = 770
	KillWindow/Z SpHrView_ILDEdit_Window
	Display/K=1/W=(310,90,310+winWdth,90+winHght) as "ILD"
	DoWindow/C SpHrView_ILDEdit_Window
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=18,margin(top)=7
	ControlBar 27
	
	variable Top = 3, leftPos = 5
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={130,20},pos={leftPos,Top+3},bodyWidth=130,fSize=9,value=ILD[SignalIndex]
	Button b_ValuesPopUp title="",pos={leftPos+130,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 155
	Button b_Edit title="✏️",pos={leftPos,Top-0},size={25,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 10
	SetVariable sv_Pnts title="Points",focusRing=0,size={83,20},pos={leftPos,Top+3},bodyWidth=30,value=NumberEditPoints[SignalIndex]
	SetVariable sv_Pnts limits={0,22,0.1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 60
	GroupBox g_ILDVariables pos={leftPos+29,Top-1},size={432,22}
	
	leftPos += 82
	SetVariable sv_ILDVal,title="ILD (ms)", pos={leftPos,Top+3},size={45.00,16.00},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc
	SetVariable sv_ILDVal,help={"Binaural ILD."},fSize=10,format="%.3g"
	SetVariable sv_ILDVal,limits={-60,60,3.0},value= root:SpHrView:Signals:Globals:ILDVal[SignalIndex]
	
	leftPos += 10
	SetVariable sv_ILD_Hz title="Center",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=ILD_Hz[SignalIndex],limits={0,20000,100}
	SetVariable sv_OctaveDenominator title="Octave 1/",focusRing=0,size={120,20},pos={leftPos+85,Top+3},bodyWidth=33,proc=SpHrView_EditWindow_SetVarProc,value=OctaveDenominator[SignalIndex],limits={0.1,48,1}
	
	leftPos += 173
	SetVariable sv_ILD_LowHz title="Range:",focusRing=0,size={123,20},pos={leftPos, Top+3},bodyWidth=45,proc=SpHrView_EditWindow_SetVarProc,value=ILD_LowHz[SignalIndex],limits={0,10000,100}
	SetVariable sv_ILD_HighHz title="to",focusRing=0,size={125,20},pos={leftPos+64, Top+3},bodyWidth=50,proc=SpHrView_EditWindow_SetVarProc,value=ILD_HighHz[SignalIndex],limits={0,96000,100}

	leftPos += 132
	SetVariable sv_InterpType title=" ",focusRing=0,size={123,20},pos={leftPos+0, Top+3},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc,value=InterpType[SignalIndex]
	Button b_InterpTypePopUp title="",pos={leftPos+123,Top+2},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	AppendToGraph ILD_Edit_Ys vs ILD_Edit_Xs
	ModifyGraph mode(ILD_Edit_Ys)=4,marker(ILD_Edit_Ys)=19,msize(ILD_Edit_Ys)=2.5,lsize(ILD_Edit_Ys)=1,rgb(ILD_Edit_Ys)=(0,0,0)

	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=5,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid=1, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	ModifyGraph lblMargin(left)=3
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5
		
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
		Label bottom "Frequency\\u#2"
	else
		ModifyGraph log=0
		Label bottom "Frequency (\\U)"
	endif
	SetAxis/A/N=0/E=1/W=SpHrView_ILDEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
	
	wavestats/Q/M=1 ILD_Edit_Ys
	if(v_max == v_min || v_max-v_min < 5)
		SetAxis/Z/W=SpHrView_ILDEdit_Window left -6,6
	else
		SetAxis/Z/W=SpHrView_ILDEdit_Window/A=2/N=2/E=2 left
	endif
	Label left "ILD (\\U)"
	ModifyGraph zero=1
	ModifyGraph lowTrip(left)=0.1

	//  append interpolated lines if they are being used
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	wave/Z ILDInterp = $DF+"ILDInterp"
	if(!Stringmatch(ILD[SignalIndex], "ILD") && waveexists(ILDInterp))
		appendtograph ILDInterp
		ModifyGraph lsize(ILDInterp)=4,rgb(ILDInterp)=(0,0,65535,13107)
	endif
	
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=fs/2,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=20,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc
	
	strswitch(ILD[SignalIndex])
		case "ILD":
			//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILDVal win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILD_Hz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_LowHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_HighHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_ILDEdit_Window, disable=2
			Button/Z b_InterpTypePopUp win=SpHrView_ILDEdit_Window, disable=2
			break
		case "Shifted Octave Band":
		case "Zeroed Octave Band":
			//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILDVal win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILD_Hz win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILD_LowHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_HighHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_InterpType win=SpHrView_ILDEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_ILDEdit_Window, disable=0
			break
		case "Shifted Range":
		case "Zeroed Range":
			//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILDVal win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILD_Hz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_LowHz win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILD_HighHz win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_InterpType win=SpHrView_ILDEdit_Window, disable=0
			Button/Z b_InterpTypePopUp win=SpHrView_ILDEdit_Window, disable=0
			break
						
		case "Arbitrary":
			//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
			SetVariable/Z sv_Pnts win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILDVal win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILD_Hz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_LowHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_HighHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_ILDEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_ILDEdit_Window, disable=0
			break

		default:
			//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=0
			SetVariable/Z sv_Pnts win=SpHrView_ILDEdit_Window, disable=0
			SetVariable/Z sv_ILDVal win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_Hz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_OctaveDenominator win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_LowHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable/Z sv_ILD_HighHz win=SpHrView_ILDEdit_Window, disable=2
			SetVariable sv_InterpType win=SpHrView_ILDEdit_Window, disable=0
			Button b_InterpTypePopUp win=SpHrView_ILDEdit_Window, disable=0
			break
	endswitch
	
	SetWindow SpHrView_ILDEdit_Window userdata(SignalData)="SIGNALPARAM:ILD;SIGNALNUM:"+num2str(SignalIndex)

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_ILDEditMakePoints(SignalIndex), Get ILD shapes
// =================================================================================================
function SpHrView_ILDEditMakePoints(SignalIndex)
	variable SignalIndex
	DFREF dfSav= GetdataFolderDFR()
	NVAR fs = $SpHrView_df_signalGlobals+"fs"
	wave/T ILD = $SpHrView_df_signalGlobals+"ILD"
	String DF = SpHrView_df_signalGlobals+"ILD:"
	SetDataFolder $DF
	wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
	if(!waveexists(ILD_Edit_Xs))
		SpHrView_ILDDefaults()
	endif
	wave NumberEditPoints = $DF+"NumberEditPoints"
	wave ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
	wave ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
	wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
	wave/T InterpType = $DF+"InterpType"
	
	strswitch(ILD[SignalIndex])
		case "ILD":
			if(!waveexists(ILD_Edit_Xs) || !waveexists(ILD_Edit_Ys))
				Make/O/N=(NumberEditPoints[SignalIndex]) ILD_Edit_Xs, ILD_Edit_Ys
			endif
			if(NumberEditPoints[SignalIndex] > 21 || NumberEditPoints[SignalIndex] < 6)
					NumberEditPoints[SignalIndex] = 6
				endif
			if(dimsize(ILD_Edit_Xs,0)!=NumberEditPoints[SignalIndex] || dimsize(ILD_Edit_Ys,0)!=NumberEditPoints[SignalIndex])
				Redimension/N=(NumberEditPoints[SignalIndex]) ILD_Edit_Xs, ILD_Edit_Ys
			endif
			SetScale/I x, log(1), log(fs/2), "Hz" ILD_Edit_Xs
			SetScale d 0,0,"Hz", ILD_Edit_Xs
			SetScale d 0,0,"dB", ILD_Edit_Ys 
			ILD_Edit_Xs = 10^x
			ILD_Edit_Ys = ILDVal[SignalIndex]
			//InterpType[SignalIndex] = "Linear"
//			SpHrView_SignalInterpEditPoints(ILD_Edit_Ys, ILD_Edit_Xs, DF+"ILDInterp", InterpType[SignalIndex])
			break
			
		case "Shifted Octave Band":
		case "Zeroed Octave Band":
			wave ILD_Hz = $DF+"ILD_Hz"
			wave OctaveDenominator = $DF+"OctaveDenominator"
	
			Make/O/N=(6) ILD_Edit_Xs, ILD_Edit_Ys//, ILD_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" ILD_Edit_Xs
			SetScale d 0,0,"Hz", ILD_Edit_Xs
			SetScale d 0,0,"dB", ILD_Edit_Ys
	
			variable width, OctaveHzLow, OctaveHzHigh
			width = 10^(3.0103/(10*OctaveDenominator[SignalIndex]))
			OctaveHzLow = ILD_Hz[SignalIndex] / width
			OctaveHzLow = max(0,OctaveHzLow)
		   OctaveHzHigh = ILD_Hz[SignalIndex] * width
		   OctaveHzHigh = min(fs/2, OctaveHzHigh)
			// Ys
			strswitch(ILD[SignalIndex])
				case "Shifted Octave Band":
					ILD_Edit_Ys[0,1] = 0
					ILD_Edit_Ys[2,3] = ILDVal[SignalIndex]
					ILD_Edit_Ys[4,5] = 0
					break
				case "Zeroed Octave Band":
					ILD_Edit_Ys[0,1] = ILDVal[SignalIndex]
					ILD_Edit_Ys[2,3] = 0
					ILD_Edit_Ys[4,5] = ILDVal[SignalIndex]
					break
			endswitch
			// Xs
			ILD_Edit_Xs[0] = min(OctaveHzLow,1)
			ILD_Edit_Xs[1] = max(1,OctaveHzLow)
			ILD_Edit_Xs[2] = max(1,OctaveHzLow)
			ILD_Edit_Xs[3] = OctaveHzHigh
			ILD_Edit_Xs[4] = OctaveHzHigh
			ILD_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				ILD_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				ILD_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, ILD_Edit_Xs, ILD_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, ILD_Edit_Xs, ILD_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
		
		case "Shifted Range":	
		case "Zeroed Range":
			wave ILD_LowHz = $DF+"ILD_LowHz"
			wave ILD_HighHz = $DF+"ILD_HighHz"
	
			Make/O/N=(6) ILD_Edit_Xs, ILD_Edit_Ys//, ILD_Edit_Labels
			SetScale/I x, log(1), log(fs/2), "Hz" ILD_Edit_Xs
			SetScale d 0,0,"Hz", ILD_Edit_Xs
			SetScale d 0,0,"dB", ILD_Edit_Ys // !! units are dB or Pa or V, defaults to dB
			// Ys
			strswitch(ILD[SignalIndex])
				case "Shifted Range":
					ILD_Edit_Ys[0,1] = 0
					ILD_Edit_Ys[2,3] = ILDVal[SignalIndex]
					ILD_Edit_Ys[4,5] = 0
					break
				case "Zeroed Range":
					ILD_Edit_Ys[0,1] = ILDVal[SignalIndex]
					ILD_Edit_Ys[2,3] = 0
					ILD_Edit_Ys[4,5] = ILDVal[SignalIndex]
					break
			endswitch
			// Xs
			ILD_Edit_Xs[0] = min(ILD_LowHz[SignalIndex],1)
			ILD_Edit_Xs[1] = max(1,ILD_LowHz[SignalIndex])
			ILD_Edit_Xs[2] = max(1,ILD_LowHz[SignalIndex])
			ILD_Edit_Xs[3] = ILD_HighHz[SignalIndex]
			ILD_Edit_Xs[4] = ILD_HighHz[SignalIndex]
			ILD_Edit_Xs[5] = fs/2
			if(NumberEditPoints[SignalIndex] >= 10)
				ILD_Edit_Xs[1] -= (1/fs)*4 // 50 //OctaveHzLow/10
				ILD_Edit_Xs[4] += (1/fs)*4 // 50 //OctaveHzHigh/10
			endif
			
			if(NumberEditPoints[SignalIndex] == 11)
				SpHrView_SignalRedimensionEditPoints(11, ILD_Edit_Xs, ILD_Edit_Ys)
			elseif(NumberEditPoints[SignalIndex] == 21)
				SpHrView_SignalRedimensionEditPoints(21, ILD_Edit_Xs, ILD_Edit_Ys)
			endif
			InterpType[SignalIndex] = "Linear"
			break
			
		default:
			// load saved wave into Spectrum_Edit_Xs amd Spectrum_Edit_Ys
			wave/Z w = $SpHrView_df_signals+ILD[SignalIndex]
			if(waveexists(w))
				//ILD[index] = S_selection
				wave/Z ILD_Edit_Xs = $DF+"ILD_Edit_Xs"
				wave/Z ILD_Edit_Ys = $DF+"ILD_Edit_Ys"
				wave NumberEditPoints = $DF+"NumberEditPoints"
				NumberEditPoints[SignalIndex] = dimsize(w,0)
				redimension/N=(NumberEditPoints[SignalIndex]) ILD_Edit_Ys, ILD_Edit_Xs
				ILD_Edit_Ys = w[p][0]
				ILD_Edit_Xs = w[p][1]
				// add point at fs/2, although unnecessary
				if(ILD_Edit_Xs[dimsize(w,0)-1] < fs/2)
					redimension/N=(dimsize(w,0)+1) ILD_Edit_Xs
					ILD_Edit_Xs[dimsize(w,0)] = fs/2
					redimension/N=(dimsize(w,0)+1) ILD_Edit_Ys
					ILD_Edit_Ys[dimsize(w,0)] = ILD_Edit_Ys[dimsize(w,0)-1]
				endif
				// add point at frequency = 1, although unnecessary
				if(ILD_Edit_Xs[0] > 1)
					insertPoints 0, 1, ILD_Edit_Xs
					ILD_Edit_Xs[0] = 1
					insertPoints 0, 1, ILD_Edit_Ys
					ILD_Edit_Ys[0] = ILD_Edit_Ys[1]
				endif
			
				//SetVariable/Z sv_ILDVal_NoiseAmb win=SpHrView_BinauralSignal_Window, disable=2
//				if(!stringmatch(ILD[SignalIndex] , "ILD_Ambient"))
//					SetVariable/Z sv_ILD_NoiseAmb win=SpHrView_BinauralSignal_Window, valueBackColor=(2,39321,1,13107)
//				endif
			endif
			break
			
	endswitch
	
	SpHrView_SignalInterpEditPoints(ILD_Edit_Ys, ILD_Edit_Xs, DF+"ILDInterp", InterpType[SignalIndex])
	
	// Write ILD values wave to be used by SpHr_SignalBinaural
	Make/O/N=(dimsize(ILD_Edit_Ys,0), 2) $SpHrView_df_signals+"ILD_"+num2str(SignalIndex)
	wave ILDWave = $SpHrView_df_signals+"ILD_"+num2str(SignalIndex)
	ILDWave[][0] = ILD_Edit_Ys[p]
	ILDWave[][1] = ILD_Edit_Xs[p]
	ILDWave[][0] = (ILDWave[p][0] < 0) ? 0 : ILDWave[p][0]
	ILDWave[][0] = (ILDWave[p][0] > 1) ? 1 : ILDWave[p][0]
	
	DoWindow SpHrView_ILDEdit_Window
	if(v_flag)
		wave/Z Ys = $DF+"ILD_Edit_Ys"
		wavestats/Q/M=1 Ys
		if(max(v_max,-v_min) < 0.000001)
			SetAxis/Z/W=SpHrView_ILDEdit_Window left -0.0001,0.0001
		else
			SetAxis/Z/W=SpHrView_ILDEdit_Window/A=2/N=2/E=2 left
		endif
	endif

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_ILDSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_ILDSetVarHandler(string ctrlName, variable index, variable dval) 
	String DF = SpHrView_df_signalGlobals+"ILD:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		case "sv_Pnts":
			wave/Z Xs = $DF+"ILD_Edit_Xs"
			if(!waveexists(Xs))
				SpHrView_ILDDefaults()
			endif
			wave Xs = $DF+"ILD_Edit_Xs"
			wave Ys = $DF+"ILD_Edit_Ys"
			wave NumberEditPoints = $DF+"NumberEditPoints"
			NumberEditPoints[index] = SpHrView_SignalRedimensionEditPoints(dval, Xs, Ys)						
			SpHrView_ILDEditMakePoints(index)	// ??
			SetAxis/A/N=0/E=1/W=SpHrView_ILDEdit_Window bottom kFreqAxisLow,* // min(kFreqAxisHigh,fs/2)
			wave/T InterpType = $DF+"InterpType"
			SpHrView_SignalInterpEditPoints(Ys, Xs, DF+"ILDInterp", InterpType[index])
			break
		case "sv_ILDVal":
		case "sv_ILDVal_NoiseAmb":
		case "sv_ILDVal_Noise1":
		case "sv_ILDVal_Noise2":
			wave ILDVal = $SpHrView_df_signalGlobals+"ILDVal"
					
			if(SpHr_Equal(dval, 0, 0.000001))
				ILDVal[index] = 0.0
			else
				ILDVal[index] = dval	
			endif
			
			SpHrView_BinauralSignalWindowColors(index)
				
			SpHrView_ILDEditMakePoints(index)
			break
			

		case "sv_ILD_LowHz":
		case "sv_ILD_HighHz":
		case "sv_ILD_Hz":
		case "sv_OctaveDenominator":
		
			wave ILD_LowHz = $DF+"ILD_LowHz"
			wave ILD_HighHz = $DF+"ILD_HighHz"
			if(ILD_HighHz[index] > fs*0.5)
					ILD_HighHz[index] = fs*0.5
			endif
			if(ILD_LowHz[index] < 1)
					ILD_LowHz[index] = 1
			endif
			variable tempVar
			if(ILD_HighHz[index] < ILD_LowHz[index])
				tempVar = ILD_LowHz[index]
				ILD_LowHz[index] = ILD_HighHz[index]
				ILD_HighHz[index] = tempVar
			endif
				
			SpHrView_ILDEditMakePoints(index)
			SetAxis/A/N=0/E=1/W=SpHrView_ILDEdit_Window bottom kFreqAxisLow, * // min(kFreqAxisHigh,fs/2)
	endswitch	
End

// =================================================================================================
// SpHrView_ToneButtonPopUpHandler(), Tone type popup button handler
// =================================================================================================
function SpHrView_ToneButtonPopUpHandler(string ctrlName, variable index) 
	SVAR toneType=$SpHrView_df_signalGlobals+"toneType"
	String DF = SpHrView_df_signalGlobals+"Tones:"
	strswitch(ctrlName)
		case "b_TonesTable":
			DoWindow TonesTable
			if(v_flag)
				killwindow/Z TonesTable
			else
				SpHrView_TonesTable()
			endif
			break

		case "b_Edit_Magnitude":
			Button b_Edit_Magnitude rename=b_QuitEdit_Magnitude, title="📌 Magnitude"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_TonesEdit_Window toneAmps
			break
		case "b_QuitEdit_Magnitude":
			Button b_QuitEdit_Magnitude rename=b_Edit_Magnitude, title="✏️ Magnitude"
			GraphNormal/W=SpHrView_TonesEdit_Window
			break
			
		case "b_Edit_Phase":
			Button b_Edit_Phase rename=b_QuitEdit_Phase, title="📌 Phase"
			GraphWaveEdit/M/NI/T=0/W=SpHrView_TonesEdit_Window tonePhis
			break
		case "b_QuitEdit_Phase":
			Button b_QuitEdit_Phase rename=b_Edit_Phase, title="✏️ Phase"
			GraphNormal/W=SpHrView_TonesEdit_Window
			break
			
			
			
		default:

			String checked= "\\M0:!" + num2char(18) + ":" // checkmark code
			String ToneTypeMenuStr = "Log Sweep;Linear Sweep;Linear 'Pink' Sweep;-;Pure Tone;-;Sidebands;-;Sawtooth wave harmonics;Square wave harmonics;Triange wave harmonics"
			PopupContextualMenu ToneTypeMenuStr
			if(V_flag>=1)
				toneType = S_selection
				strswitch(S_selection)
					case "Pure Tone":
						Button b_toneViewEdit_Tones, win=SpHrView_BinauralSignal_Window, disable=2
						SetVariable sv_F2_Tones, disable=3
						TitleBox tb_toneHz_Tones, title="\\f01Frequency\\f00 (Hz)"
						//SpHrVew_TonesEditMakePoints()
						KillWindow/Z SpHrView_TonesEdit_Window
						break
					case "Log Sweep":
					case "Linear Sweep":
					case "Linear 'Pink' Sweep":
						Button b_toneViewEdit_Tones, win=SpHrView_BinauralSignal_Window, disable=2
						SetVariable sv_F2_Tones, disable=0
						TitleBox tb_toneHz_Tones, title="\\f01F1 & F2\\f00 (Hz)"
						//SpHrVew_TonesEditMakePoints()
						KillWindow/Z SpHrView_TonesEdit_Window
						break
					case "Sawtooth wave harmonics":
					case "Square wave harmonics":
					case "Triange wave harmonics":
					case "Sidebands":
						Button b_toneViewEdit_Tones, win=SpHrView_BinauralSignal_Window, disable=0
						SetVariable sv_F2_Tones, disable=3
						//TitleBox tb_toneHz_Tones, title="\\f01Frequency\\f00 (Hz)"
						SpHrView_TonesEditMakePoints()
						SpHrView_TonesEditWindow()
						break
				endswitch
			endif
			break
	endswitch
End


// =================================================================================================
// SpHrView_ToneDefaults(), Tone Defaults
// =================================================================================================
function SpHrView_ToneDefaults()
	DFREF dfSav = GetDataFolderDFR()	
	// Make "SpHrView:Signals:Globals:" folder, SpHrView_df_signalGlobals
	if(!DataFolderExists(SpHrView_df_signalGlobals))
		return 0
	endif
	String DF = SpHrView_df_signalGlobals+"Tones:"
	if(!DataFolderExists(DF))
		String FolderStr = DF[0,strlen(DF)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $DF
	
	Variable nval
	string sval
	
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	
	SVAR toneType = $SpHrView_df_signalGlobals+"toneType"
	NVAR F1 = $SpHrView_df_signalGlobals+"F1" // already made?
	
	// Sweep
	nval= NumVarOrDefault(SpHrView_df_signalGlobals+"sweepF2", 20000)
	variable/G $SpHrView_df_signalGlobals+"sweepF2" = nval

	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"

	// Tone(s)
	nval= NumVarOrDefault(DF+"numTones", 12)
	variable/G numTones = nval
	nval= NumVarOrDefault(DF+"useDFTforTones", 0)
	variable/G useDFTforTones = nval
	
	nval= NumVarOrDefault(DF+"sidebandAtten", -6)
	variable/G sidebandAtten = nval
	nval= NumVarOrDefault(DF+"sidebandRateSpacing", 500)
	variable/G sidebandRateSpacing = nval

	// Tone(s)
	wave/Z toneFreqs = $DF+"toneFreqs"
	wave/Z toneAmps = $DF+"toneAmps"
	wave/Z tonePhis = $DF+"tonePhis"
	if(!waveexists(toneFreqs) || !waveexists(toneAmps) || !waveexists(tonePhis))
		Make/O/N=(numTones) toneFreqs, toneAmps, tonePhis
		toneFreqs = {250,500,1000}	// whatever
		toneAmps = {Amplitudes[3],Amplitudes[3],Amplitudes[3]} // whatever
		SetScale d 0,0,"dB", toneAmps
	endif
	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_TonesEditWindow(), Tones window
// =================================================================================================
function SpHrView_TonesEditWindow()
	DFREF dfSav = GetDataFolderDFR()	
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	String DF = SpHrView_df_signalGlobals+"Tones:"
	wave/Z toneFreqs = $DF+"toneFreqs", toneAmps = $DF+"toneAmps", tonePhis = $DF+"tonePhis"
	NVAR/Z numTones = $DF+"numTones"
	if(!NVAR_Exists(numTones) || !waveexists(toneFreqs) || !waveexists(toneAmps) || !waveexists(tonePhis))
		SpHrView_ToneDefaults()
		wave toneFreqs = $DF+"toneFreqs", toneAmps = $DF+"toneAmps", tonePhis = $DF+"tonePhis"
		NVAR numTones = $DF+"numTones"
	endif
	SVAR toneType = $SpHrView_df_signalGlobals+"toneType"
	NVAR F1 = $SpHrView_df_signalGlobals+"F1"
	
	variable winHght = 400
	variable winWdth = 770
	KillWindow/Z SpHrView_TonesEdit_Window
	Display/K=1/W=(310,90,310+winWdth,90+winHght) as "Tones"
	DoWindow/C SpHrView_TonesEdit_Window
	ModifyGraph margin(left)=43,margin(bottom)=36,margin(right)=43,margin(top)=14
	
	if(stringmatch(toneType, "Sidebands"))
		ControlBar/T 50
	else
		ControlBar/T 27
	endif
		
	variable leftPos = 20
	SetVariable sv_Values title=" ",noedit=1,focusRing=0,size={140,20},pos={leftPos,6},bodyWidth=145,fSize=10,value=toneType
	Button b_ValuesPopUp title="",pos={leftPos+140,6},size={15,15},picture=ProcGlobal#SpHrView_pict_PopUpList,proc=SpHrView_EditWindow_ButtonProc
	
	leftPos += 250
	SetVariable sv_F1_Tones,pos={leftPos,6},size={54.00,16.00},bodyWidth=55,proc=SpHrView_EditWindow_SetVarProc
	SetVariable sv_F1_Tones,help={"Frequency, fundemental frequency, or carrier frequency"}
	SetVariable sv_F1_Tones,fSize=10,format="%.1d"
	SetVariable sv_F1_Tones,limits={50,fs/2,50},variable= $SpHrView_df_signalGlobals+"F1"
	
	leftPos += 70
	SetVariable sv_numTonesBands title="#",size={116,20},pos={leftPos,6},bodyWidth=55,fSize=10,value=$DF+"numTones"
	SetVariable sv_numTonesBands limits={1,100,1}, proc=SpHrView_EditWindow_SetVarProc
	
	leftPos += 130
	Button b_Edit_Magnitude title="✏️ Magnitude",pos={leftPos,4},size={80,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	leftPos += 90
	Button b_Edit_Phase title="✏️ Phase",pos={leftPos,4},size={60,20},fSize=10,proc=SpHrView_EditWindow_ButtonProc
	
	if(stringmatch(toneType, "Sidebands"))
		leftPos = 208
		SetVariable sv_sidebandAtten title="Attenuation (dB)",size={116,20},pos={leftPos,27},bodyWidth=55,fSize=10,value=$DF+"sidebandAtten"
		SetVariable sv_sidebandAtten proc=SpHrView_EditWindow_SetVarProc
		leftPos += 131
		SetVariable sv_sidebandRateSpacing title="Rate (Hz)",size={116,20},pos={leftPos,27},bodyWidth=55,fSize=10,value=$DF+"sidebandRateSpacing"
		SetVariable sv_sidebandRateSpacing proc=SpHrView_EditWindow_SetVarProc
	endif
	
	Button b_TonesTable title="Table",pos={winWdth-135,4},fSize=10, proc=SpHrView_EditWindow_ButtonProc
	
	CheckBox ck_useDFTforTones pos={winWdth-70,4},title="use DFT",proc=SpHrView_EditWindow_CheckProc,variable=$DF+"useDFTforTones"

	strswitch(toneType)
		case "Pure Tone":
			SetVariable sv_F1_Tones,title="Hz"
			SetVariable sv_numTonesBands disable=1
			break
		case "Sawtooth wave harmonics":
		case "Square wave harmonics":
		case "Triange wave harmonics":
			SetVariable sv_F1_Tones,title="F1 (Hz)"
			SetVariable sv_numTonesBands title="# Harmonics"
			break
		case "Sidebands":
			SetVariable sv_F1_Tones,title="Carrier (Hz)"
			SetVariable sv_numTonesBands title="# Sidebands"
			SetVariable sv_numTonesBands limits={2,20,2}
	endswitch
		
	// Magnitude
	AppendToGraph toneAmps vs toneFreqs
	ModifyGraph mode(toneAmps)=8,marker(toneAmps)=19,msize(toneAmps)=3,lsize(toneAmps)=1//,rgb(toneAmps)=(0,0,0)
	ModifyGraph usePlusRGB(toneAmps)=1,rgb(toneAmps)=(65535,0,0),plusRGB(toneAmps)=(0,0,0)
	
	// Phase
	AppendToGraph/R tonePhis vs toneFreqs
	ModifyGraph mode(tonePhis)=3,marker(tonePhis)=8,msize(tonePhis)=3.5
	ModifyGraph usePlusRGB(toneAmps)=1,rgb(toneAmps)=(0,0,65535),plusRGB(toneAmps)=(0,0,0)
	ModifyGraph mrkThick(tonePhis)=1
	SetAxis right -pi, pi
	ModifyGraph grid(right)=0
	ModifyGraph manTick(right)={0,pi/4,0,2},manMinor(right)={0,50}
	Label right "Phase (radians)\\s(tonePhis)"

	ModifyGraph standoff=0,axThick=0.5,tick=2,lblMargin=0,ZisZ=1,zapTZ=1,zapLZ=1,btLen=6,btThick=0.5,stLen=4,stThick=0.5
	ModifyGraph nticks=10
	ModifyGraph grid(left)=2, gridRGB=(34952,34952,34952)
	ModifyGraph fSize=10,font="Arial"
	
	ModifyGraph zero(bottom)=1,zeroThick(bottom)=0.5//,minor(bottom)=1
	Label left "Amplitude (\\U) \\s(toneAmps)"
	ModifyGraph tickEnab(left)={-50,inf}
	
	Variable minHz = F1
	wavestats/Q/M=1 toneFreqs
	minHz = min(minHz, v_min)
	
	ModifyGraph tickUnit(left)=1
	NVAR logFreqAxis = $SpHrView_df_signalGlobals+"logFreqAxis"
	if(logFreqAxis)
		ModifyGraph log(bottom)=1,loglinear(bottom)=1
		Label bottom "Frequency\\u#2"
		SetAxis/A/E=1/N=2/W=SpHrView_TonesEdit_Window bottom minHz*0.75, *
	else
		ModifyGraph log=0
		Label bottom "Frequency (\\U)"
		SetAxis/A/E=1/N=2/W=SpHrView_TonesEdit_Window bottom minHz*0.75, *
	endif

	SetAxis/A/N=2/W=SpHrView_TonesEdit_Window left
	
	wavestats/Q/M=1 toneFreqs
	variable maxFreq = v_max
	
	Slider sl_bottomAxisMax vert=0,pos={winWdth-150,winHght-15},size={110,16},side=0,limits={0,fs/2,0},value=maxFreq,proc=AxisSliderProc
	Slider sl_bottomAxisMin vert=0,pos={60,winHght-15},size={110,16},side=0,limits={1,2000,0},value=maxFreq,proc=AxisSliderProc
	
	CheckBox ck_logFreqAxis title="log",pos={7,winHght-20},variable=$SpHrView_df_signalGlobals+"logFreqAxis",proc=SpHrView_EditWindow_CheckProc
		
	SetWindow SpHrView_TonesEdit_Window userdata(SignalData)="SIGNALPARAM:Tones;"

	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_TonesEditMakePoints(), Get Tones
// =================================================================================================
function SpHrView_TonesEditMakePoints()
	DFREF dfSav= GetdataFolderDFR()
	SpHrView_ToneDefaults()
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	wave Amplitudes = $SpHrView_df_signalGlobals+"Amplitudes"
	wave/T AmplitudeUnits = $SpHrView_df_signalGlobals+"AmplitudeUnits"
	SVAR toneType = $SpHrView_df_signalGlobals+"toneType"
	NVAR F1 = $SpHrView_df_signalGlobals+"F1"
	String DF = SpHrView_df_signalGlobals+"Tones:"
	wave toneFreqs = $DF+"toneFreqs", toneAmps = $DF+"toneAmps", tonePhis = $DF+"tonePhis"
	NVAR numTones = $DF+"numTones"
	NVAR sidebandAtten = $DF+"sidebandAtten"
	NVAR sidebandRateSpacing = $DF+"sidebandRateSpacing"

	ControlInfo/W=SpHrView_BinauralSignal_Window tb_toneHz_Tones
	variable top = V_top
	variable i
	strswitch(toneType)
		variable Hz, cnt
		case "Log Sweep":
		case "Linear Sweep":
		case "Linear 'Pink' Sweep":
			TitleBox tb_toneHz_Tones, win=SpHrView_BinauralSignal_Window, pos={175.00,top}, title="\\f01F1 & F2\\f00 (Hz)"
			break
		case "Pure-tone":
		case "Pure Tone":
			TitleBox tb_toneHz_Tones, win=SpHrView_BinauralSignal_Window, pos={175.00,top}, title="\\f01Frequency\\f00 (Hz)"
			break
			
		case "Sidebands":
			TitleBox tb_toneHz_Tones, win=SpHrView_BinauralSignal_Window, pos={175.00,top}, title="\\f01Carrier\\f00 (Hz)"
			if(mod(numTones,2)!=0) // even number of numTones
				numTones+=1
			endif
			variable numBands = numTones + 1
			if(numBands<3) // at least 2
				numBands=3
			endif
			Redimension/N=(numBands) toneFreqs, toneAmps, tonePhis
			toneFreqs=0; toneAmps=0; tonePhis=0
			toneFreqs[0] = F1;
			tonePhis = 0 // -pi/2 // 0 // SpHr_SignalBinaural uses cos()
			toneAmps[0] = Amplitudes[3];
			cnt = 1
			// upper bands
			for(i=1;i<ceil((numBands)/2);++i)
				Hz = F1 + (i*sidebandRateSpacing)  // toneFreqs[i-1]*2
				if(Hz < fs/2)
					toneFreqs[cnt] = Hz
					toneAmps[cnt] = Amplitudes[3] + (i*sidebandAtten)
					tonePhis[cnt] = 0 // -pi/2 // 0	// SpHr_SignalBinaural uses cos()
					cnt += 1
				endif
			endfor
			// lower bands
			//for(i=ceil((numBands)/2);i>0;--i)
			for(i=1;i<ceil((numBands)/2);++i)
				Hz = F1 - (i*sidebandRateSpacing)  // toneFreqs[i-1]*2
				if(Hz > 10)
					toneFreqs[cnt] = Hz
					toneAmps[cnt] = Amplitudes[3] + (i*sidebandAtten)
					tonePhis[cnt] = 0 // -pi/2 // 0 // SpHr_SignalBinaural uses cos()
					cnt += 1
				endif
			endfor
			Redimension/N=(cnt) toneFreqs, toneAmps, tonePhis
			sort toneFreqs,toneFreqs, toneAmps, tonePhis
			break
		
		case "Sawtooth wave harmonics":
			TitleBox tb_toneHz_Tones, win=SpHrView_BinauralSignal_Window, pos={175.00,top}, title="\\f01F1\\f00 (Hz)"
			Redimension/N=(numTones) toneFreqs, toneAmps, tonePhis
			//NVAR useDFTforTones = $DF+"useDFTforTones"; useDFTforTones = 0 // not precise enough... ????
			toneFreqs[0] = F1;
			tonePhis = -pi/2 // SpHr_SignalBinaural uses cos()
			toneAmps[0] = Amplitudes[3];
			for(i=1;i<numTones;++i)
				Hz = F1*(i+1)   // toneFreqs[i-1]*2
				if(Hz > fs/2)
					numTones = i
					Redimension/N=(numTones) toneFreqs, toneAmps, tonePhis
					break
				endif
				toneFreqs[i] = Hz
				toneAmps[i] = Amplitudes[3] + 20*log(1/(i+1)) // 20*log(1/(2^i))
				tonePhis[i] = -pi/2	// SpHr_SignalBinaural uses cos()
			endfor
			break
			
		case "Square wave harmonics":
			TitleBox tb_toneHz_Tones, win=SpHrView_BinauralSignal_Window, pos={175.00,top}, title="\\f01F1\\f00 (Hz)"
			Redimension/N=(numTones) toneFreqs, toneAmps, tonePhis
			//NVAR useDFTforTones = $DF+"useDFTforTones"; useDFTforTones = 0 // not precise enough...
			toneFreqs[0] = F1
			tonePhis = -pi/2 // SpHr_SignalBinaural uses cos()
			toneAmps[0] = Amplitudes[3];
			cnt = 1
			for(i=1;i<numTones*2;++i)	// X2 points
				if(mod(i,2)==0)				// odd numbered harmonics
					Hz = F1*(i+1)  // toneFreqs[i-1]*2
					if(Hz > fs/2)
						numTones = i
						Redimension/N=(numTones) toneFreqs, toneAmps, tonePhis
						break
					endif
					toneFreqs[cnt] = Hz
					toneAmps[cnt] = Amplitudes[3] + 20*log(1/(i+1)) // 20*log(1/(2^i))
					tonePhis[cnt] = -pi/2	// SpHr_SignalBinaural uses cos()
					cnt += 1
				endif
			endfor
			break
			
		case "Triange wave harmonics":
			TitleBox tb_toneHz_Tones, win=SpHrView_BinauralSignal_Window, pos={175.00,top}, title="\\f01F1\\f00 (Hz)"
			Redimension/N=(numTones) toneFreqs, toneAmps, tonePhis
			//NVAR useDFTforTones = $DF+"useDFTforTones"; useDFTforTones = 0 // not precise enough...
			toneFreqs[0] = F1
			tonePhis = 0 // SpHr_SignalBinaural uses cos()
			toneAmps[0] = Amplitudes[3];
			cnt = 1
			for(i=1;i<numTones*2;++i)	// X2 points
				if(mod(i,2)==0)				// odd numbered harmonics
					Hz = F1*(i+1)  // toneFreqs[i-1]*2
					if(Hz > fs/2)
						numTones = i
						Redimension/N=(numTones) toneFreqs, toneAmps, tonePhis
						break
					endif
					toneFreqs[cnt] = Hz
					toneAmps[cnt] = Amplitudes[3] + 20*log(1/(i+1)^2) // 20*log(1/(2^i))
					tonePhis[cnt] = 0 // SpHr_SignalBinaural uses cos()
					cnt += 1
				endif
			endfor
			break
			
	endswitch	
	
	SetDataFolder dfSav
End

// =================================================================================================
// SpHrView_TonesSetVarHandler(), SetVar handler
// =================================================================================================
function SpHrView_TonesSetVarHandler(string ctrlName, variable index, variable dval) 
	String DF = SpHrView_df_signalGlobals+"Tones:"
	NVAR fs = $SpHrView_df_signalGlobals+"fs" // global fs
	strswitch( ctrlName )
		case "sv_numTonesBands":
		case "sv_F1_Tones":
		case "sv_sidebandAtten":
		case "sv_sidebandRateSpacing":
			wave/Z toneFreqs = $DF+"toneFreqs"
			if(!waveexists(toneFreqs))
				SpHrView_ToneDefaults()
			endif
			SpHrView_TonesEditMakePoints()
			break
	endswitch	
End

Function SpHrView_TonesTable()
	String DF = SpHrView_df_signalGlobals+"Tones:"
	wave/Z toneFreqs = $DF+"toneFreqs"
	wave/Z toneAmps = $DF+"toneAmps"
	wave/Z tonePhis = $DF+"tonePhis"
	if(waveexists(toneFreqs))
		killwindow/Z TonesTable
		Edit/K=1/W=(1100,90,1400,480) toneFreqs,toneAmps,tonePhis
		DoWindow/C TonesTable
		ModifyTable format(Point)=1,width(toneAmps)=70,width(toneFreqs)=70,width(tonePhis)=70
	endif
End
