﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.


strconstant SpHrView_df = "root:SpHrView:"
strconstant SpHrView_df_globals = "root:SpHrView:Globals:"
strconstant SpHrView_df_files = "root:SpHrView:Files:"
strconstant SpHrView_df_waves = "root:SpHrView:Waves:"
strconstant SpHrView_df_waves_globals = "root:SpHrView:Waves:Globals:"
strconstant SpHrView_df_spectra = "root:SpHrView:Spectra:"
strconstant SpHrView_df_movies = "root:SpHrView:Movies:"

strconstant Char_swapArrowStr = "\\f01↻\\f00"
strconstant Char_swapArrow10_9_Str = "\\Z10\\f01↻ \\Z09\\f00"

// =================================================================================================
// SpHrView_ViewMenu(), pop-up menu of displays
// =================================================================================================
function SpHrView_ViewMenu()
	String ViewOptions = "", ViewSelection
	ViewOptions += "Carrier Waveforms;"
	ViewOptions += "Carrier Envelopes;"
	ViewOptions += "\M1-;"
	ViewOptions += "Filter Bank Waveforms;"
	ViewOptions += "Filter Bank Envelopes;"
	ViewOptions += "\M1-;"
	ViewOptions += "Amplitude Spectra;"
	ViewOptions += "Amplitude Spectra Over Time;"
	ViewOptions += "\M1-;"
	ViewOptions += "Correlation;"
	ViewOptions += "\M1-;"
	ViewOptions += "Cue Distributions;"
	ViewOptions += "Cues Over Time;"
	ViewOptions += "\M1-;"
	ViewOptions += "Representations At Time;"
	ViewOptions += "Representations Over Time;"
	ViewOptions += "\M1-;"
	ViewOptions += "Waveform Statistics;"
	PopupContextualMenu ViewOptions
	if(V_flag)
		ViewSelection = S_selection
	else	
		return 0
	endif
	//Print ViewSelection
	strswitch(ViewSelection)					
			case "Carrier Waveforms":
				SpHrView_DisplayCarrierWaves(0,0);
				break
			case "Carrier Envelopes":
				SpHrView_DisplayCarrierWaves(1,0);
				break
			case "Filter Bank Waveforms":
				SpHrView_DisplayFilteredWaves(0);
				break
			case "Filter Bank Envelopes":
				SpHrView_DisplayFilteredWaves(1);
				break
			case "Amplitude Spectra":
				SpHrView_DisplaySpectra();
				break
			case "Amplitude Spectra Over Time":
				SpHrView_DisplaySTFT();
				break
			case "Correlation":
				SpHr_Correlation();
				break
			case "Cue Distributions":
				SpHrView_CueHistogramOptionsWindow(-1,-1)
				break
			case "Cues Over Time":
				SpHrView_CuesOptionsWindow(-1,-1)
				break
			case "Representations At Time":
				SpHrView_AtTime_OptionsWindow(-1,-1)
				break
			case "Representations Over Time":
				SpHrView_OverTime_OptionsWindow(-1,-1)
				//SpHrView_RepOverTime_OptionsWindow(-1,-1)
				break
			case "Waveform Statistics":
				SpHrView_WaveformStatistics(1);
				break
	endswitch
end

//"Correlation", /Q, SpHr_Correlation()
//		"-"
//		"Cue Distributions", /Q, SpHrView_CueHistogramOptionsWindow(-1,-1)
//		"Cues Over Time", /Q, SpHrView_CuesOptionsWindow(-1,-1)
//		"-"
//		"Representations At Time" , /Q, SpHrView_AtTime_OptionsWindow(-1,-1)
//		"Representations Over Time" , /Q, SpHrView_OverTime_OptionsWindow(-1,-1)
//		"-"
//		"Waveform Statistics", /Q, SpHrView_WaveformStatistics()

// =================================================================================================
// SpHrView_OpenFile(), open sound file and get its peaks
// =================================================================================================
function SpHrView_OpenFile(Variable getRecentFiles)
	SpHrView_Defaults() // check defaults
	if(getRecentFiles)
		SVAR RecentFileNames = $SpHrView_df_globals+"RecentFileNames"
		SVAR RecentFilePaths = $SpHrView_df_globals+"RecentFilePaths"
		String FileNamesStr = RecentFileNames
		String FilePathsStr = RecentFilePaths
		if(ItemsinList(FileNamesStr)<1 || ItemsinList(FilePathsStr)<1)
			DoAlert/T="No files" 0, "No sound files appear to have been opened."
			return 0
		endif
		//GetMouse
		//Printf "%g, %g\r", V_left, V_top
		//PopupContextualMenu/C=(V_left, V_top) FileNamesStr
		PopupContextualMenu FileNamesStr
		if(V_flag)
			string waveNameStr = StringFromList(V_flag-1, FilePathsStr)
			Killwindow/Z SpHrViewPeaksWindow
			SpHrView_LoadPeaksFromFile(waveNameStr, 0, 0, 0, 0, 0, 0)
		endif
	else
		SpHrView_LoadPeaksFromFile("", 0, 0, 0, 0, 0, 0)
	endif
	return 0
End

// =================================================================================================
// SpHrView_Defaults(), Default datafolders, variables, strings, etc. for SpHrViewer routines
// =================================================================================================
function SpHrView_Defaults()
	Variable nval
	String sval, FolderStr
	// Make "SpHrView" folder, SpHrView_df
	if(!DataFolderExists(SpHrView_df))
		FolderStr= SpHrView_df[0,strlen(SpHrView_df)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Globals" folder, SpHrView_df_globals
	if(!DataFolderExists(SpHrView_df_globals))
		FolderStr= SpHrView_df_globals[0,strlen(SpHrView_df_globals)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Files" folder, SpHrView_df_files
	if(!DataFolderExists(SpHrView_df_files))
		FolderStr= SpHrView_df_files[0,strlen(SpHrView_df_files)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Waves" folder, SpHrView_df_waves
	if(!DataFolderExists(SpHrView_df_waves))
		FolderStr= SpHrView_df_waves[0,strlen(SpHrView_df_waves)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Waves:Globals" folder, SpHrView_df_waves_globals
	if(!DataFolderExists(SpHrView_df_waves_globals))
		FolderStr= SpHrView_df_waves_globals[0,strlen(SpHrView_df_waves_globals)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Spectra" folder, SpHrView_df_spectra
	if(!DataFolderExists(SpHrView_df_spectra))
		FolderStr= SpHrView_df_spectra[0,strlen(SpHrView_df_spectra)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	// Make "SpHrView:Movies" folder, SpHrView_df_movies
	if(!DataFolderExists(SpHrView_df_movies))
		FolderStr= SpHrView_df_movies[0,strlen(SpHrView_df_movies)-2] // remove trailing ":"
		NewDataFolder/O $FolderStr
	endif
	SetDataFolder $SpHrView_df_globals
	// screen info
	string infoStr= IgorInfo(0)
	variable/G numScreens = NumberByKey("NSCREENS", infoStr)						// Number of active displays, not used
	string screenInfo = StringByKey("SCREEN1", infoStr)								// first screen
	variable/G screenVert = str2num(ParseFilePath(0, screenInfo, ",", 1, 0))
	variable/G screenHoriz = str2num(ParseFilePath(0, screenInfo, ",", 1, 1))
	//variable/G memoryUsed = NumberByKey("USEDPHYSMEM", IgorInfo(0))/1024/1024 	// MB
	//Printf "memory used = %g MB, (%g %%)\r" memoryUsed, NumberByKey("USEDPHYSMEM", IgorInfo(0))/NumberByKey("PHYSMEM", IgorInfo(0))
	nval= NumVarOrDefault(SpHrView_df_globals+"LoadAsIgorWaveForSpeed", 1)	// load file as Igor save to speed up reading
	variable/G LoadAsIgorWaveForSpeed = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"preserveWaveTemporarily", 0)	// 
	variable/G preserveWaveTemporarily = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"NumberViewPeaks", 2000)	// ~number of points used to represent waveform peaks
	variable/G NumberViewPeaks = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"loadPromptLength", 2)			// max file length (minutes) prompting load options window
	variable/G loadPromptLength = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"fs", 0)								// current file's sampling rate
	variable/G fs = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"numSamples", 0)					// number of points 
	variable/G numSamples = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"dBscale", 0)						// option to display on dB (amplitude) scales
	variable/G dBscale = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"PeaksMindB", 0)					// minimum amplitude of loaded peaks
	variable/G PeaksMindB = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"PeaksMaxdB", 0)					// maximum amplitude of loaded peaks
	variable/G PeaksMaxdB = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"PeaksAvgdB", 0)					// average amplitude of loaded peaks
	variable/G PeaksAvgdB = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"PeaksQ1dB", 0)						// first quartile of loaded amplitude peaks
	variable/G PeaksQ1dB = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"PeaksQ2dB", 0)						// second quartile of loaded amplitude peaks (median)
	variable/G PeaksQ2dB = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"PeaksQ3dB", 0)						// third quartile of loaded amplitude peaks
	variable/G PeaksQ3dB = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"loadOutOfPhase", 0)				// reverse signal phase when loading
	variable/G loadOutOfPhase = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"file_start_min", 0)				// *prompt* for file end time (min.)
	variable/G file_start_min = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"file_end_min", 1)					// *prompt* for file end time (min.)
	variable/G file_end_min = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"normalizeSoundPlay", 1)			// normalize sound playing (to a min/max of 1)
	variable/G normalizeSoundPlay = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"numChannels", 1)					// number of Channels in File
	variable/G numChannels = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"ChannelPair", 1)					// load channels 1&2, 3&4, 5&6, or 7&8
	variable/G ChannelPair = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"ChannelPairSelected", 0)		// channel pair selected
	variable/G ChannelPairSelected = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"useDrawSelection", 1)			// use draw layer to make selection
	variable/G useDrawSelection = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"autoscaleAmplitudeAxis", 1)	// autoscale the Y axis when viewing a SubRange
	variable/G autoscaleAmplitudeAxis = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"ViewWidthProportion", 0.95)	// propotion of screen width used for display
	variable/G ViewWidthProportion = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"ViewHeightProportion", 0.2)	// propotion of screen height used for display
	variable/G ViewHeightProportion = nval

	nval= NumVarOrDefault(SpHrView_df_globals+"marqueeLeft", NaN)				// last graph marquee coordinate
	variable/G marqueeLeft = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"marqueeRight", NaN)				// last graph marquee coordinate
	variable/G marqueeRight = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"marqueeTop", NaN)					// last graph marquee coordinate
	variable/G marqueeTop = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"marqueeBottom", NaN)				// last graph marquee coordinate
	variable/G marqueeBottom = nval
	
	nval= NumVarOrDefault(SpHrView_df_globals+"isMovie", 0)						// file appears to be a movie
	variable/G isMovie = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"MovieHeight", 275)				// movie height
	variable/G MovieHeight = nval
	
	nval= NumVarOrDefault(SpHrView_df_globals+"SPLRefenceScale", 1.0)				// SPL reference scale
	variable/G SPLRefenceScale = nval
	nval= NumVarOrDefault(SpHrView_df_globals+"ApplySPLScale", 1)					// Apply SPL reference scale
	variable/G ApplySPLScale = nval
	
	nval= NumVarOrDefault(SpHrView_df_waves_globals+"PlayWaveSelection", 0)		// Play selected wave
	variable/G $SpHrView_df_waves_globals+"PlayWaveSelection" = nval
	
	sval= StrVarOrDefault(SpHrView_df_globals+"RecentFileNames", "")
	string/G RecentFileNames = sval
	sval= StrVarOrDefault(SpHrView_df_globals+"RecentFilePaths", "")
	string/G RecentFilePaths = sval

	// Check Filter defaults if SpHrView_Filter is loaded and the function SpHrView_FilterDefaults exists 
	infostr = FunctionInfo("SpHrView_FilterDefaults")
	if(strlen(infostr) > 0)
		Execute/Q "SpHrView_FilterDefaults()"
	endif
	
	SetDataFolder root:
End


// =================================================================================================
// SpHrView_PeaksWindowHook(), close and re-open SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_PeaksWindowReopen()
	// re-load sound file
	NVAR file_start_min= $SpHrView_df_globals+"file_start_min"
	NVAR file_end_min= $SpHrView_df_globals+"file_end_min"
	if(file_end_min - file_start_min > 2)
		String AlertStr
		SPrintf AlertStr, "Reload all %g minutes?", file_end_min - file_start_min
		DoAlert/T="Reload file?" 1, AlertStr
		if(V_flag==2)
			return 0
		endif
	endif
	String traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
	String wStr =  StringFromList(0,traceListStr)
	wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
	string pathAndNameStr= ""
	variable referenceIsFile=1
	pathAndNameStr= SpHrView_FilePath_Get(w, 2)
	if(strlen(pathAndNameStr)<1)
		referenceIsFile=0
		pathAndNameStr= SpHrView_FilePath_Get(w, 1)
	endif
	//Printf "referenceIsFile=%g, pathAndNameStr=%s\r", referenceIsFile, pathAndNameStr
	//Printf "file_start_min=%g, file_end_min=%g\r", file_start_min, file_end_min
	NVAR preserveWaveTemporarily = $SpHrView_df_globals+"preserveWaveTemporarily"
	preserveWaveTemporarily=11 // ?? why
	KillWindow/Z SpHrViewPeaksWindow
	if(referenceIsFile)
		Execute/P/Q/Z "SpHrView_LoadPeaksFromFile(\""+pathAndNameStr+"\", "+num2str(file_start_min*60)+","+num2str(file_end_min*60)+",0,1,0,0)"
	else
		String nameStr = ParseFilePath(0, pathAndNameStr, ":", 1, 0)
		Execute/P/Q/Z "SpHrView_LoadPeaksFromWave(\""+nameStr+"\", "+num2str(file_start_min*60)+","+num2str(file_end_min*60)+",0,1,0,0)"
	endif
End

// =================================================================================================
// SpHrView_PeaksWindowHook(s), Window hook for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_PeaksWindowHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0
	variable i, xval, xPnt, HzIndex
	String traceListStr, imageListStr, wStr, ACursorStr, IndicesStr
	Variable isImage=0
	Variable xmin, xmax, peaks_x_min, peaks_x_max, new_min, new_max
	
	switch(s.eventCode)
	
		//////////////
		// deactivate
		case 1:
			strswitch(s.winName)
				case "SpHrViewSettings":
					ControlInfo/W=SpHrViewSettings ck_keepWindowOpen
					if(!V_Value)
						Execute/Z/Q/P "killwindow/Z SpHrViewSettings"
						hookResult=1
					endif
					break
			endswitch
			break
	
		//////////////
		// kill event
		case 2:
			strswitch(s.winName)					
				case "SpHrViewPeaksWindow":

					// save valid graph marquee coordinates if drawn
					GetMarquee/W=SpHrViewPeaksWindow left, bottom
					if(V_flag)
						NVAR marqueeLeft = $SpHrView_df_globals+"marqueeLeft"; marqueeLeft=V_left
						NVAR marqueeRight = $SpHrView_df_globals+"marqueeRight"; marqueeRight= V_right
						NVAR marqueeTop = $SpHrView_df_globals+"marqueeTop"; marqueeTop=V_top
						NVAR marqueeBottom = $SpHrView_df_globals+"marqueeBottom"; marqueeBottom=V_bottom
					else
						NVAR marqueeLeft = $SpHrView_df_globals+"marqueeLeft"; marqueeLeft=NaN
						NVAR marqueeRight = $SpHrView_df_globals+"marqueeRight"; marqueeRight= NaN
						NVAR marqueeTop = $SpHrView_df_globals+"marqueeTop"; marqueeTop=NaN
						NVAR marqueeBottom = $SpHrView_df_globals+"marqueeBottom"; marqueeBottom=NaN
					endif
				
					traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)

					Execute/Z/Q/P "KillWavesInListString(\""+traceListStr+"\",\""+SpHrView_df_files+"\",\"\")"	
					Execute/Z/Q/P "KillWavesInListString(\""+traceListStr+"\",\""+SpHrView_df_files+"\",\"_subrange\")"	
					Execute/Z/Q/P "KillWavesInListString(\""+traceListStr+"\",\""+SpHrView_df_files+"\",\"_wav\")"	
					
					// close all graphs with "PACKAGE:SpHrView" in note field
					// set using:  SetWindow SpHrView_SomeWindow note="PACKAGE:SpHrView"
					Execute/Z/Q/P "KillGraphsWithNoteField(\"PACKAGE:SpHrView\")"	

					Execute/Z/Q/P "killwindow/Z WaveFilterSettings"
					Execute/Z/Q/P "killwaves/Z $\""+SpHrView_df_movies+"M_MovieFrame\""
					//Execute/Z/Q/P "killwaves/Z $\""+SpHrView_df_filter+"FilterBank\""
					
					hookResult = 0 // 1?
					break
				
				case "SpHrViewMovieWindow":

					Execute/Z/Q/P "killwaves/Z $\""+SpHrView_df_movies+"M_MovieFrame\""
					NVAR/Z movieFileRef = $SpHrView_df_movies+"movieFileRef"
					if(NVAR_Exists(movieFileRef))
						PlayMovieAction/Z setFrontMovie = movieFileRef
						PlayMovieAction/Z kill
					endif
					NVAR/Z movieClipRef = $SpHrView_df_movies+"movieClipRef"
					if(NVAR_Exists(movieClipRef))
						PlayMovieAction/Z setFrontMovie = movieClipRef
						PlayMovieAction/Z kill
					endif
					hookResult = 1 // 0?
					break
					
				case "Waveforms_Window":
					traceListStr= TraceNameList("Waveforms_Window", ";", 1)
					Execute/Z/Q/P "KillWavesInListString(\""+traceListStr+"\",\""+SpHrView_df_files+"\",\"\")"	
					
			endswitch
			break
			
		//////////////
		// keyboard event
		case 11:
			variable key= getkeystate(0) // 1: command, 2: option, 16: control // up arrow 256, 257 with commmand, down arrow 512, 513 with command
			//Printf "%g, %s, %g\r", s.keycode, s.keyText, key	
			hookResult = 1 // keep window on top if key presssed
			switch(s.keycode)
				case 13:		// return
				
					//strswitch(s.winName)
					//	case "SpHrViewPeaksWindow":
					//		break
					//endswitch
					break
					
				
								// Recent Files
				case 82:		// "r" or "R" WITHOUT modifier
				case 114:
					SpHrView_OpenFile(1)
					hookResult = 1
					break
					
								// View
				case 86:		// "v" or "V" WITHOUT modifier
				case 118:
					SpHrView_ViewMenu()
					hookResult = 1
					break
								// Histogram
				case 72:		// "h" or "H" WITHOUT modifier
				case 104:
								// or Distribution
				case 68:		// "D" or "D" WITHOUT modifier
				case 100:
					SpHrView_CueHistogram_PopUpMenu()
					hookResult = 1
					break

								// clear draw layer
				case 67:		// "c" or "C" WITHOUT modifier
				case 99:
					SpHrView_DrawSelection(0, 0, 0)
					hookResult = 1
					break
								// Stereo Waveforms
				case 87:		// "w" or "W" WITHOUT modifier
				case 119:
					SpHrView_DisplayCarrierWaves(0, 0) 	// view waveforms
					hookResult = 1
					break
					
								// Stereo Waveform Envelopes
				case 69:		// "e" or "E" WITHOUT modifier
				case 101:
					SpHrView_DisplayCarrierWaves(1, 0) 	// view waveforms
					hookResult = 1
					break
								// Play sound
				case 32:		// space bar
				case 80:		// "p" or "P" WITHOUT modifier
				case 112:
						// stop option
						NVAR/Z PlaySoundTicks=$SpHrView_df_globals+"PlaySoundTicks"
						NVAR/Z PlayStartTicks=$SpHrView_df_globals+"PlayStartTicks"
					   NVAR/Z PlayStartTime=$SpHrView_df_globals+"PlayStartTime"
					   if(NVAR_Exists(PlaySoundTicks) && NVAR_Exists(PlayStartTicks))
					   	SpHrView_PlaySoundTaskStop()
							StopPlaySound()		// stop playing sounds, IgorGeneralProcedures
					   else
							SpHrView_PlaySound()	// play sounds
						endif
						hookResult = 1
					break
								// Stop playing sound
				case 83:		// "s" or "S" WITHOUT modifier
				case 115:
						SpHrView_PlaySoundTaskStop()
						StopPlaySound()		// stop playing sounds, IgorGeneralProcedures
						hookResult = 1
					break
					
				case 28:		// left arrow
				case 64:		// left arrow, with shift
					i = -1 							// PlayMovieAction step= i
				case 29:		// right arrow
				case 128:		// right arrow, with shift
					if(i==0)
						i = 1							// PlayMovieAction step= i
					endif
					if(stringmatch(s.winName, "SpHrViewMovieWindow"))
						DFREF saveDFR = GetDataFolderDFR()
						SetDataFolder $SpHrView_df_movies
						NVAR NumFrames=$SpHrView_df_movies+"NumFrames"
						NVAR FrameLength=$SpHrView_df_movies+"FrameLength"
						PlayMovieAction step= i
						PlayMovieAction extract
						doupdate/W=SpHrViewMovieWindow
						SetDataFolder saveDFR
						// set cursor
						PlayMovieAction getTime
						Cursor/S=2/L=0/H=2/C=(0,65535/2,0, 65535/2 )/W=SpHrViewPeaksWindow A, $CsrWave(A), V_value
						DoUpdate/W=SpHrViewPeaksWindow
						// movie slider
						Slider sl_MovieControl win=SpHrViewMovieWindow, value=(V_value/FrameLength)
						hookResult = 1
						break
					endif
					if(key !=129 || key !=65 || key !=257 ||key !=513)
						hookResult = 0
						break
					endif
					// no break here
					
				case 30:		// up arrow
				case 256:		// up arrow, with shift
				case 31:		// down arrow
				case 512:		// down arrow, with shift
					if(key ==129 || key ==65 || key ==257 || key ==513)
						traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
						wStr =  StringFromList(0,traceListStr)
						wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
						GetAxis /W=SpHrViewPeaksWindow/Q bottom
						xmin=v_min; xmax=v_max
						peaks_x_min = DimOffset(w,0)
						peaks_x_max = DimSize(w,0)*DimDelta(w,0)+DimOffset(w,0)
						if(key==129) // command + left arrow -> shift displayed sounds to the left
							new_min = xmin-((xmax-xmin)*0.125)
							new_max = xmax-((xmax-xmin)*0.125)
							if(new_min > peaks_x_min)
								SetAxis/W=SpHrViewPeaksWindow bottom new_min,new_max
							else
								SetAxis/W=SpHrViewPeaksWindow bottom peaks_x_min, peaks_x_min+(xmax-xmin)
							endif
							hookResult = 1
						elseif(key==65) // command + right arrow -> shift displayed sounds to the right
							new_min = xmin+((xmax-xmin)*0.125)
							new_max = xmax+((xmax-xmin)*0.125)
							if(new_max < peaks_x_max)
								SetAxis/W=SpHrViewPeaksWindow bottom new_min,new_max
							else
								SetAxis/W=SpHrViewPeaksWindow bottom peaks_x_max-(xmax-xmin), peaks_x_max
							endif
							hookResult = 1
						elseif(key==257) // command + up arrow -> expand (zoom in)
							new_min = xmin + (xmax-xmin)/4
							new_max = xmax - (xmax-xmin)/4
							SetAxis/W=SpHrViewPeaksWindow bottom new_min,new_max
						elseif(key==513) // command + dow arrow -> contract (zoom out)
							new_min = xmin - (xmax-xmin)/4
							new_max = xmax + (xmax-xmin)/4
							SetAxis/W=SpHrViewPeaksWindow bottom new_min,new_max
						endif
						hookResult = 1
					else
						hookResult = 1
					endif
					break
			endswitch
			break
			
		//////////////////
		// Mouse event
		case 3:		// mousedown
			//if(s.eventMod==5 || s.eventMod==9) // option or command keys
			if(s.eventMod==9 && stringmatch(s.winName, "SpHrViewPeaksWindow")) // command key
				// clear draw rectangle
				SpHrView_DrawSelection(0, 0, 0)
				xval = AxisValFromPixel("SpHrViewPeaksWindow", "bottom", s.mouseLoc.h )
				ACursorStr = CsrWave(A, "SpHrViewPeaksWindow")
				// use *_snip wave if displayed
				traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
				for(i=0;i<itemsinlist(traceListStr);++i)
					wStr =  StringFromList(i,traceListStr)
					if(strsearch(wStr, "_snip", 0)>1)
						ACursorStr = wStr
						break
					endif 
				endfor
				Cursor/S=2/L=0/H=2/C=(0,65535/2,0,32767)/W=SpHrViewPeaksWindow A, $ACursorStr, xval
				DoUpdate/W=SpHrViewPeaksWindow
				
				// add code to handle cursor updates if specific window is found
				DoWindow Spectra_Window
				if(v_flag)
					SpHrView_DisplaySpectra()
				endif
				
				SpHrView_AtTime_CursorUpdate()

				SetActiveSubwindow SpHrViewPeaksWindow
				hookResult = 1
			endif
			break
			
		case 7:		// cursormoved			
			if(stringmatch(s.winName, "SpHrViewPeaksWindow") && stringmatch(s.cursorName,"A"))
			
				SpHrView_MovieExtract()					// if Movie
				
				DoWindow Spectra_Window
				if(v_flag)
					SpHrView_DisplaySpectra()
				endif
					
				SpHrView_AtTime_CursorUpdate()
				
				SetActiveSubwindow SpHrViewPeaksWindow // <<----- ??/
				hookResult = 1
			endif
			break

	endswitch
	
	// scroll, touch pad, scroll
	if(s.wheelDx != 0)
		traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
		wStr =  StringFromList(0,traceListStr)
		wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
		GetAxis /W=SpHrViewPeaksWindow/Q bottom
		xmin=v_min; xmax=v_max
		variable endGapDenominator = 500
		peaks_x_min = DimOffset(w,0)
		peaks_x_max = DimSize(w,0)*DimDelta(w,0)+DimOffset(w,0)
		if(s.wheelDx < 0)
			new_min = xmin-((xmax-xmin)*s.wheelDx/endGapDenominator)
			new_max = xmax-((xmax-xmin)*s.wheelDx/endGapDenominator)
			if(new_min < peaks_x_min || new_max > peaks_x_max)
				if(new_min < peaks_x_min)
					SetAxis/W=SpHrViewPeaksWindow bottom peaks_x_min, peaks_x_min+(xmax-xmin)
				endif
				if(new_max > peaks_x_max)
					SetAxis/W=SpHrViewPeaksWindow bottom peaks_x_max-(xmax-xmin), peaks_x_max
				endif
			else
				SetAxis/W=SpHrViewPeaksWindow bottom new_min,new_max
			endif
		else
			new_min = xmin+((xmax-xmin)*-s.wheelDx/endGapDenominator)
			new_max = xmax+((xmax-xmin)*-s.wheelDx/endGapDenominator)
			if(new_min < peaks_x_min || new_max > peaks_x_max)
				if(new_min < peaks_x_min)
					SetAxis/W=SpHrViewPeaksWindow bottom peaks_x_min, peaks_x_min+(xmax-xmin)
				endif
				if(new_max > peaks_x_max)
					SetAxis/W=SpHrViewPeaksWindow bottom peaks_x_max-(xmax-xmin), peaks_x_max
				endif
			else
				SetAxis/W=SpHrViewPeaksWindow bottom new_min,new_max
			endif
		endif
		hookResult = 1
	endif
	return hookResult
End


// =================================================================================================
// SpHrView_PlaySoundTask(), Background task to move cursor (or line) while playing
// =================================================================================================
Function SpHrView_PlaySoundTask(s)
	STRUCT WMBackgroundStruct &s
	//return 0
	variable useCursor = 1 // set same as in SpHrView_PlaySoundTaskStop
	// using Cursor becuse SetDrawLayer calls SpHrViewTimeAxisHook
	NVAR/Z PlaySoundTicks=$SpHrView_df_globals+"PlaySoundTicks"
	NVAR/Z PlayStartTicks=$SpHrView_df_globals+"PlayStartTicks"
   NVAR/Z PlayStartTime=$SpHrView_df_globals+"PlayStartTime"
   if(NVAR_Exists(PlaySoundTicks) && NVAR_Exists(PlayStartTicks))
	   if(s.curRunTicks > PlaySoundTicks + PlayStartTicks )
	   	Dowindow SpHrViewPeaksWindow
	   	if(v_flag)
		   	if(useCursor)
		   		Cursor/K/W=SpHrViewPeaksWindow B
		   	else
		   		SetDrawLayer/W=SpHrViewPeaksWindow/K ProgFront
		   	endif
		   	killvariables/Z PlaySoundTicks, PlayStartTicks, PlayStartTime
		   	//SpHrView_MovieClipClose()
		   endif
			//Dowindow RecordingMessageWindow
			//if(v_flag)
			//
			//endif
	   	return 1	// Stop background task
	   else
	   	variable xval = PlayStartTime + ((ticks-PlayStartTicks)/60)//.15)
	   	Dowindow SpHrViewPeaksWindow
	   	if(v_flag)
		   	if(useCursor)
					string traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
					string wStr = StringFromList(0,traceListStr)
					Cursor/S=2/L=0/H=2/C=(0,0,0, 32767 )/W=SpHrViewPeaksWindow B, $wStr, xval
				else
					SetDrawLayer/W=SpHrViewPeaksWindow ProgFront
					SetDrawLayer/W=SpHrViewPeaksWindow/K ProgFront
					SetDrawEnv/W=SpHrViewPeaksWindow xcoord= bottom
					DrawLine/W=SpHrViewPeaksWindow xval,-0.05,xval,1.0
				endif
				//SpHrView_MovieClipExtract(xval)
			endif
			//Dowindow RecordingMessageWindow
			//if(v_flag)
			////Print xval
			//endif
		endif
	endif
	return 0	// Continue background task
End
// =================================================================================================
// SpHrView_PlaySoundTaskStart(), Start background task to move cursor (or line) while playing
// =================================================================================================
Function SpHrView_PlaySoundTaskStart()
	CtrlNamedBackground PlaySoundBackgroundTask, period=1, proc=SpHrView_PlaySoundTask
	CtrlNamedBackground PlaySoundBackgroundTask, start
	//SpHrView_MovieClipOpen()
End
// =================================================================================================
// SpHrView_PlaySoundTaskStop(), Stop background task to move cursor (or line) while playing
// =================================================================================================
Function SpHrView_PlaySoundTaskStop()
	CtrlNamedBackground PlaySoundBackgroundTask, stop
	CtrlNamedBackground PlaySoundBackgroundTask, kill
	variable useCursor = 1 // set same as in SpHrView_PlaySoundTask
	// using Cursor becuse SetDrawLayer calls SpHrViewTimeAxisHook
	Dowindow SpHrViewPeaksWindow
	if(v_flag)
		if(useCursor)
			Cursor/K/W=SpHrViewPeaksWindow B
		else
			SetDrawLayer/W=SpHrViewPeaksWindow/K ProgFront
		endif
		//SpHrView_MovieClipClose()
	endif
	NVAR/Z PlaySoundTicks=$SpHrView_df_globals+"PlaySoundTicks"
	NVAR/Z PlayStartTicks=$SpHrView_df_globals+"PlayStartTicks"
   NVAR/Z PlayStartTime=$SpHrView_df_globals+"PlayStartTime"
	killvariables/Z PlaySoundTicks, PlayStartTicks, PlayStartTime
End


// =================================================================================================
// SpHrView_PlaySound(), Load sound waveforms and play them
// =================================================================================================
function SpHrView_PlaySound()
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, usingAxis
	[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
	wave loadedWaveforms=$SpHrView_df_files+"waveforms"
	
	if(!waveexists(loadedWaveforms))
		DoAlert 0, "SpHrView_PlaySound: Waveforms are missing"
		return 0
	endif
	
	// copy to avoid ramping and normalizing sound being analyzed...
	Duplicate/O/FREE loadedWaveforms, waveforms
	
	// ramp
	variable Ramp_ms, desiredRamp = 50 // ms
	variable soundLength_ms = (dimsize(waveforms,0)*dimdelta(waveforms,0)) * 1000 // sec to ms
	Ramp_ms = min(desiredRamp, soundLength_ms/4) // no longer than a 1/4th the length of the sound
	SpHr_RampOnOff Ramp_ms, Ramp_ms, waveforms // ramp before playing (10 ms on & off)
	NVAR normalizeSoundPlay = $SpHrView_df_globals+"normalizeSoundPlay"
	if(normalizeSoundPlay)
		wavestats/Q/M=1 waveforms
		variable maxAmp = max(v_max, -v_min)
		waveforms /= maxAmp
	endif
	
	variable keys = GetKeyState(0)
	//Print keys
	if(keys == 256 || keys == 512 || keys==260 || keys==516) // up or down arrow keys = reverse L & R channels
		Duplicate/O waveforms, reversedChannels
		waveforms[][1] = reversedChannels[p][0]
		waveforms[][0] = reversedChannels[p][1]
		killwaves/Z reversedChannels
	endif
	if(keys == 4 || keys == 260 || keys==516) 					 // shift key = reverse phase of R channel
		waveforms[][1] = -waveforms[p][1] 
	endif
	if(!usingAxis)
		SpHrView_DrawSelection(1, startSec, endSec)
	endif
	
	PlaySound/A=2 waveforms
	
	if(endSec-startSec > 0.5)
		Variable/G $SpHrView_df_globals+"PlayStartTime" = startSec
		Variable/G $SpHrView_df_globals+"PlayStartTicks" = ticks - 5 // - 5 since the cursor tends to lag a few ms...
		Variable/G $SpHrView_df_globals+"PlaySoundTicks" = ( dimsize(waveforms,0)*dimdelta(waveforms,0) )*60//.15
		SpHrView_PlaySoundTaskStart()
	endif

	killwaves/Z waveforms

	return 0
end

// =================================================================================================
// SpHrView_SaveWave(), Load sound waveforms and save them
// =================================================================================================
function SpHrView_SaveWave()
	String dfSav = GetDataFolder(1)
	SetDataFolder $SpHrView_df_files
	variable minSec, maxSec, startSec, endSec, usingMarquee=0,  usingTimeFrame=0, usingAxis=0
	// get selection if SpHrViewPeaksWindow is open
	DoWindow SpHrViewPeaksWindow
	if(v_flag)
		string traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
		string wStr =  StringFromList(0,traceListStr)
		wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
		string pathAndNameStr= SpHrView_FilePath_Get(w, 0) //note(w) 
		// get min and max times
		minSec = DimOffset(w, 0)
		maxSec = DimSize(w, 0)*DimDelta(w, 0) + DimOffset(w, 0)
		// get start and stop times
		GetMarquee/W=SpHrViewPeaksWindow/Z bottom
		//GetMarquee/Z bottom // allow marquee from another window...
		if(V_flag)
			startSec = v_left
			endSec = v_right
			usingMarquee = 1
		else
			DrawAction/W=SpHrViewPeaksWindow/L=UserBack extractOutline
			wave/Z W_PolyX=W_PolyX, W_PolyY=W_PolyY
			killwaves W_PolyY
			if(waveexists(W_PolyX) && dimsize(W_PolyX,0)==5) // object exists and is a square/rectangle
				startSec = wavemin(W_PolyX)
				endSec = wavemax(W_PolyX)
				killwaves W_PolyX
				usingTimeFrame = 1
			else
				GetAxis /W=SpHrViewPeaksWindow/Q bottom
				//GetAxis/Q bottom // allow axis from another window...
				startSec = v_min
				endSec = v_max
				usingAxis = 1
			endif
		endif
	else
		SetDatafolder $dfSav
		return 0
	endif

	// Save
	// make sure startSec and endSec are valid, since Marquees from other windows are allowed.
	startSec = max(startSec, minSec)
	endSec = min(endSec, maxSec)
	
	string fileNameStr = wStr // is traceNameStr
	fileNameStr += "_copy"
	Variable bitDepth = 2
	Variable fileFormat = 2
	Variable onoffRamps_ms = 20 // ms
	Prompt fileNameStr,"File name"
	Prompt bitDepth,"Bit Depth",popup,"16-bits;24-bits;32-bits;32-bit floating point;"
	Prompt fileFormat,"File Format",popup,"AIFF;WAV"
	Prompt startSec,"from"
	Prompt endSec,"to"
	Prompt onoffRamps_ms,"on/off ramps (ms)"
	if(usingMarquee || usingTimeFrame)
		DoPrompt "Save displayed sound to file", fileNameStr, bitDepth, fileFormat, startSec, endSec, onoffRamps_ms
	else
		DoPrompt "Save displayed sound to file", fileNameStr, bitDepth, fileFormat //, startSec, endSec, onoffRamps_ms
	endif
	if (V_Flag)
		return 0									// user canceled
	endif
	
	// load waveforms
	SpHrView_LoadSoundFromFileWave(pathAndNameStr, SpHrView_df_files, "waveforms", startSec, endSec)
	wave waveforms=$SpHrView_df_files+"waveforms"
	NVAR fs= $SpHrView_df_globals+"fs"
	SetScale/P x startSec,1/fs,"s", waveforms
	
	// ramp
	if(onoffRamps_ms > 0)
		SpHr_RampOnOff onoffRamps_ms, onoffRamps_ms, waveforms
	endif
	
	// limit to avoid clipping
	wavestats/Q/M=1 waveforms
	if(v_max > 1.0)
		waveforms *= 1/v_max
		Printf "Saved waveforms scaled (*%g) to avoid clipping!", 1/v_max
	endif
	if(v_min < -1.0)
		waveforms *= 1/-v_min
		Printf "Saved waveforms scaled (*%g) to avoid clipping!", 1/-v_min
	endif
	String fileFormatStr = "WAVE"	// fileFormat = 2
	if(fileFormat==1)
		fileFormatStr = "AIFC"		// fileFormat = 1
	endif
	variable bits = 16
	if(bitDepth==1) 					// 16-bits integer
		bits = 16
		//Duplicate/O/FREE waveforms, temp // deleting these anyway
		//wave waveforms = temp
		waveforms *= ((2^16)/2)-1
		Redimension/W waveforms
		
	elseif(bitDepth==2) 					// 24-bits integer
		bits = 24
		//Duplicate/O/FREE waveforms, temp // deleting these anyway
		//wave waveforms = temp
		waveforms *= ((2^24)/2)-1
		Redimension/I waveforms
		
	elseif(bitDepth==3) 				// 32-bits integer
		bits = 32
		//Duplicate/O/FREE waveforms, temp
		//wave waveforms = temp
		waveforms *= ((2^32)/2)-1
		Redimension/I waveforms
		
	elseif(bitDepth==4) 				// 32-bit floating point
		// 32-bit floating point
	endif
	
	
	PathInfo SoundsPath
	if(V_flag)
		//SoundSaveWave/P=SoundsPath/I/Q fileFormatStr, waveforms, fileNameStr
		//SoundSaveWave/P=SoundsPath/I/Q/BITS=(bits) fileFormatStr, waveforms, fileNameStr	/// /bits flag added for Igor 9.0
		if(IgorVersion() >= 9.00 && (bits==16 || bits==24  || bits==32) )
			Execute "SoundSaveWave/P=SoundsPath/I/Q/BITS=("+num2str(bits)+") \""+fileFormatStr+"\", "+SpHrView_df_files+"waveforms"+", \""+fileNameStr+"\""
		else
			Execute "SoundSaveWave/P=SoundsPath/I/Q \""+fileFormatStr+"\", "+SpHrView_df_files+"waveforms"+", \""+fileNameStr+"\""
		endif
	else
		//SoundSaveWave/I/Q fileFormatStr, waveforms, fileNameStr
		//SoundSaveWave/I/Q/bits=(bits) fileFormatStr, waveforms, fileNameStr	/// /bits flag added for Igor 9.0
		if(IgorVersion() >= 9.00 && (bits==16 || bits==24  || bits==32) )
			Execute "SoundSaveWave/I/Q/BITS=("+num2str(bits)+") \""+fileFormatStr+"\", "+SpHrView_df_files+"waveforms"+", \""+fileNameStr+"\""
		else
			Execute "SoundSaveWave/I/Q \""+fileFormatStr+"\", "+SpHrView_df_files+"waveforms"+", \""+fileNameStr+"\""
		endif
	endif
	killwaves/Z waveforms

	killStrings/Z S_info
	SetDatafolder $dfSav
	return 0
end

// =================================================================================================
// SpHrView_WaveformStatistics(), Get waveform statistics
// =================================================================================================
function SpHrView_WaveformStatistics(variable quiet)
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, usingAxis
	[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
	wave/Z waveforms=$SpHrView_df_files+"waveforms"
	if(!waveexists(waveforms))
		DoAlert 0, "SpHrView_WaveformStatistics: Waveforms are missing"
		return 0
	endif
	
//	display/K=1 waveforms[][0]
//	appendtograph waveforms[][1]
	
	wavestats/PCST/Q waveforms
	wave M_WaveStats=M_WaveStats
	String str = ""
	
	if(endSec > 1)
		SPrintf str, "%stime %g to %g (sec.)\r", str, startSec,  endSec
	else
		SPrintf str, "%time= s%g to %g (ms)\r", str, startSec/1000,  endSec/1000
	endif
	
	variable Lmax, Rmax, LRmax
	
	if(dimsize(M_WaveStats,1) > 0)
		Lmax = max(M_WaveStats[10][0],M_WaveStats[12][0])
		Rmax = max(M_WaveStats[10][1],M_WaveStats[12][1])
		LRmax = max(Lmax,Rmax)
		SPrintf str, "%sRMS L Pa= %.5f, dB= %.2f, |max|= %.5f\r", str, M_WaveStats[5][0], 20*log(M_WaveStats[5][0]/0.00002), Lmax
		SPrintf str, "%sRMS R Pa= %.5f, dB= %.2f, |max|= %.5f\r", str, M_WaveStats[5][1], 20*log(M_WaveStats[5][1]/0.00002), Rmax
		SPrintf str, "%sRMS L-R Pa= %.5f, dB= %.2f, |max|= %.5f\r", str, (M_WaveStats[5][0]-M_WaveStats[5][1]), 20*log(M_WaveStats[5][0]/0.00002)-20*log(M_WaveStats[5][1]/0.00002), Lmax-Rmax
	else
		Lmax = max(M_WaveStats[10],M_WaveStats[12])
		SPrintf str, "%sRMS \tPa= %.5f, dB= %.2f, |max|= %.5f\r", str, M_WaveStats[5], 20*log(M_WaveStats[5]/0.00002), Lmax
	endif
//	SPrintf str, "%sRMS Amplitude = %.6f Pa, %.2f dB (re:20µPa)\r", str, v_rms, 20*log(v_rms/0.00002)
//	SPrintf str, "%sWaveform maximum= %.5f\rWaveform minimum= %.5f\r", str, v_min, v_max
	
	SPrintf str, "%sfs= %.1f (Hz), dB (re:20µPa)\r", str, 1/dimdelta(waveforms,0)
	DoAlert/T="Waveform statistics" 0, str
	//killwaves M_WaveStats
	killwaves/Z waveforms
	if(!quiet)
		Print str
	endif
End


// =================================================================================================
// SpHrView_DisplayCarrierWaves(), Load sound waveforms and display them
// =================================================================================================
function SpHrView_DisplayCarrierWaves(getEnvelopes, overlapWaves)
	variable getEnvelopes, overlapWaves
	
	// if getEnvelopes < 0 and Waveforms_Window exists, try to determine what is displayed
	DoWindow Waveforms_Window
	if(v_flag)
		if(getEnvelopes < 0)
			GetWindow/Z Waveforms_Window note 
			if(stringmatch("WaveformEnvelopes", StringByKey("DISPLAYED", S_Value)))
				getEnvelopes = 1
			elseif(stringmatch("Waveforms", StringByKey("DISPLAYED", S_Value)))
				getEnvelopes = 0
			else
				return 0
			endif
		endif
		if(overlapWaves < 0)
			ControlInfo/W=Waveforms_Window ck_overlapEnvelopes // ck_overlapWaveforms  // ck_overlapEnvelopes
			if(V_flag)
				overlapWaves = V_Value
			else
				ControlInfo/W=Waveforms_Window ck_overlapWaveforms // ck_overlapWaveforms  // ck_overlapEnvelopes
				if(V_flag)
					overlapWaves = V_Value
				endif
			endif
		endif
	endif
	
	// Load waveforms from SpHrViewPeaksWindow
	variable startSec, endSec, usingAxis
	[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
	wave waveforms=$SpHrView_df_files+"waveforms"
	if(!waveexists(waveforms))
		DoAlert 0, "SpHrView_DisplayCarrierWaves: Waveforms are missing"
		return 0
	endif
	
	String titleStr = "Carrier Waveforms"
	if(getEnvelopes)
		SpHr_HilbertEnvelope/B/O/Pa waveforms
		titleStr = "Carrier Waveform Envelopes"
	endif
	
	variable numCols = dimsize(waveforms,1)
	NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
	NVAR screenVert= $SpHrView_df_globals+"screenVert"
	NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
	NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
	variable wdth=screenHoriz*ViewWidthProportion
	variable hgt=screenVert*ViewHeightProportion*1.5
	Variable  winLeft=(screenHoriz/6), winRight, winTop, winBottom=270
	[winLeft, winRight, winTop, winBottom] = getWindowSize("SpHrViewPeaksWindow")
	Variable marginLeft, marginRight, marginTop, marginBottom
	[marginLeft, marginRight, marginTop, marginBottom] = getWindowMargins("SpHrViewPeaksWindow")	
	DoWindow Waveforms_Window
	if(v_flag)
		RemoveTraces("Waveforms_Window")
	else
		Display /K=1/W=(winLeft, winBottom+25, winLeft+wdth, winBottom+25+hgt) as titleStr
		DoWindow/C Waveforms_Window
		ModifyGraph/W=Waveforms_Window margin(left)=marginLeft,margin(bottom)=35,margin(top)=9,margin(right)=marginRight
	endif
	
	AppendToGraph/W=Waveforms_Window/L=left_L waveforms[*][0] 
	if(overlapWaves)
		AppendToGraph/W=Waveforms_Window/L=left_L waveforms[*][1]
	elseif(numCols > 1)
		AppendToGraph/W=Waveforms_Window/L=left_R waveforms[*][1]
	endif
	ModifyGraph/W=Waveforms_Window lSize=0.5, rgb(waveforms)=(0,0,65535)
	ModifyGraph/W=Waveforms_Window tick=2,font="Arial",fSize=10
	ModifyGraph/W=Waveforms_Window standoff=0,axThick=0.5
	ModifyGraph/W=Waveforms_Window ZisZ=1, zapTZ=1, zapLZ=1
	ModifyGraph/W=Waveforms_Window btLen=4,btThick=0.5,stLen=2,stThick=0.5
	ModifyGraph/W=Waveforms_Window freePos(left_L)=0
	if(numCols > 1 && !overlapWaves)
		ModifyGraph/W=Waveforms_Window axisEnab(left_L)={0.51,1}
		ModifyGraph/W=Waveforms_Window axisEnab(left_R)={0,0.49}
		ModifyGraph/W=Waveforms_Window freePos(left_R)=0
		ModifyGraph/W=Waveforms_Window zero(left_R)=1, zeroThick(left_R)=0.5
		ModifyGraph/W=Waveforms_Window axisClip(left_R)=1
	endif
	ModifyGraph/W=Waveforms_Window zero(left_L)=1, zeroThick(left_L)=0.5
	ModifyGraph/W=Waveforms_Window axisClip(left_L)=1
	if(overlapWaves)
		Label/W=Waveforms_Window left_L "amplitude (\\U)"
	else
		Label/W=Waveforms_Window left_L "amplitude left (\\U)"
		if(numCols > 1)
			Label/W=Waveforms_Window left_R "amplitude right (\\U)"
			ModifyGraph/W=Waveforms_Window lblPos(left_R)=38
		endif
	endif
	ModifyGraph/W=Waveforms_Window lblPos(left_L)=38
	Label/W=Waveforms_Window bottom "time (\\U)"
	// scale
	wavestats/Q/M=1 waveforms
	variable abs_max = max(v_max, -v_min)
	if(getEnvelopes)
		SetAxis/W=Waveforms_Window left_L 0, abs_max
		if(numCols > 1 && !overlapWaves)
			SetAxis/W=Waveforms_Window left_R 0, abs_max
		endif
	else
		SetAxis/W=Waveforms_Window left_L -abs_max, abs_max
		if(numCols > 1 && !overlapWaves)
			SetAxis/W=Waveforms_Window left_R -abs_max, abs_max
		endif
	endif
	ControlBar/W=Waveforms_Window/L 20
	ModifyGraph/W=Waveforms_Window cbRGB=(65534,65534,65534)
	variable maxPa = wavemax(waveforms)
	// left axis slider
	Slider sl_leftAxisRangeLog win=Waveforms_Window, pos={0,30},size={5,hgt-70},side=0,proc=AxisSliderProc	
	Slider sl_leftAxisRangeLog win=Waveforms_Window, limits={log(2), log(0.001),0.001}, value=log(maxPa)
	if(getEnvelopes)
		CheckBox ck_overlapEnvelopes win=Waveforms_Window, fstyle=1,title="⩙",fSize=20,pos={8,1},side=1,Help={"overlap envelopes"},value=overlapWaves,proc=SpHrView_CheckProc
	else
		CheckBox ck_overlapWaveforms win=Waveforms_Window, fstyle=1,title="⩙",fSize=20,pos={8,1},side=1,Help={"overlap waveforms"},value=overlapWaves,proc=SpHrView_CheckProc
	endif
	// Legend
	String legendStr
	if(numCols > 1)
		legendStr = "\\F'Arial'\\Z10\\s(waveforms)left    \\s(waveforms#1)right"
	else
		legendStr = "\\F'Arial'\\Z10\\s(waveforms)mono"
	endif
	Legend/W=Waveforms_Window/C/N=text0/J/E=2/X=0.00/Y=1.00/F=0/B=1/A=MT legendStr
	// window hook
	DoUpdate/W=Waveforms_Window // for Igor 9
	SetWindow Waveforms_Window,hook(WaveformsHook)=SpHrView_PeaksWindowHook
	if(getEnvelopes)
		SetWindow Waveforms_Window note="DISPLAYED:WaveformEnvelopes;"
	else
		SetWindow Waveforms_Window note="DISPLAYED:Waveforms;"
	endif
	SetWindow Waveforms_Window note+="PACKAGE:SpHrView"
End


// =================================================================================================
// SpHrView_PeaksWindow(wStr), display sound peaks
// wStr: name of wave to be displayed, it is also the name of the generated window
// The peaks wave ($wStr) is assumed to be in the "root:SpHrView:Files:" datafolder
// =================================================================================================
Function SpHrView_PeaksWindow(wStr)
	string wStr
	wave/Z w= $SpHrView_df_files+wStr
	variable movieisopen= 0
	DoWindow SpHrViewMovieWindow
	if(V_flag)
		movieisopen= 1
	endif
	DoWindow SpHrViewPeaksWindow
	if(V_flag)
		killWindow/Z SpHrViewPeaksWindow
	endif
	if(!waveexists(w))
		DoAlert/T="Error" 0, "file missing in SpHrView(): "+SpHrView_df_files+wStr
		return 0
	endif
	NVAR dBscale= $SpHrView_df_globals+"dBscale"
	NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
	NVAR screenVert= $SpHrView_df_globals+"screenVert"
	NVAR ViewWidthProportion= $SpHrView_df_globals+"ViewWidthProportion"
	NVAR ViewHeightProportion= $SpHrView_df_globals+"ViewHeightProportion"
	variable wdth=screenHoriz*ViewWidthProportion
	variable hgt=screenVert*ViewHeightProportion
	variable leftRectPnt = (screenHoriz-wdth)/2
	variable topRectPnt = 45
	Display /K=1/W=(leftRectPnt, topRectPnt, leftRectPnt+wdth, topRectPnt+hgt) as wStr
	DoWindow/C SpHrViewPeaksWindow
	AppendToGraph w[*][0]
	if(dimsize(w,1) > 1)
		AppendToGraph w[*][1]
	endif
	ModifyGraph margin(left)=40, margin(top)=5, margin(right)=36, margin(bottom)=25
	ModifyGraph lsize=0
	ModifyGraph rgb($wStr)=(0,0,65535)
	ModifyGraph usePlusRGB=1, mode=7, hbFill=2
	ModifyGraph plusRGB($wStr)=(0,0,65535,65535*0.66)
	if(dimsize(w,1) > 1)
		ModifyGraph plusRGB($wStr+"#1")=(65535,0,0,65535*0.6)
	endif
	ModifyGraph tick=2
	ModifyGraph nticks(left)=2,nticks(bottom)=10
	ModifyGraph font="Arial"
	ModifyGraph fSize=10
	ModifyGraph highTrip(bottom)=1000
	ModifyGraph lblMargin(left)=5, lblMargin(bottom)=0
	ModifyGraph standoff=0
	ModifyGraph axThick(left)=0.5, axThick(bottom)=0.5, mirror(bottom)=1
	ModifyGraph ZisZ=1
	ModifyGraph zapTZ=1
	ModifyGraph zapLZ=1
	ModifyGraph btLen=4
	ModifyGraph btThick=0.5
	ModifyGraph stLen=2
	ModifyGraph stThick=0.5
	ModifyGraph lblPos(bottom)=23
	ModifyGraph lblPos(left)=32
	if(dBscale)
		SetAxis left 20,*
		Label left "peak dB (re: 20 µPa)\\u#2"
	else
		SetAxis left 0,*
		Label left "peak amplitude (\\U)"
	endif
	Label bottom "time (\\U)"
	ModifyGraph lowTrip(bottom)=1
	
	// general audio file View controls
	ControlBar/W=SpHrViewPeaksWindow/T 37
	if(1)
		//	ModifyGraph cbRGB =(65535,65534,65534)
		NewPanel/FG=(FL,FT,GR,GT)/HOST=# 
		RenameWindow #,PTop // SpHrViewPeaksWindow#PTop
	
		// y-axis dB or linear (Pa/V) scale
		CheckBox ck_SpHrView_dBScale win=SpHrViewPeaksWindow#PTop, pos={10,10}, title="dB",variable=$SpHrView_df_globals+"dBscale"
		CheckBox ck_SpHrView_dBScale win=SpHrViewPeaksWindow#PTop, help={"Plot using dB (re: 20 µPa) or linear scale (Pa)"}, proc=SpHrView_CheckProc
		// Open new file
		Button b_SpHrView_NewFile win=SpHrViewPeaksWindow#PTop, pos={50, 8.5}, title="", picture=ProcGlobal#SpHrView_pict_Filebox, size={(20/15)*20,20}, proc=SpHrView_ButtonProc
		Button b_SpHrView_NewFile win=SpHrViewPeaksWindow#PTop, help={"Select new sound file. Option-click or Command-click for recent files pop-up menu."}
		
		// load previously opened/generated wave
		Button b_SpHrView_LoadWave win=SpHrViewPeaksWindow#PTop, pos={82, 8}, title="", picture=ProcGlobal#SpHrView_pict_Wave, size={(20/15)*19,19}, proc=SpHrView_WavesButtonProc
		Button b_SpHrView_LoadWave win=SpHrViewPeaksWindow#PTop, help={"Wave loading & generation"} // ProcGlobal#SpHrView_pict_Wave (20/15)*20, ProcGlobal#pict_HeadHeadphones
		//GroupBox group_FileWave win=SpHrViewPeaksWindow#PTop, pos={46,6.5},size={75,23}
			
		// View (waveforms, spectra, filter bank) 
		Button b_SpHrView_View win=SpHrViewPeaksWindow#PTop, pos={117, 8}, title="", picture=ProcGlobal#SpHrView_pict_WaveView, size={(25/15)*20,20}, proc=SpHrView_ButtonProc
		Button b_SpHrView_View win=SpHrViewPeaksWindow#PTop, help={"View Waveforms, spectra, filter banks, envelopes. Uses marquee selection or current view."} // or cross covariance products
		
		// Distributions
		Button b_SpHrView_Distributions win=SpHrViewPeaksWindow#PTop, pos={152, 8}, title="", picture=ProcGlobal#SpHrView_pict_Distribution, size={(25/15)*20,20}, proc=SpHrView_CueHistogram_ButtonProc
		Button b_SpHrView_Distributions win=SpHrViewPeaksWindow#PTop, help={"View binaural cue distributions. Option-click for pop-up menu. Command-click to update."}
		
		// Cues
		Button b_SpHrView_Cues win=SpHrViewPeaksWindow#PTop, pos={188,8}, title="", picture=ProcGlobal#SpHrView_pict_Cues, size={(25/15)*20,20}, proc=SpHrView_Cues_ButtonProc
		Button b_SpHrView_Cues win=SpHrViewPeaksWindow#PTop, help={"View binaural cues. Option-click for pop-up menu. Command-click to update."}
		
		// ITD at time
		Button b_SpHrView_AtTime win=SpHrViewPeaksWindow#PTop, pos={223,8}, title="", picture=ProcGlobal#SpHrView_pict_AtTime, size={(25/15)*20,20}, proc=SpHrView_AtTime_ButtonProc
		Button b_SpHrView_AtTime win=SpHrViewPeaksWindow#PTop, help={""}
		
		// ITD at time
		Button b_SpHrView_OverTime win=SpHrViewPeaksWindow#PTop, pos={257,7.5}, title="", picture=ProcGlobal#SpHrView_pict_OverTime, size={(25/15)*20,22}, proc=SpHrView_OverTime_ButtonProc
		Button b_SpHrView_OverTime win=SpHrViewPeaksWindow#PTop, help={""}
		

		
//		GroupBox group_GenAnalyses win=SpHrViewPeaksWindow#PTop, pos={121,6.5},size={83,23}
//		
//		//AtTime
//		Button b_SpHrView_AtTime win=SpHrViewPeaksWindow#PTop, pos={220, 8}, title="", picture=ProcGlobal#pict_AtTime
//		Button b_SpHrView_AtTime win=SpHrViewPeaksWindow#PTop, size={(25/15)*20, 20}, help={"Update or select representations at end of time selection"}
//		Button b_SpHrView_AtTime win=SpHrViewPeaksWindow#PTop, proc=ViewAtCursor_ButtonProc
//		
//		//OverTime
//		Button b_SpHrView_OverTime win=SpHrViewPeaksWindow#PTop, pos={260, 8}, title="", picture=ProcGlobal#pict_OverTime
//		Button b_SpHrView_OverTime win=SpHrViewPeaksWindow#PTop, size={(25/15)*20, 20}, help={"Update or select representations over time"}
//		Button b_SpHrView_OverTime win=SpHrViewPeaksWindow#PTop, proc=ViewOverTime_ButtonProc
//
//		GroupBox group_ITDModels win=SpHrViewPeaksWindow#PTop, pos={210,6.5},size={95,23}
		
		// Play sound
		Button b_SpHrView_Play win=SpHrViewPeaksWindow#PTop, pos={313,7.0}, title="", picture=ProcGlobal#SpHrView_pict_circlePlay, size={20,20}, proc=SpHrView_ButtonProc
		Button b_SpHrView_Play win=SpHrViewPeaksWindow#PTop, help={"Play sounds within current view or within a marquee selection. Can also use the \"p\" key or space bar. Press and hold up or down arrow to reverse left & right channels."}
		// Stop playing sound
		Button b_SpHrView_StopPlay win=SpHrViewPeaksWindow#PTop, pos={340,7.0}, title="", picture=ProcGlobal#SpHrView_pict_circleStop, size={20,20}, proc=SpHrView_ButtonProc
		Button b_SpHrView_StopPlay win=SpHrViewPeaksWindow#PTop, help={"Stop playing sound. Can also use the \"s\" key"}
		GroupBox group_PlayStop win=SpHrViewPeaksWindow#PTop, pos={306,6.5},size={62,26}
		
		// bottom (time) axis controls
		Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, title="", picture=ProcGlobal#SpHrView_pict_GlassArrows, pos={375,8}, size={(20/15)*21, 21}
		Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, proc=SpHrView_ButtonProc
		Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, disable=1 // will be enabled
		Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, help={"View full x-axis range (auto scale). Same as command-a key combination."}
		Button b_SpHrViewXZoom win=SpHrViewPeaksWindow#PTop, title="", picture=ProcGlobal#SpHrView_pict_GlassZoom, pos={403,8}, size={20, 20}
		Button b_SpHrViewXZoom win=SpHrViewPeaksWindow#PTop, proc=SpHrView_ButtonProc
		Button b_SpHrViewXZoom win=SpHrViewPeaksWindow#PTop, help={"Time axis expand. Contract when control, option, or command key pressed"}
		GroupBox group_Zoom win=SpHrViewPeaksWindow#PTop, pos={370,6.5},size={58,26}
		
		// time controls
		ValDisplay vd_timeLine win=SpHrViewPeaksWindow#PTop, pos={432,10}, size={(wdth/2)-202,6},format=" ",value=_NUM:1,mode=3,barmisc={0,0}
		ValDisplay vd_timeLine win=SpHrViewPeaksWindow#PTop, limits={0,1,0}, highColor= (52428,52428,52428)
		ValDisplay vd_timeLine win=SpHrViewPeaksWindow#PTop, lowColor= (65535,65535,65535),frame=1, disable=2 // will be enabled
		
		Slider sl_bottomAxis win=SpHrViewPeaksWindow#PTop, pos={430,9},size={(wdth/2)-200,15},side=2,vert=0,proc=SpHrView_SliderProc
		Slider sl_bottomAxis win=SpHrViewPeaksWindow#PTop, limits={-1,1,0}, value=0, ticks=0, disable=0 // 2 // will be enabled
		Slider sl_bottomAxis win=SpHrViewPeaksWindow#PTop, help={"left/right axis shift. Can also use [command]+[arrow keys] to shift left, shift right, expend or contract"}
		
		// draw marquee
		Button b_drawMarquee win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+240, 8},size={(20/15)*20,20}, fSize=9, title=""
		Button b_drawMarquee win=SpHrViewPeaksWindow#PTop, picture=ProcGlobal#SpHrView_pict_DrawLayer, proc=SpHrView_ButtonProc
		Button b_drawMarquee win=SpHrViewPeaksWindow#PTop, help={"Make selection using Marquee or clear the selection. Can also specify a time frame if there is no Marquee or current selection, or if a modifier key is pressed."}
		NVAR useDrawSelection= $SpHrView_df_globals+"useDrawSelection"
		if(!useDrawSelection)
			Button b_drawMarquee win=SpHrViewPeaksWindow#PTop, disable=2
		endif
		
		// copy paste
		Button b_copyBottomAxis win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+277, 7.5},size={(25/15)*19,19}, fSize=9, title=""
		Button b_copyBottomAxis win=SpHrViewPeaksWindow#PTop, picture=ProcGlobal#SpHrView_pict_BottomCopy, proc=SpHrView_ButtonProc
		Button b_copyBottomAxis win=SpHrViewPeaksWindow#PTop, help={"Copy bottom axis range"}
		Button b_pasteBottomAxis win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+310, 7.5},size={(25/15)*19,19}, fSize=9, title=""
		Button b_pasteBottomAxis win=SpHrViewPeaksWindow#PTop, picture=ProcGlobal#SpHrView_pict_BottomPaste, proc=SpHrView_ButtonProc
		Button b_pasteBottomAxis win=SpHrViewPeaksWindow#PTop, help={"Paste bottom axis range"}
		GroupBox group_CopyPaste win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+270,6.5},size={80,26}
		
		
		CheckBox ck_ApplySPLScale win=SpHrViewPeaksWindow#PTop, title="SPL",side=0,pos={(wdth/2)+360,9},size={20,20},proc=SpHrView_CheckProc
		CheckBox ck_ApplySPLScale win=SpHrViewPeaksWindow#PTop, variable=$SpHrView_df_globals+"ApplySPLScale"
		CheckBox ck_ApplySPLScale win=SpHrViewPeaksWindow#PTop, help={"Apply SPL scale"}
		SetVariable sv_SPLscale win=SpHrViewPeaksWindow#PTop, title=" ",pos={(wdth/2)+395,10},size={35,20},side=0,value=$SpHrView_df_globals+"SPLRefenceScale",format="%.3f",limits={1e-07,100,0},fSize=10
		SetVariable sv_SPLscale win=SpHrViewPeaksWindow#PTop, help={"SPL scale, set using reference tone"},proc=SpHrView_SetVarProc
		GroupBox group_SPLScale win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+350,6.5},size={90,26}
		
		// movie
		Button b_openMovie win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+445, 7},size={21,21}, fSize=9, title=""
		Button b_openMovie win=SpHrViewPeaksWindow#PTop, picture=ProcGlobal#SpHrView_pict_Movie, proc=SpHrView_ButtonProc
		Button b_openMovie win=SpHrViewPeaksWindow#PTop, help={"Open movie window. Or show first, middle, and last frames using Marquee or selection."}
	
		
		NVAR numChannels= $SpHrView_df_globals+"numChannels"
		NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
		if(numChannels>2 || 0)
			String ChStr = "Chs\r"
			Switch(ChannelPair)
				case 1:
					ChStr+="1-2"
					break
				case 2:
					ChStr+="3-4"
					break
				case 3:
					ChStr+="5-6"
					break
				case 4:
					ChStr+="7-8"
					break
			endswitch	
			TitleBox title_Ch win=SpHrViewPeaksWindow#PTop, title=ChStr,pos={(wdth/2)+390,7},frame=0
			Button b_numChs, win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+410, 9},picture=ProcGlobal#SpHrView_pict_PopUpList, size={15.00,18.00},title=""
			Button b_numChs, win=SpHrViewPeaksWindow#PTop, proc=SpHrView_ButtonProc
			GroupBox group_Channels win=SpHrViewPeaksWindow#PTop, pos={(wdth/2)+385,6.5},size={45,23}
		endif
		
		//  filter
		// Add Filter checkbox defaults if SpHrView_Filter is loaded and the function SpHrView_FilterPeaksWindowControls exists 
		string infostr = FunctionInfo("SpHrView_FilterPeaksWindowControls")
		if (strlen(infostr) > 0)
			Execute/Q/Z "SpHrView_FilterPeaksWindowControls()"
		else
			CheckBox ck_waveformFilter win=SpHrViewPeaksWindow#PTop, pos={wdth-145,10}, side=1, title="filter2", disable=2
			CheckBox ck_waveformFilter win=SpHrViewPeaksWindow#PTop, help={"Disabled because procedure file SpHrView_Filter is missing"}
			CheckBox ck_waveformFilter win=SpHrViewPeaksWindow#PTop , proc=SpHrView_CheckProc
		endif
		
		// signal phase
		CheckBox ck_outOfPhase win=SpHrViewPeaksWindow#PTop, pos={wdth-100,10}, side=1, title="φ",variable=$SpHrView_df_globals+"loadOutOfPhase"
		CheckBox ck_outOfPhase win=SpHrViewPeaksWindow#PTop, help={"Invert phase of right channel"}
		CheckBox ck_outOfPhase win=SpHrViewPeaksWindow#PTop , proc=SpHrView_CheckProc
		
		NVAR filterbankUpdateFlag = $SpHrView_df_filter+"filterbankUpdateFlag"
		
		// File loading incicator, enabled when reading/writing
		TitleBox tb_loadingSound win=SpHrViewPeaksWindow#PTop, pos={wdth-60, 10},size={19,19}, fixedSize=1, title="💾"
		TitleBox tb_loadingSound win=SpHrViewPeaksWindow#PTop, fSize=14, frame=0,fstyle=1,fColor=(21845,21845,21845), disable=1
		
		// Settings
		Button b_SpHrView_Settings win=SpHrViewPeaksWindow#PTop, pos={wdth-37, 6}, title="", picture=ProcGlobal#SpHrView_pict_Wrench, size={23,23}
		Button b_SpHrView_Settings win=SpHrViewPeaksWindow#PTop, proc=SpHrView_ButtonProc
		Button b_SpHrView_Settings win=SpHrViewPeaksWindow#PTop, help={"Settings menu"}
		
		SetActiveSubwindow SpHrViewPeaksWindow //##
	endif
	if(1)
		// left (amplitude) axis controls
		ControlBar/W=SpHrViewPeaksWindow/L 20
		//ModifyGraph cbRGB =(65535,65534,65534)
		variable maxPa = wavemax(w)
		Slider sl_leftAxisMax win=SpHrViewPeaksWindow, pos={0,65},size={3,hgt-90},side=0,proc=AxisSliderProc
		if(dBscale)
			Slider sl_leftAxisMax win=SpHrViewPeaksWindow, limits={20*log(2/0.00002), 20*log(0.001/0.00002), 0}, value=maxPa
		else
			Slider sl_leftAxisMax win=SpHrViewPeaksWindow, limits={2,0.001,0}, value=maxPa
		endif
		Slider sl_leftAxisMax win=SpHrViewPeaksWindow, help={"Set maximum amplitude"}
		
		Button b_SpHrViewYZoom win=SpHrViewPeaksWindow, title="", picture=ProcGlobal#SpHrView_pict_GlassZoom, pos={2,40}, size={15, 15}
		Button b_SpHrViewYZoom win=SpHrViewPeaksWindow, proc=SpHrView_ButtonProc
		Button b_SpHrViewYZoom win=SpHrViewPeaksWindow, help={"Amplitude axis expand or auto scale when control, option, or command key pressed"}
		
		CheckBox ck_autoscaleY win=SpHrViewPeaksWindow, title="",pos={3,hgt-20},size={20,20},proc=SpHrView_CheckProc
		CheckBox ck_autoscaleY win=SpHrViewPeaksWindow, variable=$SpHrView_df_globals+"autoscaleAmplitudeAxis"
		CheckBox ck_autoscaleY win=SpHrViewPeaksWindow, help={"Autoscale peaks to include only the visible time range"}
	endif
	// Legend
	String LegendStr
	if(dimsize(w,1) > 1)
		LegendStr = "\\F'Arial'\\Z09\\s("+wStr+")left\\s("+wStr+"#1"+")right"
	else
		LegendStr = "\\F'Arial'\\Z09\\s("+wStr+")mono wvae"
	endif
	NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
	if(ChannelPair== 2)
		LegendStr += "\r\\JCchs 3 & 4"
	elseif(ChannelPair== 3)
		LegendStr += "\r\\JCchs 5 & 6"
	elseif(ChannelPair== 4)
		LegendStr += "\r\\JCchs 7 & 8"
	endif
	Legend/W=SpHrViewPeaksWindow/C/N=ViewLegend/J/E=2/X=1.00/Y=6.00/F=0/B=1/A=RT LegendStr
	
	// cursor(s) on initial peaks wave View
	Cursor/W=SpHrViewPeaksWindow/S=2/L=0/H=2/C=(0,65535/2,0,65535/2 ) A, $wStr, leftx(w)+((rightx(w) - leftx(w))/2)	
	

	if(1)
		NVAR isMovie=$SpHrView_df_globals+"isMovie"
		if(isMovie)
			if(movieisopen) // automatically open movie window
				string pathAndNameStr= SpHrView_FilePath_Get(w, 2) //note(w)
				//SpHrView_MovieOpen(pathAndNameStr)
				Button b_openMovie win=SpHrViewPeaksWindow#PTop, disable=0
			endif
		else
			Button b_openMovie win=SpHrViewPeaksWindow#PTop, disable=1
		endif
	endif
	

	// free axes for proper scaling and wave drawing
	GetAxis/W=SpHrViewPeaksWindow/Q bottom_free
	if(v_flag) // V_flag == 0 is used
		NewFreeAxis/B/O/W=SpHrViewPeaksWindow bottom_free
		ModifyFreeAxis/W=SpHrViewPeaksWindow bottom_free, master=bottom, hook=SpHrView_AxisChangeHook
		ModifyGraph/W=SpHrViewPeaksWindow noLabel(bottom_free)=2,axThick(bottom_free)=0
		ModifyGraph/W=SpHrViewPeaksWindow tick(bottom_free)=3,freePos(bottom_free)={0,kwFraction}
	endif
	GetAxis/W=SpHrViewPeaksWindow/Q left_free
	if(v_flag) // V_flag == 0 is used
		NewFreeAxis/L/O/W=SpHrViewPeaksWindow left_free
		ModifyFreeAxis/W=SpHrViewPeaksWindow left_free, master=left, hook=SpHrView_AxisChangeHook
		ModifyGraph/W=SpHrViewPeaksWindow noLabel(left_free)=2,axThick(left_free)=0
		ModifyGraph/W=SpHrViewPeaksWindow tick(left_free)=3,freePos(left_free)={0,kwFraction}
	endif
	
	// redraw graph marquee if coordinates are valid
	NVAR marqueeLeft = $SpHrView_df_globals+"marqueeLeft"
	NVAR marqueeRight = $SpHrView_df_globals+"marqueeRight"
	NVAR marqueeTop = $SpHrView_df_globals+"marqueeTop"
	NVAR marqueeBottom = $SpHrView_df_globals+"marqueeBottom"
	if(!numtype(marqueeLeft) && !numtype(marqueeRight) && !numtype(marqueeTop) && !numtype(marqueeBottom))
		DoUpdate/W=SpHrViewPeaksWindow 
		GetAxis/W=SpHrViewPeaksWindow/Q bottom
		//Printf "%g, %g\r", V_max, marqueeRight
		if(V_max >= marqueeRight)
			SetMarquee/W=SpHrViewPeaksWindow/HAX=bottom/VAX=left marqueeLeft, marqueeTop, marqueeRight, marqueeBottom
		endif
	endif
	
	// window hook
	DoUpdate/W=SpHrViewPeaksWindow // for Igor 9
	SetWindow SpHrViewPeaksWindow, hook(PeaksWindowHook)=SpHrView_PeaksWindowHook
	
End

// =================================================================================================
// SpHrView_AxisChangeHook(s), hook function called when bottom_free or left_free free axes change
// =================================================================================================
Function SpHrView_AxisChangeHook(s)
	STRUCT WMAxisHookStruct &s
	// axis: bottom and NewFreeAxis: bottom_free
	if(stringmatch(s.win, "SpHrViewPeaksWindow") && stringmatch(s.mastName, "bottom") && stringmatch(s.axName, "bottom_free") )
		Variable XaxisChanged = 0
		String Xuserdataname = "XInfoFor_"+s.mastName // "XInfoFor_"+"bottom"
		String Xuserdata = GetUserData(s.win, "", Xuserdataname)
		if (strlen(Xuserdata) == 0)
			XaxisChanged = 1
		else
			String Xminmaxstr = StringByKey(s.mastName, Xuserdata, "=", ";")	// data for bottom
			if (StrLen(Xminmaxstr) == 0)
				XaxisChanged = 1
			else
				Variable Xdeltamin = str2num(StringFromList(0, Xminmaxstr, ",")) - s.min
				Variable Xdeltamax = str2num(StringFromList(1, Xminmaxstr, ",")) - s.max
				Variable Xdeltasec = DateTime - str2num(StringFromList(2, Xminmaxstr, ","))
				Xdeltamin /= s.min != 0 ? s.min : 1
				Xdeltamax /= s.max != 0 ? s.max : 1
				//Printf "deltasec = %g\r" deltasec
				XaxisChanged = ((abs(Xdeltamin) > 1e-10) || (abs(Xdeltamax) > 1e-10)) && Xdeltasec > 0.25
			endif
		endif
		if (XaxisChanged)
			//Print "X axis change"
			SpHrView_AxisTimeCheck() 	// hook function
			String XnewUserData=""
			sprintf XnewUserData, "%s=%.15g,%.15g,%.1f;",s.mastName, s.min, s.max, DateTime	// e.g., bottom=0.00184807256235828,7.40522675736961,3646801785.5;
			SetWindow $(s.win), userdata($Xuserdataname) = XnewUserData
		endif
	endif
	
	// axis: left and NewFreeAxis: left_free
	if(stringmatch(s.win, "SpHrViewPeaksWindow") && stringmatch(s.mastName, "left") && stringmatch(s.axName, "left_free") )
		Variable YaxisChanged = 0
		String Yuserdataname = "YInfoFor_"+s.mastName // "YInfoFor_"+"left"
		String Yuserdata = GetUserData(s.win, "", Yuserdataname)
		if (strlen(Yuserdata) == 0)
			YaxisChanged = 1
		else
			String Yminmaxstr = StringByKey(s.mastName, Yuserdata, "=", ";")	// data for bottom
			if (StrLen(Yminmaxstr) == 0)
				YaxisChanged = 1
			else
				Variable Ydeltamin = str2num(StringFromList(0, Yminmaxstr, ",")) - s.min
				Variable Ydeltamax = str2num(StringFromList(1, Yminmaxstr, ",")) - s.max
				Variable Ydeltasec = DateTime - str2num(StringFromList(2, Yminmaxstr, ","))
				Ydeltamin /= s.min != 0 ? s.min : 1
				Ydeltamax /= s.max != 0 ? s.max : 1
				//Printf "deltasec = %g\r" deltasec
				YaxisChanged = ((abs(Ydeltamin) > 1e-10) || (abs(Ydeltamax) > 1e-10)) && Ydeltasec > 0.25
			endif
		endif
		if (YaxisChanged)
			//Print "Y axis change"
			SpHrView_AxisAmpCheck() 	// hook function
			String YnewUserData=""
			sprintf YnewUserData, "%s=%.15g,%.15g,%.1f;",s.mastName, s.min, s.max, DateTime	// e.g., left=0.00184807256235828,7.40522675736961,3646801785.5;
			SetWindow $(s.win), userdata($Yuserdataname) = YnewUserData
		endif
	endif
	return 0
End

// =================================================================================================
// SpHrView_AxisAmpCheck(), checks peaks display and updates them if necessary
// =================================================================================================
function SpHrView_AxisAmpCheck()
	String traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
	String wStr =  StringFromList(0,traceListStr), str
	String wStr_SubRange = wStr
	wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
	
	// limit left Y-axis values to those of original peaks wave
	NVAR dBscale= $SpHrView_df_globals+"dBscale"
	NVAR autoscaleAmplitudeAxis= $SpHrView_df_globals+"autoscaleAmplitudeAxis"
	if(autoscaleAmplitudeAxis)
		GetAxis /W=SpHrViewPeaksWindow/Q bottom
		Variable X_Min=v_min, X_Max=v_max
		if(dBscale)
			//variable Y_Min, Y_Max
			waveStats/Q/M=1 w
			//Y_Min = wavemin(w, X_Min, X_Max)
			//Y_Max = wavemax(w, X_Min, X_Max)
			SetAxis/W=SpHrViewPeaksWindow/A=2 left v_max-50,v_max
		else
			//Y_Max = wavemax(w, X_Min, Y_Max)
			SetAxis/W=SpHrViewPeaksWindow/A=2/E=1 left 0,*//Y_Max
		endif	
	else
		if(dBscale)
			SetAxis/W=SpHrViewPeaksWindow left *,*
		else
			SetAxis/W=SpHrViewPeaksWindow/E=1 left 0,*
		endif	
	endif
End

// =================================================================================================
// SpHrView_AxisTimeCheck(), checks peaks display and updates them if necessary
// =================================================================================================
function SpHrView_AxisTimeCheck()
	String traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
	variable numTraces
	numTraces = ItemsInList(traceListStr)
	String wStr =  StringFromList(0,traceListStr), str
	String wStr_SubRange = wStr
	wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
	variable numCols = dimsize(w,1)

	// get bottom X-axis minima and maxima
	GetAxis /W=SpHrViewPeaksWindow/Q bottom
	Variable X_Min=v_min, X_Max=v_max
	// get original peaks wave time/X minima and maxima
	variable Peaks_X_Min = DimOffset(w, 0)
	variable Peaks_X_Max = DimSize(w, 0)*DimDelta(w, 0) + DimOffset(w, 0)
	
	// limit bottom x-axis values to those of original peaks wave
	if(X_Min < Peaks_X_Min)
		SetAxis/W=SpHrViewPeaksWindow bottom Peaks_X_Min, *
	endif
	if(X_Max > Peaks_X_Max)
		SetAxis/W=SpHrViewPeaksWindow bottom *, Peaks_X_Max
	endif
	
	// make sure Draw selection doesn't exceed bottom axis range
	DrawAction/W=SpHrViewPeaksWindow/L=UserBack extractOutline
	wave/Z W_PolyX=W_PolyX, W_PolyY=W_PolyY
	killwaves/Z W_PolyY
	if(waveexists(W_PolyX) && dimsize(W_PolyX,0)==5) // object exists and is a square/rectangle
		variable startSec = wavemin(W_PolyX)
		variable endSec = wavemax(W_PolyX)
		killwaves/Z W_PolyX
		//Printf "1: startSec= %g, X_Min= %g, endSec= %g, X_Max= %g\r" startSec, X_Min, endSec, X_Max
		if(startSec < X_Min || endSec > X_Max)
			if(startSec < X_Min && endSec > X_Max)
				SpHrView_DrawSelection(1, X_Min, X_Max)
			elseif(startSec < X_Min)
				SpHrView_DrawSelection(0, X_Min, endSec)	// clear
			elseif(endSec > X_Max)
				SpHrView_DrawSelection(0, startSec, X_Max)	// clear
			endif
		endif
		//Printf "2: startSec= %g, X_Min= %g, endSec= %g, X_Max= %g\r" startSec, X_Min, endSec, X_Max
	endif
	
	variable A_hcsr_val = hcsr(A, "SpHrViewPeaksWindow")
		
	// check if subrange displayed
	variable usingSubRange= 0 
	variable useOriginalPeaks= 0
	NVAR NumberViewPeaks= $SpHrView_df_globals+"NumberViewPeaks"	
	variable Peaks_slop = Peaks_X_Max/(NumberViewPeaks/2)
	string subrangeStr = wStr+"_subrange"
	variable SR_X_Min=0, SR_X_Max=0
	if(itemsinlist(traceListStr)>2 && strsearch(traceListStr,"_subrange",0)>=0) // *_subrange displayed but not first returned
		wave/Z wSR = TraceNameToWaveRef("SpHrViewPeaksWindow", subrangeStr)
		if(waveexists(wSR))
			usingSubRange = 1
			SR_X_Min = DimOffset(wSR, 0)
			SR_X_Max = DimSize(wSR, 0)*DimDelta(wSR, 0) + DimOffset(wSR, 0)
		endif
	endif
	
	useOriginalPeaks = SpHr_equal(Peaks_X_Min, X_Min, Peaks_slop) && SpHr_equal(Peaks_X_Max, X_Max, Peaks_slop)
	
	if(useOriginalPeaks)
	
		variable i
		for(i=numTraces-1;i>=0;--i) // backwards
			str = StringFromList(i,traceListStr)
			if(strsearch(str, "_subrange", 0)>0) 
				RemoveFromGraph/W=SpHrViewPeaksWindow $str
			endif
		endfor
		ModifyGraph/W=SpHrViewPeaksWindow hideTrace($wStr)=0

		String LegendStr
		if(numCols > 1)
			ModifyGraph/W=SpHrViewPeaksWindow hideTrace($wStr+"#1")=0
			LegendStr = "\\F'Arial'\\Z09\\s("+wStr+")left\\s("+wStr+"#1"+")right"
		else
			LegendStr = "\\F'Arial'\\Z09\\s("+wStr+")mono"
		endif
		// Legend
		Legend/W=SpHrViewPeaksWindow/C/N=ViewLegend/J/E=2/X=1.00/Y=6.00/F=0/B=1/A=RT LegendStr
		ValDisplay vd_timeLine win=SpHrViewPeaksWindow#PTop, highColor= (52428,52428,52428), limits={0,1,0}, disable=2
		Slider sl_bottomAxis win=SpHrViewPeaksWindow#PTop, disable=2
		Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, disable=1
		
		// center cursor A if it is beyond the ends of the bottom axis. A_hcsr_val found above, before waves are (potentially) replaced
		if(A_hcsr_val < X_Min || A_hcsr_val > X_Max)
			Cursor/S=2/L=0/H=2/C=(0,65535/2,0,65535/2 )/W=SpHrViewPeaksWindow A, $wStr, X_Min+((X_Max-X_Min)/2)
		else
			Cursor/S=2/L=0/H=2/C=(0,65535/2,0,65535/2 )/W=SpHrViewPeaksWindow A, $wStr, A_hcsr_val
		endif
		
		Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, disable=1
		
	else

		// Hide original peaks waves
		ModifyGraph /W=SpHrViewPeaksWindow hideTrace($wStr)=2
		if(numCols > 1)
			ModifyGraph /W=SpHrViewPeaksWindow hideTrace($wStr+"#1")=2
		endif
		
		// calcualte the number of points that would be displayed if using original peaks
		variable EquivPeaksPnts =  ((X_Max-DimOffset(w,0))/DimDelta(w,0)) - ((X_Min-DimOffset(w,0))/DimDelta(w,0))	
		
		// use subrange if original peaks wave has too few poins
		variable useSubRange = (EquivPeaksPnts < NumberViewPeaks/4)
		// use subrange if axes minima & maxima are not roughly the same
		if(!useSubRange)
			variable SubRange_slop = (X_Max-X_Min)/20
			useSubRange = !SpHr_equal(X_Min, SR_X_Min, SubRange_slop) || !SpHr_equal(X_Max, SR_X_Max, SubRange_slop)
		endif
		
		// subrange values are (Hilbert) envelopes if SR_X_Max set and subrange is < 0.5 seconds
		variable useEnvelope	= SR_X_Max> 0 && abs(SR_X_Max-SR_X_Min) < 0.5
		
		if(useSubRange)
			string pathAndNameStr= ""
			variable referenceIsFile=1
			pathAndNameStr= SpHrView_FilePath_Get(w, 0) 			// Igor wave path if present or File path & name if Igor wave path is missing
			if(stringmatch(pathAndNameStr[0,3], "root")) 			// pathAndNameStr is a reference to a wave with path
				referenceIsFile=0
				pathAndNameStr= SpHrView_FilePath_Get(w, 1) 		// Igor wave path
			endif

			// get peaks (or envelopes) for subrange
			if(referenceIsFile)
				wStr_SubRange = SpHrView_LoadPeaksFromFile(pathAndNameStr, X_Min, X_Max, 1, 0, 0, useEnvelope)
			else
				wStr_SubRange = SpHrView_LoadPeaksFromWave(wStr, X_Min, X_Max, 1, useEnvelope)
			endif
			
			// Legend
			//if(dimsize(wSR,1) > 0)
			if(dimsize(w,1) > 0)
				if(useEnvelope)
					LegendStr = "\\F'Arial'\\Z09\\s("+wStr_SubRange+")left\\s("+wStr_SubRange+"#1"+")right \Z08\f02(env)"
				else
					LegendStr = "\\F'Arial'\\Z09\\s("+wStr_SubRange+")left\\s("+wStr_SubRange+"#1"+")right"// \Z08\f02(subrange)"
				endif
			else
				if(useEnvelope)
					LegendStr = "\\F'Arial'\\Z10\\s("+wStr_SubRange+")mono \Z08\f02(env)"
				else
					LegendStr = "\\F'Arial'\\Z09\\s("+wStr_SubRange+")mono"// \Z08\f02(subrange)"
				endif
			endif
			Legend /W=SpHrViewPeaksWindow /C/N=ViewLegend/J/E=2/X=1.00/Y=6.00/F=0/B=1/A=RT LegendStr
			
			wave wSR = $SpHrView_df_files+wStr_SubRange
			variable numSRCols = dimsize(wSR,1)
			
			if(ItemsInList(traceListStr) < max(1, dimsize(wSR,1))*2-0)  // < 3) // append subrange (or envelope) waves if necessary
				appendtograph /W=SpHrViewPeaksWindow wSR[][0]
				if(numSRCols > 1)
					appendtograph /W=SpHrViewPeaksWindow wSR[][1]
				endif
				ModifyGraph /W=SpHrViewPeaksWindow lsize=0
				ModifyGraph /W=SpHrViewPeaksWindow rgb($wStr_SubRange)=(0,0,65535)
				ModifyGraph /W=SpHrViewPeaksWindow usePlusRGB=1, mode=7, hbFill=2
				ModifyGraph /W=SpHrViewPeaksWindow plusRGB($wStr_SubRange)=(0,0,65535,32768)
				ModifyGraph /W=SpHrViewPeaksWindow hideTrace($wStr_SubRange)=0
				if(numSRCols > 1)
					ModifyGraph /W=SpHrViewPeaksWindow plusRGB($wStr_SubRange+"#1")=(65535,0,0,32768)
					ModifyGraph /W=SpHrViewPeaksWindow hideTrace($wStr_SubRange+"#1")=0
				endif
			endif
				
			// get new minima and maxima for subrange wave
			SR_X_Min = DimOffset(wSR, 0)
			SR_X_Max = DimSize(wSR, 0)*DimDelta(wSR, 0) + DimOffset(wSR, 0)
			ValDisplay vd_timeLine win=SpHrViewPeaksWindow#PTop, highColor= (39321,1,39321), limits={Peaks_X_Min,Peaks_X_Max,SR_X_Min}, value=_NUM:SR_X_Max
			Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, disable=0
			Slider sl_bottomAxis win=SpHrViewPeaksWindow#PTop, limits={Peaks_X_Min, Peaks_X_Max-(SR_X_Max-SR_X_Min), 0}, value=SR_X_Min, disable=0
		
			// if cursor is beyond current display, place cursor in middle of window
			if(A_hcsr_val < SR_X_Min || A_hcsr_val > SR_X_Max)
				Cursor/S=2/L=0/H=2/C=(0,65535/2,0,65535/2 )/W=SpHrViewPeaksWindow A, $wStr_SubRange, SR_X_Min+((SR_X_Max-SR_X_Min)/2)
			else
				Cursor/S=2/L=0/H=2/C=(0,65535/2,0,65535/2 )/W=SpHrViewPeaksWindow A, $wStr_SubRange, A_hcsr_val
			endif
		
		endif

	endif
	return 1
End
	
// =================================================================================================
// SpHrView_SliderProc slider control for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_SliderProc(sa) : SliderControl
	STRUCT WMSliderAction &sa
	Variable curval = sa.curval
	variable vmin, vmax, vrange
	strswitch( sa.ctrlName )	
		case "sl_bottomAxis": 
			switch( sa.eventCode )
				case 2:
				case 3:
				case 4:
				case 9:
					GetAxis/W=SpHrViewPeaksWindow/Q bottom
					if(!V_flag)
						vmin= V_min; vmax= V_max
						vmin = max(0,vmin)
						vrange = vmax-vmin
						SetAxis/W=SpHrViewPeaksWindow/Z bottom curval, curval+vrange
					endif
					break
				case -1: // control being killed
					break
				default:
					break
			endswitch // sa.eventCode
			break
	endswitch // sa.ctrlName
	return 0
End



		
// =================================================================================================
// SpHrView_LoadPeaksFromFile(...), load sound file and get its peaks
// - filePathAndName: string with both path and file name
//	- startSec: beginning or start time, in seconds, of sound to be loaded
//	- endSec: ending or stop time, in seconds, of sound to be loaded
// - isSubRange: sound loaded as a "SubRange" with *_subrange suffix
// - ignoreLength: ignore file length warning when reloading 
// - loadAsIgorWave: name and do not kill loaded wave
// =================================================================================================
function/S SpHrView_LoadPeaksFromFile(filePathAndName, startSec, endSec, isSubRange, ignoreLength, loadAsIgorWave, getEnvelope)
	String filePathAndName
	variable startSec, endSec, isSubRange, ignoreLength, LoadAsIgorWave, getEnvelope
	String dfSav = GetDataFolder(1)
	String fileNameStr
	// get file
	if(strlen(filePathAndName)<1)
		KillWindow/Z SpHrViewPeaksWindow // Kill Window if loading new file
		Variable refNum
		String message = "Select Sound file"
		String fileFilters = "Sound or Movie files (*.wav,*.mov,*.aif,*.aiff):.wav,.mov,.aif,.aiff;"
		fileFilters += "All Files:.*;"
		// check SoundsPath
		PathInfo SoundsPath
		if(V_flag && strlen(S_path)>1) // SoundsPath exists
			Open/D/R/F=fileFilters/P=SoundsPath /M=message refNum
		else
			Open/D/R/F=fileFilters /M=message refNum
		endif
		String FilePathNameStr = S_fileName
		if(strlen(FilePathNameStr)<1)
			return ""
		endif
		fileNameStr = ParseFilePath(0, FilePathNameStr, ":", 1, 0)
		String filePathStr = ParseFilePath(1, FilePathNameStr, ":", 1, 0)
		// set SoundsPath
		NewPath/C/O/Q SoundsPath, filePathStr
		filePathAndName = filePathStr+fileNameStr
	else
		fileNameStr = ParseFilePath(0, filePathAndName, ":", 1, 0) // for recent files
	endif
	// get sound file info
	SoundLoadWave/P=SoundsPath/O/Q/TMOT=10/S=(0,0.001) temp, filePathAndName
	if(V_Error)
		if(V_Error==1)
			DoAlert/T="Sound load error" 0, "Not enough memory to load sound file"
		else
			DoAlert/T="Sound load error" 0, "Unknown error"
		endif
	endif
	killwaves/Z temp
	String peaksWaveName = S_fileName
	NVAR isMovie=$SpHrView_df_globals+"isMovie"
	isMovie=0
	if(strsearch(peaksWaveName, ".mov",0, 2)>1) // add other movie extensions...
		isMovie=1
	endif
	peaksWaveName = ParseFilePath(3, peaksWaveName, ":", 0, 0) // Extract the file name without extension
	peaksWaveName = CleanupName(peaksWaveName, 0)
	
	NVAR numChannels= $SpHrView_df_globals+"numChannels"
	numChannels = str2num(StringByKey("CHANNELS", S_info))
	NVAR numSamples= $SpHrView_df_globals+"numSamples"
	numSamples = str2num(StringByKey("SAMPLES", S_info))
	NVAR fs= $SpHrView_df_globals+"fs"
	fs = round(str2num(StringByKey("RATE", S_info)))
	KillStrings/Z S_info
	variable dfs = 1/fs
	// set times
	if(!isSubRange && (startSec < 0 || endSec <= 0)) // if not a SubRange, load entire sound
		startSec = 0
		endSec = (numSamples)/fs
	endif	
	endSec = min(endSec, numSamples/fs)	// length check
	if(startSec > endSec) 					// another time check
		startSec = 0
		endSec = (numSamples)/fs
	endif
	
	// channel check
	if(numChannels<1 || numChannels>8)
		DoAlert/T="Sound load error" 0, "Expected a sound file with 1, 2, 4, 6 or 8 channels."
		return ""
	endif
	
	// Long file options
	NVAR file_start_min= $SpHrView_df_globals+"file_start_min"
	NVAR file_end_min= $SpHrView_df_globals+"file_end_min"
	NVAR loadPromptLength = $SpHrView_df_globals+"loadPromptLength" // minutes
	NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
	
	// prompt to load part of file if longer than lengthWarningLength_min
	if(!ignoreLength && !isSubRange && (endSec/60) > loadPromptLength) // - (startSec/60) > 1)
		file_start_min = startSec/60
		file_end_min = endSec/60
		SpHrView_SoundLengthPromptWindow(filePathAndName, numChannels, ChannelPair) // prompt for sound loading time frame 
		if(file_start_min >= 0 && file_end_min > 0 && file_start_min < file_end_min)
			startSec= file_start_min*60
			endSec= file_end_min*60
		else
			SetDatafolder $dfSav
			return ""
		endif
		
	// file channel selection
	elseif(!isSubRange)
		file_start_min = startSec/60
		file_end_min = endSec/60
		NVAR ChannelPairSelected = $SpHrView_df_globals+"ChannelPairSelected"
		if(numChannels > 2 && !ChannelPairSelected)
			SpHrView_SoundChannelPromptWindow(numChannels, ChannelPair)
		endif
		
	else // HELL
		file_start_min= -1
		file_end_min= 0
	endif
	
	if(startSec*fs > numSamples) 	// samples vs time check
		return ""
	endif
	variable desiredNumSamples		// another samples vs time check
	desiredNumSamples = (endSec*fs) - (startSec*fs)
	numSamples = min(desiredNumSamples, numSamples)
	
	// load sound
	SoundLoadWave/O/Q/S=(startSec, endSec) loadedFile, filePathAndName
	SoundLoadWave_Error(V_Error)
	wave loadedFile=loadedFile
	if(WaveType(loadedFile) & 0x04) // 64 bits
		Redimension/S loadedFile
	endif
	SetScale/P x DimOffset(loadedFile,0), DimDelta(loadedFile,0),"s", loadedFile
	
	// HELL
	NVAR ApplySPLScale = $SpHrView_df_globals+"ApplySPLScale"
	if(ApplySPLScale)
		NVAR SPLRefenceScale= $SpHrView_df_globals+"SPLRefenceScale"
		loadedFile *= SPLRefenceScale
	endif
	
	// allow for  > 2 pairs of binaural channels
	NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
	if(ChannelPair >= 1)
		SpHrView_SoundChannelRedimension(loadedFile, ChannelPair)
	endif
	
	NVAR dBscale= $SpHrView_df_globals+"dBscale"
	wave/Z peaks= $SpHrView_df_files+peaksWaveName
	
	if(getEnvelope)
		if(dBscale)
			if(isSubRange && waveexists(peaks))	// if getting SubRange envelope
				SpHr_HilbertEnvelope /B /dB=$SpHrView_df_files+peaksWaveName+"_subrange" loadedFile
				wave peaks = $SpHrView_df_files+peaksWaveName+"_subrange"
			else
				SpHr_HilbertEnvelope /B /dB=$SpHrView_df_files+peaksWaveName loadedFile
				wave peaks = $SpHrView_df_files+peaksWaveName
			endif
		else
			if(isSubRange && waveexists(peaks))	// if getting SubRange envelope
				SpHr_HilbertEnvelope /B /Pa=$SpHrView_df_files+peaksWaveName+"_subrange" loadedFile
				wave peaks = $SpHrView_df_files+peaksWaveName+"_subrange"
			else
				SpHr_HilbertEnvelope /B /Pa=$SpHrView_df_files+peaksWaveName loadedFile
				wave peaks = $SpHrView_df_files+peaksWaveName
			endif
		endif
	
	else

		// get peaks
		NVAR NumberViewPeaks= $SpHrView_df_globals+"NumberViewPeaks"
		variable dBref = 0
		if(dBscale)
			dBref = 0.00002 // 20 uPa
		endif
		if(isSubRange && waveexists(peaks))	// if getting SubRange peaks
			SpHr_WavePeaks /N=(NumberViewPeaks) /dB=(dBref) /Dest=$SpHrView_df_files+peaksWaveName+"_subrange" loadedFile
			wave peaks = $SpHrView_df_files+peaksWaveName+"_subrange"
		else											// getting peaks for entire file
			SpHr_WavePeaks /N=(NumberViewPeaks) /dB=(dBref) /Dest=$SpHrView_df_files+peaksWaveName loadedFile
			wave peaks = $SpHrView_df_files+peaksWaveName
		endif
		NVAR PeaksMindB= $SpHrView_df_globals+"PeaksMindB"
		PeaksMindB = V_MindB
		NVAR PeaksMaxdB= $SpHrView_df_globals+"PeaksMaxdB"
		PeaksMaxdB = V_MaxdB
		NVAR PeaksAvgdB= $SpHrView_df_globals+"PeaksAvgdB"
		PeaksAvgdB = V_AvgdB
		
		NVAR PeaksQ1dB= $SpHrView_df_globals+"PeaksQ1dB"
		PeaksQ1dB = V_Q1dB
		NVAR PeaksQ2dB= $SpHrView_df_globals+"PeaksQ2dB"
		PeaksQ2dB = V_Q2dB
		NVAR PeaksQ3dB= $SpHrView_df_globals+"PeaksQ3dB"
		PeaksQ3dB = V_Q3dB
		
		//Printf "min=%g, avg=%g, Q1=%g, Q2=%g, Q3=%g,\r", PeaksMindB, PeaksAvgdB, PeaksQ1dB, PeaksQ2dB, PeaksQ3dB
		

	endif
	
	NVAR LoadAsIgorWaveForSpeed = $SpHrView_df_globals+"LoadAsIgorWaveForSpeed"
	if(LoadAsIgorWave)
			Duplicate/O loadedFile, $SpHrView_df_waves+peaksWaveName 	// Duplicate to SpHrView_df_waves folder
			SpHrView_FilePath_Set(peaks, SpHrView_df_waves+peaksWaveName, filePathAndName)
	else
		if(LoadAsIgorWaveForSpeed) // load as Igor wave anyway (Duplicate to SpHrView_df_waves folder) to improve reading times (especially for movie files).
			Duplicate/O loadedFile, $SpHrView_df_waves+peaksWaveName
			SpHrView_FilePath_Set(peaks, SpHrView_df_waves+peaksWaveName, filePathAndName)
		else
			SpHrView_FilePath_Set(peaks, "", filePathAndName)
		endif
	endif
	killwaves/Z loadedFile

	SetDatafolder $dfSav
	
	if(isSubRange)
		// finished loading new SubRange
		TitleBox/Z tb_loadingSound win=SpHrViewPeaksWindow#PTop, disable=1; ControlUpdate/W=SpHrViewPeaksWindow#PTop tb_loadingSound // hide "loading..." titlebox
	
	else
	
		wave/Z w= $SpHrView_df_files+peaksWaveName
		if(!waveexists(w))
			DoAlert/T="Error" 0, "File missing! SpHrView_LoadPeaksFromFile(...): "+SpHrView_df_files+peaksWaveName
			//return ""
		endif
		SpHrView_PeaksWindow(peaksWaveName) // in SpHrView_df_files

	endif
	
	// Save to recent files lists
	SVAR RecentFileNames = $SpHrView_df_globals+"RecentFileNames"
	SVAR RecentFilePaths = $SpHrView_df_globals+"RecentFilePaths"
	if(WhichListItem(fileNameStr, RecentFileNames)<0)
		RecentFileNames = AddListItem(fileNameStr, RecentFileNames, ";", 0)
		RecentFilePaths = AddListItem(filePathAndName, RecentFilePaths, ";", 0)
		if(ItemsInList(RecentFileNames)>5)
			RecentFileNames = RemoveListItem(5, RecentFileNames)
			RecentFilePaths = RemoveListItem(5, RecentFilePaths)
		endif
	endif
	
	
	return NameOfWave(peaks) // may be peaksWaveName or peaksWaveName+"_subrange"
End

// =================================================================================================
// SpHrView_LoadPeaksFromWave(waveNameStr, startSec, endSec, isSnippet), get peaks of signal waveforms
//  in SpHrView_df_waves data folder
//	- waveNameStr: string name of wave to be used
//	- startSec: beginning or start *time in seconds* of sound to be loaded
//	- endSec: ending or stop *time in seconds* of sound to be loaded
// - isSnippet: sound loaded as a "snippet" with *_snip suffix
// =================================================================================================
function/S SpHrView_LoadPeaksFromWave(waveNameStr, startSec, endSec, isSubRange, getEnvelope)
	String waveNameStr
	variable startSec, endSec, isSubRange, getEnvelope
	String dfSav = GetDataFolder(1)
	// make sure SpHrView_df_waves exists
	if(!DataFolderExists(SpHrView_df_waves))
		DoAlert 0, "Data folder: "+SpHrView_df_waves+" is missing"
		return ""
	endif
	// Kill Window if loading new file
	if(strlen(waveNameStr)<1)
		SetDataFolder $SpHrView_df_waves
		String WaveListStr = WaveList("*", ";", "MINCOLS:2,MAXCOLS:2,SP:1" )
		// make sure waves exist in SpHrView_df_waves
		if(ItemsInList(WaveListStr)<1)
			DoAlert 0, "Data folder: "+SpHrView_df_waves+" is empty"
			return ""
		endif
		
		KillWindow/Z SpHrViewPeaksWindow
		
		PopupContextualMenu WaveListStr
		if(V_flag)
			waveNameStr = S_selection
		else	
			return ""
		endif
	endif
	
	SetDataFolder $SpHrView_df_files
	wave/Z w = $SpHrView_df_waves+waveNameStr
	if(!waveexists(w))
		DoAlert 0, "Signal: "+waveNameStr+" is missing"
		return ""
	endif
	
	variable dfs = DimDelta(w,0)
	NVAR fs= $SpHrView_df_globals+"fs"
	fs = 1/dfs
	NVAR numSamples= $SpHrView_df_globals+"numSamples"
	numSamples = DimSize(w,0)
	if(!isSubRange && (startSec < 0 || endSec<=startSec))
		startSec = DimOffset(w,0)
		endSec = (DimSize(w,0)*DimDelta(w,0) + DimOffset(w,0))
	endif
	
	// Are previously loaded waves always DimSize(w,1) = 2 ?
	if(DimSize(w,1)!=2)
		DoAlert/T="Signal Error" 0, "Expected a stereo wave (with 2 columns)"
	endif
	NVAR numChannels= $SpHrView_df_globals+"numChannels"
	numChannels = DimSize(w,1)

	// Shouldn't have to make this copy if startSec or endSec are not passed !!!
	Duplicate/O/R=(startSec, endSec) w, w_selection
	
	NVAR file_start_min= $SpHrView_df_globals+"file_start_min"
	NVAR file_end_min= $SpHrView_df_globals+"file_end_min"
	file_start_min = startSec/60
	file_end_min = endSec/60
	
	NVAR dBscale= $SpHrView_df_globals+"dBscale"
	wave/Z peaks= $SpHrView_df_files+waveNameStr
	
	if(getEnvelope)
		if(dBscale)
			if(isSubRange && waveexists(peaks))	// if getting SubRange envelope
				SpHr_HilbertEnvelope /B /dB=$SpHrView_df_files+waveNameStr+"_subrange" w_selection
				wave peaks = $SpHrView_df_files+waveNameStr+"_subrange"
			else
				SpHr_HilbertEnvelope /B /dB=$SpHrView_df_files+waveNameStr w_selection
				wave peaks = $SpHrView_df_files+waveNameStr
			endif
		else
			if(isSubRange && waveexists(peaks))	// if getting SubRange envelope
				SpHr_HilbertEnvelope /B /Pa=$SpHrView_df_files+waveNameStr+"_subrange" w_selection
				wave peaks = $SpHrView_df_files+waveNameStr+"_subrange"
			else
				SpHr_HilbertEnvelope /B /Pa=$SpHrView_df_files+waveNameStr w_selection
				wave peaks = $SpHrView_df_files+waveNameStr
			endif
		endif
	
	else
	
		NVAR NumberViewPeaks= $SpHrView_df_globals+"NumberViewPeaks"
		Variable dBref = 0
		if(dBscale)
			dBref = 0.00002
		endif
		if(isSubRange && waveexists(peaks))	// if getting snippet peaks
			SpHr_WavePeaks /N=(NumberViewPeaks) /dB=(dBref) /Dest=$SpHrView_df_files+waveNameStr+"_subrange" w_selection
			wave peaks = $SpHrView_df_files+waveNameStr+"_subrange"
		else											// getting peaks for entire file
			SpHr_WavePeaks /N=(NumberViewPeaks) /dB=(dBref) /Dest=$SpHrView_df_files+waveNameStr w_selection
			wave peaks = $SpHrView_df_files+waveNameStr
		endif
		NVAR PeaksMindB= $SpHrView_df_globals+"PeaksMindB"
		PeaksMindB = v_MindB
		NVAR PeaksMaxdB= $SpHrView_df_globals+"PeaksMaxdB"
		PeaksMaxdB = v_MaxdB
		NVAR PeaksAvgdB= $SpHrView_df_globals+"PeaksAvgdB"
		PeaksAvgdB = V_AvgdB
		
		NVAR PeaksQ1dB= $SpHrView_df_globals+"PeaksQ1dB"
		PeaksQ1dB = V_Q1dB
		NVAR PeaksQ2dB= $SpHrView_df_globals+"PeaksQ2dB"
		PeaksQ2dB = V_Q2dB
		NVAR PeaksQ3dB= $SpHrView_df_globals+"PeaksQ3dB"
		PeaksQ3dB = V_Q3dB

	endif
		
	killwaves/Z w_selection
	SetDatafolder $dfSav
			
	if(!isSubRange)
	
		SpHrView_FilePath_Set($SpHrView_df_files+waveNameStr, SpHrView_df_waves+waveNameStr, "") // only do this if NOT subrange
	
		wave/Z w= $SpHrView_df_files+waveNameStr
		if(!waveexists(w))
			DoAlert/T="Error" 0, "SpHrView_LoadPeaksFromWave: file missing: "+SpHrView_df_files+waveNameStr
			//return ""
		endif
		//NVAR isMovie=$SpHrView_df_globals+"isMovie"
		//isMovie=0 // why?
		
		SpHrView_PeaksWindow(waveNameStr) // in SpHrView_df_files
	
	endif
	
	return NameOfWave(peaks) // may be waveNameStr or waveNameStr+"_subrange"
end

// =================================================================================================
// SpHrView_LoadSoundFromFileWave(pathAndNameStr, waveRefDFStr, waveRefNameStr, startSec, endSec)
// Loads waveforms from a file specified by string pathAndNameStr ("Macintosh HD:Users:file")
// or copys them from the wave reference by pathAndNameStr ("root:Signals:waveName")
// to waveRefDFStr + waveRefNameStr (..."root:Signals:", "waveName"...)
// startSec and endSec specify File or Wave start and stop times
// =================================================================================================
function SpHrView_LoadSoundFromFileWave(pathAndNameStr, waveRefDFStr, waveRefNameStr, startSec, endSec)
	String pathAndNameStr, waveRefDFStr, waveRefNameStr
	Variable startSec, endSec
	
	if(stringmatch(pathAndNameStr[0,3], "root")) // pathAndNameStr is a reference to a wave with path
		wave/Z w = $pathAndNameStr
		if(!waveexists(w))
			DoAlert 0, "Signal "+pathAndNameStr+" is missing"
			return 1
		endif
		variable dfs = DimDelta(w,0)
		NVAR fs= $SpHrView_df_globals+"fs"
		fs = round(1/dfs)
		NVAR numSamples= $SpHrView_df_globals+"numSamples"
		numSamples = DimSize(w,0)
		if(startSec < 0 || endSec<=startSec) // get entire signal if startSec < 0 (or endSec<=startSec)
			startSec = DimOffset(w,0)
			endSec = ((DimSize(w,0))*DimDelta(w,0) + DimOffset(w,0))
		endif
		
		// Are previously loaded waves always DimSize(w,1) < 2 ?
		if(DimSize(w,1)>2)
			DoAlert/T="Signal Error" 0, "Expected 2 channels"
			return 1
		endif
		
		Duplicate/O/R=(startSec, endSec) w, $waveRefDFStr+waveRefNameStr
				
		// make sure the copy has the correct number of points (not sure if/why this is necessary)
		wave loadedWave = $waveRefDFStr+waveRefNameStr
		redimension/N=(round((endSec-startSec)*fs) , -1) loadedWave
		SetScale d 0,0,"Pa", loadedWave
		
	else // load waveforms, with minimal checks
		SetDataFolder $waveRefDFStr
		TitleBox/Z tb_loadingSound win=SpHrViewPeaksWindow#PTop, disable=0; ControlUpdate/W=SpHrViewPeaksWindow#PTop tb_loadingSound // activate "loading..." titlebox
		if(startSec< 0 || endSec <= startSec) 
			SoundLoadWave/P=SoundsPath/O/Q/TMOT=10 $waveRefNameStr, pathAndNameStr
		else
			SoundLoadWave/P=SoundsPath/O/Q/TMOT=10/S=(startSec, endSec) $waveRefNameStr, pathAndNameStr
		endif
		wave loadedWave = $waveRefDFStr+waveRefNameStr
		
		// allow for  > 2 pairs of binaural channels
		NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
		if(ChannelPair >= 1)
			SpHrView_SoundChannelRedimension(loadedWave, ChannelPair)
		endif

		TitleBox/Z tb_loadingSound win=SpHrViewPeaksWindow#PTop, disable=1; ControlUpdate/W=SpHrViewPeaksWindow#PTop tb_loadingSound // hide "loading..." titlebox
		SoundLoadWave_Error(V_Error)
		NVAR numSamples= $SpHrView_df_globals+"numSamples"
		numSamples = str2num(StringByKey("SAMPLES", S_info))
		NVAR fs= $SpHrView_df_globals+"fs"
		fs = round(str2num(StringByKey("RATE", S_info)))
		variable numChannels = str2num(StringByKey("CHANNELS", S_info))
		KillStrings/Z S_info
		if(numChannels<1 || numChannels>8)
			DoAlert/T="Sound load error" 0, "Expected a sound file with 1, 2, 4, 6 or 8 channels."
			return 1
		endif
		SetScale/P x 0, 1/fs,"s", loadedWave
		SetScale d 0,0,"Pa", loadedWave
		
//		NVAR ApplySPLScale = $SpHrView_df_globals+"ApplySPLScale"
//		if(ApplySPLScale)
//			NVAR SPLRefenceScale= $SpHrView_df_globals+"SPLRefenceScale"
//			loadedWave *= SPLRefenceScale
//		endif

	endif
	
	// HELL
//	NVAR LoadAsIgorWaveForSpeed = $SpHrView_df_globals+"LoadAsIgorWaveForSpeed"
//	if(!LoadAsIgorWaveForSpeed) // && !LoadAsIgorWave)
//		NVAR ApplySPLScale = $SpHrView_df_globals+"ApplySPLScale"
//		if(ApplySPLScale)
//			NVAR SPLRefenceScale= $SpHrView_df_globals+"SPLRefenceScale"
//			loadedWave *= SPLRefenceScale
//		endif
//	endif
	
	// filter waveform
	if(datafolderExists("root:SpHrView:Filter:"))		// datafolder exists
		string infostr = FunctionInfo("SpHrView_WaveformFilterApply") // function SpHrView_WaveformFilterApply exists
		if (strlen(infostr) > 0) // assuming NVAR_Exists(waveformFilter)
			NVAR waveformFilter = $"root:SpHrView:Filter:"+"waveformFilter"
			if(waveformFilter)
				String Cmd
				SPrintf Cmd, "SpHrView_WaveformFilterApply(\"%s\")", waveRefDFStr+waveRefNameStr
				Execute/Q Cmd
			endif
		endif
	endif
	
	// out of phase check box control, reverse phase of ch 2
	NVAR loadOutOfPhase = $SpHrView_df_globals+"loadOutOfPhase"
	if(loadOutOfPhase)
		loadedWave[][1] = -loadedWave[p][1] // invert second/right channel
	endif

end

// =================================================================================================
// SpHrView_LoadWaveformsFromPeaks(), Load wave named: waveforms, from SpHrViewPeaksWindow, into datafolder: SpHrView_df_files
// =================================================================================================
function [variable startSec, variable endSec, variable usingAxis] SpHrView_LoadWaveformsFromPeaks()
	String dfSav = GetDataFolder(1)
	SetDataFolder $SpHrView_df_files
	variable minSec, maxSec
	// get selection if SpHrViewPeaksWindow is open
	DoWindow SpHrViewPeaksWindow
	if(v_flag)
		string traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
		string wStr =  StringFromList(0,traceListStr)
		wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
		string pathAndNameStr= SpHrView_FilePath_Get(w, 0) //note(w)
		// get min and max times
		minSec = DimOffset(w, 0)
		maxSec = DimSize(w, 0)*DimDelta(w, 0) + DimOffset(w, 0)
		// get start and stop times
		GetMarquee/W=SpHrViewPeaksWindow/Z bottom
		//GetMarquee/Z bottom // allow marquee from another window...
		if(V_flag)
			startSec = v_left
			endSec = v_right
		else
			DrawAction/W=SpHrViewPeaksWindow/L=UserBack extractOutline
			wave/Z W_PolyX=W_PolyX, W_PolyY=W_PolyY
			killwaves W_PolyY
			if(waveexists(W_PolyX) && dimsize(W_PolyX,0)==5) // object exists and is a square/rectangle
				startSec = wavemin(W_PolyX)
				endSec = wavemax(W_PolyX)
				killwaves W_PolyX
			else
				GetAxis /W=SpHrViewPeaksWindow/Q bottom
				//GetAxis/Q bottom // allow axis from another window...
				startSec = v_min
				endSec = v_max
				usingAxis = 1
			endif
		endif
	else
		SetDatafolder $dfSav
		return [startSec, endSec, usingAxis]
	endif
	// make sure startSec and endSec are valid, since Marquees from other windows are being allowed.
	startSec = max(startSec, minSec)
	endSec = min(endSec, maxSec)
	// load waveforms
	SpHrView_LoadSoundFromFileWave(pathAndNameStr, SpHrView_df_files, "waveforms", startSec, endSec)
	wave waveforms=$SpHrView_df_files+"waveforms"
	NVAR fs= $SpHrView_df_globals+"fs"
	SetScale/P x startSec,1/fs,"s", waveforms
	
	killStrings/Z S_info
	SetDatafolder $dfSav
	
	return [startSec, endSec, usingAxis]
End



// =================================================================================================
// SpHrView_SoundLoadProgressBar(minVal, maxVal), sound file length warning prompt
// =================================================================================================
function SpHrView_SoundLoadProgressBar(minVal, maxVal)
	variable minVal, maxVal
	Killwindow/Z Progress
	NewPanel /K=1/N=Progress /W=(20,45,395,79) as "Loading sound file..."//  /FLT
	ValDisplay vd_Progress,pos={5,5},size={300,20},limits={minVal,maxVal,0},barmisc={0,0}, frame=0
	ValDisplay vd_Progress,value= _NUM:0
	ValDisplay vd_Progress,mode= 3	// bar with no fractional part
	Button b_ProgressStop,pos={315,3},size={50,22},title="Stop"
	DoUpdate /W=Progress /E=1	// mark this as our progress window
end
// =================================================================================================
// SpHrView_SoundLengthPromptWindow(PromptStr), used by SpHrView_SoundLoadProgressBar(...)
// =================================================================================================
function SpHrView_SoundLengthPromptWindow(FilePathNameStr, numChannels, ChannelPair)//(PromptStr)
	String FilePathNameStr // PromptStr
	Variable numChannels, ChannelPair
	NVAR file_start_min= $SpHrView_df_globals+"file_start_min"
	NVAR file_end_min= $SpHrView_df_globals+"file_end_min"
	variable start_min = file_start_min
	variable end_min = file_end_min
	variable ht = 240, btn_top = 105
	if(numChannels>2)
		ht = 280
		btn_top = 145
	endif
	killWindow/Z SoundLengthPromptWindow
	NewPanel/K=2/N=SoundLengthPromptWindow /W=(400,100,725,ht) as "Options for Opening long files" // do not use /FLT=1
	//ModifyPanel cbRGB=(65534,65534,65534)
	GroupBox group_length,pos={8,10},size={310,80}//,labelBack=(56797,56797,56797)
	TitleBox tb_message0 pos={25,20}, title="File Length: "+num2str(file_end_min)+" minutes.  Load a portion of it?"
	TitleBox tb_message0 frame=0,fSize=10
	SetVariable sv_FilePathNameStr pos={12,40}, size={300,43}, Value=_STR:"\\JR"+FilePathNameStr
	SetVariable sv_FilePathNameStr frame=1,fSize=9,styledText=1
	
	SetVariable sv_startTime title="Load From",pos={30,65},size={130,16},format="%.2f"
	SetVariable sv_startTime bodyWidth=50,value=_NUM:start_min //file_start_min
	SetVariable sv_startTime limits={0,inf,1},fSize=11
	SetVariable sv_endTime title="to",pos={160,65},size={70,16},format="%.2f"
	SetVariable sv_endTime bodyWidth=50,value=_NUM:end_min //file_end_min
	SetVariable sv_endTime limits={0,inf,1},fSize=11
	TitleBox tb_min pos={238,67}, frame=0,fSize=11,title="minutes"
	
	if(numChannels>2)
		GroupBox group_chs,pos={8,95},size={310,35}//,labelBack=(56797,56797,56797)
		String PopUpTileStr, PopUpStr
		if(numChannels==4)
			ChannelPair = min(ChannelPair,2)
			PopUpTileStr = "Load channels (1-2,3-4) "
			PopUpStr = "\"1 and 2;3 and 4\""
		elseif(numChannels==6)
			ChannelPair = min(ChannelPair,3)
			PopUpTileStr = "Load channels (1-2,3-4,5-6) "
			PopUpStr = "\"1 and 2;3 and 4;5 and 6\""
		elseif(numChannels==8)
			PopUpTileStr = "Load channels (1-2,3-4,5-6,7-8) "
			PopUpStr = "\"1 and 2;3 and 4;5 and 6;7 and 8\""
		endif
		PopupMenu popup_LengthChannelPair title=PopUpTileStr,pos={40,101},fSize=11, mode=max(1,ChannelPair), value=#PopUpStr
		
	endif
	
	Button b_SoundLoadCancel pos={20,btn_top},size={75,21},proc=SpHrView_SoundLengthPromptButtonProc, title="Cancel"
	Button b_SoundLoadUseTimes pos={115,btn_top},size={80,21},proc=SpHrView_SoundLengthPromptButtonProc, title="Open File",focusRing=1, fstyle=1
	Button b_SoundLoadView pos={215,btn_top},size={80,21},proc=SpHrView_SoundLengthPromptButtonProc, title="View File"
	
	DoUpdate/W=SoundLengthPromptWindow // for Igor 9
	SetWindow SoundLengthPromptWindow,hook(SoundLengthHook)=SpHrView_SoundLengthPromptHook
	PauseForUser SoundLengthPromptWindow
end
// =================================================================================================
// SpHrView_SoundLengthPromptButtonProc(ctrlName), used by SpHrView_SoundLoadProgressBar(...)
// =================================================================================================
Function SpHrView_SoundLengthPromptButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 1: // mouse down
			//Print  ba.ctrlName
			NVAR file_start_min= $SpHrView_df_globals+"file_start_min"
			NVAR file_end_min= $SpHrView_df_globals+"file_end_min"
			variable xmin, xmax
			String FilePathNameStr
			strswitch( ba.ctrlName )
				case "b_SoundLoadUseTimes":
					ControlInfo/W=SoundLengthPromptWindow sv_startTime 
					if(abs(V_flag)==5)
						file_start_min = V_Value 
					endif 
					ControlInfo/W=SoundLengthPromptWindow sv_endTime 
					if(abs(V_flag)==5)
						file_end_min = V_Value 
					endif
					ControlInfo/W=SoundLengthPromptWindow popup_LengthChannelPair
						if(abs(V_flag)==3)
							NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
							ChannelPair = V_Value
						endif 
					DoWindow/K SoundLengthPromptWindow
					break
					
				case "b_SoundLoadCancel":
					file_start_min= -1
					file_end_min= 0
					DoWindow/K SoundLengthPromptWindow
					break
					
				case "b_SoundLoadView":
					ControlInfo/W=SoundLengthPromptWindow sv_FilePathNameStr 
					if(abs(V_flag)==5)
						SpHrView_SoundLengthPromptView(S_Value[3,strlen(S_Value)], -1, -1)
					endif
					
					file_start_min= -1
					file_end_min= 0
					DoWindow/K SoundLengthPromptWindow
					break
					
				case "b_LoadFromFileView":
					SpHrView_SoundLengthPromptLoad()
					break
					
				case "b_LoadFromFileStop":
					StopPlaySound()		// stop playing sounds
					break
					
				case "b_LoadFromFilePlay":
					SpHrView_SoundLengthPromptViewPlay()
					break
						
				case "b_UpdatePeaksFromFile":
					ControlInfo/W=SpHrView_SoundLoadViewWindow sv_FilePathName
					FilePathNameStr = S_Value
					if(abs(V_flag)==5)
						GetFileFolderInfo /Q/Z FilePathNameStr
						if(!V_isFile)
							return 0
						endif
						GetAxis/W=SpHrView_SoundLoadViewWindow /Q bottom
						if(v_flag)
							return 0
						endif
						SpHrView_SoundLengthPromptView(FilePathNameStr, V_min, V_max)
					endif
					break
				case "b_RevertPeaksFromFile":
					ControlInfo/W=SpHrView_SoundLoadViewWindow sv_FilePathName
					FilePathNameStr = S_Value
					if(abs(V_flag)==5)
						GetFileFolderInfo /Q/Z FilePathNameStr
						if(!V_isFile)
							return 0
						endif
						SpHrView_SoundLengthPromptView(FilePathNameStr, -1, -1)
						SetAxis/W=SpHrView_SoundLoadViewWindow /A
					endif
					break
					
			endswitch
		break
	endswitch
End


// =================================================================================================
// SpHrView_SoundLengthPromptHook(s), used by SpHrView_SoundLoadProgressBar(...)
// =================================================================================================
Function SpHrView_SpHrView_SoundLengthPromptHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0
	strswitch(s.winName)
		case "SoundLengthPromptWindow":
			switch(s.eventCode)
				case 11:	// keyboard
					if(s.keycode==13) // return
						ControlInfo/W=SoundLengthPromptWindow popup_LengthChannelPair
						if(abs(V_flag)==3)
							NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
							ChannelPair = V_Value
						endif
						NVAR ChannelPairSelected = $SpHrView_df_globals+"ChannelPairSelected"
						ChannelPairSelected = 1
						//DoWindow/K SoundLengthPromptWindow
					endif
				break
			endswitch
			hookResult = 1
			break
			
		case "SpHrView_SoundLoadViewWindow":
			switch(s.eventCode)
				case 2:	// kill
					Execute/Z/Q/P "DFStr(\""+SpHrView_df_files+"SoundLoadView\")"
					SetDataFolder root:
					hookResult = 1 // 0?
					break
					
				case 11:	// keyboard
					switch(s.keycode)
						case 13: // return
							SpHrView_SoundLengthPromptLoad()
							hookResult = 1
							break
						
						case 32:		// space bar
						case 80:		// "p" or "P" WITHOUT modifier
						case 112:
							SpHrView_SoundLengthPromptViewPlay()
							hookResult = 1
							break
						
						case 83:		// "s" or "S" WITHOUT modifier
						case 115:
							StopPlaySound()		// stop playing sounds
							hookResult = 1
							break
						
					endswitch
					break
			endswitch
			break
	endswitch
	return hookResult
end

// =================================================================================================
// SpHrView_SoundLengthPromptView(s), sound length loading window
// =================================================================================================
Function SpHrView_SoundLengthPromptView(FilePathNameStr, StartSec, EndSec)
	String FilePathNameStr
	Variable StartSec, EndSec//, PlayFlag
	DoWindow/K SoundLengthPromptWindow
	PauseForUser/C SoundLengthPromptWindow
	DoUpdate
	GetFileFolderInfo /Q/Z FilePathNameStr
	if(!V_isFile)
		return 0
	endif
	DFREF saveDFR = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_files
	NewDataFolder/O/S SoundLoadView
	
	if(StartSec<0 || EndSec<0 || EndSec<StartSec)
		SoundLoadWave/P=SoundsPath/O/Q/TMOT=30 Sound, FilePathNameStr
	else
		SoundLoadWave/P=SoundsPath/O/Q/TMOT=30/S=(StartSec,EndSec) Sound, FilePathNameStr
	endif
	Wave Sound=Sound
	NVAR numChannels= $SpHrView_df_globals+"numChannels"
	numChannels = str2num(StringByKey("CHANNELS", S_info))
	KillStrings/Z S_info
	if(numChannels<1 || numChannels>8)
		DoAlert/T="Sound load error" 0, "Expected a sound file with 1, 2, 4, 6 or 8 channels."
		return 1
	endif

	if(dimsize(Sound,1) > 2)
		NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
		if(ChannelPair >= 1)
			SpHrView_SoundChannelRedimension(Sound, ChannelPair)
		endif
	endif
	
	String soundInfo = S_info
	variable scaleOptions = 0
	NVAR dBscale= $SpHrView_df_globals+"dBscale"
	variable dBref = 0
	if(dBscale) //set for dBscale
		dBref = 0.00002
	endif
	NVAR NumberViewPeaks= $SpHrView_df_globals+"NumberViewPeaks"
	//SpHr_WavePeaks(Sound, NumberViewPeaks, scaleOptions, $"", "SoundPeaks")
	SpHr_WavePeaks /N=(NumberViewPeaks) /dB=(dBref) /Dest=SoundPeaks Sound
	wave SoundPeaks = $"SoundPeaks"
	//KillWaves Sound // don't kill so playing is faster...
	variable i
	NVAR screenHoriz= $SpHrView_df_globals+"screenHoriz"
	NVAR screenVert= $SpHrView_df_globals+"screenVert"
	//killWindow/Z SpHrView_SoundLoadViewWindow
	DoWindow SpHrView_SoundLoadViewWindow
	if(v_flag)
		RemoveTraces("SpHrView_SoundLoadViewWindow")
	else
		Display /K=1/W=(5, 10, screenHoriz-5, 210) as "File time selection "
		DoWindow/C SpHrView_SoundLoadViewWindow
		ModifyGraph margin(bottom)=25,margin(left)=30,margin(right)=5,margin(top)=5
		ControlBar/T 27
		//ModifyPanel cbRGB=(65534,65534,65534)
		Button b_RevertPeaksFromFile title="Reload File  🗃",pos={30,1},size={90,22},fSize=10, proc=SpHrView_SoundLengthPromptButtonProc
		//Button b_UpdatePeaksFromFile title="Update Peaks",pos={130,1},size={85,22},fSize=10, proc=SpHrView_SoundLengthPromptButtonProc
		CheckBox ck_updateTimeAxis title="update on expand",pos={130,4},value=1
		
		Button b_LoadFromFileView title="Open Selection",pos={260,1},size={110,22},fstyle=1,fSize=10, proc=SpHrView_SoundLengthPromptButtonProc
		Button b_LoadFromFileView help={"Load file selection using graph marquee or bottom axis range. Can also press 'Return'"}
	
		Button b_LoadFromFilePlay pos={410,3}, title="", picture=ProcGlobal#SpHrView_pict_circlePlay, size={21,21}, proc=SpHrView_SoundLengthPromptButtonProc
		Button b_LoadFromFilePlay help={"Play sounds within current view or within a marquee selection. Can also use the \"p\" key or space bar. Press and hold up or down arrow to reverse left & right channels."}
		Button b_LoadFromFileStop pos={440,3}, title="", picture=ProcGlobal#SpHrView_pict_circleStop, size={21,21}, proc=SpHrView_SoundLengthPromptButtonProc
		Button b_LoadFromFileStop help={"Stop playing sound. Can also use the \"s\" key"}
	
	endif
	
	if(numChannels>2)
		String PopUpTileStr, PopUpStr
		if(numChannels==4)
			ChannelPair = min(ChannelPair,2)
			//PopUpTileStr = "Load channels (1-4)"
			PopUpStr = "\"1 & 2;3 & 4\""
		elseif(numChannels==6)
			ChannelPair = min(ChannelPair,3)
			//PopUpTileStr = "Load channels (1-6)"
			PopUpStr = "\"1 & 2;3 & 4;5 & 6\""
		elseif(numChannels==8)
			//PopUpTileStr = "Load channels (1-8)"
			PopUpStr = "\"1 & 2;3 & 4;5 & 6;7 & 8\""
		endif
		PopupMenu popup_LengthChannelPair title="Load channels",pos={500,4},fSize=10, mode=max(1,ChannelPair), value=#PopUpStr
		PopupMenu popup_LengthChannelPair proc=SpHrView_SoundChannelPromptPopMenuProc
	endif
	
	variable cnt=0, ch
	String LegendStr = ""
	for(i=0;i<dimsize(SoundPeaks,1);++i)
		ch = cnt+1
		if(ChannelPair==2)
			ch += 2
		endif
		if(ChannelPair==3)
			ch += 4
		endif
		if(ChannelPair==4)
			ch += 6
		endif
		if(cnt==0)
			LegendStr += "ch"+num2str(ch)+"\\s(SoundPeaks)"
		else
			LegendStr += "  ch"+num2str(ch)+"\\s(SoundPeaks#"+num2str(cnt)+")"
		endif
		cnt += 1
	endfor
	cnt=0
	
	if(dimsize(SoundPeaks,1)==2) // 2 channels
		AppendToGraph SoundPeaks[][0]
		AppendToGraph SoundPeaks[][1]
		ModifyGraph lsize=0
		ModifyGraph mode(SoundPeaks)=7,usePlusRGB(SoundPeaks)=1,hbFill(SoundPeaks)=2,plusRGB(SoundPeaks)=(0,0,65535,32768),RGB(SoundPeaks)=(0,0,65535,32768)
		ModifyGraph mode(SoundPeaks#1)=7,usePlusRGB(SoundPeaks#1)=1,hbFill(SoundPeaks#1)=2,plusRGB(SoundPeaks#1)=(65535,0,0,32768),RGB(SoundPeaks#1)=(65535,0,0,32768)
	else
		//for(i=max(0,dimsize(SoundPeaks,1)-1);i>=0;--i) // reverse
		for(i=0;i>dimsize(SoundPeaks,1);++i) // reverse
			AppendToGraph SoundPeaks[][i]
			if(cnt==0)
				ModifyGraph rgb(SoundPeaks)=(0,0,65535)
			elseif(cnt==1)
				ModifyGraph rgb(SoundPeaks#1)=(65535,0,0)
			else
				ModifyGraph rgb($"SoundPeaks#"+num2str(cnt))=(0,0,0)
			endif
			cnt += 1
		endfor
		ModifyGraph lsize=0.5
	endif
		
	ModifyGraph tick(left)=2,tick(bottom)=0,fSize=10,ZisZ=1,zapTZ=1,zapLZ=1,standoff=0,btLen=4,btThick=0.5,stLen=2,stThick=0.5,font="Arial"
	ModifyGraph axThick(bottom)=0.5, axThick(left)=0.5
	Label left "amplitude (\\U)"
	SetAxis/A=2/E=1 left
	Label bottom "time (\\U)"
	ModifyGraph nticks(bottom)=10
	TextBox/C/N=text0/B=1/F=0/M/A=LT/X=1.00/Y=1.00 LegendStr
	
	String SoundInfoStr = ""
	//SoundInfoStr += StringByKey("FILE",soundInfo)+"     "
	SoundInfoStr += StringByKey("CHANNELS",soundInfo)+" Channels, "
	SoundInfoStr += StringByKey("BITS",soundInfo)+"-bits, "
	SoundInfoStr += StringByKey("RATE",soundInfo)+" Hz sampling rate, "
	SoundInfoStr += StringByKey("DURATION",soundInfo)+" seconds,  "
	SoundInfoStr += StringByKey("FORMAT",soundInfo)
	//variable seconds = str2num(StringByKey("DURATION",soundInfo))
	//if(seconds>60)
	//	SoundInfoStr += ", "+num2str(seconds/60)+" min"
	//endif
	
	SetVariable sv_FilePathName pos={(screenHoriz/2)+0,0},size={(screenHoriz/2)-60,20},fSize=9,noedit=1,value= _STR:FilePathNameStr
	SetVariable sv_FilePathName limits={-inf,inf,0},frame=0
	
	SetVariable sv_SoundInfo pos={(screenHoriz/2)+0,12},size={(screenHoriz/2)-60,20},fSize=9,noedit=1,value= _STR:SoundInfoStr
	SetVariable sv_SoundInfo limits={-inf,inf,0},frame=0
	
	// FreeAxis
	GetAxis/W=SpHrView_SoundLoadViewWindow/Q bottom_free
	if(v_flag)
		NewFreeAxis/B/O/W=SpHrView_SoundLoadViewWindow bottom_free
		ModifyFreeAxis/W=SpHrView_SoundLoadViewWindow bottom_free, master=bottom, hook=SpHrView_SoundLoadViewTimeAxisHook
		ModifyGraph/W=SpHrView_SoundLoadViewWindow noLabel(bottom_free)=2,axThick(bottom_free)=0.5
		ModifyGraph/W=SpHrView_SoundLoadViewWindow tick(bottom_free)=0,freePos(bottom_free)={1,kwFraction}
		ModifyGraph/W=SpHrView_SoundLoadViewWindow btLen=4,btThick=0.5,stLen=2,stThick=0.5
	endif

	DoUpdate/W=SpHrView_SoundLoadViewWindow // for Igor 9
	SetWindow SpHrView_SoundLoadViewWindow, hook(SoundLoadViewWindow)=SpHrView_SoundLengthPromptHook
	SetDataFolder saveDFR
	
end
// =================================================================================================
// SpHrView_SoundLoadViewTimeAxisHook(s), sound length time axis hook function
// =================================================================================================
Function SpHrView_SoundLoadViewTimeAxisHook(s)
	STRUCT WMAxisHookStruct &s
	if(stringmatch(s.win, "SpHrView_SoundLoadViewWindow") && stringmatch(s.mastName, "bottom") && stringmatch(s.axName, "bottom_free") )
		Variable axisChanged = 0
		String userdataname = "InfoFor_"+s.mastName
		String userdata = GetUserData(s.win, "", userdataname)
		if (strlen(userdata) == 0)
			axisChanged = 1
		else
			String minmaxstr = StringByKey(s.mastName, userdata, "=", ";")
			if (StrLen(minmaxstr) == 0)
				axisChanged = 1
			else
				Variable deltamin = str2num(StringFromList(0, minmaxstr, ",")) - s.min
				Variable deltamax = str2num(StringFromList(1, minmaxstr, ",")) - s.max
				deltamin /= s.min != 0 ? s.min : 1
				deltamax /= s.max != 0 ? s.max : 1
				axisChanged = (abs(deltamin) > 1e-10) || (abs(deltamax) > 1e-10)
			endif
		endif
		if (axisChanged)
			SpHrView_SoundLengthPromptViewUpdate()
			String newUserData=""
			sprintf newUserData, "%s=%.15g,%.15g;",s.mastName, s.min, s.max
			SetWindow $(s.win), userdata($userdataname) = newUserData
		endif
	endif
	return 0
End
// =================================================================================================
// SpHrView_SoundLengthPromptViewUpdate(s), sound length update function
// =================================================================================================
Function SpHrView_SoundLengthPromptViewUpdate()
	if(!DataFolderExists(SpHrView_df_files+"SoundLoadView"))
		return 0
	endif
	DFREF saveDFR = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_files+"SoundLoadView"
	ControlInfo/W=SpHrView_SoundLoadViewWindow ck_updateTimeAxis
	if(!V_Value)
		return 0
	endif
	ControlInfo/W=SpHrView_SoundLoadViewWindow sv_FilePathName
	String FilePathNameStr = S_Value
	if(abs(V_flag)!=5)
		return 0
	endif
	GetFileFolderInfo /Q/Z FilePathNameStr
	if(!V_isFile)
		return 0
	endif
	GetAxis/W=SpHrView_SoundLoadViewWindow /Q bottom
	if(v_flag)
		return 0
	endif
	Variable x_min=V_min, x_max=V_max
	
	SoundLoadWave/P=SoundsPath/O/Q/TMOT=30/S=(x_min,x_max) Sound, FilePathNameStr
	Wave Sound=Sound
	variable numChannels = str2num(StringByKey("CHANNELS", S_info))
	KillStrings/Z S_info
	if(numChannels<1 || numChannels>8)
		DoAlert/T="Sound load error" 0, "Expected a sound file with 1, 2, 4, 6 or 8 channels."
		return 1
	endif
	if(dimsize(Sound,1) > 2)
		NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
		if(ChannelPair >= 1)
			SpHrView_SoundChannelRedimension(Sound, ChannelPair)
		endif
	endif
	String soundInfo = S_info
	variable scaleOptions = 0
	NVAR dBscale= $SpHrView_df_globals+"dBscale"
	variable dBref = 0
	if(dBscale) //set for dBscale
		dBref = 0.00002
	endif
	NVAR NumberViewPeaks= $SpHrView_df_globals+"NumberViewPeaks"
	//SpHr_WavePeaks(Sound, NumberViewPeaks, scaleOptions, $"", "SoundPeaks")
	SpHr_WavePeaks /N=(NumberViewPeaks) /dB=(dBref) /Dest=SoundPeaks Sound
	wave SoundPeaks = $"SoundPeaks"
	//KillWaves Sound // don't kill so playing is faster...
	SetAxis/A=2/E=1 left
	SetDataFolder saveDFR
End
// =================================================================================================
// SpHrView_SoundLengthPromptLoad(s), sound length load function
// =================================================================================================
Function SpHrView_SoundLengthPromptLoad()
	NVAR ChannelPairSelected = $SpHrView_df_globals+"ChannelPairSelected"
	ChannelPairSelected = 1
	ControlInfo/W=SpHrView_SoundLoadViewWindow sv_FilePathName
	String FilePathNameStr = S_Value
	Variable xmin, xmax
	if(abs(V_flag)==5)
		GetMarquee/K/W=SpHrView_SoundLoadViewWindow/Z bottom
		if(V_flag) // Marquee used
			xmin = max(0,V_left)
			xmax = V_right
			doupdate
			if(V_flag)
				//Printf "%g, %g\r" V_left, V_right
				SpHrView_LoadPeaksFromFile(FilePathNameStr, xmin, xmax, 0, 1, 0, 0)
			endif
		else
			GetAxis/W=SpHrView_SoundLoadViewWindow /Q bottom
			if(v_flag)
				return 0
			endif
			xmin = max(0,V_min)
			xmax = V_max
			KillWindow/Z SpHrView_SoundLoadViewWindow
			doupdate
			SpHrView_LoadPeaksFromFile(FilePathNameStr, xmin, xmax, 0, 1, 0, 0)
			// SpHrView_LoadPeaksFromFile(FilePathNameStr, -1, -1, 0, 1, 0) // entire file
		endif
		KillWindow/Z SpHrView_SoundLoadViewWindow
	endif
end
// =================================================================================================
// SpHrView_SoundLengthPromptViewPlay(s), sound length window Play function
// =================================================================================================
Function SpHrView_SoundLengthPromptViewPlay()
	ControlInfo/W=SpHrView_SoundLoadViewWindow sv_FilePathName
	String FilePathNameStr = S_Value
	Variable xmin, xmax
	if(abs(V_flag)==5)
		GetMarquee/K/W=SpHrView_SoundLoadViewWindow/Z bottom
		if(V_flag) // Marquee used
			xmin = max(0,V_left)
			xmax = V_right
		else
			GetAxis/W=SpHrView_SoundLoadViewWindow /Q bottom
			if(v_flag)
				return 0
			endif
			xmin = max(0,V_min)
			xmax = V_max
		endif
		//StopPlaySound()		// stop playing sounds
		Wave/Z Sound= $SpHrView_df_files+"SoundLoadView:Sound"
		if(waveexists(Sound))
			if(xmin<0 || xmax<0 || xmax<xmin)
				Duplicate/O Sound, Play
			else
				Duplicate/O/R=(xmin,xmax) Sound, Play
			endif
		else
			//Printf "'%s'\rV_isFile = %g\r" FilePathNameStr, V_isFile
			if(xmin<0 || xmax<0 || xmax<xmin)
				SoundLoadWave/P=SoundsPath/O/Q/TMOT=30 Sound, FilePathNameStr
			else
				SoundLoadWave/P=SoundsPath/O/Q/TMOT=30/S=(xmin,xmax) Play, FilePathNameStr
			endif
			if(dimsize(Sound,1) > 2)
				NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
				if(ChannelPair >= 1)
					SpHrView_SoundChannelRedimension(Play, ChannelPair)
				endif
			endif
			Wave Play=Play
		endif
		SpHr_RampOnOff 5, 5, Play
		NVAR normalizeSoundPlay = $SpHrView_df_globals+"normalizeSoundPlay"
		if(normalizeSoundPlay)
			Variable maxVal = WaveMax(Play)
			Play /= maxVal
		endif
		Playsound/A Play
		Killwaves/Z Play
	endif
end


// =================================================================================================
// SpHrView_SoundChannelPromptWindow(s), sound channel selection window
// =================================================================================================
function SpHrView_SoundChannelPromptWindow(numChannels, ChannelPair)
	Variable numChannels, ChannelPair
	if(numChannels<=2 || mod(numChannels,2) != 0)
		return 0
	endif	
	String TitleChStr="", PopUpStr=""
	if(numChannels==4)
		ChannelPair = min(ChannelPair,2)
		TitleChStr = "1-2  or  3-4"
		PopUpStr = "\"1 and 2;3 and 4\""
	elseif(numChannels==6)
		ChannelPair = min(ChannelPair,3)
		TitleChStr = "1-2,  3-4, or  5-6"
		PopUpStr = "\"1 and 2;3 and 4;5 and 6\""
	elseif(numChannels==8)
		TitleChStr = "1-2,  3-4,  5-6,  or  7-8"
		PopUpStr = "\"1 and 2;3 and 4;5 and 6;7 and 8\""
	endif
	
	killWindow/Z SoundChannelPromptWindow
	NewPanel/K=1/N=SoundChannelPromptWindow /W=(400,60,650,190) as "Select Channel pair" // do not use /FLT=1
	//ModifyPanel cbRGB=(65534,65534,65534)
	GroupBox group0,pos={10,5},size={232,45}
	TitleBox tb_message0 pos={0,10}, anchor=MC, fixedSize=1, size={250,13}, fSize=10, title="Select a pair of channels to load"
	TitleBox tb_message0 frame=0,fSize=10
	TitleBox tb_message1 pos={0,30}, anchor=MC, fixedSize=1, size={250,13}, fSize=10, title=TitleChStr
	TitleBox tb_message1 frame=0,fSize=10
	
	PopupMenu popup_ChannelPair title="Channel Pair:",pos={50,60},fSize=10,fstyle=1, mode=max(1,ChannelPair), value=#PopUpStr
	
	Button b_SoundChannelCancel pos={30,95},size={75,21},proc=SpHrView_SoundChannelPromptButtonProc, title="Cancel"
	Button b_SoundChannelOpen pos={145,95},size={80,21},proc=SpHrView_SoundChannelPromptButtonProc, title="Open File"//,focusRing=1, fstyle=1

	DoUpdate/W=SoundChannelPromptWindow // for Igor 9
	SetWindow SoundChannelPromptWindow,hook(SoundChannelPromptWindow)=SpHrView_SoundChannelPromptHook
	PauseForUser SoundChannelPromptWindow
end
// =================================================================================================
// SpHrView_SoundChannelPromptHook(s), hook function for SpHrView_SoundChannelPromptWindow window
// =================================================================================================
Function SpHrView_SoundChannelPromptHook(s)
	STRUCT WMWinHookStruct &s
	Variable hookResult = 0
	strswitch(s.winName)
		case "SoundChannelPromptWindow":
			switch(s.eventCode)
				case 2:	// kill
					ControlInfo/W=SoundChannelPromptWindow popup_ChannelPair
					if(abs(V_flag)==3)
						NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
						ChannelPair = V_Value
					endif 
					//DoWindow/K SoundChannelPromptWindow
					hookResult = 1 // 0?
					break
					
				case 11:	// keyboard
					if(s.keycode==13) // return
						ControlInfo/W=SoundChannelPromptWindow popup_ChannelPair 
						if(abs(V_flag)==3)
							NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
							ChannelPair = V_Value
						endif 
						DoWindow/K SoundChannelPromptWindow
						hookResult = 1
					endif
				break
			endswitch
			break
	endswitch
end
// =================================================================================================
// SpHrView_SoundChannelPromptButtonProc(ba), button proc for SpHrView_SoundChannelPromptWindow window
// =================================================================================================
Function SpHrView_SoundChannelPromptButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 1: // mouse down
			strswitch( ba.ctrlName )
				case "b_SoundChannelOpen":
					ControlInfo/W=SoundChannelPromptWindow popup_ChannelPair 
					if(abs(V_flag)==3)
						NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
						ChannelPair = V_Value
					endif 
					DoWindow/K SoundChannelPromptWindow
					break
					
				case "b_SoundChannelCancel":
					NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
					ChannelPair = 0
					DoWindow/K SoundChannelPromptWindow
					break	
			endswitch
		break
	endswitch
End
// =================================================================================================
// SpHrView_SoundChannelPromptPopMenuProc(w, ChannelPair), prompt for channel pair
// =================================================================================================
Function SpHrView_SoundChannelPromptPopMenuProc(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	switch( pa.eventCode )
		case 2: // mouse up
			Variable popNum = pa.popNum
			String popStr = pa.popStr
			strswitch(pa.win)
				case "SpHrView_SoundLoadViewWindow":
					NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
					ChannelPair = popNum
					ControlInfo/W=SpHrView_SoundLoadViewWindow sv_FilePathName
					String FilePathNameStr = S_Value
					SpHrView_SoundLengthPromptView(FilePathNameStr, -1, -1)
					SetAxis/W=SpHrView_SoundLoadViewWindow /A
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch
	return 0
End
// =================================================================================================
// SpHrView_SoundChannelRedimension(w, ChannelPair), redimension input to accomodate mono file 
//	or select channel pair 
// =================================================================================================
function SpHrView_SoundChannelRedimension(w, ChannelPair)
	wave w
	variable ChannelPair
	if(dimsize(w,1) <=2 || ChannelPair > 8)
		return 0
	endif
	//if(dimsize(w,1)<=1) // copy left channel to new right channel if mono
	//	redimension/N=(-1, 2) w
	//	w[][1] = w[p][0]
	//endif
	// Delete previous channels
	if(ChannelPair== 2 && dimsize(w,1)>=4)
		DeletePoints/M=1 0,2, w
	endif
	if(ChannelPair== 3 && dimsize(w,1)>=6)
		DeletePoints/M=1 0,4, w
	endif
	if(ChannelPair== 4 && dimsize(w,1)>=8)
		DeletePoints/M=1 0,6, w
	endif
	// remove latter channels
	if(dimsize(w,1)>=2)
		Redimension/N=(-1,2) w
	endif
	NVAR ChannelPairSelected = $SpHrView_df_globals+"ChannelPairSelected"
	ChannelPairSelected = 0
	return 0
end


// =================================================================================================
// SpHrView_FilePath_Set(w, FullIgorWavePathStr, FullFilePathNameStr), use wave Note to save
//	full Igor path or full file path & name
// =================================================================================================
function SpHrView_FilePath_Set(w, FullIgorWavePathStr, FullFilePathNameStr)
	wave w
	string FullIgorWavePathStr, FullFilePathNameStr
	if(!WaveExists(w))
		Print "Error setting Igor wave or file path. Wave is missing."
		return 0
	endif
	string noteStr= note(w)
	if(strlen(FullIgorWavePathStr)>1 && WaveExists($FullIgorWavePathStr)) 
		noteStr = ReplaceStringByKey("IGORWAVEPATHNAME", noteStr, FullIgorWavePathStr, ":",";",1)
	endif
	if(strlen(FullFilePathNameStr)>1)
		GetFileFolderInfo/Q/Z=1 FullFilePathNameStr // check for file
		killstrings/Z S_path, S_fileVersion, S_fileType, S_creator, S_aliasPath
		killvariables/Z V_version, V_modificationDate, V_logEOF, V_isStationery, V_isReadOnly, V_isInvisible, V_isFolder, V_isAliasShortcut, V_creationDate
		if(!V_Flag && V_isFile)
			noteStr = ReplaceStringByKey("WAVFILEPATHNAME", noteStr, FullFilePathNameStr, ":",";",1)
		endif
		killvariables/Z V_Flag, V_isFile
	endif
	//Note/K w
	Note/NOCR w, noteStr
end

// =================================================================================================
// SpHrView_FilePath_Get(w, pathType), use wave note() to get full Igor path (if it exists)
// or full file path & name
// pathType == 0: Igor wave path if present or File path & name if Igor wave path is missing
// pathType == 1: Igor wave path
// pathType == 2: File path & name
// =================================================================================================
function/S SpHrView_FilePath_Get(w, pathType)
	wave w
	variable pathType 
	string noteStr= note(w)

	String str = ""
	str = StringByKey("IGORWAVEPATHNAME", noteStr, ":",";",1)
	if(strlen(str)>1)
		if(pathType<=0 || pathType==1)
			return str // return Igor Wave Path if pathType <= 0
		endif
	endif
	str = StringByKey("WAVFILEPATHNAME", noteStr, ":",";",1)
	if(strlen(str)>1)
		if(pathType<=0  || pathType==2)
			return str	// return File Path & Name if Igor Wave Path is missing (or pathType==2)
		endif
	endif
	return ""
end



// =================================================================================================
// SpHrView_ButtonProc, Button procedure for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	String PopUpListStr, NameStr="", dfStr, traceListStr, wStr
	variable minX, maxX, rangeX, keyPressed, i
	
	strswitch( ba.ctrlName )

//		case "b_HilbertEnvHz":
//			switch( ba.eventCode )
//				case 1: // mouse down
//					getAnalysisFreqPopUp(1,0)
//					NVAR analysisFreq= $SpHrView_df_filter+"analysisFreq"
//					FilterBanks_Update(1)
//					break
//			endswitch
//			break
				
		case "b_waveformFilterType":
			switch( ba.eventCode )
				case 1: // mouse down
				DoAlert 0, "rewrite WaveformFilterMenu()"
				//WaveformFilterMenu()
				break
			endswitch
			break
			
		case "b_drawMarquee":
			switch( ba.eventCode )
				case 2: // mouse up
					GetMarquee/W=SpHrViewPeaksWindow/Z bottom
					if(V_flag)
						SpHrView_DrawSelection(1, v_left, v_right) // draw rectangle using Marquee coordinates
						GetMarquee/K

					else
						keyPressed = GetKeyState(0)
						DrawAction/W=SpHrViewPeaksWindow/L=UserBack extractOutline
						wave/Z W_PolyX=W_PolyX, W_PolyY=W_PolyY
						// draw rectangle before cursor if command, option, or control are pressed and cursor exists
						// or if no rectangle exists
						if(!waveexists(W_PolyX) || dimsize(W_PolyX,0)!=5 || keyPressed == 1 || keyPressed == 2 || keyPressed == 16) // object exists and is a square/rectangle
							if(strlen(CsrWave(A, "SpHrViewPeaksWindow"))>0)
								variable xval = hcsr(A, "SpHrViewPeaksWindow")
								Variable xrange = 5
								//GetAxis/W=SpHrViewPeaksWindow/Q bottom
								//if(!V_flag)
									//xrange = ((V_max-V_min)/4)*1000 	// 1/4th the length of bottom axis
									//xrange = round(xrange)/1000
									//xrange = min(xrange,1)				// maximum of 1 sec.
								//endif
								NVAR useDrawSelection= $SpHrView_df_globals+"useDrawSelection"
								variable SelectionTyep
								if(useDrawSelection)
									SelectionTyep = 1
								else
									SelectionTyep = 2
								endif
								Prompt xval,"Cursor time:"
								Prompt xrange, "Time frame before cursor (sec.): "
								//Prompt SelectionTyep,"Use draw layer selection",popup,"Yes;No"	
								Prompt SelectionTyep,"Set",popup,"Draw Layer;Marquee;Draw layer and Marquee"
								DoPrompt "Time frame", xval, xrange, SelectionTyep
								if (V_Flag)
									return -1								// User canceled
								else
						
									// update cursor location
									String ACursorStr = CsrWave(A, "SpHrViewPeaksWindow")
									// use *_snip wave if displayed
									traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
									for(i=0;i<itemsinlist(traceListStr);++i)
										wStr =  StringFromList(i,traceListStr)
										if(strsearch(wStr, "_snip", 0)>1)
											ACursorStr = wStr
											break
										endif 
									endfor
									Cursor/S=2/L=0/H=2/C=(0,65535/2,0, 32767 )/W=SpHrViewPeaksWindow A, $ACursorStr, xval
									DoUpdate/W=SpHrViewPeaksWindow
									
									if(SelectionTyep == 1 || SelectionTyep == 3)
										useDrawSelection = 1
										SpHrView_DrawSelection(1, xval - xrange, xval)
									endif
									if(SelectionTyep == 2 || SelectionTyep == 3)
										useDrawSelection = 0
										GetWindow/Z SpHrViewPeaksWindow  gsizeDC
										//Printf "%g, %g\r", v_top, v_bottom 
										SetMarquee/W=SpHrViewPeaksWindow/HAX=bottom xval - xrange, v_top+55, xval, v_bottom
									endif
									//shift bottom axis range if not shown
									GetAxis /W=SpHrViewPeaksWindow/Q bottom
									if(xval < v_min)
										SetAxis bottom xval-xrange, xval-xrange+(v_max-v_min)
									endif
									if(xval > v_max)
										SetAxis bottom xval+xrange-(v_max-v_min), xval+xrange
									endif
									
								endif
							endif	
							return 0
						endif
						// clear draw rectangle if no Marquee and command, option, or control are not pressed
						SpHrView_DrawSelection(0, 0, 0)
					endif
					killwaves/Z W_PolyX, W_PolyY
			endswitch
			break
			
		case "b_copyBottomAxis":
			switch( ba.eventCode )
				case 1: // mouse down
					CopyPaste_Bottom_AxisRange("SpHrViewPeaksWindow", 0)	// IgorGeneralProcedures
			endswitch
			break
		case "b_pasteBottomAxis":
			switch( ba.eventCode )
				case 1: // mouse down
					CopyPaste_Bottom_AxisRange("SpHrViewPeaksWindow", 1)	// IgorGeneralProcedures
			endswitch
			break
				
		case "b_SpHrView_NewFile":
			switch( ba.eventCode )
				case 1: // mouse down
					//SpHrView_DrawSelection(0, 0, 0)
					keyPressed = GetKeyState(0)
					if(keyPressed == 1 || keyPressed == 2 || keyPressed == 16) // command, option, or control
						SpHrView_OpenFile(1)
					else
						SpHrView_OpenFile(0)
					endif
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "b_SpHrView_Play":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_PlaySound()
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "b_SpHrView_StopPlay":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_DrawSelection(0, 0, 0)
					SpHrView_PlaySoundTaskStop()
					StopPlaySound()		// IgorGeneralProcedures
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "b_SpHrView_View":
			switch( ba.eventCode )
				case 1: // mouse down
					SpHrView_ViewMenu()
					break
				case -1: // control being killed
					break
			endswitch
			break
			
							
		case "b_SpHrViewXZoom":
			switch( ba.eventCode )
				case 1: // mouse down
					//SpHrView_DrawSelection(0, 0, 0)
					//variable minX, maxX, rangeX
					GetMarquee/W=SpHrViewPeaksWindow/Z bottom
					if(V_flag)
						SetAxis bottom v_left, v_right
						GetMarquee/K
						//SpHrView_AxisTimeCheck()
					else
						GetAxis/W=SpHrViewPeaksWindow/Q bottom
						minX= v_min; maxX= v_max; rangeX= v_max-v_min
						keyPressed = GetKeyState(0)
						// contract if command, option, or control pressed
						if(keyPressed == 1 || keyPressed == 2 || keyPressed == 16) // command, option, or control
							minX -= rangeX/2//4
							maxX += rangeX/2//4
						else // zoom if keys not pressed
							minX += rangeX/4
							maxX -= rangeX/4
						endif
						SetAxis/W=SpHrViewPeaksWindow bottom minX, maxX
						//SpHrView_AxisTimeCheck()
					endif
				break
			endswitch
			break
		case "b_SpHrViewXAuto":
			switch( ba.eventCode )
				//case 1: // mouse down
				case 2: // mouse up
					//SpHrView_DrawSelection(0, 0, 0)
					SetAxis/W=SpHrViewPeaksWindow/A/E=0 bottom
					Button b_SpHrViewXAuto win=SpHrViewPeaksWindow#PTop, disable=1
					DoUpdate/W=SpHrViewPeaksWindow
					//SpHrView_AxisTimeCheck()
				break
			endswitch
			break
			
		case "b_SpHrViewYZoom":
			switch( ba.eventCode )
				case 1: // mouse down
					//SpHrView_DrawSelection(0, 0, 0)
					GetMarquee/W=SpHrViewPeaksWindow/Z left
					if(V_flag)
						SetAxis left V_bottom, V_top
						GetMarquee/K
					else
						GetAxis/W=SpHrViewPeaksWindow/Q left
						minX= v_min; maxX= v_max; rangeX= v_max-v_min // using X for Y...
						keyPressed = GetKeyState(0)
						// contract if command, option, or control pressed
						NVAR dBscale= $SpHrView_df_globals+"dBscale"
						if(keyPressed == 1 || keyPressed == 2 || keyPressed == 16) // command, option, or control
							if(dBscale)
								SetAxis/A=2 left 20,*
							else
								SetAxis/A=2/E=1 left 0,*
							endif
						else // zoom if modifier keys not pressed (see above)
							if(dBscale)
								if(maxX-minX > 6)								// min axis range is 6 dB
									SetAxis left *,maxX/1.1					// leave lower scale alone
									//SetAxis left minX*1.05,maxX/1.1
								endif
							else
								SetAxis left 0, maxX/1.5						// leave lower scale alone
							endif
						endif
						//NVAR autoscaleSnippetYaxis= $SpHrView_df_globals+"autoscaleSnippetYaxis"
						//SpHrView_AxisTimeCheck()
					endif
				break
			endswitch
			break
			
		case "b_openMovie":
			switch( ba.eventCode )
				case 1: // mouse down
					keyPressed = GetKeyState(0)
					// open full movie in window if command, option, or control pressed
					if(keyPressed == 1 || keyPressed == 2 || keyPressed == 16)
						SpHrView_MoviePlayer("")
					else
						DoWindow SpHrViewMovieWindow
						if(V_flag)
							DoWindow/F SpHrViewMovieWindow
							SpHrView_MovieExtract()
						else
							SpHrView_MovieOpen("")
						endif
					endif
					break
			endswitch
			break
			
		case "b_numChs":
			switch( ba.eventCode )
				case 1: // mouse down
					NVAR numChannels= $SpHrView_df_globals+"numChannels"
					NVAR ChannelPair = $SpHrView_df_globals+"ChannelPair"
					String PopUpChStr="1-2;"
					if(numChannels>2)
						PopUpChStr+="3-4;"
					endif
					if(numChannels>4)
						PopUpChStr+="5-6;"
					endif
					if(numChannels>6)
						PopUpChStr+="7-8;"
					endif
					PopupContextualMenu PopUpChStr
					if(V_flag>0)
						ChannelPair = V_flag
						String ChStr = "Chs\r"
						Switch(ChannelPair)
							case 1:
								ChStr += "1-2"
								break
							case 2:
								ChStr += "3-4"
								break
							case 3:
								ChStr += "5-6"
								break
							case 4:
								ChStr += "7-8"
								break
						endswitch
						TitleBox title_Ch win=SpHrViewPeaksWindow#PTop, title=ChStr
						// re-load sound file
						NVAR file_start_min= $SpHrView_df_globals+"file_start_min"
						NVAR file_end_min= $SpHrView_df_globals+"file_end_min"
				
						traceListStr= TraceNameList("SpHrViewPeaksWindow", ";", 1)
						wStr =  StringFromList(0,traceListStr)
						wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", wStr)
						// Get file path
						string pathAndNameStr= SpHrView_FilePath_Get(w, 2) //note(w)
						
						NVAR ChannelPairSelected = $SpHrView_df_globals+"ChannelPairSelected"
						ChannelPairSelected = 1
						
						KillWindow/Z SpHrViewPeaksWindow
						SpHrView_LoadPeaksFromFile(pathAndNameStr, file_start_min*60, file_end_min*60, 0, 1, 0, 0)
						
					endif
					break
			endswitch
			break
			
		case "b_SpHrView_Settings":
			switch( ba.eventCode )
				case 2: // mouse up
					SpHrView_SettingsPopup()
					break
				case -1: // control being killed
					break
			endswitch
			break
									
	endswitch
	return 0
End

// =================================================================================================
// SpHrView_CheckProc(cba), Checkbox procedure for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	Variable checked = cba.checked
	string traceListStr="", wStr=""
	strswitch( cba.ctrlName )
		case "ck_waveformFilter":
		case "ck_outOfPhase":
			NVAR filterbankUpdateFlag = $SpHrView_df_filter+"filterbankUpdateFlag"
			filterbankUpdateFlag = 1
			DoWindow FilterBank_Window
			if(v_flag)
				SpHrView_DisplayFilteredWaves(-1) // update
			endif
			DoWindow Waveforms_Window
			if(v_flag)
				SpHrView_DisplayCarrierWaves(-1, -1)
			endif
			DoWindow Spectra_Window
			if(v_flag)
				SpHrView_DisplaySpectra()
			endif
			DoWindow STFT_Window
			if(v_flag)
				SpHrView_DisplaySTFT()
			endif
			break
			
		case "ck_overlapWaveforms":
			switch( cba.eventCode )
				case 2: // mouse up
					SpHrView_DisplayCarrierWaves(0, checked)
				case -1: // control being killed
					break
			endswitch
			break
		case "ck_overlapEnvelopes":
			switch( cba.eventCode )
				case 2: // mouse up
					SpHrView_DisplayCarrierWaves(1, checked)
				case -1: // control being killed
					break
			endswitch
			break
		
		case "ck_useDrawSelection":
			switch( cba.eventCode )
				case 2: // mouse up
					NVAR useDrawSelection= $SpHrView_df_globals+"useDrawSelection"
					dowindow SpHrViewPeaksWindow
					if(v_flag)
						if(useDrawSelection)
							Button b_drawMarquee win=SpHrViewPeaksWindow#PTop, disable=0
						else
							Button b_drawMarquee win=SpHrViewPeaksWindow#PTop, disable=2
						endif
					endif
				case -1: // control being killed
					break
			endswitch
			break
			
		case "ck_autoscaleY":
			switch( cba.eventCode )
				case 2: // mouse up
					SpHrView_AxisAmpCheck()
				case -1: // control being killed
					break
			endswitch
			break
			
		case "ck_SpHrView_dBScale":
			switch( cba.eventCode )
				case 2: // mouse up
					SpHrView_PeaksWindowReopen()
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "ck_ApplySPLScale":
			switch( cba.eventCode )
				case 2: // mouse up
					SpHrView_PeaksWindowReopen()
					break
				case -1: // control being killed
					break
			endswitch
			break
	endswitch
	return 0
End

// =================================================================================================
// SpHrView_SetVarProc(sva), SetVariable procedure for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	Variable dval = sva.dval
	//String sval = sva.sval
	variable analysisIndex
	strswitch( sva.ctrlName )
		case "sv_waveName":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
				case 3: // Live update
					//Print sval
					SVAR waveNameStr= $SpHrView_df_globals+"waveNameStr"
					waveNameStr = cleanupname(waveNameStr,0)
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "sv_MovieHeight":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
				case 3: // Live update
					killwindow/Z SpHrViewMovieWindow
					SpHrView_MovieOpen("")
					break
				case -1: // control being killed
					break
			endswitch
			break
			
		case "sv_SPLscale":
			switch( sva.eventCode )
				case 1: // mouse up
				case 2: // Enter key
				case 3: // Live update
				
					// HELL
					NVAR SPLRefenceScale = $SpHrView_df_globals+"SPLRefenceScale"
					NVAR ApplySPLScale = $SpHrView_df_globals+"ApplySPLScale"
					if(ApplySPLScale)
						DoAlert/T="Warning" 0, "De-select SPL checkbox and re-open reference signal file (tone)."
						return 0
					endif
					
					variable startSec, endSec, usingAxis
					[startSec, endSec, usingAxis] = SpHrView_LoadWaveformsFromPeaks()
					wave waveforms=$SpHrView_df_files+"waveforms"
					if(!waveexists(waveforms))
						DoAlert 0, "SpHrView_SettingsPopup: Set reference level - Waveforms are missing"
						return 0
					endif
					
					//display waveforms

					variable Level=1, refVoltage=10/1000, refVoltageType=2
					variable  microphoneSensitivity = 20
					//display waveforms
					if(dimsize(waveforms,1) > 2 )
						DoAlert 0, "SpHrView_SettingsPopup: Set reference level - Expecting 1 or 2 channels"
						return 0
					elseif(dimsize(waveforms,1) == 2 )
						SpHr_Stats/RMS/W=W_Stats waveforms
						variable leftLevel = W_Stats[4][0]//*sqrt(2) // estimate peak from rms
						variable rightLevel = W_Stats[4][1]//*sqrt(2) // estimate peak from rms
						if(leftLevel>rightLevel)
							Level = 1
						else
							Level = 2
						endif
						Prompt Level,"Recorded signal amplitude", popup "Left channel: "+num2str(leftLevel)+" RMS (~"+num2str(leftLevel*sqrt(2))+" Peaks);"+"Right channel: "+num2str(rightLevel)+" RMS ("+num2str(rightLevel*sqrt(2))+" ~Peak);"
					else
						SpHr_Stats/RMS/W=W_Stats waveforms
						Level =  W_Stats[4]
						Prompt Level,"Recorded signal amplitude"
					endif
										
					//Prompt refVoltage,"Reference voltage: 1 V (1 Pa);  0.01 V (10 mV, -40 dB)"
					Prompt refVoltageType,"Reference signal (tone)", popup, "Acoustic: 1 V/Pa (94 dB re:20 µPa);Electric: 10 mV (-40 dB);Other reference"
					Prompt microphoneSensitivity, "Microphone sensitivity mV at 1 Pa (94 dB re:20 µPa) (if using Electric reference)"
					DoPrompt/HELP="Select reference signal (tone) using the marquee and return to set new scaling factor." "Set reference level", Level, refVoltageType, microphoneSensitivity
					if(!V_Flag)
						switch(refVoltageType)
							case 1: 
								refVoltage = 1
								break
							case 2:
								refVoltage = 10/1000
								break
							case 3:
								variable  SelectedVoltage
								Prompt SelectedVoltage, "Select reference level"
								DoPrompt "Reference level", SelectedVoltage
								if(!V_Flag)
									refVoltage = SelectedVoltage
								else 
									refVoltage = 1
								endif
								break
						endswitch
					
						if(dimsize(waveforms,1) == 2 )
							switch(Level)
								case 1: 
									Level = leftLevel
									break
								case 2: 
									Level = rightLevel
									break
							endswitch
						endif
						
						if(refVoltageType!=1)
							SPLRefenceScale = (refVoltage/Level) * ((microphoneSensitivity/1000)/refVoltage)
						else
							SPLRefenceScale = refVoltage/Level
						endif
				
						switch(refVoltageType)
							case 1: 
								Printf "Scale = %g (%g dB), reference = %g V (at 1 Pa (93.98 dB re:20 µ_Pa), tone level = %g V\r", SPLRefenceScale, 20*log(SPLRefenceScale), refVoltage, microphoneSensitivity, Level
								break
							case 2:
							case 3:
								Printf "Scale = %g (%g dB), reference = %g V, microphone sensitivity = %g mV at 1 Pa (93.98 dB re:20 µ_Pa), tone level = %g V\r", SPLRefenceScale, 20*log(SPLRefenceScale), refVoltage, microphoneSensitivity, Level
								break
						endswitch
					endif
					Killwaves/Z waveforms 
					Killwaves/Z W_Stats
					
					break
				case -1: // control being killed
					break
			endswitch
			break
			
	endswitch
	return 0
End



// =================================================================================================
// SpHrView_SettingsPopup(), settings button activated menu for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_SettingsPopup()

	PopupContextualMenu "View;\M1-;Filter Bank;\M1-;IPD Neurons;ILD Neurons;" // referenced at top of function
	string infostr
	strswitch(S_selection)
		case "View":
			SpHrView_Settings(0,0)
			break
		case "FilterBank":
		case "Filter Bank":
			if(datafolderExists("root:SpHrView:Filter:"))		// datafolder exists
				infostr = FunctionInfo("SpHrView_Settings_FilterBank") // function SpHrView_Settings_FilterBank exists
				if (strlen(infostr) > 0)
					Execute/Q "SpHrView_Settings_FilterBank(0,0)"
				endif
			endif
			break
			
		case "IPD Neurons":
			SpHrView_IPDNeuron_OptionsWindow(0,0)
			break
			
		case "ILD Neurons":
			SpHrView_ILDNeuron_OptionsWindow(0,0)
			break
			
			
//		case "Cue Histograms":
//			if(datafolderExists("root:SpHrView:Histograms:"))		// datafolder exists
//				infostr = FunctionInfo("SpHrView_Settings_CueHistogram") // function SpHrView_Settings_FilterBank exists
//				if (strlen(infostr) > 0)
//					Execute/Q "SpHrView_Settings_CueHistogram()"
//				endif
//			endif
//			break
			
//		case "Spectra":
//			SpHrView_Settings_Spectra()
//			break
			
		default: // Executes function named by S_selection
			string functionNameStr= S_selection
			functionNameStr = cleanupname(functionNameStr,0)
			infostr = FunctionInfo(functionNameStr)
			if (strlen(infostr) > 0)
				Execute/Q/Z functionNameStr+"()"
				break
			endif
			break
	endswitch
end

// =================================================================================================
// SpHrView_Settings(), settings panel for SpHrViewPeaksWindow
// =================================================================================================
Function SpHrView_Settings(variable winLeft, variable winTop)
	if(winLeft == 0 || winTop == 0)
		GetMouse
		winLeft=V_left; winTop=V_top
	endif
	killwindow/Z SpHrViewSettings
	SpHrView_Defaults()
	NewPanel /N=SpHrViewSettings/K=1/W=(winLeft, winTop, winLeft+180,winTop+290) as "View"
	variable topPos = 5
	GroupBox group1,pos={7.00,topPos},size={167.00,70.00},fSize=10,title="Window size (re: screen)"
	topPos += 22
	SetVariable sv_ViewWidthProportion,pos={20.00,topPos},size={139,14.00},bodyWidth=50,fSize=10,title="Width proportion"
	SetVariable sv_ViewWidthProportion,limits={0.05,1,0.05},value= $SpHrView_df_globals+"ViewWidthProportion"
	SetVariable sv_ViewWidthProportion,help={"Display width relative to screen width"}
	topPos += 23
	SetVariable sv_ViewHeightProportion,pos={20.00,topPos},size={139,14.00},bodyWidth=50,fSize=10,title="Height proportion"
	SetVariable sv_ViewHeightProportion,limits={0.05,1,0.05},value= $SpHrView_df_globals+"ViewHeightProportion"
	SetVariable sv_ViewHeightProportion,help={"Display height relative to screen height"}
	topPos += 32
	GroupBox group0,pos={7.00,topPos-5},size={167.00,28}
	CheckBox ck_LoadAsIgorWaveForSpeed,pos={25.00,topPos},size={100.00,16.00},fSize=10,title="Load files to waves"
	CheckBox ck_LoadAsIgorWaveForSpeed,variable= $SpHrView_df_globals+"LoadAsIgorWaveForSpeed",side= 0
	CheckBox ck_LoadAsIgorWaveForSpeed,help={"Load files to Igor waves to improve access times (esp. for movies)"}
	topPos += 30
	GroupBox group2,pos={7.00,topPos-6},size={167.00,28.00}
	SetVariable sv_NumberViewPeaks,pos={20.00,topPos},size={144.00,14.00},bodyWidth=55,fSize=10,title="# View points"
	SetVariable sv_NumberViewPeaks,limits={500,100000,100},value= $SpHrView_df_globals+"NumberViewPeaks"
	SetVariable sv_NumberViewPeaks,help={"Number of peaks to display"}
	topPos += 30
	GroupBox group3,pos={7.00,topPos-6},size={167.00,28.00}
	SetVariable sv_loadPromptLength,pos={35.00,topPos},size={121.00,14.00},bodyWidth=50,fSize=10,title="File minutes \\Z11⚠️"
	SetVariable sv_loadPromptLength,limits={500,100000,100},value= $SpHrView_df_globals+"loadPromptLength"
	SetVariable sv_loadPromptLength,help={"Longer files prompt a warning/options window when opened"}
	topPos += 30
	GroupBox group4,pos={7.00,topPos-6},size={167.00,28.00}
	SetVariable sv_MovieHeight,pos={40.00,topPos},size={116.00,14.00},bodyWidth=50,fSize=10,title="Movie height"
	SetVariable sv_MovieHeight,limits={108,2160,50},value= $SpHrView_df_globals+"MovieHeight", proc=SpHrViewSetVarProc
	SetVariable sv_MovieHeight,help={"Height of movie in points (when analyzing a movie file)"}
	topPos += 30
	GroupBox group5,pos={7.00,topPos-5},size={167.00,28.00}
	CheckBox ck_useDrawSelection,pos={20.00,topPos},size={126.00,16.00},fSize=10,title="Use draw layer selection "
	CheckBox ck_useDrawSelection,variable= $SpHrView_df_globals+"useDrawSelection",side= 0, proc=SpHrView_CheckProc
	CheckBox ck_useDrawSelection,help={"Use drawing layer to make sound selections."}
	topPos += 30
	GroupBox group6,pos={7.00,topPos-5},size={167.00,28.00}
	CheckBox ck_normalizePlaying,pos={20.00,topPos},size={126.00,16.00},fSize=10,title="Normalize when playing "
	CheckBox ck_normalizePlaying,variable= $SpHrView_df_globals+"normalizeSoundPlay",side= 0
	CheckBox ck_normalizePlaying,help={"Normalize re: maximum amplitude when playing sound"}
	
	topPos += 32
	CheckBox ck_keepWindowOpen win=SpHrViewSettings, title="📌 ",fSize=10,pos={30,topPos},side=0
	CheckBox ck_keepWindowOpen win=SpHrViewSettings, help={"Keep window open"}
	
	// window hook
	DoUpdate/W=SpHrViewSettings // for Igor 9
	SetWindow SpHrViewSettings, hook(SpHrViewSettings)=SpHrView_PeaksWindowHook
End


// =================================================================================================
// SpHrView_CountGroupWindows(), Count number of window open in PACKAGE:SpHrView && GROUP:groupStr
// =================================================================================================
function SpHrView_CountGroupWindows(String groupStr)
	String str, GraphNameList = WinList("*", ";","")
	variable i, cnt=0
	for(i=0;i<itemsinlist(GraphNameList);++i)
		str =stringfromlist(i, GraphNameList)
		GetWindow/Z $str note 
		if(strsearch(S_Value, "PACKAGE:SpHrView", 0)>=0)
			if(stringmatch(groupStr, StringByKey("GROUP", S_Value)))
				//Print str
				cnt += 1
			endif
		endif
	endfor
	return cnt
End

function/T SpHrView_GroupWindowsList(String groupStr)
	String str, GraphNameList = WinList("*", ";","WIN:1")
	String listStr = ""
	variable i, cnt=0
	for(i=0;i<itemsinlist(GraphNameList);++i)
		str =stringfromlist(i, GraphNameList)
		GetWindow/Z $str note 
		if(strsearch(S_Value, "PACKAGE:SpHrView", 0)>=0)
			if(stringmatch(groupStr, StringByKey("GROUP", S_Value)))
				listStr += str+";"
			endif
		endif
	endfor
	return listStr
End


function SpHrView_getWindowSlot(String groupStr, variable numWindowSlots)
	String str, GraphNameList = WinList("*", ";","WIN:1")
	String listStr = ""
	variable i, cnt=0
	for(i=0;i<itemsinlist(GraphNameList);++i)
		str =stringfromlist(i, GraphNameList)
		GetWindow/Z $str note 
		if(strsearch(S_Value, "PACKAGE:SpHrView", 0)>=0)
			if(stringmatch(groupStr, StringByKey("GROUP", S_Value)))
				listStr += str+";"
			endif
		endif
	endfor
	variable leftRectPnt, rightRectPnt, width
	DoWindow SpHrViewPeaksWindow
	if(V_flag)
		GetWindow/Z SpHrViewPeaksWindow wsize // use SpHrViewPeaksWindow wsize
		leftRectPnt = V_left
		rightRectPnt = V_right
		width = rightRectPnt - leftRectPnt
	else
		return 0
	endif
	variable slotWidth = width/numWindowSlots
	Make/O/N=(numWindowSlots*3)/FREE windowSlots
	windowSlots=NaN
	String WinStr
	variable windowCnt = 0, firstSlot//, returnSign=1
	for(i=0;i<ItemsInList(listStr);++i)
		WinStr = StringFromList(i,listStr)
		DoWindow $WinStr
		if(v_flag)
			GetWindow/Z $WinStr wsize
			windowSlots[windowCnt] = abs(round((v_left-leftRectPnt)/slotWidth))
			windowCnt+=1
		endif
	endfor
	if(numtype(wavemax(windowSlots))!=0) // no windows
		return 0
	endif
	if(windowCnt >= numWindowSlots)		// wrap back to zero
		//returnSign = -1
		do												
			windowCnt -= numWindowSlots		// could drop wrapped windows...
		while(windowCnt >= numWindowSlots)
	endif
	sort windowSlots, windowSlots
	if(windowSlots[0]>0)						// first slot is open ??
		return 0
	endif
	windowSlots -= x
	for(i=0;i<numpnts(windowSlots);++i)	// find empty slot
		if(windowSlots[i]>0)
			return i//*returnSign
		endif
	endfor
	return windowCnt//*returnSign			// next slot
end


// =================================================================================================
// SpHrView_DrawSelection(mode, val_1, val_2), sets draw layer of SpHrViewPeaksWindow to mark sound selections
// alternative to the GraphMarquee
// mode == 0, kills the layer
// mode == 1, draws the selection between val_1 and val_2
// mode == 2, shade window to indicate that min and max axis values are used
// =================================================================================================
function SpHrView_DrawSelection(mode, val_1, val_2)
	variable mode, val_1, val_2
	dowindow SpHrViewPeaksWindow
	if(V_flag)
		NVAR useDrawSelection= $SpHrView_df_globals+"useDrawSelection"
		if(mode==1 && useDrawSelection)
			// draw box using bottom axis in UserBack
			SetDrawLayer/W=SpHrViewPeaksWindow/K UserBack
			SetDrawEnv/W=SpHrViewPeaksWindow xcoord= bottom,fillfgc= (0,52428,0,6554),linefgc= (0,32767,0,4915), linethick= 0.0
			DrawRect/W=SpHrViewPeaksWindow val_1, 0, val_2, 1
			//SetDrawEnv/W=SpHrViewPeaksWindow gstop
		elseif(mode==2 && useDrawSelection)
			GetAxis /W=SpHrViewPeaksWindow/Q bottom
//			startSec = v_min
//			endSec = v_max
			SetDrawLayer/W=SpHrViewPeaksWindow/K UserBack
			SetDrawEnv/W=SpHrViewPeaksWindow xcoord= bottom,fillfgc= (0,52428,0,4915),linefgc= (0,32767,0,4915), linethick= 0.0
			DrawRect/W=SpHrViewPeaksWindow v_min, 0, v_max, 1
		else
			// clear UserBack
			SetDrawLayer/W=SpHrViewPeaksWindow/K UserBack
		endif
	endif
end


// =================================================================================================
// SpHrView_MovieOpen(filePathAndName), Open a movie file
// =================================================================================================
function SpHrView_MovieOpen(filePathAndName)
	string filePathAndName
	NVAR isMovie=$SpHrView_df_globals+"isMovie"
	if(!isMovie)
		return 0
	endif
	if(strlen(filePathAndName)<1)
		String traceListStr = TraceNameList("SpHrViewPeaksWindow", ";", 1 )
		wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", stringfromlist(0, traceListStr)) // get wave ref
		filePathAndName= SpHrView_FilePath_Get(w, 2) //note(w)	 // get path to FILE from note
	endif
	DFREF saveDFR = GetDataFolderDFR()	
	SetDataFolder $SpHrView_df_movies
	PlayMovieAction loop=1, open = filePathAndName // ref=V_Value
	if(v_flag!=0)	// error
		isMovie = 0
		return 0
	endif
	Variable/G $SpHrView_df_movies+"movieFileRef"
	NVAR movieFileRef = $SpHrView_df_movies+"movieFileRef"
	movieFileRef = V_Value
	SpHrView_MovieVariables()
	SetDataFolder saveDFR
	SpHrView_MovieWindow()
End

// =================================================================================================
// SpHrView_MovieVariables(), calculates NumFrames, FrameLength, and FrameRateDelta
// =================================================================================================
function SpHrView_MovieVariables()
	NVAR movieFileRef = $SpHrView_df_movies+"movieFileRef" // created by SpHrView_MovieOpen()
	PlayMovieAction setFrontMovie = movieFileRef
	wave frame = $SpHrView_df_movies+"M_MovieFrame"
	PlayMovieAction stop, gotoEnd, getTime
	Variable tEnd = V_Value
	PlayMovieAction step=-1, getTime
	Variable/G $SpHrView_df_movies+"NumFrames"= tEnd/(tEnd-V_value)
	NVAR NumFrames=$SpHrView_df_movies+"NumFrames"
	Variable/G $SpHrView_df_movies+"FrameLength" = (tEnd-V_value)
	PlayMovieAction stop, gotoBeginning, getTime
	Variable tStart = V_Value
	Variable/G $SpHrView_df_movies+"FrameRateDelta" = (tEnd-tStart)/NumFrames
end
		
// =================================================================================================
// SpHrView_MovieWindow(), Display extracted image in window
// =================================================================================================
function SpHrView_MovieWindow()
	NVAR NumFrames=$SpHrView_df_movies+"NumFrames"
	NVAR FrameLength=$SpHrView_df_movies+"FrameLength"
	NVAR movieFileRef = $SpHrView_df_movies+"movieFileRef"
	SetDataFolder $SpHrView_df_movies
	PlayMovieAction setFrontMovie = movieFileRef
	PlayMovieAction extract
	wave frame = $SpHrView_df_movies+"M_MovieFrame"
	variable AspectRatio = (dimsize(frame,0)/dimsize(frame,1))
	//variable numScreens = str2num(StringByKey("NSCREENS", IgorInfo(0)))	// Number of active displays
	NVAR MovieHeight = $SpHrView_df_globals+"MovieHeight"
	variable leftRectPnt, rightRectPnt, topRectPnt, bottomRectPnt
	DoWindow SpHrViewMovieWindow
	if(V_flag)
		GetWindow/Z SpHrViewMovieWindow wsize // use SpHrViewPeaksWindow wsize
		leftRectPnt = V_left; topRectPnt = V_top
		rightRectPnt = V_right; bottomRectPnt = V_bottom
	else
		variable width = ((MovieHeight-18)*AspectRatio)
		DoWindow SpHrViewPeaksWindow
		if(V_flag)
			GetWindow/Z SpHrViewPeaksWindow wsize // use SpHrViewPeaksWindow wsize
			leftRectPnt = V_left+((V_right-V_left)/2) - (width/2) //center
			topRectPnt = V_bottom+30
			rightRectPnt = leftRectPnt + width
			bottomRectPnt = topRectPnt + MovieHeight
		else
			leftRectPnt = 200
			topRectPnt = 200
			rightRectPnt = leftRectPnt + width
			bottomRectPnt = topRectPnt + MovieHeight
		endif
	endif
	DoWindow/K SpHrViewMovieWindow
	Display /K=1/W=(leftRectPnt, topRectPnt, rightRectPnt, bottomRectPnt) as ""
	//Display /K=1 as ""
	DoWindow/C SpHrViewMovieWindow
//	ControlBar/T 37
//	NewPanel/FG=(FL,FT,GR,GT)/HOST=# 
//	RenameWindow #,PTop // SpHrViewPeaksWindow#PTop
//	SetActiveSubwindow SpHrViewMovieWindow //##
	ControlBar/T 22
	Slider sl_MovieControl win=SpHrViewMovieWindow, pos={8,5},size={(rightRectPnt-leftRectPnt)-100,20},vert=0,side=0,limits={0,NumFrames,1},ticks=0
	Slider sl_MovieControl win=SpHrViewMovieWindow, proc=SpHrView_MovieSliderProc, help={"Movie frame time. Use left and right arrow keys to step frqme-by-frame."}
	
	SetVariable sv_MovieHeight,pos={width-65,2},size={60.00,14.00},bodyWidth=40,fSize=9,title="\\Z09🖥  \\Z12↕︎"
	SetVariable sv_MovieHeight,limits={150,4320,25},value= $SpHrView_df_globals+"MovieHeight", proc=SpHrViewSetVarProc
	SetVariable sv_MovieHeight,valueBackColor=(61166,61166,61166),help={"Height of window in points"}
	
	appendimage/w=SpHrViewMovieWindow frame
	ModifyGraph/w=SpHrViewMovieWindow margin=1
	ModifyGraph/w=SpHrViewMovieWindow tick=3,noLabel=2,axThick=0,standoff=0
	SetAxis/w=SpHrViewMovieWindow/A/R left
	ModifyGraph/w=SpHrViewMovieWindow gbRGB=(32767,32767,32767), wbRGB=(32767,32767,32767), cbRGB=(56987,56987,56987)//(65535,65535,65535)
	ModifyGraph/w=SpHrViewMovieWindow height={Aspect,1/AspectRatio}
	DoUpdate/W=SpHrViewMovieWindow // for Igor 9
	SetWindow SpHrViewMovieWindow, hook(MovieHook)=SpHrViewWindowHook
	SetWindow SpHrViewMovieWindow note="PACKAGE:SpHrView"
	
	SpHrView_MovieExtract()
end

// =================================================================================================
// SpHrView_MovieExtract() Gets movie at time of cursor
// =================================================================================================
function SpHrView_MovieExtract()
	DoWindow SpHrViewMovieWindow
	if(!V_flag || !DataFolderExists(SpHrView_df_movies))
		return 0
	endif
	DFREF saveDFR = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_movies
	NVAR movieFileRef = $SpHrView_df_movies+"movieFileRef"
	PlayMovieAction setFrontMovie = movieFileRef
	NVAR NumFrames=$SpHrView_df_movies+"NumFrames"
	NVAR FrameLength=$SpHrView_df_movies+"FrameLength"
	variable xval = hcsr(A, "SpHrViewPeaksWindow")
	variable frameNum = round(xval/FrameLength)
	frameNum = max(frameNum,0); frameNum = min(frameNum,NumFrames)
	PlayMovieAction frame=frameNum
	PlayMovieAction extract
	//wave M_MovieFrame = $SpHrView_df_movies+"M_MovieFrame"
	Slider sl_MovieControl win=SpHrViewMovieWindow, value=frameNum
	SetDataFolder saveDFR
	return 0
end

// =================================================================================================
// SpHrView_MovieClipOpen() open or re-open file and extract movie clips defined by graph marquee or polygon
// =================================================================================================
function SpHrView_MovieClipOpen()
	NVAR isMovie=$SpHrView_df_globals+"isMovie"
	if(!isMovie)
		return 0
	endif
	DoWindow SpHrViewMovieWindow
	if(!V_flag || !DataFolderExists(SpHrView_df_movies))
		return 0
	endif
	DFREF saveDFR = GetDataFolderDFR()
	String traceListStr = TraceNameList("SpHrViewPeaksWindow", ";", 1 )
	wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", stringfromlist(0, traceListStr)) // get wave ref
	String filePathAndName= SpHrView_FilePath_Get(w, 2) //note(w)	 // get path to FILE from note
	SetDataFolder $SpHrView_df_movies
	NVAR NumFrames=$SpHrView_df_movies+"NumFrames"
	NVAR FrameLength=$SpHrView_df_movies+"FrameLength"
	NVAR FrameRateDelta=$SpHrView_df_movies+"FrameRateDelta"
	PlayMovieAction/Z loop=0, open = filePathAndName // open loop=0
	Variable/G $SpHrView_df_movies+"movieClipRef"
	NVAR movieClipRef = $SpHrView_df_movies+"movieClipRef"
	movieClipRef = V_Value
	PlayMovieAction/Z setFrontMovie = movieClipRef
	SetDataFolder saveDFR
end

// =================================================================================================
// SpHrView_MovieClipClose() open or re-open file and extract movie clips defined by graph marquee or polygon
// =================================================================================================
function SpHrView_MovieClipClose()
	NVAR isMovie=$SpHrView_df_globals+"isMovie"
	if(!isMovie)
		return 0
	endif
	DoWindow SpHrViewMovieWindow
	if(!V_flag || !DataFolderExists(SpHrView_df_movies))
		return 0
	endif
	NVAR/Z movieClipRef = $SpHrView_df_movies+"movieClipRef"
	if(!NVAR_Exists(movieClipRef))
		return 0
	endif
	PlayMovieAction/Z setFrontMovie = movieClipRef
	PlayMovieAction/Z kill
end

// =================================================================================================
// SpHrView_MovieClipExtract() 
// =================================================================================================
function SpHrView_MovieClipExtract(xval)
	variable xval
	NVAR isMovie=$SpHrView_df_globals+"isMovie"
	if(!isMovie)
		return 0
	endif
	DoWindow SpHrViewMovieWindow
	if(!V_flag || !DataFolderExists(SpHrView_df_movies))
		return 0
	endif
	DFREF saveDFR = GetDataFolderDFR()
	SetDataFolder $SpHrView_df_movies
	NVAR FrameLength=$SpHrView_df_movies+"FrameLength"
	variable frameNum = round(xval/FrameLength)
	//Print frame
	NVAR/Z movieClipRef = $SpHrView_df_movies+"movieClipRef"
	if(!NVAR_Exists(movieClipRef))
		return 0
	endif
	PlayMovieAction/Z setFrontMovie = movieClipRef
	PlayMovieAction/Z frame=frameNum
	PlayMovieAction/Z extract
	SetDataFolder saveDFR
end

// =================================================================================================
// SpHrView_MoviePlayer(filePathAndName), View Movie in movie window
// =================================================================================================
function SpHrView_MoviePlayer(filePathAndName)
	string filePathAndName
	NVAR isMovie=$SpHrView_df_globals+"isMovie"
	if(isMovie && strlen(filePathAndName)<1)
		String traceListStr = TraceNameList("SpHrViewPeaksWindow", ";", 1 )
		wave w = TraceNameToWaveRef("SpHrViewPeaksWindow", stringfromlist(0, traceListStr)) // get wave ref
		filePathAndName= SpHrView_FilePath_Get(w, 2) //note(w)	 // get path to FILE from note
	endif
	//PlayMovie [/I/M/P=pathName /W=(left, top, right, bottom) /Z] [as fileNameStr ]
	PlayMovie as filePathAndName
end

// =================================================================================================
// SpHrView_MovieSliderProc(sa), SliderControl procedure
// =================================================================================================
Function SpHrView_MovieSliderProc(sa) : SliderControl
	STRUCT WMSliderAction &sa
	switch( sa.eventCode )
		case -1: // control being killed
			break
		default:
			if( sa.eventCode & 1 ) // value set
				Variable curval = sa.curval
				DoWindow SpHrViewMovieWindow
				if(!V_flag || !DataFolderExists(SpHrView_df_movies))
					return 0
				endif
				DFREF saveDFR = GetDataFolderDFR()
				SetDataFolder $SpHrView_df_movies
				NVAR NumFrames=$SpHrView_df_movies+"NumFrames"
				NVAR FrameLength=$SpHrView_df_movies+"FrameLength"
				NVAR movieFileRef = $SpHrView_df_movies+"movieFileRef"
				PlayMovieAction setFrontMovie = movieFileRef
				PlayMovieAction frame=curval
				PlayMovieAction extract
				doupdate/W=SpHrViewMovieWindow
				SetDataFolder saveDFR
				// set cursor
				PlayMovieAction getTime
				Cursor/S=2/L=0/H=2/C=(0,65535/2,0, 65535/2 )/W=SpHrViewPeaksWindow A, $CsrWave(A), V_value
				DoUpdate/W=SpHrViewPeaksWindow
			endif
			break
	endswitch
	return 0
End

// PNG: width= 45, height= 15
Picture SpHrView_pict_Wrench
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#3!!!!`#Qau+!7_T;M?!V[CM?aMFBD723b*)F/n&:/@V%0:D/]IT!!!
	!]5Bq-U@r#Xt+C\npBl6'Xn%6=Y+DG@L,$-Wr0MQLRAS>dKH?*RqH;7FLH>@"NA0O6%$9WPoG\qD:A
	TV?'G\q87F#nP/,%5##@V%T<F#n/LFC?gH+F#qlD/aT44X+!&:`r,DEb-@N/i>78,!H>V4E=tE3`8@
	8+F%a>DK@jZA7dtKBQS?83\N.1GBYZ`1G3TdB.ku"3B8`H1+tC</TPB6/TZ2TFCBDGDK@$H4piDT4E
	=tE3^dP#@rc:&FD5Z2+ED")3alK9F`]bj+poG8+<YoGCi4;TA7IbHBQS?83\N.*F`M:NDfTJDA7I9(
	Ch7HpDKKqQ0eP16+poG8+<YoGCi4;TAU%p$4X+Q]FDs8o05bh`@:X:cAM.J2D(fIFBkB"Q/heq&$6U
	H6+F%a>DK@j\BkCs?,%u(?E&oX*DK@F=A8bpg/n8g:06Co?AhPkk0J=UW+<VdLG\q87F#n8RDffo=B
	Q%iQ,%u(?E&oX*DK@F=A8bpg/n8g:05tTCFDl56Df@?a/heq&$6UH6+F%a>DK@j`D/_W^BQS?83\N.
	(F"Um3Ddm91@rH3;G[YPE0eP.5+poG8+<YoGCi4;TG\qCo9g'\6FEDI_0/%3a/n&:/@V%0%Df%.P@;
	mkS/heqqD(d(?+<VdLG\q87F#nAa7<Wc9,%u(?E&oX*DK@F=A8bpg/n8g:06goE0/5(506:*<E+L\#
	ATMd4EarZLG%GQ5,9dI;+<Y6?BkBD&Bm=3*=@?k?ASuR'Df.]^0fV-:$6UH6AU%p$3`'O8ASbL;Bl.
	F!F(oQ14X)d?+poG8+D#e3Ai`:tCi=>mE+*6f4X)U)$6UH6FD5?!3_=0sB4Y.YA9;KJ+u)&:+poG8+
	EV=0Ai`M#@:s.FASu.&BK[Ot1Em/K+<Yc?AnF)%ATMd+F`_>9DH1RgF?Lg'+poG8+EV=0Aia$lATMd
	+F`_>9DET1"0JG+5+poG8+EV=0Aia'mATMd+F`_>9DET1"0JG+5+poG8+E2"4FDl56Df@a)Des?49l
	FQR4X)[+$6UH6E+j03Df]T1E&pQX6VqfAAnc'm4X+rF7m\=i77/sf3ArcI1+k:*$6UH6G\qC\9lFQV
	ApHdQFCdTZ1,(I;/MK"50f)c_3BB#Q3\r]H0JkgE0HpiH+<YoGE&p^)FCA]gFC@RGFCdTZ1,(I;/MK
	"50f)c_3BB#Q3\r]H0JkgE0Hr[f+<VdhA7IZBBln0&4piDT+<WdeA7dkgCihiJ+<VdL+?X[TAiaaB+
	F%a>3c&5BB0@G`/S&F)@<HC.,!KIYFEDG=+EV13E,8s)AKW]bGVM(#1H[=2A7dl=Bg)kH+<Vdh061T
	83^IS(4piDT+?V;f@le^GFDPMP$6UH64Et^^9hdZ>BleB7Ed9hU+<VdL4E=tE3`BU04piDT+<VdhEb
	&cCChX5!+<VdL+EMWnG'#cR@ruF'DET1_Ec5K2@qB0p$6UH6+<VeJF@C@p3cf4OFE_/6AO0s?DKJ#c
	6"X_CDJ=3<+@Kd^BkM<pEZcc)@<*JH0d&5*1,(.$$6UH6+<VeJF@C@p3d57KDET1!0JY=60Jk@91K6
	%g3]/ZR1H-m>1cd<E+tl+e+<VdL4>1_cAi`k%EBX^P+<Wd"G\qCo9fbgZF*)>@Gs2QX+?V;tA7dkjA
	TM@%BlJ0.Df.`G+?V;tA7dl#6q0?I4>1q?G\qD:ATV?E$9UOtE+*6lATT&+DIbmSE[!_31RJad!!!%
	NBgY6aF&GLp+A#!h2DI3M2D$[:0`V2%O`cfD.t^\9a$4/[X=9;?(JEe'dnkrAAP8eaek?s![b#u,rH
	N\H/M4HSXeNan?PiVDga^oJ&Lk82iL78b]q0FHPO+gbGs:D<T',>ucN-ZaUL\MJ#I:ml((U*Zo?d_3
	o.sE`17`O-'O@78Kn@ep+*TQ/p;Ut3Wpf]NBNn6q$8f*qO"752&=sLEXqk!#[tVGYFDP+T<7n$=J\n
	mn*j>-maf-b>RB/ioI^!o"4&WgQrem&KQ5,^Pj32+mb"k1_9A1&i\QQ1cZ?jn)'>TBUBdWB1AV`(o]
	snql:eoMspJg-ba@iTY]1NBh'iLhVi"EFGfb7hYU*jJ`.%1&6gK-Nu10i)\mXqAa_g(L&jt/!")TLs
	6@?chGnI[Bo[J9Y#N4t4a=jro<?se\qbF04(Jb'?7Gu/!SX<KOEZ6YY'H31N'FC%?#@^Ti5pDi'XA5
	0f'$?l/O5lC"Bh_an:cN\g*n,NFg$#iF<ErZ1m,6.]r,6<%c5B[#,)O<r_5u`*!m9P!>;cifZ2n@TT
	#[oGQ#U#UE"BuEL8$S^%ER`&*=A+)/9O6@L):qV(VmWYMb\4CJ1[S;bI;*P8>q3@nJkCnC,026b,S2
	(Dlq*<*pYOdLB::T++]0H-n)=5_#;52pHlh-ccMO1$q[Nlt&7?3IE!E,s2II7!HPe'R.%J:*Bh%&<7
	S,BCV_M(?1TJLqLO]ld#NIf#h;-o7d@o1XNCK\^@c;8cF?5u_$YV@8M)8*ih$4dO[C<V\1`E.:BJPu
	H,SH$Sl>8YK8f_dgT*c"E$jM@'@^0+=30'3EA/%I0'STnUalGaCI2GLB&;3fVHh?OLX]r9?Za43u=a
	PKW!5R@D>::0i"9]!a6ML_Fa5_3N*ZcAoSt9h;DD0krrZDK@Z*CR4W9DBh/,B#:.[<W(e'Vhc?XI00
	jahu6^:q<d/sjcCnp!8M0/"6'2Va#h!!#:rSXl<d[9B8#_!-_b(mR_TE&]BaMo4MXfk`WjBto27@.\
	pZnq-T>Jfs@ElbE;]X7*e":*Pg.)"t>&.#Rlrp]q"S^qdb,aN0U1Pdn4_-Bn6LXmH4Sa]))5E9brPb
	fn;Vc;$GQ)%;7lT3jiK5Yt9RCi!p&C?jHV@u#9eZrel"\\,Mb>-1ic3<$9s#_Gl$'K2H*B!^=Kh#.*
	Jr;HTYFS^KpH,6;UYHRuq_[eOZ1S6dTicm++Po:Y`]mBA:7>hql]=Y]#4?G6b9K-nHgqA&Y;,NIp@W
	B&US"!W&Eq5k&r9)1c-bIYdq_4MaAS#Ef'bpq*_5WGg-8R2(9I(8D?Kkh\p8$5Rhgb\)bfibeg$D[.
	."F??4.Yau<5:1=0/%7qY$K#c`HG;&!F`683]fD\)NBYk)\c>R[Bd+/)&NJQ)@QhNe_9oZM^&[c@9^
	0VEL&<f#%Q5c=nCh&T/:8FYHP-bLPP1HZidjDrjTZ@Xe_c2EQ4)2Uk!uG['[3I['YXF#X4mL;@B*$J
	fnSZ=]JIe;iF!R`66=S3&k"_&?o9<50E$,Oll#`ZX84\[H!2P$O_=Qbr=,#e&RCb_dkNXd3%W;krEe
	FaPFIPqb4MtJOC\VSN?H2\)0qU:?M(k9I>rs-G/!QVkL5pXh)*T]=Zh6p2i?$-ko-=7!O$(;CE4n*I
	o:dLbsFkrZD18LL.0nh[Dd^)N?XtlI:cON)sA7Fa.K$%;N<Id%NgZ1'U#U%YqB$ac_UAN9KKh(Lrq_
	i&Il=.U-([#$%f3K!<qpn`.ZFKnZebD#93!&8<b(^8t-Zm$mG%iK9Fl[X.W3q"T(@L%8[]Es0OD*Rj
	7pFrHIajKW48/mZ&1`JPE0rr*1T]J/+,6_i$WPtLpX.Z$Rap@]8Y6t=8t*H5<KNu()#]j?3QdO"c_O
	@#Z=mWZG#ZY.TT3u6:[[9n@TQ2NVW_81Klm$U(fr\/4WrIO!R7n6'B98ia\Hp3&Rm+HfWCAW5fUa[c
	C.P!%8j/,^&'MhM#)NE>'gha5tW)W'+R57>O@fQMlcCJ`cC?kDP_-ld.I'_M-<%7nKTqS0BOc^lG0`
	V3KdFcsue6bUAWDjdqAi?,Q-5-AE+:pNEY.L2\(5jJ#41hc&&Um@sOb/1:6mC$gET=pR_l6W#`ufe9
	='(1;\?4#sJ\b_RFQehR>;hrk()@Z#'a9S8Bkc85H*bJ!d:bASkYXPR3h<E4h<B#BO2pk]nDJ>t$jK
	(m:e7E\Jfl&ba>UPu1ig.>TV)9+jn\`JX^/Os-71&r1mCJ)l1K?e5JQROm"U6!Ze6lFMr/sWh2rof!
	!$K5\?)b?!'mAY'+tqD/6O/0M1UA4H[9u<!8cSh-Vp<b/hX<-X/i;%91VSjDL6keeS8r=DME=4WRR#
	k&:]1Z/sjajZ&D-s>gZs^PtHa=QrD2(CCdi[Jg`#q-n?g#'%k[^F2N_A'.6O;HM$FDPq,)snAC!8L=
	]aYNCp$V(fjsL5,&f3*61jF3!"5''Lh'Ai`<3eWRpH#qWO^9bEq\P[u\dG03GFY!79TZ?;XR.?S&s"
	==fclg0-K&'FH-C1c.$G<2ijR\T:7l(+U(+:JXc2kbZbmm"\"*_mQ:;X4:[a\@&^ZIOgW(XB'+\GPB
	@O2fj"[DVVa!='(`QPiV)n#lnXSCWk&MGN?/DZamfahK<EtPa%EnkNBa&2]@Z*PmYQOM*uCWlIGh1r
	:B*t>HQK;jCB$aKje(T8P*uEo:YQ*_-.\KFX`cJRVY'__-hf97]B?F1c2.1.[<WFB!ZbF*f3;%ESSC
	H`,Qd:3*f1GWiE'\GOF,#@sYgP0/&C0jj'8MkFHhRq;I)NP*;*ch;)".;ZQl(?!Y"O\b9C_g=lDB(L
	OiQo#mq,T6@3q!.aOQs1YV;%=Sa4QV8`q0O_i!YJ.SX`t#@Vb[`I4!^UX=/-+Ve_h\3`EZ[iKdZ#2)
	VPU-F]6DTO``H,=fS1Eenb2["X47Q0n4?2C#mgo;.b2_c$Qeu`j`:6:ZWaI<L#U+3)u9=*#(saGRRX
	4(jlY^JA?t&&Y@#%.B?m%']6@J]<f=rFh;(f'dhaWAGkC2sPK?d$;,L\-*u#F=]*jYZ7\d5s[eAe(P
	@r45<NFXM!.TKJf3&R`O@l@YDr9u2)VNWi<*<B"qWk?2Qn\[iUSq-A/seYBlI=VEgt('-LCYJ%R50K
	8@bJ=("C\]ANCIDqDXP>n#HqLLNoP@`<0XG9+`K$tXU`lr[FoAf0O]Q(,f4QVC:0L(-8.#*(5p_le"
	bH.>?b<[I!ke'#K&6R`?LR>X/`2RgY3;CM2[B=O[uW!%1IMmAnLa^St2Eh17<5Fh1Z4QcCI$u=]q/X
	Pr3'k$ndmId:aF:)BM"c?mABA:JY2!gS79":7j^ulddYWcDOR>e#,ucOlZQC%)]BVZMMd0PtG%Z=_Y
	9I+$Y5Pp"*PX]m<\^f9_eh`l?"RR7t;#J,As,2rAurOp8sHX]F3+e<H/\F8GOno($h"N=Z6_naQnr%
	sOT730$4NpoofDdFQl\1`BLJ<b#WL9hgUNXUYXDM2R:5F)u,4CEj*O.[A^YK4,Vp*Q_hIM*qD.Es+c
	?"S)!NTMhblg7F6mj7V`ibKJ*+4BJL]V5:$UmHn%Z=^nWQ5nJ`:6)2hjPEQ"IC:1XOW!HR@YuhJX55
	`D>D_,)!k_C7/kj967,UA%sKt59F!'kC<m>W&DqT)$fa])POX\u,>5CS"Fe1$eZ`agh>$3cC<!!!?6
	$_*TRTE\;n<84m#?at(6`f1pqMi2Fb<"&t@o&Ro&`f+o.nOfX`s$g]\Rp0*$$%c!Q<)T^FA2:udhV6
	d(H;aG/p!?6]+I\dhF)uE+>e*a5<X+$i&0)2#bKC99nC[LR:g@7Qm+ATBaZ`1h0de>]c'gKt(G6+'$
	CWqoi_(r<6m@cUbaEPBi^KWi=C,=::aj2u_^uYIace*Ga4hP,-?IG#2(gX'!!$*:U8"C]6'%J')S'+
	EOH?\[d39'U',MJ*kOX[&UIZX[.4Kh:/2fDP6D:cp[XRhm2`=6<K0UQ(rUncJBDdr@:J+5O"q9*?2E
	"#m6j*`"GB\9&GW]>??Rh?+j5WM';GAJOoY#rF+XC)u@q2-5)i6rq#R#*VFid6S=')kHf<,/9Q-!RZ
	c-8[S1d+!MgU:rR0emn@hX0OjQp,pZ>hC,_IU$#Y#ln?gRIfRcldHk?O9H"-baLE4'kROJN(2jPRlA
	Z`3RcO/ec2Od\`WMZgUD,$iue<bk2pjQeh&54)&3kikNeB`=%jqSJfoG)J'+!)IC`?*PdfP!4!l;o7
	E-8TL@XnUS$1NNc-=L0a!;uY"pYD!p%j]/%M''\W.f(P'F[G'<b;ma*Zeh3NeNIJShs?EFMihB5Q16
	'J3V"m*2[pn1n'%%&$B-GmXl$6aN;Qop%9&V6i[4`;H[%$##D'204*Eq71qnJjQ,ArRl58sg`\Tqr,
	R%qHhVE76o&/&md=u&U,4\`A7T4ehRn.&hRt\4/1`=\VTTYi(JCT&!!&A/9hnIO/M%G9=g`"pb*=J_
	'Q#N@%$d*C&rG[`=bR'4!?MTH<%>.T>Yn5s%s#=q]V+<@g=cmf^UL[2LVrB4Qc3Ej6m=ge&-#-e9Ua
	u#ijERR?G1[8:1Uc-UZ:O0IqCAO17>5JYRj9C-ko9_@Nkff@^-U^'u?84<)f)6a+-Q%V50o7r:.f^0
	ek:Q[a!%?I.+cC*^+g[-\_91K*TZGc!2mqVk8F<WiE@.XW8E&5s[efJUme$]j@=rPm%/\e*/SA0`_:
	UYSW1k)`DO-kbF3O]Y%KX3L&TWk2o73;YXHt.[>=n*\&*mT+AuKVl'cGVCWJj-I]b:$jIq//hSt;eF
	mE$8,pSg)t1N<DGErerE-^%UUbmS5(g[R[VXWb1M?:Mo9V+;:*u0Ag@i@8#7?.d!'m5S(,&u&!2._Q
	YJ'Y`B:jT1A]b]gA5*U<Z%)fQeZ2bE,SE1t!<?(u<7KIK"U"sUo#W7\Y$AM\m93CcP*_N%N=Fcoq^h
	dJ5nK_d#`ir7>jgMLrq?pepqT>ppW8UH"U,(kjlM-Wju"tP==b4OT0I2H[!D9*,pjtEfkbkVGK;%na
	TN>8JfqeA^9E3@`F?OOat#/<BRA(P#iX(Sc9m?sIK''*OK/#*@9?5u9ke>+m+J]G!#CLS`ufdnIf/u
	KCY)sHQBmirio7np6l(^R`Z&j_h7In0qXp=:DsjrD%Mbd;l(@tIVlZsNqfIBCR7oV$3AY9kfXuJc!2
	.$=rReAbVbWe/o]Qm'+H]295("1d)sct]5C)g1R7t&%c^pAc'Gk<d2)R8I]6CHRUVi56<i]4QQ]E(E
	DZ';"YahQ"\@AuU(@GiQL37Z_G'A.O3#k.I58`<P:/:4a`U^[l#WeNZTV,]UcCK6+H&fY^h;*MTo&Z
	&^:&=D#+)frEd6G$Tp?^If%j)PoKfG)o<E2>q1UOAiX_holB?m%#^OO!jV/j4Y?sqpL]mJ:Ep!p*E`
	F<ieFMs.X2t#S<*rs[Ag5=q#/$ad^%"%R+ab'g/C27UUl8WD"Qq<Jg_0A:J=')#>_VoGH&e[Mu6NI0
	m?!ZmdmK4qiB.&[FYHP(14`]W)WDf^bW^h%3>V>d)[;-'T$*FP[_sF23]Y"=O%cld7%XEaq84Z9cBk
	`&%A4V5Z=AM^g?!U`==kSlI4./bXM+!OT:U[IXT4NAT>-7;hlbJ&RIA-j3"cZ!8S"'<Gb:aBOD5,Ek
	/1`>7GOF8\hgbY_n+8Wfgt_i!cTf=lG6gJGKY_%m.[:X'Uq+%hfh*/QJ72P]=''Ti[se;+3RY^,TBt
	q2ZY.V$JACaoq/ir>['mDdBZGfLLoRJTbEjlS4aYbGXl0VK80j$[1:3j^>-2PI=tZ/nf@Rq3#Zh!l&
	<k4H0GoP'1^8RU0/)e\fs<VUhJ@(IG!n7B5UM\'c5<*I-O@S*2F">=NPT!rQnT[0-"YZ/=[SSTg"P/
	+q=<Y=kKi%rF2qrN3n5#NKK1"^KgNUqggi1Vij\ooj)V]ZVG0]T8P)KF)(Nf1!*ZL*,*=8fMU'LBVh
	Q2lJ>+HE_'"r`#_KhlWl\cin19@>Ad8GAq"sq!>/TV:BaB;e-ko13*6H@GWi`CGI"@thW=]^SYIsEu
	ChtYpcY"pK0Q;SOR*fS`Df:t5,\:OujlPTE\K#5%/Qf5blI;eA@:=%kDF5U]9I<\E?Jb54a)BkLD82
	%?%p1C6PUTOoP*3hG+7JXc%WAV&j0%#"S"#lt.OrT`b,VJA>$>*WI!fPOS2l)ur0Wk=i8EOens*T8<
	D?1^H[EA'[D6lF>p'^Jq0^8+8/`JgE?'`i[\KUmUSFTbX34tWG^+H9#6tJ:"E:nV-BgIRYR`@>0ap-
	bk0+WKHr3g@j*5S_MP?:]9j4f27:K`#O,o@6]6jQ;(Y!Q_h;(,_:8cQL)!:#EkMtS)bFK=S`MH4oN(
	2in/R'4mcorQXU8b-Vi5('/c_#,hBQ0L-*ZcB;rU"Ni/mSg"`n5*Ns)E^;*RO@):S'6W$t!@K(sdhS
	k+%s3kih4jF65TknLBg(XK5*t<]a;$a^VNLU7q\k"'I/jNCFj`dTWbRZe90"=Q$q&#(t$.6MY:MR?U
	<T='+"`ET<XfjTeq&8kVgdDVZcXMgKbr*#AVs*Zgr&qWX_CMqDjSSN01ePMP$Lgt^[mEXY5F#_EnQ`
	k-H`SiC`Leu`.^?!V/YBb#T*+(,qOR@1Tq`srsAGL1RK7=Xouhe_&UD:54Op+%@:'FKoAj7.BPQQ)>
	("OZIUQS)Q)If+G)BdnhTq9*.Ennc>3p%n7MnsZ!c]jC1&dYOW2A7T4mF>Sop)T;S?#DcB(GB[+i=3
	K]G."n;S4`L,rTAhieg"H2_QXBa`TDs!_JJ:Q#eC<".a,bRoHTU'l6Ef"55C`\(cTa'7/[\/XVs87E
	0FV`3C>,n#6Y9r-;ejq?h)uu-A':/icCQ4(#U5VY%im]Qiku.-A=LDC4MrLW[9Ahfjo,)N*B^O3)8P
	;&ioB'Q6K'`==O_).EH,tc6\B6cm(]>LBlG'DBJar%hVGYZhu3HH*VWp^A29e'?ajq/g9k^gm>(hh<
	4Q8jqa4elMM$F^,A5;lq=N-3S`2\a65_k.lt;r=2L"Ro-S4Zh179\Y9!qj)m0T9,Ft.B;[9DNp:K"1
	A.CYF3V8A;t>kX`i,SCE\@bI?Wfk``a1]R+M]j>&D;?CbVEiP4V_-k4$`bsYG3Ai\e):eut-BkuNac
	_[ENl-V_\$WO#36pK%*6a"b6/(,Lgb6G?1-\"\WUi.lO#M.BbWU=HoLUi]$A!bZgU:rTS'BX>f`m([
	6m;d=#*'cihN>N3NpCeuqYL':daCG)h>>"4bq2%?>U0pkG\*.mnF?$?8kR7Pce%E'n3T!:En5d9<<t
	)hM@14Q*'8>FWiF4@O+aen,9nEk2YQt,rpB+?(G8t&If6iYX%c,Uq!2;e3p:m*2OfV_Mbfs/IIZ=96
	*!h/"$r$1OH=%sQ=XT;4S(DsNS<+?J,fE',paiG"sZ8oIHf1VK6e_2UJ\!i<`N-KPtIr4;j]R+@D[\
	ZOHAW9qnn*#lK[Z+SfSAs'-/+<j7/^Qr-q79>?e^\kK\/3c8g6+iU?6MC=T>nhKg\33;>N$[K49"V,
	H"&h&R+@1?c<'@9?9"cH_tB$OR62E_<<1C4(F,lIDruc'pYFThsqu;^dmgi8?m7,E50&F`hhPH+?k&
	"onY;?R-6nD<0J_1?nIX3h9$De*YL&JX&4`^%^B?'.71B+,Zd,$H6O2%BA0XUSG;9h3se$lCt@.8"`
	[,daQZ^"]Gg6"(o=%n&8nFD6)\Qn#+J<*64mRBjkGD/c?PU;%1>HAIk9TNf8o:2f@EDGCm=1-ii2fc
	(!=f?*d_:,SDOe3r0A:pk,KBkIl6:^A6o\VlfY<&o$=AKj2$(NJWL3?)"SK3-t41@::;Z!7j6/V55=
	@o%eNIoe2!&!I@<f<ici>Ku7$#YLDa3;CQ,bSPC7cDOe>N]3_\-a^fQo]Vh[j`/,-0!s=>K`__mBLh
	kVF_9a3T_:Z/2(R-8OZ*@K7p=jN#m=2F[@^.h.g@WZPmg'&JiNW,Zi&H2`pi5M59V4LsrVH1@K5T\e
	M%YZ@]"5J&(G@GFJ,S\?WMul7FlB))>-6AAIM<Y;)&*bf%gFHs8kDTGs'[:;8H/VfEH-!ThVQ.s6Ns
	HiX/i5!$*NGT5!JN,nL0:39:b)%iqdTe!eK:FJHK,bC`.E9!!&6U,ts#DnVX!4+;0M6;koP>P5;Y[3
	-!rkHhS:/?9IS5_/[,cV["8@;^`>uDf<-nP=Qr/LXZ:OA29Pm*ZgoCSBU0fL]'s]&rB#^8KP!7CWjS
	M:2Fk+'h-N'KS5O::G,k[bXKVtcGJ^$==fc@XtJJqZs'S[mLin](r-+Ln$B@,_ns8[_F<\%KaS;5g7
	H)uHT-\J4XhgoK\Rr1]">Q^KgI8S$KG7cp?c(lC))^tPK@@%042/Y<k5iSc^?S%gU:rr;,P5HEGtXK
	q?hpCGVNE?DJj&uk-SjO!!)c^hm\k<OND$a5d5G!g/Y1S%AR>""#?:g@^4jq'*M0<>tq"hk]FDT@VK
	XpZjf*R7.Tj:85)\s$1#/3#XK"<04%nmDB18.TMhbOOceMWhUU0,[Z^fe;#&UGDlmB]hF5QYc\(W>\
	*qtS"$1b"St</e\Zi:L:0'EH*ZH'1If9,RF6ARFBki8iUUbU;*_`74!3fQV/siWjlYT301`B(2&B)W
	0NuY;ir5[pUZt\!g;lBJ!?qD]/iF0`h,tg_ch.?Y;qYL'\o^m`eK)S!oE&dIOc!.Md9gV1`Yj6i=^b
	AG3Q/99*J&%j02`J4MYd!ZXa,^cXcl;%A55Bk$UI@R=BXM#J@QK1W^OLQUqYT`r5BlPN&NJtc*"GIe
	'K.g+ouV^uM*u.2Y0G&/E*]$aEru@.()03F>&E_:7o<5.&4B#Y7moa-:N4*lpbe#Wd*NmNM[Kn#<@S
	`@U4IlHnaZ-')eMKDC"=A<Zalg)67`n=Q^;JE)@hN9*ZdL^g9d2QH>*LWdloIR9J%hJC:.6Ge`&R6?
	nf7+]R5-Uh;$dTlpUfU,V5<a%-)3oDn1g/]DI+$#c#cQV_!_"n?:1I[o&0;+:+>F"TT[/a]'a4KFds
	<r[TOppj[gV\ClpPKO^6'UMq-HU`"m8(a[ETSAlsk7c?0n8M^^O0hRm330!q?F[<R,a/6Guz8OZBBY
	!QNJ
	ASCII85End
End


// PNG: width= 45, height= 15
Picture SpHrView_pict_circlePlay
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#3!!!!`#Qau+!7_T;M?!V[CM?aMFBD723b*)F/n&:/@V%0:D/]IT!!!
	!]5Bq-U@r#Xt+C\npBl6'Xn%6=Y+DG@L,$-Wr0MQLRAS>dKH?*RqH;7FLH>@"NA0O6%$9WPoG\qD:A
	TV?'G\q87F#nP/,%5##@V%T<F#n/LFC?gH+F#qlD/aT44X+!&:`r,DEb-@N/i>78,!H>V4E=tE3`8@
	8+F%a>DK@jZA7dtKBQS?83\N.1GBYZ`1G3TdB.ku"3B8`H1+tC</TPB6/TZ2TFCBDGDK@$H4piDT4E
	=tE3^dP#@rc:&FD5Z2+ED")3alK9F`]bj+poG8+<YoGCi4;TA7IbHBQS?83\N.*F`M:NDfTJDA7I9(
	Ch7HpDKKqQ0eP16+poG8+<YoGCi4;TAU%p$4X+Q]FDs8o05bh`@:X:cAM.J2D(fIFBkB"Q/heq&$6U
	H6+F%a>DK@j\BkCs?,%u(?E&oX*DK@F=A8bpg/n8g:06Co?AhPkk0J=UW+<VdLG\q87F#n8RDffo=B
	Q%iQ,%u(?E&oX*DK@F=A8bpg/n8g:05tTCFDl56Df@?a/heq&$6UH6+F%a>DK@j`D/_W^BQS?83\N.
	(F"Um3Ddm91@rH3;G[YPE0eP.5+poG8+<YoGCi4;TG\qCo9g'\6FEDI_0/%3a/n&:/@V%0%Df%.P@;
	mkS/heqqD(d(?+<VdLG\q87F#nAa7<Wc9,%u(?E&oX*DK@F=A8bpg/n8g:06goE0/5(506:*<E+L\#
	ATMd4EarZLG%GQ5,9dI;+<Y6?BkBD&Bm=3*=@?k?ASuR'Df.]^0fV-:$6UH6AU%p$3`'O8ASbL;Bl.
	F!F(oQ14X)d?+poG8+D#e3Ai`:tCi=>mE+*6f4X)U)$6UH6FD5?!3_=0sB4Y.YA9;KJ+u)&:+poG8+
	EV=0Ai`M#@:s.FASu.&BK[Ot1Em/K+<Yc?AnF)%ATMd+F`_>9DH1RgF?Lg'+poG8+EV=0Aia$lATMd
	+F`_>9DET1"0JG+5+poG8+EV=0Aia'mATMd+F`_>9DET1"0JG+5+poG8+E2"4FDl56Df@a)Des?49l
	FQR4X)[+$6UH6E+j03Df]T1E&pQX6VqfAAnc'm4X+rF7m\=i77/sf3ArcI1+k:*$6UH6G\qC\9lFQV
	ApHdQFCdTZ1,(I;/MK"50eu]_1-.*C3\iKC0JkgE0HpiH+<YoGE&p^)FCA]gFC@RGFCdTZ1,(I;/MK
	"50eu]_1-.*C3\iKC0JkgE0Hr[f+<VdhA7IZBBln0&4piDT+<WdeA7dkgCihiJ+<VdL+?X[TAiaaB+
	F%a>3c&5BB0@G`/S&F)@<HC.,!KIYFEDG=+EV13E,8s)AKW]bGVM(#1H[=2A7dl=Bg)kH+<Vdh061T
	83^IS(4piDT+?V;f@le^GFDPMP$6UH64Et^^9hdZ>BleB7Ed9hU+<VdL4E=tE3`BU04piDT+<VdhEb
	&cCChX5!+<VdL+EMWnG'#cR@ruF'DET1_Ec5K2@qB0p$6UH6+<VeJF@C@p3cf4OFE_/6AO0s?DKJ#c
	6"X_CDJ=3<+@Kd^BkM<pEZcc)@<*JH0d&5*1,(.$$6UH6+<VeJF@C@p3d57KDET1!0JY=60Jk@91/o
	t`3\W6K1,CL91cd<E+tl+e+<VdL4>1_cAi`k%EBX^P+<Wd"G\qCo9fbgZF*)>@Gs2QX+?V;tA7dkjA
	TM@%BlJ0.Df.`G+?V;tA7dl#6q0?I4>1q?G\qD:ATV?E$9UOtE+*6lATT&+DIbmSE[!_3b%?e1!!!%
	NBgY6aF&GLp+A#!h2DI3M2D$[:0`V2%O`cf4.n!5Ka$40l(l`!5@5/(h)%B36P`\e+VD65sc-/%`7a
	6nDHR$X5fhPCRj4AmtJ??+XMFbk3_OB+24m.**,]dkdT$RAlHVX=,$a*T$A^/@?A93#@3Z@Q`ccl6c
	T;]\\QN8#bb1)1jOYCA6rK[dP?3fh#jj3m@FULc>5t-e&T[Jh4n<e0>-*Q(9-/><EnHIkb@D<s%'Y^
	jFKYu+oV]>]Cfn`(P_:9[p9E!kg032d3.kE?O/XomD5PMEii^r:0-7-*X[pTh[PRp$u).An=]',KkQ
	N5i`b).tM_=.:.Q6]PRXL!f#+t:t$nVat0eWUV'd??=@XP3XQgK.>>l#s(R6;[>g=g.3h]a;Z0AUPP
	#<q-aV%re*jjY-!ghj`h`P1I&HhK:fk-u"NT388uGHTc8b^EpA!9@Mh\)!Yb,nHs$oS=]]GF*%Yl2>
	O7Ff\(rg3U`O=!b3TjjN.?2>6"X'$#iF<ErZ1m,6.]r,6<%c5B[#,*dpUJ5u`*!m9N;))b64>>-JS]
	2A-BhO<=f1TKmJF6AZ=hdde#O7sBpJ&JO,j_aGS/LH3STImO*c\]o-jck1?`-"8ntDYMeLBoafJQ:+
	?B8-oDjJ0[%)fQYi63JXtLLarH@V4J!C#=Snub^,QIji`uDhg7p;KYUCY$*A`*h#dM-s*ONt1d@[Fc
	k<c5#Qi%d*6/A;+GqA`E<m_N,\8_K78Yh.9I>rP'FK_l,^cOi&4KG170k&i:_Z?ik5bQ>:"(pnOsEV
	^'Le-08J&i+QLl:26B/kR4:`S<!6F8BMP:B.TI:-T0@^^^+H7e,H[C+%80jD@<3+PQXi>@N.#N._M'
	*(mVWDj/8u5FdeL$KZ^VAZ8Z2?2$!1%10M8]DmIQjAW(5p^XQ!kM,'^I\S'W"VS<%:m\B;\M&1RlWi
	X&Z9&^NSQJ6MZ!mO[u9GWFE,7/siWh.NYP`#2JT<V,FErCUI>u_-jNMb\"1T-9L7_M7*k%;&!fRg;_
	.Pe#,uW[2OLR+58(I!'ksGi_/J=#_GmRePfCF6UO3jPtJnZco`eCb0n_$hs[b,NFg"d_)en$,)t<=+
	GbT"JSWl?)%;8O661_U1`Cf3i'AiXCA\YSe4.Ol6<MpjDc8-N@q0"4T22Fc[Oja+Ud5:RmHs`Jr^\;
	4X4=F`Pfr/LS^(n*lLk#!cH_H1%Ql6B!:Wt,0MK#^cKFf0>cq$UYq1g4A%14+?>k%3pT+)i&AiBG]W
	n^R%Yo>$(IaH/#anA@'o>\`lg3t;.[>%bS0U[ZO<=f;_c/+N;Q3VFQ*<0`S^uEEkAn*;#7"6YeLQ_$
	!)P/16D;cG`^V;=_:'f$;(@i'X4?KW3iuG`d:a4$"\mGK6/Y>C<i_M/ALr,;>\7+S!79-@@CP"IDKe
	lP:E/-H120-R]Q`i$oB4FCKEr`]j)d>QbN3b*SR#N,Ek2*Y+n2+717:re[(X.Y/sl+sPEV1H04%U+'
	Wo)79.1J]Mi3Lm<([inCj:)HGIq+_JUmim:8^o:L_i#n1]Ea(q+@KH80kXP[M"FM+:&ZMl1Oo+g"G6
	=hX`)mf%/FNqtDV+$A=LP=h.dT\(dHKVq#P$6PKoI=EoK1KTP6(Hp5<W],M*28u3.eLo+muh1"D(%C
	KBqOsDW@"sL@][C*BKPfr/Ba/rCCg[4Ftn(250XdZ^1_G[?INCJQ:?m:_`&=Q:iaiVZ@_85`IPSo4l
	_UiN[5nIQ^=`85^a5cFMlg*cuBQ\c7=Z2gGDf0G^UX=+m9Fum/lQ%&hfBIC?9+CsCB!]\(b?J1Tf[T
	<TluY3,/siWN]3?nS\'u+FYYqGg,*?7eQqSheNmm=e;c?UiVmCYQVchG5#k-T]p[$QhKM\dO1E)6SY
	?i4f)ELm+S.(&^bHa2k43Pgj`F=t,Brl?NjUK7M[r:.6LVn@b2`NcV2)Wq5dt:XngW^(WOO*+Af%)`
	;2@r29SSi=_:0Np8&.4+XGOOD-7]ouK*uuBpHG'N8i@J8f]0.'l5'c]ESPIWj4Z#!;rVH16kLl_';l
	s*hhRsj:"pI:88^cqt:/4RLom3T`#9f>Tj]5D_6D4t2EGi8_b*=J\X]CV)e*6K@?[S!ZjTIkGWDjNE
	';M=tXX`BF_"7Klmd?8d:GS>s6D:o<rVH$QYU$"\755>ZgqS?2AQ_;1b\$IsX&fN1"sb0cPtGV!AL.
	lnYUnP\f6?8pNFo8=l8QKRc,k/`%.hRgT>]c0T'+X=op]m63BfX0d:adF:BWt^<%=B^VU9&iM*s+eb
	iNk'S*fR#_c/,Y`>;kq$3cIMb\$I`_SXC8LpWH]78X8ATsqLtA_CTL)3;f=!+rS5N(ZK7'H]Md\?rW
	]ZE,I3',)&NoCHU5p&O/o\^bFP)NEEd(AGJ`A3+j$SO/c[[<h`Dm6i.L-bl-0D[m3cdBo,j_&l+>.&
	51'0Gm]e<&0lcA272dodhW29IBB$`ugYkAWGdj%`^["]OS3Kc=;-,-Tse0It)rdDbm''QqXBc`t^pS
	F`hhPH(!i@JfnZ;AhIB#eBuZ`*k!sR^"I3sV,E^T-BUi$X]C]ZFi(I[+q94pX095ELVk+P=fp8!hgM
	uI'jC2O#XY`HRnP0QqJ*Flhr!=_2D,c'[r:.V(5k<$HT[n1\[cde;X9u(b%rsL";-V_;lQ!RV<[@2C
	88BP6>L@P%j!jk30'm#2r8eskOZ)rn@"47NfK^&Sm3n79I<];'OHnBGOr)8JIk"ko]Z,BkCOguG':U
	mem!8=DH3Y7lhCDt0?;<eZW\]DEI^=dKLV`al#bm'31WePJmbui>NLhmMMmCNGY3*>SMtKDe7^[']G
	H)iPb\mc!pTF^BUZA5XUr?O//F47c";#99\nIX]6!-FbH_\r\[a`(f3;>D78PPOOrVS86=PA"Q)3/M
	NlU25C_dQ5j//&N8J@(Qn)(n+EEl"(a'3Jqe[R#0c)gons8:=fJq(9N]j9SH?@u<N[m,]hq#Xa%<j1
	+F,*>8;U"*m^ErBFXYHP/$4E%W2kND"ID<`B;mLVH/P[Yb&6;g+>B`W>n,Y=FuVK05;:8a#hU<R:ca
	b4,+G&_N,:aeR7Y?eau5!E<QEX8<YYd6jdj\UsRGr_&2aKsTJ#G>$G]=PSXV??m$/=%N9ld:A#7If,
	N^Vs`USt6u?(hb`0"=PCC363\JQ[\nhYG_@ZouFiiMel6Wkm;r\3#j.f,7q_i:S/D\/Ms%m-F*#:X]
	@)+/9H#W2-q#k#QTB#bg;aX*I'2!Y6d&m,mODu'T/<H)3mgZ80f6>H6`b_o7),McCJK$aY8\&l"jl=
	!!!%QG9RofM::uP@P,_lnLmdk+GcV\T%Ve"OsF+1=(AMfOgc>s)MHJTLCZ2/%**?/O(W]^puJ]i?/G
	R9YQ*S4-b'(ljknM:Zo$bE5S1lP`.Ys7aQ7HcbKCmG2=jlT+hZe7!)-0aS'C)Zmk;l,CZO38YZ25B'
	#p]Ge`qA8Gt4i5DJj?H6D5`1N^fd*4$/$:QqXCAl04BJ!K_sh7;N^?`l(tRP`kc!S'C2>$A,LfZGJ<
	>i>[e"9eMb?"9K-$SPFCjb*?QY4ouG49n)smhu:g<5@49Zk+;g^Ks>b%E7^:!)$g@SYuiefO1hn?`s
	iFc\6Q1KVB2g]2TQ?BkV)O=.0KkUJ=`>RVpbqqI/a%(-Ln(N(*lJ7e'lbN']a7A81$gFr.I;O7AS?m
	17@M"ji\?,mHs;CYHK$cp'B)_D/E^1iITk6o-2`+05pm]bs/u=mq_rpX<cPd]+0J0i,h!\!.M]_<+W
	s$bjt<[KM\@=9MJ6hZs8@h/CC;:H9EYZMdl5C=pKC?Bpqen;EHs%V`'?L'*Q-`Ze=a>jt/.f&5Vk:H
	2K+;)AhBAABlg?:8b^ModW6=.k<.GI/^q3K;?=.6PMt[:3Z!AF<hD_GE[7Nq;+?nps0no?iU'BktdJ
	<g[-=ZEdrR'SN^WQMoXkS;@,\RnT`dZ%Z#e*VUM$qAcN$-I&UDNImA4%\&[OfH%'mY[9B](E$n!9cC
	I$VQ>15PX?H;&9<h>Tl3I<e&B1O$=l=J8om2%g<nlKf5/>$fUfh&3YLRC?[rE0j])[Pedaj1\I5ptn
	F8rNE+(rt^4Ztq4K7RHu5Tk$O;7XBR/W-9-0)'TQBCS4oKM\<Klh#Lb&!k0i6uDLd+:)ePA29R:Vq4
	ltPtIkU6#\e'BA'sb!-'8#Qc3%Qb+>iX6FLa5UJr`1Fur"r46d_;]gNbEiQB:M??\B":.b.=^.j(Rd
	@H?Tri`e\p[!+kBcWY>042F%!8-,`/JJQ4h'S"'<ppf^`Fq*:0C]E8!iBPn!"mk8>=G71#tq6sp@n;
	+KWo:5h%ki8E57^il]l"IK;\Ju'j_.2;)-4OHlFZq"Y9WrPKE1[ml+R!M)<kcK+Jq@*0eg"b)5P#DY
	7(TIWb$RfD`/QVf]'&("V=$TN\I0)!6MA)@QgiR&S!^ODoD8+GCf#>Gp&)6Ehq?gX%u>RuJ]8F?4j*
	$j@eJ,SC(WSPCQo-9:V6id;KB-MR]KRp(8!!]qcTSe$NAN?PMZOX?:6i4pPKh0pCa9TLrirpd%=r7N
	G^qsE*U>LGPX8bVlO!W_rfp5U)q3,l4!USG#r:NBMi,#CapnItn`(i!(:QqSip9NJ,l17?@K2i+2q:
	8^NXZreklfkWYZ4N3(eiH;@G'r%5N!d+d#(`aV#!//8K7`HP"1_<!=9%DbR/X=6Z:_RW*dZ6Tgk+dd
	j:OF/[36rSOb`;l"\E*8W,F>SbbMiduX48\"9Z:EGUl_9+lh(%>Yf2G=!_lP4?nfqac1?<,OZ8M@q0
	]n[Xl9eREse7'!7f.oN/k#leHD3M@'`$/QcfO4.$r,:L;?M,k&NH^J6K'=@C&QUXQA%tr:7a4I/8cY
	55QqL+5oLH!1ckTdr`!$X]-d*Y8.HKX?WgflT&h!f7<qEb-Fg"Vq^3S1KoR..(kf!hquJB0aA8H6i[
	348[c+Efb+E[#g:,7jGs[g<]o2bUf8hO=tN&hSPIeooHhp"Ma`;\<i\CPI`JCD[Cc<[5J$I6e/tReH
	i(Q.r"3bD6`jBf'ZKZk4fl.*j`25*8CR2]k][Gt'N7tGSR11b2E9Y0SSto>!<@g?iCF2(ej(5r*VE^
	gEs2WD8l[*XA;s\aY_eS2"G.aW4$-#s///F]PGVNcMP@'"C0tsUaa.t*K?lc,s3p^lp\]4]p1WoN;Q
	6H?$:bjR^07Sf&4/.?m+(`T*=/=N/H:r^\h.`b`6rX/#J50tL,mZtc=XCB>4fJf*tCOYKjD[c)Hodi
	#o4S_Fg1$4#INMuIO#/Y)at%GQ:rt:1:Q;QJ+Z%@](h&=?g`;e\G'u<$''p-Hke#ca^h^K0I0GJ8;]
	u")'B$NioZBXT:!b\;s_:GKFl_*GlE*pdN1Z5WeIFV+8Fl$r34`q#Qf3!__=TBVXD5[mlZbsYuTYlp
	"?HOs)mYVd&2lH4IjRs!5QfBn5jj+L%A)IV6l3n#r+s5Srq?Bpmio7P")G*LlE:65Q'&i$ob"\q7<j
	i&O7nh7c][h[$Kd:#M,e4kS@h/i*UpY%H\8Jja=a%V`a$)Mgb<bYC><;2.%0!O\']R0-P2PEdJ?hEM
	)*;](<;Vk/C4a#S6e?YiGpQS2%37'%Ej`:^m4!r^N0*Kl>2Ar_Xcga67<B=JLL0+7ahFIF-&cPGNTm
	h2Qss@L>deR$FT1E8\ccYG_4<IZ(rQaT2Ac0:%g`eFtDQ`RFTD-;^Ju5EZ3i%60mlVd$nkEIX2Ra_0
	7opGD?p(qno$GT6D0"[TqpIcLWb<ToQPjctBG14,)VW"S4UE=?1nDOfhSMl+Bm$Vf+&^jh@"%KORfB
	@CY\/pZ'VO_`U'?,)%mAn/f=Qek$%;"-U)AFPA]cpI+*)YlJFhq]-_ET+cgY-+o0jM70'=NJBCMj%$
	B6"h]@:(e\&paVp,Vb2NgaRI9le;./+4Wn0]`HUA_eBu[kQm2MZ!<lTI<8nGJC0j%Z0f(L;E*rj)&A
	jCH-!0Ss09J`+<nD(D/H%)o]Fl]tE`_/L^&<]_N=ko@_-f5Xh5OGU5mRdK3&6["s+h=:?]!8sJ138N
	:\Vsn3FEnI^?=#3og^Ju]RbTsoabEslKZ7:"`Ud=\I4CJ5\;C->L2P*rQKt`hVR*63u/umh#r^kKjG
	s_<i\rmdl8W7'j-]8UiqTt^K:@3p!N*#USD+#>ph`K45*8hm-O($U+*L=,Y:iI/!Z"6A3*Mc!<<*C"
	9fWh*6YPIa*m")H1/l.K6TUYThl`nC:@:r4$,O#;MjT4=5^)E&66'ecrVARTqAkTe)0&4E`++D6R/F
	6+q:t(b2ooO#j:J/k^X\?q9(^>2r>U(e*V1"F`j7D9Soe[i0kdDY@<84ch#h+[\^4;2GbpR1B*6"'F
	2ha2rDAqS9Cl!&;:#\I9`K*82l^VApa%*!Nl_Q@D;5>+;*_P\O*NniAG]:F#KNTe-*EfXbPMF0GohA
	Sr?,`8WooJWE-'jPF"rQj<!1LU$T[bM4(WRZ0&b!a^c"aJEo6:^8b-^9dWdq)GMIiU*uTHj.QIj&&J
	L"@<-bDHdg`F(UR!e?o.>_.TC`-^8%Hu>V9Zq=e'@W^$#GqZdn"$TUsS-:JKS[[O,!X0Gq_cAW0+=d
	@(9#G_TX)WDoqG_5BU/eW[l2>M?RC]^VfQm5CHYc$rn_&E<5i+:)7Y?,ua^7n,sWYuiK@.[S>)]j<b
	9!<AdYX(?od5/n,##M^E2:Xq(W^`dtK8KO349Mq%rA?&\2a^ii[nOQ:k.)80N:^e'RA]=$2m(gEPaX
	O\Pr@N]KSiq];C,DAK5.pO@oQn2OX%fEg]=Y[Q7IFTCF5n??+'tGH2Ej%UaOqoF\[fg9$kBj4NbUQI
	W`#h_mHs.[3iT\s@"l[S#TI>.\1Y;Sh4:+-X`d>g(5!/JQ-/.XBh4CDS'ARW];\beAQ]?*C6_`Zn2F
	d.b]**h`P0(#<h7P#F?6&?6O<(&H_Z1BklVJ"0f:i]q<+Bj@^0/jGP7sR2,oXQZ$4^d/9hq"7RgS0/
	p6YGs5_98bkDbPgLN(-h/:8Ud@<tpfe#fCcfB7ibVmpgme=B[_/jh$NNarQ%YmJM1mq%XD5IpKdGNi
	r]Y'c\kP+`F#6tL[.(La^j^^BdOXIKr[0tU?/+A9dirB'mRW^`_N?@TH-Zh<fHG)M7c\Fi\p:@5`9C
	/MLAads(c&@^Dj$9F6g5/3AGW@fA1j!aLlJUT.0Gqa)3_rJ63rrTOM_7Fq$9A!S_f#sUALm<0_hSZ<
	E#^en1j">SD_d_S-I_tnrqcJ;Df>)I(gkuM`?3(lKRV?,Bq)Og58bIAa)S`']AQ*hC?).CI6j%WaN2
	H?LC^l'[lkd.)j/!PUqQg^geuprGL'X+E1oG^;?oO%F)r"qpJZ,AH@(!#4$/A>@PAYcmd=s_0pL>g:
	;4)AZM3!)/6ljn1:2U;/r0%L]">TOqtj1aGU,je^OH.AO$DLWh;-uG\(PT"IVC4GC:9R;97mqg14I4
	$E1CnsZb=&Cc":+E4l"O!hdC,2BQ\G4/"=Z&al]Ft&9WiAFA$Uhei,\Y*C%hDf+R*5q!RQ&a)OkPo2
	OhF;Q6OjVA9sa"+e87d#c4UC&UD'Ko80PK.Iin%EpBIS4,;E+G=]+3<3H7I/HN25pIor97MHkcH_I-
	n$cCM/Ld/uXI*6eG,!7ga$fsXrj,\Z/Ja9=n)"pM;5mMUS^%<W320ce>-6-6Hhd*hZY.V\qtlkfCt)
	!=%#9<Uqo'MLDtn]DPK>)lU:4%"I.>/l\?O<Ck\)adoQWKL5Th,/YH4fV/1d=Fo`u^l&[K5hFsT4d+
	84!+V5b#qcf]:CCKFS=1C:q.g1)rXNCK]5Ge=p;#)T9ZW5TjtaXh^Wo&A$CO<e6d&AmPjAR+Tro&\&
	M'82GAZii\iI(j8N)N2S#QBp[rk>*e,j?PoJ:;,KRp:P1t2r__T@F$Pb(B*d`a&mGUcdu]f!C78504
	)k7Z]Rc]c`eePba-rC=q[]@T:t`-44as\GdWNG[JK^^T<g)lEST4!=HbJkI=6N=V_\ki/1iFkNQ./,
	j9s@%Aj)pOePl1@JK"].e0mHid:dWKHQ,:bYulB;hr-aZ=s(iIngdeEi3(tmg`.7PC]%R\;GI<tKiA
	Q0nOr_FF5FItnSsc7,ji<;3#qYhrLt\YNQ+VX/iCto^:ldd_r:@eQ^3pg(<Z`N%1,RU!.lW(Q:NVU]
	l(#legKDh*P<e^PKd]"NcVa9c"Aua1d*o^M_!C4/M55lbr$tP!YDjI3Ep#]?\]=1`MH4o*"Ib23tLF
	nq=a41\2<:-jQ)FR^13'>@0\S@8#=]!)JaW`'m^7n9\$"(Ed[N9p%LpG7s-k`[!Pfb]=WuZM`AXoDJ
	lVR<$m2>//Cl.PtIt'\QfW2I]ZtB+^$"PLj^'q+k)F=)fE,hg0/[0>i+c2.aR]UU[dKkS#9FGNM%[l
	UX=b4iES!ZTPttR#enIMXR9'cQ*I$fl07H(SfVTohj$gqb:gWSkOT\6NK9,B.$.BJ8L7e6;>fRGDu>
	(!;Q5'<h08bmqWaDd"`,01,GEI$fkg@8J,'I)YW+d/IHAV:2S(6!ao(h8Ik?pX/sg?^@A0q04H.SM9
	STs_[<qZ9nN8H1_U!`2oASZe<a1(cIfKB@hgP5>:"/_*o.HB1/gh-kQHSCQ&a"i[qXi64EV4gNc1^"
	@mi=?fSlDX#<rsld$L)XnR7uU]"7!;DcQanR+V]c$()Id,JOQ3WGP17Op=(`Z0jnp<Dh!8O<%=<\k'
	B,a8=8u[hXC2'Gjp5p=c/qcB4j@K:E-ql#f:C9$\FI4V;h$T>-4t)cj><&++596nE"\9EH)R<S/[I-
	q>'h"p5r*!#qM@cqtg/.`Q)@?LeG0K.rHTKiUE1@IT\:i)2<.Ch6XGoD>Vh&ThpBX&<"L?M*sP"*uG
	hS^TDi['H]G^Ye7W-e_&EQ=!o@CgO:.!=K#9$M*rO+NT0u0?a*/1>5dfocg^@o:8`I9mC02uCRZVY0
	k<&-c"="U.bSVr)NEb)23N`C<3+QiNuus]rJchtcHa_,0o:`0#b=_gd:e]bA3-:<h;)pIQ4fog.,;R
	e+.$9--u+'SH?+&sN*Sj#FDl(Nch!Pu-5qlL/8I'\-1?"^cJl1V50IQE4O84oFY,-qS8(>JHT"3m<d
	Xc>n_2\8!2-s9H[^EM_Hb)K,4Y?X?g%$+S0r=Po&YcG01rWj#9cbfPP=M$9.#ipV:+aS?b1>N_S?\s
	huDcZ'3l5WV,H-ZET<#.E>l[!m.9DdQ^3qV27r,;BnsSL<O\=<jnnf@7j8P2AQb><-aGR-1V8?>=Ep
	DJ"9RJ1s*_F_`jNY<DYI:])hK3.e*6PuF^2%>'ZJACc7;u!GW_S(]93[IT?i#]h04uP6kBKR_17jnB
	1Xu?GoeVtnKp/IYe.OEqXs1QrVb0i,GHT@_r6@]+6#ukiSe3Mf'Y*%`!<P0HLqJ]l2^mt?LROYFH[*
	Gf$8Vo&8&9%-Vp>lMNrFuIroe>eXjV]Ln6h\Ne7A:ce%Q-:JFPTk[?OQQ=[P6[dXb55nMl.h0-Edj`
	2B%"$[YCa5NJXa^c"lY>d!b?E_]<POOZr7NdAig$H%XcSd^@Ys,Kr/AoBCK7ee.N@paA^Tk$UNCL8-
	/3ZM(Vl+0JEjDO954+jka0P("mdAf_=uMkhR3`c2AnYi`+8dCNbOM!@p`M]ki`Js'->KefN'nk`:^C
	]@j&g;baYY_,+h=610G$RI;JB]R"nt2E\Z_p/#M81W-\^Pi:5Kdb.)He9H?!G)@r#k\IRt6m_83t2Q
	S5gF<kVMk1[uMNIZ2GU!<DW/ONZb6l#cUI9aQDb'HVH^NmJ%7!qkO`JflJ,8tXGmoD)QAcI1hmQ3\W
	RmM$'6p\+<"Q35G0<@X8Y=boXJ5Ji[s=b^@(;D!>2PtKTYf/GfSge=rPKYZ1#p@`Yh*3RmATkR]*dA
	YT1V,Ab+aft?VP2jLZqH88lS>atQS'<iR4P\]GqoU[/EC*5J((5g_H[C,4EWR_U8u'dUFEHFQM[r_(
	"1;11<@X8+V1ML&`!HrDUXA%>$k3ZRVq9Zfp_6(,KM3`!Za1=)7^7"<LQp@so#E*i6p9;L?n`!!%bg
	=DmdBMpP4];kI"5le]",A>80gsTp;0iH:"(GQalRP'*"L*hi6;?jR*4U"\$n_u&7.Vl^flDQ+:'s%!
	1-dTmHm'[3od<;_8)ru<[sA^5*&WeE/@D]2+UT/>V9)-Q2%-`]F7gh'X'[g)?ZU*REt'&fke3(Y#Hk
	ja'nZT"@81r4rh/?ZdW;9i/$$,EeUt0h!Lq9.@\Qt$ADBW`%VUC7cTo',Y8u5YLDa?nG'B61HnkYdM
	rI;!<<,i!pB(r#uR$,:T=-0\r$#;rX+F7a,DAa+P!gDM_mZWs"RipNr9:akO?5u4<cWCL\a79!!!!j
	78?7R6=>B
	ASCII85End
End


// PNG: width= 45, height= 15
Picture SpHrView_pict_circleStop
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#3!!!!`#Qau+!7_T;M?!VWJS@J=:ifN)66Jig6T-YZ2E!-B/hm>+.$8
	Y7cRVC\'ZWEo7`8tnKn[JkXH4M]Yu,*G9%"R<f4O0iHS]Dc]\XTAZ;n>j&f\S$qupo%[8KCb<Yj9)#
	!M.<WKGE8SU3QpHg`uNHS2fQ.4oC`H6Ad#=lIc$Is,_d?hK'W00U:fMJBR[4>eu^=aRp&V;cWaW`B&
	R^K)<*D%0b#_F'c+jb1[--WY8bi[s+9/K'NUT*TXEE^AY'V8ir@X>j)8:W6?[*!1>tYWqZ9A%u,=>%
	qDj3Gn$1Z<"W!HAP_LbY1ZG'lUo8bU>dTN&*eV!5/f`*2&3R\n+<ur19PY8[=M[a_[-\@Yk4ZEcd2M
	/Cem"L?Q4-'RA7/^O<MLrk:^AD\+B?*TsgNW]&gO_f7jM*9ZW!.N[M1;p*GS_n_c*)^VV'%t&/3H;W
	+"gMjS2c1bi"MZ--]iblYaQXct`K>oF?=X`pD\BiPC6rrEM/n&B/3omqlnJ_ifB=unKk$S/h!"!U8=
	`XQC/gC>r/gCCAV*B,'!#M4u6pXdsSF'Tr;]AfhmR@#eUC@>-@1Y>4fJ>E2:=[jAS3JcY@K+YR,rNf
	sNh),cD%bS(9*WUL5SDe5KU,Z>fDC"VD\OaUfHd0"?9]M[k\dt-T0E#)F3f#LKFmhpT5bSW[nXYNr8
	5D)U%sWt6dh_+,(s'a70o#$.@!M0oTU+W&-S;1E?(<;V'JXh2E=h@a)Q\R?l15SaU"Ue8tcPX,DtL&
	a[n+Ob@\)K9VtW7PY*6V@B(-*,cOTg1`D@i?ne(p-BlPZ0PE9]*63!soG`Bnfj$-7`+QF0Hp1miQ[,
	0XZNq_B9Vtc=[9B]SA29Y!7\e@=0IRWmgEde8/sh.a9I>tHjbh#$:"(4>(l!UgX4:sU:'\]V$jM@#E
	:YfCho$'q#]4KuDes5(Ma`qCJR!ki+:)pj&.A89*O8SB<%aN,c/u:90rNKG>G>%GQ%Hj1_8/F6;?<U
	"36cR?GoAH+9VG:-4J'sJ;6<lG/!M9317:fY&;\BePtHbj31sLj7JI$WXhFu&)hK3nEk,AU*Z,_2)B
	)L'Q.IA@RR34ak78%4!eC>?;5,H?EH8TpV,D$8>.n7G<ib>&\O7WmcTor978]mZM4uDUU0*_8s0CDH
	,9nGhh;(#8LFYY)`De=p]tM+k]?>16O?o&*>V=WA1ch1&VphVE.a[Q:aTD"O@ZdB5R\o$g&Hq(aq1a
	Z[/J_"6k$OEcPKDn_p[?#YYUGC'.,m3m<TJJ>?E;U+PtIH<L?#clMCcr:bN2X$<%Jg.j`71qiHZIfU
	eiIoLPH*=##$Am25[T.$]QQlCMR]-5("aoMh3)@4KhY^ghq=3:QNJ(d:bAPmM]!'K:$6'@^2:qVo.J
	417;Yi7O!s3%#0UqM>mMR(<sZqjR`2te^[bQ#4*!ld:>$*o^oZLDM,cu;Q6Hi.7"M&aq"\2LjhO[O2
	p:C80j\LMP;M_a]pIMbfl$fA83RT(ks=LM;NdZ/X8qJ*65?95nGrDc/\=R*"KYgY>?Vm'$5F;Eu\76
	QVU@OU=!X"&h+TS#U,I*e^rD32)V42X04q5'ncc7PYS-K5C]8"Va8PWCA",ZG6S9WNCK[2E]*HEj1'
	1l]tM*"M+#MJ1594$TC7[k^<l+dXdH=PNXLP-^\mXq\Qf>&g2&[^hs^)[1`FW(KG>Xudr[,VgY1A5:
	IpZ0bfn;Xd#_KbD`ZtsL+s)M$jPV'I4GST.#J-n\D;&;hsYYJPtL/'fHlD!78X9p#<P"/\Qhl@"Kb]
	d5<n&SdrY=<GZg^P755>ZXKShV1sI_57\dM@p[6ttTMDDsWjnQ'a)TdKG%YH#Df;gWW>LW7R@0K"hV
	RCp*1Ea3^3mK#\&\)&bH`aele]=%h;-m:Q0R'663pR@Q"2Y\Z*=mt6jSB`3h>m6o-T[#\O)o`VdI)]
	O[p`_]CbgjNf8oV*"KMaIdIAsU0'%7_r:+)`l@Q=YuE1n(`8X3T"_#C\WkH[*;AI3Mlj*5QmV_-q\L
	"`b[tq&\:Y>"LVp0nX1;IF_m&t4q2pf%S&jT''T9q[s81^,CfV)MOsJ)f`l?$<1`B(eDN8-B__</rH
	rs`_0:0jCSaPgI^[CCPC'nUhi;NZPn%f?1.[>'rF$i;^17@Lk2[>R>V=jNSmHnbVA_h7uo('(iXhKm
	Ant5s[BB(KZL3`dr3LGd]R;?bYDJ!M#?naEW4Q/9&fBYR@O!omR9I:gAo/prtfl$4&8Z1$u*^.YXZQ
	.#R9Vr%**o+GU\%)lYM*tgZfk`QNThrg@1=9c7V,A1j`Z+IR+geknqrGj4O;P1Q]=bf3f@SVZUK](/
	g$Nsr?D#F&P0%5c(,M@P(Tn)u&/pShrpuc^*$-0,/1ZuX+$P'31,@9G8Zr12?S7[<,*<ic*Q@&YNoQ
	X)k=t#lU8WS?lX/R,lL+*>lX0\WF?3W7CZ@CWe*21*HOGNoQ1Y?6i?n:m-BjK45A"&aj1)^Fn(tg/s
	*?2d$UKrOfkbgmS]PCtPK>)68WUp@fJmXl1^KdVY?pVbn$=Q\1`@H"I+WOVUB[qH:b%aPA29]Nj#Go
	GeTJPQk4ABjIe]U5-u^0Xhr"Il_#pI[fQ;^@6R%XP*0*XfP"MS1n[Qt[]Uhu]&3;j5HM$D5__?Zb9M
	S>8/1e0^-)"9fm<3h/QV47*d:fnfX%T9eSTf&uTikT(>?b=l3)6GP5(*.0]=Y]*94A=I#_Fb"bu1Mt
	'FH.M8W?2\4lbe)T7-ia"c?Thf=-@c)TZGEVped"(?=q017<5FSIJ\N*[lnlp?Yq]<+9!REcQ2HHFa
	OqBH5&m.[A3=MT=bta2\l_2-<YFCMZJ8dGKHRS`b9t<-nTDI@&-2j*32C$O258U0At.9i"QK,Y=0[a
	N+W]e*l/,Hi<fK?E\Q6SX7*P.#Ms"]K%r2i&K#"[kB9`JD&,Om-3FtErg/rQt5CBjsZ*Z,SG%Q^0Oo
	G]$`WE&;4,g?=bl0NoU%ds3/[qr&].P><t9r%&T1gX'Jh0Gl7>V,SF6a>FV(MWbXql.l>$>p)CM*B4
	ki^"pR?5I/E<?s7jW!>!Y3(@3eE.If+G"@Y,^.=QZF.BqX<lZ!?.IMT=bn?r.C;[9C8R\6*mc`F?*9
	PEQaoO'u./D_K`t>-2sfEUg[0^,nG>$j6WqPEV?Bb_![ha,_<&NjIDTe'lcGJI@7Ni6?!?6r=pF*ek
	QZiGa2;eQ7eQ4UM;nB!Zbj@,S,TY:h_JCMViFJ2g!jSLKH=kOU8L*Q,,[rVa"+0b@0868:P`RnflBN
	fH"U2F^2,0>i*hd(=Mj(ks1>@V067FDQluq.a06kr:[`d;$'hj4@bFbg4V=XK2PW6)W2rEk*fDPC,R
	/Z*>GXJkDkC>[0C.8dg6t3oVBH82ZQ_j*5clpROBi2E*RI_4\^)3:!07A#!S@fsS4F(o^G4;sfP-66
	rK:Mo@G54,XXW"9Oubjt_/5hqsJ_+rnYro^qp+^,?g:!/Qh<s28oi^VA[fE\S!/\[d"-mp-CJgq.dV
	m<5rjP[!1K_f:'GFAsTNJ]kdiUL84uAp8G8k5L3o$(K1_M[9eONdJHGS'=FOo^;/8KmlRI@9-E0Gb1
	6m,idqFD$tsHKhNi*GOHS=&E.Rf*fjZQ4[i5!5_/$C]j<o[1t4F'Y7$!!0KhRY&./Vofke+ZCb94M+
	RJL_+bYeD@lm3GH'$525+`?/FEi%4-kmS,I!Bm<WDl8_o_\T1]=g?A2LsQLmdBLT=a"Y!X\u,T\.Tt
	Wk04j45f^maF?3^^Lcc2s^Y`ic_f2a*PdN9O^V=,('IoB*cf])<YPm1jEI;_E:aVA@MoFN.h6X,e$A
	;5('S<jpaN-n"U^Ie<0fM$]P%6Bf9rHGLY,!Hq&r?mG@'s0"@B<oT7Zui'4*PW`%u8%:7@cX8$$!1O
	'h&^dk++auL1,,5Hu*U-'#&+Z57NY5U<tC8o1l3ieUG0S6`8Ddm</c<23@T4.TJ*fE?+`KI+N?)Zj8
	:N$\iFPR65oq'=#VjZR7+JU+#a.olUc%0b![NrDKW%:ae.;YE4CWEEmF#DmP%d2[?jL0ar]bk]rL`U
	JX;gA27:`:JW?K_#b_r!a-hIB;eUZ#4+4(7+0uMh`6))0MF*c>:^PC:a2#]DnKt*DBab`C:2l8mi9&
	O(^?SRHp6H'9,*@T+X='%XI1QR2S_!t(5nGMV<[S8Ka%VKOO2P.eTD6VSe$Dq//AVRj2]3\FdK9?_^
	(F;5nG"G7n<(i__A.AMI@@>0Sr2lmK0Ul.\3*"^r4s$1p1Vh^V:l.0T)T"YG(q0)F4oAK`Yd6h!SYE
	/8uR6a)8.aXLen_B![Rqm<6*'8k3>--BlOk%L5<+k+K$s7&*UM3kel)7pRO.8u'c^MIJ?/XQoRVKVh
	]u+bU7.6O+D(JGt'=n$?ZC:G%i(X]B58nK#M$LtlRJft,!Q0Jhg2q9?6b=oh19J1NT<nu'ijU.Eb?.
	#PK6RF)k5;9&3T?#L0@;,F%;RDdir2@RKr*oQBfaALQ&1nCN%!K\FC\fC&MTn*L2AJ5!r%q$gH7@u_
	9dkhabFdc#)!5A%e?WUpV``0"hS`!^5$ZH,#$r]Z0WWcn'anI8PBpu1Z-`[u"*Etr?I"s2ORSscS6j
	R)+Y+TV3H/Nt^^8A_#%#V6+Kt,,&:Er13e7$U)3'rd=BC['F.Z.]#mo<=[;5I0)B+b`FLSCXF4]6@h
	F@Bh4!LR()9Wg\%&mlZ@B8Nf@rk/Kp0#Cc:s!]d)8T3PS^-u#mf$:FsE?'ad:s@^4at<^`VD/Jm7Id
	GpLJ1,F_l<n0SHbHZX#8j<NMZa-2t2"J,%.Y_\VXrhUeE+&#dlM6l6[npMqB+md+PaeYlsUmG#NPjJ
	Aaf9,ol_KBTu+85nIZ\bt\6iWZhE<XO'5m&ru!UPXp-g5fa?`p!94`U<IYggmJd^omp*hr_K428J8b
	XkpSp^!8K@56.GSG^d72Rcu-uY!=7u4jmouL"*''9\_Od/pR<:lbJE2@Ni!sk'"kB$6D4MS,ZO-%kI
	uSbb4GPa1p^DA+:&ND0Gk4@h&2;?>cC\1(@*Af*1+k1Gb..Uj5H3d$.9ui_!H5X49pH\J2F4TlFQ4,
	.[A.a0N:84gBh[!-jiELg$SLE2N?*t<3+Q$+J_o7:,<;&OU1SiZ!+8P$t0R^+@WS8.`&@W*:Q'$rCo
	s4;dI,t3'K/UK]@;>*q-HRWg[ndE&d.F[Ga(T<>2?2lDu5n(4d%Ng"CpeaO<Gg:Yrp"ZRHRqL1*EB:
	e/c,Thl#lr!)Q>Hp4/j+t4:ca2c5;-EdG<;2iJ'(l#$?C5>.*L"kuA#QdU;gIU-(.$[Dd^spobFh41
	,';:OZ,bq-0(l!J:pp"t<=]YbPTgF.%6%ThN,@_PpagUT5nQaEt*KS6,]f4$mK]-)eQYR3aD\qqgS7
	*6Sa#tKiBa%kE/cC.56D5eA$:dTV*AYBa6D8cjcgs9k:ce&#Jfl1;1)>djc?Od3mTlYe@2Pk]mf^=1
	DmS,ASqhIsmA-reTFO7?Qe)p[e\6c9WU0$>`A$Z=a,ccV]c[U8gY7h`E;/eV`c9qN^-7]*:JOZX/>6
	oKSqfY+!2W@YUTnh/)bH3VPERtd7j=)e')^IB<%8UmLH1B9\Qguuj)nm^E,^m]#jAVC/dfon,*@BZO
	k]7-ImsP(FLq#4EBt.f_-fpZ[P%_['\CsgiU]i?:0uS@b\"3Sp\+d[NJjUdJ4c[/o?TWgJ%q\NTpj>
	&*MII4*$1/r@@4NBhqkMI,7)/mB@!5FMr4!<9HNBXjJ9n.b3/A#>?eRPg#R06;"%Gcp!0*i(Y/[S0s
	!^8LrEgn7n;qiniZ:*lp.SBcf[Bj,O;uK5nJPAb?o?P>rnMd#(rfiJM#+Z3ZSjN0ME%2?[r$^==eJ4
	2`*@5aVoJg,fdsG$%ggagT)cXD.sEhAjW2*qWFK##'%D%n(oAViQYu@fB_ODrnY21(RC7C&N)*#6D7
	4m].lK5IeKPg+1l9qH=C>mAS#FgCWkg[\_V-_rqF2-]i6II,';:*LQ4XM:E,522E*Q`;:Oq)Za7!):
	CZoA]t:ngd:i1(?[i6o)&tT)c.h"-XnrsQ_o(u%+:,$KX/T&*g2;/KhFl]k,*>Pp:ELSt*?QB,Y-'B
	//QEakW)TeAc-99K>nKC9M59tjL1.grrqh(,:Xesce5T_;i8!al/Be%bdn^2R#:>Z!l`\(+FufeV8W
	,,WPtD4417_2?nA>d5;FqFQjlPTdOc_0YM/#Q,TqS1Uq0>LG70o/NV%u)'8WqS:1aGX:KXS=t?G(M[
	WDj91_r(Ie2m_1bV-=r\*W>cnO"AK"bfl$+Ne6udH+m6-[E*V?PtE4Z.gQ2i$BF0%DYH*IHrDt;e#4
	QD78ZjP4Gh$d:8d]aYJ+aU4]*huHi3YY-Bhk@G%+oiDJnk\)5s6*kipQo4O9(JNIC?9(Muf8gt^\4q
	=5hb*l%O5)kYH#eS.klKKp[LlI@D=6F"&1fXB4GBX.@3rHjiAFZLP!HX5VMp"YbQA1!KHR]mNb`NH?
	rBY*(f_D1E<jQ+?'ItXOUT?k!V\9P\,I@X+\PtL*,GVg>XESP$.[(7:c+ge_fp[[E=aGcOVRi<Ci^4
	rj)"C\]A3K?`ZPMpI_Vpi&I["[]=f<8PbYHRO[O5p#`dS^/h_$)E!<@X9eCec=@M*rPQ8m/:IX7jrU
	&rr!4Sj%Q0og'X[kK>Nn]XE+g5lnNQa5\_o3ZO12ko2o"IfKB@f<8P6`(Ws&f/4^)$%a;2]"4OZH9<
	5r8^_(QZqfL4%.iJtM[pmnn2r@fad/9[qtp70nEL(K_/W_\eQT*+A28FXI!g$XA2]q-C3A!4JEQaA]
	tL=>T2HBBo6bIpn']Y4%@[no0eZ+]M*rPM8>O`!Y-+p.2"mqK.^*0l$3UF7J'lcqeC<!t%?!7i^e*!
	3Cd,^USCM@k`Tf=Ib<:.A"Su:7"EFJ@rR25IlJgf`jk(Zk2M_N'F,&iF=-Lm@J,/2mp%<HnAU0mAqu
	'lNq;i'T2Zo`(<N";W1)];;MrT&'91X8NG?7gG6D4t6Z?!L'@:<SlI(Un>5SAplm<4[=R7ns$2_f,M
	euTJ,Z>:WpGkpnV1_Va79M8#ONHGZf$G3Oh1qgK3$;Q#K<;POh"0^='G-@:V"1!7)f@M/e.0ous<u`
	.uCMm'mKMX.`HN*h&F?"5!*6ZOK2rm.hW)+o?pV6_3#6`5UHG0<oK*P->cRk#`,SF6_=Xb,Ed(fk:K
	/^&U?orpPc^mFB-c7W$4[&Q.hes']5E2`2pV\c0_83tbp[=fH\BSXWHG(A\kGh`45XBZD'`.UpA"c^
	!j3)>L:JOYkBb-U^Ti`j`Ci!p+DppO;P%7?aO4W]?\Qi3Q,/gd',SE+"4:I"_1eg?<AY7:-0eq)J/Q
	f#DVpg2^+;>/_.MZVWe_AjEE&b!]\k8es3#k+E4G6r>NoRKMmE!A)hs\>X#*E!D,Rh=+)]K_8)]Q"C
	.%"^t^V?tUn%s8sPtF,"4.D_/eLg$!Rau./Y[:;/4,Lf[Hi<e`-ksNPN$&D*<9j!uL51QOG5Ht3,+2
	,BD/DQT2lM*+K7.odf'.:bH?I7!HmB>MoB$L@===<&T"PqX;$Q<#@q,TBH.AA$_83seWMuq%g"5#>:
	_8;%3k`#3(QCmM<FC;%N5DH:Ek/hp;nQSER*4Ta0a07:,*D*^qYpKn_8/FN?Ji2/D3[^>)NF1RmjA_
	S.!!-&B\IEO,=Qk%8u4jO<2!-ep3`h,7GHACjn&:1*`#F1Pc''D.[S>6LPH*>#KOm*eJq#c=q+[j.$
	[E8>Riqe-ljt+9)3iUnpJ7XJ;Oi7dlBBIk]45:@=I$Gj&85T.#No&is,eoM*qE>Rgi>%[4YI[eTQ.\
	HbEath%"Q`f;@MQ!1r?Eq+)cOE,Yf;`!/)Z^huFL,SGsUAErLHo6<B0ZagO*X1H6R/=Xal6D745fr/
	nA-LB;4PtLFe]@ZA&m2%&]an:J)OjKVqM5$s/j5Y5B%D*<AXKLj^L8l,N7Un3AXB)CDA-/\+bo_@$+
	G:SdJSoqp_h>-d6&,P<V,F9ZU!W%GB!^0"Yo&HiKgO/n>qAEAXAgYs-RYlm.\"ZuMacjs!(mEl0\G4
	u2Rg+uXa7\[k(;G0TrA)O#_`^bX*(:9S6&-,YkouO]:2iXLZ@Hnkh'okiDX-6A.;'TOt,4U7=:;Qjt
	m!$TG%S/B66-4#O0VbCbTs:<<*"!!(fUS7'8jaJc
	ASCII85End
End

// PNG: width= 59, height= 15
Picture SpHrView_pict_Filebox
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#q!!!!`#R18/!6N@53WK-[$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!#_D#6pXdsg?`*NN#Yi@5#p-_e'g,5Z,[?/iAW_+-
	qge\?5'"Y"@Y_(=s[AG+^.]Knq&'[./UeLMR;B*[m[jN;hU<66pSnPLm>83/4@ltQTrSMQG'TE(bc/
	Ta(*G&cZFC0ja<uOWOXfC]@+1nHD>h+;q4mWDV(ETDOo[3DNO$AH1L%\s#1+1TR67j#J)9YT):kJjN
	5lSog&Q3B:mG8=&pa0oDO-1)YH.&=[n%[JAO"9p?3p,XfbQsfqb!reS6u;J@aPh[G,P4Q_Z)'f%+N!
	gZ)K0L#_AUibcWc"FQ4%)[(`i:)q=&?]l[6kMKV3or-PWn/ZRc^(tp`Eo/j;^L:Y`KEg4U^jXkC[4%
	2Zpn7Y5]g;T_?i%ZYXRc<c9himYcH[JWp!m-iD&8C!00Kd35l]$*T?_rXm^qR("$h,#FTr*)-i3<5b
	Uru7^]Wfi^i=aPo*c&7+"2)J:9J.^2>Ofu^<XN1+@DWSY6/Ic[]b2j)3AVBn/?od`rS@D_`mJhq_AE
	o^]9ss`rJZ@?aRXfTKT`lO1pYr4\GJ*HS=dA\MHBoo&R,1^%BbE<AW>"daQPQoV\:.17t/0VA5CFV;
	tI'6,EU%!(+uO!2>2LdDu:naT:<#FLVOif&Cb42NUragfML'_)Yc,arnWJ$ikes,bW&h8e^!8gcY<3
	WKV#1!2KUc58YAk!Z9g7Y;-[a?:7Db,W^DP\?nQV<i\Y"5568VEQ9*u'.6P&qo'@Yr.cbhm!2_S00N
	;ud(f7U]JD`(i.D.4iF,'T_h3X=#;@!n!")7=&:ad8V[<cIE@olFO8paq,nbjH(+?,cKuAT^#k\t+!
	h!$)FWufE%CPX)\>[+M?id@/XOrCkWddt4m%Y*`H0Wm.)16T9Iin*BV%PrfkbE7Y_M'*%4o`V30>'!
	knj)r^XGiL;.N[g2r5L\d?+Y=(6T:L<`t&b[-,r;QYX(UC?"a%ZRq(>$7B@tNW`r"4J.L'9r>C]%:]
	[5cjAF9fF0>dC;]AGrFTIBs%CJt62i`[+`Vna:c@,J]r0H$CTahugn94l.%imYs-Vp:JYUUQ0KuopQ
	ea_):eu8:f-eA(/Y-Q"0p%kb1Y#(\Hcm_1iW-474Gj#,ZNo;/A!L#Qrn5]c*`5^m,!!9;*=J2\RHrk
	/]W2-pE^]4Z!9ELS)Gp'@cNAQ"1"p$uFo-!Ch3h33uB`eM$m(W!/!.7QgB[2$o"N#9cG-a:#*VXc:<
	RPsOc>>R(iH\QH00N;tcrnU+^a0Dl!JAAJm%N-Q@T,Ss<L(q\;4M:P,7\"d,Eja!h!fSKS.+2MB%GH
	q%HL\B!Ferc<5Q_L@:0,GB#C&R(k(FhXHMf4eZ7+?!+4S!\&A=Z[FT=s)(;UOP@gOUF5HsGqV*KF?<
	5*>:DFC^L;ZlRQ^7+`K:7*nq11gO5^J7/D\&C'=^OKJoc,%77^F!)f@C">q8)LXmdA3H_[MW!(^O"A
	+N=$PJc$X`RD:fc'b[O?8-bG]opUCIqfUcanSdN(^WJ^TI`*"'n39H>kr7Y[b[TGl5le^$-TXuSL?2
	pLBr@i*`&:n-?7lE`U`K^_"4STc%ZDu4+j\?sGj23eJc>(amlbihS_n_USU]#NFC&<Z>a2_nh!Cl2A
	]2NI*OP:Ep<ig9I-EX$S8d#<JCF+q3aR-4GbEqBmG#,=WrOL'qQSd,c9&,7\?J6e^AX'_T'>!IV-@i
	J73p4m94A>rI;.X3]_$<@'Q-F=pYC&Nra!-Zr>`9!m-JQQi;%L<!:ftW+4FFYqme\[3hrJ0[/.)Lo?
	u)a8ot:"o]5)o0*'"M(GC[""pnfu>8?eQfBsuS3?o-1nO>o(r!#he32((afe-(j4d9eB%]E,/`I,pu
	51/80S((`UIQ///k4n^2/[/d&IBs9,m%]V*b.iTqmWn&2pa#^PK*QVhh7NFZ?$G*nkKfb:DmH@3<DK
	.=\\(>j@6hGn2<p-[;QEZiAue4W<g-^X[`"M:UMXaAN:HG(8<_.jK0"Bt7m3SNlU6MXA*fQ]``F6C;
	MM`&boZa+an*s0:eGhq54.V!ktCpJn#+cEJ%\MUERR(OU(=P[VV>ql@Zun4)7>sq":`G`]TuI2K-KO
	jW7GkVZ!8POL0J=LB\1BVmHaN1NR-i2@#"E#@Uc(%#kA-lEmIBS?iE;t4n88*c-9QIK>,B%cS^b'3c
	P)fiC!5j5PP8%7dALhVppM@<9-kD1)XP4I"_4KIZR=$#?VFO;Ct9\#YQXJ$.CH$La>\Y8`_ou89NK,
	qh%kb4@.Dl6,oqc@#eQ"peUbXnKpu7]bf6F&>\S;[mcZ4:+3-oW&E'::mT!#aW!?=l?pDidX_>[k"2
	$/@KK+r$d3>IhtOp[#798SQ:eA,::BNjG[>^m,%sX@d6Dm;gNe;A$==fN*]5C+=e4XP1`+tAZRbj$5
	q#sUG]=e_q6PRAV@/_+W580MVE2/I^J.-&-Buko+CH(+B8mgm1fBGi]ZUq=6+#(:OmXrMr_Fp*_#q/
	ERg29Y2CebJ3qd?N#EGogW/OlPpP8s:"BfA*5CVW/oN2(f;`R'+0i#K1s8?LuRO4.:HfprDmba$0i-
	FR$BI3N.F)s2%/Q7fU)5G.]Yt1qV(OhLW`OH=ZJqTP,JUtI2=@1N:gKeFU_@e3<Bri1KS.-mfO>@o5
	_]WMe2HbX9aXHu>/J]4H#"c9?$U@3#+n^2okkN@*g9\+[4"!PnpG=l4M'iAq>eq>_r6+08cp)D!irP
	A;=*O^u</X'])[U5(ATch#4E?*-?bTc?d=`3SB9H7hSsQ%'C7t=Ir:=jQTDI5bGFnfUit%YA7uS1K*
	_=,q2n^=h.(lqf,!nNqUe=7(T@(J_N-Vb5nJLZLG7V+8*ND%*YAOF6n@r-i;QVVh[4nV?d(3&iln"r
	n@Nrk[1p]/-:[MRj1WM$rO^1'TC@YYf.3R1m@PJu5gEsTdeXNX("q@#n?_JRTr=M2ApgtR<N=B3',^
	(;h"XsSk*:YN#o?u+W9(^iY<,;'e-X&OFRJ0C2,Go_!"/(c>=j15`+P$m=`e+?"E55ic(8(W%9./uF
	6atP5Ht=*O"H8e5%05LD'[%5/[U#dX\&"l<*=UeEaqAn"<0?HKUk0e]ZS6`;+!'B*ns&.a-*_15mT#
	^hg6W@JW_s8?NP'Af[n89<-`D8rH5^DG4^),>!/S+*SUhCh2BUVi_nJ[+ZiF`[6u4i7%G)I6i8,M+d
	O1Y>dGC@YWSQMl]r7i[_Foe0:ahZ'W\tr'LM'_r@HiB2DO)R5*S)!WqQti`;Fd;mc6n"+!Q<mE*i.1
	`8norG@++A;[(NasQeci*m37+S@Q(RpN6Mm:RQN5d%HTi^=^:p$oBCT*!'=]M4Hcc(,Q\hS[+L.A!E
	cZpW_cfQ_,hW:CVR/*V11Q%#u-@;BQ?V4,(KlLBL?C-irf^1_)Jq0etSfs06P\[N+j>u)K.k[^I-b7
	,gRAYBgcsU=-R]^1A^@D"^:.<FQ(@lI<MIV&g')P5`-qV*?"*0O/f%<kG>)M)sFuqcf^WYq2B4in@p
	F)[?Kh"HWU@"$8iEU)Y:sdhCfNeTL7:fA,Op82gI4Xob'ooKDSc,1!"sB!+V=\OsX_tB94*JX1EIii
	@#$^F46t,RlZCo<EV%Q0]NLZNI-m[ib#'025?ookNUE[!MsV-Y,r/sj,8h;BrNHn0i"5u+l,Ek!L-X
	,V5d3Gcc]ps=ackk*9]]cQDLQblf2c)4NWYq#qlfRXJds#27Go]R=N]1?UG+<n:+B9fn6;jMK^FN="
	1[#Ou,e=<HQRBq+\:XZ^<ISB/;?scj&MT7pf`kjs&X0UeAfPVRY*K9E4lD?+@74,>fq1k)5R-BX'Xl
	6\c3"fs>>7BpDLRXTE6;q:V@+24s1uX90IcaS8J)@7MS12F-N%7-.'f%1io/-KXkfgTGm4A*9c]!hb
	9/""Xuf7hha8J//ijA<B1c;V;:/:#]m,=:WY65V6E<RbUPS-'h[(ir!B!dZ(5m'L`L5HSMY@)>3S>p
	#bDZElM/9e:<u5eKfEb?<5*>:DF<1^d6GG\"4[Yn=-r&[)*>5UR3?)QKuFt)bpB*i<EMJ9&D:l5>`<
	La_BQC'2Z'0/W"+"/\_O2COOIS$FG4f3gn32$u3blijV7b2u/KEc\Mb"'$=BObKF2&3kn(L0/(bj8t
	Q:BJ9iWceE^$e^6c"H?eJ6`c)m"P,I>?O0<=-0Vht\i"%mCI7ils:eZ:qH/R1;'bAi`WK8bN%We=UI
	Ht1MM^BU.hiSF9:73rDo,I2lHIj:(ffm<O']tHrt[M9s9Y5-U59-fXN#kqA!>j:np>"--5[6C]DYL2
	pX4KFglU0`Qd,S>nWenD-@:31KCbG'`gDUtrhg(oi64>3bqV5N^Ck]jtl@uieHDu9:tJ,K6YB5)X&O
	V[cOpA&.[;+&b]YNo'mnks7.&(>>_Y+'D=06!3lD;AW_]8cDI,>EG,KlV3C_c\F>%*_p]PY$`;.<Tt
	>O[(>?fO+%:2Q$*;WrgjQ,7sV@ft!LH(M??sA!9Bhglj^ZZE;H_o";&aQ_#ZHk^e2WY<DgQf69tadE
	*mes0GpGb4a,(ks<9-)EH$-q1S=VZGMFCH9rWn&>gi7HUGsc>EM4PP`caRd:e6ga\!nIMV7=C(gi?i
	:e2/<nN0dZeEf@*gMhX%hd'G:(,%oR7de$iR#4M4@oE&[JdKSoN-=!&'CF.ccbsptNZL8/qkd9F$nJ
	I;-H@$a`g3uTccaIUl"M(^\u;(@>5n9gb`&>o>%J@oE$u65AUn'Z=IjK#T%M*<^*k:D28_/Hfp3V`.
	>rU+-&adt9r;$V(i>8l1m.@MR4_`PM]%=\7i+#_+@D^#*U'drTVA/p.e.(SK`mR9Y:Gk(h7HaBh0O'
	1F*`[2_oaF,^Wr!UX!YF@AS&OHZlK6S(@0GYq,jOn/d?h5T)])Pr!P&!Yd-d%Mu9!C#,!Lu0$;q^Q`
	[/aU-(-0LB>*g@AYosnJeDhROH6f\KOp%%:T@g,(mM1CPDH?n-KKW%rF`Zl$f]HD,!YO(S5O45:K2A
	$YZ)b%8pK,%/jB0#"juZS7;06[lVl!0.Z"XV]2;p\Nn,a6Z-pqdU7LZ/V&Sb3+ja\A6K)jhYI3YBc%
	-(,@N$1;Ss7\0RtK"=]-hE$o$F^@(*.uKk<kC2(2=<dK1(G!6FgJ53pf;Ycr[a#t(_-b'+k&-J;'2g
	"L'"1.5>"o\b!t*A9QU4O>D,Jqo'&+F%k0%8lj]eei'ts1;)j+\AKB&Ke3WI@9>(4ji6l#,&O[&nmr
	%),4\TBHmN$Q33fI[nVauCAU:0U/+pOe/&$^$'RGY[ek._1N3A1^bkOaE^rQl+OA6rGj22bJY!\B=K
	-\P,3Ht&XN"st)H8A:K1!62nj]gJ.[_jg?kUX(1="WhSn=>C>%!N1NCae`hHE*bY3Uc&B(n=D238;i
	&VFl9;$o/'X"IEdWGP0g8Tf_#9C-TuCJr9i\s<20ccbTul"U#?H3P<!Sll&dV-<rFhW!ZGl=4jeL/6
	m#'ItT_jhdaADKlDHU4F%<<595;8d]^iUl!&oTs!PCR;RiGR[363Zralf%G@,nBR4a"k>3_GCe+Y<m
	&6'O8Gi;8V-"a5ZG3mM*d_3oa%K3.3TD6=Cd"oMn\bc2jGdE2\mI$p@'?`+bgg#Ud,cnT'VXZba`Ar
	d(ar[(dh$kK*CR.3+OS,9j=KaEK0\$?b>k9>JHH;K7-[m[^W$.WP;p"&B^%hCIVgB.W8-jUqsN3C1=
	/0;/*!J(D33r]-r`TZKerDeY3NUTJ,mG;\UlEaAUm@C53@M1l(REj9<tDfKBo21\n%C@>6%LN(/&M6
	MY+5hNjcnSeht%F9llg6B]?gbP>]*%7K;j?;X_$LY=Tg;h:1J7G9[Oi]W@-[SuN7'Q)\#WeP+SlZ:4
	<e(]]N7Jhq7^#neG-<0@FYTJ^Orf*eP,R7*(;B+>p^a-&V?RjtgTKdJt0cBdt=H/+H5DIBN"MU\^t!
	mkEAF\k]0=Dc^8A.C42"NQQFJ*odom!2_S00N;td%O'`+Gg(X;no,+#SJoB-HdC@YbC?Pfo`i*n?Hb
	??QDOc<X(E(=Qi9/=^Rd7Nj6"+"'&>ga-Iu`Yh4!1(>a$9>OFJ1/.'9"KS'-"<A<+tZ8u<<Wn&%XSP
	DZp8BArZJcT=Ng03>_8IR_l$Of+oDu&0OVT2t(cD8:2A'VgAJL04pcu-Q&2nTh/7J.Ad4QG9mXi+V"
	c0.3Q9_C6sP!?]g"d;)&X5g%SM?c0`-u;@3P6l/L3,m*8Pc=:0acX3.EAO@J[kT>nL3hY@2ZY^ApZu
	^\1qt71,1Q+IH*CjCEW'e?Z6O?IVel'e*^*]KjCur`l?MQmidnhVZm<p=Zbm?O#rLn_4M+%I;<Q2B!
	-H`$%2-gG,hF#W5j\Vt1Do`..VN^2Lp%KD;Ff3uS83rhX4Q&7+h@J$V2eNoAC&HaGj22:)]]GB<L_0
	00$Dlq<&flbQ=6BNfR:+5BUAd^W]ce4Td+TRd0B8><E^3KJVJTqJRS5<Y4(nHXKrn),/?48Td<@`bT
	0\Bd.!3PRuT4'`amE=@t(?cks.`eVPa1@O?0TpRp(Uik7"lP<Vn\RQIpZ\\``2$2rCDe1Gcsa>`lH4
	L?0-8c*h*gD?eA8mbN!S^]F904aZnL)fNAT++E^1G_U^I\tTZ`P;&-<HI[!=[Fm.ZA76Sbl)lM(m!7
	i0BEpS]3)Z:^"_qdMLSSf38(u(bg2$l&8J_T\e#-!bI-C<#S<+IV-7:-4!6`E&8EH<d:mY2>cRrP"g
	SUn1*b8#'Ic"!oBp8/0-A1J2V4oB&fiOk'q4aBMU+3Qfq-L7(a^i^j:I@4jf,+XXqj][QRRfTF(<42
	\=qkfR"`8E.Bn$6N"4sR9G!gUHD9T3Dq;"1k<DK&4A%'.hJ`Ni;W1nn)'No8uNMWh08th4X2sObRR`
	V.fSqbf6B>k:@k[ACGEakuH-=N.!,^m1ZHBLS"<IikHdnRdo4k^\F*U/g0l["@GLNdi`()@+Yo59^\
	r],I\c0\7=9d$^@X3M,(gW/231GeVpeP,\LoJEoWh52c-\g?@3<ZOSE9iX'q+_24Ce-g%H=25&+r:_
	<4:tE`HT<@9J>P?YQPuTu[!cT_'Cc:XdhtbPe]90$3'LnBPC,tDLpB2B9WbFH9%0`#"@,%A5H!Ye8e
	q">.D8;dLhNpF9T%59QR'<GHRPQ9i<A7E%AWd<!r4Rk,UAOAj:[4;>FHO@<o=(t+BW`I9J-Y67;(qK
	2X!JD?Rnl/O5rr.'"0%]S,TG65$OL6F,ui,Sf,_;CM)/@/-..d_K?9^OSq_E]_a.5QN$h;S['[p$OW
	q?2maX-Q)XSf\+t*d7>%hPlG#gB:E8BATD@sZmIeMFH(`sibX>'^ah/8S@JT5L8Eg!#>jr&L4[FaX$
	LE@:>j(#i@WKm`b4(`cr-E/1nV3LO+2)BGc^"Vn8[tDeoQ_&r_)2DJ_Q'E(]*D$VT8A;E2$e_Uj`i9
	[*%&gHkTU%T!H1E5?1\ctM5"oU%Ar(X!^*`bfWaamc,Jj'Q`PDENpV'-N^\fk#s8Mn;p<4CG.<Y6ab
	=X;]]9ZbQ2R'4%YUu]!Go.6L*7Xe\G4G.$T74@'[>6t!4UpdDO0tto*'?-cb)-kR0qFcc8"g'ueYQ/
	9;=:AVOjlD;!76Eq5,"&5f\:>X<N:ct#o(UL7[\"mfkec8D:b\WHr@Bi*0<oBq$bW9,[r669kSg`<E
	<j47kLT#br4'>;ebp?Ftq\AdaQRCm\ju'f1#52N0-h)CfnA+3+Vdd)`)ZQB%&%,@]Pem<&]fh4tJ%)
	TO1q3PnRAi`lWa^PH)G@R/ismOg,uV(u7/gBh&f$dRO;<ePc]VPgF#qAK&\m8Sp@U*M&[h40r3dp:U
	#SZi_<k!>lFKD3'^C+X/FUkS=7W:aR+CagHj^^b$$D/P)7c';oeE=[`u))F13`6BX%0#+:rf%?O(T3
	@'et,)oR]=="b6L?Rc2,@shhPKSQ1_<,J\:N%VE4YJr-Xm-Chi@PP;:(q%TPr:K'P/Vr[/#%J1S3m?
	='`VeWll$fV&FX;AWOK<*P<3pn(pH4Z0d0+6"qHh:O?NYJVDZaj\IU3NBR/8h(i3as)+Z<TE3Cu<'+
	d8".XBsR??URDfqJV86"Xi'oCMS2hbGMF?@!ODJY$ST+A&3D6\Lo-P;?%[,Z9oUVD$q]"UG)F!=QQ`
	>1;6[o^s'()s+M+;@-bXPqGm!:!/.#Nj_r#1"FYk9f3,`$XhM2B8sbn/1[DtXmolk`Lr#Yl_$jM8uh
	X_5IR''SsQ%_C^PPUI8?I45kj?]P99"[4F-K"h303^6UC`"^];2kp8pOI+tN]o!!#SZ:.26O@"J
	ASCII85End
End

// PNG: width= 60, height= 15
Picture SpHrView_pict_Wave
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#q!!!!`#R18/!6N@53WK-[$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!!u@k6pXdsg?`*LCfK"$h=H:S"IlYZNe,,YK5FL!e
	P;R#;Nr(Y;J\^r+Ja`QWmi;/V6g^"6gC"PeV7,,W\uU=5F'Go;W1(pTqkMaNCuK$+j-Pj"/.F$,_j6
	FLr$&4oBb=DRs;qs4Ep-0c^nPsl1eN>3TGJS3buu(kM4+_6m<3E!$K$b"U+mfs0-:B@-<*4-%!7NG4
	=Y'3-dNLE3aHjEk9KZ/>nhX4jimkqp)&MRM9!.ZXM84F_<G_=c>jrZY?!oNo>^%Pn3kCcSO`mCiB\/
	1]#hm*SWUOA"*F7^[L*Pb1j$q_'Pc&kUaq:3GZ^_^Q3>8[#hno,I`NO_'PbtkUajWc;lUHiD.)Wr%i
	I7n#[#\WPQCsTZcP9*F2Ym:\FOhFO!,00'LXPV#eCYT[N$DH1l=Q<0kil0?0L>&l<:a(8aKk.OF8,P
	m-Hp@?Z=:^bi-N.*RJk@1#W;:4LSePClLnT@$dEI7hi#0aCD_/cgAiB48MJPBUGL29nT"R2fH5qSq`
	9*F0UZT'GX4n+Z3-/(Le07hq;5Ok!p/osrR8&.HsuDuY`ML-Aa2<A3kZF,f=:!gg\hOO8F2k9J4)g+
	',*q/#ijE7i6;=?t&7%[,]pdAYN3,:Dq%*=pLE,NH/q94+?2Gk)+*o]^F=_07Wa_lhSI:uDs_):>u6
	-[/?<2d?9TL-"mM9$"i2-c,onL;FpBb^EPf5t"sT?:<s@]LEO;6K&IrrGtU&6Y`e2K1eK$(u;PQ2f3
	j`.SN-Y72ZN&(M<<k9#Ss0:*0kmBt'8'A3#S'7U*?M-'e=U?ZA/Y90f,s3<GTejOm[$gK6`/ig$AbN
	H?uF=MBG>*3#H'>;Tg.%lXSAlmkGqP4Ur)/T]@K`u^p0:U->Q'If%(i"FCLF;F")1C\3->,f&T$^&9
	Dd&f&YJ`JDBQBbp.N\huQ:/HTgQ/<[@7]a-i*i#K]>>\-nXoVc/RNq[hAdYq^D+aSkbN5CDf9K3h`H
	Scj.ZY1Z$`];E=<a;(U`9KH"S<S&,LEc/b0;g#];Ne=@q,Q)4[5=>#!&74oES,H<G_GbP["eK\-16:
	FC/A/Q[H*Z&ktht?WB^2/mM!0GTX$!ZfmP7>3jH[.aYp0fOLHMC$*-12TLs3hpC4<i"Al@FgiVHd*P
	U]5dWqn&$DaLXB.KUDQK3uQi\m]\@LJ.,3G]s@&m=nBl=r0$0X0+G;[-OVHc]lgMm1-:LS@2f*jY0'
	XHr:C:8"3p"g950?3h\oY12E\1a^Q6%M[-[iJ]TC*WT6A-EV%S7As1BjgF"9U&)$n$^^tjiDLa4r%U
	ZreRn1h\*'8<)4#srIJ#b[0sJIe/J:GA-J8Y*JXGm(</go8E)]n[T8_2/a1"W(?jqZ[r"$-e/3dMG<
	Gd47I:./!I[H#Y%GG:Eg9XK)t(I$1*.76%#b(GKlnX]optAo7a\a-Mp]1cB@EfBDEG?2oGRVPhEGZ;
	$8Z66%c^-0H"uWNaC%^fW8FKD`#EI6T:[0#C:R"@/ci@LIr]bb.qZt./BQ'L\=?%#e%TQfgZI33)b$
	T>!aGdVNR$A(DfYP/ZfU<:XCRD_-4TIn%!&"Mkok!PBTD;k9;V.1KR[<jWk%!`@\sFh#c)&13H,;g1
	HJ%!CT2dG@c'H!14I!sRK,9%EUk[WAP/[W\-2FASV>b@R60MZSVcds;M3.^/bRtZ]H)Y)n#-;OK1eI
	H7%`)-))Dps+bT)H8)NXYk6W2K$0X0+8"^6#i]c]s8G;>K1:L[&d#jCMK1eJ+d92hiEOMP%GM`/YR3
	^d_^$dWCol&&APClLnT@$f/hrF&b$U*Fg"./3Kgc'3.B_TjEbiI[p=B*pI$<-ALG,FSuf<5^KgMQkC
	Uksq9oVI4lfZC#8H":r7$0[91E_@_e1Rq3K$;MRj45jg-a]*PP47@t0Q[i875InC!U/+6jC$3&io\m
	hLAj'JR^Ah;X/e\[H!HglpaKoAk`QjFeclP?uPOPo`nGni$Aj*-HRu7E%90`Sg<bc&/re@a$/D@998
	"/!h?>_0B-uS2B[EbZh\*&R`_'Pa-U9cQ"/Z(3*@927NL]991GiS:<NgLek!idKV>`W$UNJ^Co`uH1
	Hluia?K/'&JPRA!tJ4i/p(u9CTO%dr-b!nfF)G*m@@4n,@Gbk[%/fNEspE0!@-PfqgC5kD'cnudMhT
	85ujSe!LDoMmQR!ZB]ZZ#B6@Y&FB!OIDO5[W,k-E>gLna5hapc_m>&jGnQF\O14euk!Mb<g`b7T]^3
	ii]3UYpQs`e/DW^oj;kYS9V+/J!?Wae<?S\79VJnd_Z_O\oDdB;:-1_Uq8Gs%5%[h7\@]Sm2`)b]-g
	gJI&N_k]&pKOe[rgq+'fmYoMUD'c1TqK>O&n1Xb;="5crFHLieHZibVWG@COXZ/X$PGa;[l5XS'ren
	0hSV@^lWYQ$%9d2*TVqG=JRj_'Pc'kYhPJeN*'@Au-G)el-;IUdo6%$^I'\THt(0>o:(HiD;L_iP@e
	qkn]7&"daaCUC0*no[rk\pC-ZUnn_?5,*=Ee:fCRe<m0P-"M&p@/om(iWKTflr%uW-d82gb*9EGU06
	;Gj];GVQG1RF5!c!]Y6_mHmBf@KOb:57n:(aR_UJu6Tb%O2+;rh=T<rQU<fdI[I;j`cbWCNi(.gq5a
	`2SgHqjca/NQ58kpRZ0'o8[J)nDFr+C%SoQW%>OS/$10KB.9BJ=Wbl(!aSa0MT;]>ZL_2+Yb[Xk:.K
	DXC@Le%FL@<?-q".R09JI9N!btA*qUK.ZRE"c!geF@UJIYeU[.?3VtsrCOaO_Hit)T>d3C6Q"On>Q:
	8jF/e9?2E-qF(>m<YM+TRCI:5MHnRQ[i87cfg78NjF4S.`iiOc>quKYH:^\&l<@c"0k&/ffi.`AUMS
	tTlhp/pN*/3_'Pc*kQYLujfFZ04C;f,`'__is-;&N(?A]qd%K;0:ht.tF<9YNn*TOgO>ga9dB->Q<U
	0]S.$o8Z_s%=G.j!]Til(64aP[\-CNKOVdK_9N!!!iV!.[2Irrj5>.4@-pS;t=jz8OZBBY!QNJ
	ASCII85End
End

// PNG: width= 59, height= 15
Picture SpHrView_pict_GlassArrows
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#q!!!!`#R18/!6N@53WK-[$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!"@[T6pXdsg?`*NL_SMWY5Xj:Bf.u/X0>t;(n^Og+
	V+T>$*m5E$ja\J&4G-+&C!m*4)mq6'f9MmUD[3Ar.tu,%=(doL,!uC_2!Gp0^jT"j?Y^!\*AqDdsmY
	SQ05N`CT>"I\o2fcN9>g\F`hn.of`'HmQL=">La&3FWH0&0u&iM(FTtIFG3&T6<>QL.WgA2TgDg/'I
	n3Y6D6_I-kp*PY32G\_>g+]i.V$rh7H#rJ+;O;KImE3Q,Q4<e<d<3V4pT<7&^9`+qnd!4gNU<Ti"?p
	S"s:1^Z"o$)_'6@oOXfsN74+#MaB@YR:Jt\IaKi.Ch3g5EpqkuJ*Q7].&>>V!c<kJR]l87JHH,]C_1
	^5RPjNcfN=$SP%MUW$>au"So'(f_NuJl,:@7I[t"GW_7Au,Kgh\.CV>&eC/gWhQ21rb136u/Y0:@BM
	Y%R'qtBELPE:^16uMGep%9>?KV0G8QR,"gKk(QnE%&a8>:ur>iqDKue#,Df6KVId`S2FrGjpp-kY$U
	uU&0\g'tSuTS_Qh5OhKo"/C^[jPF.gtD7U%)ZNUZi'S"e"?MF3gs108'U.eOXU/@M#dftEi?p^SMj'
	4KX"Dt"hJJ0D+r1T81GPL[d?d9Bn.P(oiLmVG8+*+EAML.ecJo$VE.orB9bRGLLr&p<;M85&"7.(l(
	s7:[9p)T/[:d6tN9])aBL'8o8HH?<72G2>.q@Yc.jN*,d66&4=I"/d*;@sqkW5N3t,Pr80&e<I?^8+
	Y,@]iqEmS5s<:QhPp\QkTIgLdeF'H+@9Ha:tiVi:)050(At*j%@E7HOH12W#d,T56O@;R6&<]"#dTZ
	<IsM6sd&Mr>6@g\1f]pP:#U/e8VR6p8,X#cta3A0e+fn`(Yn3k5+c:7uP>qU/36`8Yeoi0T.i#m[GC
	:;9S_3q_js1jo$]rCFKXq@=gDTQ_jc[qYDf_6;<q-4[!*$r83j\!p?p5G3ckA=;6&e@QHu4F`>bT1b
	gFaGE^.E[nhTGYAT5G<j*Q6jOfnmW(*YPW8*^JJi;*t2;=IbV5PNfC-nNB&-RDnWSt5H98,g.LErbW
	YGeZAlg=)Q-"Cg)488+@d(HG]QNf\n"!cSS*J<bKnF3?A#l7]!"DtS#JLU_])R@[Ma3L5;s(n[nr(P
	)d7RE/$9ZO(S3PK<'R4r$3"9PJB]`9:N([5Fsb<:2T#VUf'Mr&\aqAp5)7H@3PCQ^JN!?hbq^0ZCLb
	EqA![Vadm\J^,t9hfaqi%4G@'U.S09'=,M.fGB.q,1W+krOXbE/5r+OL^o>'Tg8uQ7Z>?6G7T#7#d9
	k6)++V8HLR9Qp!`>I#aTp]Ir,E2X`d3!9SVfL9pCE4tM!Okg?_(6UUg[P)UUk3\iTO[_K]"/n`2O=H
	up82:Io(G4!G159XiW&GZ+Sb&9!4?S1P^rUhH!jF&g=hJusQ/.0'#@[aujXG#kLo7l+,E=Rcd%ZeZX
	Eu16h1:;]F5)Eu3S4e0O(:JE-&\X:P&@j0-N:ElK6A1FeP^oHY>grIB6,1/UK),d%dpRkj7:)<6H[:
	!U:J=H(MMQ7TL_dlO1>*2O?q1&DNNi1"kD[TN$G-jY(0c4Grr2o3&Xf<`"HtBjnN%(HKD<^AE99:ZT
	gOTah!3e?"N>cX1U>&3:O-:_K"+M:QMY^KSj!,72$jcU5<P'_M1jjbiSiaTmH.(/eLgOZE=`e.q=8+
	.Za8,7JY5,CpW+dT2@8'Y8g/Odi83\\grk;.=M6V0oKpO,IbU&6:V+W.KhNgl@"/s2QbH;OnVQ.?;d
	X:ChMa189L;N,>-nQg"?DU?6\o(F*$I<TDPk30N/oc'K*<hE2G#0^;e!$0[cbu/0-FU;a>\Cu,3a[;
	OGn<pXaW+Fm`(UoX6!!.73njDNCE1ZA;mk.4.:D1SYr\S:SRRl+ZaouN7n1$l>c`.o$cQZhcnUqho6
	%bi6S^iRlPYYEBCpYNR6O!='mHDNmT#?:(!mG,?3023S_I\$a1Q-N>Uf%H[:#;JaM1!CN#uG6/r#u+
	iIWuG]@`FOsPD-@f2dY"Z4<GZ`(PtG3jjK&3iL/Su-RkZ(IPV9T>*oVl$>g&-8HuEpqo#^B)q+*?G*
	W;Rrd>G9W>3bN4,fj3%BFC3JIhYVF%(BS(T.`H;+"c19tr<NH*Fja\EuCkj[kUG#`RTYRm@<QF@Pdj
	($meE#rIYLB[]]u!a>2cY_/+$P)I]bl-"cnW-1clETJ:^(r>KtRb;S;aSISFRURnpYV&F`hua'e<=D
	,Pd(i)\^^E6FoQ]#h\77*(!C+guYaQ!_;_JnCi,.M?d:39WpG^>=!.S&(,r5P6o`Pmj#c8Z@:%VcK!
	'[0g9eaTNO>D464`f2T(m)Q7\Sj,URhNCA?S<$Q#Pm&31Ai?:.gF$q<T,ZP`0NL/CKd:/=Z499eDm7
	un_1j\&#uKT;@_.Ujiho3"V)k)B_c0b_jegCS"[F61Q0'+cXU$9^5%jinYf&a/Z]0g=dI1<!\!@t^7
	^<Z^>o(>OGeF:E/g%c=m>kKYDAk`7;.o-K_:4jS2UpMi/cbi@L\^HY$)Y9\Vsa0mWC4F")j0FD1'([
	qIDNWp/$lq;8'j6jk6F<r+\C*XF6N?![QOpd,iQR^8B,i`,Q)cauV7*_\fN[sd-cJ*]E`uX1U7/8$Q
	>i=&,97$jJ#=fn9N4T\!Z[(5HK^%9]Q[0TWclo--3K-c)4M$kuK%QLH*oZ4,P4P_0A>r#Ah\\MY8^(
	h[58b_9e!IQSS`dWGc[X^=jgAmU1-9[Gn:k&gQS4sdotYY4Ab5+oiL`UVOl*`mCU<!G3..^0-$8R3b
	!O/\@[C_s8T0i)S+0S2eZ2d#=ODiSh;-mCZIBWLDI2L4^qPW[m(n)V,RNL:l.&)@`1-NXae:o:mToS
	Xji_co)6*Od/3]8bNt#P&5"$R8Q[HM^@Lqj!5g"<Jp?g+/[r0.te\ms8k\V_S:Ej))P@P`c2T=oRL#
	oDJ&C_h,<lA>+\:W,_R+jk>_,??n.gcgY1nMWf15@DB]e7+ECqF65,n"YZbJG,7K0nW9$:+NaUSC%/
	HF%&iX$k0a9YAIs`2rb'KHOX]3/HCF01g7W.:X]ga\c`B-""N0n10RK,FKOlp#;P2`f1pAj7]=dZI%
	nKHe)9?Xr1][5+#+q)&O4c"Gnoh(3Z+]7J&-K;tTWa`%KY;gqA(/LN3i/c`MdK`8&UKCP\>i`?*-fj
	u'>I\aiZ)PV!-FZf#m:6s_na-Kjue-SAaU/.4V8fVa>@OHm&^\$t]cra%WVd"_&]^A%15`85/s\2PO
	5<&ZNVLcpnS4acp&l/dU,(NrglE#8sD(?[Lnc+&2$YUoDP@Q2>.Kk^`BM;04$*]"ecP?/<@=Vu(3n0
	7RQp[3IZ+>H*^OY!MjQm'Eig6Y*b-?/\Ffq1\Bf..`aeW.@C1kpkuLGU'nO2_0X5e84,8!WlPcIbp5
	"[;$AZ'SFPapTDio>(Z\P+FG$1*an%j-ap:q4.2mpL-"!hV;q=X.C]*;H`)[BKY^]M\BPBjO^hr)<=
	GrpQ2$UlcVPCqKt:[2"X;9r["B.Z"(h'dI+_J=;H2tJhCK4qW-'iO]N[h\T?q_-URN`3`+ni^OH.X:
	[/P&Y?[GV_HL_;"JM:g2o@s$[n4e5Hs*`%%.GQ^o&RnK;1@9Xi8B0sj@b2<Ta),CPpnX:<.#JW`u?t
	g*]F<8'KC7#eENX_)m@S[5Q>`M0i:;3UJ**Pfch2<_FPfkh,Ms]=cY4O2O[YDH03>=hcDnX`kUBKj\
	(;=Ta),cbp3brGEG]G%jC^%Yjc".k//6$S@=D^EmEG$Qr]%]`L!8erf6hQ@9Me`AP*"WnGg?rjH8mU
	^A,Y`hKd"&JC)l^J,0:\5.[_$Pc6*m2%'6<CJc$FnR\d")5f,&QA+`OkcB76NSK`pL`&kES+)K9[]\
	GBaPSj^.P(rjTMh>,8D+KkXAkKdP;;@uF%t`_i_+u:TppipL28YT[NhOC;gkVNK;=B07BTI\X2n<%M
	K`(a.WQ97@\*s57BRh9b]BJYaAq;YGqO1R\VaWHc\8MG=-5#iYq`g&;amicTdqkJQ\tSkF=!eI+/pn
	./Ji^MdO?oY_,SX2*k`KfPaM/tXM.ZP*kDT=:l/m^m-754Z]NPG#Sd^6d1m"B$=0?%W6c7dKaM+Z:o
	M_U$AAgmThq]7'adA9rtf&CE$8/PA6l?>z8OZBBY!QNJ
	ASCII85End
End


// PNG: width= 45, height= 15
Picture SpHrView_pict_GlassZoom
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#3!!!!`#R18/!*X66h#IET$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!!]f%6pXdsg?b?U9-\'da#?64W)ps5HLhZ'hSaIOp
	7Ti#WL&o#ZI$Rf9.`Fqh)):"qC<tm9D9V^D@m7\fUb4h0nk4(f96K3rLII[:NkUajNTcM8W<),bkDJ
	uR<b!ajF*ZHI!slGS^D'e`Y/XKMYuZ>#f&e=J;56I%m^1:JcIn9!I+lr>QlX'ZJjbNf9?'#S"%,qks
	V_`],Ke3`55G%!O!dLmaH'h?\Q:r!^8h55C7a()_d3Mk2n!UKsmu+g@oN(Dgu!Y_,n$m@::>IFA*qm
	M$!N'"^V]6o()@t"LA)4!7*rthqQo?ql8A%0=O"YMu?Q=3@6("IK]FG"?[4kg=k72!OLkPH&sO/K%r
	)9LPLN4&9o)dBJd;>TC^%^nHbf>*-`Ose^`2O:,%V#(u,9i0;(Ni3I.KCi+:Q]`l;%/ESdOgV&',<;
	rDrJ'8]-$$$,J-:@:l\GB\91<%?uk5<o0AfAQ8/O,T%!dAtrEp\p.9T?l0BfQ[ni+%Km:9<P@t`3-;
	I"002:SC0t**;gfCl#bXGa^hCqPo1JYg:Y)2XL!H'qcklI0]?'`d5e\E[YDbF!I>3*k<ToggC-hWT_
	mie.sH/W8ohMof%(AKj`M"0^qdb?Mi.sST9\u6;?_N!J,e9Zh;,8cUh1*$2`LJ7G&<M.M`SIQ)VPtP
	*XDn-?nc9De^\4l4$2=A+!3_rdG6hL'&_kVR"jE#!13`(_"45B"&YYf];`,E-YN1;Z^[V#W0jfe)]B
	VMAkD;*ptjla5Km(Ra?]J[\N@"$94_Z/krT@b%_`(3RQ15peC6UW1W$])e4)GX<Kp`<6YT6fb:Qf!Y
	HQ9g:8PS7qcttFr'I<AZ=*F+TJ"(J$X?t[0gM0-DaGE(W=$i1o26KH;5i9@S=Rr:6Cn?dL0YMLdMWp
	IK&!K_41RE&WLuabUB0l.h,Q(FkNk4uBoks_\q"(@S:e"D-ZsNQ]YD%hao]mph;JES"J,]jK6:Krho
	41m+?b6.$Fj>2YZ%]"YU_sl!+2X!U>Jb!CB:gC?*TrAH^c#3i/TEt\+U9G+*VMd%(mNlc_`W(],fOI
	d-K1Qo_TNqfiPn+DWg<<CEDtG&V+NQ(Am7&_;C%?!C"TrE]tQu^mF>l"4U-qP]l*P@[kddh)^F$I.D
	8ggA&2DbG?u2TriW.:3WM@-0O>ergn``=R%\U<R/gedM[Y.S54a-TM@L,HS2eZWKS_>j6;M$C*V'aa
	Gqj&<i%[\I<Kf-'Nf+9dTB<N<U#HE<2.\HfL=ZBp2*_HnQDDDl1FW?.4p?I;iuA"+n>CQIjO/AX0Ne
	p$[B^+8",9CoBQQoMd=#@09JeQVaKXo[%NeTChd]Waai-V]$"g0JojN^DW(>IJYr`2_Vt!VWeDd>',
	ZM\@7E_31$G"M#g'loFnJYf*mt19mW<iR7_iot-BP]W.U]_i=Aj]'V_^BE>V_`F]W\O-GFl9FE'TqA
	h;@,1D$;s\4R)(nBV<&PrGQ[O%(U]X6mh80T&")a<H`fT_FW"sbe2H)KB0Co\\"om<B$n:4>VV3joK
	,Lb'a8cV5<8qJt2s=o>p"Z2XXn5UFUB\luCM'dk>hP3cFcNi8AN3OI,Nb%J^(Xg"H0*^d#KS?J;0.(
	CCjB/mo0C67J@[&[.1RNG6,QMK5>]5HN7[1^B,X,c>QuDO/Hd(%XTC,!I4nb@1G>bo9KYMo3_Uf@JO
	H_7I%@V_Ukg(X=WD<lO7Ba2`[-"o[<c4@]&V:-Ud#\lOYMIXQYC\aJkY2"5PD8R5V-IgOa2cr.$L*1
	Tp_jQ'k.^T:6lS1_XEgDu,=QLFEMlf-p`MeZq7kCH\<^Se!mSa>>UHJ/+.?iL'V:4LQ3FaB5(NaGF>
	4r0(T:lXl92"7N\$dnR6*-@e/bg+L_Y<'Ztm]&)8Uj,!ckB'BS)BH$TG'b/&@c4jqWIm%`U%^mc4V)
	q>!:X-PEb8g'%/sbD*RJR+I(?6lkT?SeJGt#GiXqZ#Jf(YQqo29GHPlM#=$Q2)*mf6DGX)f'4U4*B#
	P=Si*mb"8cN+R?!PRZH#P<Q'F2.m(5jp_k!Bm+M*:W&V?Rl3M*X!P2j-Bbm!>"DM[2g&R%0>oUTX>^
	1@dO6`2$eT7^t*hYmFhftCD_<.!.-$UE]X<64)&\!49WLZ+UGkN&:fNY!"V?q*Wkip!XQS/:lXai!<
	rQ&39:">$Y[Q\!!!!j78?7R6=>B
	ASCII85End
End


// PNG: width= 75, height= 15
Picture SpHrView_pict_BottomCopy
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!$[!!!!`#R18/!1V<L=9&=$$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!"k\l6pXdsg?b@:<0LD;a*G"RO@VppA7)&:!CT0e'
	j%2[1G24(WfB55O5okR6X*c]Dfi'j*NB";[FL/TSZ1>k;lnKFLP#5@(D(TYP?,&,K'\SPP+?kNP;n[
	*&qjpBci9NUCcCf%n,;e3kIL?$pSHTtg2'S6'^A[5%BBJ_6D9T1Thm-f'r.rCR%0\j;C[rrYHKUeG3
	rHn<\_`rV5:"cf<8QEG^FdQOW@D.M<^!R\W%OedVET4U9@k7[d`hIa.kL1%>Vc-OCMo/Ku,'+8P)>i
	2Jg`LFe_"HMA7W]91hfgJV0F$0?(_$/$afm1D)"d)nU(J![SJ9p2RXu<]D(MZ-C(k9I<^'l08$.kg?
	.c]fd1cX(?r)M"i@.]=XEVl#[B?K4YVZo`5*r".'+O*<GP@)Lm8:bO<.=h>jX!1dcH%\]KsS1ilj(d
	T%qRYJHKhQ.NBHX^c_G92SkS41ct48&oA49]k*^f]W=Uf2$:k77K`f$k!I?e%pYA1&^>!lGsIoK>h<
	A3?6=/_!qk#G#3U`?HoVSLe@_G@JbUhZrT^\2T3VTL7USW3r5Ckh?en-qGRTNU"5;)'bhA+KsZ"Q<]
	eHc*TG#iCj(8$9[$VD=?Hf@6!;-n>^1M'*KS7AduU+dcb+PDNZF?m)/sc+TV.qgi,c:ir=FfRpEB4f
	@MmpNXTLY%9I@(i/hTnPm%BcaSi`)c0/'4R'FM9$MUT63g2#m'XBG-gmQ5jnFKJH?gu7VdHrp>S=]s
	EcWDko"]9iLI6["8J4?SLpK.K<3:_o>!Jn).7orpj<ZY.TbF]PZT\6&qupJr^"5*hp@WTY3;)=^i%r
	><+Yfcd>1<EN@j#2):FP`q#cN`NF<CoK&KAIKG3A9_0e)%=M&>?fihoWMg]=WCIjs-4`h7.<;_,pgO
	]<-IELSSO:oLbcT+$P!YI3]K+KlC8ho=]mb#_hJVk*\TB.<?;kiob:B9>jhU1(LL!@rV]ZLm-#WRcc
	_Y.LR?4Z6UQJ:efr9':2\gpD-DF=s%Br1K;u)WSmG`0mb_/M:J]:pphZQd700=-<%=.AXT&3u%d_gJ
	$_3ZSR^K#u,EYLuYng2KGB-*57N;JFTB6CJ\@6D6IcnqC1S#RC-n+:nmgGesq>0sI*`)`:L?t!so'Y
	>GY>(QKe^"qQF:3n%.+?\8WS+%:O&G#XND%gpZ>C:EBnd'9FoYE;\m`.rcql)BQi#.&goi]Fh7q3b*
	W;gb>(Jq8*??.``#(QW*ZdGF6aRYM`<4MnA]r#=p[0?Vr9\!11`B't&0>k9GMhNCjRepH/g#0#eudJ
	^NK'(312ZO[\_>b@>hI`U7G%WRQf$6>U0g[D`oHS]K7ef+h]4f:#\R[[AnnOS;.bmMLum^Y:jbIA`M
	UPleW<ZWaCRDC%@uHjOJ9,#a2\_2*X7V.1MYF<mqk+%m""Gq=k!WO`59C.[b)Fu9^fj*"B'`!s$^&[
	BR8FEF__+>[:3YOc!o.%,"PL`&rG\`9I=iFluR4+^4V'UQ(^R#Y5RFF:Td2U7.fc?f<8Oo<E)pokKf
	cu6dMnQ0OTEW_Lr5_^0%@^I!kou?=(@"/4D<Q41Vo4[-ta+c;lE[g'eoY9^Cm)L[;\5l4YO81i([DS
	Yo5R9%Iab*C4NUMoCGe04+RSeZ2dG#c;tCmSRcNXJ*,3[drL"\Hm^-rT)(afTqhhE.pWpJoe3U4U(K
	KF\Y\,^PG:P+2Hj,"fGGtn])^r\5o=U>IM$^3,kN%*1aDrfhn_JUS2`CX4:r\i.lhY(+%QW46Ya!d]
	-<Jpa8h,3*fH]1QK7B``#tjQ:*,82?a9hr_I]\p;ua:J*YZ;.P!$->u4A5K4B_;X/e\9gVuO$-jK8J
	LM!a;B^!#.kfsqVPF4A8RtUthK7%<Ek7?PAE`uY8$Cf`>20.d!hrj1cT7St5oTUAjP`ujlZ@ap5P9W
	[p:7cS$m+IYaNfMeS@cTg)n4Q#35RYB-85)^)HD6@W$jpD'>e5#Kf[SCr;C[rO%j*tTgt^[I*8kplS
	sBald^k"Xk^3`.:00a-Hj=u],rmtG"k_=jXVE?eYk%AU;Bb=?F`hh$l]*!(Df6OR55rZf-QP2ec59q
	8lG8)Xs+Tg4bjOPJhKnK>g7^lhSo/B>dn`2*NK&qibEjm:"+[-$gg"bg+asZq#t[3aFD3+.s*OV;s$
	ntVC]oM4%_[3,2Rd6*U+p][UGcWK1c.#tXfnr&KBMsD%TCEP=kI.<R4Ap`'%.r';dp-.PA=6GH91Hb
	cTemmE8Re!X=Oe.Ko$G!A0aQ\?3$fHfh"N>j6I)bG*U84<r2^j-[ts[\(QB<%DjVI/6"k3n,r@Id3J
	LqCY-3NKcN]oeIX`i'cJ"%1c$pdK!!N5S_Wbqa/Q+@Y7<*7/.@31m!!SpFlNKp=*X-?]duMn&:?ZQb
	*9a^2f;:nJEIkfC&LA0B4ogWC=K1N5Q:H<f@]GU+i;O]]mJG(St5tubQs$YVH-Yq">htnq,n=l,cV4
	^%_`700(-Z7'2iX,.oT6qS8?laKnAfB^rhomj!,`l2S6N1?*grHQ$-EB8PCNHQHbgRa,YO#3B<Kf@)
	.;BfQ=kLarI9i0Gpq]#La+/>gDEPPD-MikiGoZlu4qS;J7.-^ot.6G3F%Z)LeFiIV\2dM%`s"4'J8E
	]G!hNC8qYplU1*t^]2@>lKU?be=>$'2<e8Uls7I'0d]S]^a.5n6:0B+Q8Z$4a^"h6qub5F$0RR>jAt
	X.jAZV=**J@a?Z^I7Xq(pLCk=kg>TWBdA^0SO]"1)`nT[J6`CWR[*8,qGq$O""ZY.U-X/`1'p[c]q0
	5P<8i:gbUK^@n@'btj(;NDh474l[.cX3iEb3q3PABGu<!!Sk:Y&SE"-jC59nG2u+GB`dC@^3EYr]m:
	(c-=to?[j*:i*MQtPo]E+M9qV`V?ArpgW3,SF`K2k0p*b.--%&kIQW9;d1n4'5tfWnkQ)(MnVKl7WW
	5"A!%8W``s6p1!PE&<G0K1TGiHZK,R"B"p)EY``@MejJX;^)!W9)N`Pog*#7_`Pg:(uBY%+b&Ol[]e
	Y?q1nX]r9km3t-X1Ck"W)^_f<jnHAr#FsF0i25_cUIQ'jkKha*-732$h[e4N]"2_2Z8h&.O-!XL%Gr
	OW3TkFnV%Cd6KM]Ao5U)OrL"+]):,DsRc/.<+l*4a=^4]fj%:O_?Pq,oa8-?q2!$4*oO0#E6D`IHTQ
	BmgBGiUB?9!W]/!AK]FZUq_n(1c9SA*!n"#M6lj21,&`6&0%Ug:2,I0./,[&#?'1$lrSSlp-4_(*"4
	@TNiqM1Tif_A27:%'Fh(GYP//ijp<"b_4qe!6$45(FVppD1M72)-kmHM4M*+sg;h7shRn.*hOZcN.Z
	Z.%e<f't@g=$#8Ak#POH>R7-LN<PJ+qcC=cag]_-$2q#r'hIYgdu9nDM.)L6GHRfU4^=i`=mR6[O)t
	;C:cg8Cl-.5CT-rfk6q^JN"p]K4a[,,U/A,Cd\/CVMiLTeW(_94sh^-M;HcKi]q&$<rX1YDstSNIJ"
	>/(#6Q?b*4@KO<3!FKUiW;pjhbok,.#CSW'ntOW?9bc)sN*7P\^nC1cd1:PtlWY-+o6(GB+-gg)b(R
	$^upoB&bMF*"[<]N2"2To`m5k0BOpFNahb&>-jq&$LT7U';gV4@g\h+DkhC*hIQCgc;["%V,P<rQ8!
	47^8gbS(9W1`XGNqXA"_)7^fb6$M6oA.I:m)Zq6jUQZjl71NNsj:92pNN3NI6D?Zi3d[;n(b)%6@V1
	&!`8&u1_pm<%G4`ZCEQba*^9`X=5Z%IH(rV(&\6Lba@[9DAjYWl&52E&!QqtBE_U8"BsP1M6#9*7qr
	jj^8.'U4q?aD=:\X1GgNQsNTcYSYTs)4S[X"TBah?lVpO$9LYPV54e,#*#nj.<,04*jW)W#]g>tc@B
	s\4/bpIbW;TCb0"?]"ECtWUBc@K/4lg?3Z0ofCl$6*RCVnXn0$*Hp@\*\+\N-_26-_STWtcL5q8-0K
	L[D74V5iJYcohWbr=N"M9TO9,0P9@?^npXmbPJ[2DmAj[mB`7C5a1kMZNbFJ/\Mt#JQ8cTM1i"GlRa
	:)30`;\bbh.J5l+5gM[==Qg_(]\'>$&c8pT7#PITF_e4'k4t8$s)?anL)^?lXaY"o%GOFPPR"G"`Ue
	-asP&!5RD.j8<DJj?+ABBW+464NgA-^A\'+YHe0&1@N>sLZj[j`aTceKNi4[#@$OHAB?+!7*VOOb=:
	##6pleuW"[%Q$O+b5`o7h$BtHCRrJ**P4<h2&-@iXi-5>3TB;_iZcdZTo&+QcGV41rVad*^lB2`6$m
	Js[qaGC%fj#g&/]V8pQDbbY[&Cq+`BZhn6o^4i%MOI[S6tOYh^H0Ku/^+hKWP/Z'0otL\l;jDPq)ZB
	D:0Wp$3u86WoJh`l<J)Dqc>;6e4i/,9e?2pV;=^FCX[igY5dS4(ut-jEBqE7@QDbnhu--j/E]uq:El
	+rkgaYF]%)T/j^'h@*2'!MM`fQ9-2fSE07N6Zh*F"^A5bY)p:9YqtBE.Hp5<h<-H?0Cc5K)&_m_L]U
	5S#"-.Idn>S`td%:,P?.moaD!6$A#3gca=B[riL#2J5M\[o(o]3rP:;jJ@/;5fTM$&GGjcA%remh9C
	8])^7atn_l+!7,'e##hYe<Ao[KZ4U\7+oWk>-Dg8\7E\og5$=c-9F1HI)Fo^R`K^[fbU#=_WqDW5k#
	FTI!al5j2K>cin_4jP=^d1NK(-or9Jg$O6=`T,K9qo*fH&=FteW@D;2K^f2W1<`f1@:0GnrR%U,L\G
	I=ZZOETd1)+Iu)*!!J@nF5(KMPK5%Gg39jYJqLh>G7I;*F2/T$di-@Z@cc_RnNLqTU5?2oD)8t`oj_
	#St7spBP;]r\o]AYB$?W0-?PF<m-NE3LP,9Ns,?ZWfBI_^p"/:X/hRGUSM@S([&U-@4?,b^Y&6W>J^
	Yf?g/9:GV`[o18P`-3")G$2?h*Wk[!;-#rt>Y@-Z/jrcWI*nBJ`[o?/6<P%j.ZWe"MTCT0G"'oCH0u
	$9S(Zq-,aog,'LZXWLp?n<-]\+#c-f11R9gPKE_S-71&V_f.HG*BSG'15-b)>0\?8(=l^iYs,oK@;d
	u@hRoO`Y?j@t?=*&:e'0f(?=$t4?!U_\F)uD,Z(")QKH7@DB4PQ6r#7S4UgpN8,kkR;l`\)")"sPXM
	2@"7o)6h&2i@nM)K_!/UNO&u;a"[4O?X\T<AF9o?U,A_gXG-]o<QY)If/sKY>QC.a:;?Rno2Es"n%]
	-2JB;X`'[%G0Db\\,=Y?;$49rf&-s1r.EI!\a<`,YCM1q_)2$`ZWe]rn-km$'.p&m$EqJQm2V&dA4M
	2AVq;mt.]6E]bKd\WC//GFt\$uCj@-:r]W4Y39md+;pI(:7U,H<-XOE@q:Z>t+l>2!eoYub9@9E&om
	GODPaLY]e__$;.&+Hm6B8kVfiaQ7G4PtK"F0/*&'>6e^t@8inZZ1I%uPMn)F/VB<\V*13jmG#*7KJJ
	%tg-R1]dWP$84`XKYS&@';1=iA$VfMiCSjXd-F`qq/@Pa`P@HZ(89/PiH\[f8]Hq7<riSi_tZ%gWI=
	e$*;&mC..dfsVP<)5>olrZ#4-XEraJQ2AE("UO.03hJ76D;cJThm/%[AqOh".,7:q"X[\Q!i.2/nb@
	PKf++1S'@TWVeo6Iae;NZ'KS5g$AC6N\r&hJ:`.<J:ad#=3>;gSlFm0G^]=?*&G\kR#"ejq!!#SZ:.
	26O@"J
	ASCII85End
End


// PNG: width= 75, height= 15
Picture SpHrView_pict_BottomPaste
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!$[!!!!`#R18/!1V<L=9&=$$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!!ei^6pXdsg?bAMBH'qka-X4jPKMT>1hs%kmZ/(8T
	5'-!Ut^!^B%gtRV)k;BpYQ0BoKW?o6Nm?4K+)#l>+sm(VTTrHe;i4u7LT*'K*gr#TH.!8m*n=%e#i$
	&435P>I(<K7QT!*:^@6h7\_LJ>n(5nahRiNbT#J[f<<saIY,WTI#U(=G&4/Vl+bY@c63mTPKFe3*#U
	-S1ORm]Z$\TAMeWqdk:*gO0Ag2du&on%mF5$WJg=j+E1U7%kP8>iVGbr*er<GTC$Bd:bfP(_-(#5PO
	f%/D*MK'V@SdksWViCMI\Z`4*"_*+$nYc_DS!+LB"1gGm`teZ!;O'Had(N,3n?PAUY2C"eUS>Y<J3.
	>.58hi8+BE_i+K)6t19XX(@4Qk$.Y+rZ*"F?pkjYlF:0#-pn&=>W4b+$33L*3:W;rj2hq[#BT.Rf3/
	h5qVdHFdAJAkLB<`W8XQ+*<RS#`=sW8NUp+n?nTg+Jpk=&IB(-&%ih>_`B"h0-B:Qnc)Zi;PXh4q$Q
	kUVkAE/kZBBWKke-dG]I&3/aGVM3qbm+D7U(ZN<0+_qfN6oD*bLcnURk(Cn!#4Dqc_U[*dn2\Ae6!#
	;fET74b.RY`aO#]`Z@Nf),Q%Yqs%Bd,2GYeL=6_i@#XYuIdYOV<g+3[P9%D$B[)feefmp=mp<#[B(E
	7)_%HA-J=#U(dDg7cR=]'j/ia3c#7G`N%jpkhP.agUEeu@LUQ!EDCI/!p:"Q70OKKih/K-n(5k1cHg
	N2FHK+IMQZk,\8>"-g["OX.#un,+"s$U!]hHjs&,sc!N5B^"ZlIH(GB*2TV33T;cWcGd'3]^>CrAM_
	%d&k\gJjQCQ7*@o^oL`gFU]J<,gNF27gB150d0U2)[?&cn7TO?]-Q$kAl.]nE_Ub+<r*UIX.Q7^]/&
	WmDU/8Qn.\6:+E6F2f.,#>3J/FdHDPF>^0i,d4R[Z#2M!oU#PTea&2n4o>&/W1F22,[:o.ZS-5A_gm
	e-5/GAMfH^#MeVIOYHMU%'ZH7m!IY,SCg,!f9mkan$nIcJG-,>M3WO\"PH^3N5<!4cp=MbAu,mhRY_
	B*E7*5[`7>RJ#mt:XDr+XQWW6',=]r48^9uB8iJ9^OQ8RZj2]9VkTY6(>pMe-Yp3n])m(Um;u4-<,a
	VNa<>A6UO4RQVZ>>te+4K;jM$MsW<uDlo^qeSZiQ80'Qd(N:A:#(=-tAuU+s4\._+6dEerln!]f!o&
	QE%;D1Z[i>m18=[CQ9(';-j_d)d$Yf[uSK7Kmut<t5gl+D8gM'i@^I<CNg<mb5'aO6$bZTqS0X"+e7
	!"m?"EQohm^3p>p2E2cfO[<OiOI''6Aj]g^Tj;b;cjHU-S!2iO31#-?R9L\+ccj?p>C><EIHFh*+i3
	]BdB3PbG9b8ZN5gIc7c7\$1#s-8^MRu0@mnF"gJhtTt^:q;s@LqN>YNVJXSFHrGbBJoj$KqVo[[Q<5
	$?37Pltp2AjD7u5lY<lZ<HLN^<+Bl-^hLGZ1EE+smIU.nX\LO3];VE"-n47fDmoT>5_sXe3P1s-\Z"
	Bmi8cVWlK[Zl\P'r:IM9MfORhlkGs>W=pJ:%Q2`8MmaOpaQrtb9C='9CF]t>+\[rug9>8L<+n*A1tr
	jq`8SPNoYGDsoU5lE#"B6,hrgn>12Qq\#e!D[`PG@po%cg^d:iXOrgXlse."'mh#Z.PL"3\+6R;3)e
	`rjoi$<%<$_nN?oukD2'YZYf)lj^ngu/_)V'?`VuCI>W^6.?1IB:ZK!`U6C$#nDX,chEQ8Si1V.HiH
	\lAj9i3daE%+Hl,KWXB_1'(Ou+!arktRQghjg`IpHs``l?"SL4$Lt",-+]pD40j:6Ih,DFpB,FIC7c
	:Dm21FH`M&]'u,>cKk.\SncEKD4=:u=%h@Pk0*k:,lq;e36t11]5$PYHmlGr9Co,(V?*q33WZA=Wdd
	[?s21=!QS&H;Rpt7g59RFG'qK=jK@Rg#hVR*jBgA1+ZYe6BNl>H>/,:DBW.,@?GIV,412G@99qt]g<
	=AZlihS]Q@.;3AT:;CB?+1.aToP.nIf8F3#7c,+/t*M=TC4_FcR^Uu4@nPgTo)`1WUo<!n'kase^YD
	Z[`ub/"A+K8&f8g3WMul.F?3\V\tu<]LqDcjF:fNl;cD/kkK]W610/!^m!!PnZu1,5pt>K=q6^9Clg
	!d#IbD2KhjLQQbiANF(N\k(8i8eX27aqh!Wn!@>F-Z3%-Q$kg%hJk42j83X]r9cLl:Hf2fIQ#I!g=S
	>5\BHOX!@+Q#_866PmG$EhX!P%251E11Wn_[C*?QG3q=`,j;BTKFj54#U(=G&4/Vl+bY@c63u7FdI4
	6hf@Q(K_A=eho9bu8HDHc<@!Hp@F",$SO[uhg%LtC^g)l+d+#!`V9QfC8NVZQJU8;lOV5?=um]7jkf
	Z_R`4J95!j$T-Z>t##B+GULpb9=omFWlDLPoN*r!BGfE.U;uZ0E<1)aI([p!0dH`!!#SZ:.26O@"J
	ASCII85End
End

// PNG: width= 60, height= 15
Picture SpHrView_pict_DrawLayer
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#q!!!!`#R18/!6N@53WK-[$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!!,qg6pXdsg?dVL$,Sq=5gbOd,7Cgn$aZ]f"qd(^%
	*>[*GRjN1Nso8@09)2o!utMN*J=n.*N1=WL`!5,HS:>V0d!FIee"(MTQ'*f[p/9;R/?L!IJH,;:.>6
	/d>K%qn+G&`3BGI52^BBa\Bt>N2fX]G57JoEdk0=ic&s/f0b/jH#TXI;Yd;[upPM*;2U6]Sd/fpC63
	i/R#T]<*qGE.mPY<kTCihr+JC"=j")IQGj$"RPFQ)K?d%ddVKT-HI*=t7SF?o&?cp?m2+nbTWh%QAl
	51o(LciKgB63fCXh>`d:1EUFT5j3aF!MKT^#A3d=pFhpF2EDt&5j3aFJ[H[J:NaP56gn?6V7XnJ*72
	dumH-c1g)?[u(DI@-l%[Bp&bl!E@#nQO"6\kXh&QOS*.;>VJP6Hpm55G9+ETm4JC&j&mK*ds7ffYj:
	(sWTz8OZBBY!QNJ
	ASCII85End
End

Picture SpHrView_pict_PopUpList
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!/]!!!%n#R18/!6fI&,6.]D$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!#jZb6pXdsg?dY3XrCMI^iA8$gG<YKYWtMW@L!hT&
	g5iB)Wru(`Ug=V@hA5=+9m-W@b;gn"LK#Sh45BE';m@GEk0>qqWaU>r-jfODgK!AEi[u<r;C^'H9*+
	.3rf6\!+9*Y%KQP0!!')rL][VW!.Y&9)uos=!!Ekmz+Fk".z!JEE4!!!!)+N+9\!!#7a+V"JP!!!'+
	E<#t=!"_DO!rr<$J,hnUz";'Pm!!!!A^l(RaNi*O\!U=hjE>5W(!S/Lr@'S>GeoV!&?r4_/"B^ifJ:
	Q8Cen7>[?i\'4"B^ifJ:Q8Gf&TDTNZe[q4R3i7YU0D%,QM6A`I.";nAqj*1h#T&%"[ALm&qE=5?O*L
	CD$X:ks]sBlgMX)o/-&cK+AfBqL<AGHc$BF,f#iO!43X'-4W,nY>5gd*s3&kFJaLEpg?etX@WE=ip6
	Wqf%cC=Va'7$rHcTd"QUYR]#.5AqeQ5$i<O-2?qg]Q<bHrKf%Sur1H+UTHCXr+V[kWE2m3WNf7ogi,
	K#SYLW.l@rI/ZZJjp^i#AXqPe3!uCZHeI=4P(Y#^$gI4klQ8)>J22Ws*0%TH\2ieX@`<fhO4RpE&\i
	=8H;T9/C;6P4)s2(*r(&1r3*")E5cbi#+P&4U$oUCc`T2,)1#IhGTU%OpA;N\#oIbD*3;^3:r>DWI>
	sdOnFXVf!!ga@4.Y?mf&PJE"bTSYMsd-OOoP_\FZ:AB8H;TX9[HUl]/K0N+0SG#D`ot.48eI*@.UPm
	p"bSca5E/0dCj$ag@+de,[cqUiUD2bou<F'!%#kmRFg#@SHiX,cT/eoE<&,dpH-:<q=2AD!&X>V+G5
	_4kn;pU8\d\'LP%Gj!"/=6i;!rB6o\R's*]4kjRi5Io6BM)CV&,irnWs6*'S/d:4r]bEk7Q=P5q70c
	OKf2f7kB>#7%?l^NtOY9BYj1b0]b5^s`ZM1peTdE57W3f+`!5!0f;OADCl_ZL_K"ST''kV`ppM[h$_
	)IrK^]^\d\.JM(<UO7(bUqJQbq?+tV8:j[!+UClq`2fB^U88A/8Y93W(%u5cp#lmSb&!D^Xman-<GQ
	*fAs-`h8+ML-gmLO>fpi56V52PseT:le=hAL^p>*J[WrNGW-Y/N>5cFm*CE&^s`!+9:i/^-Jli>i3e
	_.R)-'B#NhQiI'J#J$I3k*Orb\bGFh^Xr^f]esN\5V9c;ZS):Vh\"qN8H?!f5`k@D[][7HW5%r^*$D
	/orC;A?qW7sl5<:sAlTDas$0pB`mY`(kFs")r-I$g/WW3:rbmAEG_nhcE!#s(.Z\GC6pmpZH&`J8\Y
	7:NGqtpQ%Dr%D\`s\:GaA0JsM/^8*k$/j#f,SQ=!,Ul0TqA_SpjB]jTFJ8?hDM#hn@[J7hl1L4!$^g
	![.p:'ZtJMY_O:T-)egJ6%IHD_HGB_g2>p_Kc#.e#_,G:ATqO>Xl>AQn%Tr:*EOtjGP5nu8WBskYc?
	.[p#5#%H%P2@@N-hrOhgM``Gq2^,CXQ[E28IFOL,B8;2UKTN;s/"u[i5fZa#=)&ce.!&$p3#cJ@3$l
	P5l_3.rS.qI(fIo+;#*na1P\t@fm7$X@Z@IQ>/>uE/uI"NmflLT<1Zb&2/Bdlb>"WTS2Y!fYHNnAPE
	61-36Bk`>=%N9\U<V4RfTPJ1oOa#6\:!X@WC'XL5[;/ho`r4Q>P9nXTneP5le>!G`;J">@KWLE1nCY
	><d,qW$;[^m_XfmsjG\!/7OFeg(5$d)p">Gp*4,R's>Eb(jE;B'!?_!&h=Dn,LLu"9>]1q2@9d*e;\
	A8c>1brnsHOo0:C\_Z<DtGV-0I!C.bdCB3#4du@]:J2IW5&o$H>%0351QX$!g\cC;,!,.<3E<+`(5\
	%P5!,E2I,U(),Cd7D+F9&Nl!C-b8KXdonV\?"FfHKPe[O)%+,QM5&*H;&o$pMN<!7iCqW1OV+z!Eoh
	Mro=Oh!!%PofF\?b#(K_TJ2IW5OqZgT!!!!AJ.Mqc!!!!"KK&r3!!!9I_uTi<!'gN-%KHJ/!!3FGz&
	3u$R!!!"L!(%Y*!!!!%&E3Zi!!"-l&-i18!!!%Q2uipY!!j^c!<<*"5QDr;z!XNcG!!!!15_oLB!!%
	NL66#t*!!!-5iW&rY!$He'#64`(!!.a4z#U%%d!!!!a!$MgP!!!!##]ThE!!!QqL][VW!.Y&9)uos=
	!!Ekmz+Fk".z!JEE4!!!!)+N+9\!!#7a+V"JP!!!'+E<#t=!"_DO!rr<$J,hnUz";'Pm!!!!AJ.Mqc
	!!!!"KK&r3!!!9I_uTi<!'gN-%KHJ/!!3FGz&3u$R!!!"L!(%Y*!!!!%&E3Zi!!"-l&-i18!!!%Q2u
	ipY!!j^c!<<*"5QDr;z!XNcG!!!!15_oLB!!%NL66#t*!!!-5iW&rY!$He'#64`(!!.a4z#U%%d!!!
	!a!$MgP!!!!##]ThE!!!QqL][VW!.Y&9)uos=!!Ekmz+Fk".z!JEE4!!!!)+N+9\!!#7a+V"JP!!!'
	+E<#t=!"_DO!rr<$J,hnUz";'Pm!!!!AJ.Mqc!!!!"KK&r3!!!9I_uTi<!'gN-%KHJ/!!3FGz&3u$R
	!!!"L!(%Y*!!!!%&E3Zi!!"-l&-i18!!!%Q2uipY!!j^c!<<*"5QDr;z!XNcG!!!!15_oLB!!%NL66
	#t*!!!-5iW&rY!$He'#64`(!!.a4z#U%%d!!!!a!$MgP!!!!##]ThE!!!QqL][VW!.Y&9)uos=!!Ek
	mz+Fk".z!JEE4!!!!)+N+9\!!#7a+V"JP!!!'+E<#t=!"_DO!rr<$J,hnUz";'Pm!!!!AJ.Mqc!!!!
	"KK&r3!!!9I_uTi<!'gN-%KHJ/!!3FGz&3u$R!!!"L!(%Y*!!!!%&E3Zi!!"-l&-i18!!!%Q2uipY!
	!j^c!<<*"5QDr;z!XNcG!!!!15_oLB!!%NL66#t*!!!-5iW&rY!$He'#64`(!!.a4z#U%%d!!!!a!$
	MgP!!!!##]ThE!!!QqL][VW!.Y&9)uos=!!Ekmz+Fk".z!JEE4!!!!)+N+;FiTLgQ?qg]Qgfb&)?;L
	V%!!'5@45p5?!!"-l&-oG!=hAd:-Oq!oeg(6o>FH,d"Ta^-i5(ib?qg]Q*:/I`fRETM"9Dn)!e:I\E
	<'PY5!&iX!7iCqaK:.-k61ubicWANacHm<9`fjbfF\@mmI2q*CB1Rc!C.j2KK(V1JC:Sla56P\f;rS
	>>l\XQ!&PmHM";el.!b"oHTR=)<phgO^mYB+%E][M]O1_S(j^?-hd]@^YU0B96ET%E_2$A=-$d6EhL
	XTGe]sTd?qg]Q#.[k=^c"PUpYcVk!3ddN5Qb3cN&WI;J.dX/CHE)4YO\kg!7iCqBM^3t9f;\G,n<)i
	"s<E>8H@<e!,,1ZDb]n6![u1&)hX1p.0#_=?qg]Q#,MZHa-4f1j2D*so$HuG@-H8Q?)p$<2Y*q@!7i
	CqBTTj'!c0[bF9eE3Y#ESaScNU>!&UdR];Bm/!L,G5%.-6_CC1&W&ca2C3Q00k!V))_r&;q-i$!#gp
	FpaR\b:;2T3o[P"Xr]*2!UEWrL])e?qg]QKO3)$3[[]N<sG<p?[_Utm[Lud`U6P$nW,ghJ6DmNB5_u
	I0/'6D:T"[cmri'P%>]2YUmp\tJ,IhMGMdiCSM^_X-36@U0+Ii<a#Sdufm.5cpYUJ8qm2ppWFlJjlF
	bm0eZ2cLa%no04d.M!mru80:e0q2Bmana4c_37:TR-<2$j9Ra<'?d0-$R\%R9+fGM]iKH2PjXs6K*.
	++F"SGk^X^VoS]/mlrq[6W3[Kl2T89I7U'DOm2H8jneKEr;?*T)]sj,=*'+fZ-VFo?;"NsnubPCf;%
	d*c'ec;P5l_\PTar&X[&(V3-7OCq8H&%L!4i33=e0;U$)>WjRT]?k]i'gH&&#0'^6)kQYXS%H80L)l
	+[DC7DtYel$6f6!j"rnO@q*B9qAp!iU]96r:Pu9o:U/R+`eNI\cY`PkJ=9Oa$9c=%lJD4TC_`L)b;S
	gB>+hrc@bga>*I7XUG$i&=HgOhA5*,Ug8,6u]c@BB5Ei#F-]6`hrXA0V(`IY3<e6dY:k/4*s*A\\HK
	WX2q-$5d2/+JW=d-XiA5*,UqPE$\GNE^b2[>uSmXr^GmrTY\FrhKJ+Q'sj2#)0r(RN1\Tr5Zci7P_d
	3<O-%8a!MEg5\..&m1@!8H;SjQ\Kt?RkGs<K?nhA\MipcI1D=/#LLK[1Ni2TGWcsm$]oppmWrD<!j"
	rnOA#E]GL$bPo8AoMK?hhLkmT`%f&/@XffTp$2t)+2O+F2@;qIhgF30jR=H`_-QR6moboM2WY<+?i@
	'0r$QN>.]J"W?`Vu,eRcfXI%7(4dA]rs9R]I(=@rVGZ&FBOO'8UrZ)!:jgcXf3CPgn_ca48bTjh7qh
	(EA1tF0aL(9"4#QV1Go#'rHAYZ4E_cbD*>0OJMkQ<"9<_[3!@h<T?G#jCY8/\"kjrRqLd+QTuXsdc0
	&7h^0-X"CD$X.+N.Q2!Aj@KT3Hr.f.G@hYg`VfGO.FGlnJ5qB2&H/8H=j-&9m0R9m6RU)Jm&u!<^]q
	!+b$HdHB3E/ZI;H^mYB+GmJCK?HeB,4[VRL@/)JG?qg^(8H?O;+`<tS#f(Q`CD(#L!W^4GE<@jpqL:
	3_7190#%F<TN!!()-Lq>AA%1PT';m.9N9`hCf!5O:"\'g0OK+]#/^mYC.P5p+c+jPul!3A`l!'l'IL
	B%;S!!(Rt_uTi<!'gN-%KHJ/!!3FGz&3u$R!!!"L!(%Y*!!!!%&E3Zi!!"-l&-i18!!!%Q2uipY!!j
	^c!<<*"5QDr;z!XNcG!!!!15_oLB!!%NL66#t*!!!-5iW&rY!$He'#64`(!!.a4z#U%%d!!!!a!$Mg
	P!!!!##]ThE!!!QqL][VW!.Y&9)uos=!!Ekmz+Fk".z!JEE4!!!!)+N+9\!!#7a+V"JP!!!'+E<#t=
	!"e)-+lH3n:JW(H*fier,K[K<5(OW#rUnd7^45b:N<A0f!WWqTN]?HMN],^4e,>k!cf`ra!D:q&2FG
	7!q#g[GJIok-Y!j9!-Js)l:4^i!M_sFQo*GCn)Bk-oDM8te`]43YE\#Fk2u1b.Es_os9Sd2EWBj"`E
	D?Pc2DrP]2pNiP5Uh@G-CYr`KH'(g?k%9JjZTe_rqufs,nT(Gl&j9u'5krVYKeRnRsX1RB]H`,!0fa
	)A!H3:]O(Ku4Z@n)O%'r5`d%^ffU<Al<D/#]_8^O!i)*.d(==JtH$+!*J/S]5$sG]r#WBs[^qH$D/:
	rlE$37kBKK-]M<`frMS6LM(f%f@4fT^B)SQc)e2*r:T7""Eh@'f7rF`)b67)TeU-L^bQD?58c$#t>(
	),UO+]][m@*.T>k.*f$Z&HIoX&;P5ar-D*=`JBO%@SQjnlPp!,,n].hKMVb9.>/V[)#te:&BAD$\@f
	lWGW\%^5ZL4H]\^;Hiue=%`=V%]1n'5?q2VQ_#<ZJ=BHkIX416=Q!)ORWNXs(<>T?\8An#!t+rgTuO
	+]JV&`[oanlXTT#:r;3a[+P^!;N&qa(L'i\qtAk-<nGKnYlHI.-(F0V%Ve__H&tAXuO`FJ3G]j:*LY
	X^b?pl6"4)IgmeCQ=^kWa6oY1>F@$td&``.:Ag/3+_9EqaioUXL!7qNs>ik)WF`=F4(5QsQ&E*Th`X
	-XD''%-sjpXLJ`0R\8Q2goo$$"`G\<KR<"'XdJ%:sR$"os0fKK(;uo*2XP/c].4EFfa@!!&l0ig<Du
	.u0TM!,P#].u+B8!7XPqUC(Jcp;]LQ"%qS8%:sL""os1!KK*R`o7h*B"opk#j0g8N!<C=k`%d_ufA=
	hk2]PXD1!\)T!)O=PNk:*emJP0P*)J[XUggkp!s$.8KK/BHcoB&GV/Rg4r'.@j#689W`:4C_;U,*FM
	JrREGamjl6oY/h$R(OF[1M@d3.U7@?<kdJ;1o#O1(iMjR*U]^ZN6:WN6Cn#M/WG[2F8CA&EE?Z9Qn;
	S[1hTu9E5'h'&m#P(g$a`,/O@`fL`IZ2eN*j=I'N=U4aNL%\OO(Ug!65+i46[=UB(!cCIr#q\_d]+=
	f1l3<K70M%I]W`41,Bmiiae3bj[pDU&4^YRVh:FD?bSmZ7d[*]J#9j7b+A!<=\_i`M202jF>h[2TTh
	Kc$0FpBgms,9`(IM/hYV*^r&]20N5aF)qNcJFs4gXA88):&moFjpgGNr^8^GTHuoE+l'`(SDTeA#II
	6bF"RFm[N`X%7Z?+$pE[TVCobJkkN30Q!-*ZORSZn@F,*_FmC@M[[_EUj%KL1-&4`\MDr0F7$ioC&4
	iVcJ/-%pt+c^ND`]+P?TKK?r3&bIM!8*#;a!P!I);!$9#,jh'@7Z%R!8*;Ca(AK^6uL9M:c<Mqc$PB
	M!(nHRZ)JN,Uaj4^0IAQp\N'`l^p\nURM?FUg`[E**n`b2:4r]LfU`Ypk-BGBS=;;7e0f$jF,*]pJ2
	.CM'Hp\>'B&rVPqC"d3<K7H3J9ALiD-Cq+%(QPZ32Eb%KN.Q66&G9&<bh=!?_Vk;Q]E?!<A?ZLncpe
	haph$"3@>$)!C#i!!)*mi[@]a1!Ue;!Edbp+\dUl!%=$B#<hea2)f.JaK)b=2n24)!'n)J3/U_Zbo9
	&1JGa/=EGj&%!)PQsNhX(JC73*HTLG]4-4p^fTR9'G`@Er.\N'aW>MMfXS7;U#:a7-D.[52L"q_?<9
	dRe=2eN*jW.PXO2+iaGmZ7dk(5@F1#R:?+%j;E0k_;#TB)nThRl@iE.KD_>KK-!;p!"FD2ukS>NcAs
	^!!#h;+cbip(o0PZJ24MLUdR0BJGTXm<2l]J-,K\IXed]t_>sX!7YmgC/HdO8%\O:`"=aun!8qi`**
	!ZP!+:4!*!!c0KK(BEa9YTYYmLJ]>D+u9QSDU!Dub%'(uk`g!.a/:EMS[F>`!D"FitK%6oY0sAI3@P
	OWe+hDu_n/#Zr8j!5S/Y3)Z_9A;P7*\^VV)6oY0sFU2u_,A7*SMAh,5#R:@6*[-Q.C5[)k+TMKBj<q
	m^`^De:ghPPOAiC\0^q,1Yd]+I0M/WH3W$-,:Du]n('B2QbjNo3i!OS]acm?O0!5oH.8?]ti-,K\iW
	Pm^@_>sYT&`RJUn-)'+8d\oL)ur5T+c^0!'&uAG!!!"4P7RtM6oY1l9%\u42]Mf_4<]`^C`1)X2upO
	u$t1>"!5LLG31@0B&/4"&(pZRA'B&ok[Ni`<UrY5:_>u>;15q'/!!(adib4WC#!aU]!gs<!!3eU0Ni
	&p!j2=+]!!(qDLkCtKDue+ek)M=2i;`l#+N2[?21h5%#J`LR#\G8#!'iPt37of%c9sVl*jK68'B&ok
	as+d?7nWtA&Is=;/.p5s2unL"#]\37U]F;L7m`XM#67RI+V&hdNH$-`!!!!Q7YlZtF/,00#J_8Gg"B
	sr)ur77KK+4<#RA/6k(GT(%KN/<66'5b-*W5F!(<1efL.\!JFX"d0OcT?M/WG_)@LC+)ur5EKK-J-E
	!)aI6buKM!s#9T&-im@A"5L`^fH+I!.]A%EA5cd^56e/PX:T:!3")!6NGSlDuc+fJ-,eMP9"WC!!!!
	15_oLB!!%NL66#t*!!!-5iW&rY!$He'#64`(!!.a4z#U%%d!!!!a!$MgP!!!!##]ThE!!!QqL][VW!
	.Y&9)uos=!!Ekmz+Fk".z!JEE4!!!!)+N+9\!!#7a+V"JP!!!'+E<#t=!"_DO!rr<$J,hnUz";'Pm!
	!!!AJ.Mqc!!!!"KK&r3!!!9I_uW,3?GCsV+98Pj!0E9R#I'l!V^D_#S)1l-l\P\Q('-[QKK+KIa5bR
	o!rr>J-m>b2:)3g0!!(Ro_uVDWb\'L2gBhg.!J#Qd$?9#$&n[40!!!",%<fYj/7;)gp\AcM:&0lm4'
	I/]9Y#>V;$-f76\r?qTE]A[TTk!G^bl2olHidlS%9i`?k'p*lpq.^HN9]jG/T@Oj+*u/'Af%t,OB@M
	>UodsLF,CDS:Oe$"'cpMe'sp;#&I\(CP:M_!3iRALLAi?I68Pk#iO/h@1Kp.=F/=X%FP^jnC_pb!2r
	8kDbZca(nV%O]3bOMN]#&:TjmKqm^X+c%0lk:b`o6uoK!H3G9$ZFjIj&!`EiM6&BE"gHM6?-G8rGbX
	dK(5Li<>jaD<:Rh&kZ<K=5D*jL[$,r9k3U\9#H7BMfB(+Q[b)TmpC%"+YFeN4=qdnfom7>?6<Kl\_H
	&N<;6B!3dM+ijoBT_4>QGB-J<gG][R<J=UWZ:`^d_ImCq3i(a=n9,$?9H\l\A:o7l\^`X)rG$pC9>W
	Vq9p"9p9G&Zf.")\\.LgkQJE/O]$!9FmRVApjI%N$>*Ym!E'kS)kSTH6GTS:`-/61`#R8g27V!ZV3i
	7u2Na!/biZQKUeT`&Yn3F9@%5KK&r3!!!;_Y,MC$!!!QqL][VW!.Y&9)uos=!!Ekmz+Fk".z!JEE4!
	!!!)+N+9\!!#7a+V"JP!!!'+E<#t=!"_DO!rr<$J,hnUz";'Pm!!!!AJ.Mqc!!!!"KK&r3!!!9I_uT
	i<!'gN-%KHJ/!!3FGz&3u$R!!!"L!(%Y*!!!!%&E3Zi!!".WrsLL\h7WR+-'.ASz8OZBBY!QNJ
	ASCII85End
End

// PNG: width= 45, height= 15
Picture SpHrView_pict_Help
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#3!!!!`#Qau+!7_T;M?!V[CM?aMFBD723b*)F/n&:/@V%0:D/]IT!!!
	!]5Bq-U@r#Xt+C\npBl6'Xn%6=Y+DG@L,$-Wr0MQLRAS>dKH?*RqH;7FLH>@"NA0O6%$9WPoG\qD:A
	TV?'G\q87F#nP/,%5##@V%T<F#n/LFC?gH+F#qlD/aT44X+!&:`r,DEb-@N/i>78,!H>V4E=tE3`8@
	8+F%a>DK@jZA7dtKBQS?83\N.1GBYZ`1G3TdB.ku"3B8`H1+tC</TPB6/TZ2TFCBDGDK@$H4piDT4E
	=tE3^dP#@rc:&FD5Z2+ED")3alK9F`]bj+poG8+<YoGCi4;TA7IbHBQS?83\N.*F`M:NDfTJDA7I9(
	Ch7HpDKKqQ0eP16+poG8+<YoGCi4;TAU%p$4X+Q]FDs8o05bh`@:X:cAM.J2D(fIFBkB"Q/heq&$6U
	H6+F%a>DK@j\BkCs?,%u(?E&oX*DK@F=A8bpg/n8g:06Co?AhPkk0J=UW+<VdLG\q87F#n8RDffo=B
	Q%iQ,%u(?E&oX*DK@F=A8bpg/n8g:05tTCFDl56Df@?a/heq&$6UH6+F%a>DK@j`D/_W^BQS?83\N.
	(F"Um3Ddm91@rH3;G[YPE0eP.5+poG8+<YoGCi4;TG\qCo9g'\6FEDI_0/%3a/n&:/@V%0%Df%.P@;
	mkS/heqqD(d(?+<VdLG\q87F#nAa7<Wc9,%u(?E&oX*DK@F=A8bpg/n8g:06goE0/5(506:*<E+L\#
	ATMd4EarZLG%GQ5,9dI;+<Y6?BkBD&Bm=3*=@?k?ASuR'Df.]^0fV-:$6UH6AU%p$3`'O8ASbL;Bl.
	F!F(oQ14X)d?+poG8+D#e3Ai`:tCi=>mE+*6f4X)U)$6UH6FD5?!3_=0sB4Y.YA9;KJ+u)&:+poG8+
	EV=0Ai`M#@:s.FASu.&BK[Ot1Em/K+<Yc?AnF)%ATMd+F`_>9DH1RgF?Lg'+poG8+EV=0Aia$lATMd
	+F`_>9DET1"0JG+5+poG8+EV=0Aia'mATMd+F`_>9DET1"0JG+5+poG8+E2"4FDl56Df@a)Des?49l
	FQR4X)[+$6UH6E+j03Df]T1E&pQX6VqfAAnc'm4X+rF7m\=i77/sf3ArcI1+k:*$6UH6G\qC\9lFQV
	ApHdQFCdTZ1,(I;/MK"50f)c`0KLpI3\`<?0JkgE0HpiH+<YoGE&p^)FCA]gFC@RGFCdTZ1,(I;/MK
	"50f)c`0KLpI3\`<?0JkgE0Hr[f+<VdhA7IZBBln0&4piDT+<WdeA7dkgCihiJ+<VdL+?X[TAiaaB+
	F%a>3c&5BB0@G`/S&F)@<HC.,!KIYFEDG=+EV13E,8s)AKW]bGVM(#1H[=2A7dl=Bg)kH+<Vdh061T
	83^IS(4piDT+?V;f@le^GFDPMP$6UH64Et^^9hdZ>BleB7Ed9hU+<VdL4E=tE3`BU04piDT+<VdhEb
	&cCChX5!+<VdL+EMWnG'#cR@ruF'DET1_Ec5K2@qB0p$6UH6+<VeJF@C@p3cf4OFE_/6AO0s?DKJ#c
	6"X_CDJ=3<+@Kd^BkM<pEZcc)@<*JH0d&5*1,(.$$6UH6+<VeJF@C@p3d57KDET1!0JY=60Jk@91K6
	(_3\`QS0eb151cd<E+tl+e+<VdL4>1_cAi`k%EBX^P+<Wd"G\qCo9fbgZF*)>@Gs2QX+?V;tA7dkjA
	TM@%BlJ0.Df.`G+?V;tA7dl#6q0?I4>1q?G\qD:ATV?E$9UOtE+*6lATT&+DIbmSE[!_3jE?r>!!!%
	NBgY6aF&GLp+A#!h2DI3M2D$[:0`V2%O`cfD.t^\9a$4/[X=9;?(JEe'dnkrAAP8eaek?s![b#u,rH
	N\H/M4HSXeNan?PiVDga^oJ&Lk82iL78b]q0FHPO+gbGs:D<T',>ucN-ZaUL\MJ#I:ml((U*Zo?d_3
	o.sE`17`O-'O@78Kn@ep+*TQ/p;Ut3Wpf]NBNn6q$8f*qO"752&=sLEXqk!#[tVGYFDP+T<7n$=J\n
	mn*j>-maf-b>RB/ioI^!o"4&WgQrem&KQ5,^Pj32+mb"k1_9A1&i\QQ1cZ?jn)'>TBUBdWB1AV`(o]
	snql:eoMspJg-ba@iTY]1NBh'iLhVi"EFGfb7hYU*jJ`.%1&6gK-Nu10i)\mXqAa_g(L&jt/!")TLs
	6@?chGnI[Bo[J9Y#N4t4a=jro<?se\qbF04(Jb'?7Gu/!SX<KOEZ6YY'H31N'FC%?#@^Ti5pDi'XA5
	0f'$?l/O5lC"Bh_an:cN\g*n,NFg$#iF<ErZ1m,6.]r,6<%c5B[#,%eN=\5u`*!m@AMr<7_r,he]S[
	m#/Y5<Bq6<G-'[OZI=D)XgHTh'\&4r8i2p&d"W+:To5k7[*VXL7>4h_g"K92Fpb$!XY#-@as)d@E'T
	06)4:0qK9T2`qsJgd2b\_[o*9d7c9(d]c$/#McFJ-930%LLX?m+<Xkm]&An^j5+#o;\GCue42C_3hV
	6ZhiU9:Z%Vl*&U%oTBkbN2q*K;GF+mbPMRr;>-I=bE_[,`I_B'bh@h['\$fIXHOu`5H1Enoa9LnbN*
	eYV9^&B*O!nddd<pZ*CS%md@?0(`shA)]N9BRG-BK3]]=.IuTDGpj>9*a3Ie9pXIU<n]a#M>eba$]=
	T$%hS?p7hnFLN&J;ak^2,\_ePZCfn`oE)m-O)O]Y*e)EL9tsm-Go?,2RY3/LMpDPuZ/SbY2Y^B6/.#
	6m@2eB<bQ$*cJrN%=C@3_r3/El",^K]"0pW1%FG!J#7D$NK$Y%7Rg3N&ENm?KB)`.?[_V3Vk6/!oSA
	q@5<loAN/S2b2P:KfL)2LLLrJA^[^Hu*?;S/&c";<KH3scV-BmHIKVE,o$:SuUo#6M'PtE@1L?CO<k
	YPGdRZW`[1B##;hqoLj(%oHT5:k9EQBkQlEcQPT]IMg*]tM,@jlKu^A'=LS@Ue?Wj'unA9]9;-U3ZC
	pdD:G?`F:QUp(kWc`8oLO>Q4X$W#Pje/1ca?Ndg]g)(LC_*Zk.Qq<QV66fi89PY,Lm?/;-&re\c@K>
	I'LoQ5;$-[d4Bd>12bOH>Q\M$^I>J^Mlm&W&nnDJj.D^-kRc8n)6REjs?>c6ABamFnth4jTfu-jg=i
	qtp8QHeuJb]<eh"WE[H#VS0\!bKF[7BS^2,:mdZ=rV"uaHp4/4k88s8k2o-&7ag?[+<[>e\9ODDbb`
	*p]VM@!Za5i*k88pW"9dB2g;;U.(^eAqZI$rpRM:B_=FYgME,`$T-DSJ(D/J[fUIXS<4A0C&BhV2/:
	!RNiC["b+L103*jaDQ*nFj)))B?db6:+::8dI"#/1g_trU&t<m-O)?Xf]4I]:5:Z+$P'S=BMLu-jXb
	+Gk'ehoB0OKJu?)efA59]n%\lLZug;-ft)=,]tM+MSK4'?D/3p&C23VDpYUK!oiIN8mbbeJlKYtIGl
	L3:>?cAu3M@=C_6C;R1b^WAEk,F-=.rG!%hCmSr`gY09MS=qhJR(C\psa/aH0du;d<QI)B)6+Mnt4/
	ND2u9=gWBkD/G8jM^p;."$84p2=9\odaHQqM$fE0-Vn'\Fq&8_YHRur`udfA7kZn8/Li52H1GKI<u9
	L\6psFL?682f(G\-:SWf7)po,2O)uKF*R$a6l]Hiie`F<gS6Nc9P]k>Y491qpFW*i%='FLn@WW35C%
	]?D0D!0*7!#lou042G@k0+]^>NtD:n%\ld9N3\S$k1e@fM#"6!Gi_q*%EHhc-=K62Ns>gDnl6o5sYO
	eL<3u7['\K=fhm]N!=p89'T#4c)^d!\Df9R`U<BlUcgp]!_/FWM/$#nY0USf3`\^[q<FEdR(j=lT"g
	DGi$tCt.naZ-f^A0,.m.8OfLPZA$X&lKV'uOXiF)l<:MSUu?91pcs457p#$=KW.._kc<$S#"+iU*h`
	rW:iAR$\F7B%Dq1dud0Hc_!FW#n0E_QS2\M)[M/7ZY.V$\7.-/R@0HY()?lhb%mOU;(*.Cl02TFUnf
	9W[0f8g_f*<g(2WU#_#r?JM!4-h)AsCn(pI*n5<hBRKjB%PHbq5[i8>`:/#`/VCMU`jQS)QQYui]_0
	FT`k0F.pi!#R,Lb)VJl88hnig=h::LB.Sp()@2_$1JFKmFnu3cs[F,H&74dNng1qS.#f7i#NI(RugQ
	M@aPSp%tdei?=$qZ*Zd"pbs?$(lHqOI1k6$1S2`eapWHqu^qod;oRM9UGR.A0*$$%k3<[MG__CQrC^
	sq%1c@7D%H?C,?s9#Fk09C3#*9.46GT=(Ze6^$S07,2*'B8AOD/>o^nTfXG[)kfPX?H3*=WGeJALI5
	.#]6Wd=<5T)]KkYDucc3q%17?:us@H_=--fO$_-J0R.>8^kI:DG"te-A0$\kO+VjqC21W\+Vp'(QPZ
	<6jS89*O=e_TU7q^*!4cK&aL5s?!*<Z?A'$crbN7Z=Upd]ifb_G%R$j?oF_U6s[6,LkXM`oO?Ef!@N
	Xh!i@aQ#1)(kN#2)R6?XI[OFZ<r(V"`/f@"1<@AhPGN32`E[h:e9/G77@A>=j-"";&*^E(72LHfg;B
	t-Vp=a%J9;b@/N:7Sl/Kc?[_UtL&]KCbTs65*64hLUVS`X%hB0X?_^';eC;-RldirPK$39<k2qhTmX
	p.EKOZ/"`HX_NXfWC6qt#'F/R,ZFp"%O<5rcQ&jcrnT>X=39>?b;W&g\hMbrTDi(]o8]\@D=\TgLNC
	cR1k=rU/LR@%/B2Nf\Ak^tIDOZ*?#=13.Q.UoC=2()BqP=%@Gi<iK%>3HCpnV[%$]q!__WYqBGMeuc
	d)_/o3_%Ys*,jq\dN865.',9rJ=Q!bPPF*#E_Q#o16nF3Z:L'G'@d*8%=NugGF4Zs7Y0fM%0C2+s#q
	u@%`j&2l+d*N>Q8SO4Ll04S&R&mA?Yh@mHHb]U[H`@*u:ad#u\K#Y6nIhUlUI@U?K[]1YTV1uDf>BR
	9:fn4LPq,qqj5UH?@5,jHb'\Vc<>,uj;c?VWOcfD+h#j-L>$BM/f;l2p'o&EUTq\8*_Qru3Rl:*;pd
	"[T7<'i<iu`[<5tX`VTk.\BU:MdcA7QrJ4IiX#T#kO"^3s5A'[tss?X-i0R$06Y0\EX?#J=9p7AEg3
	@q0#`r:3KI9;,2WiJ/N'*=jTp;JAs(d/Ek9#q_@(OsnPnBLO,-r:o<5i5#DJ@4W0E0JJSfD*5Mfe+E
	9SFEI/7)`J$Sc-=MP@PKitXCD&&qY^<jS!ocO/VAY,MMmBA%$g%OX/i:sU]b;5aR7lr*?6%i3N$k$Y
	$<T/kFHg)'XN8eqWd?\[8K\BGPBCR4(/ip`Z&jO]6@&Q'[tpq%\jjFK0_dX(k(5heC<!!!sa^@M?kd
	;!^TCL$"5iP.VWg>p\"35aaZ`J7:g"?XR&-uQY:5QXfSTLlp+Z(<^8nMjMK,6d*b'+'T1LLJ%`4rPg
	Ik5gUjD_87,^GGmor%kig[(@g>]$F[AD]C20bPD=)4YDMHj6b2I;+6:**`C2!F#Unsp(.-@qh?Ft:A
	NjcAS*AL@ma73Z[Nkm+j2)Y'^\T95+#p,L.##bN*WMpaR)G4*Y!<E(fVS0\!aiTCDVW?[OKA"r21i3
	r$9QkT&<)aRaL<3t<A7QZAG"ab:_,F0,nFYsDY`5fgDHL#n110F)h_[<Qa2Gn-.OHc#m`LeFPW,8a7
	"9\N@F6Q[_mt3m2SoS1.U$"Z#Sj,<Y9+MqVQ6c0bdJ(@-7:0rE0JH/e#1[CdaW"[0[#k?B>Be_*"M0
	"h7Japql41IEjDR>#&L'<Y9+f,Nashopq':'Mgt$A.J6Zad*PU,DD&_Tc";J"h03V4(_";'5CO#Ip-
	kns%'=1I#7&'kkdu$*\od2")Cbs/Yb-C@$,!L)S=I6)cQ8@j6ON>J!1V4C#mimWiBSh1O[oV"mVCCT
	\T-Ya=tP?N#@'sdmd@5qF/:m9*!_O:]Y(o(/o%$n%64"YqtHj8XeGo:n>B$P;Hp\-9hbZ6PiQdCKOt
	'l5q*`A8O7)Nj-U?"GUAUl*ZhK,?q:=Tl"t9SNI5"6V.dY5S>;>:WX#bH_8X@s<NS5$aQ7HX<B?0(6
	7Efl)k3ZVD;;W5:g:+5!.lTR"hh3lf3Q0RTW*K?1k<;=4_LedfsGJB;O7[8=BSdKF`j&9J95pV;/O1
	=%nb7Eq>#;^H7o+[MoC#:h00Gh6jKVcT3enoREt&sq3qT+0GbOl7K7:EDbilE-q2$_Z*L^B?Ref_0G
	pF0nd4!E!*'Q%cd0hZHtb<&:JWVs02LC:A.*C"F^6s_2)@$5]>ks*oEi]tM2@1MN'I=:@rQD6f%0Q.
	pYM\BbYEh5G'7J/AMl;$Yh<h_8&r-L3)`c)Pn".ea,_>:o^p481*%AU91qn_#7$^'H`Gh[YfctTlBQ
	!gjZgS]BN?8cqT?M)PENmoEGK>?O2l<aFU\il=1@)P=c0JhZ2FOtqYL'&^:qj]2g]muX8g[`NffHdV
	b`pWV,A(eARo>ij71J;Ou`hWd6)^FjS89*0@b@U"G5skY:hab<$*S>.\`]5?TRp=1,AFS[;1MOL+<r
	P'XL:jMT+TN$"FDR%A?USn>O_p:_&.i17:BM=>WaUVl//!fVm(%bF4k<Thr74Q@A`)m37Z`D%i3PL/
	h5R/ToR+YHRE7N>p8'LN6(0p?^J/$9L^I8(FiBG)hE9M8P)Z03-V$;c@c1l;\RS"HPqp!?BBb<E04u
	qe,C@'@$1AoZ`ja!=#BsV#pf%W1$(H9he?/HTT"c-RVDq_2tLiV$1g</sd4mZq_BIU1nI]-*/JtYHL
	IH**NWoG]%>6K_]&3'nPlD?`@"^[RF`$JbcE(jR5Z>&-f=7.+D9Si6hlJ"0s$_NME6n@IZ")6bcTQ5
	Tg%Gf\5&ZeY^(V&C0X(++Sr:1qL90i2i`.Ka"lT_^`I>_ZROngY:IeA\jI^YYYXO,nnGS;eBVgi6d&
	Qp"/K;4)PNLO*BR"Kr(r@R[U+NVQ)Bm;Rj6H!)B#i\$nkM^?ush5=J@Dc`F4LKu]'cBP;eefk8.;_$
	Jd^:-q:aU;u"l=]sRkF%;dBh/"u20JIInr<?;'<cf=hKKdT8f@TcE7it`4=1>WH>$G3LgU?MIq=A^0
	9@?+CD/EH[RTM*^(`746F:8E\3-Xd1jQX6+bKS2SK*Mpu]"4\8.?hk12)TMqE1lI#YEhYZ65h^%A?;
	NNr"#OGb/2^>NMQUT!!#SZ:.26O@"J
	ASCII85End
End

	

// PNG: width= 75, height= 15
Picture SpHrView_pict_WaveView
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!$[!!!!`#Qau+!#2O`%fcS4^1gPKFBD723b*)F/n&:/@V%0:D/]IT!!!
	!]5Bq-U@r#Xt+C\npBl6'Xn%6=Y+DG@L,$-Wr0MQLRAS>dKH?*RqH;7FLH>@"NA0O6%$9WPoG\qD:A
	TV?'G\q87F#nP/,%5##@V%T<F#n/LFC?gH+F#qlD/aT44X+!&:`r,DEb-@N/i>78,!H>V4E=tE3`8@
	8+F%a>DK@jZA7dtKBQS?83\N.1GBYZ`1G3TdB.ku"3B8`H1+tC</TPB6/TZ2TFCBDGDK@$H4piDT4E
	=tE3^dP#@rc:&FD5Z2+ED")3alK9F`]bj+poG8+<YoGCi4;TAU%p$4X+Q]FDs8o05bh`@:X:cAM.J2
	D(fIFBkB"Q/heq&$6UH6+F%a>DK@j\BkCs?,%u(?E&oX*DK@F=A8bpg/n8g:06Co?AhPkk0J=UW+<V
	dLG\q87F#n8RDffo=BQ%iQ,%u(?E&oX*DK@F=A8bpg/n8g:05tTCFDl56Df@?a/heq&$6UH6+F%a>D
	K@j`D/_W^BQS?83\N.(F"Um3Ddm91@rH3;G[YPE0eP.5+poG8+<YoGCi4;TG\qCo9g'\6FEDI_0/%3
	a/n&:/@V%0%Df%.P@;mkS/heqqD(d(?+<VdLG\q87F#nAa7<Wc9,%u(?E&oX*DK@F=A8bpg/n8g:06
	goE0/5(506:*<E+L\#ATMd4EarZLG%GQ5,9dI;+<Y6?BkBD&Bm=3*=@?k?ASuR'Df.]^1GL[0$6UH6
	AU%p$3`'O8ASbL;Bl.F!F(oQ14X)d?+poG8+D#e3Ai`:tCi=>mE+*6f4X)U)$6UH6FD5?!3_=0sB4Y
	.YA9;KJ+u:r0+poG8+EV=0Ai`M#@:s.FASu.&BK[Ot1Em/K+<Yc?AnF)%ATMd+F`_>9DH1RgF?Lg'+
	poG8+EV=0Aia$lATMd+F`_>9DET1"0JG+5+poG8+EV=0Aia'mATMd+F`_>9DET1"0JG+5+poG8+E2"
	4FDl56Df@a)Des?49lFQR4X)[+$6UH6E+j03Df]T1E&pQX6VqfAAnc'm4X+rF7m\=i77/sf3ArcI1+
	k:*$6UH6G\qC\9lFQVApHdQFCdTZ1,(FC/MSq20K2rc0fh!H3\`<?0JkgE0HpiH+<YoGE&p^)FCA]g
	FC@RGFCdTZ1,(FC/MSq20K2rc0fh!H3\`<?0JkgE0Hr[f+<VdhG\qCo9fbgZF*)>@Gs2QX+<VdhEb&
	cC;e9r4$6UH6+<WdeA7dl=BaPt*+<VdLF*'ioF?3i?FD5Z24X+icDe+!#ARltl+<VdL+<Y`I7<Wc6F
	)PZ4G@>N'6"abHF?Lg6AnGapBlnVC6tLFRB5V:$+=L0"E$-nk+>PW*3@Q-Z+<VdL+<Y`I7<Wc6GA(Q
	*4X)X80f^g?0J+t;<&$*g0K(sH0J+t83\W6300T^c+<Vdh061T83`BU04piDT+?V<%D/`3D3_3t+FD
	l2F4piDT4>1_cAi`=kF(96)E-,f4DE\CM4>1_cAi`gX7Qq/A06fO&D/a?'FC@??4?n(0@:O(aF<G.4
	A3D,W,!S9dQ3TeX!!.Xb6UO[k;Fa%r8OZ!)0f_-M/M\q5!!"F0Fcjj?69u<@^U,\r0W7Eu#RP:3@0T
	W!+<rG!&2WOXdH`=PEOj\WBH+;f$DKGl(A/Kn2J=*671hqW2Q_nkPhaPa6&@_ARIe[AjrA!YJ0g@UZ
	?i8NZFPQ&%euUd$]@d7.s>_gYg=%`FEK\[$<#Ki*b[7fS_L5Gs4kB?&l7]u1c,T$.B6D[Se;O9'i^B
	@KfbQA1bO/.4J9e"L$q`!<ou?);e>q_=c7q\^sXk%9\"*`L6]-3'pb3Hb%=\>b'hL#ZmKg7ZP%W(]?
	pou^lO[n-CMZi&>M#XE8tM+<n,7cPk&J.9<KJFauUFkUe"El5h&f5Srf<R4Wu&f]%R+uIFXP!hK&9i
	5;^%j`^_-'G+c?^(uNAg;*4mF1L0%Aq=u6Pq%S7hWjrVUYmD5u<@tWaQWqi0Udhb*8[=4XP"Bm_K=1
	sI\-;b-\S%1U:*1Xb+^]?oqZkJ$k8ha\rSV=s`5>ao!!!!*E(F,Q!!"Vr!!"Vr!I1G*FoVLb!(fRE<
	-Xr#I<U'<a0krXFc3V18ZN=67<B7i9G%?RC^3$3i4)UR(h<FHE(q;l42il!^R3rtcbp:IZ(9sdi8fH
	G.E4)#+M)6\#Q[sn"]0-LP1\SpkP[-I\#ab\.mQ_tqs='P&'WhB4$.e5^2qSi3Tgg%&:uUQ#if85j?
	L6Y@>#4^8-j8oN%k5o5lVK^>F#9i8H!XMmFnu>kKgWE[+Obh]A;mq\Y>gmU8+KYUSH;dd`.4`6drV1
	,6SGb-mF8!NkV4431Y$^XIPG?$l_YI`!nR]plHc#U8"BbY0Lpo3uj9`XtHKsBXM$=USAJ8&i)7!,9n
	F#Bh+KJg,6*:DR;W&9jkHJfWjpapE]d(d6Fd+'("L/n(]@9G=+XsZ"O_lI10h-7[l?[?[jZN1<c:^/
	m1Lf,]H1@7;'o9WMlbW0o>kFA27_/B2=pN,s+1-J2hL5rnH>kb";d:q>%tWV,V&=^IO(&"onJ7dakO
	@:CdH@P>>*NgbTf47Rm]"r>\F%+?^b5r]U`0f'a@c6KcXZDVeX1lRPVT].G?6L6&hY(KUj7ZU#-*0k
	SXsgt^]_r-X^MjL6SM*O^D,gU9Cl2iJ[\pYQLfAckKp]1Z^HnB&^ioI'])7L$O!3mdn#EU`uCLsU!3
	QS5*M?BD<r<UXR'Gj6aCY.ONhZM3^J?[5'Q7HK+8'm4'.O=tmn47l!s%j14[h,@90b3/V2O&>Z/hiI
	bU*;oTP@RZ"IV:*eJ%mM&dnV%Njqueoq"@5j;hKBj>GnpgC:`HC6WnOt&a6.rul=`uB?B:#Jc(24oV
	YT(eM2[BuY[G;K=gNKtPHlt=q=Dt`V7V$$,ZsqJ0H*=G%fd=3Klu]H44CpT*[XVH7gibtJP`oBG/GP
	c#Ye$B?<Y+8cHQeuRaou`Pq,:WW!5Up@cC`3VH%t=6psFhHFhXANJrfQ(^s\U?!U`AmHs:`F`fkl\#
	1Uc[<9#2S!>KL#obeLj2Qf,TD.3+2JpmMR@d9V>s#8qh>0.Eop269ht[c[r"m3,mW,j%rqPO5io2bl
	(Gn"0`>9,:a^eiuL8*qsH1lr9QTH/s]A6dn<Rl^:"u"uriu`^AdId'QDenj`_pI3*kC`39(S5k,e1]
	A#!foGP:8`Dl,iOA?_2&\s[Lbpc#aMIZ]H_:Hia4Q;8qK;-R0fNjl$s=-rNph.K'*dl,)R$H4sii3E
	Cl>=cd/4?.46]!%3ZugOsEW)/5'<Z>cu5lapT"<eC;trD=W(%;q'cYOJlD%2N2n\I@=SrpSX_e4oVs
	iV>oo8jHEB&2VS3aZW^O?`C&R,qgWbEbE/(Bf@sM6>MjbmXiAB#QKIg<`SUn#PK=LRAMKSt.*^3?s1
	QA,Ef1>+Z7OVXeWlA`'GV?3F!OK/bEjlS5(,Hl]_`1dp6ks,+p`Gl%EQ[!rhU0X(OkICI5.Ij@+?a:
	o=Mir)b,XV?@;:^TJtENTN>H^'N3C:QqXOZf$a_a"Ht2<f0"9GQFCc)0I%q,X8)Ha"Q??"a%m^5C.3
	;h:3q4#k,0VbOsC?cT_p<3P1=:f4aOKiS_2WK+\>E/TI=Ghm"rs1oX_V9gm#\W2B*6cBO/AZ\oPbg-
	4!_cQ?r$db%r,$HcR_PL[&W^8s5j</PA]DiM(j<]M"SD^&%#p*??0+9V+@l5=n>`a2c6=`gY+fEjFK
	N9csH6GOY_hJ%c:68XAJ:W2S+=7o,^=g$NrA2DtVe/B_\OS<7C9&ik-k3^W)HaH80+5*V>3H>HS-/6
	Ebe99kg[T0?)PR@3R4+(<$>IQWoMJg."eh,ds_\gL4iqsW"g[km'bX(-J-/m4?h\+@j'SlRD.,I+L\
	GT/K"+2<-?LrBc9H0LJ7MP;MVJfk>N^jCX9i0R\$GB^O@At'kSOb$_u,Zf?i]-BeP-m,g/:I+61K'm
	M)/ll-G^]S+rrd^K&g=kK3+gj@\54$r1cTIu7iFnARpU7N=DV22Pg9k\rd*\H)Z9sdRP)VpEP0kM8o
	Xs^krS`NYOGr[9;^5.[('tnH9W!LO(TplF68:-Aq;mr]^Km`8AcMARY]:J[=J:1W<>B23R"3gjW0XP
	dG\Jasc%5<nj15eLF<UjC%]MgFd3<W+E.T@oVa"1`KFS6W3>4fImcoD1B+Y+\$ruifs,/PMa2^F@kY
	EBI7nSpebG*h<[C5WgcZ:(]8!k]^4O:XqDN<cHop-B0mH.rI2k.iV7S^rB/2!8>-K\TsoW4?2>X6QY
	62GZpW2qP>Bc7a?m_@/Us#>%$/K9\RqCqu#mk'([j43JmpA'!/g[0IemC2W.P9`_hmI'3%(ke!0jIr
	Q#_5qOGkNC[TSW^k58B-,a%oRd.$lqD$O[gItfke1OD-Pbbc>0q;`uQ4*oTH$^)lr4!+nQ4si3;?jh
	0h4aXW]lh0`$@Y>cjqi>/S6p[s?hnSkb4aWJ]9D_B>T>a:VR@JL%EF-biFg&cb<h7]oq&C=FZ$_'$k
	*M!8)E-htb_Ks;W<E?&%ecNsY+MLlt\US?bSZa4ZK%S(S\nj_a$p?`7)eUn+W%SrN]h"[aU-ocHJ/a
	)5@`\]us*I"I6Nuc\PBJ;>^gfRojO3@.Xhg+>+E9*:X;lB>F,1\sQBmDkaoqOsWe(L4mHt6dKil-g#
	+sNOPCKFS(@m_JIa3L::q(.DT3HN]2hR=$l[V9!V!:B/g[:!$bSk'\A,</c468^J0`f*dj3HqW05($
	J`Cr?#Kk!jIbf+d<Y%a11lO,Af-"<fd.5Y.+b?C$;V3fd+VbZ$!J`u2qf%1R2HiWR(LML8A;DmS-Fd
	QF!77cRXHYmQiln*b&[E&ut=8Tb"K'F`!9Wne.2-#.SRCpdM%9D&T'"=Q=*HXfcEc10i7EI$!.8H`Q
	am!rUlbH`\B=kO;1Mk"%-lpZ+/.QL.@ra7Ds(+`S7gb]K@,Y@97n$#o%i^EGQkK[#Lp%;-ag"5sZGX
	*hk*?$j6%MXMR;2!lc6/@pZHD8WSX1l9F#E"EAdc9%=3->SUF@UG[8&LfTDdHUN)IA=K-Amu),:#mE
	d+usS[IMBh$hG,iQKS(P2U:Q/o=S9A"]=?UU'.RL779Q%An,>[@S_N3$<1@S:`LH>/h=@bP(X]/#(r
	sHMc?/T_nad$l:]OP\p[,O)N57(qqqF6b;Pi,g9bS7NuKblOc0nd3LfE)6Gf+WCI<idA?R%Z<+PW0W
	?PVX@q1W`"#k#)GNRskdi,S?9Rqd![CY7f>41dL:0ErI47'"):^AgGCMT'`q8#T#UneQ*1#%K;H0gn
	LM@^fA$Z!LhF2X"T1DJs%@M*:B(I^WQ"AolulI6uR#[!&*7SXLS8J4FRVeL^Z78Xa26GpHB5:0<ga`
	PART7D!<Fon*lnOEHj6\KVV:RU3XI@3/u/Y>-3h-S-`ZEge,noA!,4F+6ec(@+4;M8\+G,Gii-Vj#-
	?9N_ZjLiKJM5lf0RO#L.RA;r0W$poo8.`B3'G^mPK?MH375a@a(#T*\&?]a_i7M/qmD1_oNnV@4,;S
	`.>6_m<!)&H:Vf?k9]X]:Z@Obj,-74VUb_mYm8UPqLcZ'r5-Y]qpNp$58-FY6,K"qZo(DS5%Jfr5l6
	HEoL`l,gR`\!Kc>UV/q5JB)eNZD'5fAJ'm4[!cLPZjV@g\6Gj"fb@j.&p=HP:'/c6=OD5I!g<Q-.U[
	?H-O$9r.K58Y%D4Q.^/P*a`F;[nrh`8NoQ"TN<fOM,8;_OG4#t8CR@+#Y-t^Q`Ja`1QW=,^8qPHcqt
	XL6#T5p_WE0#BQpLFHDmW/<V\;gqn0?*2DE)V;)3't2apA%N5u*e$!>AIqT5ekF..X+R9+p.2_`'Nd
	'o@ggQ66HMD=3u:T'&,HokLnW"@8+p8Pp2FZ7.4OpuBT\o)KC'eCLAJ>!I*lEC/eI[9?B\(eWQQA"[
	dlU*+BcmgU$?9V/=dYo8Q4.e0Z]e'cZ(N]V;C_JAA#USD*QS)(2:A7YFrP]#TSiffK35II:MF4u4VR
	:Qg5+<f^,+C%j@2o_4>i-h="!Na+$$?/9s=90I.oYoAWH;n/j_uQ!g85,B^6k)>*;W]qmS'U&.C2+Y
	U;2VUpF4+a_YSo'M]I7gnotEI+a)OgJDF*2,8X(K_@:ZF(TkTs((m\_NL.P!0_YZ8+jYRl/H2HW7ma
	+]o)&^ACH7<Pl<%[:qqN#3UVm5HU3X#uu<**.FF`d]b5nGrDc0aZ=T!c]rP%7^Yj`0<aGb2g">$?lW
	d%mVVV<=S!H?>41j1(*Z\[jggQ4X*N%$R&X#lnVaT%/#(7h%Y$!.'E"<*)FSK3Fhr93DP\-DE/r*]g
	*WBo<.8Er:u4XVuTZW(pl3*&rHK^hBgeePW)2C:=bYF;4p%2O9[T?5.ZNi#3P7eJHC0Qcmq7bfd;J&
	0_B8+b?=8MI@@G<L.2@+sN7g6\D'J(\[7foq<'&3-nOEFL,QgL'$^b_hf%A[C63rOZu5+=H[5THjFJ
	&CN7m6Yon_blKIq"<gG$_1XeiO\6LYQbN5MYVPddd0q0KBjWZ?a0]NE%Ek/E)/LdqqU+FKn/BK4PPn
	7:3!&_H[*jjMVp3&E>cF:]H1_Ytq(3WXq=S+IK[6'KGq.(Vg[lJNZJ5<XY`rlAR"OH(hU,qfZX4=5O
	5Of#sKpW]C&DTCoM'D,)EilIDT\qfTiS[rn#lr@0ag+Tf4H7</l\3/_Y^6'%C"e+G=0J>kQZQb$I9P
	4i?4MtsI,Y8W;]%ecrXAh977??u_u'.Yh/:8EUX@/&>j8e4Nr`?PB!X9GY*J$,QP[g)1)_0i8aWgMY
	NJMo>!BQ'Y4JJE1>_Rfnq[l0'J0<8_r:+Lc\N1E8A3m2hufFelE",s"aFDeOK0rbW2r[n!!iEBio5c
	q/G3#8ND2&b&AraIK#(jAY#H/3ag+$'o9.#u![ijlA#!7*P!r77lm+!:@Mf2AZR->!+)bU*`aCJ;$j
	KC)Q=Z>;C9q%`G!9Xp(@i;P`Xk^5K/(*Ing>pq]N-^Nd&OopcU("4B3q9G=qH@*^PtV;Q0>0o%>(66
	Gkl$`UIJNd/5-eDEDEEb-V$H6H?u6]3nj2"N(/g#Ge,jS%\GF%K$a2R^]Df.P1J/L?(G..>CZqs!P*
	OOYWAXUVBE&;=uH*-0r,OhQl@=8_RhZ*^bC]X=7Itb]A6L<0*o35n3$/WCuehR2p_($+e#9TH<<j?M
	II53N'8f3103@K9T0mJ\Sn+AW54W"77Dj2!"4B2!%!^.N0-%>=OpE:AN3m,L!);I\$haZ=sFU\5R8M
	i@V!.bhOm,:2%1F1Me!?&3KC&<N"s%)XGE'N![34.OJF^VXN0K0+k5?8)a^X6#!3fDk)<Ko:-d4Si:
	S0"A"XBPY\JAto(#=_!;XV#fqdVN_t0lHXDgOmeIXU95L28>+!MpiSp-):08bA1akQ1q78X8A!?kJX
	=b\Zh^EGrQMX$+r.jhC(2N<Vn6aiJQ5Z&*<Sjh@hl<1Ft6`?S;V=PRp0se'Qc":6T>kOrP\0/t8<Fl
	efqFd4#Hto[P,Y<S1.#Pm]E;V@`dkjch7uJf`3rfNd!XPO+1)8\?&4BN01?#bR_hPWVN+u`SoEo_:9
	ncH5GlGRnSq<",Gih]W>WcHMFSL@$F`jXi1aE5A,:3UA:/p]8ZGdK#Z[91JjTZr$*nR-hXtF:jlEb#
	4(kH)u6/Z9VTI?PBPr9P?%)]H\)HbS30J#XIB!X3EcCeI?iW;j'DJrKfC2-F?P1mVDhNJKP=b]0>8_
	)chN[0>h*=T*"oaeGmDOeuoa=jopXL0@RbE2c)*!`=:B`p^5,XE![fWhP094Kcf!`k?g':MLE,RKV:
	ZFW*-b7VWG9U@!$VD_:gB3^qeeS>TuK@+$E#d#t/cQ5m7c(RL$<Ma"4/&6c;5?sIb=LV%'<\>p2e#!
	;@1U[4:jlP^>pS=)jo^9=@2^li/*7WNhV)A9&*YEX&)BCC1&0sgd$q?gI>eNaRFg2D]9@F;<Yq0Z_8
	n*c$("f8cj'c[V(gJ[0rWoRTWl,_S^q"N"rHd_Cs'$s+=!sYFD]K6P:fO4uJt!LlR%?KIc(%*toU.K
	C,`+^+3igF95nX;l&0shXB!rX80`".;<[U<J,Y=.5#sa"39b,K>mG9u<+c7BKOrtS[1,:nU2/745n#
	a.l<1,1mn@P#q?Qm)XL3c4/2!!fmqoON=?hfZ5iaT9(Ko!'p@4YYSL6D/ep=m^MA@9![MnF/2W*TB2
	*?'\@/M/PVMMe.lG.TItEAT[BBZL`31=Erl#)],m`aH7p/mgM5'[&0WQ8KQhm-)hdXm2tRY;-N]7&%
	W=`@Y4Z`Q,$Orf;QM?[o1modM=DcjA'h:AEm[Gi;`h'see?h\#Vm?5O)ihgM(O__BAO1#M?oana@YO
	sES],Y<#IWcS(@B-<']Ll3MLnf(.-^PQcAMKsQdm"U63MY(/sbAW8t-+ggY:L%.phu4lI_7fTCe]LL
	/kp>QW:p6s,*DV,C(sn8o_h[@:SeJLA^^1>:nA?CQFrCeYm'?_BZ%<$dc5]V4OG8Jo`R,K+8$I1V8W
	jM"QqW63UKEg>ih\(mYr$<*(p!_Dj2Mt<<mU&1dJ>OogY:JJ@@*CM)%cmbecb?cRPf3N=id`JJcO-p
	"TIS6F;J#\lH971`=F$n*'7\k0]O5W+XG/b8WjcV/7)TE6e"B6rU+kKj(phLP*G)>_e'SM]X,^f0Jn
	,$*]#T."81i>7q'IMs"@*<%(C^6Ub<J&%78+cUIT`Q<PgQ<:n\.pN8.[thL#2,/"7!>HA\0TNOe%.c
	>L+$EJcQYi6ap@[+>E%R$M>l=YiLO/8M%qNrkiAC`"nDW*PR#KkclsrA8osE3.>ZSOu$>TV)4b=@%\
	5bjG7Gq6FoY=`u]NDaC=?B3BKIH(\lEC"hqP!<euA8Fek6>D<ZTN7qi.?i_2]`@h&(AHN<bh5^CX/1
	g\7<JuRi3"@=)!.^T0S3[$-(uSIKq!kaa:51XPL::.f$+bLW?!U9+jIp@01-GS!219fN'ZDM"aI!g=
	V?Yu[5a!gF0)=g?;t9C2d:b'_/]F4V*66%<`aZE(Cqs#O,Iihs^fi:J.'f#SO<e\J(rhOk9UNW$XN.
	:O><<_Bj(9]2dZuIh"@3$<3HO>A))DKk-T@C9bpC!5&C.=5@g;JYF7ek/&,8b^U>W)%W;[=6B#+lTf
	oLNT!"^4_\W`\`#+D#-VpbqP'^]#6[/5kFmBdplmFO#Nj5l5Ymd;^)[FKrt^mS?]b?*2kVfO[hT/q*
	%rusgb%/K=8qatB7(C.G+$33n$"/6'%.m3;$4Spp?#)P#<'Y6D]#f5B2EbQuFc6h\t+g3,U1PR0kn`
	+Y*.nlG0F#H"$m+@s1r+B9[b0%"l/-(42MYA&Dp[I+Ys12Gl6DVjq1Q5sO.T!,j8abEH\$3(J!*GOm
	[r.bE<tZ+XEZUb3%iAkhY[]a3NUXUb0QQ5s2M)J[5N[@PKrqGM2*&30-YEKU5'S(qL,CnGQg7>EC$q
	cB*]ogu[C%HGZ-/Ie0.84SV&d#a)B'_U'gg"V)9,B=5B;&,3"SD0^%Ba<.ou_d@^0HVB/9sK[9kNa'
	QJ]?0FY[On+=/_,g#cRp=EVt%53%r]'HN2n%O.q^<tkHkK]XQ[r2KOMP?Cf=e`Dhh[!aqhR[iF6ULq
	U,UXtSpu73BG_bou)h\"Y&-R!Vf;.'Th-F;lNAY%]:o#rb.9eRM/u!N?WpH\lK(.`S+(Cs67Rk[q5C
	YB%dGWsVgM`.1M?3cLP.pkb91b<!)b*\%;1gesk+6qf#nuu=++sM"9pjkdiW,XVZ15u%/"!8=PGZ2a
	^Pp.IU(niYET>'i<_qHKVNWAh)B/qW)[uiW#/c'(5)@AlLL]MmSHM76#Z8p=4*S^10%b7<C;]l=s1C
	!$7]6SAQZiu`"LYYqG)*@C,R>SM1\9VFMqFiQf%)0>da4[T)2;Q2U>"@)KG]W$obs54l]K%a774e4m
	FlpU]iXSqN88U<9\o7!qmUF34_dsGTgR2X#-"TRF*nUTdg=d;_59rRh@ASdh_\Oba$F>cR*GSH0I\H
	-U8YB'1'$NY34_nU(i(lFL(:$sL'efCafo^@^&nTM!.^L#8!SKQ*BpiAldk.EUIU$$@,EUi?@Yb>TW
	/?RO6E2:/WB\e`n%0r6"(r7orjr\ne3CfZ=P_,>1!"(*0.FBf.$f.8Vr9[;c7'E%LR_5#V%Tm)Dd*d
	5$Y]h<JDD5Fqk`,T76X9'#s%3X/k*`#:B[I1)f&ra[n3*+Z^M_GQ@0DYMW%.\J*hL?VPh<o[E=.n+X
	#>hIp_Yc'r+m=2=EhSdp[F^ta56RS",X++3QDf,$O>W[d&@]1Im;)**j_]TcX10LS!`_r97VD-KFt"
	Y5H6($1j+[<tUTOsCOq&.k$hr9/=-oF;W/p$d`<5m^H8c\Ij9gV4N@#T5S:<J9;lQXTW+hs3N)+[#U
	7d!'\UFDTU6iJRAg$3D@)C7PtV)o]*.8^[`-NZLnjS>]P9'r!TE2X]5&"%sJip@@c)7ue]d4BI]npd
	%YM13V1UXZn:YmtPuNFB@nA$_DdT+"d-)M<"*bIt:B4roD2*G$Vl,WU_.gD;F]I1<0EGa^g:MRng]k
	3mjkkOONGF/1`o(jG54APUujmBMi#)]=YC:(0)a/8g%ph5J;KuZ;>o.E584Z6YE<C,q981D@c^3K[D
	I.J.B5-(OYe1TBk[c5us:(=-#S59W#J8]Z.t2-L]o7boCrYiFm2@&4._$13Id[;Gu_W.#OaDSo8I_4
	b!>(eu`/#mFp+Af&gZMTrZ4p2\m5,UtP&G`VjXjF5u721_&Wo_mF+h^k'cIG!#W](SFLm7M)eM2/)q
	9q[:"Vf)4#nIdVppOQCq]H!#*QGjQ$(Oc_Ip?1TDd(Ic_S>BB^QEE,D[a^i93HT],9P%;&Vc_!H%p"
	*Q#rqY_72)X%Z2nuA5<,IW,ZNEe1BN6"e='N/Vj2S!0A;&I)fDYad0BfR_R2[E1.#]+\oubh1:3btG
	kS<OL5uJ?njY%0s`tLL^qXBCrDR\:=rksD>,`NR4:J[&4n$F(qE`MPaI8-4$;NDNenbnuJ,bks+%>-
	nD.3thAQX>Jc^/:rag$mKi\u;k!^i.i)Z"-$@8U1,<n9T!E"ce5#rrH3)igSWhDJ<s1H20*,`$[>J>
	.sg`;,L1M9gnG^&m`e0GANVZ^.LR#!S.n5O8np_g[;g@8:OAL_fJ%@X#(9d++s%='je'R*3Rm'r9:+
	?U97f39:)[/VC's(/X0k'O$5&WmTrsrGB[8\\QA2N:6ln17rP-!Ung-C`"U$_VETeC!"Ei?b*%p"Ne
	=o(?+>)$f^5nUO'9fOK@aJF;50AAl%hBB-p7E`)Rgp6nAf`>C?Ong$;gi>pnO+RDn\)>a:K>28g9L,
	<InM?,cS,VZStb+B(Nln$Ql2na\,SNi:dL`f?Dc%^hpCLb@SljDV5Yu5FXU2AYTk:nf`D\ShO"H'\=
	'AJWMK?HpIca=TYl;V58.IWtasS"$p!PhL&NY6?h,MG:Zs!R@Scgo]_Neqq?DQ92SZ'C2.X(DJj>E:
	0EtSp=l.?XTu</[6SJG\Q5:bqgM:8@UUD_ouf@97$ea?c'k,0rnX>Pc^W1`r\]Li"-/#KId4IG;?AF
	%_4t2Aac8iOL'O%Y'GXbP(V+f)cZ@o*4`"3Of<Xc2[7qPY)k3ChSXZ*1B$;)1!'YYR13RJ>GrYlk6\
	*Wb*WVSprM$h*r)Jo/Z#o7!2Jnn\NN^a(]3CUFU(>lT0Q%r7MlRXIl=-LW)1Wt=%mR/L@&g9$Y"k<e
	X#COS:/.K!/K/7UN#O]n=`UhCS)8>8[%3q]$tF_]jlg-j+sS<=4`_I+rS?V\ceis(_(WOdj$W*;Y*E
	YGI_`5qcT_s>@]<@@1EB/[)X7sE%j2"12:8IKJ)A`;%j-IA>LaJTrdD"q8RLl;m8-ud<gPu-0d&,T#
	$X9$=bA<W@#MG+\R]^+oR+:,hUU.f17>4EX4:s][@7-;'6*>B>]MS"e<Q@NWQo]4Q)bJs'*p#*FPXT
	TY$V9Ud1J8-&l",qknh>Kj19>?D>H<\@7*%8TT*%HHT5H,G4P3'%4X#l2AfmlE?rS9p@P.]<EUjg_O
	lWo=S?rgqqK?5`83.];=AP-4E7*,a,[lOlL*4L`Xa@F4=J@q_<\!"TZNgA0^*Npp2O=G`f\03NW#L7
	h%d8"iKd]4]86F_//ClnC2-4-`lO8eD)N@#ZUFO_7WHPlYlt;;"!+`MBTekdc6t<%im/DohX>Y?UJ8
	.<b:VXHEtu]FUAnIea>_S!_BOlcAcCul0L1C\GA<RI5rG'<\gB(YT:BQ;b$GrIG%NEEfrHaWWlH/re
	RkEH_9o#==co?6GCq'Tk5YhUdT;nUV7[fEG.5n?A',LVd8$2$V78LX[1<7J=aU)4L``bsja6HqDq4b
	^3<\8g,Y8]B^]/c?<JJs/hD1Q&q>^L%;+X/A<+@ui_Wb8/&,k[+<>Bb&=TH6]&Z?_%QS8)U-?bONe2
	2#30V<M\90q7,l&C<#=d".Y,@a1s\mFXb=>bGK>9Hs[L&@`<2aLL>.)6#_r4/#hkWY^OdG!CcVbcX4
	d:1*he\3V,]#0n`:jNVY)1W*,h6V.`0A_*nf<ed?!\!4I@CdN0W[,WFWi(9,AAD`r@GG\kT<$[7>s\
	UO*NONE.gr_S.1CSNG2K3p9q/#_/J6DC8IJ\@66=TuH8%"_b>hQCk0EWe=!(Rb;\\/Y)XumRn^AgZ_
	bA<3f*>B<B`rnRPAko1_$TL[%i^6::Y`R.p!;N@#66db/nsaEU+)"Pn=8&XTG@#X3T2d<#7WW3e_Z&
	jO^q!tYgWot`O^5??nfA)c%I*.%DHdInXD7pjSI1dRiX-+VQRRZot5S;>1ETQNZSMnH)rf?`eM3.!O
	gZhI'3Vt[(8V)'mO#oI%fZAUe-$*Y-i$6l-l<qC/WG`aEqpa@uid^@0>^^CDHs)m2&V2T"FaR6T]Jn
	a+=!7AtO7`(hiQ*X*&Q(q6F2s.t=q*g)Ct.T8@tqY=Hk,VakHVnq@J0G:QZTEMTJ;A&bV-[Opr061Y
	'*QBeSD2m:,gB?7QQM%affXO(utVS<eoEcN<Q9!.q)XY*]E`F84FE-gR!DftB%RC^L[nIn\\pXWQf#
	Sa_Q!875J@_.l>q53SeO0Fu!+JPK5eW/nXB^hKcbNDQ8^EPCs;q$+.j501ujHH/m$OD>A)V8o^2YFN
	5!e(:Q!3#_e>Eu?2#KMk(YP"G'5=AbiW`uP$S^_=Va$%1B4`O_LGU,l\gk^d5`CouC7n_e$jPC@fMk
	>B;jVr,;BWb%s27EiR?hDNKf\=*/hjKmO1SWaD#SB#2eUdl&\^h&gk0-]'^"$L44Y"h<OC`RGkl1Op
	2cNc?KWV^(K?<b@mG"Pulp-,!IIZ=^>ArnBBn;Hq61OLhLmt.>CDGI".*)a7J:pr.WNGi'MUOcYX=O
	X!!4u8@/h,3DdfZ2k1^\MZs&5YX55k-#s7h`QB`L=\U,5-4/6-&ei4utj@0Lmpd3#ML2X5=bpL/U"W
	is[lnEm]r=+*h<l[p/1G07sT2`A&'=F[-Td;8g"NZD'*9^mi&$Pskrp"%E<BG0?<ADA/-U#g%gN.A0
	Ofb8LB8:<lMcbja^_!cmp50)S$e=5GM?.(mI_!T?ljhHRq+!UqC;:"Ir";h0$/71GfhU0^_k3=(a5?
	u`Z_d\24.4Scm=C#<lO8>kn-Sr*49R)aKlS^Z.m0#LKgLrL[p;Ii3[;>Sm9BXX)hQW\P0eD,p$ZJ`H
	B[6tQ6`DD(h#YFjofC0n@dos7!8tK1eA:;b*$<IA'k!L?`fCY_Nn'T*B[P3!FBk<0kGf.+KOhX_r.S
	YWZ^&EFBO!2loCMhd>Oa]eHiJP;#RUVI'r`kZbmg,mR>A7UXf^.MZMSJDdreQmp#i1>mUqe1-9$XZ>
	+o[h6Z/MnDqYG=or:c.d6@'ij$A$P5]=lommsp'>cL.fJ2R:nQ^F5!krKk._L's+bSEk":DYUoHKgT
	b(;^n*KZquT2Rn!/XTffpg*,,"VOiY3F)"mng8V(RmlB::SQhQm!eagI+sPn=o#jV@5,ARh=3$>T@.
	&mq5qC"HV?_r)ccjft*\/2*U'O7HP?Le:1N0jB7p9W)[T7TGC=!2L@j]PZVCMg(e]Z8I6KKd!S3as&
	h47^[atO#:+`m=OO4:JWUt^CnMtD*.3`N6\nW+ea+H#[)#O9:5@tVggjQ\Z^?@Nt,K#U:Tlne=obaE
	n#$<'@2lX,+/lZZRSMt,F2I8bf6krElq$DgIKJD#"nlAk7^Xj`auko9n[En+QJko.=XNLrI+#9X0<^
	9(_K=s5\(;C.5)VG%#@`739[,=a@3^^@jY/ClrAH@Qc]R/$HpeC5VCKlQ\q!`otcmFu(4:A)bg):8,
	'X8kKsTQNmU.tb6FiA\LPD^mOCJ7(Gh!'#p)dnWI@/6`</=6f@Mg^tgW_]0Qi`i`T-hIW&bVT$RSG/
	=A9RGc*VQ+"-<9>=?)-50(@)Z3mMh6*&L0)4*<f_RUUXI1'KN=21f!s%9)44S@q2lq9/Iej_KccN;u
	Kp:P1_*\'L;Hhf<OX&J@Yau!"rd=c49RQa[$O[WbTl>X??sIPbKP-m-f3_U_(_#p'_]cF9hgINne,K
	;,VuL2C@*t^1g3Mm*h,:Ac)1$VYlM&!tj2M7>_]0QiAl2EKY;!G"0equM1CYT9.p(Sh"W/EXp`k%Q#
	J,WK%t5?@$4gXj(q<R%D:R7UL::,H.KF_kM3.Qfkn`+GrpjWkKTCt@jU['pcEo]&*Crdq)a7_aG%.T
	X3+n.rGOOD\^h6Z$0b=W1eQ6,.7Df,@MA7;ZnGlrR_K4$,I.c0*6VRq(&HQ[upjX_N(07OEW=],O6G
	eJls#jTHP;G-DApm^GOX)(p,LhPp&AI!)++:(4=F*$`^Gs`P5P\9o1ZT<(JrJ&Y5nNP8Nrs!CIHp'G
	S.Y>a:R%?4LCHoHL-8-/>GIqMft7"T'9n5k]a2X.G(#+".8q@&-6=aZEr:"Cn]1V*49`U/C%1oX^9/
	81)m!8qYCIJ]XBBIir,H%jr",'+<a=E%1,Fn8#udb=\ofs#!^c4M%fd1Rn?_<TE@bg''`@N'IY;Z0#
	5i.G5It:0_10/^)[HHE&,\]%3></^1%=c&'r)SBX)'\D0l4e3p[92p*Pf'-N.FC$gh>Sm"2F^R+9[O
	XY\Y34Fk?:+Md'KtRlA69"+Us,R0*s^!!/CG6."Bq>IM?HrEunG_M)slS_':fa6eVb^4N('Ll6mSMa
	C:7KB)N6G\e,)h*@,WB.c<R6>'I0&WMe>fk/n0'lY.gUqkDin=BC,F2jF"%Qh,:c7big#A225V9?%5
	JI;MnmY.kD!H'j?]+r&h^s=PC`aBsC4GFjsd+f&)8)NOheJnf"KsZ=+2hf-Mk=]75kSr@QiAbLdK9V
	T>(%hh56N6pK_tZ1/TUlf57MoQ=2o:br(h*>h+O&@i_YEl@rrI?9?O@j6!qH<k!!#SZ:.26O@"J
	ASCII85End
End

// PNG: width= 45, height= 15
Picture SpHrView_pict_Movie
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!#3!!!!`#R18/!*X66h#IET$#iF<ErZ1m,6.]r,6<%c5B[#,!#]lI=EI
	J[Aor6*Eb,5:A8bpg+A$EYB4XtP@:XY'a^mR7!!qLS6pXdsg?`*^=5$7XjrK\\d%_?*X7;hG0?8Ri&
	,-0?DlZuW[A>8o0/l7#>iP0`p*Bu"F&#Ma52#;h5+0VgL>DK41.dUiCl5$Fn0CDH&eZBN#mh38KS0V
	,6F(cfD_r@Rs1)*$V0]i,8!3"CA`q]RYHP-XdfG<jmCI4-NiA#p)b]sPrDU%)/=scRR7nqb*fjYKe*
	1t3S]C`\_91&CM='t^QgOL-$g^-/hPF-7S8T@AB2Vj\Z0qA`>'b*Di>[cF="&H0XZP2R[bpPD?!lLd
	nQ*?c'oS@.15NROdLQP3;0m/TjS06?AG:_]S5!%?&-&7#T81B\-4<p?S0S\)m7$lGK:O&2PC,0:SAb
	p=i"STN(U'+`0qK7@is4V+P$kuB52`^4#kJ>r#15Men1)WOp$4OcE=NKP>/arb3h]VV=8lS]P[ZfEA
	A['nrNB+(Toa%1*Ze'(o^VJ!^Us@h`L`S>:IJa<B!\F%!$'p>p"nI#f\53=:/(,*^ZXM0ghH#tN[/)
	U/GM/e8Pc8EN<OKunWWOlLZ*3.]!fZ`^^>-=UACnnqXpRU`f]+,e?s/VM[1u@n6#8FhAGcbquj%t>u
	Vq&_hgpF,olOnO2V^aoHpD61GWW,4oK$1PFa/Jo5ck]BIsDfHIH@fT5?;6r#6+fL"HnXp:PkSWQDC+
	Eol<>U$[cB\QlZK4Cf-%]$2$B$TOn?k@c^C^OO"Yp#1sl)QgfWnT_pUEIbk!7G!>[R+lL5K#)#R_la
	PTXk(1'[klXJ=PUL_qtJAP@A-E21$nN'k_tLB'V!F8>G=@u5(ED!+.;pa_XftM3b;f3:Eg(9^O<_O-
	2d<'+V<7`>1mJ!A6`bcK%P\]O7hJh"5.hpGh+e(?/YVO(>.h2;(VP4)Hds^fc_s2%>U_:F#jKF0L_A
	g;J,he+_DVEBJtTRFg<i$h:[#UDUH>oi&DeLjQ-3*=rK:1T;<iZGL@aEhcRb%8X;I_cW4$PE5b4@J`
	l3/eG2ApHp03*/V4f^]ukXkT5m>C%@9QG^TMZ$,&qn6=^8]BVfF@iG4HpH%>b;Jq8ls0(E#".IPeq7
	)fRQ^HBgo7oTEl!?:LN/YjeMC?rTGQ\lj8uBacT+c)YUK/Shc:q(aP7c.gfLG_#tW"lj30iQ84,Yja
	8O<ECZ^9pi^g-$eLb<G@!lJaj&eBt%gIA>?W8Q@sHJ*]=lY+$Weq8R$RfJ$Cr4=_WEpM[ndbO4QGJV
	!j'-dD<.4HScK<$O.>u8J#7c?YnUI;A(9AF;P6"%i8@l&a@E>5n[L)8#&.rWje]a474]7l><TIdmuk
	Nf+oD!=1r0&H&>'_:Mu/hKbqF>Fd4gmD)(&=n(4XtT4DYeXakkZk]$X/WF:Xl9,:jO_TQ[r>\6_*G.
	4p&euk$llmAP4Wch*A(XPsUT#T9&'[7^+1.rk__d-hJA8an\GAB!9a3N]9m6\np7KDU$XU"e04[:*Q
	_D"p?O`-RSKCCWo]#^=JWZE[Uo$#Y;5IVfJD/"R8l`r`J08:7uZl2ioYcLbP`(CYK7S;F:M/J1=elK
	$^5=iW%3_>PO?sOT4-hK>M?&0Z;M.%RU>IlZ"V$/%ES(=q/E[@b/)1/]W#*I7sds?>pX:UX4kXuCY*
	n.3Il&Re&r->f^^[GF=#a2I,YfX*KDf9QfO(kijOGT)33UR\Y-It3F#<=%8_Rh3JYd*Oj:]61hM6)>
	?*=Hd)7*]\q2#-VYpI=ije28ea7#*)I?!@^gffPb^BT%l[P-NhT3#ACp]Z"OV0fl*hajLab%1/Y;Bn
	<?k)ksBbj;B8Je<,p8<1(X;`5fGJ/kAch/!BDF-nSiahr"IJk]H0IXpU>7qOM/2!@'d-7Fl4c9p_/0
	lMm'%2mi'GQ59Dpp#(emU**V3*TdQcIcj=83_tSC'p:1M$kifXADoBtHr?dE/hOXa8ETg:@atP3_@n
	"B^5HeEX]f,#V%F06WDEKakC$f-O*eIjXBL^ma=P')Kr7oY$jeQp3d#jc(W!%e7`_C4@+i),7XRD#@
	iDG4FZ1GUMpd3d=QW<l2rH:cG])Ktl3.Y*DKQBrO^9i=LX!')ZI#TT<*E['GSd1[bEhn,C4V5gWWeD
	6r44Ja^C$e@DtRY9<=:@+GStdeNZ+,NL.KQ*4(SeM`[q=^'K2H*qC9\VU9q`-:8VgQO%(?i[D;*;J(
	9In]#(JUl6/G)?<(0jbN4EVI4iPq,4_'b$"?'*)e35Y^c!FtY:JlGlRt=HNE*Z'-+=*Cd\obGFOkJn
	$RFcRmLkG#%Br$EdT@=OaHKi'C#MKk?eft(]?ghuoh)_Wbufe(<qj7EpM$47%_')rn3pHfKdN;9E6F
	`;D0BIBk`)8L[#.H`_C]\nnCR-R\#5)#<iad8Q8bo$Ed91^)n*D)N=@F![uJ[$CTAU./^PcUC>r7JR
	NEO9q64]^?uKq#rP;ka_tk`.a7Bh65C^D3[94p=r-H'E_;CeYa+&@/,J?:u-;Se$V$3k.I]M!>IqND
	,g.*!P_-i)Kqd5'6Kg>*&dl,*\/*5AI4oInIgB/(J/<-X*$n]Y)",lK6Nkno]WWe=-N2=8[A0W#YH>
	/M$,eV>3`tBtAipetTR)A3QI2T($Y`&qH?QF10V?DgBq>V]IN^U@YN'8.eDuDlU5!GE)o*hlf\&EY-
	lUCQ8d_JV=]jB$'[su>6eJ#G(X\u.ZM0&JAs/#2CFh):4L0/qF9GXjimHe>fF4/OGPmG5c)Dr$,crJ
	2lfO?#^;u,?K^"6QIQNRCcmu;O*@#uL-H4ONs!p<5CU]V+rDiocrNJ70QZ6Lq`7j>-CY7",SPu<16n
	G9)HARe^XV0He5nD:2&[c?1]\#1hid9;b',U>d&&eZBN#mh38KS0V,6:->Q+X+^dO<c4R$SVR>C<m5
	?V9So<!!!!j78?7R6=>B
	ASCII85End
End
